<?php
namespace App\Http\Controllers;

use Laravel\Lumen\Routing\Controller as BaseController;
use Illuminate\Http\Request;
use DB;
use Quartz;
use Log;
use DateTime, DateTimeZone, DateInterval;
use App\Jobs\TripComplete;

class LocalTime extends BaseController
{

  public function find(Request $request) {
    $token = $request->input('token');
    if(!$token)
      return response(json_encode(['error' => 'no token provided']))->header('Content-Type', 'application/json');

    $db = DB::table('databases')->where('read_token','=',$token)->first();
    if(!$db)
      return response(json_encode(['error' => 'invalid token']))->header('Content-Type', 'application/json');

    $qz = new Quartz\DB(env('STORAGE_DIR').$db->name, 'r');

    $timezones = [
      '-23:00','-22:00','-21:00','-20:00',
      '-19:00','-18:00','-17:00','-16:00','-15:00','-14:00','-13:00','-12:00','-11:00','-10:00',
      '-09:00','-08:00','-07:00','-06:00','-05:00','-04:00','-03:00','-02:00','-01:00','+00:00',
      '+01:00','+02:00','+03:00','+04:00','+05:00','+06:00','+07:00','+08:00','+09:00','+10:00',
      '+11:00','+12:00','+13:00','+14:00','+15:00','+16:00','+17:00','+18:00','+19:00','+20:00',
      '+21:00','+22:00','+23:00',
    ];
    
    $date = false;
    if($input=$request->input('input')) {
      // Strict input format
      if(preg_match('/^[0-9]{4}-[0-9]{2}-[0-9]{2} [0-9]{2}:[0-9]{2}:[0-9]{2}$/', $input)) {
        $date = $input;
      } else {
        // Invalid format
      }
    }
    if(!$date) {
      return response(json_encode(['error' => 'invalid date provided']))->header('Content-Type', 'application/json');
    }

    $candidates = [];
    
    foreach($timezones as $tz) {
      // Interpret the input date in each timezone
      $date = DateTime::createFromFormat('Y-m-d H:i:s', $input, new DateTimeZone($tz));

      // Find the closest record to that absolute timestamp
      if($record = $this->_find_closest_record($qz, $date)) {
        if($record->data) {
          $local = $this->_timezone_for_location($record->data->geometry->coordinates[1], $record->data->geometry->coordinates[0], $record->date->format('c'));
          
          $diff = strtotime($local->localtime) - strtotime($date->format('c'));
  
          // Find the record where the localized timezone offset matches the candidate offset
          if($tz == $local->date->format('P')) {
            $candidates[] = [
              'record' => $record->data,
              'local' => $local,
              'tz' => $tz,
              'diff' => $diff
            ];
          }
        }
      }
    }
    
    // Choose the candidate with the smallest absolute time difference
    usort($candidates, function($a, $b){ 
      return abs($a['diff']) < abs($b['diff']) ? -1 : 1;
    });

    if(count($candidates)) {
      $record = $candidates[0];
      
      $response = [
        'data' => $record['record'],
        'timezone' => [
          'offset' => $record['local']->offset,
          'seconds' => $record['local']->seconds,
          'localtime' => $record['local']->localtime,
          'name' => $record['local']->name,
        ]
      ];
    } else {
      $response = ['data'=>null];
    }
    
    return response(json_encode($response))->header('Content-Type', 'application/json');
  }
  
  private function _timezone_for_location($lat, $lng, $date) {
    return Timezone::timezone_for_location($lat, $lng, $date);
  }
  
  private function _find_closest_record($qz, $date) {
    // TODO: move this logic into QuartzDB
    
    // Load the shard for the given date
    $shard = $qz->shardForDate($date);
    // If the shard doesn't exist, check one day before
    if(!$shard->exists()) {
      $date = $date->sub(new DateInterval('PT86400S'));
      $shard = $qz->shardForDate($date);
    }
    // Now if the shard doesn't exist, return an empty result
    if(!$shard->exists()) {
      return false;
    }

    // Start iterating through the shard and look for the last line that is before the given date
    $shard->init();
    $record = false;
    foreach($shard as $r) {
      if($r->date > $date)
        break;
      $record = $r;
    }
    return $record;
  }

}

// TODO: Move the Timezone and TimezoneResult classes to a library that both Atlas and Compass can use

class TimezoneResult {
  public $timezone = null;

  private $_now;
  private $_name;

  public function __construct($timezone, $date=false) {
    if($date)
      $this->_now = new DateTime($date);
    else
      $this->_now = new DateTime();
    $this->_now->setTimeZone(new DateTimeZone($timezone));
    $this->_name = $timezone;
  }

  public function __get($key) {
    switch($key) {
      case 'date':
        return $this->_now;
      case 'offset': 
        return $this->_now->format('P');
      case 'seconds':
        return (int)$this->_now->format('Z');
      case 'localtime':
        return $this->_now->format('c');
      case 'name':
        return $this->_name;
    }
  }

  public function __toString() {
    return $this->_name;
  }
}


class Timezone {

  public static function timezone_for_location($lat, $lng, $date=false) {
    $start = microtime(true);

    $tzfile = self::tz_data();

    // header("Content-type: text/plain");

    // echo "Loaded file in " . ((microtime(true)-$start)*1000) . "ms\n";
    // $start = microtime(true);

    $timezones = [];
    foreach($tzfile as $line) {
      $tz = explode(" ", trim($line));
      $distance = self::gc_distance((float)$tz[0], (float)$tz[1], $lat, $lng);
      if($distance < 200000)
        $timezones[] = array_merge($tz, [$distance]);
    }

    // echo "Found distances in " . ((microtime(true)-$start)*1000) . "ms\n";
    // $start = microtime(true);

    usort($timezones, function($a,$b) {
      return $a[3] < $b[3] ? -1 : 1;
    });

    // echo "Sorted in " . ((microtime(true)-$start)*1000). "ms\n";

    if(count($timezones) > 0)
      return new TimezoneResult($timezones[0][2], $date);
    else
      return null;
  }

  public static function gc_distance($lat1, $lng1, $lat2, $lng2) {
    return ( 6378100 * acos( cos( deg2rad($lat1) ) * cos( deg2rad($lat2) ) * cos( deg2rad($lng2) - deg2rad($lng1) ) + sin( deg2rad($lat1) ) * sin( deg2rad($lat2) ) ) );
  }

  private static function tz_data() {
    return explode("\n",'42.50729 1.53414 Europe/Andorra
42.50779 1.52109 Europe/Andorra
25.56473 55.55517 Asia/Dubai
25.78953 55.9432 Asia/Dubai
25.33132 56.34199 Asia/Dubai
25.25817 55.30472 Asia/Dubai
25.61955 56.27291 Asia/Dubai
25.35731 55.4033 Asia/Dubai
24.11028 52.73056 Asia/Dubai
25.11641 56.34141 Asia/Dubai
24.19167 55.76056 Asia/Dubai
25.41111 55.43504 Asia/Dubai
25.28812 55.88157 Asia/Dubai
24.46667 54.36667 Asia/Dubai
36.73605 69.53451 Asia/Kabul
33.30294 62.1474 Asia/Kabul
36.66757 65.7529 Asia/Kabul
34.10738 64.3052 Asia/Kabul
36.21544 65.93249 Asia/Kabul
35.84972 66.43694 Asia/Kabul
36.26468 68.01551 Asia/Kabul
37.12604 69.83045 Asia/Kabul
37.41853 66.04358 Asia/Kabul
37.22019 68.78041 Asia/Kabul
35.94458 68.71512 Asia/Kabul
34.5875 68.95333 Asia/Kabul
36.0649 69.13343 Asia/Kabul
35.92139 64.78361 Asia/Kabul
34.67139 70.20944 Asia/Kabul
36.70904 67.11087 Asia/Kabul
31.59382 64.37161 Asia/Kabul
33.29565 61.95221 Asia/Kabul
36.72896 68.857 Asia/Kabul
33.33951 69.92041 Asia/Kabul
36.69736 67.69826 Asia/Kabul
31.52919 62.79055 Asia/Kabul
36.6825 69.11556 Asia/Kabul
34.48108 62.5863 Asia/Kabul
31.61332 65.71013 Asia/Kabul
34.52813 69.17233 Asia/Kabul
34.42647 70.45153 Asia/Kabul
35.11833 69.23778 Asia/Kabul
34.34817 62.19967 Asia/Kabul
35.73062 63.78264 Asia/Kabul
33.55356 68.42689 Asia/Kabul
31.82089 64.57005 Asia/Kabul
33.59744 69.22592 Asia/Kabul
37.11664 70.58002 Asia/Kabul
32.37451 62.11638 Asia/Kabul
34.66667 61.06667 Asia/Kabul
35.01361 69.17139 Asia/Kabul
33.9675 68.94861 Asia/Kabul
36.75635 66.8972 Asia/Kabul
36.13068 68.70829 Asia/Kabul
37.08571 69.47958 Asia/Kabul
35.03333 71.35809 Asia/Kabul
36.95293 65.12376 Asia/Kabul
34.12583 70.70778 Asia/Kabul
34.26545 67.34516 Asia/Kabul
17.11667 -61.85 America/Antigua
18.21704 -63.05783 America/Anguilla
39.87556 20.00528 Europe/Tirane
42.07694 20.42194 Europe/Tirane
40.61861 20.78083 Europe/Tirane
40.07583 20.13889 Europe/Tirane
41.1125 20.08222 Europe/Tirane
41.61028 20.00889 Europe/Tirane
40.46667 19.48972 Europe/Tirane
41.3275 19.81889 Europe/Tirane
42.06828 19.51258 Europe/Tirane
40.64278 19.65083 Europe/Tirane
40.94194 19.705 Europe/Tirane
41.78361 19.64361 Europe/Tirane
41.63556 19.71306 Europe/Tirane
40.80028 19.91667 Europe/Tirane
41.50917 19.79278 Europe/Tirane
41.18556 19.55694 Europe/Tirane
40.71667 19.56667 Europe/Tirane
40.72389 19.55611 Europe/Tirane
41.32306 19.44139 Europe/Tirane
40.70583 19.95222 Europe/Tirane
39.20754 46.40576 Asia/Yerevan
39.51288 46.33816 Asia/Yerevan
39.46405 45.97047 Asia/Yerevan
39.96144 44.54447 Asia/Yerevan
39.83166 44.70489 Asia/Yerevan
40.18111 44.51361 Asia/Yerevan
40.16557 44.29462 Asia/Yerevan
40.8322 44.2673 Asia/Yerevan
40.5484 44.94868 Asia/Yerevan
40.06762 44.43591 Asia/Yerevan
40.80456 44.4939 Asia/Yerevan
40.35398 45.12386 Asia/Yerevan
40.49748 44.7662 Asia/Yerevan
40.15446 44.03815 Asia/Yerevan
40.7942 43.84528 Asia/Yerevan
40.2991 44.36204 Asia/Yerevan
40.26741 44.62656 Asia/Yerevan
-9.66078 20.39155 Africa/Luanda
-8.41915 20.74466 Africa/Luanda
-10.70727 22.22466 Africa/Luanda
-7.60874 15.06131 Africa/Luanda
-6.1349 12.36894 Africa/Luanda
-7.23116 12.8666 Africa/Luanda
-6.26703 14.2401 Africa/Luanda
-9.54015 16.34096 Africa/Luanda
-8.83682 13.23432 Africa/Luanda
-8.57848 13.66425 Africa/Luanda
-5.55 12.2 Africa/Luanda
-11.20605 13.84371 Africa/Luanda
-15.19611 12.15222 Africa/Luanda
-14.6585 17.69099 Africa/Luanda
-11.78333 19.91667 Africa/Luanda
-14.91717 13.4925 Africa/Luanda
-12.90667 15.25333 Africa/Luanda
-12.34806 13.54556 Africa/Luanda
-12.38333 16.93333 Africa/Luanda
-12.77611 15.73917 Africa/Luanda
-12.43139 13.54722 Africa/Luanda
-12.15 17.28333 Africa/Luanda
-12.01667 17.48333 Africa/Luanda
-13.78333 14.68333 Africa/Luanda
-12.8525 15.56056 Africa/Luanda
-12.57626 13.40547 Africa/Luanda
-34.09814 -59.02858 America/Argentina/Buenos_Aires
-28.48752 -59.35515 America/Argentina/Cordoba
-31.8653 -59.02689 America/Argentina/Cordoba
-37.26394 -56.97304 America/Argentina/Buenos_Aires
-37.32167 -59.13316 America/Argentina/Buenos_Aires
-26.61667 -54.13333 America/Argentina/Cordoba
-28.54939 -56.04077 America/Argentina/Cordoba
-30.94768 -59.78696 America/Argentina/Cordoba
-26.62207 -54.10842 America/Argentina/Cordoba
-27.5079 -58.55454 America/Argentina/Cordoba
-28.13306 -58.76733 America/Argentina/Cordoba
-30.57781 -59.9317 America/Argentina/Cordoba
-28.25384 -58.62591 America/Argentina/Cordoba
-34.58333 -58.38333 America/Argentina/Buenos_Aires
-27.46056 -58.98389 America/Argentina/Cordoba
-29.15 -59.65 America/Argentina/Cordoba
-34.72418 -58.25265 America/Argentina/Buenos_Aires
-26.79598 -55.02402 America/Argentina/Cordoba
-25.59912 -54.57355 America/Argentina/Cordoba
-26.01517 -54.67306 America/Argentina/Cordoba
-26.40842 -54.69463 America/Argentina/Cordoba
-27.36708 -55.89608 America/Argentina/Cordoba
-34.74974 -58.68696 America/Argentina/Buenos_Aires
-25.73239 -59.10879 America/Argentina/Cordoba
-29.71311 -57.08991 America/Argentina/Cordoba
-27.48706 -55.11994 America/Argentina/Cordoba
-38.54726 -58.73675 America/Argentina/Buenos_Aires
-34.65344 -58.61975 America/Argentina/Buenos_Aires
-30.25359 -57.63626 America/Argentina/Cordoba
-26.5662 -54.757 America/Argentina/Cordoba
-34.65146 -59.43068 America/Argentina/Buenos_Aires
-29.18186 -58.07895 America/Argentina/Cordoba
-38.00228 -57.55754 America/Argentina/Buenos_Aires
-34.57028 -59.105 America/Argentina/Buenos_Aires
-34.92145 -57.95453 America/Argentina/Buenos_Aires
-30.74485 -59.64566 America/Argentina/Cordoba
-27.04346 -55.22698 America/Argentina/Cordoba
-33.00938 -58.51722 America/Argentina/Cordoba
-33.14156 -59.30966 America/Argentina/Cordoba
-29.14003 -59.26256 America/Argentina/Cordoba
-28.05 -56.03333 America/Argentina/Cordoba
-26.53743 -59.34158 America/Argentina/Cordoba
-27.48171 -55.82921 America/Argentina/Cordoba
-26.17753 -58.17814 America/Argentina/Cordoba
-27.41813 -59.02392 America/Argentina/Cordoba
-30.95465 -58.78326 America/Argentina/Cordoba
-30.01444 -59.52719 America/Argentina/Cordoba
-27.29846 -54.19877 America/Argentina/Cordoba
-36.31322 -57.67918 America/Argentina/Buenos_Aires
-29.79171 -58.0546 America/Argentina/Cordoba
-27.4806 -58.8341 America/Argentina/Cordoba
-31.39296 -58.02089 America/Argentina/Cordoba
-32.48249 -58.23722 America/Argentina/Cordoba
-34.57365 -58.44924 America/Argentina/Buenos_Aires
-30.75048 -57.97962 America/Argentina/Cordoba
-34.16874 -58.95914 America/Argentina/Buenos_Aires
-34.61315 -58.37723 America/Argentina/Buenos_Aires
-27.48299 -58.93579 America/Argentina/Cordoba
-36.77698 -59.85854 America/Argentina/Buenos_Aires
-29.11761 -59.65834 America/Argentina/Cordoba
-27.09625 -54.89626 America/Argentina/Cordoba
-38.89916 -70.05442 America/Argentina/Salta
-26.81667 -65.31667 America/Argentina/Tucuman
-39.1 -67.06667 America/Argentina/Salta
-31.4933 -68.53838 America/Argentina/San_Juan
-32.43293 -63.24763 America/Argentina/Cordoba
-32.40751 -63.24016 America/Argentina/Cordoba
-31.94585 -65.18958 America/Argentina/Cordoba
-33.22778 -60.3297 America/Argentina/Cordoba
-31.42414 -64.49778 America/Argentina/Cordoba
-27.57383 -60.71526 America/Argentina/Cordoba
-31.29458 -64.29538 America/Argentina/Cordoba
-40.81345 -62.99668 America/Argentina/Buenos_Aires
-32.61841 -60.15478 America/Argentina/Cordoba
-29.4593 -60.21261 America/Argentina/Cordoba
-33.74556 -61.96884 America/Argentina/Cordoba
-35.4323 -60.17271 America/Argentina/Buenos_Aires
-54.8 -68.3 America/Argentina/Ushuaia
-31.23073 -64.31615 America/Argentina/Cordoba
-26.34066 -60.43207 America/Argentina/Cordoba
-38.37394 -60.27978 America/Argentina/Buenos_Aires
-43.24895 -65.30505 America/Argentina/Catamarca
-27.49983 -64.86042 America/Argentina/Cordoba
-22.51636 -63.80131 America/Argentina/Salta
-26.73201 -65.25921 America/Argentina/Tucuman
-30.94404 -61.56148 America/Argentina/Cordoba
-31.66274 -60.7653 America/Argentina/Cordoba
-27.79511 -64.26149 America/Argentina/Cordoba
-36.61667 -64.28333 America/Argentina/Salta
-31.53987 -68.49503 America/Argentina/San_Juan
-31.63333 -60.7 America/Argentina/Cordoba
-24.19456 -65.29712 America/Argentina/Jujuy
-23.13223 -64.32598 America/Argentina/Salta
-34.61772 -68.33007 America/Argentina/Mendoza
-24.23127 -64.86614 America/Argentina/Jujuy
-33.33578 -60.22523 America/Argentina/Buenos_Aires
-26.82414 -65.2226 America/Argentina/Tucuman
-40.16038 -71.34865 America/Argentina/Salta
-33.08103 -68.46814 America/Argentina/Mendoza
-33.29501 -66.33563 America/Argentina/San_Luis
-30.78913 -60.59189 America/Argentina/Cordoba
-31.5375 -68.53639 America/Argentina/San_Juan
-30.24057 -68.74693 America/Argentina/San_Juan
-31.89618 -61.85984 America/Argentina/Cordoba
-31.42797 -62.08266 America/Argentina/Cordoba
-28.46957 -65.78524 America/Argentina/Catamarca
-40.73193 -64.94769 America/Argentina/Salta
-24.7859 -65.41166 America/Argentina/Salta
-34.26827 -62.71262 America/Argentina/Cordoba
-32.94682 -60.63932 America/Argentina/Cordoba
-32.17301 -64.11406 America/Argentina/Cordoba
-31.6526 -63.9099 America/Argentina/Cordoba
-51.62261 -69.21813 America/Argentina/Rio_Gallegos
-33.13067 -64.34992 America/Argentina/Cordoba
-31.16486 -64.32241 America/Argentina/Cordoba
-43.30016 -65.10228 America/Argentina/Catamarca
-31.25033 -61.4867 America/Argentina/Cordoba
-26.86913 -60.21683 America/Argentina/Cordoba
-38.87588 -62.07359 America/Argentina/Buenos_Aires
-42.7692 -65.03851 America/Argentina/Catamarca
-26.78522 -60.43876 America/Argentina/Cordoba
-31.68333 -68.58333 America/Argentina/San_Juan
-38.96667 -68.23333 America/Argentina/Salta
-33.88995 -60.57357 America/Argentina/Buenos_Aires
-32.99835 -60.76791 America/Argentina/Cordoba
-31.73197 -60.5238 America/Argentina/Cordoba
-24.25648 -65.21163 America/Argentina/Jujuy
-36.89272 -60.32254 America/Argentina/Buenos_Aires
-35.44437 -60.88313 America/Argentina/Buenos_Aires
-38.95161 -68.0591 America/Argentina/Salta
-30.71164 -61.99862 America/Argentina/Cordoba
-27.16742 -65.49832 America/Argentina/Tucuman
-32.89084 -68.82717 America/Argentina/Mendoza
-32.6978 -62.10672 America/Argentina/Cordoba
-26.92614 -60.04956 America/Argentina/Cordoba
-34.86649 -61.5302 America/Argentina/Buenos_Aires
-23.80644 -64.78757 America/Argentina/Jujuy
-27.08966 -61.08161 America/Argentina/Cordoba
-29.41105 -66.85067 America/Argentina/La_Rioja
-31.08841 -64.48987 America/Argentina/Cordoba
-31.34377 -64.33529 America/Argentina/Cordoba
-34.12662 -63.39119 America/Argentina/Cordoba
-34.58382 -60.94332 America/Argentina/Buenos_Aires
-25.08333 -64.18333 America/Argentina/Salta
-30.98153 -64.09424 America/Argentina/Cordoba
-32.85683 -60.71754 America/Argentina/Cordoba
-33.03016 -60.64045 America/Argentina/Cordoba
-39.03333 -67.58333 America/Argentina/Salta
-27.31667 -61.28333 America/Argentina/Cordoba
-35.65662 -63.75682 America/Argentina/Salta
-22.59588 -63.81255 America/Argentina/Salta
-32.02927 -61.22103 America/Argentina/Cordoba
-33.45937 -61.4832 America/Argentina/Cordoba
-27.05413 -65.40329 America/Argentina/Tucuman
-42.9 -71.31667 America/Argentina/Catamarca
-31.4488 -60.93173 America/Argentina/Cordoba
-23.20898 -64.09605 America/Argentina/Salta
-32.18 -64.41809 America/Argentina/Cordoba
-41.96667 -71.51667 America/Argentina/Salta
-32.06641 -60.63836 America/Argentina/Cordoba
-30.42036 -64.34984 America/Argentina/Cordoba
-38.94688 -69.20889 America/Argentina/Salta
-30.72644 -64.80387 America/Argentina/Cordoba
-32.02873 -60.30658 America/Argentina/Cordoba
-31.24508 -64.46563 America/Argentina/Cordoba
-37.45467 -61.93343 America/Argentina/Buenos_Aires
-31.97263 -60.91982 America/Argentina/Cordoba
-31.4135 -64.18105 America/Argentina/Cordoba
-45.86667 -67.5 America/Argentina/Catamarca
-38.93392 -67.99032 America/Argentina/Salta
-38.82224 -68.06293 America/Argentina/Salta
-34.89566 -60.01667 America/Argentina/Buenos_Aires
-31.48333 -68.53333 America/Argentina/San_Juan
-29.16195 -67.4974 America/Argentina/La_Rioja
-27.21438 -61.18795 America/Argentina/Cordoba
-34.64167 -60.47389 America/Argentina/Buenos_Aires
-38.80433 -68.13998 America/Argentina/Salta
-31.65179 -68.28105 America/Argentina/San_Juan
-37.87907 -67.7956 America/Argentina/Salta
-25.94679 -60.61947 America/Argentina/Cordoba
-33.04417 -61.16806 America/Argentina/Cordoba
-32.85679 -61.15331 America/Argentina/Cordoba
-32.82262 -60.71852 America/Argentina/Cordoba
-32.81636 -61.39493 America/Argentina/Cordoba
-46.43929 -67.52814 America/Argentina/Rio_Gallegos
-32.62591 -62.68873 America/Argentina/Cordoba
-27.03084 -65.30315 America/Argentina/Tucuman
-38.7196 -62.27243 America/Argentina/Buenos_Aires
-33.15489 -60.50863 America/Argentina/Cordoba
-31.42022 -63.05002 America/Argentina/Cordoba
-28.46064 -62.83472 America/Argentina/Cordoba
-31.65292 -64.42826 America/Argentina/Cordoba
-38.97736 -67.82714 America/Argentina/Salta
-26.81667 -65.13333 America/Argentina/Tucuman
-31.43722 -68.52556 America/Argentina/San_Juan
-27.4338 -65.61427 America/Argentina/Tucuman
-34.61082 -58.481 America/Argentina/Buenos_Aires
-33.67571 -65.45783 America/Argentina/San_Luis
-41.14557 -71.30822 America/Argentina/Salta
-14.27806 -170.7025 Pacific/Pago_Pago
46.84056 14.84417 Europe/Vienna
47.8 16.25 Europe/Vienna
48.20849 16.37208 Europe/Vienna
48.16667 14.03333 Europe/Vienna
48.4 15.6 Europe/Vienna
46.61028 13.85583 Europe/Vienna
48.22656 14.23459 Europe/Vienna
48.01485 16.29324 Europe/Vienna
47.71565 16.03575 Europe/Vienna
48.04274 14.42127 Europe/Vienna
46.8 13.5 Europe/Vienna
48.13333 16.46667 Europe/Vienna
48.2 15.63333 Europe/Vienna
47.79941 13.04399 Europe/Vienna
47.42681 12.848 Europe/Vienna
48.08605 16.28921 Europe/Vienna
47.42642 9.65851 Europe/Vienna
48.30639 14.28611 Europe/Vienna
48.27965 14.2533 Europe/Vienna
47.38333 15.1 Europe/Vienna
47.58333 12.16667 Europe/Vienna
48.40921 15.61415 Europe/Vienna
48.30521 16.32522 Europe/Vienna
46.62472 14.30528 Europe/Vienna
47.44458 15.29331 Europe/Vienna
47.26266 11.39454 Europe/Vienna
47.68333 13.1 Europe/Vienna
47.06667 15.45 Europe/Vienna
47.23306 9.6 Europe/Vienna
47.41667 9.73306 Europe/Vienna
47.50311 9.7471 Europe/Vienna
48.25628 13.04343 Europe/Vienna
48.00543 16.23264 Europe/Vienna
48.1229 14.87206 Europe/Vienna
47.965 13.04944 Europe/Vienna
48.00221 16.23097 Europe/Vienna
-33.03333 137.58333 Australia/Adelaide
-20.78333 117.13333 Australia/Perth
-31.93333 115.83333 Australia/Perth
-35.11667 139.26667 Australia/Adelaide
-20.73333 139.5 Australia/Brisbane
-35.13333 138.51667 Australia/Adelaide
-32.55 115.7 Australia/Perth
-32.25 115.76667 Australia/Perth
-30.75 121.46667 Australia/Perth
-32.08333 116 Australia/Perth
-28.76667 114.6 Australia/Perth
-34.6 138.73333 Australia/Adelaide
-12.46113 130.84185 Australia/Darwin
-33.33333 115.63333 Australia/Perth
-23.7 133.88333 Australia/Darwin
-35.01694 117.89167 Australia/Perth
-34.93333 138.6 Australia/Adelaide
-36.11667 146.88333 Australia/Melbourne
-38.38333 142.48333 Australia/Melbourne
-35.11667 147.36667 Australia/Sydney
-38.18333 146.53333 Australia/Melbourne
-19.25 146.8 Australia/Brisbane
-25.28333 152.86667 Australia/Brisbane
-27.55 151.96667 Australia/Brisbane
-31.9 152.46667 Australia/Sydney
-31.1 150.93333 Australia/Sydney
-33.86785 151.20732 Australia/Sydney
-28.00274 153.42999 Australia/Brisbane
-27.58333 153.05 Australia/Brisbane
-37.58333 144.73333 Australia/Melbourne
-32.6966 152.06348 Australia/Sydney
-27.96667 153.4 Australia/Brisbane
-29.7 152.95 Australia/Sydney
-36.38333 145.4 Australia/Melbourne
-37.8676 144.98099 Australia/Melbourne
-23.38333 150.5 Australia/Brisbane
-35.35 149.23333 Australia/Sydney
-33.73333 150.88333 Australia/Sydney
-31.43333 152.91667 Australia/Sydney
-33.28397 149.10018 Australia/Sydney
-34.88333 150.6 Australia/Sydney
-37.905 145.1427 Australia/Melbourne
-32.92715 151.77647 Australia/Sydney
-27.98333 153.33333 Australia/Brisbane
-37.91667 145.2 Australia/Melbourne
-37.83333 140.76667 Australia/Adelaide
-33.82778 151.23333 Australia/Sydney
-38.21667 145.03333 Australia/Melbourne
-26.68333 153.11667 Australia/Brisbane
-34.2 142.15 Australia/Melbourne
-37.68333 144.58333 Australia/Melbourne
-37.814 144.96332 Australia/Melbourne
-25.53333 152.7 Australia/Brisbane
-33.91667 151.16667 Australia/Sydney
-33.95 151.23333 Australia/Sydney
-21.15 149.2 Australia/Brisbane
-33.9 150.93333 Australia/Sydney
-28.81354 153.2773 Australia/Sydney
-41.45 147.16667 Australia/Hobart
-33.7 150.3 Australia/Sydney
-33.70244 151.09931 Australia/Sydney
-42.87936 147.32941 Australia/Hobart
-34.28333 146.03333 Australia/Sydney
-37.70462 145.10302 Australia/Melbourne
-33.83333 151.01667 Australia/Sydney
-34.75 149.71667 Australia/Sydney
-28 153.43333 Australia/Brisbane
-23.85 151.25 Australia/Brisbane
-38.13333 144.35 Australia/Melbourne
-38.14711 144.36069 Australia/Melbourne
-38.13333 145.13333 Australia/Melbourne
-32.16667 152.51667 Australia/Sydney
-33.77271 151.08184 Australia/Sydney
-34.06667 151.01667 Australia/Sydney
-36.13333 144.75 Australia/Sydney
-33.95 151.1 Australia/Sydney
-32.25 148.61667 Australia/Sydney
-41.16667 146.35 Australia/Hobart
-27.2 153.03333 Australia/Brisbane
-34.05 151.15 Australia/Sydney
-38.1 145.28333 Australia/Melbourne
-37.6 144.95 Australia/Melbourne
-30.3 153.13333 Australia/Brisbane
-34.43333 150.88333 Australia/Sydney
-32.83333 151.35 Australia/Sydney
-33.73333 151 Australia/Sydney
-33.78269 151.04888 Australia/Sydney
-34.05 151.13333 Australia/Sydney
-35.28346 149.12807 Australia/Sydney
-37.84205 145.0694 Australia/Melbourne
-26.8 153.13333 Australia/Brisbane
-16.91667 145.76667 Australia/Brisbane
-27.08333 152.95 Australia/Brisbane
-41.06667 145.91667 Australia/Hobart
-24.85 152.35 Australia/Brisbane
-26.68333 153.05 Australia/Brisbane
-37.76667 144.96667 Australia/Melbourne
-31.95 141.43333 Australia/Broken_Hill
-27.46794 153.02809 Australia/Brisbane
-33.76667 150.91667 Australia/Sydney
-36.76667 144.28333 Australia/Melbourne
-33.41665 149.5806 Australia/Sydney
-28.21298 153.53634 Australia/Sydney
-37.56622 143.84957 Australia/Melbourne
-30.51667 151.65 Australia/Sydney
-36.08333 146.91667 Australia/Sydney
-27.48333 153.01667 Australia/Brisbane
-33.91667 151.24167 Australia/Sydney
-33.75 151.3 Australia/Sydney
-12.48602 130.9833 Australia/Darwin
-27.3171 153.03097 Australia/Brisbane
-25.90432 153.09174 Australia/Brisbane
-31.40237 152.90185 Australia/Sydney
-37.88264 144.7003 Australia/Melbourne
-27.63917 153.10944 Australia/Brisbane
-27.50578 153.10236 Australia/Brisbane
-33.8178 151.00348 Australia/Sydney
-37.88461 145.29539 Australia/Melbourne
-33.81667 151 Australia/Sydney
-34.91118 138.70735 Australia/Adelaide
-32.05798 115.91814 Australia/Perth
-33.79068 150.6693 Australia/Sydney
-37.85995 145.05489 Australia/Melbourne
-37.79086 145.09386 Australia/Melbourne
-37.88867 145.05713 Australia/Melbourne
-37.87397 145.04253 Australia/Melbourne
-37.90141 145.01884 Australia/Melbourne
-37.9201 145.05987 Australia/Melbourne
-27.61328 152.78944 Australia/Brisbane
-37.74496 144.80049 Australia/Melbourne
12.52398 -70.02703 America/Aruba
12.53333 -69.98333 America/Aruba
12.51667 -69.95 America/Aruba
60.09726 19.93481 Europe/Mariehamn
39.8177 46.7528 Asia/Baku
39.87867 48.39902 Asia/Baku
39.75893 46.74919 Asia/Baku
39.59362 48.98055 Asia/Baku
39.37417 49.24722 Asia/Baku
39.20889 45.41222 Asia/Baku
38.75428 48.85062 Asia/Ashgabat
39.87095 48.05995 Asia/Baku
39.20963 48.49186 Asia/Baku
39.45833 48.545 Asia/Baku
38.45598 48.87498 Asia/Baku
39.93788 48.92855 Asia/Baku
39.99096 46.92736 Asia/Baku
41.63361 46.64333 Asia/Baku
40.4761 49.94158 Asia/Baku
40.61832 47.15014 Asia/Baku
40.58694 46.31583 Asia/Baku
41.46349 48.80605 Asia/Baku
40.51902 47.65423 Asia/Baku
40.345 46.92889 Asia/Baku
40.58972 49.66861 Asia/Baku
40.39667 49.97361 Asia/Baku
40.82972 46.01889 Asia/Baku
40.63141 48.64137 Asia/Baku
41.19194 47.17056 Asia/Baku
40.4425 49.94806 Asia/Baku
40.00946 48.47515 Asia/Baku
41.42728 48.42893 Asia/Baku
41.36259 48.51519 Asia/Baku
40.04084 48.94451 Asia/Baku
41.09333 45.36611 Asia/Baku
40.62333 49.55861 Asia/Baku
40.76395 47.05953 Asia/Baku
40.53278 50.00389 Asia/Baku
40.49222 50.13889 Asia/Baku
40.32372 49.72804 Asia/Baku
40.65306 47.74056 Asia/Baku
40.37444 50.08528 Asia/Baku
40.68278 46.36056 Asia/Baku
40.51789 50.1139 Asia/Baku
40.45193 50.08564 Asia/Baku
40.37444 47.12667 Asia/Baku
40.4444 49.80566 Asia/Baku
40.37767 49.89201 Asia/Baku
40.42639 49.98361 Asia/Baku
40.57028 48.40087 Asia/Baku
40.65 47.47611 Asia/Baku
40.05278 47.46139 Asia/Baku
40.42167 49.96444 Asia/Baku
44.20139 17.90639 Europe/Sarajevo
43.98889 18.17806 Europe/Sarajevo
45.18944 15.81167 Europe/Sarajevo
44.54278 18.66944 Europe/Sarajevo
42.71194 18.34417 Europe/Sarajevo
44.22667 17.67056 Europe/Sarajevo
43.84864 18.35644 Europe/Sarajevo
44.98139 16.71306 Europe/Sarajevo
43.34333 17.80806 Europe/Sarajevo
44.88083 18.42694 Europe/Sarajevo
44.70333 18.30972 Europe/Sarajevo
43.66694 18.97639 Europe/Sarajevo
44.73472 18.08778 Europe/Sarajevo
44.96694 15.94306 Europe/Sarajevo
44.05722 17.45083 Europe/Sarajevo
44.87278 18.80833 Europe/Sarajevo
44.8825 16.15139 Europe/Sarajevo
44.75694 19.21611 Europe/Sarajevo
44.81694 15.87083 Europe/Sarajevo
44.77583 17.18556 Europe/Sarajevo
43.65194 17.96139 Europe/Sarajevo
13.1 -59.61667 America/Barbados
26.03333 88.46667 Asia/Dhaka
20.86243 92.30582 Asia/Dhaka
23.89 90.40583 Asia/Dhaka
24.89667 91.87167 Asia/Dhaka
23.58833 90.14694 Asia/Dhaka
24.91667 89.93333 Asia/Dhaka
24.68293 88.15975 Asia/Dhaka
22.71667 89.1 Asia/Dhaka
24.45 89.71667 Asia/Dhaka
24.88333 90.71667 Asia/Dhaka
23.92083 90.71806 Asia/Dhaka
22.46778 91.45611 Asia/Dhaka
24.16667 89.6 Asia/Dhaka
23.1 90.85 Asia/Dhaka
24.36667 88.6 Asia/Dhaka
22.56667 89.98333 Asia/Dhaka
26.33129 88.562 Asia/Dhaka
22.29312 91.97621 Asia/Dhaka
25.66491 88.9247 Asia/Dhaka
23.62333 90.50111 Asia/Dhaka
22.63333 90.28333 Asia/Dhaka
25.96881 89.69299 Asia/Dhaka
24.75 90.4 Asia/Dhaka
24.76667 90.23333 Asia/Dhaka
24.1 90.1 Asia/Dhaka
24.48333 91.78333 Asia/Dhaka
22.4537 89.8587 Asia/Dhaka
22.81667 90.53333 Asia/Dhaka
22.2855 89.9629 Asia/Dhaka
25.91244 89.44729 Asia/Dhaka
23.24722 91.1425 Asia/Dhaka
23.45778 91.20444 Asia/Dhaka
25.75 89.25 Asia/Dhaka
23.90105 89.12207 Asia/Dhaka
23.4095 89.1402 Asia/Dhaka
23.1131 89.0938 Asia/Dhaka
25.09433 89.02169 Asia/Dhaka
24.13333 89.08333 Asia/Dhaka
24.38333 91.41667 Asia/Dhaka
22.96667 90.23333 Asia/Dhaka
24.43333 90.56667 Asia/Dhaka
23.01861 91.41 Asia/Dhaka
23.05 90.76667 Asia/Dhaka
22.3082 89.7897 Asia/Dhaka
23.7104 90.40744 Asia/Dhaka
25.55592 89.67152 Asia/Dhaka
23.03 91.50694 Asia/Dhaka
22.3384 90.7419 Asia/Dhaka
23.1 91.96667 Asia/Dhaka
25.03333 91.66667 Asia/Dhaka
23.01531 89.4432 Asia/Dhaka
24.0232 88.9925 Asia/Dhaka
24.06667 90.96667 Asia/Dhaka
22.1965 92.2262 Asia/Dhaka
23.0396 89.6328 Asia/Dhaka
24.51667 91.36667 Asia/Dhaka
24.21667 90.95 Asia/Dhaka
25.67403 89.0549 Asia/Dhaka
23.16667 89.5 Asia/Dhaka
22.9 89.91667 Asia/Dhaka
24.73333 89.83333 Asia/Dhaka
24.3 90.18333 Asia/Dhaka
22.53702 91.91984 Asia/Dhaka
22.7 89.51667 Asia/Dhaka
23.21667 90.35 Asia/Dhaka
24.80287 88.95084 Asia/Dhaka
23.89167 90.97333 Asia/Dhaka
22.95 90.83333 Asia/Dhaka
22.91667 89.21667 Asia/Dhaka
23.25111 90.85917 Asia/Dhaka
23.6 89.83333 Asia/Dhaka
25.62715 88.63864 Asia/Dhaka
22.23322 90.71323 Asia/Dhaka
22.33306 91.83639 Asia/Dhaka
23.31667 90.21667 Asia/Dhaka
24.08333 89.61667 Asia/Dhaka
22.49184 90.72346 Asia/Dhaka
22.0796 92.0572 Asia/Dhaka
21.45324 91.97977 Asia/Dhaka
22.81348 89.56723 Asia/Dhaka
22.68333 90.65 Asia/Dhaka
22.70194 90.37111 Asia/Dhaka
23.16869 89.21315 Asia/Dhaka
24 89.25 Asia/Dhaka
24.25 89.91667 Asia/Dhaka
24.85 89.36667 Asia/Dhaka
25.8604 88.3635 Asia/Dhaka
24.59112 88.27102 Asia/Dhaka
23.17528 90.20722 Asia/Dhaka
25.01881 90.01751 Asia/Dhaka
24.43333 90.76667 Asia/Dhaka
22.8408 91.844 Asia/Dhaka
22.4861 90.0609 Asia/Dhaka
22.68726 91.7853 Asia/Dhaka
25.77769 88.89169 Asia/Dhaka
51.21979 4.32664 Europe/Brussels
50.81268 3.33848 Europe/Brussels
50.86956 3.81052 Europe/Brussels
50.99064 5.36819 Europe/Brussels
51.26825 4.71296 Europe/Brussels
50.98318 4.46079 Europe/Brussels
51.06566 4.0403 Europe/Brussels
51.14236 3.1368 Europe/Brussels
50.88365 4.47298 Europe/Brussels
51.39214 4.59546 Europe/Brussels
51.06041 4.36019 Europe/Brussels
50.8 3.16667 Europe/Brussels
51.00526 3.88341 Europe/Brussels
51.09049 4.91544 Europe/Brussels
50.78069 3.03854 Europe/Brussels
50.71717 4.60138 Europe/Brussels
50.71255 4.4014 Europe/Brussels
50.88898 3.42756 Europe/Brussels
50.25401 4.43796 Europe/Brussels
50.7376 5.69907 Europe/Brussels
50.92814 4.42938 Europe/Brussels
50.58907 5.86241 Europe/Brussels
51.32254 4.94471 Europe/Brussels
50.69059 4.2009 Europe/Brussels
50.60715 3.38932 Europe/Brussels
51.0656 3.10085 Europe/Brussels
50.78054 5.46484 Europe/Brussels
50.80744 4.9378 Europe/Brussels
50.99931 3.32707 Europe/Brussels
51.06513 5.08856 Europe/Brussels
50.82372 4.51418 Europe/Brussels
51.12794 4.21372 Europe/Brussels
51.2099 4.0365 Europe/Brussels
51.33189 4.37127 Europe/Brussels
50.61385 5.74679 Europe/Brussels
50.57904 4.07129 Europe/Brussels
50.81679 5.18647 Europe/Brussels
50.77926 4.24355 Europe/Brussels
51.16509 4.1437 Europe/Brussels
51.06691 4.53469 Europe/Brussels
51.21914 4.12374 Europe/Brussels
50.74645 4.35754 Europe/Brussels
50.58362 5.50115 Europe/Brussels
51.25251 4.50268 Europe/Brussels
51.24107 4.58336 Europe/Brussels
50.62837 5.53243 Europe/Brussels
50.44816 3.81886 Europe/Brussels
50.95302 4.71665 Europe/Brussels
50.74574 3.6005 Europe/Brussels
50.94653 3.12269 Europe/Brussels
50.71229 4.52529 Europe/Brussels
50.80995 5.60131 Europe/Brussels
51.18983 4.56533 Europe/Brussels
50.44067 3.8653 Europe/Brussels
51.07409 4.28844 Europe/Brussels
51.05337 4.63263 Europe/Brussels
50.85386 2.72659 Europe/Brussels
50.50518 4.36887 Europe/Brussels
50.50819 3.59373 Europe/Brussels
51.1303 5.45952 Europe/Brussels
50.77436 4.53461 Europe/Brussels
50.71184 5.6468 Europe/Brussels
50.85168 3.60891 Europe/Brussels
51.15432 3.23128 Europe/Brussels
51.22339 2.91185 Europe/Brussels
50.59833 4.32848 Europe/Brussels
50.82776 4.02657 Europe/Brussels
51.16096 4.67008 Europe/Brussels
51.22807 5.4427 Europe/Brussels
50.4669 4.86746 Europe/Brussels
50.74497 3.20639 Europe/Brussels
51.16697 4.45127 Europe/Brussels
50.45502 4.24519 Europe/Brussels
50.45413 3.95229 Europe/Brussels
51.19188 5.11662 Europe/Brussels
51.18532 2.82077 Europe/Brussels
50.99447 3.74621 Europe/Brussels
50.79722 3.12245 Europe/Brussels
50.93934 4.32655 Europe/Brussels
51.02574 4.47762 Europe/Brussels
50.22678 5.34416 Europe/Brussels
50.50312 4.23589 Europe/Brussels
51.20737 3.44511 Europe/Brussels
50.96545 5.69452 Europe/Brussels
51.09802 5.78379 Europe/Brussels
50.66829 4.61443 Europe/Brussels
51.00468 4.30304 Europe/Brussels
51.23074 5.31349 Europe/Brussels
51.10364 3.99339 Europe/Brussels
51.09644 3.83194 Europe/Brussels
51.24197 4.82312 Europe/Brussels
51.13128 4.57042 Europe/Brussels
50.63373 5.56749 Europe/Brussels
50.87959 4.70093 Europe/Brussels
50.71104 3.83579 Europe/Brussels
50.96626 3.98594 Europe/Brussels
51.00464 4.13457 Europe/Brussels
50.89318 5.6468 Europe/Brussels
50.48657 4.18785 Europe/Brussels
50.82803 3.26487 Europe/Brussels
50.88982 4.54353 Europe/Brussels
51.13213 4.44706 Europe/Brussels
51.11642 2.63772 Europe/Brussels
51.35 3.26667 Europe/Brussels
51.24118 4.96651 Europe/Brussels
51.31377 4.43539 Europe/Brussels
51.38442 4.47556 Europe/Brussels
50.91396 3.21378 Europe/Brussels
50.85114 2.88569 Europe/Brussels
50.51894 5.23284 Europe/Brussels
51.03427 5.37429 Europe/Brussels
51.40029 4.76034 Europe/Brussels
51.17611 4.34844 Europe/Brussels
51.03664 5.28013 Europe/Brussels
50.88681 3.89014 Europe/Brussels
50.64083 5.79353 Europe/Brussels
50.66415 5.62346 Europe/Brussels
51.17655 4.83248 Europe/Brussels
50.90861 4.67056 Europe/Brussels
51.05591 5.38244 Europe/Brussels
51.07537 4.72827 Europe/Brussels
50.93106 5.33781 Europe/Brussels
50.85343 3.30935 Europe/Brussels
51.09822 4.13705 Europe/Brussels
50.73385 4.23454 Europe/Brussels
50.90634 4.00093 Europe/Brussels
50.93409 4.37213 Europe/Brussels
50.77343 3.88223 Europe/Brussels
51.05 3.71667 Europe/Brussels
50.965 5.50082 Europe/Brussels
50.56149 4.69889 Europe/Brussels
51.16557 4.98917 Europe/Brussels
50.40578 3.89603 Europe/Brussels
50.48351 4.55006 Europe/Brussels
50.61516 5.68062 Europe/Brussels
50.59994 5.44471 Europe/Brussels
51.1053 3.704 Europe/Brussels
50.6279 6.03647 Europe/Brussels
51.46272 4.45393 Europe/Brussels
51.18703 3.55654 Europe/Brussels
51.15662 4.44504 Europe/Brussels
51.09554 4.50903 Europe/Brussels
50.39583 3.77792 Europe/Brussels
50.84799 4.25972 Europe/Brussels
51.03248 2.86384 Europe/Brussels
50.98923 5.05062 Europe/Brussels
50.90769 5.41875 Europe/Brussels
51.05952 3.79899 Europe/Brussels
51.02869 4.10106 Europe/Brussels
50.88506 4.07601 Europe/Brussels
50.98175 3.53096 Europe/Brussels
50.46379 4.3747 Europe/Brussels
50.4141 3.85569 Europe/Brussels
50.5828 5.6341 Europe/Brussels
50.40338 4.52826 Europe/Brussels
50.42842 3.95001 Europe/Brussels
50.41136 4.44448 Europe/Brussels
50.85045 4.34878 Europe/Brussels
51.20892 3.22424 Europe/Brussels
51.35024 4.63829 Europe/Brussels
51.2912 4.49182 Europe/Brussels
50.60979 4.14658 Europe/Brussels
50.68363 4.36784 Europe/Brussels
50.43417 3.7944 Europe/Brussels
51.09716 4.24364 Europe/Brussels
51.09242 4.3717 Europe/Brussels
51.31306 3.13227 Europe/Brussels
50.41155 4.16469 Europe/Brussels
50.87325 5.5184 Europe/Brussels
51.21187 4.25633 Europe/Brussels
51.04954 5.22606 Europe/Brussels
50.76589 4.3002 Europe/Brussels
51.31927 4.85304 Europe/Brussels
51.16837 5.17027 Europe/Brussels
50.62937 3.77801 Europe/Brussels
50.91011 4.19836 Europe/Brussels
49.68333 5.81667 Europe/Brussels
51.21989 4.40346 Europe/Brussels
50.6623 5.52029 Europe/Brussels
50.48941 5.09513 Europe/Brussels
50.98715 4.83695 Europe/Brussels
51.09017 3.44693 Europe/Brussels
50.93604 4.0355 Europe/Brussels
12.24889 -0.61583 Africa/Ouagadougou
12.96667 -2.26667 Africa/Ouagadougou
13.06667 -3.06667 Africa/Ouagadougou
13.76667 -2.06667 Africa/Ouagadougou
11.78 -0.36972 Africa/Ouagadougou
12.31667 -2.46667 Africa/Ouagadougou
11.16972 -1.145 Africa/Ouagadougou
13.58278 -2.42158 Africa/Ouagadougou
12.36423 -1.53834 Africa/Ouagadougou
12.73333 -3.86667 Africa/Ouagadougou
11.66361 -1.07306 Africa/Ouagadougou
11.1 -2.1 Africa/Ouagadougou
12.17944 -0.35167 Africa/Ouagadougou
12.25 -2.36667 Africa/Ouagadougou
13.32583 -1.53472 Africa/Ouagadougou
12.06556 -1.3375 Africa/Ouagadougou
12.18972 -1.88556 Africa/Ouagadougou
13.09167 -1.08444 Africa/Ouagadougou
11.5 -3.51667 Africa/Ouagadougou
13.21667 -2.35 Africa/Ouagadougou
11.8 -0.55056 Africa/Ouagadougou
12.06222 0.35778 Africa/Ouagadougou
14.0354 -0.0345 Africa/Ouagadougou
14.10222 -1.63056 Africa/Ouagadougou
12.07083 1.78889 Africa/Ouagadougou
12.46667 -3.46667 Africa/Ouagadougou
12.66056 -1.89222 Africa/Ouagadougou
12.6675 -0.57583 Africa/Ouagadougou
11.17715 -4.2979 Africa/Ouagadougou
10.63333 -4.76667 Africa/Ouagadougou
10.29917 -3.25083 Africa/Ouagadougou
10.94917 -4.93417 Africa/Ouagadougou
42.48333 26.5 Europe/Sofia
43.21 23.5625 Europe/Sofia
43.99 22.8725 Europe/Sofia
42.02724 23.99569 Europe/Sofia
43.08124 25.62904 Europe/Sofia
43.21667 27.91667 Europe/Sofia
43.25917 26.58917 Europe/Sofia
42.89427 24.71589 Europe/Sofia
42.9 25.31667 Europe/Sofia
43.56667 27.83333 Europe/Sofia
43.62306 25.35389 Europe/Sofia
41.76667 26.2 Europe/Sofia
42.43278 25.64194 Europe/Sofia
42.26667 23.11667 Europe/Sofia
42.69751 23.32415 Europe/Sofia
41.58528 24.69194 Europe/Sofia
42.68583 26.32917 Europe/Sofia
44.11667 27.26667 Europe/Sofia
43.27064 26.92286 Europe/Sofia
43.02583 25.11361 Europe/Sofia
41.56667 23.28333 Europe/Sofia
42.33333 23.55 Europe/Sofia
43.85639 25.97083 Europe/Sofia
43.53333 26.51667 Europe/Sofia
42.3 24.96667 Europe/Sofia
43.35 26.23333 Europe/Sofia
42.15 24.75 Europe/Sofia
43.41667 24.61667 Europe/Sofia
41.4 23.21667 Europe/Sofia
42.03333 24.3 Europe/Sofia
42.6 23.03333 Europe/Sofia
42.2 24.33333 Europe/Sofia
42.5 24.18333 Europe/Sofia
42.48333 26.01667 Europe/Sofia
43.4125 23.225 Europe/Sofia
43.13333 24.71667 Europe/Sofia
43.81389 23.23611 Europe/Sofia
42.28389 22.69111 Europe/Sofia
41.65 25.36667 Europe/Sofia
41.94028 25.56944 Europe/Sofia
41.93333 25.9 Europe/Sofia
42.61667 25.4 Europe/Sofia
42.65 26.98333 Europe/Sofia
42.63333 24.8 Europe/Sofia
41.56667 23.73333 Europe/Sofia
43.12778 25.70167 Europe/Sofia
42.87472 25.33417 Europe/Sofia
42.05 25.6 Europe/Sofia
42.2 25.33333 Europe/Sofia
43.26667 24.1 Europe/Sofia
42.50606 27.46781 Europe/Sofia
42.9 23.78333 Europe/Sofia
42.01667 23.1 Europe/Sofia
43.23611 23.12583 Europe/Sofia
42.7 27.25 Europe/Sofia
42.01667 24.86667 Europe/Sofia
26.15472 50.62056 Asia/Bahrain
26.17361 50.54778 Asia/Bahrain
26.21861 50.54778 Asia/Bahrain
26.11528 50.50694 Asia/Bahrain
26.06861 50.50389 Asia/Bahrain
26.25722 50.61194 Asia/Bahrain
26.21536 50.5832 Asia/Bahrain
26.13 50.555 Asia/Bahrain
-4.1348 29.804 Africa/Bujumbura
-3.94877 29.62438 Africa/Bujumbura
-3.3822 29.3644 Africa/Bujumbura
-3.2682 29.6079 Africa/Bujumbura
-3.4264 29.9308 Africa/Bujumbura
-3.47639 30.24861 Africa/Bujumbura
-2.9075 29.8306 Africa/Bujumbura
-2.9221 29.6293 Africa/Bujumbura
-2.8451 30.3414 Africa/Bujumbura
-3.9279 29.992 Africa/Bujumbura
8.88649 2.59752 Africa/Porto-Novo
10.62118 1.26651 Africa/Porto-Novo
8.04253 2.48599 Africa/Porto-Novo
7.92807 1.97558 Africa/Porto-Novo
6.73618 2.65866 Africa/Porto-Novo
6.49646 2.60359 Africa/Porto-Novo
6.98008 2.6649 Africa/Porto-Novo
9.33716 2.63031 Africa/Porto-Novo
6.36307 2.08506 Africa/Porto-Novo
9.94009 3.21075 Africa/Porto-Novo
10.30416 1.37962 Africa/Porto-Novo
11.86846 3.38989 Africa/Porto-Novo
6.63869 1.71674 Africa/Porto-Novo
7.36332 2.59978 Africa/Porto-Novo
11.13417 2.93861 Africa/Porto-Novo
6.81667 1.78333 Africa/Porto-Novo
9.70853 1.66598 Africa/Porto-Novo
7.75 2.18333 Africa/Porto-Novo
7.22097 2.34017 Africa/Porto-Novo
6.36536 2.41833 Africa/Porto-Novo
6.40764 1.88198 Africa/Porto-Novo
7.17826 2.0667 Africa/Porto-Novo
10.22827 2.66335 Africa/Porto-Novo
9.00814 1.6654 Africa/Porto-Novo
11.29845 2.43856 Africa/Porto-Novo
6.93333 1.68333 Africa/Porto-Novo
6.66547 2.15138 Africa/Porto-Novo
6.44852 2.35566 Africa/Porto-Novo
7.18286 1.99119 Africa/Porto-Novo
17.89618 -62.84978 America/St_Barthelemy
32.29149 -64.77797 Atlantic/Bermuda
4.8 114.65 Asia/Brunei
4.61667 114.31667 Asia/Brunei
4.58333 114.18333 Asia/Brunei
4.94029 114.94806 Asia/Brunei
-22.03799 -63.67968 America/Argentina/Salta
-17.5 -63.16667 America/La_Paz
-22.08659 -65.59422 America/La_Paz
-17.4 -63.83333 America/La_Paz
-21.25 -63.5 America/La_Paz
-21.45 -65.71667 America/La_Paz
-14.83333 -64.9 America/La_Paz
-21.53549 -64.72956 America/La_Paz
-19.03332 -65.26274 America/La_Paz
-17.98333 -63.38333 America/La_Paz
-17.8 -63.16667 America/La_Paz
-16.36667 -60.95 America/La_Paz
-14.81667 -66.85 America/La_Paz
-10.98333 -66.1 America/La_Paz
-17.55 -65.83333 America/La_Paz
-19.58361 -65.75306 America/La_Paz
-17.98333 -67.15 America/La_Paz
-17.33333 -63.25 America/La_Paz
-17.94101 -65.34016 America/La_Paz
-18.41667 -66.63333 America/La_Paz
-16.5 -68.15 America/La_Paz
-18.26667 -66.85 America/La_Paz
-10.83676 -65.36136 America/La_Paz
-17.81667 -63.05 America/La_Paz
-17.3895 -66.1568 America/La_Paz
-11.02671 -68.76918 America/La_Paz
-20.05 -63.51667 America/La_Paz
-3.46222 -44.87056 America/Fortaleza
-8.11806 -35.29139 America/Recife
-1.19667 -46.14 America/Santarem
-7.25972 -34.9075 America/Fortaleza
-0.85833 -48.14167 America/Santarem
-3.56222 -41.09222 America/Fortaleza
-9.37139 -36.24083 America/Maceio
-3.22028 -45.00361 America/Fortaleza
-5.35 -40.38333 America/Fortaleza
-4.19444 -40.47667 America/Fortaleza
-3.54306 -43.91583 America/Fortaleza
-6.4075 -41.74556 America/Fortaleza
-9.16278 -36.03194 America/Maceio
-4.58583 -42.86417 America/Fortaleza
-5.25806 -44.64889 America/Fortaleza
-3.76611 -49.6725 America/Santarem
-0.18333 -50.05 America/Santarem
-7.76194 -40.26778 America/Recife
-3.27778 -39.26889 America/Fortaleza
-8.00667 -36.05667 America/Recife
-2.41889 -48.15222 America/Santarem
-5.09417 -42.83667 America/Fortaleza
-4.25528 -43.94056 America/Fortaleza
-7.50528 -35.31833 America/Recife
-3.73222 -40.99167 America/Fortaleza
-5.08917 -42.80194 America/Fortaleza
-6 -40.28333 America/Fortaleza
-8.75972 -35.10472 America/Recife
-7.59083 -37.53944 America/Recife
-7.83306 -35.75472 America/Recife
-6.75917 -38.22806 America/Fortaleza
-0.71667 -48.52333 America/Santarem
-6.755 -35.54 America/Fortaleza
-3.68611 -40.34972 America/Fortaleza
-8.07361 -37.26444 America/Recife
-7.99194 -38.29833 America/Recife
-5.58806 -39.37167 America/Fortaleza
-9.56333 -35.82444 America/Maceio
-9.01528 -42.69944 America/Fortaleza
-9.78111 -36.09361 America/Maceio
-1.62667 -47.48333 America/Santarem
-4.04167 -44.475 America/Fortaleza
-9.31833 -35.56111 America/Maceio
-2.52972 -44.30278 America/Fortaleza
-8.00222 -35.01833 America/Recife
-7.47889 -37.27444 America/Recife
-2.56194 -44.05417 America/Fortaleza
-6.07472 -35.23778 America/Fortaleza
-6.495 -43.70222 America/Fortaleza
-3.60722 -38.96833 America/Fortaleza
-6.64472 -51.995 America/Santarem
-5.57583 -44.38528 America/Fortaleza
-6.28333 -37.75 America/Fortaleza
-2.69583 -44.82139 America/Fortaleza
-7.11389 -34.97806 America/Fortaleza
-2.44306 -54.70833 America/Santarem
-3.51556 -42.54667 America/Fortaleza
-4.33194 -40.15667 America/Fortaleza
-9.37833 -37.24528 America/Maceio
-7.63333 -47.2 America/Fortaleza
-3.66667 -45.38 America/Fortaleza
-2.23111 -45.3 America/Fortaleza
-7.9575 -36.20472 America/Recife
-6.22944 -36.02278 America/Fortaleza
-0.61361 -47.35611 America/Santarem
-8.07417 -39.11917 America/Recife
-4.94028 -37.97583 America/Fortaleza
-9.47833 -35.85333 America/Maceio
-8.66361 -35.15861 America/Recife
-8.51444 -35.37778 America/Recife
-8.05389 -34.88111 America/Recife
-5.19917 -39.29278 America/Fortaleza
-4.97139 -39.01528 America/Fortaleza
-5.29 -44.49 America/Fortaleza
-0.05833 -51.18167 America/Belem
-9.045 -35.39833 America/Maceio
-1.93556 -50.82111 America/Belem
-8.14139 -35.39583 America/Recife
-6.77028 -37.80167 America/Fortaleza
-4.27333 -41.77694 America/Fortaleza
-3.92806 -41.70917 America/Fortaleza
-2.52139 -45.0825 America/Fortaleza
-3.60833 -45.34333 America/Fortaleza
-9.59722 -35.95667 America/Maceio
-7.07694 -41.46694 America/Fortaleza
-9.39861 -40.50083 America/Recife
-9.18306 -38.26889 America/Recife
-8.35778 -36.69639 America/Recife
-3.79278 -39.27028 America/Fortaleza
-3.29417 -45.17361 America/Fortaleza
-4.42472 -41.45861 America/Fortaleza
-5.45417 -39.71722 America/Fortaleza
-9.40611 -38.21472 America/Bahia
-7.94083 -34.87306 America/Recife
-7.02444 -37.28 America/Fortaleza
-5.91556 -35.26278 America/Fortaleza
-2.90472 -41.77667 America/Fortaleza
-2.62833 -56.73583 America/Manaus
-6.68778 -36.6575 America/Fortaleza
-3.43944 -39.14833 America/Fortaleza
-2.96667 -47.48333 America/Santarem
-3.41 -39.03056 America/Fortaleza
-8.68333 -35.59167 America/Recife
-3.98417 -38.62028 America/Fortaleza
-4.1725 -38.46056 America/Fortaleza
-7.8825 -40.08167 America/Recife
-6.24444 -38.91361 America/Fortaleza
-1.76556 -55.86611 America/Santarem
-8.00889 -34.85528 America/Recife
-7.02528 -42.13111 America/Fortaleza
-1.9175 -55.51806 America/Santarem
-4.70667 -40.56306 America/Fortaleza
-6.47806 -35.43389 America/Fortaleza
-7.74167 -35.22778 America/Recife
-5.795 -35.20944 America/Fortaleza
-9.30667 -35.94333 America/Maceio
-8.11861 -35.09222 America/Recife
-5.10667 -38.3725 America/Fortaleza
-7.88944 -37.12 America/Fortaleza
-2.00778 -54.06917 America/Santarem
-5.74306 -39.6275 America/Fortaleza
-1.88389 -48.76889 America/Santarem
-5.1875 -37.34417 America/Fortaleza
-2.58417 -49.50722 America/Santarem
-3.38361 -57.71861 America/Manaus
-9.15167 -35.53333 America/Maceio
-7.06 -35.31944 America/Fortaleza
-9.71028 -35.895 America/Maceio
-9.01222 -35.2225 America/Maceio
-3.87667 -38.62556 America/Fortaleza
-5.36861 -49.11778 America/Santarem
-6.83861 -35.12611 America/Fortaleza
-9.66583 -35.73528 America/Maceio
-5.115 -36.63444 America/Fortaleza
0.03889 -51.06639 America/Belem
-5.85833 -35.35389 America/Fortaleza
-5.14556 -38.09806 America/Fortaleza
-7.87472 -35.45028 America/Recife
-6.75333 -38.96444 America/Fortaleza
-8.66361 -36.32 America/Recife
-4.33333 -45.16667 America/Fortaleza
-7.93611 -35.29028 America/Recife
-7.21306 -39.31528 America/Fortaleza
-4.75639 -42.57556 America/Fortaleza
-7.115 -34.86306 America/Fortaleza
-5.5375 -35.81972 America/Fortaleza
-4.83389 -37.78111 America/Fortaleza
-5.89056 -38.62194 America/Fortaleza
-8.18028 -35.00139 America/Recife
-5.13472 -49.32667 America/Santarem
-3.14306 -58.44417 America/Manaus
-7.30444 -38.15028 America/Fortaleza
-7.77639 -34.89222 America/Recife
-3.49444 -39.57861 America/Fortaleza
-3.3925 -44.35861 America/Fortaleza
-3.68667 -39.58611 America/Fortaleza
-4.27611 -55.98361 America/Santarem
-3.96944 -38.52806 America/Fortaleza
-7.32861 -35.3325 America/Fortaleza
-7.03333 -40.45 America/Fortaleza
-7.65194 -40.14889 America/Recife
-4.32222 -40.71083 America/Fortaleza
-8.39889 -35.06389 America/Recife
-5.52639 -47.49167 America/Fortaleza
-6.35944 -39.29861 America/Fortaleza
-1.975 -48.95972 America/Santarem
-1.12889 -47.62 America/Santarem
-7.83417 -34.90639 America/Recife
-6.40111 -38.86222 America/Fortaleza
-4.1 -38.48333 America/Fortaleza
-4.16694 -40.7475 America/Fortaleza
-6.85472 -35.49 America/Fortaleza
-4.03972 -38.63722 America/Fortaleza
-8.20111 -35.56472 America/Recife
-3.12028 -40.82611 America/Fortaleza
-5.81944 -46.13861 America/Fortaleza
-7.56056 -35.0025 America/Recife
-8.00167 -35.29278 America/Recife
-8.89028 -36.49278 America/Recife
-8.58444 -35.38667 America/Recife
-3.71722 -38.54306 America/Fortaleza
-6.76694 -43.0225 America/Fortaleza
-8.60111 -38.56861 America/Recife
-3.89 -38.45056 America/Fortaleza
-5.70556 -35.30722 America/Fortaleza
-5.78333 -43.25 America/Fortaleza
-3.90167 -42.23361 America/Fortaleza
-7.03306 -35.85722 America/Fortaleza
-8.35917 -35.22361 America/Recife
-4.48333 -44.45 America/Fortaleza
-5.35833 -42.67639 America/Fortaleza
-9.38861 -37.99917 America/Maceio
-8.0875 -37.64306 America/Recife
-1.82833 -44.86833 America/Fortaleza
-6.26083 -36.51778 America/Fortaleza
-8.61667 -35.95 America/Recife
-7.23417 -39.40944 America/Fortaleza
-5.17833 -40.6775 America/Fortaleza
-4.13 -44.12417 America/Fortaleza
-7.58583 -35.10583 America/Recife
-8.25778 -49.26472 America/Araguaina
-6.02583 -44.24917 America/Fortaleza
-4.25667 -43.01278 America/Fortaleza
-4.45528 -43.88556 America/Fortaleza
-3.74167 -43.36028 America/Fortaleza
-8.23833 -35.46167 America/Recife
-5.63444 -35.42556 America/Fortaleza
-3.73611 -38.65306 America/Fortaleza
-6.34389 -37.74667 America/Fortaleza
-8.66667 -35.71667 America/Recife
-1.29389 -47.92639 America/Santarem
-4.13306 -38.24194 America/Fortaleza
-8.28333 -35.97611 America/Recife
-7.85083 -35.25472 America/Recife
-7.33278 -47.46944 America/Fortaleza
-1.74639 -47.05944 America/Santarem
-1.19583 -47.18083 America/Santarem
-4.35889 -39.31167 America/Fortaleza
-6.38 -35.12889 America/Fortaleza
-7.07444 -40.37611 America/Fortaleza
-4.82778 -42.16861 America/Fortaleza
-9.78194 -36.35083 America/Maceio
-7.23056 -35.88111 America/Fortaleza
-2.90222 -40.84111 America/Fortaleza
-2.24444 -49.49583 America/Belem
-9.71667 -36.45 America/Maceio
-6.89028 -38.55528 America/Fortaleza
-6.45833 -37.09778 America/Fortaleza
-8.51417 -39.31 America/Recife
-8.28333 -35.03333 America/Recife
-6.98111 -34.83389 America/Fortaleza
-5.83722 -43.83361 America/Fortaleza
-8.62306 -37.15583 America/Recife
-1.68222 -50.48028 America/Santarem
-7.49333 -38.98722 America/Fortaleza
-8.14583 -36.37111 America/Recife
-1.05361 -46.76556 America/Santarem
-9.16972 -36.67972 America/Recife
-5.1275 -39.73222 America/Fortaleza
-8.23333 -35.79694 America/Recife
-1.36139 -48.24472 America/Santarem
-8.33556 -36.42417 America/Recife
-6.69167 -35.53333 America/Fortaleza
-1.45583 -48.50444 America/Belem
-4.17972 -38.13056 America/Fortaleza
-7.125 -34.93222 America/Fortaleza
-4.32861 -38.88472 America/Fortaleza
-8.81833 -35.18639 America/Recife
-2.74694 -42.82639 America/Fortaleza
-4.24444 -42.29444 America/Fortaleza
-5.5 -45.25 America/Fortaleza
-1.50583 -48.62583 America/Santarem
-7.31111 -39.30417 America/Fortaleza
-7.5325 -46.03556 America/Fortaleza
-4.29167 -44.79167 America/Fortaleza
-1.02167 -46.635 America/Santarem
-9.50194 -36.02278 America/Maceio
-4.95611 -37.13694 America/Fortaleza
-8.41889 -37.05389 America/Recife
-7.57611 -40.49833 America/Recife
-3.45361 -44.78 America/Fortaleza
-9.7525 -36.66111 America/Maceio
-7.19111 -48.20722 America/Araguaina
-4.56167 -37.76972 America/Fortaleza
-3.90139 -38.39111 America/Fortaleza
-5.66417 -37.79889 America/Fortaleza
-1.36556 -48.37222 America/Santarem
-8.38306 -35.4525 America/Recife
-5.03806 -42.46 America/Fortaleza
-3.20333 -52.20639 America/Santarem
-9.9 -55.9 America/Cuiaba
-1.52333 -52.58167 America/Santarem
-1.94167 -54.73833 America/Santarem
-7.15833 -35.63 America/Fortaleza
-9.11139 -37.12306 America/Recife
-8.7075 -35.53056 America/Recife
-7.75083 -37.63917 America/Recife
-5.57667 -36.90861 America/Fortaleza
-6.09528 -39.4525 America/Fortaleza
-2.88556 -40.12 America/Fortaleza
-7.91167 -34.90278 America/Recife
-1.71806 -48.8825 America/Santarem
-10.82167 -42.73111 America/Bahia
-26.87694 -52.40417 America/Sao_Paulo
-20.42278 -49.97278 America/Sao_Paulo
-23.54667 -47.43778 America/Sao_Paulo
-22.52306 -44.10417 America/Sao_Paulo
-14.86611 -40.83944 America/Bahia
-20.31944 -40.33778 America/Sao_Paulo
-21.01028 -42.84056 America/Sao_Paulo
-20.87306 -48.29694 America/Sao_Paulo
-23.02972 -46.97528 America/Sao_Paulo
-20.32972 -40.2925 America/Sao_Paulo
-27.00833 -51.15167 America/Sao_Paulo
-20.75389 -42.88194 America/Sao_Paulo
-20.39028 -40.49611 America/Sao_Paulo
-30.08111 -51.02333 America/Sao_Paulo
-19.69194 -43.92333 America/Sao_Paulo
-28.93611 -51.54944 America/Sao_Paulo
-12.63333 -41.03333 America/Bahia
-23.87389 -49.80278 America/Sao_Paulo
-29.60639 -52.19194 America/Sao_Paulo
-17.98694 -46.90778 America/Sao_Paulo
-22.40389 -43.6625 America/Sao_Paulo
-23.21139 -46.82833 America/Sao_Paulo
-15.64667 -56.1325 America/Cuiaba
-17.59806 -44.73083 America/Sao_Paulo
-21.55139 -45.43028 America/Sao_Paulo
-21.83222 -46.89361 America/Sao_Paulo
-21.22778 -50.86833 America/Sao_Paulo
-22.97056 -46.99583 America/Sao_Paulo
-22.24556 -43.70028 America/Sao_Paulo
-13.37028 -39.07306 America/Bahia
-28.51222 -50.93389 America/Sao_Paulo
-29.75472 -57.08833 America/Sao_Paulo
-14.59306 -39.28444 America/Bahia
-14.52472 -49.14083 America/Sao_Paulo
-26.23 -51.08639 America/Sao_Paulo
-16.3575 -46.90611 America/Sao_Paulo
-15.29333 -39.07528 America/Bahia
-23.76639 -53.325 America/Sao_Paulo
-18.91861 -48.27722 America/Sao_Paulo
-19.74833 -47.93194 America/Sao_Paulo
-23.43389 -45.07111 America/Sao_Paulo
-14.21389 -39.52278 America/Bahia
-14.3125 -39.32333 America/Bahia
-21.12 -42.94278 America/Sao_Paulo
-29.08056 -53.83583 America/Sao_Paulo
-18.59222 -48.705 America/Sao_Paulo
-21.93472 -50.51361 America/Sao_Paulo
-10.96306 -38.78667 America/Bahia
-28.46667 -49.00694 America/Sao_Paulo
-16.64944 -49.48889 America/Sao_Paulo
-22.11667 -43.20917 America/Sao_Paulo
-21.36667 -45.5125 America/Sao_Paulo
-27.45556 -53.93194 America/Sao_Paulo
-20.75111 -51.67833 America/Campo_Grande
-27.77333 -54.24 America/Sao_Paulo
-29.51694 -50.77778 America/Sao_Paulo
-21.69694 -45.25333 America/Sao_Paulo
-22.95833 -45.54944 America/Sao_Paulo
-29.98472 -50.13361 America/Sao_Paulo
-29.33528 -49.72694 America/Sao_Paulo
-24.71361 -53.74306 America/Sao_Paulo
-11.18389 -37.99833 America/Maceio
-19.5825 -42.64444 America/Sao_Paulo
-26.82333 -49.27167 America/Sao_Paulo
-27.24139 -48.63361 America/Sao_Paulo
-23.10194 -47.71472 America/Sao_Paulo
-29.44806 -51.80639 America/Sao_Paulo
-22.41222 -42.96556 America/Sao_Paulo
-17.8575 -41.50528 America/Sao_Paulo
-22.5325 -52.1675 America/Sao_Paulo
-24.32389 -50.61556 America/Sao_Paulo
-23.02639 -45.55528 America/Sao_Paulo
-23.35556 -47.85694 America/Sao_Paulo
-23.53306 -49.24444 America/Sao_Paulo
-21.40611 -48.50472 America/Sao_Paulo
-29.79972 -51.86444 America/Sao_Paulo
-29.65056 -50.78056 America/Sao_Paulo
-11.84722 -40.79139 America/Bahia
-30.67333 -51.39583 America/Sao_Paulo
-22.73028 -42.71417 America/Sao_Paulo
-20.62639 -49.64917 America/Sao_Paulo
-21.705 -47.27444 America/Sao_Paulo
-15.80778 -42.23306 America/Sao_Paulo
-23.62611 -46.79167 America/Sao_Paulo
-23.5425 -46.31083 America/Sao_Paulo
-22.82194 -47.26694 America/Sao_Paulo
-23.50167 -47.45806 America/Sao_Paulo
-28.81833 -52.51028 America/Sao_Paulo
-22.59139 -46.52889 America/Sao_Paulo
-12.83333 -39.1 America/Bahia
-10.73833 -37.81111 America/Maceio
-22.65083 -42.39167 America/Sao_Paulo
-20.93194 -54.96139 America/Campo_Grande
-19.46583 -44.24667 America/Sao_Paulo
-21.13778 -47.99028 America/Sao_Paulo
-11.66417 -39.0075 America/Bahia
-22.61222 -46.70056 America/Sao_Paulo
-21.21139 -47.59556 America/Sao_Paulo
-20.12861 -40.30778 America/Sao_Paulo
-22.74389 -43.7075 America/Sao_Paulo
-10.46139 -40.18944 America/Bahia
-16.70806 -49.09306 America/Sao_Paulo
-12.41861 -41.77028 America/Bahia
-26.4125 -49.07306 America/Sao_Paulo
-12.7375 -38.76861 America/Bahia
-20.03528 -44.14472 America/Sao_Paulo
-27.94389 -52.92306 America/Sao_Paulo
-23.44361 -51.87389 America/Sao_Paulo
-22.92 -42.51028 America/Sao_Paulo
-29.83333 -51.15 America/Sao_Paulo
-29.63806 -51.00694 America/Sao_Paulo
-23.96306 -46.39194 America/Sao_Paulo
-30.16056 -53.56528 America/Sao_Paulo
-12.5125 -38.49528 America/Bahia
-20.91694 -46.99139 America/Sao_Paulo
-29.58667 -51.37556 America/Sao_Paulo
-23.76 -45.40972 America/Sao_Paulo
-23.52917 -47.13528 America/Sao_Paulo
-22.83917 -42.10278 America/Sao_Paulo
-22.54861 -47.91389 America/Sao_Paulo
-23.5475 -46.63611 America/Sao_Paulo
-25.34806 -54.23778 America/Sao_Paulo
-13.275 -50.16278 America/Sao_Paulo
-25.87417 -50.38278 America/Sao_Paulo
-18.71611 -39.85889 America/Sao_Paulo
-28.97111 -51.06806 America/Sao_Paulo
-22.73111 -48.57056 America/Sao_Paulo
-28.40833 -54.96083 America/Sao_Paulo
-16.525 -50.37222 America/Sao_Paulo
-31.36528 -51.97833 America/Sao_Paulo
-22.11639 -45.05444 America/Sao_Paulo
-29.76028 -51.14722 America/Sao_Paulo
-25.53472 -49.20639 America/Sao_Paulo
-23.17944 -45.88694 America/Sao_Paulo
-20.81972 -49.37944 America/Sao_Paulo
-21.59556 -46.88861 America/Sao_Paulo
-28.23333 -49.16667 America/Sao_Paulo
-20.58139 -47.85472 America/Sao_Paulo
-28.29389 -49.93167 America/Sao_Paulo
-21.54 -43.01056 America/Sao_Paulo
-22.80389 -43.37222 America/Sao_Paulo
-21.13556 -44.26167 America/Sao_Paulo
-21.96917 -46.79806 America/Sao_Paulo
-21.64028 -41.05111 America/Sao_Paulo
-29.95917 -51.72222 America/Sao_Paulo
-19.31111 -46.04889 America/Sao_Paulo
-21.89222 -45.59528 America/Sao_Paulo
-30.33639 -54.32 America/Sao_Paulo
-26.24333 -48.63806 America/Sao_Paulo
-12.6275 -38.68 America/Bahia
-15.94861 -44.86444 America/Sao_Paulo
-21.64611 -41.74694 America/Sao_Paulo
-11.01472 -37.20639 America/Maceio
-22.0175 -47.89083 America/Sao_Paulo
-23.62306 -46.55111 America/Sao_Paulo
-28.66056 -56.00444 America/Sao_Paulo
-23.69389 -46.565 America/Sao_Paulo
-26.25028 -49.37861 America/Sao_Paulo
-21.45667 -43.5525 America/Sao_Paulo
-23.96083 -46.33361 America/Sao_Paulo
-12.43028 -39.25139 America/Bahia
-20.08722 -45.29361 America/Sao_Paulo
-20.94639 -44.91889 America/Sao_Paulo
-22.60611 -46.91944 America/Sao_Paulo
-21.53944 -42.18028 America/Sao_Paulo
-12.96889 -39.26139 America/Bahia
-23.295 -50.07722 America/Sao_Paulo
-28.29917 -54.26306 America/Sao_Paulo
-23.66389 -46.53833 America/Sao_Paulo
-21.83833 -51.61028 America/Sao_Paulo
-27.68806 -48.77861 America/Sao_Paulo
-12.54667 -38.71194 America/Bahia
-29.19167 -54.86722 America/Sao_Paulo
-33.51889 -53.36806 America/Sao_Paulo
-21.47278 -47.36306 America/Sao_Paulo
-27.87083 -54.48139 America/Sao_Paulo
-22.25222 -45.70333 America/Sao_Paulo
-21.71028 -47.47806 America/Sao_Paulo
-19.36361 -42.56861 America/Sao_Paulo
-30.89083 -55.53278 America/Sao_Paulo
-23.44417 -46.91778 America/Sao_Paulo
-13.39472 -44.18861 America/Bahia
-29.68417 -53.80694 America/Sao_Paulo
-19.76972 -43.85139 America/Sao_Paulo
-11.25583 -39.37472 America/Bahia
-23.31556 -46.22139 America/Sao_Paulo
-17.81361 -50.59694 America/Sao_Paulo
-22.45667 -47.53028 America/Sao_Paulo
-20.21111 -50.92583 America/Sao_Paulo
-29.7175 -52.42583 America/Sao_Paulo
-22.89889 -49.6325 America/Sao_Paulo
-21.82694 -47.24861 America/Sao_Paulo
-16.27806 -39.02472 America/Bahia
-26.96083 -50.42694 America/Sao_Paulo
-22.75361 -47.41361 America/Sao_Paulo
-12.97111 -38.51083 America/Bahia
-23.64889 -47.57333 America/Sao_Paulo
-23.20083 -47.28694 America/Sao_Paulo
-16.17028 -42.29028 America/Sao_Paulo
-19.86528 -47.44 America/Sao_Paulo
-12.28389 -40.49389 America/Bahia
-15.16444 -49.80333 America/Sao_Paulo
-30.25833 -54.91417 America/Sao_Paulo
-16.47083 -54.63556 America/Cuiaba
-29.65056 -50.57583 America/Sao_Paulo
-23.30972 -51.36917 America/Sao_Paulo
-18.91806 -54.84417 America/Campo_Grande
-11.48472 -37.93278 America/Bahia
-29.98972 -52.37806 America/Sao_Paulo
-26.10583 -49.7975 America/Sao_Paulo
-26.25444 -49.51833 America/Sao_Paulo
-23.74417 -46.39833 America/Sao_Paulo
-32.035 -52.09861 America/Sao_Paulo
-27.21417 -49.64306 America/Sao_Paulo
-22.90278 -43.2075 America/Sao_Paulo
-22.84333 -47.60611 America/Sao_Paulo
-22.52694 -41.945 America/Sao_Paulo
-22.41139 -47.56139 America/Sao_Paulo
-21.80194 -54.54639 America/Campo_Grande
-25.19 -49.31417 America/Sao_Paulo
-22.70861 -42.60972 America/Sao_Paulo
-21.1775 -47.81028 America/Sao_Paulo
-23.71056 -46.41333 America/Sao_Paulo
-19.76694 -44.08667 America/Sao_Paulo
-27.69934 -48.53219 America/Sao_Paulo
-10.83444 -38.53583 America/Bahia
-11.80694 -39.38556 America/Bahia
-19.32556 -41.25528 America/Sao_Paulo
-22.46889 -44.44667 America/Sao_Paulo
-24.4875 -47.84361 America/Sao_Paulo
-22.22139 -51.30278 America/Sao_Paulo
-22.22917 -50.89306 America/Sao_Paulo
-18.44833 -50.45167 America/Sao_Paulo
-22.71611 -43.55528 America/Sao_Paulo
-25.36556 -49.07694 America/Sao_Paulo
-30.3875 -56.45139 America/Sao_Paulo
-25.21306 -50.97778 America/Sao_Paulo
-10.21111 -36.84028 America/Maceio
-21.53667 -49.85806 America/Sao_Paulo
-21.87611 -51.84389 America/Sao_Paulo
-22.12556 -51.38889 America/Sao_Paulo
-21.76333 -52.11556 America/Sao_Paulo
-19.30722 -48.92417 America/Sao_Paulo
-24.00583 -46.40278 America/Sao_Paulo
-17.34111 -39.22083 America/Bahia
-22.23 -45.93639 America/Sao_Paulo
-14.09306 -46.36944 America/Sao_Paulo
-26.23806 -51.07833 America/Sao_Paulo
-16.44972 -39.06472 America/Bahia
-21.85389 -47.47917 America/Sao_Paulo
-23.21472 -47.52389 America/Sao_Paulo
-30.03306 -51.23 America/Sao_Paulo
-29.70167 -51.24194 America/Sao_Paulo
-13.44083 -49.14861 America/Sao_Paulo
-15.22611 -59.33528 America/Cuiaba
-20.41639 -42.90861 America/Sao_Paulo
-22.53611 -55.72556 America/Campo_Grande
-21.0225 -48.03722 America/Sao_Paulo
-25.095 -50.16194 America/Sao_Paulo
-19.22444 -44.93528 America/Sao_Paulo
-22.10861 -50.17167 America/Sao_Paulo
-26.74056 -49.17694 America/Sao_Paulo
-21.78778 -46.56139 America/Sao_Paulo
-16.25667 -56.62278 America/Cuiaba
-14.52972 -40.36528 America/Bahia
-23.52806 -46.34472 America/Sao_Paulo
-15.45278 -47.61417 America/Sao_Paulo
-20.83778 -40.72194 America/Sao_Paulo
-20.46528 -45.95806 America/Sao_Paulo
-19.68278 -44.89028 America/Sao_Paulo
-21.00944 -48.22167 America/Sao_Paulo
-24.75722 -51.76139 America/Sao_Paulo
-11.73028 -40.55528 America/Bahia
-17.29972 -48.27944 America/Sao_Paulo
-25.44167 -49.06333 America/Sao_Paulo
-22.27528 -51.5 America/Sao_Paulo
-17.345 -44.94194 America/Sao_Paulo
-21.99861 -49.45722 America/Sao_Paulo
-23.19361 -49.38389 America/Sao_Paulo
-24.52611 -49.94861 America/Sao_Paulo
-22.62917 -43.89806 America/Sao_Paulo
-21.99611 -47.42583 America/Sao_Paulo
-22.72528 -47.64917 America/Sao_Paulo
-17.30278 -49.01667 America/Sao_Paulo
-23.05389 -46.35806 America/Sao_Paulo
-22.51278 -44.00056 America/Sao_Paulo
-25.69556 -51.65972 America/Sao_Paulo
-22.11583 -46.68278 America/Sao_Paulo
-10.74167 -40.36083 America/Bahia
-22.92389 -45.46167 America/Sao_Paulo
-23.81306 -47.71639 America/Sao_Paulo
-23.71194 -47.42778 America/Sao_Paulo
-22.505 -43.17861 America/Sao_Paulo
-24.32 -46.99833 America/Sao_Paulo
-20.63833 -51.10917 America/Sao_Paulo
-21.09083 -45.09139 America/Sao_Paulo
-26.76944 -48.64583 America/Sao_Paulo
-10.29028 -36.58639 America/Maceio
-21.41972 -50.0775 America/Sao_Paulo
-31.77194 -52.3425 America/Sao_Paulo
-19.61806 -44.04306 America/Sao_Paulo
-22.74194 -46.90139 America/Sao_Paulo
-16.00528 -41.29722 America/Sao_Paulo
-22.35167 -48.775 America/Sao_Paulo
-22.76111 -47.15417 America/Sao_Paulo
-18.94389 -46.9925 America/Sao_Paulo
-18.57889 -46.51806 America/Sao_Paulo
-26.22861 -52.67056 America/Sao_Paulo
-22.42861 -43.41861 America/Sao_Paulo
-20.71889 -46.60972 America/Sao_Paulo
-28.26278 -52.40667 America/Sao_Paulo
-29.62861 -50.83472 America/Sao_Paulo
-23.21778 -44.71306 America/Sao_Paulo
-23.07306 -52.46528 America/Sao_Paulo
-23.38861 -48.72278 America/Sao_Paulo
-19.67722 -51.19083 America/Campo_Grande
-25.52 -48.50917 America/Sao_Paulo
-22.16194 -43.29278 America/Sao_Paulo
-22.41278 -50.57583 America/Sao_Paulo
-21.54722 -45.7375 America/Sao_Paulo
-19.86028 -44.60833 America/Sao_Paulo
-17.22222 -46.87472 America/Sao_Paulo
-22.60861 -43.71278 America/Sao_Paulo
-28.2925 -53.50167 America/Sao_Paulo
-24.28389 -53.84 America/Sao_Paulo
-22.78889 -50.2175 America/Sao_Paulo
-27.89944 -53.31361 America/Sao_Paulo
-25.42944 -50.00639 America/Sao_Paulo
-26.48417 -51.99056 America/Sao_Paulo
-27.64528 -48.66778 America/Sao_Paulo
-23.4575 -52.04861 America/Sao_Paulo
-15.35 -48.5 America/Sao_Paulo
-20.39484 -43.50517 America/Sao_Paulo
-20.52083 -43.69194 America/Sao_Paulo
-22.97889 -49.87056 America/Sao_Paulo
-21.79667 -50.87861 America/Sao_Paulo
-29.88667 -50.26972 America/Sao_Paulo
-23.5325 -46.79167 America/Sao_Paulo
-28.35889 -49.29139 America/Sao_Paulo
-20.72028 -47.88667 America/Sao_Paulo
-20.69639 -44.82722 America/Sao_Paulo
-20.73722 -48.91472 America/Sao_Paulo
-21.46806 -49.22083 America/Sao_Paulo
-29.67833 -51.13056 America/Sao_Paulo
-17.89194 -39.37194 America/Bahia
-18.71056 -40.40056 America/Sao_Paulo
-28.78389 -51.61 America/Sao_Paulo
-29.37639 -51.11444 America/Sao_Paulo
-14.79722 -57.28806 America/Cuiaba
-22.7775 -47.29583 America/Sao_Paulo
-19.98556 -43.84667 America/Sao_Paulo
-22.75917 -43.45111 America/Sao_Paulo
-20.53389 -49.31417 America/Sao_Paulo
-22.28194 -42.53111 America/Sao_Paulo
-19.75 -43.0375 America/Sao_Paulo
-10.855 -37.12611 America/Maceio
-10.21833 -37.42028 America/Maceio
-22.88333 -43.10361 America/Sao_Paulo
-14.47389 -48.45972 America/Sao_Paulo
-22.8075 -43.41389 America/Sao_Paulo
-16.40639 -49.21861 America/Sao_Paulo
-21.23583 -45.23583 America/Sao_Paulo
-13.035 -39.01444 America/Bahia
-23.065 -54.19056 America/Campo_Grande
-26.89889 -48.65417 America/Sao_Paulo
-17.83917 -40.35389 America/Sao_Paulo
-21.37583 -46.52556 America/Sao_Paulo
-12.91667 -39.25 America/Bahia
-21.13056 -42.36639 America/Sao_Paulo
-18.08639 -39.55083 America/Bahia
-11.55 -41.15611 America/Bahia
-20.73139 -48.05778 America/Sao_Paulo
-17.73111 -49.09944 America/Sao_Paulo
-16.735 -43.86167 America/Sao_Paulo
-21.18972 -46.98028 America/Sao_Paulo
-22.94667 -47.31583 America/Sao_Paulo
-18.72472 -47.49861 America/Sao_Paulo
-20.90722 -48.64139 America/Sao_Paulo
-20.7725 -49.71417 America/Sao_Paulo
-21.26111 -48.49639 America/Sao_Paulo
-24.09306 -46.62083 America/Sao_Paulo
-22.43194 -46.95778 America/Sao_Paulo
-22.37222 -46.94222 America/Sao_Paulo
-23.52278 -46.18833 America/Sao_Paulo
-21.46778 -47.00472 America/Sao_Paulo
-21.13361 -51.10167 America/Sao_Paulo
-21.41222 -42.19667 America/Sao_Paulo
-17.56944 -52.55111 America/Sao_Paulo
-22.45389 -43.46889 America/Sao_Paulo
-20.17944 -48.03194 America/Sao_Paulo
-22.52667 -43.73278 America/Sao_Paulo
-25.29528 -54.09389 America/Sao_Paulo
-17.37389 -40.22056 America/Bahia
-23.66778 -46.46139 America/Sao_Paulo
-19.55778 -44.08139 America/Sao_Paulo
-19.98639 -44.42778 America/Sao_Paulo
-21.60333 -48.36583 America/Sao_Paulo
-12.53028 -38.29917 America/Bahia
-15.56306 -39.3025 America/Bahia
-22.14583 -51.17083 America/Sao_Paulo
-23.42528 -51.93861 America/Sao_Paulo
-22.21389 -49.94583 America/Sao_Paulo
-22.91944 -42.81861 America/Sao_Paulo
-20.37778 -43.41611 America/Sao_Paulo
-23.485 -51.79167 America/Sao_Paulo
-24.55611 -54.05667 America/Sao_Paulo
-28.44917 -52.2 America/Sao_Paulo
-21.04333 -40.82444 America/Sao_Paulo
-12.77778 -38.91944 America/Bahia
-13.44111 -40.43083 America/Bahia
-21.61444 -55.16833 America/Campo_Grande
-20.35778 -41.95806 America/Sao_Paulo
-20.25806 -42.03361 America/Sao_Paulo
-22.95972 -44.04056 America/Sao_Paulo
-23.5475 -51.67083 America/Sao_Paulo
-23.31861 -46.58667 America/Sao_Paulo
-23.54583 -47.18333 America/Sao_Paulo
-26.11139 -49.80528 America/Sao_Paulo
-21.67472 -45.91972 America/Sao_Paulo
-22.50222 -48.71139 America/Sao_Paulo
-22.37083 -41.78694 America/Sao_Paulo
-16.2525 -47.95028 America/Sao_Paulo
-16.33333 -55.93333 America/Cuiaba
-23.08639 -46.95056 America/Sao_Paulo
-22.73083 -45.12472 America/Sao_Paulo
-23.31028 -51.16278 America/Sao_Paulo
-22.92306 -53.13722 America/Sao_Paulo
-13.64306 -41.84056 America/Bahia
-21.67861 -49.7425 America/Sao_Paulo
-19.39111 -40.07222 America/Sao_Paulo
-22.56472 -47.40167 America/Sao_Paulo
-21.53194 -42.64306 America/Sao_Paulo
-22.59861 -48.80028 America/Sao_Paulo
-22.18556 -47.39028 America/Sao_Paulo
-21.24528 -44.99972 America/Sao_Paulo
-25.40778 -52.41611 America/Sao_Paulo
-10.80639 -37.17 America/Maceio
-23.04972 -47.83667 America/Sao_Paulo
-25.76972 -49.71583 America/Sao_Paulo
-20.15139 -41.62278 America/Sao_Paulo
-27.81611 -50.32611 America/Sao_Paulo
-29.46694 -51.96139 America/Sao_Paulo
-28.4825 -48.78083 America/Sao_Paulo
-28.20861 -51.52583 America/Sao_Paulo
-19.62722 -43.88972 America/Sao_Paulo
-20.0225 -45.54361 America/Sao_Paulo
-10.91722 -37.65 America/Maceio
-19.00472 -57.60167 America/Campo_Grande
-23.18639 -46.88417 America/Sao_Paulo
-29.22694 -53.68167 America/Sao_Paulo
-21.76417 -43.35028 America/Sao_Paulo
-19.95194 -44.34278 America/Sao_Paulo
-21.05278 -49.68833 America/Sao_Paulo
-26.30444 -48.84556 America/Sao_Paulo
-17.7425 -46.1725 America/Sao_Paulo
-19.81 -43.17361 America/Sao_Paulo
-27.17806 -51.50472 America/Sao_Paulo
-10.075 -38.48083 America/Bahia
-16.43389 -41.00333 America/Sao_Paulo
-13.8575 -40.08361 America/Bahia
-22.29639 -48.55778 America/Sao_Paulo
-17.88139 -51.71444 America/Sao_Paulo
-23.10139 -46.72833 America/Sao_Paulo
-21.01778 -47.76389 America/Sao_Paulo
-21.48028 -56.13806 America/Campo_Grande
-26.48611 -49.06667 America/Sao_Paulo
-15.75694 -49.33444 America/Sao_Paulo
-22.64306 -43.65333 America/Sao_Paulo
-15.48333 -44.36667 America/Sao_Paulo
-23.5275 -46.9025 America/Sao_Paulo
-23.60306 -51.64333 America/Sao_Paulo
-15.8025 -43.30889 America/Sao_Paulo
-20.26889 -50.54583 America/Sao_Paulo
-22.70556 -46.98583 America/Sao_Paulo
-24.25111 -49.70583 America/Sao_Paulo
-10.26389 -40.19583 America/Bahia
-32.56611 -53.37583 America/Sao_Paulo
-13.53056 -39.97083 America/Bahia
-22.28556 -46.61222 America/Sao_Paulo
-11.18056 -40.51833 America/Bahia
-15.96528 -54.96833 America/Cuiaba
-23.16056 -49.96944 America/Sao_Paulo
-23.30528 -45.96583 America/Sao_Paulo
-21.25472 -48.32222 America/Sao_Paulo
-29.59111 -51.16056 America/Sao_Paulo
-20.33944 -47.78056 America/Sao_Paulo
-19.72806 -50.19556 America/Sao_Paulo
-23.15306 -47.05778 America/Sao_Paulo
-18.41917 -49.21528 America/Sao_Paulo
-18.96889 -49.465 America/Sao_Paulo
-13.73222 -39.14917 America/Bahia
-23.26417 -47.29917 America/Sao_Paulo
-15.11694 -40.07028 America/Bahia
-20.07528 -44.57639 America/Sao_Paulo
-23.10167 -48.61583 America/Sao_Paulo
-23.00583 -46.83889 America/Sao_Paulo
-24.1125 -49.33167 America/Sao_Paulo
-29.12528 -56.55306 America/Sao_Paulo
-23.48611 -46.34833 America/Sao_Paulo
-15.56222 -49.94861 America/Sao_Paulo
-21.59556 -48.81278 America/Sao_Paulo
-22.43611 -46.82167 America/Sao_Paulo
-23.54889 -46.93417 America/Sao_Paulo
-23.98222 -48.87556 America/Sao_Paulo
-23.59167 -48.05306 America/Sao_Paulo
-15.24889 -40.24778 America/Bahia
-21.205 -41.88778 America/Sao_Paulo
-25.22 -49.34778 America/Sao_Paulo
-21.01111 -40.83389 America/Sao_Paulo
-27.09028 -48.61139 America/Sao_Paulo
-23.71694 -46.84917 America/Sao_Paulo
-20.4725 -45.12556 America/Sao_Paulo
-12.88833 -38.67861 America/Bahia
-14.95083 -49.54944 America/Sao_Paulo
-21.66917 -42.07611 America/Sao_Paulo
-24.18306 -46.78889 America/Sao_Paulo
-15.245 -40.62444 America/Bahia
-17.85722 -42.85889 America/Sao_Paulo
-17.03917 -39.53111 America/Bahia
-14.67806 -39.375 America/Bahia
-22.42556 -45.45278 America/Sao_Paulo
-26.90778 -48.66194 America/Sao_Paulo
-23.41778 -49.09056 America/Sao_Paulo
-22.85222 -43.77528 America/Sao_Paulo
-14.78556 -39.28028 America/Bahia
-22.74444 -42.85944 America/Sao_Paulo
-20.25333 -43.80139 America/Sao_Paulo
-19.61917 -43.22694 America/Sao_Paulo
-16.02028 -49.81028 America/Sao_Paulo
-12.5275 -40.30694 America/Bahia
-11.27389 -37.79 America/Maceio
-10.685 -37.42528 America/Maceio
-11.30417 -41.85583 America/Bahia
-25.46722 -50.65111 America/Sao_Paulo
-22.58056 -47.51861 America/Sao_Paulo
-16.44194 -51.11778 America/Sao_Paulo
-12.15833 -39.73722 America/Bahia
-14.13722 -39.73389 America/Bahia
-23.35028 -47.68861 America/Sao_Paulo
-19.46833 -42.53667 America/Sao_Paulo
-17.72194 -48.15972 America/Sao_Paulo
-19.41361 -42.41944 America/Sao_Paulo
-16.35778 -49.49611 America/Sao_Paulo
-23.09028 -47.21806 America/Sao_Paulo
-26.89778 -49.23167 America/Sao_Paulo
-25.23 -50.60444 America/Sao_Paulo
-28.24 -48.67028 America/Sao_Paulo
-14.78889 -39.04944 America/Bahia
-20.43278 -51.3425 America/Sao_Paulo
-23.77806 -45.35806 America/Sao_Paulo
-28.38778 -53.91472 America/Sao_Paulo
-24.70806 -47.55528 America/Sao_Paulo
-29.57444 -50.79028 America/Sao_Paulo
-20.07028 -44.30167 America/Sao_Paulo
-20.03833 -47.74694 America/Sao_Paulo
-22.50917 -48.55778 America/Sao_Paulo
-28.71333 -49.3 America/Sao_Paulo
-12.18528 -43.22056 America/Bahia
-23.65639 -47.2225 America/Sao_Paulo
-21.75778 -48.82889 America/Sao_Paulo
-20.02194 -44.05889 America/Sao_Paulo
-14.06694 -39.64056 America/Bahia
-27.05694 -49.51778 America/Sao_Paulo
-23.26917 -51.04806 America/Sao_Paulo
-14.865 -39.5875 America/Bahia
-19.47833 -46.53889 America/Sao_Paulo
-21.95472 -47.99667 America/Sao_Paulo
-23.84861 -50.18778 America/Sao_Paulo
-12.76722 -40.21167 America/Bahia
-22.85833 -47.22 America/Sao_Paulo
-32.02361 -53.39556 America/Sao_Paulo
-11.72917 -49.06861 America/Araguaina
-21.30528 -46.71278 America/Sao_Paulo
-23.46278 -46.53333 America/Sao_Paulo
-23.99306 -46.25639 America/Sao_Paulo
-21.36 -48.22833 America/Sao_Paulo
-25.88278 -48.57472 America/Sao_Paulo
-22.81639 -45.1925 America/Sao_Paulo
-23.415 -46.035 America/Sao_Paulo
-21.26083 -50.64278 America/Sao_Paulo
-25.39528 -51.45806 America/Sao_Paulo
-20.66667 -40.4975 America/Sao_Paulo
-21.29917 -46.8025 America/Sao_Paulo
-26.47306 -49.00278 America/Sao_Paulo
-20.42833 -47.82417 America/Sao_Paulo
-28.84556 -51.89028 America/Sao_Paulo
-22.53722 -42.98194 America/Sao_Paulo
-18.775 -42.9325 America/Sao_Paulo
-14.22333 -42.78139 America/Bahia
-20.31833 -48.31056 America/Sao_Paulo
-20.77556 -41.67944 America/Sao_Paulo
-29.94444 -50.99194 America/Sao_Paulo
-18.85111 -41.94944 America/Sao_Paulo
-18.0125 -49.35472 America/Sao_Paulo
-15.93444 -50.14028 America/Sao_Paulo
-16.49611 -49.42639 America/Sao_Paulo
-16.67861 -49.25389 America/Sao_Paulo
-15.3175 -49.1175 America/Sao_Paulo
-26.93139 -48.95889 America/Sao_Paulo
-29.25611 -51.53361 America/Sao_Paulo
-22.21056 -49.65611 America/Sao_Paulo
-13.74389 -39.48667 America/Bahia
-20.02472 -48.94056 America/Sao_Paulo
-27.35917 -53.39444 America/Sao_Paulo
-23.32167 -46.72694 America/Sao_Paulo
-23.28167 -46.74528 America/Sao_Paulo
-26.08111 -53.055 America/Sao_Paulo
-20.53861 -47.40083 America/Sao_Paulo
-25.54778 -54.58806 America/Sao_Paulo
-28.7475 -49.47222 America/Sao_Paulo
-15.53722 -47.33444 America/Sao_Paulo
-20.46444 -45.42639 America/Sao_Paulo
-27.59667 -48.54917 America/Sao_Paulo
-29.02889 -51.18167 America/Sao_Paulo
-23.54083 -46.36861 America/Sao_Paulo
-20.28389 -50.24639 America/Sao_Paulo
-12.26667 -38.96667 America/Bahia
-29.225 -51.34778 America/Sao_Paulo
-10.5075 -39.01583 America/Bahia
-28.06111 -50.93833 America/Sao_Paulo
-29.86139 -51.17917 America/Sao_Paulo
-29.64833 -51.17389 America/Sao_Paulo
-11.26833 -37.43833 America/Maceio
-11.79611 -37.945 America/Bahia
-14.92611 -42.81917 America/Sao_Paulo
-19.7625 -44.31389 America/Sao_Paulo
-27.63417 -52.27389 America/Sao_Paulo
-11.94194 -38.08444 America/Bahia
-30.54389 -52.52194 America/Sao_Paulo
-29.23611 -51.86972 America/Sao_Paulo
-23.83222 -46.81139 America/Sao_Paulo
-23.64889 -46.85222 America/Sao_Paulo
-21.61 -45.56528 America/Sao_Paulo
-22.78556 -43.31167 America/Sao_Paulo
-22.22111 -54.80556 America/Campo_Grande
-30.98278 -54.67306 America/Sao_Paulo
-25.73361 -53.05722 America/Sao_Paulo
-22.36611 -48.38028 America/Sao_Paulo
-20.13889 -44.88389 America/Sao_Paulo
-14.40861 -56.44611 America/Cuiaba
-18.24944 -43.60028 America/Sao_Paulo
-23.68611 -46.62278 America/Sao_Paulo
-21.90389 -47.61944 America/Sao_Paulo
-18.75639 -44.43083 America/Sao_Paulo
-27.28278 -50.58444 America/Sao_Paulo
-25.42778 -49.27306 America/Sao_Paulo
-15.59611 -56.09667 America/Cuiaba
-23.895 -46.42528 America/Sao_Paulo
-23.785 -53.07333 America/Sao_Paulo
-22.57611 -44.96278 America/Sao_Paulo
-12.67 -39.10194 America/Bahia
-28.63861 -53.60639 America/Sao_Paulo
-16.76861 -47.61361 America/Sao_Paulo
-28.6775 -49.36972 America/Sao_Paulo
-21.34028 -47.72944 America/Sao_Paulo
-18.50667 -54.76 America/Campo_Grande
-23.60389 -46.91917 America/Sao_Paulo
-22.64583 -47.19611 America/Sao_Paulo
-10.12556 -36.17556 America/Maceio
-19.00917 -57.65333 America/Campo_Grande
-25.97972 -52.56778 America/Sao_Paulo
-19.51861 -42.62889 America/Sao_Paulo
-18.47333 -47.20028 America/Sao_Paulo
-18.38083 -44.45639 America/Sao_Paulo
-22.48194 -47.45667 America/Sao_Paulo
-22.02861 -42.36083 America/Sao_Paulo
-19.93167 -44.05361 America/Sao_Paulo
-20.66028 -43.78611 America/Sao_Paulo
-20.49972 -43.85778 America/Sao_Paulo
-11.81361 -37.61056 America/Bahia
-27.23417 -52.02778 America/Sao_Paulo
-22.33028 -47.1725 America/Sao_Paulo
-11.56389 -39.28278 America/Bahia
-12.31667 -38.76667 America/Bahia
-19.91472 -48.38833 America/Sao_Paulo
-12.50583 -38.99861 America/Bahia
-18.59333 -39.73222 America/Bahia
-22.8375 -51.97306 America/Sao_Paulo
-25.29167 -49.22417 America/Sao_Paulo
-19.53944 -40.63056 America/Sao_Paulo
-14.64083 -39.55111 America/Bahia
-20.44333 -44.76583 America/Sao_Paulo
-10.6 -38.38333 America/Bahia
-23.66333 -52.605 America/Sao_Paulo
-29.95472 -51.62528 America/Sao_Paulo
-27.09639 -52.61833 America/Sao_Paulo
-23.165 -47.74361 America/Sao_Paulo
-15.30833 -49.59833 America/Sao_Paulo
-27.63444 -51.33639 America/Sao_Paulo
-29.16806 -51.17944 America/Sao_Paulo
-21.97722 -44.9325 America/Sao_Paulo
-12.35306 -38.37889 America/Bahia
-21.13778 -48.97278 America/Sao_Paulo
-18.16583 -47.94639 America/Sao_Paulo
-21.38917 -42.69667 America/Sao_Paulo
-24.79111 -50.01194 America/Sao_Paulo
-20.60361 -41.18472 America/Sao_Paulo
-19.11333 -51.73417 America/Campo_Grande
-22.48056 -42.20417 America/Sao_Paulo
-24.95583 -53.45528 America/Sao_Paulo
-21.77389 -47.08639 America/Sao_Paulo
-19.00083 -46.31611 America/Sao_Paulo
-20.18417 -44.77111 America/Sao_Paulo
-29.2975 -51.50361 America/Sao_Paulo
-28.28389 -52.78639 America/Sao_Paulo
-19.78972 -42.13917 America/Sao_Paulo
-23.5225 -46.83556 America/Sao_Paulo
-20.73306 -42.02944 America/Sao_Paulo
-20.95361 -43.80639 America/Sao_Paulo
-23.62028 -45.41306 America/Sao_Paulo
-22.995 -47.50778 America/Sao_Paulo
-27.34361 -51.61194 America/Sao_Paulo
-11.38111 -40.01278 America/Bahia
-17.69139 -42.51583 America/Sao_Paulo
-10.50333 -37.05278 America/Maceio
-29.74556 -50.00972 America/Sao_Paulo
-24.00583 -48.34944 America/Sao_Paulo
-26.17722 -50.39 America/Sao_Paulo
-29.91778 -51.18361 America/Sao_Paulo
-31.395 -52.67556 America/Sao_Paulo
-29.36556 -50.81556 America/Sao_Paulo
-22.74639 -50.38694 America/Sao_Paulo
-29.66917 -52.78889 America/Sao_Paulo
-15.675 -38.94722 America/Bahia
-20.41667 -54.06667 America/Campo_Grande
-27.40167 -51.225 America/Sao_Paulo
-21.235 -45.75861 America/Sao_Paulo
-22.73944 -45.59139 America/Sao_Paulo
-13.03667 -46.77167 America/Sao_Paulo
-21.75 -41.3 America/Sao_Paulo
-24.04556 -52.38306 America/Sao_Paulo
-25.45861 -49.52833 America/Sao_Paulo
-20.44278 -54.64639 America/Campo_Grande
-10.5075 -40.32139 America/Bahia
-20.89722 -45.27722 America/Sao_Paulo
-22.90556 -47.06083 America/Sao_Paulo
-25.30556 -49.05528 America/Sao_Paulo
-22.61222 -46.0575 America/Sao_Paulo
-23.27583 -51.27833 America/Sao_Paulo
-23.04639 -50.07361 America/Sao_Paulo
-30.85111 -51.81222 America/Sao_Paulo
-22.75528 -46.14472 America/Sao_Paulo
-12.6975 -38.32417 America/Bahia
-17.74167 -48.625 America/Sao_Paulo
-21.27528 -47.30417 America/Sao_Paulo
-24.73611 -48.12278 America/Sao_Paulo
-23.35611 -46.87694 America/Sao_Paulo
-23.36417 -46.74056 America/Sao_Paulo
-14.06944 -42.475 America/Bahia
-19.88 -43.66972 America/Sao_Paulo
-20.84889 -41.11278 America/Sao_Paulo
-29.95111 -51.09389 America/Sao_Paulo
-22.4625 -42.65306 America/Sao_Paulo
-30.03917 -52.89389 America/Sao_Paulo
-12.61833 -38.95583 America/Bahia
-30.51222 -53.49139 America/Sao_Paulo
-23.10083 -45.70694 America/Sao_Paulo
-26.77528 -51.015 America/Sao_Paulo
-23.3075 -47.13278 America/Sao_Paulo
-22.87944 -42.01861 America/Sao_Paulo
-30.11972 -51.96222 America/Sao_Paulo
-17.35111 -44.96222 America/Sao_Paulo
-15.61778 -46.42333 America/Sao_Paulo
-23.7975 -48.59278 America/Sao_Paulo
-14.95944 -39.29972 America/Bahia
-27.09806 -48.9175 America/Sao_Paulo
-14.20361 -41.66528 America/Bahia
-20.14333 -44.19972 America/Sao_Paulo
-22.28417 -48.12667 America/Sao_Paulo
-20.99139 -47.65861 America/Sao_Paulo
-15.77972 -47.92972 America/Sao_Paulo
-22.95194 -46.54194 America/Sao_Paulo
-28.275 -49.16556 America/Sao_Paulo
-22.88583 -48.445 America/Sao_Paulo
-21.13389 -41.67972 America/Sao_Paulo
-13.255 -43.41806 America/Bahia
-19.73639 -45.25222 America/Sao_Paulo
-23.28333 -47.67222 America/Sao_Paulo
-17.10778 -43.815 America/Sao_Paulo
-21.09 -45.56583 America/Sao_Paulo
-26.91944 -49.06611 America/Sao_Paulo
-23.5725 -46.03861 America/Sao_Paulo
-21.28861 -50.34 America/Sao_Paulo
-27.49417 -48.65556 America/Sao_Paulo
-19.96778 -44.19833 America/Sao_Paulo
-23.85444 -46.13861 America/Sao_Paulo
-29.17139 -51.51917 America/Sao_Paulo
-19.22 -42.48361 America/Sao_Paulo
-19.92083 -43.93778 America/Sao_Paulo
-22.76417 -43.39944 America/Sao_Paulo
-22.10889 -56.52111 America/Campo_Grande
-20.94944 -48.47917 America/Sao_Paulo
-22.31472 -49.06056 America/Sao_Paulo
-20.89111 -47.585 America/Sao_Paulo
-21.92194 -50.73389 America/Sao_Paulo
-23.51056 -46.87611 America/Sao_Paulo
-21.18694 -43.97583 America/Sao_Paulo
-21.19361 -48.16389 America/Sao_Paulo
-20.55722 -48.56778 America/Sao_Paulo
-15.61667 -43.6 America/Sao_Paulo
-12.15278 -44.99 America/Bahia
-26.63222 -48.68472 America/Sao_Paulo
-22.54417 -44.17139 America/Sao_Paulo
-10.90889 -37.03861 America/Noronha
-22.47 -43.82556 America/Sao_Paulo
-15.89 -52.25667 America/Cuiaba
-15.0725 -57.18111 America/Cuiaba
-18.755 -40.89083 America/Sao_Paulo
-22.49472 -48.55806 America/Sao_Paulo
-11.08944 -43.14167 America/Bahia
-22.07444 -48.74028 America/Sao_Paulo
-21.22583 -43.77361 America/Sao_Paulo
-19.94583 -43.48722 America/Sao_Paulo
-23.11 -50.3675 America/Sao_Paulo
-20.00639 -45.97694 America/Sao_Paulo
-26.99056 -48.63472 America/Sao_Paulo
-19.51889 -41.01583 America/Sao_Paulo
-31.33139 -54.10694 America/Sao_Paulo
-23.09861 -48.92583 America/Sao_Paulo
-23.11694 -46.55028 America/Sao_Paulo
-23.2325 -51.66556 America/Sao_Paulo
-22.66167 -50.41222 America/Sao_Paulo
-23.39611 -46.32083 America/Sao_Paulo
-22.57306 -47.1725 America/Sao_Paulo
-32.2375 -53.08694 America/Sao_Paulo
-29.40111 -51.945 America/Sao_Paulo
-22.96611 -42.02778 America/Sao_Paulo
-22.74694 -41.88167 America/Sao_Paulo
-20.28194 -45.53944 America/Sao_Paulo
-19.59333 -46.94056 America/Sao_Paulo
-25.59306 -49.41028 America/Sao_Paulo
-22.87278 -42.34306 America/Sao_Paulo
-22.35694 -47.38417 America/Sao_Paulo
-21.79444 -48.17556 America/Sao_Paulo
-28.93472 -49.48583 America/Sao_Paulo
-23.41944 -51.42444 America/Sao_Paulo
-18.64722 -48.18722 America/Sao_Paulo
-15.8975 -52.25083 America/Sao_Paulo
-16.84972 -42.07028 America/Sao_Paulo
-19.82028 -40.27333 America/Sao_Paulo
-23.50528 -47.61417 America/Sao_Paulo
-11.33333 -38.96667 America/Bahia
-21.20889 -50.43278 America/Sao_Paulo
-10.91111 -37.07167 America/Maceio
-20.47111 -55.78722 America/Campo_Grande
-23.55083 -51.46083 America/Sao_Paulo
-24.50944 -48.8425 America/Sao_Paulo
-20.08667 -51.09361 America/Campo_Grande
-22.84694 -45.22972 America/Sao_Paulo
-25.42861 -48.71194 America/Sao_Paulo
-16.46111 -49.96167 America/Sao_Paulo
-23.00667 -44.31806 America/Sao_Paulo
-20.89611 -51.37944 America/Sao_Paulo
-22.06806 -46.56917 America/Sao_Paulo
-20.48361 -55.80694 America/Campo_Grande
-16.32667 -48.95278 America/Sao_Paulo
-22.70111 -46.76444 America/Sao_Paulo
-21.72444 -48.10167 America/Sao_Paulo
-22.73917 -47.33139 America/Sao_Paulo
-13.03028 -39.60472 America/Bahia
-22.07944 -51.47194 America/Sao_Paulo
-25.32472 -49.31 America/Sao_Paulo
-16.18361 -40.69444 America/Sao_Paulo
-21.42917 -45.94722 America/Sao_Paulo
-21.88778 -42.70444 America/Sao_Paulo
-29.78306 -55.79194 America/Sao_Paulo
-20.76361 -41.53306 America/Sao_Paulo
-12.13556 -38.41917 America/Bahia
-19.49583 -41.06389 America/Sao_Paulo
-22.46917 -48.9875 America/Sao_Paulo
-15.74722 -41.46 America/Sao_Paulo
-22.47639 -46.63278 America/Sao_Paulo
-22.05944 -46.97861 America/Sao_Paulo
-21.68528 -51.0725 America/Sao_Paulo
-19.16 -45.44583 America/Sao_Paulo
-30.11389 -51.325 America/Sao_Paulo
-10.21278 -48.36028 America/Araguaina
-3.35417 -64.71139 America/Manaus
-8.16139 -70.76556 America/Rio_Branco
-4.2525 -69.93806 America/Manaus
-9.06556 -68.65694 America/Manaus
-0.13028 -67.08917 America/Manaus
-9.97472 -67.81 America/Rio_Branco
-8.76194 -63.90389 America/Porto_Velho
-3.10194 -60.025 America/Manaus
-3.29972 -60.62056 America/Manaus
-7.50611 -63.02083 America/Manaus
-2.51389 -66.09167 America/Manaus
-6.66028 -69.87361 America/Manaus
-4.085 -63.14139 America/Manaus
-4.88278 -66.89583 America/Manaus
2.81972 -60.67333 America/Boa_Vista
-9.91333 -63.04083 America/Porto_Velho
-9.16667 -60.63333 America/Cuiaba
-12.74056 -60.14583 America/Porto_Velho
-13.08333 -62.26667 America/Porto_Velho
-11.6725 -61.19361 America/Porto_Velho
-10.74806 -62.21583 America/Porto_Velho
-10.43889 -62.46639 America/Porto_Velho
-10.78278 -65.33944 America/Porto_Velho
-11.43861 -61.44722 America/Porto_Velho
-16.82333 -49.24389 America/Sao_Paulo
-27.59444 -48.60694 America/Sao_Paulo
-8.11278 -35.01472 America/Recife
-12.89444 -38.32722 America/Bahia
-25.44472 -49.1925 America/Sao_Paulo
-2.69944 -59.69972 America/Manaus
-12.78444 -38.40389 America/Bahia
-3.62092 -38.82913 America/Fortaleza
-20.30285 -40.33433 America/Sao_Paulo
-27.58612 -48.52335 America/Sao_Paulo
-27.71773 -48.56266 America/Sao_Paulo
25.05823 -77.34306 America/Nassau
26.53333 -78.66667 America/Nassau
26.53333 -78.7 America/Nassau
27.46609 89.64191 Asia/Thimphu
26.85164 89.38837 Asia/Thimphu
-21.48333 27.48333 Africa/Gaborone
-24.67014 25.53975 Africa/Gaborone
-22.38333 26.71667 Africa/Gaborone
-21.97903 27.84983 Africa/Gaborone
-24.87158 25.86989 Africa/Gaborone
-22.54605 27.12507 Africa/Gaborone
-24.7718 25.42156 Africa/Gaborone
-24.40659 25.49508 Africa/Gaborone
-24.62694 25.86556 Africa/Gaborone
-24.41667 26.15 Africa/Gaborone
-19.98333 23.41667 Africa/Gaborone
-23.10407 26.81421 Africa/Gaborone
-25.21667 25.66667 Africa/Gaborone
-21.41667 25.58333 Africa/Gaborone
-24.96675 25.33274 Africa/Gaborone
-25.41667 25.55 Africa/Gaborone
-24.65451 25.90859 Africa/Gaborone
-21.16995 27.50788 Africa/Gaborone
54.09658 28.3341 Europe/Minsk
52.8926 30.024 Europe/Minsk
55.19048 30.2033 Europe/Minsk
54.4896 26.90624 Europe/Minsk
53.1561 24.4513 Europe/Minsk
52.6329 29.7389 Europe/Minsk
53.4785 26.7434 Europe/Minsk
54.47906 26.3934 Europe/Minsk
53.0274 27.5597 Europe/Minsk
53.0869 25.3163 Europe/Minsk
53.6014 24.7465 Europe/Minsk
52.7876 27.5415 Europe/Minsk
53.0934 30.0495 Europe/Minsk
52.3617 30.3916 Europe/Minsk
52.556 24.4573 Europe/Minsk
55.48592 28.77556 Europe/Minsk
52.1229 26.0951 Europe/Minsk
55.11676 26.83263 Europe/Minsk
53.3011 28.6386 Europe/Minsk
54.5116 30.42658 Europe/Minsk
55.53628 28.64244 Europe/Minsk
53.5942 25.8191 Europe/Minsk
53.9 27.56667 Europe/Minsk
52.0495 29.2456 Europe/Minsk
53.4122 24.5387 Europe/Minsk
53.509 28.147 Europe/Minsk
54.32052 26.86436 Europe/Minsk
53.9141 30.33764 Europe/Minsk
54.88143 28.69323 Europe/Minsk
52.2472 26.8047 Europe/Minsk
53.88333 25.29972 Europe/Minsk
53.69448 31.70994 Europe/Minsk
52.2138 24.3564 Europe/Minsk
52.1323 29.3257 Europe/Minsk
52.709 25.3401 Europe/Minsk
53.6884 23.8258 Europe/Minsk
54.2861 30.98424 Europe/Minsk
52.4345 30.9754 Europe/Minsk
55.13823 27.68793 Europe/Minsk
53.6832 27.138 Europe/Minsk
52.4089 31.3237 Europe/Minsk
53.51935 30.24693 Europe/Minsk
52.5314 24.9786 Europe/Minsk
52.09755 23.68775 Europe/Minsk
54.23123 28.50485 Europe/Minsk
53.1327 26.0139 Europe/Minsk
53.1384 29.2214 Europe/Minsk
17.15611 -89.07139 America/Belize
18.08124 -88.56328 America/Belize
17.25 -88.76667 America/Belize
17.49952 -88.19756 America/Belize
49.05798 -122.25257 America/Vancouver
51.30011 -114.03528 America/Edmonton
43.85012 -79.03288 America/Toronto
48.55009 -71.6491 America/Montreal
48.56688 -78.11624 America/Montreal
49.21679 -68.14894 America/Montreal
44.40011 -79.66634 America/Toronto
45.43341 -73.86586 America/Montreal
44.17876 -77.37053 America/Toronto
45.56678 -73.19915 America/Montreal
45.66678 -73.88249 America/Montreal
45.61678 -73.83249 America/Montreal
45.59104 -73.43605 America/Montreal
44.11681 -79.61633 America/Toronto
43.68341 -79.76633 America/Toronto
49.84692 -99.95306 America/Winnipeg
43.1334 -80.34967 America/Toronto
43.1334 -80.26636 America/Toronto
44.58341 -75.68264 America/New_York
45.45008 -73.46583 America/Montreal
43.38621 -79.83713 America/Toronto
49.26636 -122.95263 America/Vancouver
51.05011 -114.08529 America/Edmonton
43.3601 -80.31269 America/Toronto
50.01634 -125.24459 America/Vancouver
53.01684 -112.83525 America/Edmonton
45.38338 -73.51587 America/Montreal
45.45008 -73.28246 America/Montreal
46.23525 -63.12671 America/Halifax
45.38338 -73.74919 America/Montreal
42.40009 -82.1831 America/Toronto
49.17468 -121.94427 America/Vancouver
45.5501 -75.29101 America/Toronto
43.95977 -78.16515 America/Toronto
51.18341 -114.46871 America/Edmonton
44.4834 -80.21638 America/Toronto
47.49989 -52.99806 America/St_Johns
49.28297 -122.75262 America/Vancouver
48.96671 -57.9484 America/St_Johns
45.01809 -74.72815 America/Toronto
45.46536 -73.66585 America/Montreal
49.68657 -124.9936 America/Vancouver
49.49991 -115.76879 America/Edmonton
44.67134 -63.57719 America/Halifax
49.14399 -122.9068 America/Vancouver
45.53455 -73.90168 America/Montreal
46.07844 -64.68735 America/Moncton
45.49452 -73.82419 America/Montreal
45.4473 -73.75335 America/Montreal
45.88336 -72.48241 America/Montreal
48.78293 -123.70266 America/Vancouver
53.55014 -113.46871 America/Edmonton
43.64639 -79.56985 America/Toronto
42.90012 -78.93286 America/Toronto
56.72676 -111.38103 America/Edmonton
56.24988 -120.85292 America/Dawson_Creek
45.94541 -66.66558 America/Moncton
45.47723 -75.70164 America/Montreal
46.19695 -59.95698 America/Glace_Bay
45.40008 -72.73243 America/Montreal
55.16667 -118.80271 America/Edmonton
46.49 -80.99001 America/Toronto
44.25012 -76.94944 America/Toronto
43.5501 -80.24967 America/Toronto
42.98341 -79.86633 America/Toronto
43.23341 -79.94964 America/Toronto
45.33341 -79.21632 America/Toronto
46.01677 -73.44915 America/Montreal
50.66648 -120.3192 America/Vancouver
44.58342 -78.83288 America/Toronto
49.88307 -119.48568 America/Vancouver
49.81671 -94.43373 America/Winnipeg
44.25011 -79.46632 America/Toronto
44.22976 -76.48098 America/Toronto
45.45008 -73.86586 America/Montreal
43.42537 -80.5112 America/Toronto
48.44963 -123.50261 America/Vancouver
49.09967 -122.6526 America/Vancouver
49.08297 -122.58589 America/Vancouver
45.41678 -73.49917 America/Montreal
45.82318 -73.4294 America/Montreal
45.56995 -73.692 America/Montreal
42.0549 -82.6062 America/Toronto
53.26682 -113.55201 America/Edmonton
49.69999 -112.81856 America/Edmonton
53.28346 -110.00157 America/Edmonton
42.98339 -81.23304 America/Toronto
45.53121 -73.51806 America/Montreal
45.26678 -72.14909 America/Montreal
49.21939 -122.60193 America/Vancouver
43.86682 -79.2663 America/Toronto
45.74965 -73.59956 America/Montreal
50.05006 -110.66834 America/Edmonton
44.7501 -79.88296 America/Toronto
45.65008 -74.08251 America/Montreal
47.00431 -65.46544 America/Moncton
49.13377 -122.3144 America/Vancouver
43.5789 -79.6583 America/Toronto
46.11594 -64.80186 America/Moncton
45.50884 -73.58781 America/Montreal
45.51675 -73.64918 America/Montreal
45.56678 -73.19915 America/Montreal
50.40005 -105.53445 America/Regina
47.51659 -52.78135 America/St_Johns
49.16634 -123.93601 America/Vancouver
45.58344 -62.64863 America/Halifax
44.05011 -79.46631 America/Toronto
49.20678 -122.91092 America/Vancouver
43.10012 -79.06627 America/Toronto
42.8334 -80.38297 America/Toronto
52.78344 -108.28465 America/Regina
46.3168 -79.46633 America/Toronto
48.84133 -123.68596 America/Vancouver
49.31636 -123.06934 America/Vancouver
43.76681 -79.4163 America/Toronto
43.45011 -79.68292 America/Toronto
43.9168 -80.09967 America/Toronto
44.60868 -79.42068 America/Toronto
43.90012 -78.84957 America/Toronto
45.41117 -75.69812 America/Toronto
44.56717 -80.94349 America/Toronto
49.31633 -124.31945 America/Vancouver
45.81681 -77.11616 America/Toronto
49.48062 -119.58584 America/Vancouver
44.30012 -78.31623 America/Toronto
43.90012 -79.13289 America/Toronto
49.22119 -122.68965 America/Vancouver
45.44868 -73.81669 America/Montreal
49.24133 -124.8028 America/Vancouver
42.90012 -79.23288 America/Toronto
49.28297 -122.85263 America/Vancouver
53.20008 -105.76772 America/Regina
44.00012 -77.24946 America/Toronto
53.9166 -122.75301 America/Vancouver
44.18342 -77.56618 America/Toronto
52.26682 -113.802 America/Edmonton
50.45008 -104.6178 America/Regina
45.74222 -73.45008 America/Montreal
49.17003 -123.13683 America/Vancouver
48.23985 -79.02878 America/Montreal
48.41675 -71.06573 America/Montreal
45.53338 -73.28246 America/Montreal
45.53341 -73.34916 America/Montreal
45.36678 -73.56588 America/Montreal
46.31836 -72.56632 America/Montreal
45.58338 -73.33246 America/Montreal
45.63922 -73.82757 America/Montreal
45.565 -73.90554 America/Montreal
45.61678 -72.94914 America/Montreal
45.31678 -73.26586 America/Montreal
45.78036 -74.00365 America/Montreal
45.27271 -66.06766 America/Moncton
48.95272 -79.4593 America/Montreal
45.50008 -73.66585 America/Montreal
45.40008 -74.13256 America/Montreal
45.25008 -74.13253 America/Montreal
50.6998 -119.30237 America/Vancouver
42.97866 -82.40407 America/Toronto
52.11679 -106.63452 America/Regina
46.51677 -84.33325 America/Toronto
50.20011 -66.38208 America/Montreal
46.56675 -72.74913 America/Montreal
45.40008 -71.89908 America/Montreal
53.51684 -113.3187 America/Edmonton
46.03336 -73.11585 America/Montreal
53.53344 -113.91874 America/Edmonton
53.63344 -113.63533 America/Edmonton
43.16681 -79.24958 America/Toronto
43.36679 -80.94972 America/Toronto
42.77361 -81.18038 America/Toronto
49.10635 -122.82509 America/Vancouver
54.51634 -128.60345 America/Vancouver
45.70004 -73.64732 America/Montreal
43.11682 -79.19958 America/Toronto
48.4001 -89.31683 America/Thunder_Bay
48.46686 -81.33312 America/Toronto
43.70011 -79.4163 America/Toronto
46.35006 -72.54912 America/Montreal
45.36685 -63.26538 America/Halifax
48.10018 -77.7828 America/Montreal
49.24966 -123.11934 America/Vancouver
45.68338 -73.43246 America/Montreal
45.40008 -74.03251 America/Montreal
43.8361 -79.49827 America/Toronto
50.25809 -119.26905 America/Vancouver
48.43294 -123.3693 America/Vancouver
46.05007 -71.96579 America/Montreal
43.4668 -80.51639 America/Toronto
42.98342 -79.24958 America/Toronto
45.48341 -73.59918 America/Montreal
60.71611 -135.05375 America/Whitehorse
49.01636 -122.8026 America/Vancouver
42.30008 -83.01654 America/Toronto
49.8844 -97.14704 America/Winnipeg
43.13339 -80.7497 America/Toronto
62.456 -114.35255 America/Yellowknife
51.2167 -102.46766 America/Regina
44.64533 -63.57239 America/Halifax
47.56494 -52.70931 America/St_Johns
46.81228 -71.21454 America/Montreal
46.80326 -71.17793 America/Montreal
48.44879 -68.52396 America/Montreal
47.83044 -69.53419 America/Montreal
46.1351 -60.1831 America/Glace_Bay
46.79392 -71.35191 America/Montreal
47.3737 -68.32512 America/Moncton
46.09371 -71.30539 America/Montreal
44.67244 -63.47506 America/Halifax
50.36386 -119.34997 America/Vancouver
49.8625 -119.58333 America/Vancouver
46.66667 -70.71667 America/Montreal
48.41648 -71.24884 America/Montreal
46.74064 -71.45131 America/Montreal
49.08938 -123.08241 America/Vancouver
49.16473 -122.64042 America/Vancouver
-12.15681 96.82251 Indian/Cocos
0.81021 24.43359 Africa/Lubumbashi
3.03716 29.53551 Africa/Lubumbashi
2.14838 27.99466 Africa/Lubumbashi
-3.40667 29.14583 Africa/Lubumbashi
-6.41667 20.8 Africa/Lubumbashi
-1.57361 29.045 Africa/Lubumbashi
-7 23.45 Africa/Lubumbashi
-4.85 21.56667 Africa/Lubumbashi
-6.15 23.6 Africa/Lubumbashi
-4.96667 23.45 Africa/Lubumbashi
-5.35 21.41667 Africa/Lubumbashi
-5.36667 25.75 Africa/Lubumbashi
-3.48333 23.43333 Africa/Lubumbashi
2.15 21.51667 Africa/Kinshasa
-5.38333 27 Africa/Lubumbashi
0.51667 25.2 Africa/Lubumbashi
-2.95 25.95 Africa/Lubumbashi
-4.45 26.66667 Africa/Lubumbashi
-7.56667 24.18333 Africa/Lubumbashi
-5.89583 22.41778 Africa/Lubumbashi
-3.6 26.66667 Africa/Lubumbashi
-8.73861 24.99056 Africa/Lubumbashi
-5.94749 29.19471 Africa/Lubumbashi
-6.13333 24.48333 Africa/Lubumbashi
-2.46833 28.82417 Africa/Lubumbashi
-6.05 26.91667 Africa/Lubumbashi
2.77391 27.61603 Africa/Lubumbashi
-4.31667 20.58333 Africa/Lubumbashi
-1.67917 29.22278 Africa/Lubumbashi
4.28333 21.01667 Africa/Kinshasa
-6.75 23.95 Africa/Lubumbashi
-5.51 22.26667 Africa/Lubumbashi
0.15 29.28333 Africa/Lubumbashi
2.78582 24.72998 Africa/Lubumbashi
3.33333 20.88333 Africa/Kinshasa
1.56667 30.25 Africa/Lubumbashi
2.18333 22.46667 Africa/Kinshasa
-2.50833 28.86083 Africa/Lubumbashi
-9.2 25.85 Africa/Lubumbashi
3.81461 23.68665 Africa/Lubumbashi
-0.21667 20.86667 Africa/Kinshasa
0.5 29.46667 Africa/Lubumbashi
1.23909 23.61598 Africa/Lubumbashi
2.73877 23.78326 Africa/Lubumbashi
-11.66089 27.47938 Africa/Lubumbashi
-10.98139 26.73333 Africa/Lubumbashi
-10.71484 25.46674 Africa/Lubumbashi
-11.76667 27.23333 Africa/Lubumbashi
-10.87639 26.59694 Africa/Lubumbashi
-4.98333 12.93333 Africa/Kinshasa
-2.71667 17.68333 Africa/Kinshasa
-3.01667 16.9 Africa/Kinshasa
-5.25 14.86667 Africa/Kinshasa
0.04865 18.26034 Africa/Kinshasa
-5.81667 13.45 Africa/Kinshasa
-4.05 19.53333 Africa/Kinshasa
3.65 18.63333 Africa/Kinshasa
-4.32459 15.32146 Africa/Kinshasa
-5.04098 18.81619 Africa/Kinshasa
-6.46667 16.81667 Africa/Kinshasa
-4.59111 15.17083 Africa/Kinshasa
-1.95 18.26667 Africa/Kinshasa
3.25 19.76667 Africa/Kinshasa
-4.55 18.6 Africa/Kinshasa
-2.16667 16.23333 Africa/Kinshasa
-3.31667 17.36667 Africa/Kinshasa
-4.38361 15.39139 Africa/Kinshasa
4.31902 21.17861 Africa/Bangui
6.26793 21.22468 Africa/Bangui
6.54233 21.98633 Africa/Bangui
4.74132 22.81838 Africa/Bangui
5.76795 20.67565 Africa/Bangui
5.71801 19.07389 Africa/Bangui
7.24269 16.44059 Africa/Bangui
3.52716 16.04 Africa/Bangui
3.86781 17.98923 Africa/Bangui
6.98961 19.18744 Africa/Bangui
4.96075 18.7035 Africa/Bangui
4.9409 15.86645 Africa/Bangui
6.31933 16.37992 Africa/Bangui
5.93404 15.59599 Africa/Bangui
6.49263 17.45518 Africa/Bangui
4.31887 17.46953 Africa/Bangui
4.25671 18.41583 Africa/Bangui
4.26116 15.79216 Africa/Bangui
7.30082 18.2833 Africa/Bangui
4.36122 18.55496 Africa/Bangui
-3.68192 13.34985 Africa/Brazzaville
-4.77609 11.86352 Africa/Brazzaville
-0.48193 15.89988 Africa/Brazzaville
1.61361 16.05167 Africa/Brazzaville
-2.95056 12.72611 Africa/Brazzaville
-4.15361 13.55 Africa/Brazzaville
-4.19972 12.67389 Africa/Brazzaville
-4.75611 11.85778 Africa/Brazzaville
-4.16556 13.29278 Africa/Brazzaville
1.63806 18.06667 Africa/Brazzaville
-1.87639 15.86444 Africa/Brazzaville
-4.26583 15.28318 Africa/Brazzaville
47.36667 8.55 Europe/Zurich
47.17242 8.51744 Europe/Zurich
46.77852 6.64115 Europe/Zurich
47.5 8.75 Europe/Zurich
47.46152 9.04552 Europe/Zurich
47.4705 8.31636 Europe/Zurich
46.46116 6.84328 Europe/Zurich
46.21702 6.08497 Europe/Zurich
47.34713 8.72091 Europe/Zurich
46.75118 7.62166 Europe/Zurich
46.77807 7.63249 Europe/Zurich
46.22908 7.35942 Europe/Zurich
47.42181 8.54779 Europe/Zurich
47.69732 8.63493 Europe/Zurich
47.42391 9.37477 Europe/Zurich
46.53989 6.5881 Europe/Zurich
47.22557 8.82228 Europe/Zurich
46.51027 6.66183 Europe/Zurich
46.18333 6.1 Europe/Zurich
47.34999 7.90329 Europe/Zurich
47.40823 8.54258 Europe/Zurich
46.38318 6.23955 Europe/Zurich
46.99179 6.931 Europe/Zurich
47.52271 7.64511 Europe/Zurich
46.43301 6.91143 Europe/Zurich
46.25546 6.96066 Europe/Zurich
46.23424 6.08025 Europe/Zurich
47.05048 8.30635 Europe/Zurich
46.01008 8.96004 Europe/Zurich
47.05 8.26274 Europe/Zurich
46.44527 6.89908 Europe/Zurich
46.516 6.63282 Europe/Zurich
47.09993 6.82586 Europe/Zurich
47.03537 8.27631 Europe/Zurich
47.65 9.18333 Europe/Zurich
46.92436 7.41457 Europe/Zurich
47.45152 8.58491 Europe/Zurich
47.22983 8.83884 Europe/Zurich
47.25579 8.60027 Europe/Zurich
47.40313 8.4971 Europe/Zurich
47.38615 9.27916 Europe/Zurich
47.1921 7.39586 Europe/Zurich
47.41694 9.25125 Europe/Zurich
46.20222 6.14569 Europe/Zurich
46.80237 7.15128 Europe/Zurich
47.55816 8.89854 Europe/Zurich
47.0811 8.30477 Europe/Zurich
47.39724 8.61872 Europe/Zurich
47.40165 8.40015 Europe/Zurich
46.84986 9.53287 Europe/Zurich
46.18096 6.13921 Europe/Zurich
47.1324 7.24411 Europe/Zurich
46.94809 7.44744 Europe/Zurich
46.19278 9.01703 Europe/Zurich
47.5584 7.57327 Europe/Zurich
47.47333 8.30592 Europe/Zurich
47.19625 8.52954 Europe/Zurich
47.37752 8.52127 Europe/Zurich
47.55074 7.53599 Europe/Zurich
47.30997 8.52462 Europe/Zurich
47.39254 8.04422 Europe/Zurich
47.57884 7.64683 Europe/Zurich
47.39503 8.52529 Europe/Zurich
47.41814 8.5122 Europe/Zurich
47.3401 8.53134 Europe/Zurich
47.37382 8.51164 Europe/Zurich
47.3953 8.53721 Europe/Zurich
47.37398 8.49007 Europe/Zurich
47.38946 8.48533 Europe/Zurich
47.49494 8.71954 Europe/Zurich
47.40372 8.57608 Europe/Zurich
47.47646 8.76996 Europe/Zurich
47.35785 8.50296 Europe/Zurich
47.42326 8.52166 Europe/Zurich
47.38245 8.47993 Europe/Zurich
47.51692 8.76863 Europe/Zurich
47.40773 8.5005 Europe/Zurich
47.33756 8.5211 Europe/Zurich
47.3548 8.56097 Europe/Zurich
47.37328 8.58038 Europe/Zurich
47.39223 8.54381 Europe/Zurich
46.18981 6.11441 Europe/Zurich
47.69681 8.63346 Europe/Zurich
7.42917 -6.04306 Africa/Abidjan
6.82055 -5.27674 Africa/Abidjan
7.38194 -6.47778 Africa/Abidjan
6.56128 -5.01575 Africa/Abidjan
8.28333 -7.68333 Africa/Abidjan
10.48694 -6.40972 Africa/Abidjan
5.89833 -4.82833 Africa/Abidjan
7.80639 -3.17139 Africa/Abidjan
4.42295 -7.3528 Africa/Abidjan
6.61667 -5.91667 Africa/Abidjan
4.95 -6.08333 Africa/Abidjan
4.74851 -6.6363 Africa/Abidjan
7.45462 -5.29263 Africa/Abidjan
6.38309 -5.41759 Africa/Abidjan
9.51 -7.56917 Africa/Abidjan
8.05861 -6.18972 Africa/Abidjan
7.41251 -7.55383 Africa/Abidjan
5.85278 -5.68833 Africa/Abidjan
9.45 -5.63333 Africa/Abidjan
8.13333 -5.1 Africa/Abidjan
6.49224 -6.58558 Africa/Abidjan
6.54559 -7.49659 Africa/Abidjan
5.2118 -3.73884 Africa/Abidjan
6.315 -3.96028 Africa/Abidjan
6.12926 -5.94371 Africa/Abidjan
9.6 -5.2 Africa/Abidjan
6.74719 -7.34959 Africa/Abidjan
5.83972 -5.36 Africa/Abidjan
6.6451 -4.70477 Africa/Abidjan
7.0591 -3.9631 Africa/Abidjan
7.25957 -8.15498 Africa/Abidjan
6.87736 -6.45022 Africa/Abidjan
5.32556 -4.37685 Africa/Abidjan
9.52167 -6.48694 Africa/Abidjan
9.26667 -3 Africa/Abidjan
7.68949 -5.02177 Africa/Abidjan
6.99041 -5.7442 Africa/Abidjan
5.27414 -3.59502 Africa/Abidjan
6.64861 -4.19917 Africa/Abidjan
8.03333 -2.8 Africa/Abidjan
5.35838 -3.88687 Africa/Abidjan
7.73909 -7.61377 Africa/Abidjan
7.67356 -5.57644 Africa/Abidjan
7.00694 -7.475 Africa/Abidjan
6.66528 -3.97222 Africa/Abidjan
5.49462 -4.05183 Africa/Abidjan
6.39 -3.90028 Africa/Abidjan
7.13444 -3.20472 Africa/Abidjan
5.92801 -4.21319 Africa/Abidjan
6.10694 -3.86194 Africa/Abidjan
5.28634 -3.30402 Africa/Abidjan
5.46779 -3.20711 Africa/Abidjan
5.40861 -4.01601 Africa/Abidjan
5.30943 -4.01972 Africa/Abidjan
6.72972 -3.49639 Africa/Abidjan
7.96111 -6.67306 Africa/Abidjan
5.78556 -6.60833 Africa/Abidjan
-21.20778 -159.775 Pacific/Rarotonga
-33.02457 -71.55183 America/Santiago
-39.28569 -72.2279 America/Santiago
-33.04222 -71.37333 America/Santiago
-38.21667 -72.33333 America/Santiago
-33.03932 -71.62725 America/Santiago
-28.57083 -70.75806 America/Santiago
-39.81422 -73.24589 America/Santiago
-36.61667 -72.95 America/Santiago
-22.09198 -70.19792 America/Santiago
-38.73333 -72.6 America/Santiago
-36.71667 -73.11667 America/Santiago
-35.43333 -71.66667 America/Santiago
-33.66667 -70.93333 America/Santiago
-34.43333 -71.08333 America/Santiago
-33.45694 -70.64827 America/Santiago
-34.63333 -71.36667 America/Santiago
-35.6 -71.75 America/Santiago
-32.75 -70.73333 America/Santiago
-36.42477 -71.958 America/Santiago
-33.6 -70.71667 America/Santiago
-33.59333 -71.62167 America/Santiago
-40.31667 -72.96667 America/Santiago
-34.41667 -70.86667 America/Santiago
-34.17083 -70.74444 America/Santiago
-33.045 -71.44944 America/Santiago
-32.88333 -71.26667 America/Santiago
-53.15 -70.91667 America/Santiago
-41.31946 -72.98538 America/Santiago
-43.11667 -73.61667 America/Santiago
-51.72363 -72.48745 America/Santiago
-41.46985 -72.94474 America/Santiago
-45.4 -72.7 America/Santiago
-33.61667 -70.58333 America/Santiago
-39.28223 -71.95427 America/Santiago
-36.73333 -72.98333 America/Santiago
-33.61667 -70.91667 America/Santiago
-36.15 -71.83333 America/Santiago
-39.63333 -72.33333 America/Santiago
-33.81667 -70.75 America/Santiago
-30.59833 -71.20028 America/Santiago
-40.56667 -73.15 America/Santiago
-38.73333 -72.95 America/Santiago
-37.5 -72.66667 America/Santiago
-37.71667 -72.23333 America/Santiago
-35.11667 -71.28333 America/Santiago
-33.7 -71.21667 America/Santiago
-34.18333 -70.66667 America/Santiago
-37.08333 -73.16667 America/Santiago
-37.46667 -72.35 America/Santiago
-32.83369 -70.59827 America/Santiago
-39.36667 -72.63333 America/Santiago
-32.85 -70.96667 America/Santiago
-35.85 -71.6 America/Santiago
-33.01667 -71.26667 America/Santiago
-37.61667 -73.65 America/Santiago
-38.51667 -72.45 America/Santiago
-40.28333 -73.08333 America/Santiago
-29.90778 -71.25417 America/Santiago
-33.28333 -70.9 America/Santiago
-32.45242 -71.23106 America/Santiago
-37.26667 -72.7 America/Santiago
-20.22083 -70.14306 America/Santiago
-31.63083 -71.16528 America/Santiago
-32.78333 -71.21667 America/Santiago
-34.06667 -70.73333 America/Santiago
-41.12676 -73.04372 America/Santiago
-33.68333 -71.01667 America/Santiago
-26.36667 -70.05 America/Santiago
-34.98333 -71.23333 America/Santiago
-37.46667 -73.35 America/Santiago
-37.01667 -73.13333 America/Santiago
-29.95333 -71.34361 America/Santiago
-27.36667 -70.33333 America/Santiago
-35.33333 -72.41667 America/Santiago
-36.82699 -73.04977 America/Santiago
-37.95 -72.43333 America/Santiago
-45.57524 -72.06619 America/Santiago
-34.7 -71.05 America/Santiago
-36.60664 -72.10344 America/Santiago
-36.91667 -73.01667 America/Santiago
-35.9671 -72.32248 America/Santiago
-42.4721 -73.77319 America/Santiago
-33.55417 -71.60528 America/Santiago
-37.8 -73.4 America/Santiago
-22.46667 -68.93333 America/Santiago
-37.03333 -72.4 America/Santiago
-33.73333 -70.75 America/Santiago
-18.475 -70.30417 America/Santiago
-37.2463 -73.31752 America/Santiago
-23.65 -70.4 America/Santiago
-37.8 -72.71667 America/Santiago
-41.86972 -73.82028 America/Santiago
-39.80867 -73.21821 America/Santiago
-33.58331 -70.63419 America/Santiago
-33.4443 -70.72552 America/Santiago
3.86667 11.51667 Africa/Douala
10.34107 15.23288 Africa/Douala
6.38333 10.06667 Africa/Douala
4.96667 10.7 Africa/Douala
4.0745 9.3699 Africa/Douala
6.46667 12.63333 Africa/Douala
8.4022 14.1698 Africa/Douala
2.93333 11.98333 Africa/Douala
4.63911 9.67987 Africa/Douala
4.16667 11.53333 Africa/Douala
4.51667 12.03333 Africa/Douala
4.9547 9.9404 Africa/Douala
7.31667 13.58333 Africa/Douala
4.68333 12.36667 Africa/Douala
4.2898 9.4103 Africa/Douala
4.0913 9.3144 Africa/Douala
11.04611 14.14011 Africa/Douala
10.73978 13.80188 Africa/Douala
5.1218 9.96143 Africa/Douala
6.51667 14.3 Africa/Douala
5.62578 10.25517 Africa/Douala
4.5016 9.5671 Africa/Douala
4.45 11.9 Africa/Douala
3.51667 11.5 Africa/Douala
10.59095 14.31592 Africa/Douala
4.8428 9.8217 Africa/Douala
5.754 9.3123 Africa/Douala
4.7182 9.7351 Africa/Douala
3.23333 10.73333 Africa/Douala
4.0242 9.2149 Africa/Douala
9.05 13.73333 Africa/Douala
6.2 10.66667 Africa/Douala
4.6363 9.4469 Africa/Douala
2.95 9.91667 Africa/Douala
12.07689 15.03063 Africa/Douala
10.10917 14.45083 Africa/Douala
9.93417 13.94861 Africa/Douala
5.88333 14.55 Africa/Douala
9.3 13.4 Africa/Douala
6.25 10.26667 Africa/Douala
5.50787 10.6356 Africa/Douala
5.7291 10.90011 Africa/Douala
5.4685 9.8818 Africa/Douala
3.65 10.76667 Africa/Douala
3.8 10.13333 Africa/Douala
2.9 11.15 Africa/Douala
5.44559 10.055 Africa/Douala
4.04827 9.70428 Africa/Douala
3.76667 9.98333 Africa/Douala
4.1527 9.241 Africa/Douala
10.7336 14.60928 Africa/Douala
4.58333 13.68333 Africa/Douala
4.93333 13.3 Africa/Douala
4.43333 14.36667 Africa/Douala
6.75 11.81667 Africa/Douala
5.14091 10.51975 Africa/Douala
4.4591 8.9027 Africa/Douala
5.95266 10.15824 Africa/Douala
5.88724 10.01248 Africa/Douala
5.47366 10.41786 Africa/Douala
4.75 11.23333 Africa/Douala
5.15625 10.17882 Africa/Douala
3.76667 12.25 Africa/Douala
4.2475 9.00472 Africa/Douala
29.25 88.88333 Asia/Urumqi
39.74318 98.51736 Asia/Chongqing
38.41667 77.24056 Asia/Kashgar
31.16667 97.23333 Asia/Chongqing
31.5 92 Asia/Chongqing
29.65 91.1 Asia/Chongqing
39.83333 97.73333 Asia/Urumqi
39.45472 75.97972 Asia/Kashgar
39.81667 98.3 Asia/Urumqi
37.09972 79.92694 Asia/Kashgar
29.96178 90.71875 Asia/Chongqing
43.80096 87.60046 Asia/Urumqi
42.93333 89.16667 Asia/Urumqi
44.3 86.03333 Asia/Urumqi
41.72778 82.93639 Asia/Urumqi
41.75972 86.14694 Asia/Urumqi
42.8 93.45 Asia/Urumqi
44.01667 87.31667 Asia/Urumqi
45.63333 85.18333 Asia/Urumqi
40.51556 81.26361 Asia/Kashgar
47.86667 88.11667 Asia/Urumqi
41.12306 80.26444 Asia/Kashgar
27.68667 106.90722 Asia/Chongqing
35.40056 116.96556 Asia/Shanghai
30.12108 104.64811 Asia/Chongqing
29.34162 104.77689 Asia/Chongqing
27.83333 113.15 Asia/Chongqing
32.97944 114.02944 Asia/Shanghai
34.45 115.65 Asia/Shanghai
29.71876 120.24233 Asia/Shanghai
35.99472 119.3975 Asia/Shanghai
39.70083 122.99111 Asia/Harbin
31.11614 120.8451 Asia/Shanghai
33.63333 114.63333 Asia/Shanghai
36.81667 117.81667 Asia/Shanghai
24.355 102.54222 Asia/Chongqing
33.70389 118.67917 Asia/Shanghai
24.51667 103.76667 Asia/Chongqing
31.00611 119.90361 Asia/Shanghai
30.29556 111.50472 Asia/Chongqing
32.26917 119.17889 Asia/Shanghai
32.20917 119.43417 Asia/Shanghai
29.9695 121.68753 Asia/Shanghai
34.75778 113.64861 Asia/Shanghai
32.93917 119.83417 Asia/Shanghai
27.31667 103.71667 Asia/Chongqing
23.05116 112.45972 Asia/Chongqing
39.76667 118.4 Asia/Harbin
37.35917 120.39639 Asia/Shanghai
21.28145 110.34271 Asia/Chongqing
24.51333 117.65556 Asia/Shanghai
38.93417 100.45167 Asia/Chongqing
36.79056 118.06333 Asia/Shanghai
36.09944 114.32889 Asia/Shanghai
34.86472 117.55417 Asia/Shanghai
32.12722 112.75417 Asia/Chongqing
30.05 121.14944 Asia/Shanghai
34.16278 113.46389 Asia/Shanghai
34.06154 108.62905 Asia/Chongqing
31.37762 120.9543 Asia/Shanghai
33.44743 112.71422 Asia/Chongqing
31.06251 113.76545 Asia/Chongqing
21.99102 100.73409 Asia/Chongqing
34.31139 117.95028 Asia/Shanghai
22.93056 112.0373 Asia/Chongqing
35.02306 110.99278 Asia/Chongqing
38.29056 109.74944 Asia/Chongqing
22.63333 110.15 Asia/Chongqing
29.385 106.51944 Asia/Chongqing
37.68028 112.73194 Asia/Chongqing
34.92889 116.46528 Asia/Shanghai
38.71528 112.7575 Asia/Shanghai
30.41513 112.88595 Asia/Chongqing
23.73333 114.68333 Asia/Chongqing
27.42829 112.18338 Asia/Chongqing
29.35139 105.89472 Asia/Chongqing
35.78472 118.62806 Asia/Shanghai
31.16611 112.58306 Asia/Chongqing
32.62945 116.27014 Asia/Shanghai
30.95 113.55 Asia/Chongqing
38.46806 106.27306 Asia/Chongqing
34.73806 111.88389 Asia/Chongqing
35.81139 114.31667 Asia/Shanghai
36.69667 118.47972 Asia/Shanghai
27.83333 114.4 Asia/Shanghai
31.70472 112.25611 Asia/Chongqing
31.37053 119.87027 Asia/Shanghai
30.71444 111.28472 Asia/Chongqing
28.76667 104.62383 Asia/Chongqing
37.15 122.38333 Asia/Shanghai
22.1975 109.94194 Asia/Chongqing
35.55278 116.82861 Asia/Shanghai
37.53333 121.4 Asia/Shanghai
34.65918 109.22921 Asia/Chongqing
32.39722 119.43583 Asia/Shanghai
24.78081 110.48967 Asia/Chongqing
31.865 120.53889 Asia/Shanghai
37.8575 113.56333 Asia/Shanghai
39.1375 116.99972 Asia/Shanghai
36.11056 115.77528 Asia/Shanghai
39.36389 117.06028 Asia/Shanghai
33.38556 120.12528 Asia/Shanghai
34.18045 117.15707 Asia/Shanghai
20.35 110.12884 Asia/Chongqing
28.45322 117.9686 Asia/Shanghai
28.45433 104.71498 Asia/Chongqing
34.01667 113.81667 Asia/Shanghai
30.9525 118.75528 Asia/Shanghai
35.16278 112.865 Asia/Chongqing
29.71667 112.4 Asia/Chongqing
25.11667 114.3 Asia/Chongqing
28.28857 117.21329 Asia/Shanghai
38.40917 112.73333 Asia/Shanghai
30.86667 114.8 Asia/Shanghai
36.49889 111.70472 Asia/Chongqing
27.80429 114.93335 Asia/Shanghai
32.51861 112.35222 Asia/Chongqing
32.12278 114.06556 Asia/Shanghai
35.30889 113.86722 Asia/Shanghai
35.90056 117.75194 Asia/Shanghai
31.04704 113.14098 Asia/Chongqing
34.59972 119.15944 Asia/Shanghai
37.90278 115.20361 Asia/Shanghai
36.61667 101.76667 Asia/Chongqing
27.74278 111.29556 Asia/Chongqing
37.06306 114.49417 Asia/Shanghai
32.68 109.01722 Asia/Chongqing
36.7975 118.29444 Asia/Shanghai
29.81667 113.46667 Asia/Chongqing
36.24861 116.76583 Asia/Shanghai
23.15486 112.89161 Asia/Chongqing
24.96389 118.37944 Asia/Shanghai
33.78333 114.51667 Asia/Shanghai
31.68636 113.46585 Asia/Chongqing
27.89642 102.26342 Asia/Chongqing
34.91611 118.63889 Asia/Shanghai
34.80222 117.11167 Asia/Shanghai
30.53629 120.68638 Asia/Shanghai
34.76 112.97139 Asia/Chongqing
26.19028 107.5125 Asia/Chongqing
30.16746 120.25883 Asia/Shanghai
32.03333 118.85 Asia/Shanghai
30.91667 113.9 Asia/Chongqing
34.33778 108.70261 Asia/Chongqing
30.38333 113.4 Asia/Chongqing
38.985 117.38278 Asia/Shanghai
32.43667 119.55083 Asia/Shanghai
29.88333 114.21667 Asia/Shanghai
28.85 120.73333 Asia/Shanghai
22.27694 113.56778 Asia/Chongqing
28.66667 112.88333 Asia/Chongqing
27.73333 112.53333 Asia/Chongqing
27.85 112.9 Asia/Chongqing
32.0422 112.14479 Asia/Chongqing
33.84703 113.4778 Asia/Shanghai
34.25833 108.92861 Asia/Chongqing
24.47979 118.08187 Asia/Shanghai
23.48333 111.31667 Asia/Chongqing
31.9925 116.24722 Asia/Shanghai
29.85058 115.5525 Asia/Shanghai
31.56887 120.28857 Asia/Shanghai
29.26778 120.22528 Asia/Shanghai
29.04638 111.6783 Asia/Shanghai
31.33657 118.37548 Asia/Shanghai
30.58333 114.26667 Asia/Shanghai
39.66472 106.81222 Asia/Chongqing
39.49944 106.71167 Asia/Chongqing
29.60077 118.17495 Asia/Shanghai
27.99942 120.66682 Asia/Shanghai
35.7275 116.49611 Asia/Shanghai
28.36694 121.36 Asia/Shanghai
37.19361 122.05111 Asia/Shanghai
34.50355 109.50891 Asia/Chongqing
37.50167 122.11361 Asia/Shanghai
36.71 119.10194 Asia/Shanghai
30.80389 108.38972 Asia/Chongqing
18.80028 110.39667 Asia/Chongqing
35.87861 119.97528 Asia/Shanghai
39.61833 122.00806 Asia/Harbin
29.71139 118.3125 Asia/Shanghai
28.1375 106.82 Asia/Chongqing
39.90528 116.59944 Asia/Harbin
27.32734 120.214 Asia/Shanghai
27.71722 109.18528 Asia/Chongqing
30.95 117.78333 Asia/Shanghai
31.09407 105.08731 Asia/Chongqing
30.98664 103.93933 Asia/Chongqing
39.14222 117.17667 Asia/Shanghai
35.07706 117.15176 Asia/Shanghai
34.85 117.33333 Asia/Shanghai
26.03139 119.59861 Asia/Shanghai
34.43278 116.59111 Asia/Shanghai
39.63333 118.18333 Asia/Harbin
22.02647 111.93229 Asia/Chongqing
39.73333 118.45 Asia/Harbin
32.68833 112.8275 Asia/Chongqing
39.02111 117.64694 Asia/Shanghai
32.49333 119.91063 Asia/Shanghai
37.86944 112.56028 Asia/Chongqing
32.16667 120.01361 Asia/Shanghai
22.25135 112.7799 Asia/Chongqing
36.18528 117.12 Asia/Shanghai
33.63611 116.97889 Asia/Shanghai
36.95389 118.10472 Asia/Shanghai
31.71111 113.36306 Asia/Chongqing
33.89067 116.77472 Asia/Shanghai
33.8963 117.93307 Asia/Shanghai
30.50802 105.57332 Asia/Chongqing
31.03595 121.2146 Asia/Shanghai
35.64889 117.27583 Asia/Shanghai
36.85917 120.52694 Asia/Shanghai
36.88 118.7375 Asia/Shanghai
39.23333 106.76944 Asia/Chongqing
35.17111 118.82889 Asia/Shanghai
32.6475 110.77806 Asia/Chongqing
32.56667 110.78333 Asia/Chongqing
22.99693 113.07988 Asia/Chongqing
39.23417 106.34389 Asia/Chongqing
22.93736 113.36063 Asia/Chongqing
22.51682 113.38521 Asia/Chongqing
24.44778 117.81111 Asia/Shanghai
23.11227 113.84248 Asia/Chongqing
35.08056 109.08972 Asia/Chongqing
38.04139 114.47861 Asia/Shanghai
22.54554 114.0683 Asia/Chongqing
36.24111 115.66722 Asia/Shanghai
29.95762 122.29802 Asia/Shanghai
30.30722 112.24472 Asia/Chongqing
22.75452 112.95652 Asia/Chongqing
30.01102 120.57153 Asia/Shanghai
27.34089 117.4831 Asia/Shanghai
24.8 113.58333 Asia/Chongqing
34.79528 116.08167 Asia/Shanghai
22.78199 115.3475 Asia/Shanghai
23.36814 116.71479 Asia/Shanghai
35.07528 117.45778 Asia/Shanghai
31.22222 121.45806 Asia/Shanghai
36.93833 114.50583 Asia/Shanghai
18.24306 109.505 Asia/Chongqing
26.24861 117.61861 Asia/Shanghai
33.00111 114.35417 Asia/Shanghai
25.725 119.37944 Asia/Shanghai
23.52886 116.36416 Asia/Shanghai
35.4275 119.45528 Asia/Shanghai
38.69889 116.09361 Asia/Shanghai
28.95944 118.86861 Asia/Shanghai
25.48333 103.78333 Asia/Chongqing
35.59667 116.99111 Asia/Shanghai
24.91389 118.58583 Asia/Shanghai
26.57307 111.84184 Asia/Chongqing
37.2975 120.82556 Asia/Shanghai
20.00583 110.35417 Asia/Chongqing
28.84298 112.35217 Asia/Chongqing
21.95 108.61667 Asia/Chongqing
33.25306 119.91333 Asia/Shanghai
39.93167 119.58833 Asia/Harbin
24.81978 118.57414 Asia/Shanghai
30.45113 115.25593 Asia/Shanghai
33.58861 119.01917 Asia/Shanghai
36.09861 120.37194 Asia/Shanghai
23.7 113.03333 Asia/Chongqing
26.76837 112.10814 Asia/Chongqing
35.70278 115.00528 Asia/Shanghai
29.46028 119.88611 Asia/Shanghai
25.43944 119.01028 Asia/Shanghai
29.71667 113.88333 Asia/Chongqing
22.75 108.48333 Asia/Chongqing
39.39528 121.96694 Asia/Harbin
28.99242 116.66754 Asia/Shanghai
36.28306 116.44528 Asia/Shanghai
35.50056 117.63083 Asia/Shanghai
37.2025 112.17806 Asia/Chongqing
27.61667 113.85 Asia/Chongqing
22.99376 114.71311 Asia/Chongqing
23.5543 110.39757 Asia/Chongqing
35.53917 106.68611 Asia/Chongqing
36.78444 119.94639 Asia/Shanghai
33.73847 113.30119 Asia/Chongqing
36.43111 114.17 Asia/Shanghai
26.5479 101.70539 Asia/Chongqing
35.76417 116.79139 Asia/Shanghai
28.25 112.55 Asia/Chongqing
37.38333 121.58333 Asia/Shanghai
29.28917 121.42472 Asia/Shanghai
29.87819 121.54945 Asia/Shanghai
29.58354 105.06216 Asia/Chongqing
29.35955 112.40243 Asia/Chongqing
31.78394 111.82752 Asia/Chongqing
32.99472 112.53278 Asia/Chongqing
32.03028 120.87472 Asia/Shanghai
27.92333 118.53333 Asia/Shanghai
26.645 118.17361 Asia/Shanghai
22.81667 108.31667 Asia/Chongqing
36.18167 118.15833 Asia/Shanghai
31.35333 106.06309 Asia/Chongqing
32.06167 118.77778 Asia/Shanghai
37.35806 115.37444 Asia/Shanghai
23.74344 111.79593 Asia/Chongqing
22.8525 110.82333 Asia/Chongqing
36.74833 118.05583 Asia/Shanghai
30.79508 106.08474 Asia/Chongqing
28.68333 115.88333 Asia/Shanghai
24.40417 103.44278 Asia/Chongqing
36.71667 117.5 Asia/Shanghai
32.78017 117.96378 Asia/Shanghai
32.45861 114.04861 Asia/Shanghai
31.45934 104.75424 Asia/Chongqing
39.93917 116.09167 Asia/Harbin
35.70694 117.92639 Asia/Shanghai
33.26611 116.56605 Asia/Shanghai
24.29769 116.10724 Asia/Shanghai
21.45713 110.76591 Asia/Chongqing
25.03333 102.63611 Asia/Chongqing
30.42139 111.75333 Asia/Chongqing
31.17833 115.03194 Asia/Shanghai
24.68412 113.59839 Asia/Chongqing
38.8 121.26667 Asia/Shanghai
28.58084 121.37715 Asia/Shanghai
34.68361 112.45361 Asia/Chongqing
24.96222 118.68111 Asia/Shanghai
23.16244 114.27342 Asia/Chongqing
24.40583 109.60861 Asia/Chongqing
33.57167 114.03528 Asia/Shanghai
29.38481 104.02862 Asia/Chongqing
31.23357 117.28057 Asia/Shanghai
30.05127 101.96033 Asia/Chongqing
23.17422 112.28094 Asia/Chongqing
37.87917 114.65167 Asia/Shanghai
27.73444 111.99444 Asia/Chongqing
24.67192 102.1613 Asia/Chongqing
37.65083 120.32861 Asia/Shanghai
31.4308 119.52399 Asia/Shanghai
28.15 113.63333 Asia/Chongqing
23.31072 116.16869 Asia/Shanghai
28.46042 119.91029 Asia/Shanghai
34.37803 109.20892 Asia/Chongqing
35.06306 118.34278 Asia/Shanghai
35.60028 103.20639 Asia/Chongqing
39.7 118.43333 Asia/Harbin
36.42472 114.20472 Asia/Shanghai
36.51556 118.53972 Asia/Shanghai
30.41587 103.46088 Asia/Chongqing
36.84556 115.71167 Asia/Shanghai
30.4225 120.29722 Asia/Shanghai
28.85 121.11667 Asia/Shanghai
22.70722 110.34917 Asia/Chongqing
19.90778 109.68583 Asia/Chongqing
36.08889 111.51889 Asia/Chongqing
27.66667 113.5 Asia/Chongqing
30.3 108.85 Asia/Chongqing
23.29554 113.82836 Asia/Chongqing
36.44389 115.96472 Asia/Shanghai
24.78111 112.3825 Asia/Chongqing
21.6682 109.16946 Asia/Chongqing
24.92271 102.48496 Asia/Chongqing
39.73194 116.14 Asia/Shanghai
21.64673 110.28172 Asia/Chongqing
29.56228 103.76386 Asia/Chongqing
26.4111 111.59559 Asia/Chongqing
27.68806 111.42944 Asia/Chongqing
26.40238 112.85908 Asia/Chongqing
25.12383 113.32542 Asia/Chongqing
32.38583 111.66778 Asia/Chongqing
36.05639 103.79222 Asia/Chongqing
29.21946 119.47695 Asia/Shanghai
27.68833 111.66417 Asia/Chongqing
39.50972 116.69472 Asia/Shanghai
37.17167 119.92139 Asia/Shanghai
36.97583 120.71361 Asia/Shanghai
36.19278 117.65694 Asia/Shanghai
23.7 109.26667 Asia/Chongqing
27.66583 120.56583 Asia/Shanghai
25.03889 102.71833 Asia/Chongqing
23.69767 103.30372 Asia/Chongqing
23.3687 104.28 Asia/Chongqing
34.79111 114.34833 Asia/Shanghai
35.39472 116.08833 Asia/Shanghai
32.31737 121.18552 Asia/Shanghai
29.73333 115.98333 Asia/Shanghai
33.73333 115.4 Asia/Shanghai
28.31667 109.71667 Asia/Chongqing
39.1 121.71667 Asia/Harbin
27.43265 120.60625 Asia/Shanghai
29.60487 111.87012 Asia/Chongqing
32.08982 121.07355 Asia/Shanghai
19.72917 110.00639 Asia/Chongqing
23.22806 110.82611 Asia/Chongqing
35.405 116.58139 Asia/Shanghai
29.10778 119.65472 Asia/Shanghai
30.35028 112.19028 Asia/Chongqing
37.99806 114.01556 Asia/Shanghai
31.03361 112.20472 Asia/Chongqing
30.65 113.1 Asia/Chongqing
29.2947 117.20789 Asia/Shanghai
38.49528 102.17389 Asia/Chongqing
35.50222 112.83278 Asia/Chongqing
36.66833 116.99722 Asia/Shanghai
36.38972 120.46222 Asia/Shanghai
29.29056 106.25417 Asia/Chongqing
37.02444 111.9125 Asia/Chongqing
33.26338 115.36108 Asia/Shanghai
22.81338 115.82574 Asia/Shanghai
35.54278 118.455 Asia/Shanghai
22.87791 116.06705 Asia/Shanghai
30.7522 120.75 Asia/Shanghai
35.23972 113.23306 Asia/Shanghai
36.28389 120.00333 Asia/Shanghai
26.66167 119.52278 Asia/Shanghai
27.04694 118.32528 Asia/Shanghai
32.50611 120.14278 Asia/Shanghai
28.19377 115.7836 Asia/Shanghai
22.58333 113.08333 Asia/Chongqing
25.48722 119.19861 Asia/Shanghai
21.85 111.96667 Asia/Chongqing
27.11716 114.97927 Asia/Shanghai
32.44202 105.823 Asia/Chongqing
30.8703 120.0933 Asia/Shanghai
31.53333 119.48333 Asia/Shanghai
32.35473 116.2939 Asia/Shanghai
22.81898 113.67306 Asia/Chongqing
23.08333 114.4 Asia/Chongqing
31.81111 121.655 Asia/Shanghai
23.03845 116.28988 Asia/Shanghai
22.456 113.0482 Asia/Chongqing
21.63333 110.58333 Asia/Chongqing
30.45 114.8 Asia/Shanghai
28.64778 121.25944 Asia/Shanghai
30.20417 115.07761 Asia/Shanghai
23.10998 113.44437 Asia/Chongqing
30.88453 114.37789 Asia/Shanghai
30.19235 116.02496 Asia/Shanghai
23.67704 116.99961 Asia/Shanghai
39.72139 116.32917 Asia/Shanghai
32.12722 115.03944 Asia/Shanghai
32.95893 117.16566 Asia/Shanghai
32.62639 116.99694 Asia/Shanghai
23.90513 112.19314 Asia/Chongqing
27.54944 109.95917 Asia/Chongqing
33.43333 115.03333 Asia/Shanghai
33.97444 116.79167 Asia/Shanghai
27.11 109.99556 Asia/Chongqing
35.24306 115.44111 Asia/Shanghai
29.99472 106.25722 Asia/Chongqing
28.58917 112.32833 Asia/Chongqing
23.43077 115.82991 Asia/Shanghai
26.88806 112.615 Asia/Chongqing
37.73222 115.70111 Asia/Shanghai
31.86389 117.28083 Asia/Shanghai
33.76806 120.25167 Asia/Shanghai
36.53333 114.11111 Asia/Shanghai
35.89917 114.1925 Asia/Shanghai
33.07278 107.03028 Asia/Chongqing
27.08917 119.64528 Asia/Shanghai
36.77083 119.21083 Asia/Shanghai
30.29365 120.16142 Asia/Shanghai
39.24889 117.78917 Asia/Shanghai
36.60056 114.46778 Asia/Shanghai
30.65 113.76667 Asia/Chongqing
35.46028 110.42917 Asia/Chongqing
34.58167 119.12889 Asia/Shanghai
28.68028 121.44278 Asia/Shanghai
23.19349 116.61422 Asia/Shanghai
24.77985 102.57548 Asia/Chongqing
20.04583 110.34167 Asia/Chongqing
39.73333 118.41667 Asia/Harbin
34.36591 107.35904 Asia/Chongqing
28.88333 120.03333 Asia/Shanghai
26.58333 106.71667 Asia/Chongqing
33.66972 118.18889 Asia/Shanghai
23.3925 110.08139 Asia/Chongqing
25.28194 110.28639 Asia/Chongqing
23.09639 109.60917 Asia/Chongqing
32.26604 111.63476 Asia/Chongqing
23.11667 113.25 Asia/Chongqing
31.6199 113.9978 Asia/Shanghai
23.30602 103.16351 Asia/Chongqing
21.93924 110.84607 Asia/Chongqing
32.78583 119.42778 Asia/Shanghai
30.77576 106.10294 Asia/Chongqing
36.38333 119.75278 Asia/Shanghai
34.0175 119.18861 Asia/Shanghai
26.06139 119.30611 Asia/Shanghai
32.9 115.81667 Asia/Shanghai
30.05333 119.95194 Asia/Shanghai
37.49583 121.25806 Asia/Shanghai
29.70222 107.39194 Asia/Chongqing
23.02677 113.13148 Asia/Chongqing
27.81117 114.66805 Asia/Shanghai
34.70388 116.58717 Asia/Shanghai
39.83333 118.11667 Asia/Harbin
30.08333 113.26667 Asia/Chongqing
35.26056 117.9675 Asia/Shanghai
39.69833 115.9925 Asia/Shanghai
30.4 114.83333 Asia/Shanghai
30.3 109.48333 Asia/Chongqing
22.18389 112.3026 Asia/Chongqing
26.26667 107.51667 Asia/Chongqing
30.67 112.68952 Asia/Chongqing
23.23729 111.53172 Asia/Chongqing
22.37303 110.94746 Asia/Chongqing
37.45639 118.48556 Asia/Shanghai
32.85231 120.30947 Asia/Shanghai
39.81608 109.97763 Asia/Chongqing
33.99972 119.83083 Asia/Shanghai
22.94594 115.64204 Asia/Shanghai
23.04889 113.74472 Asia/Chongqing
35.85 117.7 Asia/Shanghai
36.77667 121.15972 Asia/Shanghai
38.51306 114.99556 Asia/Shanghai
35.06833 115.5625 Asia/Shanghai
37.44861 116.2925 Asia/Shanghai
31.13019 104.38198 Asia/Chongqing
30.54485 119.9599 Asia/Shanghai
37.80861 120.75083 Asia/Shanghai
32.68222 112.08194 Asia/Chongqing
34.45528 113.02806 Asia/Chongqing
33.19973 120.4578 Asia/Shanghai
29.12944 110.47833 Asia/Chongqing
30.08333 114.95 Asia/Shanghai
26.86879 100.22072 Asia/Chongqing
31.21592 107.50092 Asia/Chongqing
39.04194 106.39583 Asia/Chongqing
32.62082 117.06318 Asia/Shanghai
29.65628 121.4064 Asia/Shanghai
35.56389 114.50583 Asia/Shanghai
22.7984 114.46716 Asia/Chongqing
32.54278 111.50861 Asia/Chongqing
31.55978 118.48289 Asia/Shanghai
33.63333 115.18333 Asia/Shanghai
22.85042 113.24701 Asia/Chongqing
38.91222 121.60222 Asia/Shanghai
25.7 100.18333 Asia/Chongqing
32.32194 118.29778 Asia/Shanghai
22.16667 111.78333 Asia/Chongqing
29.31506 120.07676 Asia/Shanghai
29.56278 106.55278 Asia/Chongqing
29.78062 104.85224 Asia/Chongqing
30.65778 117.48306 Asia/Shanghai
25.8 113.03333 Asia/Chongqing
35.57944 118.83278 Asia/Shanghai
31.92825 120.31257 Asia/Shanghai
30.66667 104.06667 Asia/Chongqing
23.46132 116.77007 Asia/Shanghai
23.66513 116.63786 Asia/Shanghai
31.6 117.86667 Asia/Shanghai
31.78333 119.96667 Asia/Shanghai
35.20889 111.73861 Asia/Chongqing
28.2 112.96667 Asia/Chongqing
36.5575 116.72722 Asia/Shanghai
39.70417 119.15056 Asia/Harbin
28.7 115.81667 Asia/Shanghai
30.2297 115.43346 Asia/Shanghai
26.85 104.23333 Asia/Chongqing
38.31667 116.86667 Asia/Shanghai
30.58333 114.03333 Asia/Shanghai
30.28757 112.22979 Asia/Chongqing
33.87722 115.77028 Asia/Shanghai
38.06667 116.56667 Asia/Shanghai
36.48333 117.83333 Asia/Shanghai
23.89972 106.61333 Asia/Chongqing
37.36667 118.01667 Asia/Shanghai
23.22378 108.80216 Asia/Chongqing
27.3125 105.29417 Asia/Chongqing
34.84861 118.04472 Asia/Shanghai
32.94083 117.36083 Asia/Shanghai
39.9075 116.39723 Asia/Harbin
21.48333 109.1 Asia/Chongqing
34.56861 105.89333 Asia/Chongqing
39.82222 119.51806 Asia/Harbin
29.82556 106.43389 Asia/Chongqing
33.22917 119.30917 Asia/Shanghai
31.55037 105.99381 Asia/Chongqing
38.85111 115.49028 Asia/Shanghai
36.55833 104.20806 Asia/Chongqing
22.15 107.23333 Asia/Chongqing
30.01556 120.87111 Asia/Shanghai
24.41667 111.51667 Asia/Chongqing
29.4 112.15 Asia/Chongqing
26.25 105.93333 Asia/Chongqing
36.43417 119.1925 Asia/Shanghai
30.50917 117.05056 Asia/Shanghai
23.01279 104.45081 Asia/Chongqing
31.2575 113.67833 Asia/Chongqing
27.31944 110.10306 Asia/Chongqing
23.44895 116.68092 Asia/Shanghai
31.76667 104.71667 Asia/Chongqing
31.31139 120.61806 Asia/Shanghai
30.01075 122.20465 Asia/Shanghai
31.25196 120.51179 Asia/Shanghai
31.1933 120.71758 Asia/Shanghai
21.32256 110.58291 Asia/Chongqing
29.99152 120.89845 Asia/Shanghai
35.91222 116.31167 Asia/Shanghai
39.02861 121.695 Asia/Harbin
29.33333 113.09194 Asia/Chongqing
26.46722 111.96806 Asia/Chongqing
45.84955 123.2973 Asia/Harbin
43.50639 123.50639 Asia/Harbin
45.68333 125.31667 Asia/Harbin
45.5 125.13333 Asia/Harbin
46.08333 125.98333 Asia/Harbin
40.81 114.87944 Asia/Harbin
48 122.71667 Asia/Harbin
44.8 126.53333 Asia/Harbin
47.86667 128.83333 Asia/Harbin
40.66482 122.22833 Asia/Harbin
46.31667 129.56667 Asia/Harbin
47.7 128.9 Asia/Harbin
41.3975 119.64083 Asia/Harbin
43.29194 126.00944 Asia/Harbin
42.9075 129.50778 Asia/Harbin
49.28333 120.73333 Asia/Harbin
40.61028 115.04472 Asia/Harbin
40.29278 123.27444 Asia/Harbin
48.23333 129.5 Asia/Harbin
41.99083 122.82528 Asia/Harbin
43.95611 125.46611 Asia/Harbin
40.61667 120.71667 Asia/Harbin
43.96667 116.03333 Asia/Harbin
42.73722 124.72222 Asia/Harbin
41.29711 124.12092 Asia/Harbin
44.91428 127.15001 Asia/Harbin
43.32179 129.76342 Asia/Harbin
42.05972 126.91583 Asia/Harbin
46.83333 126.5 Asia/Harbin
46.08333 122.08333 Asia/Harbin
42.96611 129.8425 Asia/Harbin
43.6125 122.26528 Asia/Harbin
42.29306 123.84139 Asia/Harbin
46.95 128.05 Asia/Harbin
45.33333 122.78333 Asia/Harbin
46.38333 123.4 Asia/Harbin
46.83333 124.41667 Asia/Harbin
52.33333 124.73333 Asia/Harbin
41.65917 123.33917 Asia/Harbin
47.25 127.08333 Asia/Harbin
46.64056 126.99694 Asia/Harbin
44.4 131.16667 Asia/Harbin
42.1859 127.47895 Asia/Harbin
43.16333 124.36861 Asia/Harbin
40.125 116.64528 Asia/Harbin
44.41667 126.95 Asia/Harbin
46.63611 131.15389 Asia/Harbin
43.52417 125.67361 Asia/Harbin
45.35 126.28333 Asia/Harbin
40.70583 110.28556 Asia/Chongqing
41.79222 123.43278 Asia/Harbin
44.7 127.2 Asia/Harbin
45.21667 127.96667 Asia/Harbin
42.08167 126.60028 Asia/Harbin
40.54139 110.51083 Asia/Chongqing
46.65 124.86667 Asia/Harbin
45.8 130.85 Asia/Harbin
47.34083 123.96722 Asia/Harbin
46.68333 126.08333 Asia/Harbin
45.11712 124.85676 Asia/Harbin
42.03722 119.28889 Asia/Harbin
42.94222 126.05611 Asia/Harbin
41.18806 122.04944 Asia/Harbin
47.51667 122.88333 Asia/Harbin
49.18333 125.21667 Asia/Harbin
48.48333 124.83333 Asia/Harbin
40.9241 122.80437 Asia/Harbin
41.09822 120.74792 Asia/Harbin
47.13333 129.26667 Asia/Harbin
42.11667 118.78333 Asia/Harbin
44.58333 129.6 Asia/Harbin
45.55 131.88333 Asia/Harbin
43.10694 128.92167 Asia/Harbin
47.16667 125.91667 Asia/Harbin
42.52722 125.67528 Asia/Harbin
49.6 117.43333 Asia/Harbin
42.77139 129.42333 Asia/Harbin
47.33028 123.18361 Asia/Harbin
46.55 125.11667 Asia/Harbin
42.24007 125.71717 Asia/Harbin
43.30472 124.32778 Asia/Harbin
45.3 130.28333 Asia/Harbin
41.80694 126.90778 Asia/Harbin
41.24 119.40111 Asia/Harbin
46.56694 131.14528 Asia/Harbin
41.50611 122.72417 Asia/Harbin
42.90361 125.13583 Asia/Harbin
41.27194 123.17306 Asia/Harbin
46.26222 126.28556 Asia/Harbin
46.95 128.88333 Asia/Harbin
40.02611 124.32861 Asia/Harbin
40.72861 124.78472 Asia/Harbin
42.53306 124.04028 Asia/Harbin
44.81351 123.15 Asia/Harbin
45.3 130.96667 Asia/Harbin
44.1525 125.83278 Asia/Harbin
41.06667 122.95 Asia/Harbin
44.31667 126.8 Asia/Harbin
41.10778 121.14167 Asia/Harbin
40.76432 120.85327 Asia/Harbin
41.0275 113.10583 Asia/Harbin
43.85083 126.56028 Asia/Harbin
45.21667 131.08333 Asia/Harbin
43.72861 127.34472 Asia/Harbin
46.83333 130.35 Asia/Harbin
49.45 117.7 Asia/Harbin
50.41667 124.11667 Asia/Harbin
41.93222 123.51139 Asia/Harbin
42.8675 130.35806 Asia/Harbin
47.20417 123.63333 Asia/Harbin
45.98333 126.6 Asia/Harbin
42.6225 126.26139 Asia/Harbin
41.26472 125.36667 Asia/Harbin
43.55833 128.02389 Asia/Harbin
46.21667 130.51667 Asia/Harbin
42.96333 126.74778 Asia/Harbin
46.46667 124.86667 Asia/Harbin
40.81056 111.65222 Asia/Chongqing
45.2 130.91667 Asia/Harbin
42.53974 128.99722 Asia/Harbin
41.68917 122.11278 Asia/Harbin
50.24413 127.49016 Asia/Harbin
47.4 130.36667 Asia/Harbin
45.75 126.65 Asia/Harbin
47.45 126.93333 Asia/Harbin
44.58333 129.41667 Asia/Harbin
49.2 119.7 Asia/Harbin
40.85158 122.74754 Asia/Harbin
43.50075 124.81979 Asia/Harbin
41.11667 123.45 Asia/Harbin
50.78333 121.51667 Asia/Harbin
47.91472 123.50306 Asia/Harbin
48.36306 134.28917 Asia/Harbin
46.43333 127.08333 Asia/Harbin
47.78333 124.45 Asia/Harbin
45.18333 124.81667 Asia/Harbin
42.05889 121.745 Asia/Harbin
42.01556 121.65889 Asia/Harbin
41.85583 123.92333 Asia/Harbin
46.72167 131.13944 Asia/Harbin
47.26 132.03222 Asia/Harbin
40.43944 113.15361 Asia/Harbin
47.58333 130.83333 Asia/Harbin
40.45361 124.07167 Asia/Harbin
43.6475 111.97667 Asia/Harbin
41.77639 126.03194 Asia/Harbin
43.3725 128.2425 Asia/Harbin
44.06219 131.12075 Asia/Harbin
41.81444 123.57583 Asia/Harbin
42.64031 125.51176 Asia/Harbin
45.36667 130.8 Asia/Harbin
44.53333 125.7 Asia/Harbin
40.09361 113.29139 Asia/Shanghai
43.30667 128.51139 Asia/Harbin
40.63732 122.50251 Asia/Harbin
46.58333 125 Asia/Harbin
40.12917 124.39472 Asia/Harbin
41.16528 121.36667 Asia/Harbin
45.5 124.3 Asia/Harbin
42.26833 118.96361 Asia/Harbin
45.33333 131.06667 Asia/Harbin
40.9725 117.93611 Asia/Harbin
42.66223 126.0263 Asia/Harbin
41.57028 120.45861 Asia/Harbin
42.77884 124.09545 Asia/Harbin
40.21861 116.21972 Asia/Harbin
44.26532 124.00045 Asia/Harbin
43.88 125.32278 Asia/Harbin
44.76667 129.7 Asia/Harbin
45.76667 130.51667 Asia/Harbin
45.75281 127.47986 Asia/Harbin
41.28861 123.765 Asia/Harbin
41.79194 120.77917 Asia/Harbin
46.08333 127.4 Asia/Harbin
40.65222 109.82222 Asia/Chongqing
46.55861 131.42444 Asia/Harbin
46.33167 132.21111 Asia/Harbin
44.93333 130.53333 Asia/Harbin
43.58333 127.56667 Asia/Harbin
47.58333 126.08333 Asia/Harbin
45.61667 122.81667 Asia/Harbin
41.94306 126.42861 Asia/Harbin
41.12361 122.99 Asia/Harbin
46.4 125.31667 Asia/Harbin
50.56667 123.71667 Asia/Harbin
45.54545 126.97703 Asia/Harbin
21.02448 110.39613 Asia/Urumqi
31.64615 120.74221 Asia/Shanghai
24.94824 114.06572 Asia/Shanghai
30.84918 120.92583 Asia/Shanghai
27.93557 109.59961 Asia/Shanghai
35.99502 119.40259 Asia/Shanghai
5.02208 -74.00481 America/Bogota
4.39462 -76.0715 America/Bogota
7.48971 -74.86919 America/Bogota
3.58234 -76.49146 America/Bogota
5.33775 -72.39586 America/Bogota
6.96321 -75.41738 America/Bogota
5.06242 -75.87159 America/Bogota
5.00886 -74.47226 America/Bogota
4.142 -73.62664 America/Bogota
7.83389 -72.47417 America/Bogota
10.60528 -72.98 America/Bogota
5.28333 -71.96667 America/Bogota
5.04492 -75.5146 America/Bogota
10.46314 -73.25322 America/Bogota
6.3152 -76.1342 America/Bogota
5.30933 -73.81574 America/Bogota
8.09368 -76.72691 America/Bogota
10.32944 -75.41137 America/Bogota
1.09234 -77.62085 America/Bogota
5.53528 -73.36778 America/Bogota
1.79861 -78.81556 America/Bogota
4.08466 -76.19536 America/Bogota
9.52392 -75.58139 America/Bogota
8.17361 -76.05917 America/Bogota
6.461 -71.73004 America/Bogota
8.81136 -74.72084 America/Bogota
5.71062 -75.31068 America/Bogota
10.91843 -74.76459 America/Bogota
5.71434 -72.93391 America/Bogota
6.46838 -73.26022 America/Bogota
4.57937 -74.21682 America/Bogota
9.30472 -75.39778 America/Bogota
9.24391 -75.14675 America/Bogota
4.48425 -74.24499 America/Bogota
4.26425 -75.93085 America/Bogota
7.07993 -74.6989 America/Bogota
6.13833 -75.26417 America/Bogota
10.75773 -74.75451 America/Bogota
4.86806 -75.62139 America/Bogota
3.00945 -76.48494 America/Bogota
11.24079 -74.19904 America/Bogota
10.3242 -74.96017 America/Bogota
9.73586 -75.52626 America/Bogota
3.69637 -73.69958 America/Bogota
8.65972 -75.12809 America/Bogota
9.95156 -75.08198 America/Bogota
10.77108 -73.00314 America/Bogota
9.82767 -75.1217 America/Bogota
6.55573 -73.13309 America/Bogota
8.79577 -75.69947 America/Bogota
7.79177 -74.77316 America/Bogota
8.92901 -75.02709 America/Bogota
12.58472 -81.70056 America/Bogota
9.18361 -75.38167 America/Bogota
5.40733 -75.48749 America/Bogota
8.94616 -75.44275 America/Bogota
6.15 -75.6 America/Bogota
10.62962 -74.92063 America/Bogota
10.79115 -74.76059 America/Bogota
4.41256 -76.15457 America/Bogota
5.42164 -75.70318 America/Bogota
6.15515 -75.37371 America/Bogota
11.54444 -72.90722 America/Bogota
10.49194 -75.12917 America/Bogota
4.62306 -75.76278 America/Bogota
5.69472 -76.66111 America/Bogota
3.23114 -76.41668 America/Bogota
8.36361 -72.4063 America/Bogota
4.08451 -72.95597 America/Bogota
10.98778 -74.95472 America/Bogota
5.97584 -74.58846 America/Bogota
6.49156 -74.40326 America/Bogota
0.51328 -76.50075 America/Bogota
3.42111 -76.24472 America/Bogota
2.43823 -76.61316 America/Bogota
8.41472 -75.58833 America/Bogota
10.46168 -74.61621 America/Bogota
1.84966 -76.04785 America/Bogota
6.98789 -73.04953 America/Bogota
4.81333 -75.69611 America/Bogota
2.06895 -77.05273 America/Bogota
1.21361 -77.28111 America/Bogota
7.37565 -72.64795 America/Bogota
3.53944 -76.30361 America/Bogota
10.74055 -74.75443 America/Bogota
2.65472 -75.99278 America/Bogota
5.13278 -74.15977 America/Bogota
8.23773 -73.35604 America/Bogota
2.9273 -75.28188 America/Bogota
4.70592 -74.23021 America/Bogota
8.2752 -73.86884 America/Bogota
8.74798 -75.88143 America/Bogota
4.56639 -75.75111 America/Bogota
7.98288 -75.42293 America/Bogota
9.24194 -74.42667 America/Bogota
1.14933 -76.64661 America/Bogota
4.20475 -74.64075 America/Bogota
6.25184 -75.56359 America/Bogota
5.19889 -74.89295 America/Bogota
6.17358 -75.33621 America/Bogota
9.9832 -75.30155 America/Bogota
5.32472 -75.15694 America/Bogota
5.06889 -75.51738 America/Bogota
10.85953 -74.77386 America/Bogota
6.69903 -72.73233 America/Bogota
11.38321 -72.24321 America/Bogota
9.24202 -74.75467 America/Bogota
4.73244 -74.26419 America/Bogota
7.83793 -72.5037 America/Bogota
9.23648 -75.8135 America/Bogota
4.9218 -75.06232 America/Bogota
-4.21528 -69.94056 America/Bogota
4.86242 -74.90977 America/Bogota
4.89972 -75.8825 America/Bogota
4.53282 -76.10318 America/Bogota
1.6045 -77.13152 America/Bogota
4.45265 -75.78746 America/Bogota
2.39341 -75.89232 America/Bogota
5.26667 -73.91667 America/Bogota
9.56228 -73.33405 America/Bogota
6.15769 -75.64317 America/Bogota
5.44783 -74.66311 America/Bogota
6.03082 -75.4315 America/Bogota
3.26074 -76.53499 America/Bogota
6.17194 -75.61139 America/Bogota
0.83018 -77.64959 America/Bogota
4.43889 -75.23222 America/Bogota
5.20856 -74.73584 America/Bogota
3.76383 -76.33292 America/Bogota
3.53861 -73.70056 America/Bogota
7.0682 -73.16981 America/Bogota
4.29866 -74.80468 America/Bogota
2.19593 -75.62777 America/Bogota
10.89686 -74.886 America/Bogota
4.33646 -74.36378 America/Bogota
4.71638 -74.21195 America/Bogota
10.52066 -74.18504 America/Bogota
5.15264 -75.03624 America/Bogota
10.88606 -72.8487 America/Bogota
7.06222 -73.08644 America/Bogota
3.3223 -76.2348 America/Bogota
1.61438 -75.60623 America/Bogota
4.29413 -74.81901 America/Bogota
4.81367 -74.35453 America/Bogota
4.14924 -74.88429 America/Bogota
6.17064 -75.58531 America/Bogota
10.61135 -74.26824 America/Bogota
10.15031 -73.9614 America/Bogota
2.47691 -78.11049 America/Bogota
3.68549 -76.31372 America/Bogota
9.7174 -75.12023 America/Bogota
9.00114 -73.97581 America/Bogota
7.59272 -74.80858 America/Bogota
5.8245 -73.03408 America/Bogota
4.83916 -75.66726 America/Bogota
9.19992 -73.54274 America/Bogota
7.88333 -72.50528 America/Bogota
9.31505 -75.29283 America/Bogota
3.17407 -76.25993 America/Bogota
6.34633 -75.50888 America/Bogota
4.61889 -75.63583 America/Bogota
8.87443 -75.62028 America/Bogota
11.00703 -74.24765 America/Bogota
9.36238 -73.60314 America/Bogota
5.61637 -73.81748 America/Bogota
9.10569 -75.39812 America/Bogota
4.9825 -75.60361 America/Bogota
9.25778 -73.81228 America/Bogota
7.66638 -76.68106 America/Bogota
4.85876 -74.05866 America/Bogota
3.72315 -75.48316 America/Bogota
8.88479 -75.79052 America/Bogota
7.98654 -75.19349 America/Bogota
4.74639 -75.91167 America/Bogota
10.39972 -75.51444 America/Bogota
6.08236 -75.33509 America/Bogota
3.40671 -76.34819 America/Bogota
10.37808 -74.88356 America/Bogota
2.68489 -75.32311 America/Bogota
3.43722 -76.5225 America/Bogota
6.09106 -75.63569 America/Bogota
4.52949 -75.64091 America/Bogota
4.91857 -74.02799 America/Bogota
4.3324 -75.82665 America/Bogota
3.90089 -76.29783 America/Bogota
3.8801 -77.03116 America/Bogota
3.58333 -77 America/Bogota
7.12539 -73.1198 America/Bogota
4.60971 -74.08175 America/Bogota
6.33732 -75.55795 America/Bogota
5.20088 -75.86865 America/Bogota
10.96389 -74.79639 America/Bogota
10.95756 -72.78769 America/Bogota
7.06528 -73.85472 America/Bogota
6.43809 -75.33136 America/Bogota
5.93168 -73.61506 America/Bogota
10.79408 -74.9164 America/Bogota
8.31372 -75.13982 America/Bogota
4.53389 -75.68111 America/Bogota
10.25444 -75.34389 America/Bogota
10.25 -74 America/Bogota
7.08471 -70.75908 America/Bogota
10.59181 -74.18983 America/Bogota
7.88299 -76.62587 America/Bogota
5.33278 -75.79111 America/Bogota
5.6561 -75.87877 America/Bogota
4.17061 -76.16641 America/Bogota
5.17282 -72.54706 America/Bogota
5.61161 -75.45624 America/Bogota
8.30844 -73.6166 America/Bogota
3.98695 -73.75797 America/Bogota
2.75701 -76.61723 America/Bogota
7.75849 -76.65254 America/Bogota
5.85389 -76.02528 America/Bogota
10.03409 -73.23611 America/Bogota
9.79029 -74.78244 America/Bogota
2.57286 -72.64591 America/Bogota
9.90467 -83.68352 America/Costa_Rica
9.74622 -84.23368 America/Costa_Rica
10.09748 -83.50659 America/Costa_Rica
9.96016 -84.04762 America/Costa_Rica
9.87556 -84.07661 America/Costa_Rica
9.831 -84.29008 America/Costa_Rica
9.97454 -84.20974 America/Costa_Rica
9.92787 -84.13722 America/Costa_Rica
9.92829 -84.05074 America/Costa_Rica
9.99559 -84.09656 America/Costa_Rica
9.87121 -84.06084 America/Costa_Rica
9.8773 -84.0847 America/Costa_Rica
9.95974 -84.08165 America/Costa_Rica
10.95173 -85.1361 America/Costa_Rica
9.93333 -84.08333 America/Costa_Rica
9.3721 -83.70399 America/Costa_Rica
9.99299 -84.12934 America/Costa_Rica
9.90488 -84.10551 America/Costa_Rica
9.89898 -84.00287 America/Costa_Rica
10.32381 -84.42714 America/Costa_Rica
9.95808 -84.0305 America/Costa_Rica
9.97625 -84.83836 America/Costa_Rica
10 -83.03333 America/Costa_Rica
9.88071 -84.03501 America/Costa_Rica
9.83832 -83.86556 America/Costa_Rica
10.14828 -85.45201 America/Costa_Rica
10.00696 -84.13396 America/Costa_Rica
10.63504 -85.43772 America/Costa_Rica
9.96746 -84.01326 America/Costa_Rica
10 -84.11667 America/Costa_Rica
10.21588 -83.78661 America/Costa_Rica
9.94611 -84.05279 America/Costa_Rica
9.99447 -84.66466 America/Costa_Rica
9.91178 -84.03407 America/Costa_Rica
9.95091 -84.08503 America/Costa_Rica
9.98289 -84.77232 America/Costa_Rica
9.86667 -83.91667 America/Costa_Rica
10.4294 -85.09311 America/Costa_Rica
9.95 -84.06667 America/Costa_Rica
9.85876 -84.0923 America/Costa_Rica
10.01667 -84.21667 America/Costa_Rica
9.96164 -84.0488 America/Costa_Rica
20.27528 -76.95333 America/Havana
22.32722 -79.23778 America/Havana
22.61639 -83.70778 America/Havana
21.25944 -78.14639 America/Havana
21.73528 -78.79639 America/Havana
23.15361 -81.25139 America/Havana
22.79528 -81.5375 America/Havana
21.80194 -79.98417 America/Havana
21.235 -77.52639 America/Havana
22.58667 -80.24361 America/Havana
22.97 -82.38694 America/Havana
20.02472 -75.82194 America/Havana
20.71472 -77.99556 America/Havana
23.15389 -81.92361 America/Havana
22.4 -79.96667 America/Havana
20.1875 -75.85083 America/Havana
22.96139 -82.15111 America/Havana
21.92972 -79.4425 America/Havana
22.71417 -83.04806 America/Havana
22.89111 -82.49917 America/Havana
22.80667 -80.07556 America/Havana
20.58194 -75.24139 America/Havana
22.34083 -80.55722 America/Havana
20.36667 -75.01667 America/Havana
20.56028 -76.91611 America/Havana
22.49472 -79.54583 America/Havana
22.37722 -80.14889 America/Havana
21.195 -76.60278 America/Havana
21.94694 -78.42833 America/Havana
22.31194 -79.65333 America/Havana
22.4175 -83.69806 America/Havana
22.76917 -81.01889 America/Havana
22.72722 -81.29 America/Havana
22.24167 -80.39028 America/Havana
20.21722 -75.99889 America/Havana
21.54528 -77.26444 America/Havana
21.88333 -82.8 America/Havana
20.04306 -77.58278 America/Havana
22.10944 -78.6275 America/Havana
20.65694 -74.94028 America/Havana
21.48333 -77.61667 America/Havana
20.14194 -77.435 America/Havana
23.04111 -81.5775 America/Havana
22.9875 -82.75361 America/Havana
20.34333 -77.11667 America/Havana
22.14944 -79.97361 America/Havana
22.91139 -81.85611 America/Havana
22.58722 -83.24861 America/Havana
20.96167 -76.95111 America/Havana
22.87139 -82.42389 America/Havana
23.13302 -82.38304 America/Havana
22.80444 -81.19444 America/Havana
20.96917 -76.29944 America/Havana
20.36667 -76.42722 America/Havana
21.94194 -79.17 America/Havana
23.04306 -82.01083 America/Havana
22.52694 -81.12861 America/Havana
20.88722 -76.26306 America/Havana
20.255 -76.53861 America/Havana
22.79056 -82.50528 America/Havana
22.83611 -82.02806 America/Havana
20.14444 -75.20917 America/Havana
22.2025 -84.0875 America/Havana
22.92639 -82.6875 America/Havana
21.05 -77.35 America/Havana
21.10972 -76.13167 America/Havana
22.10389 -79.72167 America/Havana
21.52472 -78.22583 America/Havana
21.8525 -78.11694 America/Havana
22.61694 -79.87111 America/Havana
22.14944 -80.20222 America/Havana
20.64389 -75.93556 America/Havana
22.34111 -80.26667 America/Havana
22.98194 -80.58556 America/Havana
20.29778 -76.24139 America/Havana
22.50472 -83.51361 America/Havana
21.87667 -79.84028 America/Havana
22.71917 -80.90583 America/Havana
20.98806 -77.42972 America/Havana
22.645 -80.04611 America/Havana
22.14611 -80.43556 America/Havana
21.84 -78.76194 America/Havana
22.19278 -78.91361 America/Havana
23.0375 -81.20472 America/Havana
20.23417 -77.27889 America/Havana
22.48333 -79.75 America/Havana
21.38083 -77.91694 America/Havana
22.53333 -80.90611 America/Havana
22.52056 -79.46694 America/Havana
22.07889 -79.49917 America/Havana
22.92861 -82.38861 America/Havana
20.37917 -76.64333 America/Havana
22.98278 -82.54639 America/Havana
21.69667 -78.63278 America/Havana
20.34667 -74.49583 America/Havana
20.9625 -75.71861 America/Havana
22.90361 -83.15917 America/Havana
22.81306 -82.76194 America/Havana
22.80583 -82.58417 America/Havana
23.15833 -82.27694 America/Havana
22.385 -80.85083 America/Havana
22.2775 -80.5675 America/Havana
20.75167 -76.02694 America/Havana
16.6 -22.9 Atlantic/Cape_Verde
14.92148 -23.50868 Atlantic/Cape_Verde
16.89014 -24.98042 Atlantic/Cape_Verde
12.1084 -68.93354 America/Curacao
-10.42172 105.67912 Indian/Christmas
35.0125 34.05833 Asia/Nicosia
34.76667 32.41667 Asia/Nicosia
35.16667 33.36667 Asia/Nicosia
34.675 33.03333 Asia/Nicosia
34.91667 33.62917 Asia/Nicosia
35.34167 33.31667 Asia/Nicosia
35.125 33.95 Asia/Nicosia
50.43172 15.81402 Europe/Prague
48.8555 16.0488 Europe/Prague
49.22665 17.66633 Europe/Prague
49.21667 17.66667 Europe/Prague
49.58726 15.93216 Europe/Prague
49.56263 15.93924 Europe/Prague
50.32717 13.54577 Europe/Prague
49.27747 16.99897 Europe/Prague
49.33871 17.99619 Europe/Prague
50.91154 14.61824 Europe/Prague
49.47181 17.97113 Europe/Prague
49.97387 16.39361 Europe/Prague
50.6607 14.03227 Europe/Prague
49.02513 17.64715 Europe/Prague
49.06975 17.45969 Europe/Prague
50.56101 15.9127 Europe/Prague
49.67763 18.67078 Europe/Prague
49.21492 15.88166 Europe/Prague
50.6404 13.82451 Europe/Prague
49.41441 14.6578 Europe/Prague
49.75594 16.46829 Europe/Prague
49.96528 16.97061 Europe/Prague
49.26141 13.90237 Europe/Prague
49.9169 18.33619 Europe/Prague
50.1813 12.6401 Europe/Prague
50.23046 14.08693 Europe/Prague
49.45853 18.14302 Europe/Prague
50.1037 13.7334 Europe/Prague
49.47188 17.11184 Europe/Prague
50.11667 14.5 Europe/Prague
49.68988 14.01043 Europe/Prague
49.45511 17.4509 Europe/Prague
50.08804 14.42076 Europe/Prague
49.74747 13.37759 Europe/Prague
49.3088 14.1475 Europe/Prague
49.43134 15.22336 Europe/Prague
50.04075 15.77659 Europe/Prague
49.21667 17.51667 Europe/Prague
49.20934 17.53944 Europe/Prague
50.30592 12.93907 Europe/Prague
49.83465 18.28204 Europe/Prague
49.84527 18.43011 Europe/Prague
49.93866 17.90257 Europe/Prague
49.59552 17.25175 Europe/Prague
49.59438 18.01028 Europe/Prague
50.25926 14.51759 Europe/Prague
50.4167 16.16289 Europe/Prague
50.50301 13.63617 Europe/Prague
50.01116 14.4096 Europe/Prague
50.41135 14.90318 Europe/Prague
50.3505 14.47411 Europe/Prague
50.35699 13.79667 Europe/Prague
50.6042 13.61812 Europe/Prague
50.53348 14.1318 Europe/Prague
50.76711 15.05619 Europe/Prague
50.10819 14.47457 Europe/Prague
50.13333 14.51667 Europe/Prague
49.94839 15.26816 Europe/Prague
49.29785 17.39312 Europe/Prague
50.08967 17.70385 Europe/Prague
50.24107 14.31149 Europe/Prague
49.59947 18.1448 Europe/Prague
50.02814 15.20057 Europe/Prague
49.39552 13.29505 Europe/Prague
50.38445 13.17128 Europe/Prague
50.14734 14.10285 Europe/Prague
49.854 18.54169 Europe/Prague
50.23271 12.87117 Europe/Prague
50.38333 13.26667 Europe/Prague
50.37598 13.27131 Europe/Prague
50.49979 13.44769 Europe/Prague
49.14404 15.00301 Europe/Prague
49.3961 15.59124 Europe/Prague
50.43723 15.35162 Europe/Prague
50.72431 15.17108 Europe/Prague
49.54796 17.73469 Europe/Prague
50.20923 15.83277 Europe/Prague
48.84893 17.13244 Europe/Prague
49.60785 15.58069 Europe/Prague
49.77984 18.43688 Europe/Prague
49.68333 18.35 Europe/Prague
50.78215 14.21478 Europe/Prague
49.95109 15.79558 Europe/Prague
50.46048 13.41779 Europe/Prague
50.07963 12.37392 Europe/Prague
49.74613 18.62613 Europe/Prague
48.97447 14.47434 Europe/Prague
49.90436 16.44413 Europe/Prague
50.68551 14.53764 Europe/Prague
49.98844 17.4647 Europe/Prague
49.19522 16.60796 Europe/Prague
48.75897 16.88203 Europe/Prague
50.03597 14.41235 Europe/Prague
50.18709 14.66326 Europe/Prague
49.90411 18.35755 Europe/Prague
49.36304 16.64446 Europe/Prague
50.55 13.76667 Europe/Prague
50.54854 13.77535 Europe/Prague
49.96382 14.072 Europe/Prague
49.78162 14.68697 Europe/Prague
50.10475 14.57974 Europe/Prague
50.73333 12.5 Europe/Berlin
49.25 7.36667 Europe/Berlin
50.7 6.65 Europe/Berlin
52.21667 13.45 Europe/Berlin
50.9 14.83333 Europe/Berlin
49.4424 10.95414 Europe/Berlin
51.96667 12.08333 Europe/Berlin
51.05 12.15 Europe/Berlin
51.66667 6.45 Europe/Berlin
51.36667 12.73333 Europe/Berlin
49.78778 9.93611 Europe/Berlin
50.81667 6.13333 Europe/Berlin
51.26667 7.18333 Europe/Berlin
52.43333 9.41667 Europe/Berlin
51.28333 7.05 Europe/Berlin
49.05806 8.24861 Europe/Berlin
49.63556 8.35972 Europe/Berlin
52.43333 10.8 Europe/Berlin
47.91289 11.42166 Europe/Berlin
52.16667 10.55 Europe/Berlin
51.66667 12.28333 Europe/Berlin
51.33333 9.86667 Europe/Berlin
53.16118 12.48287 Europe/Berlin
53.56667 7.78333 Europe/Berlin
49.98333 6.88333 Europe/Berlin
53 11.75 Europe/Berlin
51.43333 7.33333 Europe/Berlin
53.9 11.46667 Europe/Berlin
51.11667 7.4 Europe/Berlin
53.6 10 Europe/Berlin
53.36667 10.21667 Europe/Berlin
48.87563 9.39819 Europe/Berlin
50.81667 8.1 Europe/Berlin
51.26667 6.55 Europe/Berlin
53.51667 8.13333 Europe/Berlin
52.9 8.43333 Europe/Berlin
49.29778 8.70139 Europe/Berlin
50.08333 8.25 Europe/Berlin
50.95 7.53333 Europe/Berlin
50.55 8.5 Europe/Berlin
51.38333 7.4 Europe/Berlin
53.25 7.93333 Europe/Berlin
50.83333 6.98333 Europe/Berlin
51.66667 6.61667 Europe/Berlin
49.7575 9.50944 Europe/Berlin
51.83333 10.78333 Europe/Berlin
51.66446 7.63421 Europe/Berlin
51.15 7.21667 Europe/Berlin
51.55 7.91667 Europe/Berlin
51.26667 7.76667 Europe/Berlin
52.37874 12.934 Europe/Berlin
50.73333 12.38333 Europe/Berlin
48.67124 9.37632 Europe/Berlin
50.96667 7.86667 Europe/Berlin
49.35361 11.14833 Europe/Berlin
49.90778 8.59528 Europe/Berlin
51.51667 14.63333 Europe/Berlin
51.2 11.96667 Europe/Berlin
49.03095 10.97221 Europe/Berlin
48.81311 9.36387 Europe/Berlin
49.54833 8.66611 Europe/Berlin
47.8 9.63333 Europe/Berlin
50.98333 11.31667 Europe/Berlin
47.83333 11.15 Europe/Berlin
50.76667 6.83333 Europe/Berlin
48.75 8.86667 Europe/Berlin
47.59331 7.62082 Europe/Berlin
49.6775 12.15667 Europe/Berlin
51.15 6.26667 Europe/Berlin
53.16667 7.35 Europe/Berlin
53.58374 9.69835 Europe/Berlin
51.1 6.15 Europe/Berlin
51.45 8.36667 Europe/Berlin
51.95 7.98333 Europe/Berlin
53.51667 12.68333 Europe/Berlin
53.06667 8.2 Europe/Berlin
51.5 9.16667 Europe/Berlin
47.68333 9.83333 Europe/Berlin
52.74196 13.45799 Europe/Berlin
51.63333 7.4 Europe/Berlin
52.86667 9.6 Europe/Berlin
52.35 8.01667 Europe/Berlin
47.62323 8.21717 Europe/Berlin
48.20854 12.39893 Europe/Berlin
48.09585 7.96371 Europe/Berlin
50.88333 7.61667 Europe/Berlin
48.83241 9.31641 Europe/Berlin
49.25639 8.50722 Europe/Berlin
49.26667 6.78333 Europe/Berlin
49.53333 6.88333 Europe/Berlin
50.63333 7.1 Europe/Berlin
52.03792 6.828 Europe/Berlin
49.25162 6.85873 Europe/Berlin
51.6 6.68333 Europe/Berlin
52.1653 8.85996 Europe/Berlin
48.63333 13.18333 Europe/Berlin
48.06667 8.45 Europe/Berlin
51.25 6.38333 Europe/Berlin
49.54167 8.57861 Europe/Berlin
52.05 8.15 Europe/Berlin
51.88333 8.51667 Europe/Berlin
52.91667 9.23333 Europe/Berlin
51.33333 7.05 Europe/Berlin
52.71667 8.28333 Europe/Berlin
52.26038 10.36491 Europe/Berlin
48.10537 11.76825 Europe/Berlin
53.39693 8.13621 Europe/Berlin
48.93333 8.96667 Europe/Berlin
53.68333 9.66667 Europe/Berlin
51.6569 9.63501 Europe/Berlin
48.28333 11.56667 Europe/Berlin
48.06047 11.61873 Europe/Berlin
47.91667 7.7 Europe/Berlin
51.53333 7.68333 Europe/Berlin
48.39841 9.99155 Europe/Berlin
52.96667 10.56667 Europe/Berlin
47.76667 9.16667 Europe/Berlin
50.93333 6.11667 Europe/Berlin
47.98333 8.81667 Europe/Berlin
48.52266 9.05222 Europe/Berlin
48.07667 8.64409 Europe/Berlin
50.81667 7.16667 Europe/Berlin
49.75565 6.63935 Europe/Berlin
47.88333 12.65 Europe/Berlin
47.96269 12.59231 Europe/Berlin
51.56667 13 Europe/Berlin
51.31667 6.45 Europe/Berlin
47.66667 9.6 Europe/Berlin
53.11865 13.5022 Europe/Berlin
52.4 13.26667 Europe/Berlin
51.98333 7.78333 Europe/Berlin
50.15 8.15 Europe/Berlin
48.0432 11.61907 Europe/Berlin
52.9 8.83333 Europe/Berlin
51.33333 8 Europe/Berlin
49.49833 11.75167 Europe/Berlin
49.3 7.06667 Europe/Berlin
50.6 10.7 Europe/Berlin
48.78232 9.17702 Europe/Berlin
53.03333 8.75 Europe/Berlin
52.57859 13.88741 Europe/Berlin
48.88333 12.56667 Europe/Berlin
54.30911 13.0818 Europe/Berlin
51.45 6.26667 Europe/Berlin
50.76667 6.23333 Europe/Berlin
53.9 10.65 Europe/Berlin
47.85 9.01667 Europe/Berlin
52.6 11.85 Europe/Berlin
53.5922 9.9287 Europe/Berlin
52 8.4 Europe/Berlin
52.15045 7.33664 Europe/Berlin
53.61028 10.05917 Europe/Berlin
51.86667 11.58333 Europe/Berlin
48 11.35 Europe/Berlin
51.98333 6.93333 Europe/Berlin
52.32333 9.20311 Europe/Berlin
50.83333 9.01667 Europe/Berlin
53.6 9.48333 Europe/Berlin
51.35 7.25 Europe/Berlin
52.20845 9.55416 Europe/Berlin
49.32972 8.42778 Europe/Berlin
52.13333 8.48333 Europe/Berlin
47.51667 10.28333 Europe/Berlin
50.35 11.16667 Europe/Berlin
51.36667 10.86667 Europe/Berlin
51.15 11.1 Europe/Berlin
52.98333 9.83333 Europe/Berlin
51.18333 7.08333 Europe/Berlin
51.57558 8.10619 Europe/Berlin
50.55 7.25 Europe/Berlin
49.2529 8.87867 Europe/Berlin
47.75935 8.8403 Europe/Berlin
48.7 9.01667 Europe/Berlin
50.6 6.3 Europe/Berlin
48.08333 9.21667 Europe/Berlin
50.86667 8.03333 Europe/Berlin
50.8 7.2 Europe/Berlin
51.51667 14.01667 Europe/Berlin
51.85 7.5 Europe/Berlin
48.32441 10.04442 Europe/Berlin
51.7 7.46667 Europe/Berlin
50.0432 8.97394 Europe/Berlin
50.17058 12.13054 Europe/Berlin
52.31394 9.9682 Europe/Berlin
53.4 9.96667 Europe/Berlin
51.9 10.18333 Europe/Berlin
52.4 9.6 Europe/Berlin
49.37806 8.58194 Europe/Berlin
51.45 7.56667 Europe/Berlin
53.62937 11.41316 Europe/Berlin
51.28333 7.28333 Europe/Berlin
50.05 10.23333 Europe/Berlin
53.05963 14.28154 Europe/Berlin
50.55 12.78333 Europe/Berlin
53.23333 8.6 Europe/Berlin
49.32639 12.10972 Europe/Berlin
51.21667 6.26667 Europe/Berlin
50.93333 9.21667 Europe/Berlin
49.3 6.81667 Europe/Berlin
49.11127 9.73908 Europe/Berlin
48.79947 9.79809 Europe/Berlin
49.32917 11.02556 Europe/Berlin
48.55 11.26667 Europe/Berlin
48.22399 8.38583 Europe/Berlin
53.53333 7.95 Europe/Berlin
48.80537 9.52721 Europe/Berlin
47.65105 7.82089 Europe/Berlin
52.01667 11.75 Europe/Berlin
53.11667 9.8 Europe/Berlin
50.6 12.63333 Europe/Berlin
49.43333 6.85 Europe/Berlin
51.15 8.3 Europe/Berlin
50.71667 10.45 Europe/Berlin
50.34891 9.52532 Europe/Berlin
54.51667 9.55 Europe/Berlin
51.4 12.21667 Europe/Berlin
49.36667 7.13333 Europe/Berlin
49.38722 8.37472 Europe/Berlin
53.60278 9.82333 Europe/Berlin
48.01667 9.5 Europe/Berlin
53.65385 10.11184 Europe/Berlin
52.23333 9.85 Europe/Berlin
49.46667 7.16667 Europe/Berlin
49.28333 7.11667 Europe/Berlin
50.76667 7.18333 Europe/Berlin
51.46667 11.3 Europe/Berlin
52.85 11.15 Europe/Berlin
51.6717 8.60092 Europe/Berlin
49.31667 6.75 Europe/Berlin
49.23333 7 Europe/Berlin
50.65 11.36667 Europe/Berlin
49.9925 8.435 Europe/Berlin
50.71667 11.33333 Europe/Berlin
48.16783 8.62719 Europe/Berlin
48.47629 8.93528 Europe/Berlin
49.245 11.09111 Europe/Berlin
53.11667 9.4 Europe/Berlin
54.0887 12.14049 Europe/Berlin
50.89559 7.18175 Europe/Berlin
47.85637 12.12247 Europe/Berlin
52.31939 9.65544 Europe/Berlin
50.88333 6.98333 Europe/Berlin
52.18604 9.07917 Europe/Berlin
51.8 8.43333 Europe/Berlin
51.3 13.3 Europe/Berlin
49.3 6.93333 Europe/Berlin
54.25 12.46667 Europe/Berlin
47.56667 7.8 Europe/Berlin
52.28333 7.45 Europe/Berlin
51.55 6.6 Europe/Berlin
50.62562 6.94911 Europe/Berlin
51.83333 6.7 Europe/Berlin
51.84967 8.30017 Europe/Berlin
48.49144 9.20427 Europe/Berlin
48.76667 8.93333 Europe/Berlin
54.3 9.66667 Europe/Berlin
51.18333 7.2 Europe/Berlin
50.57879 7.22703 Europe/Berlin
52.56667 13.33333 Europe/Berlin
49.82694 8.83611 Europe/Berlin
53.51667 10.25 Europe/Berlin
50.61667 12.3 Europe/Berlin
49.015 12.09556 Europe/Berlin
51.76667 6.4 Europe/Berlin
51.61667 7.2 Europe/Berlin
47.78333 9.61667 Europe/Berlin
51.29724 6.84929 Europe/Berlin
52.60659 12.33696 Europe/Berlin
53.95 10.73333 Europe/Berlin
53.25 8.2 Europe/Berlin
48.85 8.2 Europe/Berlin
52.43333 8.61667 Europe/Berlin
47.73333 8.96667 Europe/Berlin
51.2 7.35 Europe/Berlin
51.10654 13.66047 Europe/Berlin
51.11667 13.91667 Europe/Berlin
53.73333 9.9 Europe/Berlin
51.78333 11.15 Europe/Berlin
49.2855 6.88723 Europe/Berlin
51 6.8 Europe/Berlin
48.15 11.35 Europe/Berlin
53.31667 13.86667 Europe/Berlin
54.23333 10.28333 Europe/Berlin
52.39886 13.06566 Europe/Berlin
52.23779 8.91901 Europe/Berlin
53.65917 10.08472 Europe/Berlin
51.21667 7.88333 Europe/Berlin
50.5 12.13333 Europe/Berlin
50.95843 13.93702 Europe/Berlin
49.20145 7.60529 Europe/Berlin
53.65 9.8 Europe/Berlin
49.80639 8.60694 Europe/Berlin
48.46458 9.22796 Europe/Berlin
48.88333 8.7 Europe/Berlin
48.53333 11.51667 Europe/Berlin
52.38333 8.96667 Europe/Berlin
47.75 11.38333 Europe/Berlin
52.31667 10.23333 Europe/Berlin
48.57116 13.44898 Europe/Berlin
53.42631 11.84875 Europe/Berlin
53.06667 7.4 Europe/Berlin
52.56926 13.40186 Europe/Berlin
51.71905 8.75439 Europe/Berlin
53.05 9.01667 Europe/Berlin
50.93275 7.28389 Europe/Berlin
49.40133 7.16424 Europe/Berlin
48.06489 11.66327 Europe/Berlin
53.23333 8.8 Europe/Berlin
52.26667 8.05 Europe/Berlin
52.03333 11.25 Europe/Berlin
51.3 13.11667 Europe/Berlin
52.75 13.23333 Europe/Berlin
51.06667 7.01667 Europe/Berlin
51.35 8.5 Europe/Berlin
51.03333 7.85 Europe/Berlin
53.16667 8.2 Europe/Berlin
48.2 11.33333 Europe/Berlin
49.20167 9.50389 Europe/Berlin
48.48333 7.93333 Europe/Berlin
50.1 8.76667 Europe/Berlin
51.95453 8.6622 Europe/Berlin
51.63333 7.25 Europe/Berlin
51.81667 8.15 Europe/Berlin
51.03333 7.11667 Europe/Berlin
52.21667 7.18333 Europe/Berlin
50.2 8.58333 Europe/Berlin
50.08333 8.85 Europe/Berlin
49.83361 8.75194 Europe/Berlin
48.53241 8.07864 Europe/Berlin
51.46667 6.85 Europe/Berlin
49.42275 10.95766 Europe/Berlin
48.62565 9.34203 Europe/Berlin
49.44778 11.06833 Europe/Berlin
50.9 7.55 Europe/Berlin
51.93333 7.35 Europe/Berlin
51.7 10 Europe/Berlin
48.85122 10.48868 Europe/Berlin
52.43333 7.08333 Europe/Berlin
51.51667 10.8 Europe/Berlin
53.7 10.01667 Europe/Berlin
53.5 8.48333 Europe/Berlin
53.6 7.2 Europe/Berlin
52.63333 9.21667 Europe/Berlin
51.2 6.21667 Europe/Berlin
50.81667 7.03333 Europe/Berlin
49.97756 8.07246 Europe/Berlin
50.25 8.9 Europe/Berlin
50.41667 9 Europe/Berlin
53.46667 9.8 Europe/Berlin
50.43333 7.46667 Europe/Berlin
48.39279 10.01112 Europe/Berlin
53.36667 13.08333 Europe/Berlin
54.10707 10.8145 Europe/Berlin
50.31667 11.11667 Europe/Berlin
49.35667 8.13778 Europe/Berlin
52.50462 9.45871 Europe/Berlin
52.15 11.63333 Europe/Berlin
51.2 6.68333 Europe/Berlin
52.92815 12.80311 Europe/Berlin
49.35 7.18333 Europe/Berlin
54.06667 9.98333 Europe/Berlin
49.28028 11.46278 Europe/Berlin
50.05 8.7 Europe/Berlin
48.31667 11.66667 Europe/Berlin
52.52985 13.68914 Europe/Berlin
48.73333 11.18333 Europe/Berlin
53.56667 13.26667 Europe/Berlin
50.31667 8.5 Europe/Berlin
51.31667 6.28333 Europe/Berlin
50.91667 8.1 Europe/Berlin
49.19278 9.22611 Europe/Berlin
51.15 11.81667 Europe/Berlin
52.60701 12.87374 Europe/Berlin
48.5498 8.72366 Europe/Berlin
52.98333 10.08333 Europe/Berlin
51.96236 7.62571 Europe/Berlin
51.41505 9.65046 Europe/Berlin
48.13743 11.57549 Europe/Berlin
47.8082 7.63035 Europe/Berlin
51.43333 6.88333 Europe/Berlin
50.11667 8.83333 Europe/Berlin
48.84232 9.23028 Europe/Berlin
51.21667 10.45 Europe/Berlin
48.25 12.53333 Europe/Berlin
48.95 8.83333 Europe/Berlin
50.91667 7.4 Europe/Berlin
48.40567 9.05419 Europe/Berlin
49.35389 9.14278 Europe/Berlin
48.46667 11.93333 Europe/Berlin
51.08333 6.88333 Europe/Berlin
51.2 6.43333 Europe/Berlin
53.63333 10.68333 Europe/Berlin
51.45 6.65 Europe/Berlin
50.98333 12.98333 Europe/Berlin
52.28333 8.91667 Europe/Berlin
49.67361 9.0025 Europe/Berlin
48.53695 9.2833 Europe/Berlin
51.25 6.96667 Europe/Berlin
51.35 8.28333 Europe/Berlin
49.45 6.63333 Europe/Berlin
51.36667 12 Europe/Berlin
52.69064 7.29097 Europe/Berlin
51.43333 7.8 Europe/Berlin
47.98372 10.18527 Europe/Berlin
52.2 8.35 Europe/Berlin
51.16158 13.4737 Europe/Berlin
50.56787 10.41521 Europe/Berlin
51.11667 7.65 Europe/Berlin
51.28333 6.66667 Europe/Berlin
50.85 12.46667 Europe/Berlin
50.63333 7.01667 Europe/Berlin
50.6 6.65 Europe/Berlin
50.33333 7.21667 Europe/Berlin
51.46171 8.84949 Europe/Berlin
51.65 7.08333 Europe/Berlin
50.005 12.08111 Europe/Berlin
47.78333 10.61667 Europe/Berlin
51.28333 12.36667 Europe/Berlin
50.81667 8.76667 Europe/Berlin
48.93964 9.25995 Europe/Berlin
49.48833 8.46472 Europe/Berlin
50 8.27111 Europe/Berlin
50.15 8.83333 Europe/Berlin
52.16667 11.66667 Europe/Berlin
51.61667 7.51667 Europe/Berlin
53.25 10.4 Europe/Berlin
49.48111 8.43528 Europe/Berlin
52.3 13.26667 Europe/Berlin
48.89731 9.19161 Europe/Berlin
51.76667 7.46667 Europe/Berlin
51.21667 7.61667 Europe/Berlin
52.08333 13.16667 Europe/Berlin
53.86893 10.68729 Europe/Berlin
51.86667 13.96667 Europe/Berlin
52.3 8.61667 Europe/Berlin
53.46667 8.65 Europe/Berlin
49.51667 6.75 Europe/Berlin
47.61497 7.66457 Europe/Berlin
49.99361 9.57833 Europe/Berlin
52.66667 8.23333 Europe/Berlin
50.83333 7.21667 Europe/Berlin
51.1 14.66667 Europe/Berlin
51.66667 8.35 Europe/Berlin
52.51667 7.31667 Europe/Berlin
51.01959 7.37758 Europe/Berlin
47.55 9.68333 Europe/Berlin
50.38333 8.05 Europe/Berlin
50.86667 12.75 Europe/Berlin
53.13333 8.91667 Europe/Berlin
50.14567 11.05928 Europe/Berlin
51.03333 7 Europe/Berlin
47.825 10.01667 Europe/Berlin
52.01246 8.69834 Europe/Berlin
48.8 9.01667 Europe/Berlin
51.11667 8.06667 Europe/Berlin
52.18333 7.86667 Europe/Berlin
52.03333 8.9 Europe/Berlin
51.33962 12.37129 Europe/Berlin
48.69406 9.16809 Europe/Berlin
49.35111 8.68944 Europe/Berlin
51.11667 7.01667 Europe/Berlin
52.38333 9.96667 Europe/Berlin
53.23157 7.461 Europe/Berlin
49.4 6.91667 Europe/Berlin
48.23333 9.88333 Europe/Berlin
49.51028 11.2775 Europe/Berlin
51.5 13.8 Europe/Berlin
53.66556 10.00137 Europe/Berlin
52.45 9.75 Europe/Berlin
51.1 6.95 Europe/Berlin
53.60554 8.59509 Europe/Berlin
49.99333 8.67278 Europe/Berlin
48.53333 12.15 Europe/Berlin
48.04819 10.88282 Europe/Berlin
49.2075 8.11333 Europe/Berlin
49.60194 8.47194 Europe/Berlin
48.35 7.86667 Europe/Berlin
50.3 7.61667 Europe/Berlin
51.99223 8.79301 Europe/Berlin
52.31667 9.8 Europe/Berlin
50.93333 8.4 Europe/Berlin
51.05 7.26667 Europe/Berlin
49.28194 9.68889 Europe/Berlin
50.5442 9.71792 Europe/Berlin
50.1 11.45 Europe/Berlin
50.17745 8.51252 Europe/Berlin
50.23963 11.33308 Europe/Berlin
50.96667 7.98333 Europe/Berlin
50.75 6.48333 Europe/Berlin
51.33333 6.56667 Europe/Berlin
51.75 11.96667 Europe/Berlin
51.19139 6.51352 Europe/Berlin
48.86158 9.18569 Europe/Berlin
48.83218 9.1214 Europe/Berlin
51.27561 8.873 Europe/Berlin
52.44254 13.58228 Europe/Berlin
49.7 6.58333 Europe/Berlin
47.66033 9.17582 Europe/Berlin
52.30141 13.633 Europe/Berlin
50.68333 7.18333 Europe/Berlin
50.18333 8.46667 Europe/Berlin
52.25 10.81667 Europe/Berlin
48.26667 10.88333 Europe/Berlin
50.93333 6.95 Europe/Berlin
47.85 12.06667 Europe/Berlin
50.35 7.6 Europe/Berlin
51.78333 6.15 Europe/Berlin
52.40786 13.22514 Europe/Berlin
49.73333 10.16667 Europe/Berlin
52.2 8.63333 Europe/Berlin
48.64683 9.45378 Europe/Berlin
50.81667 8.96667 Europe/Berlin
51.13333 7.58333 Europe/Berlin
54.32133 10.13489 Europe/Berlin
51.58333 6.25 Europe/Berlin
50.86667 6.7 Europe/Berlin
47.72674 10.31389 Europe/Berlin
51.36667 6.41667 Europe/Berlin
50.13333 8.45 Europe/Berlin
48.91667 11.86667 Europe/Berlin
48.57297 7.81523 Europe/Berlin
47.88333 10.61667 Europe/Berlin
51.31667 9.5 Europe/Berlin
49.96028 9.77222 Europe/Berlin
49.00472 8.38583 Europe/Berlin
48.21667 11.46667 Europe/Berlin
50.23019 8.77155 Europe/Berlin
51.5 6.53333 Europe/Berlin
51.26798 14.09374 Europe/Berlin
51.6 7.66667 Europe/Berlin
53.83333 9.96667 Europe/Berlin
49.45 7.75 Europe/Berlin
51.23333 6.61667 Europe/Berlin
50.93333 6.36667 Europe/Berlin
51.1 6.5 Europe/Berlin
50.93333 11.58333 Europe/Berlin
53.92099 9.51529 Europe/Berlin
52.46667 9.83333 Europe/Berlin
51.36667 7.7 Europe/Berlin
48.76667 11.43333 Europe/Berlin
50.68333 10.9 Europe/Berlin
49.36667 7.05 Europe/Berlin
48.22336 10.10347 Europe/Berlin
50.23333 8.26667 Europe/Berlin
49.7 7.3 Europe/Berlin
52.26667 7.73333 Europe/Berlin
54.4858 9.05239 Europe/Berlin
50.87079 6.86761 Europe/Berlin
50.67966 9.76727 Europe/Berlin
53.64773 10.04149 Europe/Berlin
53.11667 8.46667 Europe/Berlin
51.15 7.35 Europe/Berlin
51.05 6.21667 Europe/Berlin
51.43333 14.25 Europe/Berlin
51.77501 9.38155 Europe/Berlin
51.81667 8.65 Europe/Berlin
52.31667 7.58333 Europe/Berlin
48.44423 8.6913 Europe/Berlin
49.31667 7.33333 Europe/Berlin
51.5 7.63333 Europe/Berlin
51.82798 9.4455 Europe/Berlin
47.87663 11.70181 Europe/Berlin
50.8 12.71667 Europe/Berlin
52.66667 13.28333 Europe/Berlin
50.08333 8.45 Europe/Berlin
51.48333 9.4 Europe/Berlin
50.31667 11.91667 Europe/Berlin
49.32278 8.54583 Europe/Berlin
50.01667 8.35 Europe/Berlin
52.33333 8.75 Europe/Berlin
52.15077 9.95112 Europe/Berlin
51.16667 6.93333 Europe/Berlin
50.98333 8.1 Europe/Berlin
52.16667 8.61667 Europe/Berlin
49.35 6.93333 Europe/Berlin
50.06667 8.8 Europe/Berlin
51.65 11.5 Europe/Berlin
52.16667 9.25 Europe/Berlin
50.87067 6.10136 Europe/Berlin
49.56798 10.88565 Europe/Berlin
51.6 7.13333 Europe/Berlin
48.59523 8.86648 Europe/Berlin
51.55 7.21667 Europe/Berlin
52.13333 8.68333 Europe/Berlin
51.4 7.43333 Europe/Berlin
50.68333 8.31667 Europe/Berlin
49.64083 8.63722 Europe/Berlin
52.63598 13.20419 Europe/Berlin
50.77555 7.28308 Europe/Berlin
52.31425 9.72359 Europe/Berlin
51.38333 7.76667 Europe/Berlin
52.2279 11.00985 Europe/Berlin
51.06667 6.08333 Europe/Berlin
51.38333 10.13333 Europe/Berlin
51.32662 6.97106 Europe/Berlin
49.14028 9.22 Europe/Berlin
48.67798 10.15162 Europe/Berlin
50.97221 13.86741 Europe/Berlin
49.40768 8.69079 Europe/Berlin
54.2 9.1 Europe/Berlin
48.35149 8.96317 Europe/Berlin
51.4 7.16667 Europe/Berlin
50.06667 8.46667 Europe/Berlin
49.36278 8.25806 Europe/Berlin
51.96667 8.23333 Europe/Berlin
52.78333 7.23333 Europe/Berlin
53.46667 9.98333 Europe/Berlin
52.37052 9.73322 Europe/Berlin
50.13333 8.91667 Europe/Berlin
51.73333 6.58333 Europe/Berlin
51.68033 7.82089 Europe/Berlin
52.10397 9.35623 Europe/Berlin
53.56667 10.1 Europe/Berlin
53.56667 10.08333 Europe/Berlin
53.56667 9.98333 Europe/Berlin
53.55 9.93333 Europe/Berlin
53.55 10 Europe/Berlin
51.18612 7.49817 Europe/Berlin
51.74297 7.18163 Europe/Berlin
53.63333 9.85 Europe/Berlin
52.06667 8.36667 Europe/Berlin
51.5 12 Europe/Berlin
52.28906 11.40982 Europe/Berlin
51.9 11.05 Europe/Berlin
50.75 8.21667 Europe/Berlin
51.35 7.46667 Europe/Berlin
48.10881 11.72653 Europe/Berlin
51.2 7 Europe/Berlin
51.9 8.38333 Europe/Berlin
53.7972 12.17337 Europe/Berlin
49.1 10.75 Europe/Berlin
48.45 10.26667 Europe/Berlin
51.03333 7.55 Europe/Berlin
51.94987 14.71447 Europe/Berlin
49.86778 8.93389 Europe/Berlin
49.92111 9.08139 Europe/Berlin
49.92389 8.49361 Europe/Berlin
51.28333 13.55 Europe/Berlin
52.2 7.03333 Europe/Berlin
48.2 11.36667 Europe/Berlin
51.23333 12.71667 Europe/Berlin
49.86278 8.57528 Europe/Berlin
51.09102 6.5827 Europe/Berlin
52.09364 7.59396 Europe/Berlin
50.65 12.2 Europe/Berlin
54.1 13.38333 Europe/Berlin
51.3363 6.34072 Europe/Berlin
51.53333 9.93333 Europe/Berlin
50.94823 10.70193 Europe/Berlin
51.90425 10.42766 Europe/Berlin
51.15518 14.98853 Europe/Berlin
48.7028 9.65488 Europe/Berlin
51.66667 6.16667 Europe/Berlin
53.55 10.2 Europe/Berlin
50.81667 12.53333 Europe/Berlin
51.56667 6.98333 Europe/Berlin
49.98583 8.3425 Europe/Berlin
48.11667 11.3 Europe/Berlin
52.48333 10.55 Europe/Berlin
50.58333 8.65 Europe/Berlin
48.62219 10.24312 Europe/Berlin
51.31667 7.33333 Europe/Berlin
51.65 8.51667 Europe/Berlin
51.954 7.00481 Europe/Berlin
48.41667 10.88333 Europe/Berlin
49.22306 8.36389 Europe/Berlin
48.13333 11.36667 Europe/Berlin
48.79954 9.06316 Europe/Berlin
47.85775 11.48054 Europe/Berlin
50.88029 12.08187 Europe/Berlin
52.2 8.05 Europe/Berlin
51.51667 7.05 Europe/Berlin
50.2 9.18333 Europe/Berlin
51.51667 6.33333 Europe/Berlin
48.62423 9.82736 Europe/Berlin
50.96667 6.11667 Europe/Berlin
53.43333 10.36667 Europe/Berlin
48.06919 11.37703 Europe/Berlin
47.49209 11.09576 Europe/Berlin
48.25 11.65 Europe/Berlin
52.41667 9.6 Europe/Berlin
53.03333 8.53333 Europe/Berlin
48.8 8.33333 Europe/Berlin
49.47593 10.98856 Europe/Berlin
52.36667 14.06667 Europe/Berlin
48.17524 11.25961 Europe/Berlin
50.55162 9.67518 Europe/Berlin
51.46667 7.76667 Europe/Berlin
53.01667 7.85 Europe/Berlin
47.65 9.48333 Europe/Berlin
50.24962 8.64281 Europe/Berlin
50.35 8.76667 Europe/Berlin
48.35693 10.98461 Europe/Berlin
48.43333 8.41667 Europe/Berlin
50.9 7.86667 Europe/Berlin
51.00166 13.6488 Europe/Berlin
48.40351 11.74876 Europe/Berlin
47.85 12.98333 Europe/Berlin
47.9959 7.85222 Europe/Berlin
48.93196 9.2024 Europe/Berlin
50.91667 13.36667 Europe/Berlin
50.91667 6.81667 Europe/Berlin
50.11667 8.68333 Europe/Berlin
52.34714 14.55062 Europe/Berlin
49.53722 8.355 Europe/Berlin
51.06667 8.8 Europe/Berlin
50.91667 13.03333 Europe/Berlin
51.73333 14.63333 Europe/Berlin
49.71754 11.05877 Europe/Berlin
50.01667 8.43333 Europe/Berlin
54.78333 9.43333 Europe/Berlin
51.63333 13.71667 Europe/Berlin
51.16667 7.96667 Europe/Berlin
48.80867 9.15719 Europe/Berlin
48.80912 9.27697 Europe/Berlin
52.56667 13.08333 Europe/Berlin
54.13333 10.61667 Europe/Berlin
50.66667 6.78333 Europe/Berlin
48.94094 8.40763 Europe/Berlin
48.73961 9.30473 Europe/Berlin
51.45 7.01667 Europe/Berlin
52.38333 8.61667 Europe/Berlin
50.81667 6.28333 Europe/Berlin
51.18333 10.06667 Europe/Berlin
50.13333 8.55 Europe/Berlin
51.61667 8.35 Europe/Berlin
49.58972 11.00389 Europe/Berlin
51.21667 6.9 Europe/Berlin
51.08333 6.31667 Europe/Berlin
50.98333 11.03333 Europe/Berlin
50.81667 6.76667 Europe/Berlin
48.30603 11.90686 Europe/Berlin
49.13694 8.91028 Europe/Berlin
49.4 6.96667 Europe/Berlin
51.83333 8.01667 Europe/Berlin
51.29848 7.3629 Europe/Berlin
52.13333 8.56667 Europe/Berlin
50.98333 7.4 Europe/Berlin
52.18333 7.53333 Europe/Berlin
51.83333 6.25 Europe/Berlin
48.12096 7.85359 Europe/Berlin
53.36667 7.21667 Europe/Berlin
50.03333 8.11667 Europe/Berlin
50.93739 6.56828 Europe/Berlin
53.75 9.65 Europe/Berlin
48.96164 10.13173 Europe/Berlin
50.76667 7.45 Europe/Berlin
48.69515 9.70676 Europe/Berlin
52.15 14.65 Europe/Berlin
50.98333 10.31667 Europe/Berlin
51.81667 9.86667 Europe/Berlin
51.45984 12.63338 Europe/Berlin
48.28259 9.72749 Europe/Berlin
49.09006 8.39879 Europe/Berlin
53.13333 7.98333 Europe/Berlin
54.46667 9.83333 Europe/Berlin
52.83492 13.81951 Europe/Berlin
48.716 9.5236 Europe/Berlin
49.46944 8.99278 Europe/Berlin
51.22172 6.77616 Europe/Berlin
50.8 6.48333 Europe/Berlin
51.83333 7.3 Europe/Berlin
51.43333 6.75 Europe/Berlin
51.51312 10.25951 Europe/Berlin
51.05089 13.73832 Europe/Berlin
51.79535 7.73815 Europe/Berlin
50 8.7 Europe/Berlin
51.51667 7.45 Europe/Berlin
51.66667 6.96667 Europe/Berlin
51.1 6.83333 Europe/Berlin
48.7 10.8 Europe/Berlin
47.95 8.5 Europe/Berlin
51.11667 13.11667 Europe/Berlin
48.82672 9.06703 Europe/Berlin
51.56667 6.73333 Europe/Berlin
48.63333 12.5 Europe/Berlin
48.56667 10.48333 Europe/Berlin
49.35 6.73333 Europe/Berlin
50.73333 8.28333 Europe/Berlin
50.01667 8.78333 Europe/Berlin
52.6 8.36667 Europe/Berlin
49.89889 8.84444 Europe/Berlin
51.93855 8.87318 Europe/Berlin
51.83864 12.24555 Europe/Berlin
53.05 8.61667 Europe/Berlin
51.53333 12.35 Europe/Berlin
51.76667 8.56667 Europe/Berlin
48.83333 12.96667 Europe/Berlin
51.66667 7.38333 Europe/Berlin
49.87056 8.64944 Europe/Berlin
52.51667 8.2 Europe/Berlin
48.26 11.43402 Europe/Berlin
53.87176 8.69087 Europe/Berlin
50.81667 12.38333 Europe/Berlin
49.13444 10.07193 Europe/Berlin
51.76667 14.33333 Europe/Berlin
51.13333 13.58333 Europe/Berlin
51.93333 7.15 Europe/Berlin
50.26667 10.96667 Europe/Berlin
52.85 8.03333 Europe/Berlin
51.8 10.33333 Europe/Berlin
50.83333 12.91667 Europe/Berlin
49.22556 12.66361 Europe/Berlin
52.62264 10.08047 Europe/Berlin
51.55 7.31667 Europe/Berlin
48.71419 8.74031 Europe/Berlin
53.45 9.7 Europe/Berlin
50.43395 8.67122 Europe/Berlin
49.64361 8.46 Europe/Berlin
51.1 7.11667 Europe/Berlin
48.16667 12.83333 Europe/Berlin
52.44628 10.0064 Europe/Berlin
52.27152 11.85493 Europe/Berlin
51.55 8.56667 Europe/Berlin
52.2 8.58333 Europe/Berlin
48.7 8.15 Europe/Berlin
50.3 9.11667 Europe/Berlin
52.26065 9.04939 Europe/Berlin
53.33333 9.86667 Europe/Berlin
49.52389 9.32306 Europe/Berlin
50.82928 6.90499 Europe/Berlin
51.24053 6.18376 Europe/Berlin
47.88333 11.91667 Europe/Berlin
49.12806 8.58361 Europe/Berlin
50.18333 8.91667 Europe/Berlin
51.4 8.58333 Europe/Berlin
49.03917 8.70278 Europe/Berlin
53.48333 9.13333 Europe/Berlin
53.55 8.58333 Europe/Berlin
53.07516 8.80777 Europe/Berlin
52.26667 10.53333 Europe/Berlin
52.41667 12.55 Europe/Berlin
52.4 7.98333 Europe/Berlin
51.71667 9.18333 Europe/Berlin
53.33333 8.48333 Europe/Berlin
49.07972 9.065 Europe/Berlin
51.51667 6.91667 Europe/Berlin
50.76667 7 Europe/Berlin
51.11667 12.5 Europe/Berlin
51.85 6.86667 Europe/Berlin
50.23085 7.58992 Europe/Berlin
50.73333 7.1 Europe/Berlin
51.6 7.76667 Europe/Berlin
51.48333 7.21667 Europe/Berlin
51.83333 6.6 Europe/Berlin
48.68333 9.01667 Europe/Berlin
48.26667 10.83333 Europe/Berlin
51.94331 9.09067 Europe/Berlin
49.23724 7.25617 Europe/Berlin
51.78333 10.95 Europe/Berlin
51.61667 12.31667 Europe/Berlin
49.96667 7.9 Europe/Berlin
48.96667 9.13333 Europe/Berlin
52.03333 8.53333 Europe/Berlin
48.09345 9.79053 Europe/Berlin
49.34615 7.25527 Europe/Berlin
51.66667 9.36667 Europe/Berlin
51.8 11.73333 Europe/Berlin
52.67982 13.58708 Europe/Berlin
52.52437 13.41053 Europe/Berlin
51.03333 7.65 Europe/Berlin
51.63333 7.63333 Europe/Berlin
50.98333 7.13333 Europe/Berlin
50.95572 6.63986 Europe/Berlin
49.68028 8.61611 Europe/Berlin
50.43333 7.56667 Europe/Berlin
50.99258 6.57128 Europe/Berlin
51.75 8.03333 Europe/Berlin
49.4 6.7 Europe/Berlin
49.94806 11.57833 Europe/Berlin
51.18333 14.43333 Europe/Berlin
51.25182 9.40747 Europe/Berlin
52.85 8.73333 Europe/Berlin
49.87948 10.88634 Europe/Berlin
48.27525 8.85464 Europe/Berlin
48.5 8.38333 Europe/Berlin
52.2 8.71667 Europe/Berlin
50.90964 6.18874 Europe/Berlin
53.18333 8 Europe/Berlin
51.11963 9.12475 Europe/Berlin
47.91667 9.76667 Europe/Berlin
50.17866 8.73756 Europe/Berlin
47.76111 11.5589 Europe/Berlin
50.1408 8.50449 Europe/Berlin
53.93333 10.31667 Europe/Berlin
53.91887 10.69691 Europe/Berlin
50.81667 10.23333 Europe/Berlin
52.0862 8.74434 Europe/Berlin
47.55371 7.94612 Europe/Berlin
47.73333 12.88333 Europe/Berlin
49.24056 9.1025 Europe/Berlin
51.98333 9.25 Europe/Berlin
53.81167 10.37417 Europe/Berlin
52.2 8.8 Europe/Berlin
50.33333 10.21667 Europe/Berlin
50.55 7.1 Europe/Berlin
50.36463 8.73859 Europe/Berlin
50.55 6.76667 Europe/Berlin
52.19551 9.46421 Europe/Berlin
49.4925 9.77361 Europe/Berlin
51.78333 8.81667 Europe/Berlin
51.1 10.65 Europe/Berlin
49.83333 7.86667 Europe/Berlin
50.2 10.08333 Europe/Berlin
50.64336 7.2278 Europe/Berlin
50.22667 8.61963 Europe/Berlin
50.86667 9.7 Europe/Berlin
51.88333 10.56667 Europe/Berlin
52.31667 8.33333 Europe/Berlin
48.75 8.25 Europe/Berlin
49.46861 8.16639 Europe/Berlin
51.73333 9.01667 Europe/Berlin
51.05 8.4 Europe/Berlin
52.31667 7.16667 Europe/Berlin
47.8638 12.01055 Europe/Berlin
48.95 9.43333 Europe/Berlin
49.96667 8.95778 Europe/Berlin
53.46667 7.48333 Europe/Berlin
48.36667 10.88333 Europe/Berlin
50.51667 12.4 Europe/Berlin
50.58333 12.7 Europe/Berlin
51.12645 7.90333 Europe/Berlin
51.75 11.46667 Europe/Berlin
51.78333 7.61667 Europe/Berlin
49.97389 9.14917 Europe/Berlin
51.37982 9.01445 Europe/Berlin
50.83333 10.95 Europe/Berlin
51.38333 8.08333 Europe/Berlin
51.01667 11.5 Europe/Berlin
49.3 10.58333 Europe/Berlin
50.57953 13.00627 Europe/Berlin
53.01499 13.99924 Europe/Berlin
50.43333 7.4 Europe/Berlin
49.44139 11.86222 Europe/Berlin
49.75167 8.11611 Europe/Berlin
50.08333 9.06667 Europe/Berlin
50.98333 12.45 Europe/Berlin
51.3 7.66667 Europe/Berlin
49.3875 11.35722 Europe/Berlin
50.75 9.26667 Europe/Berlin
50.88333 6.16667 Europe/Berlin
50.73333 7.01667 Europe/Berlin
51.98382 9.81989 Europe/Berlin
48.21644 9.02596 Europe/Berlin
48.45755 11.13413 Europe/Berlin
53.68333 10.25 Europe/Berlin
51.76338 7.8887 Europe/Berlin
52.06667 7 Europe/Berlin
53.01416 9.0263 Europe/Berlin
48.63115 8.07607 Europe/Berlin
48.83777 10.0933 Europe/Berlin
51.35806 9.47974 Europe/Berlin
51.9 8.61667 Europe/Berlin
49.36667 6.68333 Europe/Berlin
53.79132 9.97979 Europe/Berlin
50.77664 6.08342 Europe/Berlin
49.99472 8.58361 Europe/Berlin
49.83411 8.49621 Europe/Berlin
49.56528 9.70417 Europe/Berlin
48.65698 9.22049 Europe/Berlin
48.72704 9.24954 Europe/Berlin
50.02627 8.88588 Europe/Berlin
49.765 8.65194 Europe/Berlin
52.52003 13.40489 Europe/Berlin
51.56971 14.37355 Europe/Berlin
48.9685 8.30704 Europe/Berlin
48.78363 9.21032 Europe/Berlin
51.50168 7.1756 Europe/Berlin
53.557 9.964 Europe/Berlin
50.9426 7.00678 Europe/Berlin
50.9263 6.91012 Europe/Berlin
50.9631 7.00488 Europe/Berlin
50.963 6.9541 Europe/Berlin
53.60697 9.90538 Europe/Berlin
51.47924 11.91605 Europe/Berlin
53.48462 10.22904 Europe/Berlin
52.43485 13.24183 Europe/Berlin
52.50097 13.29097 Europe/Berlin
52.5511 13.19921 Europe/Berlin
52.47693 13.41027 Europe/Berlin
52.47457 13.34839 Europe/Berlin
52.49376 13.44469 Europe/Berlin
50.08309 8.62852 Europe/Berlin
51.2357 14.02576 Europe/Berlin
53.6052 10.03988 Europe/Berlin
53.60639 10.11972 Europe/Berlin
11.78528 42.88444 Africa/Djibouti
11.96306 43.29056 Africa/Djibouti
11.58767 43.14468 Africa/Djibouti
11.15583 42.7125 Africa/Djibouti
56.45319 9.40201 Europe/Copenhagen
55.70927 9.5357 Europe/Copenhagen
55.65173 12.29216 Europe/Copenhagen
55.05982 10.60677 Europe/Copenhagen
55.76828 12.19723 Europe/Copenhagen
54.90926 9.80737 Europe/Copenhagen
55.40276 11.35459 Europe/Copenhagen
56.56667 9.03333 Europe/Copenhagen
56.1697 9.54508 Europe/Copenhagen
55.64152 12.08035 Europe/Copenhagen
55.68062 12.45373 Europe/Copenhagen
55.4426 11.79011 Europe/Copenhagen
56.46667 10.05 Europe/Copenhagen
55.39594 10.38831 Europe/Copenhagen
54.76906 11.87425 Europe/Copenhagen
55.31274 10.78964 Europe/Copenhagen
55.22992 11.76092 Europe/Copenhagen
55.87496 12.34579 Europe/Copenhagen
55.32993 11.13857 Europe/Copenhagen
55.45802 12.18214 Europe/Copenhagen
55.67594 12.56553 Europe/Copenhagen
55.4904 9.47216 Europe/Copenhagen
55.67954 11.08864 Europe/Copenhagen
55.61543 12.35182 Europe/Copenhagen
55.65719 12.47364 Europe/Copenhagen
55.86066 9.85034 Europe/Copenhagen
55.88333 12.5 Europe/Copenhagen
56.36009 8.61607 Europe/Copenhagen
55.71667 11.71667 Europe/Copenhagen
57.46417 9.98229 Europe/Copenhagen
55.92667 12.31091 Europe/Copenhagen
56.13932 8.97378 Europe/Copenhagen
56.03606 12.6136 Europe/Copenhagen
55.25377 9.48982 Europe/Copenhagen
55.58333 12.3 Europe/Copenhagen
55.66667 12.4 Europe/Copenhagen
57.44073 10.53661 Europe/Copenhagen
55.67938 12.53463 Europe/Copenhagen
55.56568 9.75257 Europe/Copenhagen
55.80858 12.36066 Europe/Copenhagen
55.46667 8.45 Europe/Copenhagen
55.75367 12.59181 Europe/Copenhagen
55.84759 12.42791 Europe/Copenhagen
55.73165 12.36328 Europe/Copenhagen
56.15674 10.21076 Europe/Copenhagen
57.048 9.9187 Europe/Copenhagen
55.65691 12.36381 Europe/Copenhagen
55.04434 9.41741 Europe/Copenhagen
15.30174 -61.38808 America/Dominica
19.56378 -70.87582 America/Santo_Domingo
18.67297 -70.17095 America/Santo_Domingo
19.48538 -70.61104 America/Santo_Domingo
18.50012 -69.98857 America/Santo_Domingo
19.45 -70.7 America/Santo_Domingo
18.76559 -69.03886 America/Santo_Domingo
18.20854 -71.10077 America/Santo_Domingo
18.46156 -69.29718 America/Santo_Domingo
18.80588 -71.22991 America/Santo_Domingo
18.55 -70.5 America/Santo_Domingo
19.29518 -70.25543 America/Santo_Domingo
19.86667 -71.65 America/Santo_Domingo
19.79344 -70.6884 America/Santo_Domingo
18.61501 -68.70798 America/Santo_Domingo
19.47793 -71.34125 America/Santo_Domingo
18.94498 -69.79331 America/Santo_Domingo
18.55542 -69.40814 America/Santo_Domingo
18.58182 -68.40431 America/Santo_Domingo
18.48138 -71.41965 America/Santo_Domingo
19.3832 -69.8474 America/Santo_Domingo
18.807 -69.78399 America/Santo_Domingo
19.39352 -70.52598 America/Santo_Domingo
19.55186 -71.07813 America/Santo_Domingo
18.87309 -71.51656 America/Santo_Domingo
18.42734 -68.97285 America/Santo_Domingo
19.12178 -70.64218 America/Santo_Domingo
18.76278 -69.25681 America/Santo_Domingo
19.5847 -70.98489 America/Santo_Domingo
19.54878 -71.70829 America/Santo_Domingo
19.05272 -70.14939 America/Santo_Domingo
18.90919 -70.74499 America/Santo_Domingo
19.22207 -70.52956 America/Santo_Domingo
18.93687 -70.40923 America/Santo_Domingo
18.41667 -70.1 America/Santo_Domingo
18.75639 -69.63616 America/Santo_Domingo
18.28333 -70.33333 America/Santo_Domingo
18.41538 -70.03317 America/Santo_Domingo
18.45319 -70.7349 America/Santo_Domingo
36.76639 3.47717 Africa/Algiers
34.68284 6.51109 Africa/Algiers
36.71169 2.84244 Africa/Algiers
35.72251 0.75509 Africa/Algiers
33.10527 6.05796 Africa/Algiers
34.72224 5.37845 Africa/Algiers
34.87833 -1.315 Africa/Algiers
36.67176 4.19176 Africa/Algiers
36.54569 4.05712 Africa/Algiers
36.58839 3.77445 Africa/Algiers
36.65393 3.98143 Africa/Algiers
36.8 4.26667 Africa/Algiers
29.26388 0.23098 Africa/Algiers
36.65 4.77361 Africa/Algiers
35.87111 2.02806 Africa/Algiers
36.72544 3.55665 Africa/Algiers
36.11653 6.35434 Africa/Algiers
33.11667 6.08333 Africa/Algiers
35.48171 6.26074 Africa/Algiers
22.785 5.52278 Africa/Algiers
36.83763 6.64018 Africa/Algiers
36.74413 3.90045 Africa/Algiers
36.14766 3.69123 Africa/Algiers
36.51833 2.90528 Africa/Algiers
35.18568 1.49612 Africa/Algiers
36.87617 6.90921 Africa/Algiers
35.52832 -0.19369 Africa/Algiers
34.74512 5.89833 Africa/Algiers
36.60637 3.08783 Africa/Algiers
36.52056 6.26111 Africa/Algiers
34.387 4.98785 Africa/Algiers
35.65903 -0.52168 Africa/Algiers
35.18994 -0.63085 Africa/Algiers
33.49885 6.00803 Africa/Algiers
36.46472 1.30258 Africa/Algiers
35.88548 3.77236 Africa/Algiers
35.06937 -1.13706 Africa/Algiers
35.23464 -0.24435 Africa/Algiers
36.12868 7.53376 Africa/Algiers
36.54722 4.68611 Africa/Algiers
34.63703 -1.33143 Africa/Algiers
36.70456 3.02462 Africa/Algiers
35.8545 5.29053 Africa/Algiers
34.83034 0.15171 Africa/Algiers
31.92427 5.35018 Africa/Algiers
36.73829 3.28079 Africa/Algiers
36.45774 6.04267 Africa/Algiers
33.27966 6.90984 Africa/Algiers
35.06196 -1.43362 Africa/Algiers
33.56391 6.70326 Africa/Algiers
36.73587 3.34018 Africa/Algiers
26.71576 0.1714 Africa/Algiers
35.9441 5.03107 Africa/Algiers
35.67384 5.6453 Africa/Algiers
35.87541 7.11353 Africa/Algiers
34.90472 -1.03394 Africa/Algiers
36.10124 1.19949 Africa/Algiers
35.96124 0.91896 Africa/Algiers
36.18503 1.53299 Africa/Algiers
36.55528 2.79028 Africa/Algiers
35.45595 0.68778 Africa/Algiers
31.94932 5.32502 Africa/Algiers
35.69111 -0.64167 Africa/Algiers
35.01361 -1.74799 Africa/Algiers
36.74625 3.83163 Africa/Algiers
36.46695 2.68991 Africa/Algiers
35.93115 0.08918 Africa/Algiers
36.45028 6.26444 Africa/Algiers
32.26667 3.63333 Africa/Algiers
34.15429 3.50309 Africa/Algiers
35.63058 7.66606 Africa/Algiers
35.7279 -0.7081 Africa/Algiers
35.63106 5.91186 Africa/Algiers
35.97999 4.18665 Africa/Algiers
36.68178 4.26378 Africa/Algiers
35.43058 1.75714 Africa/Algiers
33.19195 6.08695 Africa/Algiers
36.6204 3.22248 Africa/Algiers
36.26417 2.75393 Africa/Algiers
36.12232 0.89865 Africa/Algiers
36.08725 4.45192 Africa/Algiers
34.86158 -1.33935 Africa/Algiers
36.78567 4.06273 Africa/Algiers
36.63112 4.19864 Africa/Algiers
36.56471 3.15434 Africa/Algiers
36.56463 3.5933 Africa/Algiers
33.8 2.86514 Africa/Algiers
35.88889 2.74905 Africa/Algiers
35.21222 2.31889 Africa/Algiers
36.63888 2.76845 Africa/Algiers
36.26104 2.22015 Africa/Algiers
36.64997 3.3308 Africa/Algiers
36.92917 6.58556 Africa/Algiers
35.92989 0.82871 Africa/Algiers
36.71859 3.66924 Africa/Algiers
27.19351 2.46069 Africa/Algiers
36.46295 4.50532 Africa/Algiers
34.95139 -1.36806 Africa/Algiers
36.50361 7.44278 Africa/Algiers
31.68041 6.07286 Africa/Algiers
35.44862 7.95184 Africa/Algiers
35.37889 -0.96778 Africa/Algiers
36.41205 6.59603 Africa/Algiers
36.51257 2.41382 Africa/Algiers
36.46214 7.42608 Africa/Algiers
35.06544 1.04945 Africa/Algiers
36.75234 4.3155 Africa/Algiers
36.5604 4.85454 Africa/Algiers
35.64779 -0.62397 Africa/Algiers
35.39138 -1.09238 Africa/Algiers
36.67942 4.8555 Africa/Algiers
36.26333 6.69361 Africa/Algiers
36.89556 8.44333 Africa/Algiers
34.44542 2.52749 Africa/Algiers
32.61336 5.51259 Africa/Algiers
36.80377 7.73684 Africa/Algiers
36.15281 5.69016 Africa/Algiers
36.22393 1.67187 Africa/Algiers
35.86691 7.88672 Africa/Algiers
35.52439 -1.01577 Africa/Algiers
36.4701 2.62528 Africa/Algiers
36.06386 4.62744 Africa/Algiers
32.893 0.54839 Africa/Algiers
36.26951 1.68609 Africa/Algiers
36.19591 1.25538 Africa/Algiers
36.68482 7.75111 Africa/Algiers
36.53628 3.8334 Africa/Algiers
36.73436 3.96223 Africa/Algiers
36.67 2.94444 Africa/Algiers
33.53388 5.99306 Africa/Algiers
36.4525 6.63639 Africa/Algiers
36.91716 3.91311 Africa/Algiers
33.51667 6.95 Africa/Algiers
36.71333 3.2125 Africa/Algiers
34.89638 3.48543 Africa/Algiers
36.365 6.61472 Africa/Algiers
36.36505 4.32636 Africa/Algiers
36.46293 2.73873 Africa/Algiers
34.92129 -1.29512 Africa/Algiers
35.27306 7.75194 Africa/Algiers
36.76775 2.95924 Africa/Algiers
36.6 4.61667 Africa/Algiers
36.16286 6.16651 Africa/Algiers
36.57722 3.00917 Africa/Algiers
34.62098 2.79503 Africa/Algiers
36.63709 3.69474 Africa/Algiers
33.09892 1.26082 Africa/Algiers
35.57272 -0.8996 Africa/Algiers
36.45833 7.51389 Africa/Algiers
36.06629 1.12602 Africa/Algiers
36.64262 2.69007 Africa/Algiers
36.53167 2.99194 Africa/Algiers
35.31473 -0.05037 Africa/Algiers
36.54178 3.081 Africa/Algiers
36.33293 5.08843 Africa/Algiers
36.57412 2.91214 Africa/Algiers
36.72735 3.40995 Africa/Algiers
36.80218 4.15187 Africa/Algiers
36.46298 2.81464 Africa/Algiers
36.27462 4.85668 Africa/Algiers
35.90111 4.89806 Africa/Algiers
36.74871 3.19248 Africa/Algiers
36.54222 3.95306 Africa/Algiers
36.71499 3.05002 Africa/Algiers
35.635 3.225 Africa/Algiers
35.72 -0.545 Africa/Algiers
34.74488 8.06024 Africa/Algiers
36.70222 7.84722 Africa/Algiers
36.13516 2.91085 Africa/Algiers
32.82648 3.7669 Africa/Algiers
36.83528 7.45333 Africa/Algiers
35.07465 -1.22431 Africa/Algiers
36.76967 7.9064 Africa/Algiers
35.30099 -1.38226 Africa/Algiers
34.87045 -1.42319 Africa/Algiers
36.52389 2.86131 Africa/Algiers
36.61954 4.08282 Africa/Algiers
36.66774 3.59115 Africa/Algiers
36.75587 5.08433 Africa/Algiers
31.61667 -2.21667 Africa/Algiers
35.55597 6.17414 Africa/Algiers
35.38901 5.36584 Africa/Algiers
36.56667 4.96667 Africa/Algiers
36.66655 3.09606 Africa/Algiers
36.72615 3.18291 Africa/Algiers
36.73944 7.10528 Africa/Algiers
36.74472 4.37222 Africa/Algiers
35.25881 6.34706 Africa/Algiers
36.79361 4.31158 Africa/Algiers
36.63773 3.37127 Africa/Algiers
26.96667 1.08333 Africa/Algiers
36.9 7.76667 Africa/Algiers
35.86781 1.11143 Africa/Algiers
36.64022 4.90131 Africa/Algiers
36.7525 3.04197 Africa/Algiers
36.4575 4.53494 Africa/Algiers
35.37675 5.90001 Africa/Algiers
35.29749 -1.14037 Africa/Algiers
36.79333 3.28694 Africa/Algiers
36.2674 6.50135 Africa/Algiers
32.75 -0.58333 Africa/Algiers
35.45139 2.90583 Africa/Algiers
36.16277 0.97037 Africa/Algiers
35.92472 6.69528 Africa/Algiers
35.97108 6.87374 Africa/Algiers
35.74381 -0.7693 Africa/Algiers
34.84146 4.16383 Africa/Algiers
36.56471 4.30619 Africa/Algiers
35.67003 3.88153 Africa/Algiers
35.80389 -0.30178 Africa/Algiers
35.36395 -0.51278 Africa/Algiers
34.3438 3.22475 Africa/Algiers
34.84218 1.54698 Africa/Algiers
36.29333 3.67319 Africa/Algiers
36.80277 2.92185 Africa/Algiers
35.79639 7.39278 Africa/Algiers
36.18683 5.31347 Africa/Algiers
36.23194 6.94333 Africa/Algiers
34.1128 2.10228 Africa/Algiers
36.41657 1.19616 Africa/Algiers
36.48994 5.5393 Africa/Algiers
-4.06917 -78.95667 America/Guayaquil
-2.11667 -79.68333 America/Guayaquil
-1.55 -79.73333 America/Guayaquil
-1.45 -79.46667 America/Guayaquil
-1.05 -79.61667 America/Guayaquil
-0.98333 -77.81667 America/Guayaquil
-1.26667 -80.43333 America/Guayaquil
-0.25 -79.15 America/Guayaquil
-3.45 -79.96667 America/Guayaquil
-2.23333 -80.85 America/Guayaquil
1.28833 -78.83694 America/Guayaquil
0.6 -77.81667 America/Guayaquil
-1.95 -79.73333 America/Guayaquil
-2.21667 -80.96667 America/Guayaquil
0.33333 -79.46667 America/Guayaquil
-1.66667 -78.63333 America/Guayaquil
-0.22985 -78.52495 America/Guayaquil
-1.03333 -79.45 America/Guayaquil
-1.46667 -77.98333 America/Guayaquil
-0.95 -78.68333 America/Guayaquil
0.93333 -79.68333 America/Guayaquil
-1.05 -80.45 America/Guayaquil
-2.63333 -80.38333 America/Guayaquil
-3.66667 -79.65 America/Guayaquil
-1.31667 -78.53333 America/Guayaquil
-1.83333 -80.23333 America/Guayaquil
-3.33333 -79.81667 America/Guayaquil
0.23333 -78.26667 America/Guayaquil
-2.21667 -79.48333 America/Guayaquil
-2.66667 -79.61667 America/Guayaquil
-1.05 -80.66667 America/Guayaquil
-1.8 -79.33333 America/Guayaquil
-0.95 -80.73333 America/Guayaquil
-3.26667 -79.96667 America/Guayaquil
-0.5 -78.56667 America/Guayaquil
-2.31667 -78.11667 America/Guayaquil
-3.99313 -79.20422 America/Guayaquil
-2.4 -79.33333 America/Guayaquil
-0.93333 -78.61667 America/Guayaquil
-0.93333 -79.21667 America/Guayaquil
-2.23333 -80.9 America/Guayaquil
0.08472 -76.88278 America/Guayaquil
-1.33333 -80.58333 America/Guayaquil
0.35 -78.11667 America/Guayaquil
-3.48333 -80.23333 America/Guayaquil
-2.16667 -79.9 America/Guayaquil
-1.6 -79 America/Guayaquil
-2.9 -78.78333 America/Guayaquil
-1.93333 -79.96667 America/Guayaquil
-2.2 -79.83333 America/Guayaquil
-2.88333 -78.98333 America/Guayaquil
-0.68333 -80.1 America/Guayaquil
0.05 -78.13333 America/Guayaquil
-3.98333 -79.35 America/Guayaquil
-4.33333 -79.55 America/Guayaquil
-0.85 -80.16667 America/Guayaquil
-0.71667 -77.13333 America/Guayaquil
-1.36667 -79.9 America/Guayaquil
-0.6 -80.41667 America/Guayaquil
-1.81667 -79.51667 America/Guayaquil
-2.73333 -78.83333 America/Guayaquil
0.33333 -78.21667 America/Guayaquil
-1.24908 -78.61675 America/Guayaquil
58.36389 25.59 Europe/Tallinn
58.38062 26.72509 Europe/Tallinn
59.43696 24.75353 Europe/Tallinn
59.39972 27.75472 Europe/Tallinn
59.34639 26.35583 Europe/Tallinn
58.38588 24.49711 Europe/Tallinn
59.37722 28.19028 Europe/Tallinn
59.47667 25.025 Europe/Tallinn
59.39861 27.27306 Europe/Tallinn
30.7142 31.24425 Africa/Cairo
30.35394 31.20071 Africa/Cairo
30.78847 31.00192 Africa/Cairo
29.47639 30.96119 Africa/Cairo
31.0539 31.37787 Africa/Cairo
30.68025 30.94292 Africa/Cairo
26.7693 31.50214 Africa/Cairo
28.91667 30.85 Africa/Cairo
26.55695 31.69478 Africa/Cairo
31.27105 30.78665 Africa/Cairo
31.19544 31.52127 Africa/Cairo
30.31269 31.32018 Africa/Cairo
30.96033 31.24332 Africa/Cairo
28.31214 30.71007 Africa/Cairo
31.40444 30.41639 Africa/Cairo
30.56482 31.15777 Africa/Cairo
30.97225 30.95614 Africa/Cairo
25.91407 32.76362 Africa/Cairo
26.16418 32.72671 Africa/Cairo
30.17922 31.2056 Africa/Cairo
26.04949 32.24142 Africa/Cairo
31.12294 31.6399 Africa/Cairo
30.46579 30.93164 Africa/Cairo
28.41899 30.77924 Africa/Cairo
30.36056 31.3776 Africa/Cairo
31.35254 27.24528 Africa/Cairo
27.3104 30.97004 Africa/Cairo
27.7314 30.84165 Africa/Cairo
29.81667 31.05 Africa/Cairo
24.47669 32.94626 Africa/Cairo
30.76096 30.69733 Africa/Cairo
30.79336 31.62575 Africa/Cairo
30.8248 30.81805 Africa/Cairo
31.1143 30.94012 Africa/Cairo
31.13385 30.12843 Africa/Cairo
26.67319 31.4976 Africa/Cairo
26.33826 31.89161 Africa/Cairo
31.5084 31.84106 Africa/Cairo
29.2376 30.78944 Africa/Cairo
25.29336 32.55402 Africa/Cairo
31.30782 30.29901 Africa/Cairo
29.35804 30.68142 Africa/Cairo
29.84144 31.30084 Africa/Cairo
30.6713 31.58801 Africa/Cairo
30.9128 30.29018 Africa/Cairo
31.20291 30.55018 Africa/Cairo
26.05494 32.16329 Africa/Cairo
31.32977 31.71507 Africa/Cairo
30.73006 31.80182 Africa/Cairo
31.41648 31.81332 Africa/Cairo
30.75422 31.43925 Africa/Cairo
31.13305 30.64649 Africa/Cairo
26.12467 32.47598 Africa/Cairo
31.08871 31.59427 Africa/Cairo
27.55602 30.80764 Africa/Cairo
27.64176 30.84662 Africa/Cairo
31.03917 30.46914 Africa/Cairo
29.14816 31.12733 Africa/Cairo
31.25654 32.28412 Africa/Cairo
26.72918 33.93651 Africa/Cairo
31.21452 31.35798 Africa/Cairo
30.42039 31.56223 Africa/Cairo
30.93976 30.81338 Africa/Cairo
29.07441 31.09785 Africa/Cairo
28.5036 30.8004 Africa/Cairo
30.45906 31.17858 Africa/Cairo
30.58768 31.502 Africa/Cairo
30.12303 31.13571 Africa/Cairo
30.54448 31.78525 Africa/Cairo
27.18096 31.18368 Africa/Cairo
24.09343 32.90704 Africa/Cairo
29.97371 32.52627 Africa/Cairo
29.56472 31.28111 Africa/Cairo
30.59618 30.89875 Africa/Cairo
30.29857 30.97635 Africa/Cairo
29.33778 31.20556 Africa/Cairo
25.69893 32.6421 Africa/Cairo
27.4402 30.81841 Africa/Cairo
26.10426 34.27793 Africa/Cairo
30.61618 31.73514 Africa/Cairo
30.6188 31.46099 Africa/Cairo
30.19327 31.13703 Africa/Cairo
30.06263 31.24967 Africa/Cairo
28.10988 30.7503 Africa/Cairo
31.18287 32.03108 Africa/Cairo
31.15815 31.937 Africa/Cairo
31.03637 31.38069 Africa/Cairo
26.47686 31.8035 Africa/Cairo
30.9745 31.16499 Africa/Cairo
30.21035 31.36812 Africa/Cairo
30.00808 31.21093 Africa/Cairo
31.18086 31.86518 Africa/Cairo
30.60427 32.27225 Africa/Cairo
31.21564 29.95527 Africa/Cairo
30.71877 31.56299 Africa/Cairo
29.9 31.25 Africa/Cairo
31.31146 31.14766 Africa/Cairo
27.25738 33.81291 Africa/Cairo
29.30995 30.8418 Africa/Cairo
28.82431 30.89948 Africa/Cairo
28.34919 28.86591 Africa/Cairo
26.23568 32.00347 Africa/Cairo
30.43026 31.03681 Africa/Cairo
26.99257 31.41554 Africa/Cairo
29.61972 31.2575 Africa/Cairo
31.13159 33.79844 Africa/Cairo
26.56217 31.74503 Africa/Cairo
30.94154 31.29151 Africa/Cairo
30.82791 30.53633 Africa/Cairo
27.04411 31.31897 Africa/Cairo
27.9312 30.83841 Africa/Cairo
30.72508 31.67148 Africa/Cairo
30.91018 30.17438 Africa/Cairo
27.2696 31.15105 Africa/Cairo
24.98028 32.87472 Africa/Cairo
31.20829 31.63554 Africa/Cairo
29.60018 32.31671 Africa/Cairo
26.73841 -11.67194 Africa/El_Aaiun
27.16224 -13.20315 Africa/El_Aaiun
23.68477 -15.95798 Africa/El_Aaiun
15.60972 39.45 Africa/Asmara
15.77778 38.45806 Africa/Asmara
15.11389 37.59278 Africa/Asmara
15.33333 38.93333 Africa/Asmara
13.00917 42.73944 Africa/Asmara
14.88722 38.81528 Africa/Asmara
37.11906 -3.584 Europe/Madrid
38.41667 -6.41667 Europe/Madrid
38.61365 -1.11468 Europe/Madrid
38.6373 -0.86568 Europe/Madrid
39.26992 -2.60118 Europe/Madrid
39.93333 -0.1 Europe/Madrid
38.97655 -5.7974 Europe/Madrid
38.50754 -0.23346 Europe/Madrid
36.83155 -2.64273 Europe/Madrid
36.77262 -4.10045 Europe/Madrid
39.46975 -0.37739 Europe/Madrid
38.76211 -3.38483 Europe/Madrid
37.18516 -5.78093 Europe/Madrid
36.67778 -5.446 Europe/Madrid
38.01328 -3.3705 Europe/Madrid
37.7688 -1.50229 Europe/Madrid
36.75793 -3.95233 Europe/Madrid
37.97872 -0.68222 Europe/Madrid
37.74293 -0.95396 Europe/Madrid
39.43705 -0.46546 Europe/Madrid
36.62035 -4.49976 Europe/Madrid
39.15218 -3.02431 Europe/Madrid
37.37281 -6.04589 Europe/Madrid
39.8581 -4.02263 Europe/Madrid
28.96108 -13.64502 Atlantic/Canary
27.99242 -15.41915 Atlantic/Canary
29.06049 -13.56398 Atlantic/Canary
36.01393 -5.60695 Europe/Madrid
39.96348 -4.83076 Europe/Madrid
28.47688 -16.41016 Atlantic/Canary
39.06667 -0.26667 Europe/Madrid
39.2026 -0.31114 Europe/Madrid
39.36667 -0.41667 Europe/Madrid
37.38241 -5.97613 Europe/Madrid
38.3964 -0.5255 Europe/Madrid
38.06147 -1.04877 Europe/Madrid
38.19165 -0.5658 Europe/Madrid
27.91174 -15.54071 Atlantic/Canary
38.98457 1.53409 Europe/Madrid
28.46824 -16.25462 Atlantic/Canary
28.68351 -17.76421 Atlantic/Canary
28.03197 -15.50425 Atlantic/Canary
36.21067 -5.38415 Europe/Madrid
37.83568 -0.79102 Europe/Madrid
28.09826 -16.61708 Atlantic/Canary
36.77808 -6.3515 Europe/Madrid
37.35812 -6.03731 Europe/Madrid
38.40148 -0.43623 Europe/Madrid
38.91667 1.28333 Europe/Madrid
37.80626 -0.83736 Europe/Madrid
36.4759 -6.19817 Europe/Madrid
28.4853 -16.32014 Atlantic/Canary
27.92481 -15.57329 Atlantic/Canary
29.00093 -13.613 Atlantic/Canary
38.98068 1.30362 Europe/Madrid
39.68333 -0.26667 Europe/Madrid
36.62236 -6.35859 Europe/Madrid
36.76419 -2.61475 Europe/Madrid
36.74231 -5.16709 Europe/Madrid
38.08799 -0.72544 Europe/Madrid
36.71715 -4.27583 Europe/Madrid
39.55 -0.56667 Europe/Madrid
39.48834 -1.10044 Europe/Madrid
28.37645 -16.58575 Atlantic/Canary
39.61667 -0.3 Europe/Madrid
36.52819 -6.19011 Europe/Madrid
38.68712 -4.10734 Europe/Madrid
28.50038 -13.86272 Atlantic/Canary
28.41397 -16.54867 Atlantic/Canary
37.38943 -4.76686 Europe/Madrid
39.6 -0.55 Europe/Madrid
37.43807 -4.19523 Europe/Madrid
38.37906 -4.84827 Europe/Madrid
39.87678 3.01626 Europe/Madrid
37.86591 -0.79256 Europe/Madrid
39.36667 -0.45 Europe/Madrid
39.5 -0.43333 Europe/Madrid
39.56939 2.65024 Europe/Madrid
37.70024 -5.28121 Europe/Madrid
28.35039 -14.1076 Atlantic/Canary
39.43333 -0.41667 Europe/Madrid
37.23765 -5.10746 Europe/Madrid
38.08483 -0.94401 Europe/Madrid
38.81667 -0.61667 Europe/Madrid
39.96667 -0.25 Europe/Madrid
38.91971 -0.11935 Europe/Madrid
38.38479 -0.76773 Europe/Madrid
36.96655 -2.20595 Europe/Madrid
36.75855 -3.88727 Europe/Madrid
39.89158 -5.54064 Europe/Madrid
37.98704 -1.13004 Europe/Madrid
38.04094 -1.49014 Europe/Madrid
38.4158 -0.44529 Europe/Madrid
36.75066 -3.5179 Europe/Madrid
37.12084 -5.45403 Europe/Madrid
37.58627 -4.63805 Europe/Madrid
38.90839 -6.61785 Europe/Madrid
39.54555 -0.39551 Europe/Madrid
38.05456 -1.20763 Europe/Madrid
37.27559 -6.83851 Europe/Madrid
27.88385 -15.72538 Atlantic/Canary
39.47523 -0.41825 Europe/Madrid
36.59575 -4.63728 Europe/Madrid
38.91611 -6.34366 Europe/Madrid
35.29369 -2.93833 Africa/Ceuta
37.5992 -1.31493 Europe/Madrid
39.56667 -0.33333 Europe/Madrid
37.72107 -3.97264 Europe/Madrid
39.64208 2.75388 Europe/Madrid
37.329 -5.41682 Europe/Madrid
36.51543 -4.88583 Europe/Madrid
37.20764 -3.63493 Europe/Madrid
38.99915 -3.36991 Europe/Madrid
39.49139 -0.46349 Europe/Madrid
39.56964 3.20955 Europe/Madrid
36.72016 -4.42034 Europe/Madrid
37.34461 -6.06391 Europe/Madrid
37.37301 -5.74952 Europe/Madrid
39.88853 4.26583 Europe/Madrid
37.40881 -4.48522 Europe/Madrid
37.16181 -5.92433 Europe/Madrid
28.65851 -17.91821 Atlantic/Canary
36.18482 -5.49213 Europe/Madrid
37.74425 -0.85041 Europe/Madrid
37.67119 -1.7017 Europe/Madrid
37.65896 -5.52751 Europe/Madrid
37.16887 -4.15129 Europe/Madrid
39.49093 2.89108 Europe/Madrid
39.63333 -0.6 Europe/Madrid
38.09519 -3.63602 Europe/Madrid
37.25482 -7.20433 Europe/Madrid
36.92077 -6.07529 Europe/Madrid
37.61915 -0.87799 Europe/Madrid
38.02822 -1.24188 Europe/Madrid
28.09973 -15.41343 Atlantic/Canary
38.94422 -3.2381 Europe/Madrid
36.9838 -5.93933 Europe/Madrid
39.21667 -2.15 Europe/Madrid
37.48734 -5.98258 Europe/Madrid
28.39076 -16.52309 Atlantic/Canary
28.61052 -13.92912 Atlantic/Canary
38.61372 -0.1269 Europe/Madrid
36.16809 -5.34777 Europe/Madrid
39.56667 -0.53333 Europe/Madrid
38.27559 -3.61534 Europe/Madrid
38.47917 -1.325 Europe/Madrid
36.68645 -6.13606 Europe/Madrid
38.78333 0.16667 Europe/Madrid
38.98333 -0.51667 Europe/Madrid
37.76667 -3.78333 Europe/Madrid
37.2 -7.31667 Europe/Madrid
27.92086 -15.4406 Atlantic/Canary
39.7211 2.91093 Europe/Madrid
28.37241 -16.71188 Atlantic/Canary
38.90883 1.43296 Europe/Madrid
38.62533 -0.57225 Europe/Madrid
37.38897 -1.94536 Europe/Madrid
37.25833 -6.95083 Europe/Madrid
38.5106 -1.70096 Europe/Madrid
28.3039 -16.43254 Atlantic/Canary
28.21154 -16.77947 Atlantic/Canary
38.09031 -0.65556 Europe/Madrid
37.29932 -3.13922 Europe/Madrid
28.11882 -16.57599 Atlantic/Canary
37.18817 -3.60667 Europe/Madrid
38.96667 -0.18333 Europe/Madrid
28.14701 -15.6502 Atlantic/Canary
36.53998 -4.62473 Europe/Madrid
39.4696 3.14831 Europe/Madrid
36.42764 -5.14589 Europe/Madrid
37.39106 -5.71993 Europe/Madrid
36.59389 -6.23298 Europe/Madrid
36.77629 -2.81456 Europe/Madrid
38.47783 -0.79157 Europe/Madrid
38.26218 -0.70107 Europe/Madrid
37.26273 -5.5453 Europe/Madrid
37.5422 -5.0826 Europe/Madrid
37.28287 -5.92088 Europe/Madrid
38.95627 -5.86162 Europe/Madrid
38.84078 0.10574 Europe/Madrid
39.07004 -3.61498 Europe/Madrid
39.16667 -0.25 Europe/Madrid
39.48139 -0.43937 Europe/Madrid
38.24994 -0.80975 Europe/Madrid
37.28766 -6.0541 Europe/Madrid
37.88333 -4.76667 Europe/Madrid
36.27719 -6.0885 Europe/Madrid
36.65947 -4.75639 Europe/Madrid
38.98333 -3.93333 Europe/Madrid
38.23998 -1.41987 Europe/Madrid
39.46588 -0.42589 Europe/Madrid
36.73663 -6.43703 Europe/Madrid
36.41915 -6.14941 Europe/Madrid
38.09242 -1.7985 Europe/Madrid
39.4 -0.4 Europe/Madrid
37.38594 -6.05258 Europe/Madrid
39.98333 -0.03333 Europe/Madrid
37.28114 -7.15071 Europe/Madrid
36.71068 -4.63297 Europe/Madrid
37.60512 -0.98623 Europe/Madrid
37.47125 -5.64608 Europe/Madrid
39.23333 -0.51667 Europe/Madrid
39.13333 -0.45 Europe/Madrid
38.10558 -1.86342 Europe/Madrid
28.3548 -16.37268 Atlantic/Canary
39.40463 -3.12492 Europe/Madrid
38.42885 -0.39774 Europe/Madrid
37.40202 -6.03314 Europe/Madrid
39.5657 2.50621 Europe/Madrid
38.6447 0.0445 Europe/Madrid
38.12497 -0.87822 Europe/Madrid
36.53361 -6.29944 Europe/Madrid
39.47649 -6.37224 Europe/Madrid
37.47249 -4.44206 Europe/Madrid
39.88901 -0.08499 Europe/Madrid
39.51667 -0.41667 Europe/Madrid
37.37358 -6.07232 Europe/Madrid
39.58333 -0.45 Europe/Madrid
36.84693 -2.94966 Europe/Madrid
38.53816 -0.13098 Europe/Madrid
39.41667 -0.4 Europe/Madrid
36.59548 -4.56937 Europe/Madrid
37.49073 -2.77259 Europe/Madrid
36.19237 -5.92186 Europe/Madrid
38.09639 -3.77786 Europe/Madrid
37.99384 -3.47103 Europe/Madrid
37.6167 -4.32245 Europe/Madrid
38.87789 -6.97061 Europe/Madrid
37.20994 -7.40266 Europe/Madrid
37.22479 -3.68686 Europe/Madrid
38.34511 -0.76722 Europe/Madrid
28.11983 -15.52325 Atlantic/Canary
28.96302 -13.54769 Atlantic/Canary
28.09962 -16.68102 Atlantic/Canary
37.14102 -3.61854 Europe/Madrid
36.75075 -5.81056 Europe/Madrid
38.11631 -1.30043 Europe/Madrid
37.01938 -4.56123 Europe/Madrid
38.03922 -4.05078 Europe/Madrid
38.59885 -0.05139 Europe/Madrid
36.73393 -3.69072 Europe/Madrid
38.10879 -0.79197 Europe/Madrid
37.2647 -6.51667 Europe/Madrid
36.83814 -2.45974 Europe/Madrid
38.68316 -6.40747 Europe/Madrid
39.95 -0.05 Europe/Madrid
38.86917 -1.09713 Europe/Madrid
37.2699 -7.02312 Europe/Madrid
38.34517 -0.48149 Europe/Madrid
36.643 -4.68728 Europe/Madrid
36.66401 -4.56139 Europe/Madrid
37.85103 -1.42507 Europe/Madrid
39.18333 -0.43333 Europe/Madrid
36.13326 -5.45051 Europe/Madrid
39.41667 -0.38333 Europe/Madrid
39.46569 -0.46005 Europe/Madrid
39.85316 3.12138 Europe/Madrid
38.70545 -0.47432 Europe/Madrid
39.15 -0.43333 Europe/Madrid
39.39011 -3.20827 Europe/Madrid
37.96939 -1.21714 Europe/Madrid
37.4614 -3.92301 Europe/Madrid
37.33791 -5.83951 Europe/Madrid
39.5 -0.35 Europe/Madrid
37.23088 -3.6551 Europe/Madrid
39.4 -0.41667 Europe/Madrid
38.98333 -1.85 Europe/Madrid
39.45568 -0.461 Europe/Madrid
27.90539 -15.44609 Atlantic/Canary
37.4063 -1.58289 Europe/Madrid
36.74961 -3.02055 Europe/Madrid
28.12271 -16.726 Atlantic/Canary
39.64167 -0.23889 Europe/Madrid
43.28444 -2.16992 Europe/Madrid
41.65606 -0.87734 Europe/Madrid
41.5 -5.75 Europe/Madrid
43.66228 -7.59344 Europe/Madrid
42.85 -2.66667 Europe/Madrid
40.47033 0.47559 Europe/Madrid
40.35692 -3.90011 Europe/Madrid
42.64685 -5.55835 Europe/Madrid
41.22392 1.72511 Europe/Madrid
40.48333 -3.96667 Europe/Madrid
40.4469 -4.00428 Europe/Madrid
43.3 -7.68333 Europe/Madrid
42.59631 -8.76426 Europe/Madrid
41.35 1.7 Europe/Madrid
41.11667 1.15 Europe/Madrid
42.06174 2.25528 Europe/Madrid
41.31405 2.01427 Europe/Madrid
42.23282 -8.72264 Europe/Madrid
41.93012 2.25486 Europe/Madrid
41.21667 1.53333 Europe/Madrid
41.28612 1.24993 Europe/Madrid
41.65518 -4.72372 Europe/Madrid
40.19081 -3.67887 Europe/Madrid
40.32605 -3.51088 Europe/Madrid
41.71394 -0.99487 Europe/Madrid
42.04713 -8.64435 Europe/Madrid
42.06166 -1.60452 Europe/Madrid
40.81249 0.5216 Europe/Madrid
40.57654 -3.92658 Europe/Madrid
43.34944 -4.04785 Europe/Madrid
40.45535 -3.46973 Europe/Madrid
41.14505 1.39861 Europe/Madrid
41.69914 2.71888 Europe/Madrid
43.13484 -2.07801 Europe/Madrid
40.3456 -1.10646 Europe/Madrid
42.75 -8.5 Europe/Madrid
41.64704 1.13957 Europe/Madrid
41.56667 2.01667 Europe/Madrid
41.11667 1.25 Europe/Madrid
40.00851 -3.00731 Europe/Madrid
41.76401 -2.46883 Europe/Madrid
41.23506 1.81193 Europe/Madrid
43.30975 -3.00716 Europe/Madrid
40.10473 -3.69793 Europe/Madrid
40.95 -4.11667 Europe/Madrid
41.49109 2.14079 Europe/Madrid
41.39317 2.00689 Europe/Madrid
43.29564 -2.99729 Europe/Madrid
43.32842 -3.03248 Europe/Madrid
41.38602 2.07573 Europe/Madrid
42.88052 -8.54569 Europe/Madrid
41.53333 2.18333 Europe/Madrid
43.46472 -3.80444 Europe/Madrid
41.5159 2.12457 Europe/Madrid
41.45152 2.2081 Europe/Madrid
40.54433 -3.61588 Europe/Madrid
43.31283 -1.97499 Europe/Madrid
41.53333 2.08333 Europe/Madrid
41.26667 1.76667 Europe/Madrid
40.20735 -3.57063 Europe/Madrid
40.59144 -4.14738 Europe/Madrid
41.50961 2.39365 Europe/Madrid
41.36667 2.06667 Europe/Madrid
42.39996 -8.80698 Europe/Madrid
40.42386 -3.53261 Europe/Madrid
41.38333 2.05 Europe/Madrid
41.78333 3.03333 Europe/Madrid
41.46667 2.08333 Europe/Madrid
41.68333 2.48333 Europe/Madrid
40.61667 0.6 Europe/Madrid
41.34357 2.03659 Europe/Madrid
42.61174 -5.61671 Europe/Madrid
41.44659 1.97187 Europe/Madrid
41.43073 2.21855 Europe/Madrid
43.29568 -5.68416 Europe/Madrid
41.97489 2.79281 Europe/Madrid
41.07663 1.14163 Europe/Madrid
40.96667 -5.65 Europe/Madrid
41.54329 2.10942 Europe/Madrid
41.49226 2.03305 Europe/Madrid
42.26199 3.17689 Europe/Madrid
41.49686 2.15739 Europe/Madrid
42.7461 -8.44392 Europe/Madrid
41.15612 1.10687 Europe/Madrid
43.31195 -1.90234 Europe/Madrid
42.28337 -8.6096 Europe/Madrid
42.18333 -8.5 Europe/Madrid
41.49206 2.36524 Europe/Madrid
40.43293 -3.81338 Europe/Madrid
42.44423 -8.71482 Europe/Madrid
43.32099 -3.02064 Europe/Madrid
42.16156 -8.6198 Europe/Madrid
42.431 -8.64435 Europe/Madrid
42.54664 -6.59619 Europe/Madrid
43.39228 -5.66335 Europe/Madrid
40.03116 -6.08845 Europe/Madrid
40.24147 -3.69999 Europe/Madrid
41.62763 2.6889 Europe/Madrid
40.23604 -3.76752 Europe/Madrid
41.57481 2.23306 Europe/Madrid
42.81687 -1.64323 Europe/Madrid
42.01667 -4.53333 Europe/Madrid
41.84843 3.12912 Europe/Madrid
41.91738 3.1631 Europe/Madrid
43.36029 -5.84476 Europe/Madrid
43.25541 -2.01873 Europe/Madrid
42.33333 -7.85 Europe/Madrid
42.18096 2.49012 Europe/Madrid
41.54372 1.89408 Europe/Madrid
43.33333 -8.31667 Europe/Madrid
42.14153 -8.80656 Europe/Madrid
40.28908 -4.01197 Europe/Madrid
43.51667 -8.15278 Europe/Madrid
43.35 -2.83333 Europe/Madrid
40.32234 -3.86496 Europe/Madrid
41.91084 0.19406 Europe/Madrid
41.53333 2.26667 Europe/Madrid
42.52165 -7.51422 Europe/Madrid
43.06441 -2.48977 Europe/Madrid
41.48333 2.18333 Europe/Madrid
41.54026 2.21306 Europe/Madrid
41.41667 2.01667 Europe/Madrid
42.28333 -8.75 Europe/Madrid
42.6865 -2.94695 Europe/Madrid
43.25 -5.76667 Europe/Madrid
40.39283 -3.48194 Europe/Madrid
41.31239 -4.91413 Europe/Madrid
41.54211 2.4445 Europe/Madrid
41.47978 2.3188 Europe/Madrid
41.47402 1.93062 Europe/Madrid
42.38333 -8.7 Europe/Madrid
41.72498 1.82656 Europe/Madrid
42.00228 2.28476 Europe/Madrid
41.64662 2.74135 Europe/Madrid
40.47353 -3.87182 Europe/Madrid
40.4165 -3.70256 Europe/Madrid
43 -7.56667 Europe/Madrid
42.46667 -2.45 Europe/Madrid
41.69993 2.84565 Europe/Madrid
43.14322 -2.96204 Europe/Madrid
41.61667 0.61667 Europe/Madrid
42.6 -5.56667 Europe/Madrid
43.33333 -2.98333 Europe/Madrid
40.32718 -3.7635 Europe/Madrid
40.49292 -3.87371 Europe/Madrid
43.26774 -2.02169 Europe/Madrid
41.07625 1.18515 Europe/Madrid
42.66085 -8.11285 Europe/Madrid
41.58151 -4.72332 Europe/Madrid
42.68333 -8.48333 Europe/Madrid
43.37135 -8.396 Europe/Madrid
43.33904 -1.78938 Europe/Madrid
40.12213 -3.84704 Europe/Madrid
41.58098 1.6172 Europe/Madrid
40.25038 -3.83062 Europe/Madrid
42.13615 -0.4087 Europe/Madrid
41.35967 2.10028 Europe/Madrid
43.26615 -1.97615 Europe/Madrid
43.31667 -2.68333 Europe/Madrid
43.35689 -3.01146 Europe/Madrid
40.63333 -3.16667 Europe/Madrid
41.60797 2.28773 Europe/Madrid
43.53573 -5.66152 Europe/Madrid
40.30571 -3.73295 Europe/Madrid
41.98311 2.82493 Europe/Madrid
41.30605 2.00123 Europe/Madrid
43.23333 -2.83333 Europe/Madrid
40.5783 -4.00426 Europe/Madrid
43.37202 -1.79448 Europe/Madrid
40.28419 -3.79415 Europe/Madrid
43.53943 -7.02559 Europe/Madrid
42.26645 2.96163 Europe/Madrid
41.37732 2.08809 Europe/Madrid
41.53333 1.86667 Europe/Madrid
43.18333 -2.5 Europe/Madrid
43.30788 -2.94502 Europe/Madrid
41.32784 2.09472 Europe/Madrid
43.48321 -8.23689 Europe/Madrid
43.40094 -3.82051 Europe/Madrid
42.12632 -1.13716 Europe/Madrid
43.18493 -2.47158 Europe/Madrid
43.17124 -2.6338 Europe/Madrid
43.28788 -8.38858 Europe/Madrid
40.06667 -2.13333 Europe/Madrid
40.42378 -3.56129 Europe/Madrid
43.26697 -3.9471 Europe/Madrid
41.35 2.08333 Europe/Madrid
40.65909 -3.76762 Europe/Madrid
40.63506 -4.00486 Europe/Madrid
40.00112 3.84144 Europe/Madrid
40.15913 -3.62103 Europe/Madrid
43.38286 -3.22043 Europe/Madrid
43.39788 -6.79217 Europe/Madrid
41.27794 1.97033 Europe/Madrid
41.61667 2.08333 Europe/Madrid
41.63976 2.35739 Europe/Madrid
43.213 -8.69104 Europe/Madrid
41.61667 2.28333 Europe/Madrid
42.26413 -8.78462 Europe/Madrid
41.07479 1.05244 Europe/Madrid
43.29438 -8.34736 Europe/Madrid
43.40744 -3.88498 Europe/Madrid
41.61802 2.66781 Europe/Madrid
41.63333 2.16667 Europe/Madrid
41.35353 -1.64318 Europe/Madrid
42.30506 -1.96521 Europe/Madrid
41.19997 1.5683 Europe/Madrid
42.82562 -1.61671 Europe/Madrid
42.35 -3.7 Europe/Madrid
42.65 -8.9 Europe/Madrid
40.405 -3.87835 Europe/Madrid
41.67419 2.79036 Europe/Madrid
43.26271 -2.92528 Europe/Madrid
43.42088 -2.72152 Europe/Madrid
42.10429 1.84628 Europe/Madrid
40.05 0.06667 Europe/Madrid
40.4165 0.42709 Europe/Madrid
42.00249 -5.67826 Europe/Madrid
40.38641 -5.76341 Europe/Madrid
41.38879 2.15899 Europe/Madrid
42.03565 0.12686 Europe/Madrid
42.80567 -1.67731 Europe/Madrid
42.11667 2.76667 Europe/Madrid
41.79117 0.81094 Europe/Madrid
41.45004 2.24741 Europe/Madrid
40.56688 -3.25992 Europe/Madrid
43.55473 -5.92483 Europe/Madrid
40.65 -4.7 Europe/Madrid
43.30482 -8.50749 Europe/Madrid
40.30076 -3.43722 Europe/Madrid
40.03108 -3.60246 Europe/Madrid
41.67041 -3.6892 Europe/Madrid
40.71308 0.58103 Europe/Madrid
43.21667 -2.73333 Europe/Madrid
42.9 -8.63333 Europe/Madrid
43.34927 -3.0094 Europe/Madrid
40.59711 -3.49743 Europe/Madrid
40.34582 -3.82487 Europe/Madrid
40.54746 -3.64197 Europe/Madrid
41.05 -0.13333 Europe/Madrid
40.48205 -3.35996 Europe/Madrid
41.44163 2.17727 Europe/Madrid
40.47903 -3.66836 Europe/Madrid
27.7567 -15.5787 Atlantic/Canary
28.92313 -13.66579 Atlantic/Canary
35.88933 -5.31979 Africa/Ceuta
41.38896 2.16179 Europe/Madrid
41.41814 2.19933 Europe/Madrid
41.38022 2.17319 Europe/Madrid
38.58055 -0.10321 Europe/Madrid
37.13296 -3.69765 Europe/Madrid
41.64928 -0.90757 Europe/Madrid
41.66124 -0.90169 Europe/Madrid
41.62965 -0.93873 Europe/Madrid
41.64454 -0.93349 Europe/Madrid
43.25347 -2.9161 Europe/Madrid
28.36739 -16.58335 Atlantic/Canary
43.3253 -1.92707 Europe/Madrid
43.2397 -2.8858 Europe/Madrid
41.43806 2.2195 Europe/Madrid
43.51062 -5.8691 Europe/Madrid
36.93068 -3.19565 Europe/Madrid
42.80847 -1.65825 Europe/Madrid
42.81084 -1.66409 Europe/Madrid
42.81483 -1.64898 Europe/Madrid
42.8139 -1.64295 Europe/Madrid
7.93333 38.71667 Africa/Addis_Ababa
6.75 38.41667 Africa/Addis_Ababa
4.88333 38.08333 Africa/Addis_Ababa
11.91667 37.7 Africa/Addis_Ababa
8.45 39.28333 Africa/Addis_Ababa
7.2 35.45 Africa/Addis_Ababa
7.2 38.6 Africa/Addis_Ababa
9.56667 37.1 Africa/Addis_Ababa
5.75 38.91667 Africa/Addis_Ababa
8.91667 38.61667 Africa/Addis_Ababa
12.01667 39.63333 Africa/Addis_Ababa
9.5 35.5 Africa/Addis_Ababa
8.55 39.26667 Africa/Addis_Ababa
8.6 39.11667 Africa/Addis_Ababa
8.3 35.58333 Africa/Addis_Ababa
8.9 39.91667 Africa/Addis_Ababa
9.8 35.1 Africa/Addis_Ababa
13.49667 39.47528 Africa/Addis_Ababa
12.7875 39.54222 Africa/Addis_Ababa
12.50583 39.52278 Africa/Addis_Ababa
7.31667 38.08333 Africa/Addis_Ababa
5.88333 38.98333 Africa/Addis_Ababa
10.71667 39.86667 Africa/Addis_Ababa
11.08155 39.74339 Africa/Addis_Ababa
5.65 36.65 Africa/Addis_Ababa
7.66667 36.83333 Africa/Addis_Ababa
9.35 42.8 Africa/Addis_Ababa
14.10307 38.28289 Africa/Addis_Ababa
9.30944 42.12583 Africa/Addis_Ababa
8.98333 37.85 Africa/Addis_Ababa
12.6 37.46667 Africa/Addis_Ababa
7.01667 39.98333 Africa/Addis_Ababa
8.53333 37.98333 Africa/Addis_Ababa
7.13333 40.7 Africa/Addis_Ababa
9.16667 35.83333 Africa/Addis_Ababa
9.06667 38.5 Africa/Addis_Ababa
8.81667 40.51667 Africa/Addis_Ababa
9.8 38.4 Africa/Addis_Ababa
8.25 34.58333 Africa/Addis_Ababa
10.7 37.26667 Africa/Addis_Ababa
9.8 38.73333 Africa/Addis_Ababa
6.3 36.88333 Africa/Addis_Ababa
11.73611 41.08528 Africa/Addis_Ababa
6.98333 39.18333 Africa/Addis_Ababa
9.59306 41.86611 Africa/Addis_Ababa
6.41667 38.31667 Africa/Addis_Ababa
11.13333 39.63333 Africa/Addis_Ababa
8.53333 34.8 Africa/Addis_Ababa
8.75 38.98333 Africa/Addis_Ababa
11.85 38.01667 Africa/Addis_Ababa
10.35 37.73333 Africa/Addis_Ababa
9.68333 39.53333 Africa/Addis_Ababa
13.15611 37.89806 Africa/Addis_Ababa
8.11667 38.36667 Africa/Addis_Ababa
10.7 37.06667 Africa/Addis_Ababa
7.28333 36.23333 Africa/Addis_Ababa
6.96667 37.86667 Africa/Addis_Ababa
10.45 38.2 Africa/Addis_Ababa
8.9 40.78333 Africa/Addis_Ababa
8.456 36.35302 Africa/Addis_Ababa
11.18333 40.01667 Africa/Addis_Ababa
5.78333 36.56667 Africa/Addis_Ababa
11.59364 37.39077 Africa/Addis_Ababa
7.05 38.46667 Africa/Addis_Ababa
10.06667 34.53333 Africa/Addis_Ababa
9.08333 40.86667 Africa/Addis_Ababa
11.56361 41.43944 Africa/Addis_Ababa
7.1 39.2 Africa/Addis_Ababa
7.06667 37.7 Africa/Addis_Ababa
6.03333 37.55 Africa/Addis_Ababa
14.12448 38.72444 Africa/Addis_Ababa
5.63333 38.23333 Africa/Addis_Ababa
7.85 36.65 Africa/Addis_Ababa
12.11667 37.78333 Africa/Addis_Ababa
9.02497 38.74689 Africa/Addis_Ababa
14.277 39.462 Africa/Addis_Ababa
11.26667 37.48333 Africa/Addis_Ababa
9.98333 39.98333 Africa/Addis_Ababa
61.55632 23.59606 Europe/Helsinki
60.41699 24.31965 Europe/Helsinki
62.31533 27.873 Europe/Helsinki
60.29414 25.04099 Europe/Helsinki
61.26421 24.03122 Europe/Helsinki
63.096 21.61577 Europe/Helsinki
60.80043 21.40841 Europe/Helsinki
60.40368 25.02638 Europe/Helsinki
60.45148 22.26869 Europe/Helsinki
65.84811 24.14662 Europe/Helsinki
61.49911 23.78712 Europe/Helsinki
63.08333 27.66667 Europe/Helsinki
60.37752 25.26906 Europe/Helsinki
62.8 22.83333 Europe/Helsinki
61.8699 28.87999 Europe/Helsinki
60.38333 23.13333 Europe/Helsinki
66.5 25.71667 Europe/Helsinki
60.73769 24.77726 Europe/Helsinki
61.12724 21.51127 Europe/Helsinki
60.48592 22.16895 Europe/Helsinki
64.68333 24.48333 Europe/Helsinki
61.48333 21.78333 Europe/Helsinki
61.46125 23.63228 Europe/Helsinki
65.01236 25.46816 Europe/Helsinki
60.46407 24.8073 Europe/Helsinki
61.46667 23.5 Europe/Helsinki
61.68857 27.27227 Europe/Helsinki
60.63333 25.31667 Europe/Helsinki
60.24859 24.06534 Europe/Helsinki
60.51032 22.46176 Europe/Helsinki
61.31667 23.75 Europe/Helsinki
62.41407 25.95194 Europe/Helsinki
61.05871 28.18871 Europe/Helsinki
60.98267 25.66151 Europe/Helsinki
60.12381 24.43853 Europe/Helsinki
60.9 26.63333 Europe/Helsinki
65.96667 29.18333 Europe/Helsinki
62.89238 27.67703 Europe/Helsinki
60.86667 26.7 Europe/Helsinki
60.46667 26.91667 Europe/Helsinki
63.11418 21.68216 Europe/Helsinki
63.83847 23.13066 Europe/Helsinki
60.40338 25.105 Europe/Helsinki
65.73641 24.56371 Europe/Helsinki
60.51667 26.95 Europe/Helsinki
61.46383 24.07602 Europe/Helsinki
64.22728 27.72846 Europe/Helsinki
60.40724 22.36904 Europe/Helsinki
62.23333 25.73333 Europe/Helsinki
62.60118 29.76316 Europe/Helsinki
60.47369 25.08992 Europe/Helsinki
60.9 24.6 Europe/Helsinki
61.8642 25.19002 Europe/Helsinki
63.67486 22.70256 Europe/Helsinki
61.17185 28.75242 Europe/Helsinki
63.55915 27.19067 Europe/Helsinki
60.63333 24.86667 Europe/Helsinki
61.05 25.43333 Europe/Helsinki
60.16952 24.93545 Europe/Helsinki
61.21667 26.03333 Europe/Helsinki
65.17654 25.35233 Europe/Helsinki
60.56974 27.19794 Europe/Helsinki
60.99596 24.46434 Europe/Helsinki
60.81462 23.62146 Europe/Helsinki
60.2052 24.6522 Europe/Helsinki
60.39233 25.66507 Europe/Helsinki
60.68333 26.83333 Europe/Helsinki
60.29972 22.3004 Europe/Helsinki
-18.14161 178.44149 Pacific/Fiji
-17.8 177.41667 Pacific/Fiji
-16.41667 179.38333 Pacific/Fiji
-51.7 -57.85 Atlantic/Stanley
6.92477 158.16109 Pacific/Pohnpei
62.00973 -6.77164 Atlantic/Faroe
48.71785 2.49338 Europe/Paris
47.8078 7.33702 Europe/Paris
50.70118 3.21812 Europe/Paris
50.67043 3.13382 Europe/Paris
45.36395 5.59209 Europe/Paris
48.78716 2.40332 Europe/Paris
48.73333 4.58333 Europe/Paris
43.46 5.24861 Europe/Paris
48.11776 -1.20577 Europe/Paris
48.67212 2.39318 Europe/Paris
48.80078 2.16181 Europe/Paris
48.8486 2.43769 Europe/Paris
48.831 2.54844 Europe/Paris
49.00875 2.39819 Europe/Paris
45.76667 4.88333 Europe/Paris
48.67103 6.15083 Europe/Paris
48.96203 2.53253 Europe/Paris
48.94208 2.61463 Europe/Paris
44.40854 0.70415 Europe/Paris
48.73219 2.44925 Europe/Paris
48.73684 2.40081 Europe/Paris
48.93935 2.31478 Europe/Paris
44.77327 -0.5442 Europe/Paris
48.88333 2.5 Europe/Paris
48.7939 2.35992 Europe/Paris
45.98333 4.71667 Europe/Paris
45.6125 5.14558 Europe/Paris
48.70291 2.41357 Europe/Paris
47.21667 2.08333 Europe/Paris
45.51667 4.86667 Europe/Paris
46.11667 3.41667 Europe/Paris
47.63333 6.16667 Europe/Paris
47.16869 -1.46929 Europe/Paris
48.8 2.13333 Europe/Paris
48.74565 2.26796 Europe/Paris
49.09292 1.46332 Europe/Paris
48.97388 1.9648 Europe/Paris
49.16667 5.38333 Europe/Paris
45.70254 4.87147 Europe/Paris
47.8 1.06667 Europe/Paris
43.72162 7.11232 Europe/Paris
48.78198 2.19395 Europe/Paris
49.03333 2.03333 Europe/Paris
45.7818 4.92085 Europe/Paris
48.82345 2.29025 Europe/Paris
47.66667 -2.75 Europe/Paris
48.65 6.18333 Europe/Paris
43.57803 7.05451 Europe/Paris
50.35 3.53333 Europe/Paris
44.93333 4.9 Europe/Paris
45.26667 1.76667 Europe/Paris
48.3 4.08333 Europe/Paris
48.94956 2.5684 Europe/Paris
48.77413 2.01781 Europe/Paris
47.38333 0.68333 Europe/Paris
43.58872 1.31922 Europe/Paris
49.63828 -1.56639 Europe/Paris
50.72391 3.16117 Europe/Paris
43.60426 1.44367 Europe/Paris
43.11667 5.93333 Europe/Paris
48.68333 5.9 Europe/Paris
48.85 2.65 Europe/Paris
46.36667 6.48333 Europe/Paris
49.36667 6.16667 Europe/Paris
48.76496 2.3961 Europe/Paris
49.65607 3.30107 Europe/Paris
49.02542 2.21691 Europe/Paris
45.75499 4.78812 Europe/Paris
43.23333 0.08333 Europe/Paris
44.80477 -0.59543 Europe/Paris
48.87143 2.22929 Europe/Paris
48.76872 2.53474 Europe/Paris
48.58342 7.74296 Europe/Paris
48.95 2.38333 Europe/Paris
49.40972 1.09006 Europe/Paris
44.01023 4.87381 Europe/Paris
48.98813 2.30156 Europe/Paris
49.38167 3.32361 Europe/Paris
43.1 5.85 Europe/Paris
50.36159 3.13113 Europe/Paris
45.88549 6.08831 Europe/Paris
48.82292 2.21757 Europe/Paris
48.94472 2.52746 Europe/Paris
43.40176 3.6966 Europe/Paris
48.2 3.28333 Europe/Paris
49.2 2.58333 Europe/Paris
48.26667 7.45 Europe/Paris
49.7 4.95 Europe/Paris
48.60749 7.74931 Europe/Paris
48.77644 2.29026 Europe/Paris
48.67678 2.34835 Europe/Paris
48.57409 2.58287 Europe/Paris
47.26667 -0.08333 Europe/Paris
48.9482 2.19169 Europe/Paris
49.10995 7.06747 Europe/Paris
49 2.38333 Europe/Paris
47.95013 1.87601 Europe/Paris
48.96667 2.25 Europe/Paris
43.11784 5.80006 Europe/Paris
43.64074 5.09545 Europe/Paris
45.94423 6.63162 Europe/Paris
47.20768 -1.50332 Europe/Paris
43.42332 6.7735 Europe/Paris
49.84889 3.28757 Europe/Paris
45.69612 4.93892 Europe/Paris
51.03116 2.33984 Europe/Paris
47.38623 0.74849 Europe/Paris
49.04353 2.12134 Europe/Paris
48.9 2.33333 Europe/Paris
50.75 2.25 Europe/Paris
47.28333 -2.2 Europe/Paris
48.63479 2.30831 Europe/Paris
44.89692 -0.72136 Europe/Paris
43.45214 5.86219 Europe/Paris
48.79394 2.49323 Europe/Paris
45.16528 5.76337 Europe/Paris
48.83864 2.41579 Europe/Paris
48.65 -2.01667 Europe/Paris
47.58836 7.56247 Europe/Paris
49.11624 -1.09031 Europe/Paris
49.01667 2.25 Europe/Paris
46.7306 4.50083 Europe/Paris
43.67097 7.17606 Europe/Paris
47.91127 1.86483 Europe/Paris
47.91303 1.97705 Europe/Paris
47.21765 -1.64841 Europe/Paris
48.9735 2.28729 Europe/Paris
48.9 2.08333 Europe/Paris
45.69558 4.7934 Europe/Paris
45.7088 4.86057 Europe/Paris
49.37794 1.10467 Europe/Paris
45.43333 4.4 Europe/Paris
45.75 -0.63333 Europe/Paris
48.64682 2.31965 Europe/Paris
45.73692 4.79688 Europe/Paris
45.23333 5.68333 Europe/Paris
48.63333 4.95 Europe/Paris
48.28333 6.95 Europe/Paris
48.93333 2.36667 Europe/Paris
47.4 0.66667 Europe/Paris
48.79865 2.06814 Europe/Paris
48.84598 2.20289 Europe/Paris
45.47365 4.52541 Europe/Paris
48.51667 -2.78333 Europe/Paris
49.1027 6.70064 Europe/Paris
47.36357 0.73993 Europe/Paris
50.44718 3.43076 Europe/Paris
48.8765 2.18967 Europe/Paris
45.62846 -1.0281 Europe/Paris
49.44313 1.09932 Europe/Paris
50.69421 3.17456 Europe/Paris
48.87017 2.4991 Europe/Paris
50.59883 3.09056 Europe/Paris
47.36667 1.75 Europe/Paris
45.04639 5.0562 Europe/Paris
48.8854 2.43482 Europe/Paris
48.79159 2.64747 Europe/Paris
44.35258 2.57338 Europe/Paris
45.93333 -0.98333 Europe/Paris
46.03333 4.06667 Europe/Paris
48.6511 2.41406 Europe/Paris
45.9 3.11667 Europe/Paris
45.81667 4.9 Europe/Paris
47.18144 -1.54965 Europe/Paris
48.11198 -1.67429 Europe/Paris
49.25 4.03333 Europe/Paris
48.65 1.83333 Europe/Paris
48 -4.1 Europe/Paris
48.88341 2.23894 Europe/Paris
43.40657 4.9809 Europe/Paris
49.05 2.1 Europe/Paris
48.06667 -2.98333 Europe/Paris
46.90347 6.35542 Europe/Paris
48.90786 6.0589 Europe/Paris
46.58333 0.33333 Europe/Paris
48.92902 2.04952 Europe/Paris
47.73333 -3.43333 Europe/Paris
48.82319 1.9541 Europe/Paris
43.56566 1.29749 Europe/Paris
48.96691 2.36104 Europe/Paris
44.81011 -0.64129 Europe/Paris
43.69252 5.50285 Europe/Paris
42.69764 2.89541 Europe/Paris
45.18333 0.71667 Europe/Paris
43.3 -0.36667 Europe/Paris
48.85341 2.3488 Europe/Paris
48.89437 2.40935 Europe/Paris
43.11667 1.6 Europe/Paris
48.71667 2.25 Europe/Paris
48.76699 2.66871 Europe/Paris
46.25917 5.65727 Europe/Paris
50.70535 1.5897 Europe/Paris
45.71774 4.80382 Europe/Paris
49.05937 2.06183 Europe/Paris
47.27095 -1.6219 Europe/Paris
48.69572 2.18727 Europe/Paris
48.74792 2.39252 Europe/Paris
47.90289 1.90389 Europe/Paris
44.1392 4.80791 Europe/Paris
47.86219 1.8991 Europe/Paris
49.62612 -1.64349 Europe/Paris
49.58333 3 Europe/Paris
48.89148 2.46451 Europe/Paris
48.84979 2.56266 Europe/Paris
48.84868 2.62435 Europe/Paris
49.27158 2.47074 Europe/Paris
48.83333 2.48333 Europe/Paris
46.32313 -0.45877 Europe/Paris
43.83333 4.35 Europe/Paris
43.70313 7.26608 Europe/Paris
46.98956 3.159 Europe/Paris
48.88333 2.26667 Europe/Paris
48.85373 2.54903 Europe/Paris
48.86342 2.506 Europe/Paris
43.18333 3 Europe/Paris
47.21725 -1.55336 Europe/Paris
48.89198 2.20675 Europe/Paris
48.68333 6.2 Europe/Paris
43.46667 1.35 Europe/Paris
47.75 7.33333 Europe/Paris
46.56667 3.33333 Europe/Paris
43.59873 7.00434 Europe/Paris
48.6618 2.35338 Europe/Paris
48.58333 -3.83333 Europe/Paris
49.46307 1.09364 Europe/Paris
48.8162 2.31394 Europe/Paris
48.86415 2.44322 Europe/Paris
43.61092 3.87723 Europe/Paris
48.98826 2.3434 Europe/Paris
46.33333 2.6 Europe/Paris
49.54518 0.18769 Europe/Paris
49.09435 6.15167 Europe/Paris
48.98202 2.20035 Europe/Paris
48.76636 2.03405 Europe/Paris
48.70543 2.45039 Europe/Paris
48.8982 2.57913 Europe/Paris
48.90924 2.13754 Europe/Paris
48.38333 2.95 Europe/Paris
44.55838 4.75094 Europe/Paris
43.89022 -0.49713 Europe/Paris
46.66667 4.36667 Europe/Paris
45.6 4.05 Europe/Paris
47.51667 6.8 Europe/Paris
44.01667 1.35 Europe/Paris
48 2.75 Europe/Paris
50.63333 3.11667 Europe/Paris
48.62605 2.60125 Europe/Paris
48.98333 2.61667 Europe/Paris
43.58666 5.00464 Europe/Paris
44.1 3.08333 Europe/Paris
45.76667 5 Europe/Paris
45.21988 5.79007 Europe/Paris
48.81381 2.235 Europe/Paris
49.11911 6.17269 Europe/Paris
44.83248 -0.63381 Europe/Paris
43.77649 7.50435 Europe/Paris
48.53333 2.66667 Europe/Paris
48.95 2.86667 Europe/Paris
48.3 -0.61667 Europe/Paris
48.76486 1.92923 Europe/Paris
43.6181 4.00739 Europe/Paris
50.27875 3.97267 Europe/Paris
48.72692 2.28301 Europe/Paris
43.40735 5.05526 Europe/Paris
43.29695 5.38107 Europe/Paris
44.5 0.16667 Europe/Paris
48.86667 2.08333 Europe/Paris
43.41604 5.21453 Europe/Paris
50.66667 3.08333 Europe/Paris
48.97374 1.70253 Europe/Paris
48.98333 1.71667 Europe/Paris
43.82883 5.78688 Europe/Paris
43.55 6.93333 Europe/Paris
48.81999 2.29998 Europe/Paris
48.95264 2.14521 Europe/Paris
48.81171 2.43945 Europe/Paris
46.3 4.83333 Europe/Paris
45.74846 4.84671 Europe/Paris
48.6 6.5 Europe/Paris
43.67778 4.13611 Europe/Paris
48.4369 1.46359 Europe/Paris
49.21667 1.16667 Europe/Paris
43.1 -0.05 Europe/Paris
44.87495 -0.51782 Europe/Paris
47.75 -3.36667 Europe/Paris
50.60982 3.01874 Europe/Paris
46.66667 5.55 Europe/Paris
48.69503 2.30736 Europe/Paris
50.64358 2.98715 Europe/Paris
48.83541 2.62998 Europe/Paris
48.9193 2.54305 Europe/Paris
43.91915 5.05092 Europe/Paris
49.1466 0.22925 Europe/Paris
48.55752 7.68253 Europe/Paris
45.83153 1.2578 Europe/Paris
48.7448 2.48705 Europe/Paris
48.99553 1.74081 Europe/Paris
50.63297 3.05858 Europe/Paris
50.4198 2.78068 Europe/Paris
44.91667 -0.23333 Europe/Paris
48.78333 2.33333 Europe/Paris
48.89281 2.13308 Europe/Paris
48.89389 2.28864 Europe/Paris
46.5 -1.78333 Europe/Paris
43.41028 5.31306 Europe/Paris
48.90683 2.50648 Europe/Paris
48.99173 1.90972 Europe/Paris
48.87992 2.42057 Europe/Paris
46.86667 -1.01667 Europe/Paris
48.82206 1.98677 Europe/Paris
45.04366 3.88523 Europe/Paris
48.88549 2.40422 Europe/Paris
43.96119 4.86008 Europe/Paris
48.81074 2.57363 Europe/Paris
48.78889 2.27078 Europe/Paris
49.41338 1.06155 Europe/Paris
48.85 2.5 Europe/Paris
48.89317 2.10371 Europe/Paris
50.43302 2.82791 Europe/Paris
48.53157 2.62829 Europe/Paris
48 0.2 Europe/Paris
48.81471 2.36073 Europe/Paris
49.4938 0.10767 Europe/Paris
49.40076 1.04582 Europe/Paris
46.80714 4.41632 Europe/Paris
48.8222 2.12213 Europe/Paris
43.57662 7.01912 Europe/Paris
44.866 -0.59411 Europe/Paris
48.93872 2.46138 Europe/Paris
48.68315 6.148 Europe/Paris
43.13741 5.98314 Europe/Paris
48.06667 -0.76667 Europe/Paris
43.56752 3.9046 Europe/Paris
44.62875 -1.14059 Europe/Paris
43.09818 5.88472 Europe/Paris
46.66667 -1.43333 Europe/Paris
46.16667 -1.15 Europe/Paris
49.56667 3.61667 Europe/Paris
48.73333 -3.46667 Europe/Paris
47.76132 -3.33965 Europe/Paris
48.45252 -4.25252 Europe/Paris
50.65 3.03333 Europe/Paris
50.64603 3.07585 Europe/Paris
48.86667 2.71667 Europe/Paris
48.90472 2.2469 Europe/Paris
43.12277 6.00745 Europe/Paris
47.69815 -0.07553 Europe/Paris
43.14981 6.07425 Europe/Paris
48.92805 2.39627 Europe/Paris
43.17476 5.60449 Europe/Paris
47.29584 -1.55309 Europe/Paris
48.85029 2.14523 Europe/Paris
49.01068 2.04028 Europe/Paris
47.35223 0.66906 Europe/Paris
48.82538 2.47458 Europe/Paris
48.81568 2.38487 Europe/Paris
43.51667 4.98333 Europe/Paris
48.82104 2.27718 Europe/Paris
45.55 3.25 Europe/Paris
47.78088 7.34662 Europe/Paris
48.52894 7.71523 Europe/Paris
43.11954 6.13897 Europe/Paris
48.92161 2.19263 Europe/Paris
49.21088 -0.30653 Europe/Paris
48.98994 2.1699 Europe/Paris
50.41359 2.96485 Europe/Paris
50.65256 3.18681 Europe/Paris
50.72374 2.53729 Europe/Paris
49.33428 6.05268 Europe/Paris
50.25077 3.92143 Europe/Paris
49.35571 6.1926 Europe/Paris
50.60826 2.99143 Europe/Paris
50.78628 3.12611 Europe/Paris
48.81557 7.79051 Europe/Paris
48.77334 2.07393 Europe/Paris
44.63333 -1.06667 Europe/Paris
46.16667 1.86667 Europe/Paris
47.32911 -2.42829 Europe/Paris
48.65412 2.39343 Europe/Paris
45.16667 5.71667 Europe/Paris
43.66667 6.91667 Europe/Paris
51.0154 2.29975 Europe/Paris
44.77262 -0.61393 Europe/Paris
49.01367 2.46595 Europe/Paris
48.98693 2.44892 Europe/Paris
45.58415 4.77256 Europe/Paris
48.68333 2.13333 Europe/Paris
47.69332 2.63094 Europe/Paris
48.81294 2.3417 Europe/Paris
48.93333 2.3 Europe/Paris
48.96791 2.39781 Europe/Paris
43.45491 5.46913 Europe/Paris
48.84226 2.18232 Europe/Paris
44.56667 6.08333 Europe/Paris
48.88333 2.53333 Europe/Paris
43.44848 3.754 Europe/Paris
48.75568 2.32241 Europe/Paris
43.43286 6.73524 Europe/Paris
48.98333 2.23333 Europe/Paris
48.35 -1.2 Europe/Paris
43.43774 4.94457 Europe/Paris
49.18333 6.9 Europe/Paris
48.85442 2.48268 Europe/Paris
46.46667 -0.81667 Europe/Paris
48.79325 2.29275 Europe/Paris
48.4 2.7 Europe/Paris
45.19839 5.68265 Europe/Paris
44.83238 -0.51411 Europe/Paris
47.93328 1.91811 Europe/Paris
48.73722 -0.57466 Europe/Paris
45.38732 4.29074 Europe/Paris
49.75787 0.37457 Europe/Paris
50.58333 3.06667 Europe/Paris
44.88352 -0.64686 Europe/Paris
48.63333 2.45 Europe/Paris
49.02414 1.15082 Europe/Paris
48.43333 2.15 Europe/Paris
47.29221 -2.36395 Europe/Paris
48.99004 2.25804 Europe/Paris
49.01667 2.1 Europe/Paris
49.64868 -1.65306 Europe/Paris
48.9535 2.31514 Europe/Paris
48.18333 6.45 Europe/Paris
49.05 3.95 Europe/Paris
49.28669 1.00288 Europe/Paris
48.7842 1.9552 Europe/Paris
45.77726 4.77481 Europe/Paris
45.1439 5.72883 Europe/Paris
48.99712 2.28249 Europe/Paris
51.05 2.36667 Europe/Paris
48.73333 1.36667 Europe/Paris
48.68466 2.41418 Europe/Paris
48.92578 2.44533 Europe/Paris
43.53333 6.46667 Europe/Paris
48.09542 -4.32904 Europe/Paris
50.36667 3.06667 Europe/Paris
49.02782 2.32638 Europe/Paris
47.1 5.5 Europe/Paris
47.31667 5.01667 Europe/Paris
44.09252 6.23199 Europe/Paris
49.93333 1.08333 Europe/Paris
48.97674 2.32722 Europe/Paris
50.3293 3.3943 Europe/Paris
45.76899 4.97277 Europe/Paris
43.71032 -1.05366 Europe/Paris
48.51667 2.65 Europe/Paris
43.53635 1.34428 Europe/Paris
50.67846 3.1493 Europe/Paris
48.78333 2.46667 Europe/Paris
49.23359 2.88807 Europe/Paris
49.25672 2.48477 Europe/Paris
45.9 6.1 Europe/Paris
45.73543 3.19608 Europe/Paris
48.89672 2.25666 Europe/Paris
48.81451 3.08498 Europe/Paris
47.21508 -1.72171 Europe/Paris
51.02288 2.39359 Europe/Paris
48.97111 2.20491 Europe/Paris
48.60603 2.48757 Europe/Paris
49.00158 2.09694 Europe/Paris
47.87502 -3.92245 Europe/Paris
49.41794 2.82606 Europe/Paris
48.66497 2.56957 Europe/Paris
48.79813 2.60676 Europe/Paris
43.61247 1.33278 Europe/Paris
48.91882 2.25404 Europe/Paris
48.08333 7.36667 Europe/Paris
45.7 -0.33333 Europe/Paris
46.06251 6.57497 Europe/Paris
48.9102 2.55324 Europe/Paris
48.90018 2.30952 Europe/Paris
45.77966 3.08628 Europe/Paris
48.80299 2.26692 Europe/Paris
47.06667 -0.88333 Europe/Paris
48.76846 2.41874 Europe/Paris
48.71489 2.31638 Europe/Paris
48.76476 2.3503 Europe/Paris
49.63984 -1.61636 Europe/Paris
47.29323 5.00457 Europe/Paris
48.79702 2.54046 Europe/Paris
48.88109 2.59295 Europe/Paris
48.80565 2.18864 Europe/Paris
48.11667 5.13333 Europe/Paris
48.8898 2.15863 Europe/Paris
48.8024 2.29346 Europe/Paris
48.76507 2.26655 Europe/Paris
46.8 0.53333 Europe/Paris
49.05 3.4 Europe/Paris
46.81667 1.7 Europe/Paris
48.08333 1.33333 Europe/Paris
48.44685 1.48925 Europe/Paris
49.76667 4.71667 Europe/Paris
48.82209 2.41217 Europe/Paris
48.85 2.6 Europe/Paris
48.81642 2.49366 Europe/Paris
45.56667 5.93333 Europe/Paris
45.77559 3.0631 Europe/Paris
46.78333 4.85 Europe/Paris
48.95393 4.36724 Europe/Paris
46.84363 -1.8749 Europe/Paris
44.74345 -0.67905 Europe/Paris
48.1212 -1.603 Europe/Paris
49.03645 2.07613 Europe/Paris
44.8559 -0.51839 Europe/Paris
49.55858 1.62803 Europe/Paris
43.91667 5.28333 Europe/Paris
43.83125 5.03586 Europe/Paris
43.6 2.25 Europe/Paris
43.63333 3.9 Europe/Paris
50.49235 2.95815 Europe/Paris
48.94952 2.04068 Europe/Paris
47.29822 -1.49024 Europe/Paris
44.05 5.05 Europe/Paris
43.21667 2.35 Europe/Paris
49.44065 1.02459 Europe/Paris
43.55135 7.01275 Europe/Paris
50.16667 3.23333 Europe/Paris
45.79041 4.84112 Europe/Paris
50.9581 1.85205 Europe/Paris
44.4491 1.43663 Europe/Paris
43.6646 7.15339 Europe/Paris
49.18585 -0.35912 Europe/Paris
48.79632 2.33661 Europe/Paris
48.83811 2.52488 Europe/Paris
48.6942 2.49223 Europe/Paris
50.48333 2.55 Europe/Paris
45.73333 4.91667 Europe/Paris
45.15 1.53333 Europe/Paris
43.4 6.06667 Europe/Paris
48.68333 2.61667 Europe/Paris
48.61135 2.30593 Europe/Paris
48.4 -4.48333 Europe/Paris
46.85 -0.48333 Europe/Paris
45.58333 5.28333 Europe/Paris
44.95601 4.91023 Europe/Paris
48.77888 2.31781 Europe/Paris
47.08333 2.4 Europe/Paris
46.20574 5.2258 Europe/Paris
50.71667 1.61667 Europe/Paris
48.83333 2.25 Europe/Paris
47.17762 -1.62143 Europe/Paris
44.84044 -0.5805 Europe/Paris
48.7695 2.4793 Europe/Paris
48.9018 2.48931 Europe/Paris
48.7515 2.51163 Europe/Paris
48.91936 2.27484 Europe/Paris
48.9 2.45 Europe/Paris
47.58333 1.33333 Europe/Paris
44.91248 -0.63663 Europe/Paris
43.63276 1.39399 Europe/Paris
48.61612 7.75343 Europe/Paris
43.48012 -1.55558 Europe/Paris
48.92426 2.2128 Europe/Paris
43.34762 3.21899 Europe/Paris
50.53333 2.63333 Europe/Paris
47.24878 6.01815 Europe/Paris
44.85 0.48333 Europe/Paris
50.4 1.6 Europe/Paris
50.40704 1.56446 Europe/Paris
47.63333 6.86667 Europe/Paris
44.80845 -0.54809 Europe/Paris
49.43333 2.08333 Europe/Paris
47.03333 4.83333 Europe/Paris
43.48333 -1.48333 Europe/Paris
49.27732 -0.7039 Europe/Paris
42.70278 9.45 Europe/Paris
48.78333 5.16667 Europe/Paris
43.61111 1.49944 Europe/Paris
44.1599 4.61776 Europe/Paris
48.86667 2.41667 Europe/Paris
48.79564 2.30796 Europe/Paris
48.4022 2.72022 Europe/Paris
50.41038 2.83053 Europe/Paris
43.94834 4.80892 Europe/Paris
47.8 3.56667 Europe/Paris
46.95 4.3 Europe/Paris
44.91667 2.45 Europe/Paris
48.93814 2.49402 Europe/Paris
47.48327 6.85341 Europe/Paris
43.65 0.58333 Europe/Paris
48.91667 2.38333 Europe/Paris
43.29276 5.57067 Europe/Paris
48.70522 2.39147 Europe/Paris
48.91667 2.28333 Europe/Paris
50.29301 2.78186 Europe/Paris
50.68568 2.88214 Europe/Paris
43.67681 4.63031 Europe/Paris
48.95 2.25 Europe/Paris
48.75 -0.01667 Europe/Paris
48.79993 2.33256 Europe/Paris
48.75 2.3 Europe/Paris
43.56241 7.12777 Europe/Paris
45.2397 4.67075 Europe/Paris
46.19439 6.23775 Europe/Paris
45.91972 6.14393 Europe/Paris
45.9 6.11667 Europe/Paris
45.65 0.15 Europe/Paris
43.47664 -1.51346 Europe/Paris
47.46667 -0.55 Europe/Paris
49.9 2.3 Europe/Paris
43.33573 5.48201 Europe/Paris
48.80575 2.4204 Europe/Paris
44.13333 4.08333 Europe/Paris
48.43333 0.08333 Europe/Paris
43.9298 2.148 Europe/Paris
45.67452 6.39061 Europe/Paris
41.92723 8.73462 Europe/Paris
45.68338 5.92241 Europe/Paris
43.5283 5.44973 Europe/Paris
44.2 0.63333 Europe/Paris
43.31083 3.47583 Europe/Paris
48.96116 2.06882 Europe/Paris
50.1 1.83333 Europe/Paris
50.61669 3.16664 Europe/Paris
48.68167 2.16944 Europe/Paris
45.58611 5.27361 Europe/Paris
-2.85 11.03333 Africa/Libreville
-0.71933 8.78151 Africa/Libreville
1.5995 11.57933 Africa/Libreville
-1.86846 11.05594 Africa/Libreville
-1.55746 13.21775 Africa/Libreville
0.3925 9.45365 Africa/Libreville
-0.7001 10.24055 Africa/Libreville
-1.13032 12.47359 Africa/Libreville
-1.63333 13.58357 Africa/Libreville
53.95763 -1.08271 Europe/London
50.94159 -2.63211 Europe/London
53.04664 -2.99132 Europe/London
50.81448 -0.37126 Europe/London
53.30182 -1.12404 Europe/London
54.6425 -3.54413 Europe/London
52.18935 -2.22001 Europe/London
53.52189 -1.39698 Europe/London
52.58547 -2.12296 Europe/London
51.4112 -0.83565 Europe/London
51.31903 -0.55893 Europe/London
51.7836 -1.4854 Europe/London
51.80007 0.64038 Europe/London
55.76667 -3.91667 Europe/London
52.66622 0.15938 Europe/London
53.18333 -2.51667 Europe/London
51.48333 -0.6 Europe/London
51.06513 -1.3187 Europe/London
53.32803 -2.23148 Europe/London
52.58128 -1.09248 Europe/London
53.53333 -2.61667 Europe/London
53.3618 -2.73406 Europe/London
51.61101 0.52331 Europe/London
51.3607 1.0257 Europe/London
55.03973 -1.44713 Europe/London
54.54897 -3.58412 Europe/London
53.55 -2.3 Europe/London
54.94561 -1.67635 Europe/London
50.61136 -2.45334 Europe/London
51.34603 -2.97665 Europe/London
53.54899 -2.52464 Europe/London
52.51868 -1.9945 Europe/London
52.92979 -1.12537 Europe/London
51.80174 -0.20691 Europe/London
52.7 -2.51667 Europe/London
52.30273 -0.69446 Europe/London
53.50291 -1.3458 Europe/London
51.65531 -0.39602 Europe/London
50.88067 -1.0304 Europe/London
54.9 -1.51667 Europe/London
52.28333 -1.58333 Europe/London
53.39254 -2.58024 Europe/London
51.20434 -2.17873 Europe/London
51.81265 -0.02832 Europe/London
51.687 -0.00421 Europe/London
52.58528 -1.98396 Europe/London
54.99111 -1.53397 Europe/London
53.42324 -3.06497 Europe/London
53.51667 -2.4 Europe/London
53.68331 -1.49768 Europe/London
53.44852 -2.35419 Europe/London
50.96948 0.09589 Europe/London
55.01788 -1.42559 Europe/London
53.51393 -2.46754 Europe/London
50.26526 -5.05436 Europe/London
51.31889 -2.20861 Europe/London
50.91877 -1.49037 Europe/London
50.46384 -3.51434 Europe/London
51.19242 0.27532 Europe/London
50.9 -3.48333 Europe/London
54.53333 -1.3 Europe/London
52.41667 0.75 Europe/London
51.40366 -1.26049 Europe/London
50.54768 -3.49637 Europe/London
51.02111 -3.10472 Europe/London
52.63399 -1.69587 Europe/London
51.35045 -1.1285 Europe/London
51.55797 -1.78116 Europe/London
51.62079 -3.94323 Europe/London
51.44713 0.31028 Europe/London
52.774 -1.55744 Europe/London
53.12542 -1.26135 Europe/London
52.56667 -1.81667 Europe/London
51.35 -0.2 Europe/London
54.90465 -1.38222 Europe/London
51.40606 -0.4137 Europe/London
51.75 -2.2 Europe/London
51.39594 0.49361 Europe/London
53.45 -2.31667 Europe/London
52.19166 -1.70734 Europe/London
52.18893 0.99774 Europe/London
52.45608 -2.14317 Europe/London
53.00415 -2.18538 Europe/London
54.56848 -1.3187 Europe/London
53.40979 -2.15761 Europe/London
56.11903 -3.93682 Europe/London
51.90224 -0.20256 Europe/London
53.26667 -1.35 Europe/London
52.65 -0.48333 Europe/London
53.48414 -2.05908 Europe/London
51.43092 -0.50606 Europe/London
52.80521 -2.11636 Europe/London
54.6988 -1.60229 Europe/London
52.78709 -0.15141 Europe/London
54.99859 -1.4323 Europe/London
53.64779 -3.00648 Europe/London
51.50799 0.28333 Europe/London
51.53782 0.71433 Europe/London
53.59709 -1.28034 Europe/London
51.55295 0.55962 Europe/London
50.90395 -1.40428 Europe/London
52.41426 -1.78094 Europe/London
51.50949 -0.59541 Europe/London
52.99826 -0.40941 Europe/London
53.55024 -2.77348 Europe/London
53.14362 0.3363 Europe/London
51.34128 0.73282 Europe/London
52.71009 -2.75208 Europe/London
51.3334 0.17801 Europe/London
53.83333 -1.76667 Europe/London
53.38297 -1.4659 Europe/London
51.26997 0.19278 Europe/London
53.78333 -1.06667 Europe/London
54.83903 -1.34575 Europe/London
50.77141 0.10268 Europe/London
53.57905 -0.65437 Europe/London
54.27966 -0.40443 Europe/London
50.65158 -1.16103 Europe/London
53.14515 -2.36251 Europe/London
51.06931 -1.79569 Europe/London
53.48771 -2.29042 Europe/London
53.42519 -2.32443 Europe/London
52.21667 -0.26667 Europe/London
53.45 -2.73333 Europe/London
50.33833 -4.76583 Europe/London
56.33871 -2.79902 Europe/London
51.75 -0.33333 Europe/London
52.61667 -2.35 Europe/London
50.72999 -1.1621 Europe/London
52.29139 -0.59923 Europe/London
53.34174 -2.73124 Europe/London
52.7593 -1.93694 Europe/London
52.37092 -1.26417 Europe/London
53.56507 -2.12267 Europe/London
51.13245 0.26333 Europe/London
50.80984 -0.05939 Europe/London
53.43012 -1.35678 Europe/London
50.98906 -1.49989 Europe/London
51.58198 0.70673 Europe/London
51.38764 0.50546 Europe/London
53.61766 -2.1552 Europe/London
51.60799 -3.10081 Europe/London
54.13521 -1.52122 Europe/London
53.03333 -1.4 Europe/London
53.31955 -3.48862 Europe/London
51.65896 -3.44885 Europe/London
55.87197 -4.39253 Europe/London
51.23758 -0.2078 Europe/London
51.24048 -0.17044 Europe/London
52.3065 -1.94569 Europe/London
54.61657 -1.05999 Europe/London
51.45625 -0.97113 Europe/London
53.70076 -2.28442 Europe/London
53.46062 -1.34437 Europe/London
51.33576 1.41552 Europe/London
53.64789 -2.31683 Europe/London
53.55 -2.33333 Europe/London
53.79538 -1.66134 Europe/London
55.48333 -4.61667 Europe/London
53.53333 -2.28333 Europe/London
53.76667 -2.71667 Europe/London
53.33748 -3.40776 Europe/London
53.42948 -2.80031 Europe/London
53.83333 -2.98333 Europe/London
51.69353 -0.17835 Europe/London
50.79899 -1.09125 Europe/London
50.84335 -0.21544 Europe/London
51.48199 -2.76973 Europe/London
51.47903 -3.70362 Europe/London
55.93464 -4.6895 Europe/London
50.71667 -2 Europe/London
51.6021 -3.34211 Europe/London
51.70111 -3.04444 Europe/London
53.69107 -1.31269 Europe/London
55.9905 -3.70737 Europe/London
50.37153 -4.14305 Europe/London
54.76032 -1.33649 Europe/London
57.50584 -1.79806 Europe/London
52.57364 -0.24777 Europe/London
56.39522 -3.43139 Europe/London
50.11861 -5.53715 Europe/London
55.83166 -3.22423 Europe/London
51.43942 -3.17609 Europe/London
55.83173 -4.43254 Europe/London
50.43565 -3.56789 Europe/London
51.75222 -1.25596 Europe/London
52.86195 -3.05497 Europe/London
53.67978 -1.58006 Europe/London
53.56685 -2.88178 Europe/London
54.6 -7.3 Europe/London
53.54051 -2.1183 Europe/London
52.60621 -1.08354 Europe/London
52.52323 -1.46523 Europe/London
52.9536 -1.15047 Europe/London
52.62783 1.29834 Europe/London
53.26138 -2.51225 Europe/London
55.01646 -1.44925 Europe/London
52.25 -0.88333 Europe/London
54.33901 -1.43243 Europe/London
54.59236 -5.69092 Europe/London
54.65983 -5.90858 Europe/London
55.77334 -4.33339 Europe/London
53.45 -2.6 Europe/London
54.61842 -1.5719 Europe/London
50.52858 -3.61186 Europe/London
54.17841 -6.33739 Europe/London
50.41317 -5.08186 Europe/London
52.08731 -0.72218 Europe/London
51.58774 -2.99835 Europe/London
50.70146 -1.29124 Europe/London
52.24467 0.40418 Europe/London
54.97328 -1.61396 Europe/London
53 -2.23333 Europe/London
51.40033 -1.32059 Europe/London
54.9876 -1.74415 Europe/London
53.06667 -0.81667 Europe/London
53.83333 -2.2 Europe/London
51.66317 -3.80443 Europe/London
51.43239 -2.75847 Europe/London
55.9417 -3.04991 Europe/London
55.78924 -3.99187 Europe/London
53.74013 -1.59877 Europe/London
54.06835 -2.86108 Europe/London
53.67343 -1.69636 Europe/London
52.04172 -0.75583 Europe/London
53.55 -2.2 Europe/London
54.57623 -1.23483 Europe/London
53.49389 -1.29243 Europe/London
51.74794 -3.37779 Europe/London
52.76588 -0.88693 Europe/London
53.39452 -2.06292 Europe/London
51.56933 -0.77415 Europe/London
52.4776 -0.92053 Europe/London
51.38132 1.38617 Europe/London
52.55131 0.08828 Europe/London
53.16495 -1.19384 Europe/London
53.13333 -1.2 Europe/London
51.4878 -2.50403 Europe/London
53.48095 -2.23743 Europe/London
53.41667 -1.2 Europe/London
51.7311 0.67463 Europe/London
51.26667 0.51667 Europe/London
51.52279 -0.71986 Europe/London
51.60926 -3.65823 Europe/London
53.26023 -2.12564 Europe/London
51.87967 -0.41748 Europe/London
52.4752 1.75159 Europe/London
53.36664 -0.00438 Europe/London
52.76667 -1.2 Europe/London
51.3969 0.30212 Europe/London
52.89855 -1.27136 Europe/London
54.99721 -7.30917 Europe/London
54.9981 -7.30934 Europe/London
51.51279 -0.09184 Europe/London
51.50853 -0.12574 Europe/London
53.72947 -1.49697 Europe/London
51.68195 -4.16191 Europe/London
53.32498 -3.83148 Europe/London
55.90288 -3.52261 Europe/London
53.41058 -2.97794 Europe/London
50.80691 -0.53782 Europe/London
53.46993 -2.99809 Europe/London
54.52337 -6.03527 Europe/London
53.22683 -0.53792 Europe/London
52.68154 -1.82549 Europe/London
53.69786 -2.68758 Europe/London
50.87363 0.01133 Europe/London
51.97944 -0.2284 Europe/London
51.91722 -0.65802 Europe/London
53.48333 -2.51667 Europe/London
52.6386 -1.13169 Europe/London
53.10434 -2.02207 Europe/London
53.79648 -1.54785 Europe/London
51.29593 -0.3259 Europe/London
52.3 -1.53333 Europe/London
54.85 -5.81667 Europe/London
55.73333 -3.96667 Europe/London
54.06667 -2.83333 Europe/London
55.93933 -4.15262 Europe/London
56.10982 -3.16149 Europe/London
53.09982 -1.24379 Europe/London
53.48333 -2.9 Europe/London
51.45278 -2.50833 Europe/London
53.7446 -0.33525 Europe/London
52.75172 0.39516 Europe/London
55.65333 -4.70666 Europe/London
55.61171 -4.49581 Europe/London
53.08691 -2.23777 Europe/London
51.82166 -1.2886 Europe/London
52.38819 -2.25 Europe/London
51.41387 -2.4978 Europe/London
52.39836 -0.72571 Europe/London
52.34286 -1.58015 Europe/London
54.32681 -2.74757 Europe/London
52.11599 -0.50044 Europe/London
53.86791 -1.90664 Europe/London
55.82906 -4.51605 Europe/London
54.98036 -1.48423 Europe/London
51.53622 -0.10304 Europe/London
55.6194 -4.65508 Europe/London
53.44253 -2.42323 Europe/London
52.05917 1.15545 Europe/London
57.47908 -4.22398 Europe/London
52.97055 -1.30951 Europe/London
50.86004 -1.40162 Europe/London
53.45131 -2.07943 Europe/London
53.4115 -2.83935 Europe/London
52.33049 -0.18651 Europe/London
53.64904 -1.78416 Europe/London
53.03333 -1.2 Europe/London
53.5 -1.45 Europe/London
50.83088 -0.1672 Europe/London
54.84034 -1.46427 Europe/London
53.60126 -2.54975 Europe/London
51.06395 -0.32719 Europe/London
53.83797 -1.64036 Europe/London
51.76148 -0.01144 Europe/London
51.95314 -0.26519 Europe/London
53.53333 -2.58333 Europe/London
52.5389 -1.37613 Europe/London
51.62907 -0.74934 Europe/London
55.78438 -4.10007 Europe/London
53.32733 -3.09648 Europe/London
51.79588 -0.07854 Europe/London
51.373 1.12857 Europe/London
52.05684 -2.71482 Europe/London
51.75369 -0.47517 Europe/London
53.01372 -1.35383 Europe/London
50.99769 -0.10313 Europe/London
53.46723 -2.68166 Europe/London
53.18478 -3.02578 Europe/London
52.08226 0.43891 Europe/London
50.8567 -0.98559 Europe/London
51.76338 -0.22419 Europe/London
50.85519 0.57292 Europe/London
53.70326 -2.32382 Europe/London
51.94194 1.28437 Europe/London
54.68611 -1.2125 Europe/London
53.99078 -1.5373 Europe/London
51.81684 -0.35706 Europe/London
51.77655 0.11158 Europe/London
55.76667 -4.03333 Europe/London
53.71667 -1.85 Europe/London
52.44859 -2.04938 Europe/London
51.21667 -0.78333 Europe/London
50.8622 0.25775 Europe/London
54.53478 -1.05606 Europe/London
51.23536 -0.57427 Europe/London
51.60954 -1.42187 Europe/London
55.94838 -4.76121 Europe/London
52.60831 1.73052 Europe/London
52.66277 -2.01111 Europe/London
53.39234 -2.63994 Europe/London
52.11161 -2.32515 Europe/London
51.47566 0.32521 Europe/London
51.44138 0.37371 Europe/London
52.91149 -0.64184 Europe/London
56.01141 -3.72183 Europe/London
50.79509 -1.12902 Europe/London
55 -1.61667 Europe/London
51.66931 -4.04163 Europe/London
53.71667 -0.86667 Europe/London
53.47693 -2.59651 Europe/London
51.18462 -0.61725 Europe/London
51.86568 -2.2431 Europe/London
53.44325 -1.949 Europe/London
56.19514 -3.17316 Europe/London
55.86515 -4.25763 Europe/London
51.38914 0.54863 Europe/London
55.80373 -4.29488 Europe/London
51.66444 -3.25611 Europe/London
54.96209 -1.60168 Europe/London
53.79173 -1.38067 Europe/London
53.38333 -0.76667 Europe/London
53.83333 -2.91667 Europe/London
51.22834 -2.32211 Europe/London
51.83877 1.24625 Europe/London
53.55364 -3.06816 Europe/London
51.08333 1.18333 Europe/London
53.92527 -3.01085 Europe/London
51.28333 -0.83333 Europe/London
54.95297 -1.57152 Europe/London
51.96375 1.3511 Europe/London
51.3148 0.88856 Europe/London
53.55 -2.4 Europe/London
51.21433 -0.79587 Europe/London
51.29424 -0.75565 Europe/London
50.85162 -1.17929 Europe/London
50.15441 -5.07113 Europe/London
55.99917 -3.78713 Europe/London
53.51667 -2.13333 Europe/London
50.7236 -3.52751 Europe/London
52.09237 -1.94887 Europe/London
51.36926 -0.36572 Europe/London
51.3305 -0.27011 Europe/London
50.84779 -0.93697 Europe/London
53.27875 -2.90134 Europe/London
57.65 -3.33333 Europe/London
51.43158 -0.55239 Europe/London
55.95206 -3.19648 Europe/London
53.48333 -2.33333 Europe/London
51.77714 -3.20792 Europe/London
53 -1.3 Europe/London
53.32213 -0.94315 Europe/London
50.96667 -1.35 Europe/London
55.76667 -4.16667 Europe/London
51.12382 -0.0061 Europe/London
52.68333 0.93333 Europe/London
50.76871 0.28453 Europe/London
54.77676 -1.57566 Europe/London
51.886 -0.52099 Europe/London
56.07156 -3.45887 Europe/London
56.5 -2.96667 Europe/London
55.94433 -4.57061 Europe/London
53.47497 -2.08809 Europe/London
52.5 -2.08333 Europe/London
53.48005 -2.14543 Europe/London
53.30221 -1.47507 Europe/London
52.26667 -2.15 Europe/London
51.13333 1.3 Europe/London
51.2329 -0.32942 Europe/London
50.71667 -2.43333 Europe/London
53.52327 -1.13691 Europe/London
51.60928 -1.24214 Europe/London
53.69076 -1.62907 Europe/London
52.92277 -1.47663 Europe/London
53.45678 -2.11822 Europe/London
52.25688 -1.16066 Europe/London
53.69803 -2.46494 Europe/London
53.58705 -1.52676 Europe/London
51.44352 0.21964 Europe/London
54.52429 -1.55039 Europe/London
51.65446 -3.02281 Europe/London
55.94685 -3.99051 Europe/London
51.37027 -0.79219 Europe/London
51.06044 0.16171 Europe/London
53.09787 -2.44161 Europe/London
51.11303 -0.18312 Europe/London
55.08652 -1.58598 Europe/London
50.76306 -1.29772 Europe/London
52.40656 -1.51217 Europe/London
52.49637 -0.68939 Europe/London
54.85404 -1.8316 Europe/London
53.48188 -1.23214 Europe/London
53.16314 -2.21253 Europe/London
53.29483 -3.72674 Europe/London
55.13333 -6.66667 Europe/London
51.88921 0.90421 Europe/London
51.522 -3.55531 Europe/London
51.32997 -0.4113 Europe/London
52.71667 -1.36667 Europe/London
55.90137 -4.4057 Europe/London
51.68333 -3.9 Europe/London
53.86667 -2.4 Europe/London
51.44057 -2.85745 Europe/London
53.56047 -0.03225 Europe/London
53.72405 -1.71294 Europe/London
51.78967 1.15597 Europe/London
51.71708 -1.96825 Europe/London
50.73306 -1.77567 Europe/London
53.65 -2.61667 Europe/London
51.53813 -2.39379 Europe/London
51.46 -2.12472 Europe/London
50.83673 -0.78003 Europe/London
54.85862 -1.57408 Europe/London
53.25 -1.41667 Europe/London
53.1905 -2.89189 Europe/London
51.70791 -0.03739 Europe/London
51.7 -0.6 Europe/London
51.9 -2.08333 Europe/London
51.48755 -0.16936 Europe/London
51.73575 0.46958 Europe/London
53.3761 -2.1897 Europe/London
51.37891 0.52786 Europe/London
53.46506 -1.47217 Europe/London
51.60885 -0.55618 Europe/London
54.5735 -5.88472 Europe/London
53.72587 -1.36256 Europe/London
54.7158 -5.8058 Europe/London
51.85552 -4.30535 Europe/London
54.8951 -2.9382 Europe/London
51.48 -3.18 Europe/London
51.27904 1.07992 Europe/London
52.69045 -2.03085 Europe/London
52.2 0.11667 Europe/London
51.33705 -0.74261 Europe/London
51.57452 -3.218 Europe/London
53.25741 -1.90982 Europe/London
51.64316 -0.36053 Europe/London
52.2463 0.71111 Europe/London
53.6 -2.3 Europe/London
52.80728 -1.64263 Europe/London
52.68075 -1.92759 Europe/London
53.8 -2.23333 Europe/London
50.95843 -0.13287 Europe/London
53.16667 -3.08333 Europe/London
56.17149 -3.03377 Europe/London
53.06667 -3.06667 Europe/London
52.63333 -1.93333 Europe/London
51.35908 1.43938 Europe/London
50.39431 -3.51585 Europe/London
51.63106 -3.81898 Europe/London
51.45523 -2.59665 Europe/London
50.82838 -0.13947 Europe/London
53.70322 -1.78428 Europe/London
54.08306 -0.19192 Europe/London
51.12837 -3.00356 Europe/London
51.62127 0.30556 Europe/London
51.87819 0.55292 Europe/London
53.79391 -1.75206 Europe/London
51.41363 -0.75054 Europe/London
50.72048 -1.8795 Europe/London
52.97633 -0.02664 Europe/London
51.65468 -0.27762 Europe/London
53.46667 -3.01667 Europe/London
53.58333 -2.43333 Europe/London
50.78206 -0.67978 Europe/London
55.12708 -1.50856 Europe/London
51.99334 -0.73471 Europe/London
53.81667 -3.05 Europe/London
53.75 -2.48333 Europe/London
50.96643 -1.32832 Europe/London
51.87113 0.15868 Europe/London
55.90669 -4.21869 Europe/London
54.65554 -1.67706 Europe/London
52.48142 -1.89983 Europe/London
53.39337 -3.01479 Europe/London
53.84861 -1.83857 Europe/London
54.60828 -1.29214 Europe/London
51.62867 0.41963 Europe/London
52.08652 -0.26493 Europe/London
51.01787 -4.20458 Europe/London
53.11724 -2.17584 Europe/London
51.89998 -1.15357 Europe/London
50.85023 0.47095 Europe/London
53.84587 -0.42332 Europe/London
55.78333 -2 Europe/London
53.53333 -1.15 Europe/London
53.0233 -1.48119 Europe/London
55.81667 -4.01667 Europe/London
54.58333 -5.93333 Europe/London
52.4791 -1.46909 Europe/London
55.13061 -1.59319 Europe/London
52.13459 -0.46632 Europe/London
53.35 -3.01667 Europe/London
55.91536 -4.33279 Europe/London
53.70291 -1.6337 Europe/London
55.90204 -3.64398 Europe/London
51.37795 -2.35907 Europe/London
51.26249 -1.08708 Europe/London
51.56844 0.45782 Europe/London
51.40667 -3.26944 Europe/London
54.11667 -3.23333 Europe/London
55.79916 -4.39285 Europe/London
51.08022 -4.05808 Europe/London
53.55 -1.48333 Europe/London
54.65338 -5.66895 Europe/London
53.22647 -4.13459 Europe/London
52.0602 -1.34029 Europe/London
54.35 -6.28333 Europe/London
54.86357 -6.27628 Europe/London
53.84711 -1.78785 Europe/London
55.46273 -4.63393 Europe/London
51.81665 -0.81458 Europe/London
53.52371 -2.49354 Europe/London
53.48876 -2.0989 Europe/London
53.48333 -2.65 Europe/London
51.14586 0.87281 Europe/London
51.41082 -0.6748 Europe/London
53 -1.13333 Europe/London
56.56317 -2.58736 Europe/London
54.7 -6.2 Europe/London
51.20828 -1.48246 Europe/London
51.66667 -0.61667 Europe/London
53.38752 -2.34848 Europe/London
51.14931 -0.97469 Europe/London
56.11586 -3.78997 Europe/London
53.0961 -1.38832 Europe/London
52.60549 -1.91715 Europe/London
51.24827 -0.76389 Europe/London
55.86602 -3.98025 Europe/London
51.50901 -0.2762 Europe/London
53.75379 -2.35863 Europe/London
51.67109 -1.28278 Europe/London
52.41548 -4.08292 Europe/London
53.28436 -3.5822 Europe/London
57.14369 -2.09814 Europe/London
51.71438 -3.44918 Europe/London
53.47778 -3.03333 Europe/London
51.66778 -3.2075 Europe/London
51.41222 -2.20056 Europe/London
51.54057 -0.14334 Europe/London
52.67659 -2.44926 Europe/London
54.44709 -6.387 Europe/London
51.52124 -0.11347 Europe/London
51.54377 -0.1399 Europe/London
51.51116 -0.18426 Europe/London
52.63884 1.21885 Europe/London
50.91234 -1.30076 Europe/London
53.37831 -2.33271 Europe/London
51.67468 -0.60742 Europe/London
51.47475 -0.15547 Europe/London
51.55685 0.21664 Europe/London
51.34948 -0.2494 Europe/London
51.5529 0.129 Europe/London
51.54062 0.14801 Europe/London
51.46593 -0.10652 Europe/London
51.52718 -0.06109 Europe/London
53.50484 -2.16568 Europe/London
53.56178 -2.32455 Europe/London
53.59245 -2.21941 Europe/London
53.45801 -2.20104 Europe/London
50.72044 -3.49646 Europe/London
50.80743 -1.89975 Europe/London
53.7426 -2.997 Europe/London
51.66669 -0.16981 Europe/London
53.82901 -1.53834 Europe/London
51.4647 0.0079 Europe/London
52.08956 -0.49908 Europe/London
51.2372 -2.6266 Europe/London
51.42708 -0.91979 Europe/London
53.34914 -0.9785 Europe/London
52.43532 -1.99707 Europe/London
51.56667 -0.33333 Europe/London
51.4439 -0.1854 Europe/London
12.05644 -61.74849 America/Grenada
42.5088 41.87088 Asia/Tbilisi
42.11 43.0525 Asia/Tbilisi
42.22764 43.96861 Asia/Tbilisi
42.84035 41.68007 Asia/Tbilisi
41.91978 45.47315 Asia/Tbilisi
41.69411 44.83368 Asia/Tbilisi
43.00697 40.9893 Asia/Tbilisi
42.27042 42.0675 Asia/Tbilisi
42.1537 42.33517 Asia/Tbilisi
41.54949 44.99323 Asia/Tbilisi
42.14616 41.67197 Asia/Tbilisi
42.71232 41.46863 Asia/Tbilisi
41.47588 44.80894 Asia/Tbilisi
42.24961 42.69974 Asia/Tbilisi
41.82 41.77528 Asia/Tbilisi
41.99414 43.59994 Asia/Tbilisi
41.98422 44.11578 Asia/Tbilisi
41.64159 41.63593 Asia/Tbilisi
41.63901 42.98262 Asia/Tbilisi
43.08056 40.83833 Asia/Tbilisi
42.34129 42.5976 Asia/Tbilisi
41.69278 44.80154 Asia/Tbilisi
5.5 -54.03333 America/Cayenne
4.91667 -52.26667 America/Cayenne
4.84827 -52.33172 America/Cayenne
5.15518 -52.64779 America/Cayenne
4.93333 -52.33333 America/Cayenne
49.45981 -2.53527 Europe/Guernsey
9.44272 -0.00991 Africa/Accra
5.35113 -0.62313 Africa/Accra
7.73916 -2.10456 Africa/Accra
10.06069 -2.50192 Africa/Accra
5.58365 -0.10722 Africa/Accra
5.66698 -0.01657 Africa/Accra
7.58616 -1.94137 Africa/Accra
5.30644 -1.98474 Africa/Accra
9.40078 -0.8393 Africa/Accra
4.88447 -1.75536 Africa/Accra
6.73453 -1.61275 Africa/Accra
5.53711 -0.69984 Africa/Accra
7.33991 -2.32676 Africa/Accra
6.04089 -0.45004 Africa/Accra
5.00872 -1.63011 Africa/Accra
4.934 -1.7137 Africa/Accra
9.62441 -0.8253 Africa/Accra
5.20913 -1.06058 Africa/Accra
8.55083 -0.51875 Africa/Accra
5.43274 -2.14284 Africa/Accra
5.53302 -0.4224 Africa/Accra
5.92313 -0.99507 Africa/Accra
6.20602 -1.66191 Africa/Accra
5.60106 -0.07713 Africa/Accra
5.80893 -0.35026 Africa/Accra
6.55121 -0.7662 Africa/Accra
10.89557 -1.0921 Africa/Accra
7.06273 -1.4001 Africa/Accra
5.68234 -0.16413 Africa/Accra
6.68848 -1.62443 Africa/Accra
6.99536 0.29306 Africa/Accra
6.61667 -1.21667 Africa/Accra
6.09408 -0.25913 Africa/Accra
8.05627 -1.73058 Africa/Accra
5.91793 0.9879 Africa/Accra
7.15181 0.47362 Africa/Accra
6.6 0.46667 Africa/Accra
5.57714 -0.31035 Africa/Accra
5.70119 -1.28657 Africa/Accra
5.08981 -1.34804 Africa/Accra
7.38558 -1.35617 Africa/Accra
5.96562 -1.77995 Africa/Accra
5.65003 -0.2361 Africa/Accra
5.10535 -1.2466 Africa/Accra
10.78556 -0.85139 Africa/Accra
6.46346 -2.31938 Africa/Accra
7.4534 -2.58404 Africa/Accra
6.38706 -0.37738 Africa/Accra
11.0616 -0.24168 Africa/Accra
4.86992 -2.24046 Africa/Accra
5.86006 -0.6635 Africa/Accra
5.28483 -0.73711 Africa/Accra
5.79473 0.89728 Africa/Accra
6.04024 -0.80876 Africa/Accra
6.80004 -1.08193 Africa/Accra
5.7 -0.33333 Africa/Accra
5.55602 -0.1969 Africa/Accra
5.84718 -0.17754 Africa/Accra
36.14474 -5.35257 Europe/Gibraltar
64.18347 -51.72157 America/Godthab
13.41033 -16.70815 Africa/Banjul
13.35222 -16.43389 Africa/Banjul
13.56667 -15.6 Africa/Banjul
13.27136 -16.64944 Africa/Banjul
13.45274 -16.57803 Africa/Banjul
13.47806 -16.68194 Africa/Banjul
11.45 -11.68333 Africa/Conakry
10.9 -13.03333 Africa/Conakry
11.42282 -9.16852 Africa/Conakry
11.08333 -12.4 Africa/Conakry
7.75624 -8.8179 Africa/Conakry
10.37546 -12.09148 Africa/Conakry
8.54351 -9.47099 Africa/Conakry
11.31823 -12.28332 Africa/Conakry
9.1848 -10.09987 Africa/Conakry
10.05692 -12.86576 Africa/Conakry
10.38542 -9.30568 Africa/Conakry
10.66667 -14.6 Africa/Conakry
8.5625 -10.1325 Africa/Conakry
10.45 -13.53333 Africa/Conakry
9.70556 -13.37694 Africa/Conakry
9.53795 -13.67729 Africa/Conakry
9.535 -13.68778 Africa/Conakry
10.94094 -14.29665 Africa/Conakry
16.33238 -61.69793 America/Guadeloupe
16.22636 -61.37919 America/Guadeloupe
16.2422 -61.5343 America/Guadeloupe
16.19135 -61.59159 America/Guadeloupe
16.27095 -61.50451 America/Guadeloupe
16.33315 -61.34728 America/Guadeloupe
16.20685 -61.49329 America/Guadeloupe
16.04322 -61.56596 America/Guadeloupe
15.99854 -61.72548 America/Guadeloupe
16.26738 -61.58543 America/Guadeloupe
2.15106 11.33528 Africa/Malabo
3.75 8.78333 Africa/Malabo
1.86391 9.76582 Africa/Malabo
37.84221 23.77651 Europe/Athens
39.36667 22.94583 Europe/Athens
37.96667 23.75 Europe/Athens
37.50889 22.37944 Europe/Athens
39.55528 21.7675 Europe/Athens
38.325 23.31889 Europe/Athens
37.07333 22.42972 Europe/Athens
37.96427 23.49649 Europe/Athens
35.36472 24.47139 Europe/Athens
38.95 20.75 Europe/Athens
37.67513 21.44102 Europe/Athens
37.94745 23.63708 Europe/Athens
38.04187 23.68494 Europe/Athens
38.01667 23.7 Europe/Athens
37.96775 23.5721 Europe/Athens
38.24444 21.73444 Europe/Athens
37.92066 23.70916 Europe/Athens
37.96667 23.65 Europe/Athens
37.95 23.71667 Europe/Athens
38.03333 23.7 Europe/Athens
38.03333 23.75 Europe/Athens
38.03333 23.73333 Europe/Athens
38.1 23.81667 Europe/Athens
38.39167 21.8275 Europe/Athens
37.95 23.66667 Europe/Athens
39.11 26.55472 Europe/Athens
38.05 23.83333 Europe/Athens
38 23.34556 Europe/Athens
38.435 22.87389 Europe/Athens
39.63722 22.42028 Europe/Athens
38.9 22.43333 Europe/Athens
36.89333 27.28889 Europe/Athens
37.89886 23.87181 Europe/Athens
37.94068 22.95734 Europe/Athens
38.07438 23.81106 Europe/Athens
38 23.8 Europe/Athens
38.36778 26.13583 Europe/Athens
35.51222 24.01556 Europe/Athens
38.46361 23.59944 Europe/Athens
38.03058 23.79793 Europe/Athens
37.9625 23.61972 Europe/Athens
39.36556 21.92167 Europe/Athens
37.95 23.7 Europe/Athens
37.03889 22.11417 Europe/Athens
37.91033 23.72361 Europe/Athens
37.96667 23.8 Europe/Athens
38.06667 23.76667 Europe/Athens
35.32787 25.14341 Europe/Athens
39.6675 20.85083 Europe/Athens
38.01667 23.75 Europe/Athens
37.88333 23.73333 Europe/Athens
38.04135 23.54295 Europe/Athens
37.98333 23.66667 Europe/Athens
38.01667 23.83333 Europe/Athens
37.97945 23.71622 Europe/Athens
38.06667 23.58333 Europe/Athens
39.16056 20.98528 Europe/Athens
37.63333 22.73333 Europe/Athens
38.08333 23.7 Europe/Athens
38.05 23.8 Europe/Athens
37.8 21.35 Europe/Athens
38.08333 23.73333 Europe/Athens
38.24861 22.08194 Europe/Athens
37.98333 23.68333 Europe/Athens
38.62139 21.40778 Europe/Athens
36.44083 28.2225 Europe/Athens
40.79194 22.4075 Europe/Athens
41.14139 24.88361 Europe/Athens
40.52333 22.20361 Europe/Athens
40.64028 22.94389 Europe/Athens
40.64944 22.95083 Europe/Athens
41.08556 23.54972 Europe/Athens
40.51472 21.67861 Europe/Athens
40.66671 22.94881 Europe/Athens
40.60056 22.9875 Europe/Athens
40.50028 22.925 Europe/Athens
40.58779 23.0315 Europe/Athens
41.50306 26.52972 Europe/Athens
40.73083 22.91722 Europe/Athens
40.62944 22.06806 Europe/Athens
40.65694 22.9 Europe/Athens
40.30111 21.78639 Europe/Athens
41.12278 25.39639 Europe/Athens
40.99194 22.87083 Europe/Athens
40.93972 24.40194 Europe/Athens
40.27194 22.5025 Europe/Athens
40.5825 22.95028 Europe/Athens
40.78197 21.40981 Europe/Athens
40.80056 22.04722 Europe/Athens
41.15444 24.13944 Europe/Athens
40.8475 25.87444 Europe/Athens
39.62 19.91972 Europe/Athens
38.03381 23.82962 Europe/Athens
37.91667 23.71667 Europe/Athens
38.37019 21.42952 Europe/Athens
38.05404 23.806 Europe/Athens
-54.28111 -36.5092 Atlantic/South_Georgia
14.96667 -89.53333 America/Guatemala
14.52694 -90.5875 America/Guatemala
14.48139 -90.53167 America/Guatemala
14.91667 -91.36667 America/Guatemala
14.76694 -90.99417 America/Guatemala
14.64639 -90.73389 America/Guatemala
14.76667 -91.18333 America/Guatemala
14.63528 -90.67639 America/Guatemala
14.63823 -91.22901 America/Guatemala
14.51833 -90.72667 America/Guatemala
14.33333 -91.01667 America/Guatemala
15.03056 -91.14889 America/Guatemala
14.56889 -90.49528 America/Guatemala
14.96667 -91.76667 America/Guatemala
14.78667 -90.45111 America/Guatemala
14.58333 -91.45 America/Guatemala
14.96667 -91.8 America/Guatemala
14.60972 -90.65694 America/Guatemala
14.71889 -90.64417 America/Guatemala
14.54611 -90.41139 America/Guatemala
14.95 -91.45 America/Guatemala
15.38333 -90.4 America/Guatemala
16.91667 -89.9 America/Guatemala
14.795 -90.19222 America/Guatemala
14.62 -90.84417 America/Guatemala
14.53333 -91.68333 America/Guatemala
14.83333 -91.51667 America/Guatemala
13.92556 -90.82444 America/Guatemala
15.71667 -88.6 America/Guatemala
14.28333 -91.36667 America/Guatemala
16.33111 -89.41694 America/Guatemala
14.50278 -90.55167 America/Guatemala
14.68333 -91.01667 America/Guatemala
14.63167 -90.92694 America/Guatemala
15.4 -89.66667 America/Guatemala
14.40556 -90.69833 America/Guatemala
14.66472 -90.35861 America/Guatemala
14.86667 -91.61667 America/Guatemala
14.6 -91.7 America/Guatemala
15.40583 -91.14611 America/Guatemala
14.85 -91.31667 America/Guatemala
15.48333 -88.81667 America/Guatemala
15.04361 -91.40861 America/Guatemala
14.63333 -90.60639 America/Guatemala
14.53333 -91.5 America/Guatemala
14.08333 -91.05 America/Guatemala
14.86667 -91.56667 America/Guatemala
14.28333 -89.9 America/Guatemala
14.58194 -90.74361 America/Guatemala
14.63333 -89.98333 America/Guatemala
15.66667 -91.73333 America/Guatemala
15.31972 -91.47083 America/Guatemala
15.13333 -89.36667 America/Guatemala
14.46528 -90.44083 America/Guatemala
16.93333 -89.88333 America/Guatemala
14.56667 -89.35 America/Guatemala
14.305 -90.785 America/Guatemala
14.64778 -90.79278 America/Guatemala
14.65 -91.58333 America/Guatemala
15.53333 -89.35 America/Guatemala
14.27639 -90.30028 America/Guatemala
15.08333 -91.71667 America/Guatemala
14.74111 -90.8875 America/Guatemala
14.71667 -91.73333 America/Guatemala
15.48333 -90.36667 America/Guatemala
14.7 -91.86667 America/Guatemala
14.52639 -90.75972 America/Guatemala
14.64072 -90.51327 America/Guatemala
15.81667 -90.28333 America/Guatemala
14.8 -89.55 America/Guatemala
14.70833 -90.49944 America/Guatemala
14.66861 -90.81667 America/Guatemala
14.93333 -91.11667 America/Guatemala
14.53333 -91.31667 America/Guatemala
14.81667 -91.45 America/Guatemala
14.30944 -90.36111 America/Guatemala
14.33083 -89.71083 America/Guatemala
14.56111 -90.73444 America/Guatemala
14.4875 -90.61528 America/Guatemala
14.48028 -90.8075 America/Guatemala
13.48773 144.78138 Pacific/Guam
13.53605 144.88855 Pacific/Guam
13.47191 144.74978 Pacific/Guam
13.51777 144.8391 Pacific/Guam
13.47567 144.74886 Pacific/Guam
13.44761 144.80109 Pacific/Guam
11.86357 -15.59767 Africa/Bissau
12.16667 -14.66667 Africa/Bissau
6.25 -57.51667 America/Guyana
6 -58.3 America/Guyana
6.80448 -58.15527 America/Guyana
22.36667 114.1 Asia/Hong_Kong
22.31667 114.18333 Asia/Hong_Kong
22.28552 114.15769 Asia/Hong_Kong
15.83333 -87.95 America/Tegucigalpa
15.13333 -87.13333 America/Tegucigalpa
15.31667 -88 America/Tegucigalpa
15.68333 -86 America/Tegucigalpa
15.78333 -87.45 America/Tegucigalpa
14.0818 -87.20681 America/Tegucigalpa
14.6 -87.83333 America/Tegucigalpa
14.76667 -88.78333 America/Tegucigalpa
14.91667 -88.23333 America/Tegucigalpa
15.5 -88.03333 America/Tegucigalpa
13.42417 -87.44722 America/Tegucigalpa
15.23333 -87.96667 America/Tegucigalpa
15.5 -86.56667 America/Tegucigalpa
14.31667 -87.68333 America/Tegucigalpa
15.43333 -87.91667 America/Tegucigalpa
15.75971 -86.78221 America/Tegucigalpa
14.65 -86.2 America/Tegucigalpa
15.4 -87.8 America/Tegucigalpa
13.86667 -86.55 America/Tegucigalpa
14.03333 -86.58333 America/Tegucigalpa
14.45 -87.63333 America/Tegucigalpa
15.4 -88.15 America/Tegucigalpa
13.30028 -87.19083 America/Tegucigalpa
15.61444 -87.95302 America/Tegucigalpa
45.85639 15.80778 Europe/Zagreb
45.81444 15.97798 Europe/Zagreb
44.11972 15.24222 Europe/Zagreb
45.34333 18.99972 Europe/Zagreb
45.83194 17.38389 Europe/Zagreb
45.28833 18.80472 Europe/Zagreb
45.7125 16.07556 Europe/Zagreb
46.30444 16.33778 Europe/Zagreb
43.50891 16.43915 Europe/Zagreb
43.55 16.5 Europe/Zagreb
45.16028 18.01556 Europe/Zagreb
45.34028 17.68528 Europe/Zagreb
45.46611 16.37833 Europe/Zagreb
43.72722 15.90583 Europe/Zagreb
45.83111 16.11639 Europe/Zagreb
45.80306 15.71806 Europe/Zagreb
45.34306 14.40917 Europe/Zagreb
44.86833 13.84806 Europe/Zagreb
45.55111 18.69389 Europe/Zagreb
46.16278 16.8275 Europe/Zagreb
45.48722 15.54778 Europe/Zagreb
42.64807 18.09216 Europe/Zagreb
46.38444 16.43389 Europe/Zagreb
45.89861 16.84889 Europe/Zagreb
45.81313 15.97753 Europe/Zagreb
19.05 -72.46667 America/Port-au-Prince
18.65194 -72.09472 America/Port-au-Prince
19.43333 -72.2 America/Port-au-Prince
19.10819 -72.69379 America/Port-au-Prince
18.53917 -72.335 America/Port-au-Prince
18.43139 -72.86694 America/Port-au-Prince
18.5125 -72.28528 America/Port-au-Prince
18.44232 -73.08758 America/Port-au-Prince
19.7 -72.4 America/Port-au-Prince
18.51083 -72.63389 America/Port-au-Prince
18.45056 -72.28694 America/Port-au-Prince
18.65 -74.11667 America/Port-au-Prince
18.23417 -72.53472 America/Port-au-Prince
19.15 -72.01667 America/Port-au-Prince
18.55 -72.51667 America/Port-au-Prince
18.42889 -72.77056 America/Port-au-Prince
19.45 -72.68333 America/Port-au-Prince
18.50583 -71.97667 America/Port-au-Prince
18.99167 -72.38889 America/Port-au-Prince
18.54472 -72.30278 America/Port-au-Prince
18.575 -72.225 America/Port-au-Prince
18.2 -73.75 America/Port-au-Prince
18.54114 -72.39922 America/Port-au-Prince
19.75778 -72.20417 America/Port-au-Prince
19.93333 -72.83333 America/Port-au-Prince
47.18333 20.41667 Europe/Budapest
47.93333 21.08333 Europe/Budapest
47.18333 20.2 Europe/Budapest
46.65 20.26667 Europe/Budapest
46.253 20.14824 Europe/Budapest
46.86667 20.55 Europe/Budapest
48.4 21.66667 Europe/Budapest
47.31667 21.11667 Europe/Budapest
48.21667 20.3 Europe/Budapest
46.56667 20.66667 Europe/Budapest
47.95539 21.71671 Europe/Budapest
48.1 20.78333 Europe/Budapest
47 20.63333 Europe/Budapest
47.81667 20.58333 Europe/Budapest
47.95528 22.32348 Europe/Budapest
46.21667 20.48333 Europe/Budapest
48.21667 22.08333 Europe/Budapest
48.25 20.63333 Europe/Budapest
47.31667 20.93333 Europe/Budapest
46.41667 20.33333 Europe/Budapest
47.45 21.4 Europe/Budapest
47.85 21.43333 Europe/Budapest
47.66667 21.51667 Europe/Budapest
46.65 21.28333 Europe/Budapest
46.93333 20.83333 Europe/Budapest
47.90265 20.37329 Europe/Budapest
47.53333 21.63333 Europe/Budapest
46.71332 20.14242 Europe/Budapest
47.21667 21.55 Europe/Budapest
46.68333 21.1 Europe/Budapest
46.76667 21.13333 Europe/Budapest
47.61667 21.35 Europe/Budapest
47.1899 20.00476 Europe/Budapest
46.84 16.84389 Europe/Budapest
47.09327 17.91149 Europe/Budapest
47.40705 19.28648 Europe/Budapest
47.19936 18.13954 Europe/Budapest
47.77591 19.13612 Europe/Budapest
47.58494 18.39325 Europe/Budapest
47.65289 18.31838 Europe/Budapest
46.88152 17.44117 Europe/Budapest
47.23088 16.62155 Europe/Budapest
47.34382 19.04335 Europe/Budapest
47.66943 19.07561 Europe/Budapest
46.35014 18.70905 Europe/Budapest
47.18995 18.41034 Europe/Budapest
47.32579 18.92142 Europe/Budapest
47.68501 16.59049 Europe/Budapest
46.90412 18.058 Europe/Budapest
47.25395 16.93525 Europe/Budapest
48.09872 19.80303 Europe/Budapest
46.08333 18.23333 Europe/Budapest
47.9126 19.97709 Europe/Budapest
47.33004 17.4674 Europe/Budapest
46.62648 18.85962 Europe/Budapest
47.48671 18.31225 Europe/Budapest
47.03419 19.77857 Europe/Budapest
46.45347 16.99104 Europe/Budapest
47.86788 17.26994 Europe/Budapest
47.35133 19.44734 Europe/Budapest
45.99306 18.68306 Europe/Budapest
46.19278 18.26494 Europe/Budapest
47.74318 18.11913 Europe/Budapest
46.43402 19.4848 Europe/Budapest
46.71213 19.84458 Europe/Budapest
46.62139 19.28528 Europe/Budapest
46.76812 17.24317 Europe/Budapest
46.90618 19.69128 Europe/Budapest
46.36667 17.8 Europe/Budapest
46.52639 18.98583 Europe/Budapest
47.5 19.91667 Europe/Budapest
47.66667 19.68333 Europe/Budapest
47.68333 17.63512 Europe/Budapest
47.78257 19.928 Europe/Budapest
47.38449 19.2214 Europe/Budapest
47.59657 19.35515 Europe/Budapest
47.68324 19.13417 Europe/Budapest
47.6177 19.1887 Europe/Budapest
47.7928 18.74148 Europe/Budapest
47.39489 18.9136 Europe/Budapest
46.98333 18.93333 Europe/Budapest
47.6364 19.13864 Europe/Budapest
47.3545 19.09822 Europe/Budapest
46.37657 18.13696 Europe/Budapest
47.18594 19.31091 Europe/Budapest
47.17266 19.79952 Europe/Budapest
47.49801 19.03991 Europe/Budapest
47.46181 18.95845 Europe/Budapest
48.07296 19.29614 Europe/Budapest
46.17496 18.95639 Europe/Budapest
47.10196 17.55892 Europe/Budapest
47.49192 19.01493 Europe/Budapest
47.47603 19.03605 Europe/Budapest
47.48299 19.06591 Europe/Budapest
47.48919 19.07012 Europe/Budapest
47.50045 19.06875 Europe/Budapest
47.50369 19.06583 Europe/Budapest
47.5183 19.10789 Europe/Budapest
47.52978 19.08068 Europe/Budapest
47.56182 19.08909 Europe/Budapest
47.56263 19.11681 Europe/Budapest
47.51482 19.17028 Europe/Budapest
47.4791 19.15835 Europe/Budapest
47.45293 19.14943 Europe/Budapest
47.44417 19.17595 Europe/Budapest
47.39788 19.11492 Europe/Budapest
47.42698 19.04016 Europe/Budapest
47.43047 19.07098 Europe/Budapest
47.43674 19.10093 Europe/Budapest
47.47997 19.25388 Europe/Budapest
47.54157 19.04501 Europe/Budapest
47.51291 19.03184 Europe/Budapest
47.49705 19.03961 Europe/Budapest
3.001 99.8164 Asia/Jakarta
3.3285 99.1625 Asia/Jakarta
4.0613 98.3699 Asia/Jakarta
2.96667 99.8 Asia/Jakarta
3.5765 98.6151 Asia/Jakarta
3.761 98.4513 Asia/Jakarta
5.3848 95.9609 Asia/Jakarta
1.7427 98.7792 Asia/Jakarta
5.89333 95.32139 Asia/Jakarta
5.21667 96.28333 Asia/Jakarta
2.1 99.83333 Asia/Jakarta
3.6253 98.864 Asia/Jakarta
3.5679 98.956 Asia/Jakarta
2.9595 99.0687 Asia/Jakarta
4.0238 98.2782 Asia/Jakarta
1.36667 99.26667 Asia/Jakarta
4.1363 96.1285 Asia/Pontianak
3.58333 98.66667 Asia/Jakarta
5.1801 97.1507 Asia/Jakarta
4.4683 97.9683 Asia/Jakarta
3.7278 98.6738 Asia/Jakarta
2.9845 99.6158 Asia/Jakarta
3.1001 98.4908 Asia/Jakarta
3.5078 98.6839 Asia/Jakarta
5.203 96.7009 Asia/Jakarta
3.6001 98.4854 Asia/Jakarta
3.7755 98.6832 Asia/Jakarta
2.05 99.75 Asia/Jakarta
5.5577 95.3222 Asia/Jakarta
-7.78278 110.36083 Asia/Jakarta
-7.36 109.9025 Asia/Jakarta
-6.98333 109.61667 Asia/Jakarta
-7.9908 114.4009 Asia/Jakarta
-6.8922 109.619 Asia/Jakarta
-6.711 108.5037 Asia/Jakarta
-6.9713 110.0666 Asia/Jakarta
-6.8 110.71667 Asia/Jakarta
-7.74306 110.57944 Asia/Jakarta
-4.5386 120.3279 Asia/Makassar
-7.51611 109.05389 Asia/Jakarta
-7.1749 107.9808 Asia/Jakarta
-9.6567 120.2641 Asia/Makassar
-7.13972 110.405 Asia/Jakarta
-5.14 119.4221 Asia/Makassar
-8.5098 115.2654 Asia/Makassar
-8.0657 111.9025 Asia/Jakarta
-7.4737 112.6505 Asia/Jakarta
-6.8976 112.0649 Asia/Jakarta
-5.66667 132.75 Asia/Jayapura
-7.71833 110.65889 Asia/Jakarta
-8.05 111.71667 Asia/Jakarta
1.3038 124.9112 Asia/Makassar
1.3346 124.8392 Asia/Makassar
0.8 127.4 Asia/Jayapura
-4.86667 105.23333 Asia/Jakarta
-6.8694 109.1402 Asia/Jakarta
-6.5397 111.0518 Asia/Jakarta
-7.3274 108.2207 Asia/Jakarta
-6.93333 109.16667 Asia/Jakarta
3.3 117.63333 Asia/Makassar
0.91667 104.45 Asia/Pontianak
-2.75 107.65 Asia/Jakarta
-5.42544 105.25803 Asia/Jakarta
-3.93333 103.8 Asia/Jakarta
-7.49958 112.69992 Asia/Jakarta
-6.17806 106.63 Asia/Jakarta
-8.5413 115.12522 Asia/Makassar
-7.55611 110.83167 Asia/Jakarta
-7.24917 112.75083 Asia/Jakarta
0.7 108.9 Asia/Makassar
-2.08333 101.38333 Asia/Jakarta
-1.85 106.13333 Asia/Jakarta
-7.01667 113.86667 Asia/Jakarta
-6.85 107.91667 Asia/Jakarta
-8.15856 112.48292 Asia/Jakarta
-6.76028 108.48306 Asia/Jakarta
-8.5 117.43333 Asia/Makassar
-7.45806 109.28806 Asia/Jakarta
-6.91806 106.92667 Asia/Jakarta
-8.40003 114.26661 Asia/Jakarta
-7.93861 110.25056 Asia/Jakarta
-7.42639 111.02222 Asia/Jakarta
-0.88333 131.25 Asia/Jayapura
-7.03306 107.51833 Asia/Jakarta
-0.79183 100.65932 Asia/Jakarta
-7.48315 112.42704 Asia/Jakarta
-9.86071 124.28395 Asia/Makassar
-7.71556 110.35556 Asia/Jakarta
-6.9816 109.1407 Asia/Jakarta
-7.70623 114.00976 Asia/Jakarta
-5.1241 120.253 Asia/Makassar
-7.8924 112.6658 Asia/Jakarta
-8.31667 114.23333 Asia/Jakarta
0.9 109 Asia/Pontianak
-4.1279 120.0297 Asia/Makassar
-8.112 115.08818 Asia/Jakarta
-7.3515 108.111 Asia/Jakarta
-1.26667 104.08333 Asia/Jakarta
-0.68762 100.95365 Asia/Jakarta
-7.4478 112.7183 Asia/Jakarta
-7.4846 108.7923 Asia/Jakarta
-7.87639 110.35889 Asia/Jakarta
-6.31694 106.66417 Asia/Jakarta
-6.1149 106.1502 Asia/Jakarta
-6.11889 106.575 Asia/Jakarta
-6.9932 110.4203 Asia/Jakarta
-7.78333 110.86667 Asia/Jakarta
-6.40278 106.77444 Asia/Jakarta
-2.53333 112.95 Asia/Pontianak
-7.1872 113.2394 Asia/Jakarta
-0.5 117.15 Asia/Makassar
-7.33194 110.49278 Asia/Jakarta
-8.6136 120.4721 Asia/Makassar
-6.15917 107.29806 Asia/Jakarta
-6.7036 111.3416 Asia/Jakarta
-2.9701 119.8978 Asia/Makassar
-6.3591 106.2494 Asia/Jakarta
-7.0981 109.3243 Asia/Jakarta
-7.221 108.1896 Asia/Jakarta
-7.42139 109.23444 Asia/Jakarta
-7.0868 110.9158 Asia/Jakarta
-6.55694 107.44333 Asia/Jakarta
-7.38806 109.36389 Asia/Jakarta
-7.95222 110.34861 Asia/Jakarta
-7.7543 113.2159 Asia/Jakarta
-7.68333 112.61667 Asia/Jakarta
-8.70536 116.27036 Asia/Makassar
-7.75 110.49417 Asia/Jakarta
-1.3959 120.7524 Asia/Makassar
-0.03333 109.33333 Asia/Pontianak
-7.8685 111.462 Asia/Jakarta
-3.4324 119.3435 Asia/Makassar
-6.705 108.47278 Asia/Jakarta
-6.88333 109.43333 Asia/Jakarta
-3.45 104.25 Asia/Jakarta
1.16667 108.96667 Asia/Pontianak
-6.9 109.36667 Asia/Jakarta
-6.9875 106.55139 Asia/Jakarta
0.53333 101.45 Asia/Jakarta
-6.8886 109.6753 Asia/Jakarta
-6.6978 110.7107 Asia/Jakarta
-0.22019 100.63078 Asia/Jakarta
-6.7559 111.038 Asia/Jakarta
-7.6453 112.9075 Asia/Jakarta
-7.1026 107.7641 Asia/Jakarta
-6.17028 106.53028 Asia/Jakarta
-6.42139 106.73306 Asia/Jakarta
-0.61898 100.11997 Asia/Jakarta
-7.7679 112.198 Asia/Jakarta
-7.72528 114.09954 Asia/Jakarta
-2.13333 106.13333 Asia/Jakarta
-2.68333 111.61667 Asia/Pontianak
-6.3084 106.1067 Asia/Jakarta
-7.65268 112.6875 Asia/Jakarta
-7.91306 110.29361 Asia/Jakarta
-7.70181 113.91844 Asia/Jakarta
-6.34278 106.73833 Asia/Jakarta
-7.01833 107.60389 Asia/Jakarta
-7.1568 113.4746 Asia/Jakarta
-6.28417 107.81056 Asia/Jakarta
-0.8917 119.8707 Asia/Makassar
-6.70694 108.42417 Asia/Jakarta
-2.91673 104.7458 Asia/Jakarta
-2.2 113.83333 Asia/Pontianak
-8.0665 112.5981 Asia/Jakarta
-4.01667 103.26667 Asia/Jakarta
-0.94924 100.35427 Asia/Jakarta
-6.87666 112.37606 Asia/Jakarta
-8.1058 112.01591 Asia/Jakarta
-7.68386 112.25804 Asia/Jakarta
-7.4038 111.4461 Asia/Jakarta
-7.6051 111.9035 Asia/Jakarta
-8.31507 114.60255 Asia/Makassar
-3.36667 135.48333 Asia/Jayapura
-2.06667 105.18333 Asia/Jakarta
-7.58111 110.29278 Asia/Jakarta
-8.43333 114.33333 Asia/Jakarta
-7.0268 110.5158 Asia/Jakarta
-7.4664 112.4338 Asia/Jakarta
-7.56667 112.35 Asia/Jakarta
-6.53333 110.7 Asia/Jakarta
-5.11306 105.30667 Asia/Jakarta
-7.52 110.22639 Asia/Jakarta
-7.73333 110.36667 Asia/Jakarta
-8.6199 122.2111 Asia/Makassar
-8.58333 116.11667 Asia/Makassar
-3.41667 114.85 Asia/Jakarta
-7.1 109.01667 Asia/Jakarta
-6.97083 107.5675 Asia/Jakarta
-0.86667 134.08333 Asia/Jayapura
-2.46667 111.03333 Asia/Pontianak
-2.88333 108.26667 Asia/Jakarta
-1.03333 103.58333 Asia/Jakarta
1.487 124.8455 Asia/Makassar
-2.6748 118.8885 Asia/Makassar
-7.9797 112.6304 Asia/Jakarta
-3.5403 118.9707 Asia/Makassar
-7.2975 108.7642 Asia/Jakarta
-6.83611 108.22778 Asia/Jakarta
-7.47056 110.21778 Asia/Jakarta
-7.6298 111.5239 Asia/Jakarta
-0.9516 122.7875 Asia/Makassar
-8.1335 113.2248 Asia/Jakarta
-3.3 102.86667 Asia/Jakarta
-0.58295 117.09503 Asia/Makassar
-6.81167 107.6175 Asia/Jakarta
-7.0496 109.1441 Asia/Jakarta
-7.8353 112.6947 Asia/Jakarta
-6.6922 111.4527 Asia/Jakarta
-7.11667 112.41667 Asia/Jakarta
-3.8 103.53333 Asia/Jakarta
-8.4964 119.8877 Asia/Makassar
-6.3784 105.83 Asia/Jakarta
-7.71694 109.91278 Asia/Jakarta
-8.72332 115.17234 Asia/Jakarta
-6.97583 108.48306 Asia/Jakarta
-6.8048 110.8405 Asia/Jakarta
-0.81667 103.46667 Asia/Jakarta
-3.00913 114.38759 Asia/Pontianak
-7.63306 109.24611 Asia/Jakarta
-7.4104 112.5792 Asia/Jakarta
-6.13139 106.37972 Asia/Jakarta
-7.75845 113.39624 Asia/Jakarta
-4.83333 104.9 Asia/Jakarta
-8.53333 115.4 Asia/Makassar
-7.70583 110.60639 Asia/Jakarta
-6.70944 108.44 Asia/Jakarta
0.9 104.63333 Asia/Pontianak
-6.9383 108.891 Asia/Jakarta
-7.58333 112.1 Asia/Jakarta
-8.1303 112.5727 Asia/Jakarta
-3.945 122.49889 Asia/Makassar
-8.28333 113.36667 Asia/Jakarta
-9.44667 124.47806 Asia/Makassar
-6.97038 109.64794 Asia/Jakarta
-8.06667 111.91667 Asia/Jakarta
-7.81667 112.01667 Asia/Jakarta
-7.70028 110.56306 Asia/Jakarta
-7.16667 112.6333 Asia/Jakarta
-7.3817 108.2082 Asia/Jakarta
-4.9333 122.5167 Asia/Makassar
-6.8487 108.6422 Asia/Jakarta
-8.45 115.61667 Asia/Makassar
-7.64889 110.625 Asia/Jakarta
-6.46222 108.45194 Asia/Jakarta
-7.16778 112.71917 Asia/Jakarta
-7.05 113.93333 Asia/Jakarta
-6.715 111.1514 Asia/Jakarta
-7.54595 112.23307 Asia/Jakarta
-7.70361 110.53611 Asia/Jakarta
-8.16604 113.70317 Asia/Jakarta
-6.8057 110.9262 Asia/Jakarta
-6.73361 108.26278 Asia/Jakarta
-7.88333 111.11667 Asia/Jakarta
-6.47472 108.31528 Asia/Jakarta
-7.57722 110.8975 Asia/Jakarta
-1.6 103.61667 Asia/Jakarta
-6.21462 106.84513 Asia/Jakarta
-6.32639 108.32 Asia/Jakarta
-7.60111 110.81861 Asia/Jakarta
-7.15389 112.65611 Asia/Jakarta
0.5412 123.0595 Asia/Makassar
-8.17529 112.63594 Asia/Jakarta
-7.60722 109.51417 Asia/Jakarta
-7.76972 110.29389 Asia/Jakarta
-8.36667 114.15 Asia/Jakarta
-7.39083 112.72667 Asia/Jakarta
-6.735 110.8444 Asia/Jakarta
-7.76667 112.01667 Asia/Jakarta
-8.3939 114.1464 Asia/Jakarta
-5.3166 119.3661 Asia/Makassar
-8.8432 121.6623 Asia/Makassar
1.68333 101.45 Asia/Jakarta
-6.9 109.08333 Asia/Jakarta
-7.3659 112.6219 Asia/Jakarta
-7.57897 112.23109 Asia/Jakarta
-7.7625 110.43167 Asia/Jakarta
-6.4 106.81861 Asia/Jakarta
-8.65 115.21667 Asia/Makassar
-6.8909 110.6396 Asia/Jakarta
-7.61667 110.68333 Asia/Jakarta
-8.21162 112.74934 Asia/Jakarta
-3.46667 102.53333 Asia/Jakarta
-6.26583 106.55639 Asia/Jakarta
-6.9053 109.5347 Asia/Jakarta
-7.53333 110.75 Asia/Jakarta
-6.48556 106.88194 Asia/Jakarta
-6.7063 108.557 Asia/Jakarta
-6.2375 106.69556 Asia/Jakarta
-6.87222 107.5425 Asia/Jakarta
-6.93889 107.75278 Asia/Jakarta
-6.39472 106.95917 Asia/Jakarta
-6.26111 107.15278 Asia/Jakarta
-6.41972 107.45583 Asia/Jakarta
-6.78139 106.7825 Asia/Jakarta
-6.48167 106.85417 Asia/Jakarta
-7.3257 108.3534 Asia/Jakarta
-7.1475 111.5906 Asia/Jakarta
-7.67417 110.67889 Asia/Jakarta
-6.70611 106.82139 Asia/Jakarta
-6.55472 106.70083 Asia/Jakarta
-6.8748 108.9559 Asia/Jakarta
-0.30907 100.37055 Asia/Jakarta
-7.4281 112.7234 Asia/Jakarta
-7.05 109.55 Asia/Jakarta
-8.1181 111.8935 Asia/Jakarta
-7.53306 110.59583 Asia/Jakarta
0.13333 117.5 Asia/Makassar
-7.91346 113.82145 Asia/Jakarta
-7.1502 111.8817 Asia/Jakarta
-6.59444 106.78917 Asia/Jakarta
-6.9698 111.4186 Asia/Jakarta
-8.0983 112.1681 Asia/Jakarta
1.4451 125.1824 Asia/Makassar
-8.46667 118.71667 Asia/Makassar
-7.73379 113.69785 Asia/Makassar
-3.80044 102.26554 Asia/Jakarta
-6.2349 106.9896 Asia/Jakarta
-4.13333 104.16667 Asia/Jakarta
-7.3 109.21667 Asia/Jakarta
-7.87 112.52833 Asia/Jakarta
-6.4846 110.7083 Asia/Jakarta
-2.58333 115.38333 Asia/Makassar
-8.2325 114.35755 Asia/Makassar
-7.51417 109.29417 Asia/Jakarta
-7.88806 110.32889 Asia/Jakarta
-3.32442 114.591 Asia/Makassar
-7.04528 107.58778 Asia/Jakarta
-8.19 114.9675 Asia/Makassar
-7.1955 107.4313 Asia/Jakarta
-7.0455 112.7351 Asia/Makassar
-7.59939 112.8186 Asia/Makassar
-6.90389 107.61861 Asia/Jakarta
-7.95 110.28333 Asia/Jakarta
-7.73333 113.91667 Asia/Makassar
-1.26753 116.82887 Asia/Makassar
-7.05 109.08333 Asia/Jakarta
1.05 101.28333 Asia/Pontianak
-7.61278 110.78389 Asia/Jakarta
-6.7674 110.8541 Asia/Jakarta
-7.11282 112.16354 Asia/Jakarta
-9.10611 124.8925 Asia/Makassar
-6.8017 108.6311 Asia/Jakarta
-6.64528 108.41028 Asia/Jakarta
-2.41773 115.24941 Asia/Makassar
-3.69543 128.1814 Asia/Jayapura
-7.26333 110.3975 Asia/Jakarta
-3.33333 128.91667 Asia/Jayapura
-6.9375 109.1325 Asia/Jakarta
-6.83778 107.47278 Asia/Jakarta
-6.82 107.25722 Asia/Jakarta
-6.23639 106.50833 Asia/Jakarta
-6.09889 106.63806 Asia/Jakarta
-7.96556 110.60361 Asia/Jakarta
-7.79556 110.32639 Asia/Jakarta
-7.82694 110.32917 Asia/Jakarta
-7.54972 110.71639 Asia/Jakarta
-7.55194 110.73778 Asia/Jakarta
-7.59083 110.70444 Asia/Jakarta
-10.16667 123.58333 Asia/Makassar
-2.53333 140.7 Asia/Jayapura
-2.63333 140.58333 Asia/Jayapura
-8.1928 114.9388 Asia/Makassar
-6.36836 107.9558 Asia/Jakarta
52.33417 -6.4575 Europe/Dublin
52.25833 -7.11194 Europe/Dublin
52.27042 -9.70264 Europe/Dublin
53.2859 -6.37344 Europe/Dublin
53.45972 -6.21806 Europe/Dublin
54.26969 -8.46943 Europe/Dublin
53.18194 -6.79667 Europe/Dublin
53.65278 -6.68139 Europe/Dublin
53.21583 -6.66694 Europe/Dublin
53.53333 -7.35 Europe/Dublin
52.66472 -8.62306 Europe/Dublin
54.95 -7.73333 Europe/Dublin
53.36583 -6.49556 Europe/Dublin
52.65417 -7.25222 Europe/Dublin
53.27194 -9.04889 Europe/Dublin
52.84361 -8.98639 Europe/Dublin
53.29395 -6.13586 Europe/Dublin
54 -6.41667 Europe/Dublin
53.34399 -6.26719 Europe/Dublin
53.71889 -6.34778 Europe/Dublin
51.89797 -8.47061 Europe/Dublin
52.355 -7.70389 Europe/Dublin
53.33861 -6.54361 Europe/Dublin
52.84083 -6.92611 Europe/Dublin
53.20278 -6.09833 Europe/Dublin
53.3015 -6.1778 Europe/Dublin
53.43333 -7.95 Europe/Dublin
31.77902 35.2253 Asia/Jerusalem
32.96564 35.50049 Asia/Jerusalem
31.87639 34.74 Asia/Jerusalem
32.05043 34.75224 Asia/Jerusalem
32.51725 35.15349 Asia/Jerusalem
32.76167 34.97194 Asia/Jerusalem
32.7922 35.53124 Asia/Jerusalem
32.08088 34.78057 Asia/Jerusalem
33.01667 35.26667 Asia/Jerusalem
32.84861 35.20472 Asia/Jerusalem
31.525 34.59693 Asia/Jerusalem
32.86222 35.29333 Asia/Jerusalem
31.96417 34.80444 Asia/Jerusalem
31.92528 34.86694 Asia/Jerusalem
32.14722 34.84167 Asia/Jerusalem
32.08056 34.81417 Asia/Jerusalem
32.85083 35.07028 Asia/Jerusalem
33.20733 35.57212 Asia/Jerusalem
32.83639 35.07528 Asia/Jerusalem
31.60998 34.76422 Asia/Jerusalem
32.8275 35.08583 Asia/Jerusalem
32.80056 35.10639 Asia/Jerusalem
32.28493 34.98106 Asia/Jerusalem
32.03111 34.84583 Asia/Jerusalem
31.3141 34.62025 Asia/Jerusalem
31.42215 34.58858 Asia/Jerusalem
32.33292 34.85992 Asia/Jerusalem
31.92933 34.79868 Asia/Jerusalem
32.765 35.05 Asia/Jerusalem
32.70361 35.29556 Asia/Jerusalem
33.00586 35.09409 Asia/Jerusalem
32.67139 35.24056 Asia/Jerusalem
31.72177 35.10669 Asia/Jerusalem
32.88984 35.40703 Asia/Jerusalem
31.95139 34.89528 Asia/Jerusalem
32.175 34.90694 Asia/Jerusalem
32.91722 35.30417 Asia/Jerusalem
32.11417 34.97389 Asia/Jerusalem
32.80722 35.26028 Asia/Jerusalem
32.7466 35.34242 Asia/Jerusalem
32.92556 35.155 Asia/Jerusalem
32.01139 34.77222 Asia/Jerusalem
32.15083 34.88833 Asia/Jerusalem
32.16583 34.83667 Asia/Jerusalem
32.81556 34.98917 Asia/Jerusalem
32.43645 34.91956 Asia/Jerusalem
32.07528 34.84667 Asia/Jerusalem
32.06972 34.81167 Asia/Jerusalem
32.2341 34.95023 Asia/Jerusalem
32.26616 35.00893 Asia/Jerusalem
32.72361 35.3125 Asia/Jerusalem
29.55805 34.94821 Asia/Jerusalem
31.06667 35.03333 Asia/Jerusalem
32.69361 35.04083 Asia/Jerusalem
31.74959 34.98877 Asia/Jerusalem
32.5 35.5 Asia/Jerusalem
32.09028 34.83972 Asia/Jerusalem
31.25181 34.7913 Asia/Jerusalem
32.02306 34.75028 Asia/Jerusalem
31.66926 34.57149 Asia/Jerusalem
31.81667 34.65 Asia/Jerusalem
32.92361 35.07278 Asia/Jerusalem
32.63323 35.32522 Asia/Jerusalem
32.09174 34.88503 Asia/Jerusalem
31.77667 35.23417 Asia/Jerusalem
31.78199 35.21961 Asia/Jerusalem
54.15 -4.48333 Europe/Isle_of_Man
33.76788 74.08974 Asia/Kolkata
9.23333 78.78333 Asia/Kolkata
25.96667 94.51667 Asia/Kolkata
25.43333 83.56667 Asia/Kolkata
26.83333 81.33333 Asia/Kolkata
17.68333 77.61667 Asia/Kolkata
20.03333 74.48333 Asia/Kolkata
14.96667 74.71667 Asia/Kolkata
17.6 80.33333 Asia/Kolkata
13.1075 77.60028 Asia/Kolkata
20.4 78.13333 Asia/Kolkata
21.16667 75.7 Asia/Kolkata
16.73333 82.21667 Asia/Kolkata
30.1 77.28333 Asia/Kolkata
16.76667 77.13333 Asia/Kolkata
26.1 94.26667 Asia/Kolkata
27 77.16667 Asia/Kolkata
11.36667 76.8 Asia/Kolkata
28.21667 79.05 Asia/Kolkata
20.1 77.15 Asia/Kolkata
21.46667 78.26667 Asia/Kolkata
20.23333 79 Asia/Kolkata
25.01667 85.63333 Asia/Kolkata
20.75 78.61667 Asia/Kolkata
21.75 80.03333 Asia/Kolkata
18 79.58333 Asia/Kolkata
16.36667 78.06667 Asia/Kolkata
22.61667 70.93333 Asia/Kolkata
20.06667 78.95 Asia/Kolkata
12.93333 79.38333 Asia/Kolkata
17.93333 73.9 Asia/Kolkata
17.06667 76.98333 Asia/Kolkata
21.11667 73.4 Asia/Kolkata
16.36667 80.85 Asia/Kolkata
27.58333 77.7 Asia/Kolkata
11.5 79.33333 Asia/Kolkata
18.11667 83.41667 Asia/Kolkata
17.28333 74.55 Asia/Kolkata
23.7 72.55 Asia/Kolkata
17.69004 83.20925 Asia/Kolkata
21.38333 70.68333 Asia/Kolkata
9.6 77.96667 Asia/Kolkata
8.7 77.53333 Asia/Kolkata
12.2 75.8 Asia/Kolkata
19.46667 72.8 Asia/Kolkata
16.05 79.75 Asia/Kolkata
11.93333 79.48333 Asia/Kolkata
17.33333 77.9 Asia/Kolkata
16.51667 80.61667 Asia/Kolkata
23.56667 72.75 Asia/Kolkata
23.53333 77.81667 Asia/Kolkata
10.56667 76.93333 Asia/Kolkata
15.78333 80.31667 Asia/Kolkata
20.9 70.36667 Asia/Kolkata
17.77844 83.21577 Asia/Kolkata
13.96667 79.58333 Asia/Kolkata
18.46667 78.88333 Asia/Kolkata
11.1 78.01667 Asia/Kolkata
12.93333 79.13333 Asia/Kolkata
22.68333 73.56667 Asia/Kolkata
10.36667 79.85 Asia/Kolkata
9.7 76.33333 Asia/Kolkata
10.16667 77.76667 Asia/Kolkata
9.23333 77.41667 Asia/Kolkata
19.41667 73.26667 Asia/Kolkata
15.39585 73.81568 Asia/Kolkata
22.66667 72.73333 Asia/Kolkata
8.7341 76.70671 Asia/Kolkata
21.01667 75.9 Asia/Kolkata
25.33333 83 Asia/Kolkata
12.68333 78.61667 Asia/Kolkata
12.5 79.61667 Asia/Kolkata
20.63333 72.93333 Asia/Kolkata
10.36667 76.96667 Asia/Kolkata
22.53333 72.9 Asia/Kolkata
21.88333 71.86667 Asia/Kolkata
9.76667 76.4 Asia/Kolkata
19.91667 74.73333 Asia/Kolkata
22.3 73.2 Asia/Kolkata
23.78333 72.63333 Asia/Kolkata
14.31667 79.8 Asia/Kolkata
10.08333 77.95 Asia/Kolkata
10.46667 78.08333 Asia/Kolkata
8.38333 77.65 Asia/Kolkata
19.65 73.13333 Asia/Kolkata
12.6 79.76667 Asia/Kolkata
30.73333 78.45 Asia/Kolkata
9.8 77.33333 Asia/Kolkata
27.31667 82.41667 Asia/Kolkata
9.96667 77.8 Asia/Kolkata
27.7986 79.2393 Asia/Kolkata
14.95 77.26667 Asia/Kolkata
18.87694 72.93972 Asia/Kolkata
21.73333 70.28333 Asia/Kolkata
17.40577 78.55911 Asia/Kolkata
26.53333 80.5 Asia/Kolkata
23.8 72.4 Asia/Kolkata
23.33333 75.56667 Asia/Kolkata
31.48333 76.28333 Asia/Kolkata
20.81667 71.03333 Asia/Kolkata
29.58472 77.25417 Asia/Kolkata
23.88333 71.76667 Asia/Kolkata
22.7 73.11667 Asia/Kolkata
20.85 79.33333 Asia/Kolkata
19.66667 82.21667 Asia/Kolkata
19.6 77.7 Asia/Kolkata
23.53333 80.83333 Asia/Kolkata
17.83333 76.61667 Asia/Kolkata
12.8 74.85 Asia/Kolkata
19.21667 73.15 Asia/Kolkata
23.18333 75.76667 Asia/Kolkata
28.01667 79.01667 Asia/Kolkata
10.58333 77.25 Asia/Kolkata
13.35 74.75 Asia/Kolkata
32.93333 75.13333 Asia/Kolkata
18.38333 77.11667 Asia/Kolkata
8.43333 78.01667 Asia/Kolkata
26.76667 92.13333 Asia/Kolkata
23.08333 78.5 Asia/Kolkata
27.71667 75.48333 Asia/Kolkata
24.57117 73.69183 Asia/Kolkata
23.53333 91.48333 Asia/Kolkata
11.4 76.7 Asia/Kolkata
29.46667 76.16667 Asia/Kolkata
11.16667 78.61667 Asia/Kolkata
25.51983 90.22007 Asia/Kolkata
17.35 82.55 Asia/Kolkata
27.2 78.28333 Asia/Kolkata
21.38333 79.73333 Asia/Kolkata
13.34222 77.10167 Asia/Kolkata
27.55 82.41667 Asia/Kolkata
18 76.08333 Asia/Kolkata
26.31667 89.66667 Asia/Kolkata
26.28333 94.83333 Asia/Kolkata
8.48333 76.91667 Asia/Kolkata
10.51667 76.21667 Asia/Kolkata
26.16667 75.78333 Asia/Kolkata
9.73333 79.01667 Asia/Kolkata
29.7 75.9 Asia/Kolkata
26.01667 75.48333 Asia/Kolkata
26.91667 76.81667 Asia/Kolkata
20.3 83.15 Asia/Kolkata
22.73639 88.37361 Asia/Kolkata
8.33333 77.88333 Asia/Kolkata
12.66667 79.55 Asia/Kolkata
12.21667 79.06667 Asia/Kolkata
13.15 79.91667 Asia/Kolkata
9.38333 76.56667 Asia/Kolkata
13.18333 79.63333 Asia/Kolkata
9.48333 77.83333 Asia/Kolkata
10.9 75.91667 Asia/Kolkata
9.83333 78.26667 Asia/Kolkata
11.1 77.35 Asia/Kolkata
9.86667 78.06667 Asia/Kolkata
13.65 79.41667 Asia/Kolkata
8.73333 77.7 Asia/Kolkata
13.68333 79.35 Asia/Kolkata
11.95 79.2 Asia/Kolkata
11.38333 77.93333 Asia/Kolkata
10.81667 78.68333 Asia/Kolkata
8.48333 78.11667 Asia/Kolkata
13.7 75.23333 Asia/Kolkata
13.25861 76.47833 Asia/Kolkata
27.5 95.36667 Asia/Kolkata
13.11139 80.02611 Asia/Kolkata
12.25 79.65 Asia/Kolkata
27.98333 79.73333 Asia/Kolkata
24.745 78.8333 Asia/Kolkata
27.93333 76.85 Asia/Kolkata
24.63333 94.01667 Asia/Kolkata
10.76667 79.65 Asia/Kolkata
22.8 73.21667 Asia/Kolkata
24.39611 71.62667 Asia/Kolkata
10.8 79.15 Asia/Kolkata
29.98333 76.81667 Asia/Kolkata
19.2 72.96667 Asia/Kolkata
29.58639 77.41667 Asia/Kolkata
22.56667 71.18333 Asia/Kolkata
26.45 88.13333 Asia/Kolkata
29.2 78.85 Asia/Kolkata
26.63333 92.8 Asia/Kolkata
16.5 75.05 Asia/Kolkata
24.98333 81.65 Asia/Kolkata
8.96667 77.3 Asia/Kolkata
10 77.48333 Asia/Kolkata
11.75 75.53333 Asia/Kolkata
21.02694 76.83889 Asia/Kolkata
18.61667 84.23333 Asia/Kolkata
15.53333 76.88333 Asia/Kolkata
24.93333 84.83333 Asia/Kolkata
30.38333 78.48333 Asia/Kolkata
25.48333 85.95 Asia/Kolkata
17.03333 74.6 Asia/Kolkata
31.45 74.92528 Asia/Kolkata
13.71667 75.81667 Asia/Kolkata
11.03194 79.85278 Asia/Kolkata
28.6709 75.0351 Asia/Kolkata
23.33333 76.03333 Asia/Kolkata
11.7 77.98333 Asia/Kolkata
22.885 88.01722 Asia/Kolkata
28.21667 76.95 Asia/Kolkata
16.75 81.7 Asia/Kolkata
17.23333 77.58333 Asia/Kolkata
28.98333 78.93333 Asia/Kolkata
26.55 82.65 Asia/Kolkata
29.08333 80.11667 Asia/Kolkata
22.3 87.91667 Asia/Kolkata
31.94861 75.86667 Asia/Kolkata
30.85 74.93333 Asia/Kolkata
21.56667 74.21667 Asia/Kolkata
12.05 75.35 Asia/Kolkata
16.48333 76.31667 Asia/Kolkata
15.46667 73.8 Asia/Kolkata
18.71667 73.68333 Asia/Kolkata
20.95 85.21667 Asia/Kolkata
21.35 72.05 Asia/Kolkata
22.58611 88.92139 Asia/Kolkata
22.15 81.86667 Asia/Kolkata
25.33333 73 Asia/Kolkata
29.16667 78.48333 Asia/Kolkata
14.91667 78.01667 Asia/Kolkata
16.83333 81.5 Asia/Kolkata
16.48333 80.6 Asia/Kolkata
17.15 79.61667 Asia/Kolkata
25.46667 82.41667 Asia/Kolkata
22.7 71.68333 Asia/Kolkata
29.31667 73.9 Asia/Kolkata
21.16667 72.83333 Asia/Kolkata
8.96667 77.4 Asia/Kolkata
28.3124 75.7347 Asia/Kolkata
26.11667 86.6 Asia/Kolkata
24.36667 75.96667 Asia/Kolkata
31.53333 76.88333 Asia/Kolkata
22.11667 84.03333 Asia/Kolkata
30.13333 75.8 Asia/Kolkata
12.56667 75.38333 Asia/Kolkata
13.7 80.01667 Asia/Kolkata
11.03333 77.13333 Asia/Kolkata
31.21528 75.19972 Asia/Kolkata
26.26667 82.06667 Asia/Kolkata
24.65 76.03333 Asia/Kolkata
29.03333 79.05 Asia/Kolkata
9.51667 77.63333 Asia/Kolkata
18.03333 73.01667 Asia/Kolkata
8.61667 77.93333 Asia/Kolkata
16.08333 78.86667 Asia/Kolkata
17.26652 78.25544 Asia/Kolkata
12.96889 79.94889 Asia/Kolkata
13.33333 78.21667 Asia/Kolkata
34.08333 74.81667 Asia/Kolkata
30.21667 78.78333 Asia/Kolkata
27.46667 75.6 Asia/Kolkata
29.84194 73.45417 Asia/Kolkata
18.3 83.9 Asia/Kolkata
28.08333 74.01667 Asia/Kolkata
20.6 75.61667 Asia/Kolkata
27.88333 78.75 Asia/Kolkata
21.28806 86.68583 Asia/Kolkata
19.75 84.43333 Asia/Kolkata
34.3 74.46667 Asia/Kolkata
28.98333 77.01667 Asia/Kolkata
21.16667 73.56667 Asia/Kolkata
20.83333 83.91667 Asia/Kolkata
27.06667 95.03333 Asia/Kolkata
23.3 87.41667 Asia/Kolkata
18.93333 84.6 Asia/Kolkata
13.5 75.06667 Asia/Kolkata
30.91667 77.11667 Asia/Kolkata
22.55 72.71667 Asia/Kolkata
25.91667 73.66667 Asia/Kolkata
28.25 77.06667 Asia/Kolkata
22.7 78.2 Asia/Kolkata
25.63333 72.41667 Asia/Kolkata
26.21667 84.36667 Asia/Kolkata
9.45 77.81667 Asia/Kolkata
11.11667 77.8 Asia/Kolkata
9.33333 77.43333 Asia/Kolkata
9.86667 78.48333 Asia/Kolkata
23.91667 87.53333 Asia/Kolkata
28.93333 79.7 Asia/Kolkata
27.56667 80.68333 Asia/Kolkata
26.6 85.48333 Asia/Kolkata
27.15 83.76667 Asia/Kolkata
29.41667 77.46667 Asia/Kolkata
18.83333 74.38333 Asia/Kolkata
11.33333 77.01667 Asia/Kolkata
15.63333 76.9 Asia/Kolkata
18.38333 78.83333 Asia/Kolkata
28.65 78.65 Asia/Kolkata
14.61667 74.85 Asia/Kolkata
27.05 78.7 Asia/Kolkata
29.53333 75.01667 Asia/Kolkata
24.1 77.7 Asia/Kolkata
24.88333 72.86667 Asia/Kolkata
11.23333 79.73333 Asia/Kolkata
30.64556 76.3825 Asia/Kolkata
13.74528 76.90917 Asia/Kolkata
19.85 74 Asia/Kolkata
22.80917 88.22944 Asia/Kolkata
15.25 80.03333 Asia/Kolkata
11 77.01667 Asia/Kolkata
15.78333 76.76667 Asia/Kolkata
16.91667 76.23333 Asia/Kolkata
31.1 77.16667 Asia/Kolkata
22.61667 84.51667 Asia/Kolkata
20.26667 73.01667 Asia/Kolkata
20.3 75.65 Asia/Kolkata
24.81667 92.8 Asia/Kolkata
25.08333 85.41667 Asia/Kolkata
27.61667 75.15 Asia/Kolkata
27.7 78.4 Asia/Kolkata
26.05 84.05 Asia/Kolkata
28.45 77.7 Asia/Kolkata
22.43333 69.83333 Asia/Kolkata
23.78333 86.31667 Asia/Kolkata
23.48333 80.11667 Asia/Kolkata
21.7 71.96667 Asia/Kolkata
13.38806 77.86278 Asia/Kolkata
24.41667 81.88333 Asia/Kolkata
27.28333 80.83333 Asia/Kolkata
18.1 78.85 Asia/Kolkata
23.91667 72.38333 Asia/Kolkata
26.98333 94.63333 Asia/Kolkata
23.4 76.71667 Asia/Kolkata
12.41361 76.70417 Asia/Kolkata
22.75278 88.34222 Asia/Kolkata
18.61667 74.68333 Asia/Kolkata
16.51667 76.75 Asia/Kolkata
10.76667 76.28333 Asia/Kolkata
13.11667 79.41667 Asia/Kolkata
17.68333 75.91667 Asia/Kolkata
25.43333 77.65 Asia/Kolkata
28.71667 79.31667 Asia/Kolkata
21.35 74.88333 Asia/Kolkata
15.23333 75.58333 Asia/Kolkata
19.76667 74.48333 Asia/Kolkata
13.91667 75.56667 Asia/Kolkata
25.56892 91.88313 Asia/Kolkata
26.7 88.43333 Asia/Kolkata
27.1 78.6 Asia/Kolkata
28.28333 78.01667 Asia/Kolkata
14.26667 75.35 Asia/Kolkata
15 75.23333 Asia/Kolkata
9.7 76.33333 Asia/Kolkata
29.35 78.58333 Asia/Kolkata
24.56667 84.78333 Asia/Kolkata
25.66667 76.7 Asia/Kolkata
26.51667 85.3 Asia/Kolkata
25.15 73.06667 Asia/Kolkata
20.78333 76.68333 Asia/Kolkata
23.25 88.46667 Asia/Kolkata
27.53333 79.43333 Asia/Kolkata
27.01667 78.13333 Asia/Kolkata
29.45 77.31667 Asia/Kolkata
24.18333 75.63333 Asia/Kolkata
23.43333 76.26667 Asia/Kolkata
25.15 85.85 Asia/Kolkata
27.38333 75.96667 Asia/Kolkata
25.63333 74.93333 Asia/Kolkata
29.35 77.55 Asia/Kolkata
25.58333 84.45 Asia/Kolkata
21.23333 76.21667 Asia/Kolkata
16.7 76.83333 Asia/Kolkata
27.88333 79.91667 Asia/Kolkata
28.55 79.31667 Asia/Kolkata
26.05 82.68333 Asia/Kolkata
23.28333 81.35 Asia/Kolkata
21.55 74.46667 Asia/Kolkata
30.1681 76.8715 Asia/Kolkata
28.56667 79.01667 Asia/Kolkata
27.65 79.95 Asia/Kolkata
17.13333 76.93333 Asia/Kolkata
23.3 92.83333 Asia/Kolkata
17.18333 77.28333 Asia/Kolkata
22.45 77.46667 Asia/Kolkata
22.08333 79.53333 Asia/Kolkata
26.15778 78.77972 Asia/Kolkata
29.21667 78.58333 Asia/Kolkata
21.68333 75.1 Asia/Kolkata
23.2 77.08333 Asia/Kolkata
22.55 71.46667 Asia/Kolkata
25.98333 76.36667 Asia/Kolkata
21.15 75.88333 Asia/Kolkata
14.96667 75.35 Asia/Kolkata
15.9 73.81667 Asia/Kolkata
21.65 78.78333 Asia/Kolkata
15.78333 75.11667 Asia/Kolkata
11.51667 77.25 Asia/Kolkata
9.36667 77.93333 Asia/Kolkata
16.4 80.18333 Asia/Kolkata
24.58333 80.83333 Asia/Kolkata
17.68333 73.98333 Asia/Kolkata
20.58333 74.2 Asia/Kolkata
18.33333 74.03333 Asia/Kolkata
26.06667 75 Asia/Kolkata
22.98333 72.5 Asia/Kolkata
29.7 75.25 Asia/Kolkata
29.15 77.61667 Asia/Kolkata
28.43333 74.48333 Asia/Kolkata
28.5 79.1 Asia/Kolkata
23.56667 76.46667 Asia/Kolkata
21.33333 83 Asia/Kolkata
26.01667 82.91667 Asia/Kolkata
25.38333 81.51667 Asia/Kolkata
21.38333 78.9 Asia/Kolkata
22.55 88.2 Asia/Kolkata
16.26667 74.48333 Asia/Kolkata
30.23333 75.83333 Asia/Kolkata
17.43333 75.2 Asia/Kolkata
24.91667 76.28333 Asia/Kolkata
16.85438 74.56417 Asia/Kolkata
29.8 74.45 Asia/Kolkata
17.62944 78.09167 Asia/Kolkata
19.56667 74.21667 Asia/Kolkata
15.1 76.55 Asia/Kolkata
27.08333 80.51667 Asia/Kolkata
27.3 79.95 Asia/Kolkata
15.36667 73.9 Asia/Kolkata
24.75528 71.77222 Asia/Kolkata
22.18333 76.06667 Asia/Kolkata
30.3025 76.45528 Asia/Kolkata
22.98333 72.38333 Asia/Kolkata
25.85 78.91667 Asia/Kolkata
30.83667 76.18972 Asia/Kolkata
25.81667 72.58333 Asia/Kolkata
26.91667 75.2 Asia/Kolkata
28.58333 78.55 Asia/Kolkata
21.45 83.96667 Asia/Kolkata
32.56667 75.11667 Asia/Kolkata
25.85 85.78333 Asia/Kolkata
17.05 82.18333 Asia/Kolkata
29.23333 77.01667 Asia/Kolkata
18.53333 83.21667 Asia/Kolkata
24.13333 74.05 Asia/Kolkata
11.65 78.16667 Asia/Kolkata
22.31667 69.6 Asia/Kolkata
22.03333 82.96667 Asia/Kolkata
12.96667 75.78333 Asia/Kolkata
12.99639 80.20306 Asia/Kolkata
23.95 87.66667 Asia/Kolkata
19.46667 76.46667 Asia/Kolkata
22.48333 92.96667 Asia/Kolkata
25.55 83.18333 Asia/Kolkata
25.25 87.65 Asia/Kolkata
27.8 78.85 Asia/Kolkata
28.08333 78.75 Asia/Kolkata
29.13333 78.61667 Asia/Kolkata
25.88333 86.6 Asia/Kolkata
29.96667 77.55 Asia/Kolkata
26.78333 84.73333 Asia/Kolkata
23.83333 78.71667 Asia/Kolkata
14.16667 75.03333 Asia/Kolkata
26.73333 80.35 Asia/Kolkata
29.41667 76.66667 Asia/Kolkata
25.18333 73.43333 Asia/Kolkata
17.61667 77.95 Asia/Kolkata
16.56667 74.55 Asia/Kolkata
27.45 78.05 Asia/Kolkata
26.25 77.4 Asia/Kolkata
25.75 86.03333 Asia/Kolkata
26.48333 79.9 Asia/Kolkata
30.96639 76.53306 Asia/Kolkata
26.43944 83.61444 Asia/Kolkata
29.86667 77.88333 Asia/Kolkata
15.66667 75.73333 Asia/Kolkata
28.9 76.56667 Asia/Kolkata
18.43333 73.11667 Asia/Kolkata
12.96667 78.28333 Asia/Kolkata
24.7 83.06667 Asia/Kolkata
19.96667 76.78333 Asia/Kolkata
22.71417 88.34778 Asia/Kolkata
30.11667 78.31667 Asia/Kolkata
27.35 75.56667 Asia/Kolkata
28.7 79.51667 Asia/Kolkata
28.18333 76.61667 Asia/Kolkata
24.53333 81.3 Asia/Kolkata
25.78333 84.66667 Asia/Kolkata
16.01667 80.85 Asia/Kolkata
25.85 84.38333 Asia/Kolkata
24.2 83.03333 Asia/Kolkata
13.65 79.51667 Asia/Kolkata
21.52611 86.87222 Asia/Kolkata
23.63333 79.08333 Asia/Kolkata
18.46667 83.66667 Asia/Kolkata
16.48333 74.78333 Asia/Kolkata
14.7 76.86667 Asia/Kolkata
14.05 78.75 Asia/Kolkata
27.56667 77.78333 Asia/Kolkata
26.98333 84.85 Asia/Kolkata
29.28333 74.38333 Asia/Kolkata
24.93333 75.58333 Asia/Kolkata
21.25 76.03333 Asia/Kolkata
16.98333 73.3 Asia/Kolkata
23.31667 75.06667 Asia/Kolkata
29.68333 75.58333 Asia/Kolkata
25.58333 79.56667 Asia/Kolkata
22.3 82.16667 Asia/Kolkata
28.08333 74.6 Asia/Kolkata
25.85 83.85 Asia/Kolkata
11.46667 78.16667 Asia/Kolkata
23.56667 70.63333 Asia/Kolkata
25.25 79.06667 Asia/Kolkata
29.65 79.41667 Asia/Kolkata
23.61667 87.13333 Asia/Kolkata
14.61667 75.61667 Asia/Kolkata
29.53333 74.83333 Asia/Kolkata
26.46667 91.63333 Asia/Kolkata
26.81667 92.65 Asia/Kolkata
23.35 85.33333 Asia/Kolkata
21.68417 69.74083 Asia/Kolkata
23.18333 88.58333 Asia/Kolkata
21.4 79.33333 Asia/Kolkata
24.16667 87.78333 Asia/Kolkata
30.25 75.23333 Asia/Kolkata
24.46667 75.43333 Asia/Kolkata
29.81667 77.45 Asia/Kolkata
28.81667 79.03333 Asia/Kolkata
29.4 79.11667 Asia/Kolkata
27.16667 84.31667 Asia/Kolkata
25.28333 83.03333 Asia/Kolkata
22.83333 87.61667 Asia/Kolkata
27.25 75.18333 Asia/Kolkata
23.63333 85.51667 Asia/Kolkata
24.65 75.93333 Asia/Kolkata
9.28333 79.3 Asia/Kolkata
13.11667 79.2 Asia/Kolkata
9.38333 78.83333 Asia/Kolkata
12.71667 77.3 Asia/Kolkata
16.85 82.01667 Asia/Kolkata
19.78333 79.36667 Asia/Kolkata
21.05 71.43333 Asia/Kolkata
25.06667 73.88333 Asia/Kolkata
30.48306 76.595 Asia/Kolkata
22.3 74.35 Asia/Kolkata
21.93333 75.13333 Asia/Kolkata
21.86667 73.5 Asia/Kolkata
21.1 81.03333 Asia/Kolkata
25.05 87.83333 Asia/Kolkata
22.3 70.78333 Asia/Kolkata
18.86667 73.9 Asia/Kolkata
25.03333 85.41667 Asia/Kolkata
28.63333 75.38333 Asia/Kolkata
27.23333 76.63333 Asia/Kolkata
24 76.71667 Asia/Kolkata
22.68333 74.95 Asia/Kolkata
33.38333 74.3 Asia/Kolkata
9.45 77.56667 Asia/Kolkata
14.18333 79.16667 Asia/Kolkata
28.03333 74.46667 Asia/Kolkata
26.89611 78.17139 Asia/Kolkata
16.98333 81.78333 Asia/Kolkata
29.53583 73.44917 Asia/Kolkata
23.33333 77.8 Asia/Kolkata
30.31667 78.1 Asia/Kolkata
26.05 74.01667 Asia/Kolkata
21.23333 81.63333 Asia/Kolkata
30.65 75.6 Asia/Kolkata
18.25 73.43333 Asia/Kolkata
21.9 83.4 Asia/Kolkata
25.61667 88.11667 Asia/Kolkata
16.2 77.36667 Asia/Kolkata
19.38333 74.65 Asia/Kolkata
17.6 74.2 Asia/Kolkata
23.78333 78.36667 Asia/Kolkata
23.55 86.66667 Asia/Kolkata
24.45 77.2 Asia/Kolkata
24.81667 84.65 Asia/Kolkata
26.21667 81.23333 Asia/Kolkata
23.83333 71.6 Asia/Kolkata
16.46667 75.1 Asia/Kolkata
8.88333 76.6 Asia/Kolkata
25.85 87.55 Asia/Kolkata
31.81778 75.39194 Asia/Kolkata
13.45 79.55 Asia/Kolkata
12.76667 75.21667 Asia/Kolkata
26.5 74.55 Asia/Kolkata
19.9 77.58333 Asia/Kolkata
26.46667 80.78333 Asia/Kolkata
23.33333 86.36667 Asia/Kolkata
25.78333 87.46667 Asia/Kolkata
19.18333 77.05 Asia/Kolkata
19.8 85.85 Asia/Kolkata
28.51667 80.15 Asia/Kolkata
26.48333 85.71667 Asia/Kolkata
11.35 77.2 Asia/Kolkata
13.36667 78.58333 Asia/Kolkata
18.51957 73.85535 Asia/Kolkata
29.75 76.55 Asia/Kolkata
22.23333 76.4 Asia/Kolkata
9 76.93333 Asia/Kolkata
27.86667 77.2 Asia/Kolkata
33.87405 74.89955 Asia/Kolkata
9.16667 77.41667 Asia/Kolkata
14.41667 78.23333 Asia/Kolkata
20.73333 78.33333 Asia/Kolkata
26.23333 79.85 Asia/Kolkata
10.38333 78.81667 Asia/Kolkata
14.73333 78.55 Asia/Kolkata
24.03333 74.78333 Asia/Kolkata
11.66667 92.75 Asia/Kolkata
26.67 78.37472 Asia/Kolkata
21.64219 69.60929 Asia/Kolkata
13.04833 80.1075 Asia/Kolkata
16.06667 80.56667 Asia/Kolkata
13.31667 80.2 Asia/Kolkata
10.76695 75.92523 Asia/Kolkata
11.93 79.83 Asia/Kolkata
15.4 74.01667 Asia/Kolkata
12.5 79.13333 Asia/Kolkata
10.66667 77.01667 Asia/Kolkata
19.7 84.81667 Asia/Kolkata
26.91667 71.91667 Asia/Kolkata
29.58333 80.21667 Asia/Kolkata
17.11667 82.26667 Asia/Kolkata
25.33333 84.41667 Asia/Kolkata
12.33972 76.09917 Asia/Kolkata
9.86667 76.5 Asia/Kolkata
19.8 75.53333 Asia/Kolkata
26.82639 83.53 Asia/Kolkata
20.11667 85.83333 Asia/Kolkata
26.38556 73.53778 Asia/Kolkata
30.79722 76.91722 Asia/Kolkata
24.8 73.06667 Asia/Kolkata
26.88389 78.37806 Asia/Kolkata
18.61667 73.8 Asia/Kolkata
28.71667 77.65 Asia/Kolkata
28.63333 79.8 Asia/Kolkata
29.45 74.08333 Asia/Kolkata
28.3684 75.6041 Asia/Kolkata
27.63333 80.2 Asia/Kolkata
25.55 82.1 Asia/Kolkata
26.86667 75.23333 Asia/Kolkata
20.46667 84.23333 Asia/Kolkata
16.3 80.26667 Asia/Kolkata
31.02917 75.78417 Asia/Kolkata
25.66667 94.5 Asia/Kolkata
26.6 79.46667 Asia/Kolkata
17.98333 74.43333 Asia/Kolkata
27.13333 72.36667 Asia/Kolkata
29.18333 77.81667 Asia/Kolkata
31.21778 75.76944 Asia/Kolkata
22.46667 72.8 Asia/Kolkata
11.26667 77.58333 Asia/Kolkata
10.13333 76.48333 Asia/Kolkata
11.15 76.93333 Asia/Kolkata
10.11667 77.55 Asia/Kolkata
10.3 79.18333 Asia/Kolkata
12.93333 78.71667 Asia/Kolkata
11.23333 78.88333 Asia/Kolkata
14.08333 77.58333 Asia/Kolkata
16.66667 81.73333 Asia/Kolkata
12.13333 77.9 Asia/Kolkata
11.4 79.23333 Asia/Kolkata
18.75 73.08333 Asia/Kolkata
29.98333 76.58333 Asia/Kolkata
17.08333 82.13333 Asia/Kolkata
18.61667 79.36667 Asia/Kolkata
16.26667 81.16667 Asia/Kolkata
12.1 75.2 Asia/Kolkata
28.06667 80.1 Asia/Kolkata
14.1 77.26667 Asia/Kolkata
30.15 78.78333 Asia/Kolkata
20.78333 79.63333 Asia/Kolkata
20.45 76.93333 Asia/Kolkata
10.43333 79.31667 Asia/Kolkata
31.28083 74.85722 Asia/Kolkata
20.71667 83.15 Asia/Kolkata
25.6 85.11667 Asia/Kolkata
30.32667 76.40028 Asia/Kolkata
19.25 76.45 Asia/Kolkata
23.9 79.2 Asia/Kolkata
23.66667 86.43333 Asia/Kolkata
19.16667 75.18333 Asia/Kolkata
32.28333 75.65 Asia/Kolkata
9.26667 76.78333 Asia/Kolkata
22.56667 83.46667 Asia/Kolkata
28.31667 76.78333 Asia/Kolkata
17.53139 78.265 Asia/Kolkata
23.83333 72.11667 Asia/Kolkata
20.56667 86.56667 Asia/Kolkata
28.06667 95.33333 Asia/Kolkata
22.85 82.2 Asia/Kolkata
26.88333 74.76667 Asia/Kolkata
18.78333 83.43333 Asia/Kolkata
19.6 76.21667 Asia/Kolkata
20.88333 75.11667 Asia/Kolkata
18.85 76.53333 Asia/Kolkata
18.76667 84.08333 Asia/Kolkata
28.98333 77.93333 Asia/Kolkata
20.51667 72.95 Asia/Kolkata
19.26667 76.78333 Asia/Kolkata
8.78333 76.7 Asia/Kolkata
22.2 78.76667 Asia/Kolkata
9.55 78.6 Asia/Kolkata
20.31667 86.61667 Asia/Kolkata
11.95 75.35 Asia/Kolkata
10.93333 79.28333 Asia/Kolkata
30.45 77.61667 Asia/Kolkata
18.98333 73.1 Asia/Kolkata
11.76667 79.55 Asia/Kolkata
24.71667 80.2 Asia/Kolkata
29.38889 76.96806 Asia/Kolkata
22.69417 88.37444 Asia/Kolkata
23.08333 88.28333 Asia/Kolkata
21.6 78.51667 Asia/Kolkata
17.66667 75.33333 Asia/Kolkata
12.50056 76.67333 Asia/Kolkata
15.48333 73.83333 Asia/Kolkata
23.3 79.98333 Asia/Kolkata
28.15 77.33333 Asia/Kolkata
17.6 80.68333 Asia/Kolkata
10.46667 77.53333 Asia/Kolkata
13.2 78.75 Asia/Kolkata
11.93333 78.41667 Asia/Kolkata
12.91667 78.93333 Asia/Kolkata
12.97611 80.18361 Asia/Kolkata
10.7 77.88333 Asia/Kolkata
10.98333 77.3 Asia/Kolkata
18.6 83.75 Asia/Kolkata
21.51667 71.83333 Asia/Kolkata
28.45 80.58333 Asia/Kolkata
25.76667 73.33333 Asia/Kolkata
23.35 81.05 Asia/Kolkata
10.78333 76.65 Asia/Kolkata
19.68333 72.75 Asia/Kolkata
25.01667 79.23333 Asia/Kolkata
18.76667 84.41667 Asia/Kolkata
24.16667 72.43333 Asia/Kolkata
16.53333 81.73333 Asia/Kolkata
12.3 78.08333 Asia/Kolkata
24.63333 87.85 Asia/Kolkata
13.46667 79.11667 Asia/Kolkata
19.48333 75.38333 Asia/Kolkata
28.18333 78.05 Asia/Kolkata
26.9025 83.98194 Asia/Kolkata
22.23333 73.08333 Asia/Kolkata
8.23333 77.33333 Asia/Kolkata
29.68333 73.61667 Asia/Kolkata
20.98333 83.06667 Asia/Kolkata
27.51667 82.65 Asia/Kolkata
20.66667 75.35 Asia/Kolkata
10.76667 76.38333 Asia/Kolkata
18.16667 76.05 Asia/Kolkata
25.98333 79.46667 Asia/Kolkata
15.5 80.05 Asia/Kolkata
22.46944 69.06056 Asia/Kolkata
20.1 73.93333 Asia/Kolkata
24.41667 82.98333 Asia/Kolkata
16.78333 80.85 Asia/Kolkata
29.15 78.41667 Asia/Kolkata
27.23333 94.11667 Asia/Kolkata
26.20139 91.72361 Asia/Kolkata
25.51667 91.26667 Asia/Kolkata
27.6 73.41667 Asia/Kolkata
29.18333 74.76667 Asia/Kolkata
22.15 85.53333 Asia/Kolkata
18.66667 78.11667 Asia/Kolkata
26.31667 86.58333 Asia/Kolkata
19.1 78.35 Asia/Kolkata
16.4 74.38333 Asia/Kolkata
27.73333 75.8 Asia/Kolkata
24.61667 74.68333 Asia/Kolkata
20.06667 86.01667 Asia/Kolkata
26.14972 74 Asia/Kolkata
29.83333 76.91667 Asia/Kolkata
12.25 75.1 Asia/Kolkata
18.1 76.76667 Asia/Kolkata
10.16667 77.86667 Asia/Kolkata
21.45806 86.76694 Asia/Kolkata
29.33333 78.38333 Asia/Kolkata
16.91667 81.66667 Asia/Kolkata
27.31667 83.73333 Asia/Kolkata
8.4 77.08333 Asia/Kolkata
28.63576 77.22445 Asia/Kolkata
19.03333 73.31667 Asia/Kolkata
21.46667 76.4 Asia/Kolkata
14.43333 79.96667 Asia/Kolkata
11.76667 79.68333 Asia/Kolkata
13.10222 77.39 Asia/Kolkata
8.6 77 Asia/Kolkata
13.9 79.9 Asia/Kolkata
20.13333 85.1 Asia/Kolkata
31.11667 76.13333 Asia/Kolkata
27.85 75.26667 Asia/Kolkata
26.35 75.91667 Asia/Kolkata
24.88333 85.53333 Asia/Kolkata
28.55 79.63333 Asia/Kolkata
26.93333 81.21667 Asia/Kolkata
26.86667 82.13333 Asia/Kolkata
27.01667 75 Asia/Kolkata
15.56667 75.36667 Asia/Kolkata
23.41667 88.36667 Asia/Kolkata
27.43333 83.41667 Asia/Kolkata
25.4 87.1 Asia/Kolkata
10.23333 78.23333 Asia/Kolkata
24.93333 73.81667 Asia/Kolkata
22.68333 77.26667 Asia/Kolkata
25.05 84.33333 Asia/Kolkata
18.83333 79.45 Asia/Kolkata
26.3 74.73333 Asia/Kolkata
19.98333 73.8 Asia/Kolkata
25.65 77.9 Asia/Kolkata
29.61667 76.11667 Asia/Kolkata
17.66667 82.61667 Asia/Kolkata
23.7 77.1 Asia/Kolkata
22.95 79.2 Asia/Kolkata
29.21667 76.15 Asia/Kolkata
28.04444 76.10833 Asia/Kolkata
15.71667 75.38333 Asia/Kolkata
15.58333 75.81667 Asia/Kolkata
16.73333 77.5 Asia/Kolkata
30.48333 77.13333 Asia/Kolkata
13.19556 80.17417 Asia/Kolkata
28.2 78.38333 Asia/Kolkata
28.5 78.71667 Asia/Kolkata
16.25 80.06667 Asia/Kolkata
16.45 81.66667 Asia/Kolkata
18.41667 84.05 Asia/Kolkata
25.18333 80.48333 Asia/Kolkata
26.78333 75.2 Asia/Kolkata
27.96667 73.55 Asia/Kolkata
27.86667 81.5 Asia/Kolkata
12.11972 76.68278 Asia/Kolkata
28.68333 77.06667 Asia/Kolkata
31.36667 76.38333 Asia/Kolkata
15.48333 78.48333 Asia/Kolkata
21.36667 74.25 Asia/Kolkata
20.83333 76.45 Asia/Kolkata
15.86667 78.26667 Asia/Kolkata
16.78333 80.3 Asia/Kolkata
20.31667 74.65 Asia/Kolkata
29.7125 77.41583 Asia/Kolkata
27.18333 95.33333 Asia/Kolkata
11.36667 77.33333 Asia/Kolkata
11.22126 78.16524 Asia/Kolkata
11.46667 78.26667 Asia/Kolkata
24.3 87.81667 Asia/Kolkata
17.05 79.26667 Asia/Kolkata
17.81667 76.3 Asia/Kolkata
29.91667 77.3 Asia/Kolkata
26.68333 88.21667 Asia/Kolkata
31.12583 75.47333 Asia/Kolkata
29.63333 78.33333 Asia/Kolkata
25.76667 75.85 Asia/Kolkata
22.43333 80.11667 Asia/Kolkata
29.39743 79.44686 Asia/Kolkata
22.90278 88.41694 Asia/Kolkata
27.28333 95.33333 Asia/Kolkata
30.55 77.3 Asia/Kolkata
21.15 79.1 Asia/Kolkata
24.56667 80.6 Asia/Kolkata
29.45 78.45 Asia/Kolkata
8.16667 77.43333 Asia/Kolkata
23.45 75.41667 Asia/Kolkata
27.2 73.73333 Asia/Kolkata
16.4821 78.32471 Asia/Kolkata
13.33333 79.58333 Asia/Kolkata
27.43333 77.1 Asia/Kolkata
10.76667 79.83333 Asia/Kolkata
12.81556 76.7575 Asia/Kolkata
22.7 72.86667 Asia/Kolkata
27.23333 77.2 Asia/Kolkata
11.7 75.66667 Asia/Kolkata
24.61667 84.11667 Asia/Kolkata
30.37472 76.14861 Asia/Kolkata
12.30722 76.64972 Asia/Kolkata
26.11667 85.4 Asia/Kolkata
29.46667 77.68333 Asia/Kolkata
9.96667 76.58333 Asia/Kolkata
10.4 79.48333 Asia/Kolkata
30.45 78.08333 Asia/Kolkata
10.93333 78.45 Asia/Kolkata
22.51667 86.45 Asia/Kolkata
23.85 80.4 Asia/Kolkata
20.73333 77.38333 Asia/Kolkata
24.18333 88.26667 Asia/Kolkata
25.9 86.98333 Asia/Kolkata
30.79194 76.49639 Asia/Kolkata
19.25 73.4 Asia/Kolkata
28.78333 77.5 Asia/Kolkata
10.1 77.06667 Asia/Kolkata
25.38333 86.46667 Asia/Kolkata
22.06667 81.68333 Asia/Kolkata
24.41667 78.1 Asia/Kolkata
27.06667 73.81667 Asia/Kolkata
22.85 69.73333 Asia/Kolkata
14.96667 75.03333 Asia/Kolkata
15.21667 75.9 Asia/Kolkata
21.76667 78.25 Asia/Kolkata
13.1 74.8 Asia/Kolkata
15.25 75.53333 Asia/Kolkata
13.16667 78.4 Asia/Kolkata
11.8 75.45 Asia/Kolkata
20.06667 79.66667 Asia/Kolkata
30.48333 74.51667 Asia/Kolkata
18.7 77.36667 Asia/Kolkata
31.95056 75.61583 Asia/Kolkata
27.31667 79.45 Asia/Kolkata
26.03306 83.38389 Asia/Kolkata
25.63333 83.75 Asia/Kolkata
25.3 83.11667 Asia/Kolkata
19.16667 77.51667 Asia/Kolkata
16.35 75.28333 Asia/Kolkata
16.01667 76.43333 Asia/Kolkata
16.33333 76.13333 Asia/Kolkata
13.08333 74.98333 Asia/Kolkata
26.09 83.29222 Asia/Kolkata
26.65 84.91667 Asia/Kolkata
22.9 73.83333 Asia/Kolkata
21.33917 78.01306 Asia/Kolkata
26.49694 78 Asia/Kolkata
22.81667 70.83333 Asia/Kolkata
26.22611 78.22611 Asia/Kolkata
17.8 76.46667 Asia/Kolkata
28.83333 78.78333 Asia/Kolkata
26.75 95.1 Asia/Kolkata
26.33333 94.53333 Asia/Kolkata
25.4 85.91667 Asia/Kolkata
24.5 93.76667 Asia/Kolkata
30.8 75.16667 Asia/Kolkata
23.46667 73.3 Asia/Kolkata
27.45 80.51667 Asia/Kolkata
25.15 82.58333 Asia/Kolkata
16.86667 79.58333 Asia/Kolkata
28.55 79.2 Asia/Kolkata
28.03333 79.65 Asia/Kolkata
29.3 77.93333 Asia/Kolkata
13.26667 80.26667 Asia/Kolkata
28.61667 79.18333 Asia/Kolkata
26.28444 78.98167 Asia/Kolkata
17.63333 74.78333 Asia/Kolkata
11.8 77.8 Asia/Kolkata
11.3 76.95 Asia/Kolkata
26.65 74.03333 Asia/Kolkata
21.31667 70.43333 Asia/Kolkata
23.2 88.11667 Asia/Kolkata
10.05 78.33333 Asia/Kolkata
26.97528 83.11167 Asia/Kolkata
20.15 76.56667 Asia/Kolkata
28.98333 77.7 Asia/Kolkata
22.43333 87.33333 Asia/Kolkata
18.03333 78.26667 Asia/Kolkata
11.1 79.66667 Asia/Kolkata
24.61667 93.88333 Asia/Kolkata
29.1 77.91667 Asia/Kolkata
11.26667 75.91667 Asia/Kolkata
9.26667 76.55 Asia/Kolkata
30.08333 75.25 Asia/Kolkata
24.68333 81.88333 Asia/Kolkata
25.68333 80.11667 Asia/Kolkata
25.7 81.91667 Asia/Kolkata
26.26694 78.67306 Asia/Kolkata
25.95 83.55 Asia/Kolkata
11.91667 75.58333 Asia/Kolkata
27.5 77.68333 Asia/Kolkata
26.33333 89.21667 Asia/Kolkata
25.35 85.03333 Asia/Kolkata
15.4 73.8 Asia/Kolkata
15.73333 79.28333 Asia/Kolkata
26.66667 94.33333 Asia/Kolkata
25.61667 82.61667 Asia/Kolkata
25.96667 84.86667 Asia/Kolkata
27.28333 95.68333 Asia/Kolkata
12.2 79.95 Asia/Kolkata
27.73333 78.58333 Asia/Kolkata
15.59154 73.80898 Asia/Kolkata
19.3 76.5 Asia/Kolkata
15.98333 77.05 Asia/Kolkata
18.65 79.66667 Asia/Kolkata
29.98333 75.38333 Asia/Kolkata
23.43333 72.66667 Asia/Kolkata
27.3 75.95 Asia/Kolkata
10.66667 79.43333 Asia/Kolkata
10.98333 76.46667 Asia/Kolkata
20.25 74.45 Asia/Kolkata
25.53333 89.86667 Asia/Kolkata
19.15 76.23333 Asia/Kolkata
25.53333 81.38333 Asia/Kolkata
11.11667 76.11667 Asia/Kolkata
25.35 87.63333 Asia/Kolkata
25.98333 84.16667 Asia/Kolkata
20.31667 77.35 Asia/Kolkata
25.33333 76.51667 Asia/Kolkata
21.11667 70.11667 Asia/Kolkata
29.8 77.86667 Asia/Kolkata
12.86667 74.88333 Asia/Kolkata
26.43333 92.03333 Asia/Kolkata
16.43333 80.55 Asia/Kolkata
25.65 84.88333 Asia/Kolkata
12.52417 76.89583 Asia/Kolkata
22.83333 69.36667 Asia/Kolkata
21.25 73.3 Asia/Kolkata
24.06667 75.06667 Asia/Kolkata
22.6 80.38333 Asia/Kolkata
31.71667 76.91667 Asia/Kolkata
29.5 78.13333 Asia/Kolkata
16.86667 81.93333 Asia/Kolkata
9.28333 79.11667 Asia/Kolkata
25.2 75.1 Asia/Kolkata
25.45 74.56667 Asia/Kolkata
18.86667 79.43333 Asia/Kolkata
22.23333 75.08333 Asia/Kolkata
21.5 70.13333 Asia/Kolkata
24.48371 75.26871 Asia/Kolkata
10.6 78.41667 Asia/Kolkata
9.7 78.48333 Asia/Kolkata
13.16667 80.26667 Asia/Kolkata
16.06667 73.46667 Asia/Kolkata
13.0075 77.93778 Asia/Kolkata
26.28333 75.38333 Asia/Kolkata
13.35 74.71667 Asia/Kolkata
11.48333 78.03333 Asia/Kolkata
20.88333 76.2 Asia/Kolkata
18.35 81.9 Asia/Kolkata
26.91667 80.71667 Asia/Kolkata
30.51667 75.88333 Asia/Kolkata
20.55 74.53333 Asia/Kolkata
12.38333 77.08333 Asia/Kolkata
30.21667 74.48333 Asia/Kolkata
11.06667 76.06667 Asia/Kolkata
27.5 95.45 Asia/Kolkata
23.26667 76.15 Asia/Kolkata
27.05 74.71667 Asia/Kolkata
26.23333 84.15 Asia/Kolkata
27.23333 79.01667 Asia/Kolkata
26.56667 88.81667 Asia/Kolkata
24.26667 80.75 Asia/Kolkata
27.05 76.93333 Asia/Kolkata
22.81667 72.93333 Asia/Kolkata
27.66667 80.46667 Asia/Kolkata
25.28333 79.86667 Asia/Kolkata
27.3 81.11667 Asia/Kolkata
22.18333 87.98333 Asia/Kolkata
26.5 78.60417 Asia/Kolkata
22.18333 75.58333 Asia/Kolkata
28.28333 76.15 Asia/Kolkata
22.83333 72.76667 Asia/Kolkata
11.7 75.53333 Asia/Kolkata
16.73333 77.98333 Asia/Kolkata
17.61667 80.01667 Asia/Kolkata
21.1 82.1 Asia/Kolkata
27.13333 83.56667 Asia/Kolkata
26.11667 84.48333 Asia/Kolkata
28.98333 76.3 Asia/Kolkata
16.38333 75.11667 Asia/Kolkata
18.08333 73.41667 Asia/Kolkata
26.75722 83.12778 Asia/Kolkata
12.96667 77.23333 Asia/Kolkata
12.51667 79.9 Asia/Kolkata
9.93333 78.11667 Asia/Kolkata
10.48333 79.4 Asia/Kolkata
10.9 76.96667 Asia/Kolkata
13.08784 80.27847 Asia/Kolkata
12.41667 75.73333 Asia/Kolkata
22.7 88.45 Asia/Kolkata
24.25 86.65 Asia/Kolkata
13.66056 77.20917 Asia/Kolkata
26.36667 86.08333 Asia/Kolkata
25.91667 86.78333 Asia/Kolkata
15.3 73.95 Asia/Kolkata
12.6 77.06667 Asia/Kolkata
13.55 78.5 Asia/Kolkata
16.16667 81.13333 Asia/Kolkata
25.68333 82.41667 Asia/Kolkata
30.91472 76.19833 Asia/Kolkata
16.48333 79.43333 Asia/Kolkata
22.88333 92.73333 Asia/Kolkata
23.13333 73.61667 Asia/Kolkata
30.9 75.85 Asia/Kolkata
26.85 80.91667 Asia/Kolkata
25.18333 86.08333 Asia/Kolkata
27.4 74.91667 Asia/Kolkata
28.75 77.28333 Asia/Kolkata
18.75 73.41667 Asia/Kolkata
19.98333 76.53333 Asia/Kolkata
23.43333 84.68333 Asia/Kolkata
16.16667 76.51667 Asia/Kolkata
22.56667 71.8 Asia/Kolkata
24.05 77.4 Asia/Kolkata
34.16667 77.58333 Asia/Kolkata
29.11667 77.76667 Asia/Kolkata
30.21667 75.68333 Asia/Kolkata
18.4 76.58333 Asia/Kolkata
21.71667 71.38333 Asia/Kolkata
23.75 84.5 Asia/Kolkata
26.20389 83.97167 Asia/Kolkata
26.56667 76.33333 Asia/Kolkata
22.2 69.96667 Asia/Kolkata
24.68333 78.41667 Asia/Kolkata
10.86667 78.83333 Asia/Kolkata
24.41667 88.25 Asia/Kolkata
25.93333 81.7 Asia/Kolkata
25.86667 85.18333 Asia/Kolkata
15.13333 75.46667 Asia/Kolkata
29.76667 78.05 Asia/Kolkata
23.66667 86.66667 Asia/Kolkata
22.6 79.6 Asia/Kolkata
27.95 80.76667 Asia/Kolkata
25.66667 76.16667 Asia/Kolkata
27.71667 80.9 Asia/Kolkata
26.19833 78.945 Asia/Kolkata
29.99444 77.04444 Asia/Kolkata
27.65 74.38333 Asia/Kolkata
27.81667 75.03333 Asia/Kolkata
8.31667 77.18333 Asia/Kolkata
10.7 79.53333 Asia/Kolkata
21.62333 69.98167 Asia/Kolkata
15.76667 76.2 Asia/Kolkata
11.56667 79.6 Asia/Kolkata
18.08333 75.43333 Asia/Kolkata
16.68333 74.58333 Asia/Kolkata
12.75 78.36667 Asia/Kolkata
10.65 76.08333 Asia/Kolkata
11.30459 75.87772 Asia/Kolkata
13.02222 77.02667 Asia/Kolkata
21.33333 71.3 Asia/Kolkata
15.25 75.25 Asia/Kolkata
28.7 78.78333 Asia/Kolkata
25.71667 81.51667 Asia/Kolkata
14.41667 74.4 Asia/Kolkata
27.31667 77.36667 Asia/Kolkata
21.26667 81.51667 Asia/Kolkata
24.36667 77.05 Asia/Kolkata
10.96667 79.38333 Asia/Kolkata
31.96667 77.1 Asia/Kolkata
23.73333 86.85 Asia/Kolkata
25.31667 79.65 Asia/Kolkata
10.91667 78.41667 Asia/Kolkata
33.65 75.01667 Asia/Kolkata
22.2 74.75 Asia/Kolkata
17.48333 78.41667 Asia/Kolkata
23.71667 85.5 Asia/Kolkata
14.9 76.38333 Asia/Kolkata
16.63333 74.85 Asia/Kolkata
26.98389 73.97194 Asia/Kolkata
27.15 74.85 Asia/Kolkata
22.26667 86.16667 Asia/Kolkata
12.655 76.48833 Asia/Kolkata
23.4 88.5 Asia/Kolkata
12.53333 78.23333 Asia/Kolkata
11.25 75.76667 Asia/Kolkata
17.01667 81.73333 Asia/Kolkata
14.48333 79.98333 Asia/Kolkata
9.16667 77.86667 Asia/Kolkata
14.81667 76.21667 Asia/Kolkata
9.58333 76.51667 Asia/Kolkata
17.55 80.63333 Asia/Kolkata
27.71667 76.2 Asia/Kolkata
23.2 81.96667 Asia/Kolkata
29.75 78.53333 Asia/Kolkata
19.15 82.35 Asia/Kolkata
10.06667 76.63333 Asia/Kolkata
11.43333 76.88333 Asia/Kolkata
25.18333 75.83333 Asia/Kolkata
22.3 82.03333 Asia/Kolkata
15.85 77.26667 Asia/Kolkata
27.8 77.43333 Asia/Kolkata
21.48333 72.95 Asia/Kolkata
24.1167 78.0419 Asia/Kolkata
18.65 74.08333 Asia/Kolkata
22.35 82.68333 Asia/Kolkata
18.81667 78.71667 Asia/Kolkata
18.81667 82.71667 Asia/Kolkata
15.35 76.15 Asia/Kolkata
19.88333 74.48333 Asia/Kolkata
26.02333 83.56583 Asia/Kolkata
16.2 74.75 Asia/Kolkata
22.7 88.3475 Asia/Kolkata
16.61667 80.53333 Asia/Kolkata
19.6 81.66667 Asia/Kolkata
25.98333 79.15 Asia/Kolkata
19.9 86.11667 Asia/Kolkata
24.23333 92.7 Asia/Kolkata
12.15 77.11667 Asia/Kolkata
16.7 74.21667 Asia/Kolkata
25.23333 77.6 Asia/Kolkata
13.13333 78.13333 Asia/Kolkata
8.16667 77.25 Asia/Kolkata
26.39904 90.27182 Asia/Kolkata
25.66667 94.11667 Asia/Kolkata
25.58333 84.8 Asia/Kolkata
10.21667 76.21667 Asia/Kolkata
16.88333 74.2 Asia/Kolkata
20.79028 70.70306 Asia/Kolkata
24.46667 85.6 Asia/Kolkata
16.98333 79.96667 Asia/Kolkata
10.23333 77.48333 Asia/Kolkata
26.31667 89.43333 Asia/Kolkata
25.31667 84.26667 Asia/Kolkata
10.3 76.35 Asia/Kolkata
28.86667 77.93333 Asia/Kolkata
33.31667 75.76667 Asia/Kolkata
26.56667 74.86667 Asia/Kolkata
26.11667 87.93333 Asia/Kolkata
29.51667 78.2 Asia/Kolkata
27.15 77.78333 Asia/Kolkata
19.63333 78.2 Asia/Kolkata
28.91667 79.5 Asia/Kolkata
28.2 80.28333 Asia/Kolkata
28.25 77.85 Asia/Kolkata
20.18333 85.61667 Asia/Kolkata
24.0436 78.3319 Asia/Kolkata
23.08333 85.28333 Asia/Kolkata
24.1 91.63333 Asia/Kolkata
18.78333 73.33333 Asia/Kolkata
22.16667 76.85 Asia/Kolkata
24.03333 76.56667 Asia/Kolkata
27.98333 75.8 Asia/Kolkata
21.66667 74.58333 Asia/Kolkata
27.9 80.8 Asia/Kolkata
23.88333 72.61667 Asia/Kolkata
28.86667 77.28333 Asia/Kolkata
24.03333 73.05 Asia/Kolkata
22.75 72.68333 Asia/Kolkata
22.98333 86.85 Asia/Kolkata
28.91667 79.96667 Asia/Kolkata
22.6 76.91667 Asia/Kolkata
29.28333 77.71667 Asia/Kolkata
26.51667 92.13333 Asia/Kolkata
21.96667 83.11667 Asia/Kolkata
28.88333 76.85 Asia/Kolkata
21.81667 75.6 Asia/Kolkata
22.71861 88.37806 Asia/Kolkata
30.74444 76.64778 Asia/Kolkata
18.43333 73.76667 Asia/Kolkata
25.11667 86.55 Asia/Kolkata
22.33333 87.33333 Asia/Kolkata
21.43333 78.96667 Asia/Kolkata
30.70444 76.22306 Asia/Kolkata
21.83333 76.33333 Asia/Kolkata
27.6 75.5 Asia/Kolkata
15.63333 74.51667 Asia/Kolkata
17.25 80.15 Asia/Kolkata
20.68333 76.56667 Asia/Kolkata
22.3 72.61667 Asia/Kolkata
22.20722 69.66833 Asia/Kolkata
23.21667 79.88333 Asia/Kolkata
26.775 83.07361 Asia/Kolkata
24.85 79.93333 Asia/Kolkata
26.95 77.81667 Asia/Kolkata
21.41667 80.96667 Asia/Kolkata
27.53333 80.75 Asia/Kolkata
27.95 77.83333 Asia/Kolkata
25.58333 85.05 Asia/Kolkata
25.5 86.48333 Asia/Kolkata
18.56667 73.86667 Asia/Kolkata
27.18333 83.88333 Asia/Kolkata
23.41667 75.28333 Asia/Kolkata
20.2 83.23333 Asia/Kolkata
25.3 75.93333 Asia/Kolkata
21.3 70.25 Asia/Kolkata
16.01667 75.56667 Asia/Kolkata
20.5 86.41667 Asia/Kolkata
23.2 86.53333 Asia/Kolkata
28.81667 79.21667 Asia/Kolkata
25.96667 75.15 Asia/Kolkata
9.18333 76.5 Asia/Kolkata
8.56667 78.11667 Asia/Kolkata
22.01667 81.25 Asia/Kolkata
14.91667 79.98333 Asia/Kolkata
13.21667 80.31667 Asia/Kolkata
23.8 86.28333 Asia/Kolkata
12.98333 79.13333 Asia/Kolkata
23.65 88.13333 Asia/Kolkata
21.26667 78.58333 Asia/Kolkata
25.53333 87.58333 Asia/Kolkata
32.36667 75.51667 Asia/Kolkata
21.3 72.93333 Asia/Kolkata
22.5 82.55 Asia/Kolkata
23.45 79.78333 Asia/Kolkata
21.78333 79.78333 Asia/Kolkata
22.13333 75.6 Asia/Kolkata
29.21667 78.95 Asia/Kolkata
27.81667 78.65 Asia/Kolkata
12.5 75 Asia/Kolkata
14.8 74.13333 Asia/Kolkata
10.95 78.08333 Asia/Kolkata
31.44 75.49972 Asia/Kolkata
26.88251 88.27729 Asia/Kolkata
29.68333 76.98333 Asia/Kolkata
18.41667 75.2 Asia/Kolkata
13.2 74.98333 Asia/Kolkata
18.91667 73.33333 Asia/Kolkata
18.43333 79.15 Asia/Kolkata
24.86667 92.35 Asia/Kolkata
27.00028 78.94139 Asia/Kolkata
25.46667 78.15 Asia/Kolkata
22.91667 79.06667 Asia/Kolkata
26.5 77.01667 Asia/Kolkata
20.48333 77.48333 Asia/Kolkata
22.55 72.9 Asia/Kolkata
11.24058 76.96009 Asia/Kolkata
10.06667 78.78333 Asia/Kolkata
10.91667 79.83333 Asia/Kolkata
17.28333 74.2 Asia/Kolkata
31.37944 75.38472 Asia/Kolkata
25.41667 76.06667 Asia/Kolkata
23.01667 73.06667 Asia/Kolkata
29.06667 78.63333 Asia/Kolkata
20.48333 82.91667 Asia/Kolkata
27.8 79.8 Asia/Kolkata
26.46667 80.35 Asia/Kolkata
22.66667 76.73333 Asia/Kolkata
8.08333 77.56667 Asia/Kolkata
27.0546 79.922 Asia/Kolkata
12.30814 75.10632 Asia/Kolkata
20.26667 75.13333 Asia/Kolkata
20.27194 81.49306 Asia/Kolkata
23.43333 85.31667 Asia/Kolkata
16.26667 73.7 Asia/Kolkata
15.4 79.51667 Asia/Kolkata
11 77.56667 Asia/Kolkata
15.21667 79.91667 Asia/Kolkata
23.03333 70.21667 Asia/Kolkata
23.95 88.03333 Asia/Kolkata
29.31667 77.26667 Asia/Kolkata
12.83333 79.71667 Asia/Kolkata
12.55 77.41667 Asia/Kolkata
21.23333 79.2 Asia/Kolkata
15.4 76.61667 Asia/Kolkata
9.73333 77.3 Asia/Kolkata
22.67111 88.37472 Asia/Kolkata
18.31667 78.35 Asia/Kolkata
27.65 77.26667 Asia/Kolkata
27.26667 79.65 Asia/Kolkata
20.93333 85.55 Asia/Kolkata
22.98333 88.48333 Asia/Kolkata
19.25 73.15 Asia/Kolkata
9.15 77.71667 Asia/Kolkata
26.11667 79.73333 Asia/Kolkata
11.61056 76.08222 Asia/Kolkata
22.6 73.45 Asia/Kolkata
23.21667 88.36667 Asia/Kolkata
21.23333 78.9 Asia/Kolkata
8.66667 77.48333 Asia/Kolkata
11.73333 78.96667 Asia/Kolkata
30.83444 76.93333 Asia/Kolkata
25.63333 88.31667 Asia/Kolkata
27.06667 88.48333 Asia/Kolkata
15.18333 74.96667 Asia/Kolkata
22.21667 70.38333 Asia/Kolkata
29.85 74.95 Asia/Kolkata
28.83333 76.4 Asia/Kolkata
19.66667 77.33333 Asia/Kolkata
19.05 73.95 Asia/Kolkata
8.5 77.56667 Asia/Kolkata
27.8927 79.1965 Asia/Kolkata
26.88333 80.8 Asia/Kolkata
16.93333 82.21667 Asia/Kolkata
24.48333 93.98333 Asia/Kolkata
29.8 76.38333 Asia/Kolkata
29.4 77.2 Asia/Kolkata
23.38333 79.75 Asia/Kolkata
27.56667 79.35 Asia/Kolkata
24.33333 92.01667 Asia/Kolkata
26.31667 77.61667 Asia/Kolkata
16.56667 81.2 Asia/Kolkata
16.58333 74.31667 Asia/Kolkata
13.55333 76.01306 Asia/Kolkata
21.21667 73.23333 Asia/Kolkata
14.11667 78.16667 Asia/Kolkata
23.3 72.33333 Asia/Kolkata
9.08333 77.35 Asia/Kolkata
22.97 88.43194 Asia/Kolkata
25.21667 82.71667 Asia/Kolkata
25.41667 80.01667 Asia/Kolkata
19.2 73.88333 Asia/Kolkata
19.86667 82.93333 Asia/Kolkata
21.51667 70.46667 Asia/Kolkata
24.43333 85.51667 Asia/Kolkata
31.32556 75.57917 Asia/Kolkata
22.76667 86.18333 Asia/Kolkata
26.75 94.21667 Asia/Kolkata
26.33333 77.81667 Asia/Kolkata
12.56667 78.58333 Asia/Kolkata
26.41667 87.25 Asia/Kolkata
22.71667 70.28333 Asia/Kolkata
26.28667 73.03 Asia/Kolkata
21.88333 70.03333 Asia/Kolkata
19.61667 76.7 Asia/Kolkata
29.31667 76.31667 Asia/Kolkata
25.43333 81.93333 Asia/Kolkata
28.1307 75.4014 Asia/Kolkata
29.51667 77.21667 Asia/Kolkata
26.56667 79.73333 Asia/Kolkata
21.85 84.03333 Asia/Kolkata
23.75 86.4 Asia/Kolkata
22.45 86.98333 Asia/Kolkata
25.43333 78.58333 Asia/Kolkata
26.26667 86.28333 Asia/Kolkata
29.35 78.25 Asia/Kolkata
24.55 76.16667 Asia/Kolkata
23.36667 85.96667 Asia/Kolkata
24.6 76.15 Asia/Kolkata
28.6063 76.6565 Asia/Kolkata
24.76667 86.36667 Asia/Kolkata
22.76667 74.6 Asia/Kolkata
28.13333 77.55 Asia/Kolkata
17.01667 76.76667 Asia/Kolkata
21.73333 70.61667 Asia/Kolkata
18.85 82.58333 Asia/Kolkata
26.58333 86.15 Asia/Kolkata
22.17583 88.41778 Asia/Kolkata
11.21667 79.36667 Asia/Kolkata
24.6 74.85 Asia/Kolkata
25.73333 82.68333 Asia/Kolkata
25.01667 79.05 Asia/Kolkata
20.16667 85.7 Asia/Kolkata
26.87861 78.90278 Asia/Kolkata
29.28333 78.81667 Asia/Kolkata
24.51667 86.65 Asia/Kolkata
22.9 84.15 Asia/Kolkata
22.03333 71.2 Asia/Kolkata
27.16667 81.55 Asia/Kolkata
23.63333 75.13333 Asia/Kolkata
29.33333 77.85 Asia/Kolkata
24.46667 88.06667 Asia/Kolkata
17.71667 79.18333 Asia/Kolkata
31.16667 75.61667 Asia/Kolkata
23.7 87.08333 Asia/Kolkata
24.91667 86.21667 Asia/Kolkata
23.95 86.8 Asia/Kolkata
22.8 86.18333 Asia/Kolkata
22.46667 70.06667 Asia/Kolkata
32.73333 74.86667 Asia/Kolkata
14.83333 78.4 Asia/Kolkata
16.51667 75.3 Asia/Kolkata
22.05 72.8 Asia/Kolkata
25.3 86.5 Asia/Kolkata
22.2 78.58333 Asia/Kolkata
23.71667 86.4 Asia/Kolkata
26.51667 88.73333 Asia/Kolkata
25.35 72.61667 Asia/Kolkata
19.83333 75.88333 Asia/Kolkata
21.04861 76.53444 Asia/Kolkata
21.01667 75.56667 Asia/Kolkata
21.81667 87.21667 Asia/Kolkata
27.48333 78.31667 Asia/Kolkata
26.1451 79.3366 Asia/Kolkata
26.31667 82.73333 Asia/Kolkata
20.96667 72.9 Asia/Kolkata
27.86667 78.26667 Asia/Kolkata
30.61667 74.25 Asia/Kolkata
29.61861 77.43833 Asia/Kolkata
27.7242 79.6599 Asia/Kolkata
20.85 86.33333 Asia/Kolkata
30.46667 74.88333 Asia/Kolkata
26.20306 73.93639 Asia/Kolkata
16.8 74.56667 Asia/Kolkata
26.91468 70.91808 Asia/Kolkata
26.25 81.53333 Asia/Kolkata
26.91667 75.81667 Asia/Kolkata
25.61667 75.28333 Asia/Kolkata
28.41667 78.1 Asia/Kolkata
25.21667 84.98333 Asia/Kolkata
18.8 78.93333 Asia/Kolkata
30.78333 75.48333 Asia/Kolkata
16.9 80.1 Asia/Kolkata
25.48333 84.41667 Asia/Kolkata
19.06667 82.03333 Asia/Kolkata
20.26667 86.16667 Asia/Kolkata
14.53333 76.35 Asia/Kolkata
30.16667 77.3 Asia/Kolkata
23.16697 79.95006 Asia/Kolkata
27.25 78.2 Asia/Kolkata
22.61667 77.75 Asia/Kolkata
27.1 93.61667 Asia/Kolkata
26.26667 88.2 Asia/Kolkata
25.15 85.2 Asia/Kolkata
28.33333 78.71667 Asia/Kolkata
11.01667 77.06667 Asia/Kolkata
10.33333 76.23333 Asia/Kolkata
25 88.15 Asia/Kolkata
29.88 77.07111 Asia/Kolkata
22.71792 75.8333 Asia/Kolkata
17.16667 75.96667 Asia/Kolkata
26.9352 79.6724 Asia/Kolkata
18.3 73.25 Asia/Kolkata
24.81667 93.95 Asia/Kolkata
15.96667 76.13333 Asia/Kolkata
19.7 73.55 Asia/Kolkata
11.58333 77.85 Asia/Kolkata
19.11667 84.7 Asia/Kolkata
16.7 74.46667 Asia/Kolkata
17.37528 78.47444 Asia/Kolkata
15.01667 75.95 Asia/Kolkata
24.53333 84.01667 Asia/Kolkata
12.3075 76.28778 Asia/Kolkata
16.06667 76.05 Asia/Kolkata
16.23333 74.6 Asia/Kolkata
22.89556 88.4025 Asia/Kolkata
15.35 75.16667 Asia/Kolkata
26.43333 90.96667 Asia/Kolkata
12.71667 77.81667 Asia/Kolkata
15.26667 76.4 Asia/Kolkata
13.06694 77.79833 Asia/Kolkata
31.53222 75.91722 Asia/Kolkata
22.75 77.71667 Asia/Kolkata
13.79806 76.28611 Asia/Kolkata
14.25 75.66667 Asia/Kolkata
14.28333 74.45 Asia/Kolkata
17.76667 77.13333 Asia/Kolkata
12.78278 76.24306 Asia/Kolkata
14.03333 76.18333 Asia/Kolkata
26 92.86667 Asia/Kolkata
27.9 77.36667 Asia/Kolkata
24.83333 85.41667 Asia/Kolkata
29.16667 75.71667 Asia/Kolkata
13.94556 76.61917 Asia/Kolkata
14.46667 75.38333 Asia/Kolkata
21.51667 83.86667 Asia/Kolkata
19.48333 84.75 Asia/Kolkata
19.71667 77.15 Asia/Kolkata
20.56667 78.83333 Asia/Kolkata
13.82889 77.49333 Asia/Kolkata
23.9 79.56667 Asia/Kolkata
26.71667 77.01667 Asia/Kolkata
23.6 72.95 Asia/Kolkata
25.31667 85.28333 Asia/Kolkata
23.98333 85.35 Asia/Kolkata
14.8 75.4 Asia/Kolkata
24.11667 79.6 Asia/Kolkata
27.6 78.05 Asia/Kolkata
29.16667 78.01667 Asia/Kolkata
13.00056 76.09944 Asia/Kolkata
26.74704 89.35189 Asia/Kolkata
28.71667 78.28333 Asia/Kolkata
12.06667 78.5 Asia/Kolkata
22.1 76.73333 Asia/Kolkata
14.8 75.98333 Asia/Kolkata
25.2884 79.3347 Asia/Kolkata
23.7 71.9 Asia/Kolkata
14.51667 75.8 Asia/Kolkata
29.96667 78.16667 Asia/Kolkata
27.41667 80.11667 Asia/Kolkata
22.33333 77.1 Asia/Kolkata
28.71667 77.78333 Asia/Kolkata
22.58917 88.31028 Asia/Kolkata
29.58333 74.31667 Asia/Kolkata
29.1 75.96667 Asia/Kolkata
14.76667 75.13333 Asia/Kolkata
25.38333 82.18333 Asia/Kolkata
31.68333 76.51667 Asia/Kolkata
25.95 80.15 Asia/Kolkata
23.01667 71.18333 Asia/Kolkata
22.5 73.46667 Asia/Kolkata
15.33333 74.76667 Asia/Kolkata
22.94972 88.41944 Asia/Kolkata
29.21667 79.51667 Asia/Kolkata
29.3 78.28333 Asia/Kolkata
26.25 91.53333 Asia/Kolkata
25.68333 85.21667 Asia/Kolkata
24.68333 92.56667 Asia/Kolkata
25.18333 93.03333 Asia/Kolkata
19.5 77.66667 Asia/Kolkata
22.83444 88.6275 Asia/Kolkata
26.22361 78.17917 Asia/Kolkata
23.5 87.75 Asia/Kolkata
10.5943 76.0411 Asia/Kolkata
30.71667 74.41667 Asia/Kolkata
25.6148 79.1788 Asia/Kolkata
27.11667 79.71667 Asia/Kolkata
16.86667 77.4 Asia/Kolkata
28.46667 77.03333 Asia/Kolkata
19.08333 83.81667 Asia/Kolkata
16.3 80.45 Asia/Kolkata
15.16667 77.38333 Asia/Kolkata
28.25 78.43333 Asia/Kolkata
11.8 76.68333 Asia/Kolkata
24.65 77.31667 Asia/Kolkata
13.4 80.15 Asia/Kolkata
23.05 84.55 Asia/Kolkata
23.78333 85.81667 Asia/Kolkata
16.05 75.8 Asia/Kolkata
17.33333 76.83333 Asia/Kolkata
28.6 77.78333 Asia/Kolkata
25.9 74.66667 Asia/Kolkata
12.8425 80.06389 Asia/Kolkata
14.13333 79.85 Asia/Kolkata
12.95 78.86667 Asia/Kolkata
16.45 80.98333 Asia/Kolkata
9.68333 77.26667 Asia/Kolkata
13.31056 76.94444 Asia/Kolkata
26.69984 89.02565 Asia/Kolkata
27.5 77.46667 Asia/Kolkata
29.45 75.68333 Asia/Kolkata
26.755 83.37389 Asia/Kolkata
11.46667 77.45 Asia/Kolkata
26.46667 84.43333 Asia/Kolkata
21.45 80.2 Asia/Kolkata
21.96667 70.8 Asia/Kolkata
27.13333 81.93333 Asia/Kolkata
23.86667 86.16667 Asia/Kolkata
28.08333 80.46667 Asia/Kolkata
26.51667 93.96667 Asia/Kolkata
14.55 74.31667 Asia/Kolkata
16.16667 74.83333 Asia/Kolkata
29.13333 76.7 Asia/Kolkata
26.42611 78.44472 Asia/Kolkata
22.75 73.63333 Asia/Kolkata
24.83333 87.21667 Asia/Kolkata
22.63333 86.06667 Asia/Kolkata
22.87472 88.75861 Asia/Kolkata
26.16667 90.61667 Asia/Kolkata
24.18333 86.3 Asia/Kolkata
12.25 79.41667 Asia/Kolkata
30.2 74.66667 Asia/Kolkata
15.35 78.91667 Asia/Kolkata
19.93333 79.13333 Asia/Kolkata
19.71667 73.63333 Asia/Kolkata
26.10833 83.54361 Asia/Kolkata
25.58333 83.56667 Asia/Kolkata
28.66667 77.43333 Asia/Kolkata
22.6 86.48333 Asia/Kolkata
20.13333 78.31667 Asia/Kolkata
26.16667 80.16667 Asia/Kolkata
22.66667 87.71667 Asia/Kolkata
29.5375 76.97167 Asia/Kolkata
19.26667 75.75 Asia/Kolkata
24.78333 85 Asia/Kolkata
26.08352 89.96158 Asia/Kolkata
13.61111 77.51667 Asia/Kolkata
26.18617 91.75095 Asia/Kolkata
24.18333 83.81667 Asia/Kolkata
22.63333 88.4 Asia/Kolkata
21.53333 71.58333 Asia/Kolkata
31.21667 76.13333 Asia/Kolkata
28.8 78.1 Asia/Kolkata
23.76667 79.15 Asia/Kolkata
16.53333 80.8 Asia/Kolkata
27.33333 88.61667 Asia/Kolkata
13.65 74.66667 Asia/Kolkata
29.76667 77.25 Asia/Kolkata
15.43333 76.53333 Asia/Kolkata
25.4 88.51667 Asia/Kolkata
26.48333 76.71667 Asia/Kolkata
25.21667 74.26667 Asia/Kolkata
19.68333 75.01667 Asia/Kolkata
29.91667 73.88333 Asia/Kolkata
18.95 76.75 Asia/Kolkata
23.21667 72.68333 Asia/Kolkata
20.81667 72.98333 Asia/Kolkata
34.23333 74.78333 Asia/Kolkata
28.85 78.23333 Asia/Kolkata
15.73333 75.98333 Asia/Kolkata
16.23333 77.8 Asia/Kolkata
16.23333 74.35 Asia/Kolkata
21.96667 71.56667 Asia/Kolkata
22.91667 78.78333 Asia/Kolkata
15.41667 75.61667 Asia/Kolkata
22.50444 88.18333 Asia/Kolkata
26.3 87.25 Asia/Kolkata
27.8 76.95 Asia/Kolkata
30.91667 74.6 Asia/Kolkata
27.15 78.41667 Asia/Kolkata
11.18333 75.85 Asia/Kolkata
30.4 74.03333 Asia/Kolkata
25.51667 85.31667 Asia/Kolkata
27.0937 77.66003 Asia/Kolkata
27.98333 74.95 Asia/Kolkata
27.16667 81.21667 Asia/Kolkata
25.93333 80.8 Asia/Kolkata
31.86207 74.95534 Asia/Kolkata
28.46667 79.3 Asia/Kolkata
29.51667 75.45 Asia/Kolkata
27.01667 78.31667 Asia/Kolkata
17.07778 78.20111 Asia/Kolkata
27.4 79.56667 Asia/Kolkata
28.21667 79.55 Asia/Kolkata
30.66667 74.75 Asia/Kolkata
28.43333 77.31667 Asia/Kolkata
24.81667 87.9 Asia/Kolkata
26.52084 89.20545 Asia/Kolkata
21.16667 75.85 Asia/Kolkata
26.78333 82.13333 Asia/Kolkata
26.7769 79.0239 Asia/Kolkata
14.63333 78.53333 Asia/Kolkata
11.35 77.73333 Asia/Kolkata
9.7 76.78333 Asia/Kolkata
20.91667 75.33333 Asia/Kolkata
15.73333 77.48333 Asia/Kolkata
16.7 81.1 Asia/Kolkata
10.06667 76.28333 Asia/Kolkata
29.45 74.65 Asia/Kolkata
17.55 82.86667 Asia/Kolkata
21.9 87.53333 Asia/Kolkata
22.23944 68.96778 Asia/Kolkata
23.48333 87.31667 Asia/Kolkata
20 79.3 Asia/Kolkata
21.18333 81.28333 Asia/Kolkata
23.83333 73.71667 Asia/Kolkata
27.73333 78.95 Asia/Kolkata
25.55 84.15 Asia/Kolkata
26.56667 85.51667 Asia/Kolkata
24.26667 87.25 Asia/Kolkata
27.56667 95.56667 Asia/Kolkata
22.62222 88.41694 Asia/Kolkata
27.36667 95.31667 Asia/Kolkata
23.75 86.16667 Asia/Kolkata
23.8 87.38333 Asia/Kolkata
17.45 80.16667 Asia/Kolkata
30.81667 76.01667 Asia/Kolkata
21.2 80.73333 Asia/Kolkata
21.33333 74.56667 Asia/Kolkata
19.21667 73.08333 Asia/Kolkata
13.29194 77.54306 Asia/Kolkata
33.13333 75.56667 Asia/Kolkata
20.71405 70.98224 Asia/Kolkata
24.25 72.16667 Asia/Kolkata
25.83333 93.43333 Asia/Kolkata
26.13333 89.46667 Asia/Kolkata
22.95 81.08333 Asia/Kolkata
10.35 77.95 Asia/Kolkata
32.15 75.46667 Asia/Kolkata
25.9 93.73333 Asia/Kolkata
20.11667 77.71667 Asia/Kolkata
18.55 77.6 Asia/Kolkata
25.73333 85 Asia/Kolkata
27.38333 95.63333 Asia/Kolkata
27.46667 77.33333 Asia/Kolkata
27.4 74.56667 Asia/Kolkata
15.6 73.95 Asia/Kolkata
27.48333 94.9 Asia/Kolkata
28.21667 78.25 Asia/Kolkata
22.19167 88.18472 Asia/Kolkata
30.36667 75.86667 Asia/Kolkata
26.58575 89.0145 Asia/Kolkata
24.68333 87.96667 Asia/Kolkata
20.9 74.78333 Asia/Kolkata
26.03333 89.96667 Asia/Kolkata
22.56667 70.41667 Asia/Kolkata
22.98333 71.46667 Asia/Kolkata
21.73333 70.45 Asia/Kolkata
15.41667 77.88333 Asia/Kolkata
22.71667 72.46667 Asia/Kolkata
26.46667 92.46667 Asia/Kolkata
20.66667 85.6 Asia/Kolkata
26.7 92.5 Asia/Kolkata
28 81.08333 Asia/Kolkata
26.7 77.9 Asia/Kolkata
18.81667 76.11667 Asia/Kolkata
28.21667 76.78333 Asia/Kolkata
32.24194 76.32133 Asia/Kolkata
14.43333 77.71667 Asia/Kolkata
12.13333 78.16667 Asia/Kolkata
24.36667 92.16667 Asia/Kolkata
11.78333 75.43333 Asia/Kolkata
18.9 77.85 Asia/Kolkata
31.95417 75.31917 Asia/Kolkata
21.33333 71.01667 Asia/Kolkata
10.73333 77.51667 Asia/Kolkata
21.01667 75.26667 Asia/Kolkata
20.53333 73.18333 Asia/Kolkata
22.6 75.3 Asia/Kolkata
24.51667 72.01667 Asia/Kolkata
22.36667 71.98333 Asia/Kolkata
23.8 86.45 Asia/Kolkata
28.96667 78.25 Asia/Kolkata
30.28333 75.58333 Asia/Kolkata
20.70722 81.54972 Asia/Kolkata
29.31667 78.51667 Asia/Kolkata
22.21667 75.46667 Asia/Kolkata
26.68333 85.16667 Asia/Kolkata
22.96667 76.06667 Asia/Kolkata
25.75 75.38333 Asia/Kolkata
25.53333 73.9 Asia/Kolkata
22.7 73.9 Asia/Kolkata
16.7 78.93333 Asia/Kolkata
13.2325 77.69917 Asia/Kolkata
9.95 78.81667 Asia/Kolkata
20.01667 76.03333 Asia/Kolkata
27.8 73.35 Asia/Kolkata
22.85 75.55 Asia/Kolkata
23.4 79.01667 Asia/Kolkata
26.50472 83.78722 Asia/Kolkata
28.63333 79.48333 Asia/Kolkata
20.66667 78.48333 Asia/Kolkata
19.95 73.83333 Asia/Kolkata
21.53333 84.73333 Asia/Kolkata
29.7 77.68333 Asia/Kolkata
12.53333 77.8 Asia/Kolkata
28.66667 77.21667 Asia/Kolkata
24.86667 84.18333 Asia/Kolkata
30.31667 78.03333 Asia/Kolkata
26.88333 76.33333 Asia/Kolkata
18.46667 74.6 Asia/Kolkata
25.03333 84.4 Asia/Kolkata
20.76667 78.16667 Asia/Kolkata
25.66667 78.46667 Asia/Kolkata
28.03333 79.4 Asia/Kolkata
31.81833 75.65972 Asia/Kolkata
28.68333 77.53333 Asia/Kolkata
20.93333 77.33333 Asia/Kolkata
20.31667 77.76667 Asia/Kolkata
27.03333 88.26667 Asia/Kolkata
26.16667 85.9 Asia/Kolkata
15.26667 74.61667 Asia/Kolkata
25.63333 85.05 Asia/Kolkata
23.83333 79.45 Asia/Kolkata
21.7 71.51667 Asia/Kolkata
20.41667 72.85 Asia/Kolkata
24.03333 84.06667 Asia/Kolkata
25.66667 85.83333 Asia/Kolkata
25.85 87.85 Asia/Kolkata
22.75 73.15 Asia/Kolkata
22.83333 74.26667 Asia/Kolkata
23.16667 72.81667 Asia/Kolkata
19.96667 72.73333 Asia/Kolkata
28.56667 77.55 Asia/Kolkata
29.95 74.73333 Asia/Kolkata
25.9 78.33333 Asia/Kolkata
26.0025 78.87611 Asia/Kolkata
22.18333 73.43333 Asia/Kolkata
20.46497 85.87927 Asia/Kolkata
15.25 74.1 Asia/Kolkata
15.16667 73.98333 Asia/Kolkata
15.56667 79.11667 Asia/Kolkata
14.46667 78.81667 Asia/Kolkata
11.75 79.75 Asia/Kolkata
13.63333 74.7 Asia/Kolkata
27.13333 81.7 Asia/Kolkata
25.26667 87.21667 Asia/Kolkata
15.5439 73.7553 Asia/Kolkata
11 76.96667 Asia/Kolkata
9.93988 76.26022 Asia/Kolkata
30.26667 78.06667 Asia/Kolkata
28.3 74.95 Asia/Kolkata
24.33333 93.66667 Asia/Kolkata
25.13333 82.9 Asia/Kolkata
22.41667 71.18333 Asia/Kolkata
21.25 75.3 Asia/Kolkata
17.83333 82.95 Asia/Kolkata
17.45 81.76667 Asia/Kolkata
10.7 76.75 Asia/Kolkata
24.88333 74.63333 Asia/Kolkata
23.86667 86.86667 Asia/Kolkata
14.23333 76.4 Asia/Kolkata
17.11667 77.08333 Asia/Kolkata
28.2401 75.6456 Asia/Kolkata
15.81667 80.35 Asia/Kolkata
18.3 83.56667 Asia/Kolkata
17.53333 73.51667 Asia/Kolkata
13.4 78.06667 Asia/Kolkata
11.65 78.9 Asia/Kolkata
9.83333 77.38333 Asia/Kolkata
17.46667 77.43333 Asia/Kolkata
26.28222 83.50833 Asia/Kolkata
16.08333 80.16667 Asia/Kolkata
16.43333 74.6 Asia/Kolkata
13.41722 76.62278 Asia/Kolkata
13.31667 75.78333 Asia/Kolkata
20.35 76.25 Asia/Kolkata
13.43417 77.72417 Asia/Kolkata
11.4 79.7 Asia/Kolkata
21.46667 79.68333 Asia/Kolkata
24.38333 74.7 Asia/Kolkata
22.31667 74.01667 Asia/Kolkata
22.06667 78.93333 Asia/Kolkata
27.1483 79.4979 Asia/Kolkata
24.9 79.6 Asia/Kolkata
26.21667 87.01667 Asia/Kolkata
27.71667 77.5 Asia/Kolkata
27.93333 78.41667 Asia/Kolkata
29.21667 77.18333 Asia/Kolkata
27.81667 74.4 Asia/Kolkata
23.3 72.76667 Asia/Kolkata
24.66667 76.83333 Asia/Kolkata
10.9 77.06667 Asia/Kolkata
13.07 80.24083 Asia/Kolkata
11.16667 77.61667 Asia/Kolkata
9.33333 76.63333 Asia/Kolkata
12.3 78.8 Asia/Kolkata
12.7 79.98333 Asia/Kolkata
10.53333 76.05 Asia/Kolkata
26.6 75.95 Asia/Kolkata
19.35 84.98333 Asia/Kolkata
24.21667 84.86667 Asia/Kolkata
23.63333 86.16667 Asia/Kolkata
29.55 77.58333 Asia/Kolkata
28.6 76.26667 Asia/Kolkata
25.4 79.75 Asia/Kolkata
26.26667 90.46667 Asia/Kolkata
12.90444 76.39167 Asia/Kolkata
12.65 77.21667 Asia/Kolkata
14.03333 75.93333 Asia/Kolkata
9.46667 76.55 Asia/Kolkata
20.33333 74.25 Asia/Kolkata
21.24167 77.74556 Asia/Kolkata
20.81667 77.96667 Asia/Kolkata
19.73333 79.18333 Asia/Kolkata
19.95 79.3 Asia/Kolkata
22.73333 87.51667 Asia/Kolkata
29.15 78.26667 Asia/Kolkata
30.7343 76.7933 Asia/Kolkata
24.71667 78.13333 Asia/Kolkata
28.45 78.76667 Asia/Kolkata
25.26667 83.26667 Asia/Kolkata
22.86917 88.37722 Asia/Kolkata
23.71667 72.11667 Asia/Kolkata
11.91667 76.95 Asia/Kolkata
22.05 82.65 Asia/Kolkata
32.56667 76.13333 Asia/Kolkata
16.11667 80.93333 Asia/Kolkata
14.31667 76.65 Asia/Kolkata
20.46667 75.01667 Asia/Kolkata
21.41667 71.16667 Asia/Kolkata
22.7 85.63333 Asia/Kolkata
22.65 72.93333 Asia/Kolkata
26.41667 85.05 Asia/Kolkata
18.75 73.85 Asia/Kolkata
22.56667 85.81667 Asia/Kolkata
22.31889 88.67139 Asia/Kolkata
11.86752 75.35763 Asia/Kolkata
22.56972 88.36972 Asia/Kolkata
14.68333 75.48333 Asia/Kolkata
25.58333 83.98333 Asia/Kolkata
21.5 83.86667 Asia/Kolkata
23.21667 81.53333 Asia/Kolkata
21.3 76.23333 Asia/Kolkata
23.18333 85.58333 Asia/Kolkata
25.44167 75.64167 Asia/Kolkata
20.53333 76.18333 Asia/Kolkata
28.4 77.85 Asia/Kolkata
29.93333 75.56667 Asia/Kolkata
29.28333 77.46667 Asia/Kolkata
28.05 79.11667 Asia/Kolkata
21.81667 83.91667 Asia/Kolkata
19.31667 84.78333 Asia/Kolkata
22.16667 71.66667 Asia/Kolkata
22.41667 72.9 Asia/Kolkata
19.23333 72.85 Asia/Kolkata
26.47822 90.55676 Asia/Kolkata
19.07283 72.88261 Asia/Kolkata
23.66667 87.71667 Asia/Kolkata
23.78333 85.96667 Asia/Kolkata
26.01667 93.78333 Asia/Kolkata
19.8 72.75 Asia/Kolkata
10.01667 77.35 Asia/Kolkata
24.7 84.98333 Asia/Kolkata
18.66667 77.9 Asia/Kolkata
18.56667 83.36667 Asia/Kolkata
27.5 81 Asia/Kolkata
28.25 75.08333 Asia/Kolkata
23.08333 87.31667 Asia/Kolkata
28.31667 78.93333 Asia/Kolkata
28.3 79.8 Asia/Kolkata
13.61667 75.96667 Asia/Kolkata
26.51667 87.01667 Asia/Kolkata
22.4 84.76667 Asia/Kolkata
21.03333 83.8 Asia/Kolkata
26.03333 80.6 Asia/Kolkata
24.1855 78.2031 Asia/Kolkata
26.12306 83.86889 Asia/Kolkata
28.13333 78.91667 Asia/Kolkata
28.25 79.95 Asia/Kolkata
26.85 80.08333 Asia/Kolkata
27.18333 80.03333 Asia/Kolkata
16.35 75.61667 Asia/Kolkata
28.88333 79.26667 Asia/Kolkata
22.08333 82.15 Asia/Kolkata
26.23195 90.23475 Asia/Kolkata
28.63333 78.8 Asia/Kolkata
26.17917 73.70556 Asia/Kolkata
25.2 84.25 Asia/Kolkata
28.01667 73.3 Asia/Kolkata
29.36667 78.13333 Asia/Kolkata
33.8 75.1 Asia/Kolkata
24.63333 79.5 Asia/Kolkata
16.83333 75.7 Asia/Kolkata
25.18333 85.51667 Asia/Kolkata
25.73333 86.98333 Asia/Kolkata
26.8022 79.5088 Asia/Kolkata
17.9 77.55 Asia/Kolkata
23.86667 76.91667 Asia/Kolkata
11.46667 79.63333 Asia/Kolkata
21.05 75.76667 Asia/Kolkata
18.46667 75.66667 Asia/Kolkata
23.26667 69.66667 Asia/Kolkata
16.9 74.6 Asia/Kolkata
20.23333 85.83333 Asia/Kolkata
20.88333 85.83333 Asia/Kolkata
18.16667 73.85 Asia/Kolkata
23.26667 77.4 Asia/Kolkata
17.51083 78.88889 Asia/Kolkata
27.25 79.18333 Asia/Kolkata
31.55 75.6325 Asia/Kolkata
28.78333 76.13333 Asia/Kolkata
19.3 73.06667 Asia/Kolkata
25.8 78.11667 Asia/Kolkata
25 72.25 Asia/Kolkata
27.71667 81.93333 Asia/Kolkata
24.5 74.18333 Asia/Kolkata
26.56417 78.78833 Asia/Kolkata
17.88333 83.43333 Asia/Kolkata
16.53333 81.53333 Asia/Kolkata
25.35 74.63333 Asia/Kolkata
21.21667 81.43333 Asia/Kolkata
30.06667 75.53333 Asia/Kolkata
21.86667 75.95 Asia/Kolkata
21.85 70.25 Asia/Kolkata
19.9 83.16667 Asia/Kolkata
30.26667 76.035 Asia/Kolkata
24.41667 75.83333 Asia/Kolkata
21.76667 72.15 Asia/Kolkata
11.45 77.68333 Asia/Kolkata
16.1 80.78333 Asia/Kolkata
22.87139 88.40889 Asia/Kolkata
13.96667 74.56667 Asia/Kolkata
30.2 74.95 Asia/Kolkata
21.73333 81.93333 Asia/Kolkata
27.03333 77.03333 Asia/Kolkata
25.55 81.5 Asia/Kolkata
21.7 72.96667 Asia/Kolkata
26.7571 79.2248 Asia/Kolkata
27.21667 77.48333 Asia/Kolkata
21.92639 69.77361 Asia/Kolkata
21.1 80.91667 Asia/Kolkata
24.513 75.7469 Asia/Kolkata
19.93333 84.58333 Asia/Kolkata
25.73333 78.75 Asia/Kolkata
21.16667 79.65 Asia/Kolkata
18.03333 77.21667 Asia/Kolkata
19.1 77.96667 Asia/Kolkata
21.64667 77.6325 Asia/Kolkata
25.25 87 Asia/Kolkata
22.82444 88.35056 Asia/Kolkata
13.86667 75.71667 Asia/Kolkata
21.05278 86.52 Asia/Kolkata
17.66667 80.88333 Asia/Kolkata
29.11667 75.16667 Asia/Kolkata
25.41667 82.56667 Asia/Kolkata
30.48333 75.31667 Asia/Kolkata
28.28333 74.31667 Asia/Kolkata
23.28333 70.35 Asia/Kolkata
25.05 83.61667 Asia/Kolkata
11.18333 75.81667 Asia/Kolkata
27.2184 79.3005 Asia/Kolkata
21.91528 77.89611 Asia/Kolkata
26.8 84.5 Asia/Kolkata
15.46667 78.16667 Asia/Kolkata
28.7 76.58333 Asia/Kolkata
23.63333 77.43333 Asia/Kolkata
24.05 81.38333 Asia/Kolkata
21.7 81.53333 Asia/Kolkata
13.16667 75.86667 Asia/Kolkata
26.45 85.4 Asia/Kolkata
23.25278 91.45139 Asia/Kolkata
15.15 76.93333 Asia/Kolkata
15.86667 74.5 Asia/Kolkata
23.93333 88.25 Asia/Kolkata
25.93333 81.98333 Asia/Kolkata
27.88333 76.28333 Asia/Kolkata
30.16667 77.61667 Asia/Kolkata
25.41667 86.13333 Asia/Kolkata
24.98333 75 Asia/Kolkata
23.6 78.33333 Asia/Kolkata
22.5 70.05 Asia/Kolkata
26.1 74.31667 Asia/Kolkata
29.15 79.11667 Asia/Kolkata
26.9 77.28333 Asia/Kolkata
28.8 77.03333 Asia/Kolkata
20.83333 84.31667 Asia/Kolkata
31.81861 75.20278 Asia/Kolkata
27.15 76.58333 Asia/Kolkata
21.13528 86.73861 Asia/Kolkata
26.8 82.71667 Asia/Kolkata
23.85 77.93333 Asia/Kolkata
27.16667 73.65 Asia/Kolkata
19.31667 77.16667 Asia/Kolkata
30.69167 76.39861 Asia/Kolkata
30.5888 76.8456 Asia/Kolkata
26.83333 76.03333 Asia/Kolkata
16.58333 75.96667 Asia/Kolkata
17.86667 76.95 Asia/Kolkata
22.03333 74.9 Asia/Kolkata
29.38333 75.91667 Asia/Kolkata
25.48333 85.98333 Asia/Kolkata
22.34972 88.43917 Asia/Kolkata
18.23333 75.7 Asia/Kolkata
26.31667 91 Asia/Kolkata
21.2 83.58333 Asia/Kolkata
30.38333 75.55 Asia/Kolkata
25.75 71.38333 Asia/Kolkata
24.16667 85.88333 Asia/Kolkata
23.61667 85.48333 Asia/Kolkata
23.61667 91.36667 Asia/Kolkata
24.41667 74.46667 Asia/Kolkata
26.65 77.6 Asia/Kolkata
25.28333 86.03333 Asia/Kolkata
25.48333 85.71667 Asia/Kolkata
22.98333 79.86667 Asia/Kolkata
21.33333 83.61667 Asia/Kolkata
28.35 79.41667 Asia/Kolkata
21.11667 73.11667 Asia/Kolkata
23.25 87.85 Asia/Kolkata
25.21667 85.73333 Asia/Kolkata
29.1 77.26667 Asia/Kolkata
26.4 84.58333 Asia/Kolkata
22.71667 88.51667 Asia/Kolkata
22.64333 88.36528 Asia/Kolkata
25.1 76.51667 Asia/Kolkata
34.2 74.35 Asia/Kolkata
18.15 74.58333 Asia/Kolkata
22.76139 88.37194 Asia/Kolkata
15.9 80.46667 Asia/Kolkata
30.55556 76.71833 Asia/Kolkata
12.9 75.03333 Asia/Kolkata
21.48639 70.06889 Asia/Kolkata
23.55 74.45 Asia/Kolkata
18.38333 77.88333 Asia/Kolkata
27.18333 82.93333 Asia/Kolkata
25.88333 84.21667 Asia/Kolkata
22.97194 88.39944 Asia/Kolkata
12.33083 76.86306 Asia/Kolkata
25.88333 87.18333 Asia/Kolkata
23.25 87.06667 Asia/Kolkata
20.38333 85.53333 Asia/Kolkata
24.88333 86.91667 Asia/Kolkata
26.9 80.21667 Asia/Kolkata
12.96667 78.2 Asia/Kolkata
23.06667 88.81667 Asia/Kolkata
15.31667 78.23333 Asia/Kolkata
12.97623 77.60329 Asia/Kolkata
31.18694 75.99222 Asia/Kolkata
34.41667 74.65 Asia/Kolkata
27.05 76.56667 Asia/Kolkata
25.48333 80.33333 Asia/Kolkata
24.0448 78.9626 Asia/Kolkata
29.46667 77.35 Asia/Kolkata
19.78333 85.18333 Asia/Kolkata
24.9 78.15 Asia/Kolkata
25.21667 88.76667 Asia/Kolkata
20.16667 85.1 Asia/Kolkata
27.43333 82.18333 Asia/Kolkata
25.83333 72.23333 Asia/Kolkata
21.66667 82.16667 Asia/Kolkata
20.73 81.20472 Asia/Kolkata
22.64611 88.34028 Asia/Kolkata
19.83333 79.35 Asia/Kolkata
25.18333 73.28333 Asia/Kolkata
21.49417 86.93167 Asia/Kolkata
23.11667 86.21667 Asia/Kolkata
20.66667 76.76667 Asia/Kolkata
20.71667 83.48333 Asia/Kolkata
21.8 80.18333 Asia/Kolkata
31.06667 76.31667 Asia/Kolkata
25.46667 85.51667 Asia/Kolkata
22.47472 88.17028 Asia/Kolkata
26.75 85.28333 Asia/Kolkata
13.86667 74.63333 Asia/Kolkata
15.81667 74.86667 Asia/Kolkata
22.1 80.55 Asia/Kolkata
22.785 88.32472 Asia/Kolkata
27.58333 81.6 Asia/Kolkata
28.4 78.61667 Asia/Kolkata
28.78333 79.5 Asia/Kolkata
24.1 88.25 Asia/Kolkata
28.68333 76.91667 Asia/Kolkata
26.26667 87.81667 Asia/Kolkata
26.87 78.5975 Asia/Kolkata
23.31667 88.65 Asia/Kolkata
28.95 77.21667 Asia/Kolkata
26.7 88.31667 Asia/Kolkata
30.68333 75.1 Asia/Kolkata
13.78472 77.79306 Asia/Kolkata
21.48333 70.95 Asia/Kolkata
28.1874 75.5005 Asia/Kolkata
16.18333 75.7 Asia/Kolkata
27.1 84.08333 Asia/Kolkata
14.75 79.05 Asia/Kolkata
22.74167 88.78583 Asia/Kolkata
23.01667 75.21667 Asia/Kolkata
19.15 73.26667 Asia/Kolkata
15.91495 75.67683 Asia/Kolkata
11.6 75.58333 Asia/Kolkata
22.11667 85.4 Asia/Kolkata
28.93333 78.21667 Asia/Kolkata
28.26667 78.4 Asia/Kolkata
21.85 71.3 Asia/Kolkata
25.25 78.46667 Asia/Kolkata
25.55 80.71667 Asia/Kolkata
22.7 77.93333 Asia/Kolkata
26.06 83.18611 Asia/Kolkata
26.8 82.2 Asia/Kolkata
10.45 77.55 Asia/Kolkata
16.02148 80.91808 Asia/Kolkata
11.2 77.28333 Asia/Kolkata
13.11556 80.10167 Asia/Kolkata
18.25 76.5 Asia/Kolkata
24.75 84.36667 Asia/Kolkata
19.88333 75.33333 Asia/Kolkata
26.46667 79.51667 Asia/Kolkata
18.25 77.43333 Asia/Kolkata
11.6 78.61667 Asia/Kolkata
8.68333 76.83333 Asia/Kolkata
28.03333 78.28333 Asia/Kolkata
15.88333 78.58333 Asia/Kolkata
10.35 79.4 Asia/Kolkata
16.73333 75.06667 Asia/Kolkata
20.53333 85.61667 Asia/Kolkata
25.28333 80.56667 Asia/Kolkata
25.73333 74.33333 Asia/Kolkata
19.6 84.65 Asia/Kolkata
19.36667 79.28333 Asia/Kolkata
23.01667 76.71667 Asia/Kolkata
16.95 74.4 Asia/Kolkata
24.56667 77.71667 Asia/Kolkata
23.68333 86.98333 Asia/Kolkata
29.51667 76.6 Asia/Kolkata
20.98333 78.23333 Asia/Kolkata
9.51667 78.1 Asia/Kolkata
8.56667 78.11667 Asia/Kolkata
9.86667 76.35 Asia/Kolkata
13.31389 76.25611 Asia/Kolkata
24.38333 77.41667 Asia/Kolkata
12.765 76.05694 Asia/Kolkata
11.13333 79.08333 Asia/Kolkata
12.9 79.33333 Asia/Kolkata
26.15 87.51667 Asia/Kolkata
10.16667 78.98333 Asia/Kolkata
12.66667 79.28333 Asia/Kolkata
18.66667 75.18333 Asia/Kolkata
21.2 81.96667 Asia/Kolkata
22.88333 87.78333 Asia/Kolkata
13.1 79.66667 Asia/Kolkata
25.56667 84.66667 Asia/Kolkata
28.28333 79.15 Asia/Kolkata
28.36667 78.26667 Asia/Kolkata
23.1 81.68333 Asia/Kolkata
29.19111 73.20861 Asia/Kolkata
25.15 76.3 Asia/Kolkata
20.03333 77.31667 Asia/Kolkata
11.23333 77.13333 Asia/Kolkata
15.43333 75.43333 Asia/Kolkata
21.6 73 Asia/Kolkata
23.13333 70.01667 Asia/Kolkata
21.16306 77.30944 Asia/Kolkata
22.03333 75.05 Asia/Kolkata
20.85 85.1 Asia/Kolkata
10.2 76.4 Asia/Kolkata
12.7 77.7 Asia/Kolkata
17.81306 78.07722 Asia/Kolkata
11.58333 77.6 Asia/Kolkata
9.98333 77.63333 Asia/Kolkata
33.73333 75.15 Asia/Kolkata
14.68333 77.6 Asia/Kolkata
31.25 76.5 Asia/Kolkata
22.56667 72.93333 Asia/Kolkata
17.68333 83.01667 Asia/Kolkata
10.58333 76.93333 Asia/Kolkata
18.41667 83.9 Asia/Kolkata
22.58 88.00833 Asia/Kolkata
21.3 72.85 Asia/Kolkata
28.91667 78.46667 Asia/Kolkata
31.63306 74.86556 Asia/Kolkata
21.61667 71.23333 Asia/Kolkata
20.93333 77.75 Asia/Kolkata
21.98333 72.9 Asia/Kolkata
20.28333 73.01667 Asia/Kolkata
22.83333 87.33333 Asia/Kolkata
21.93333 78.11667 Asia/Kolkata
25.3 73.93333 Asia/Kolkata
12.78333 78.7 Asia/Kolkata
23.11667 83.2 Asia/Kolkata
13.09833 80.16222 Asia/Kolkata
8.7 77.46667 Asia/Kolkata
30.37833 76.78083 Asia/Kolkata
18.73333 76.38333 Asia/Kolkata
26.7075 78.22611 Asia/Kolkata
19.61667 75.8 Asia/Kolkata
25.03333 86.9 Asia/Kolkata
24.31667 80.98333 Asia/Kolkata
19.2 73.16667 Asia/Kolkata
21.05 75.06667 Asia/Kolkata
16.58333 82.01667 Asia/Kolkata
10.11667 76.35 Asia/Kolkata
27.56667 76.6 Asia/Kolkata
23.76667 75.55 Asia/Kolkata
28.16667 94.76667 Asia/Kolkata
15.43333 74.73333 Asia/Kolkata
29.61667 79.66667 Asia/Kolkata
9.49004 76.3264 Asia/Kolkata
25.45 81.85 Asia/Kolkata
28.8 77.15 Asia/Kolkata
27.88333 78.08333 Asia/Kolkata
27.5 79.18333 Asia/Kolkata
18.64111 72.87917 Asia/Kolkata
8.86667 77.5 Asia/Kolkata
12.6 78.75 Asia/Kolkata
13.0025 80.20611 Asia/Kolkata
18.66667 73.9 Asia/Kolkata
17.56667 76.56667 Asia/Kolkata
21.09639 77.05861 Asia/Kolkata
20.73333 77 Asia/Kolkata
24.41667 76.56667 Asia/Kolkata
16.6 81.38333 Asia/Kolkata
26.41667 82.55 Asia/Kolkata
26.38333 79.95 Asia/Kolkata
22.01667 82.43333 Asia/Kolkata
17.53333 76.21667 Asia/Kolkata
16.11667 74.2 Asia/Kolkata
31.8425 74.75889 Asia/Kolkata
26.45 74.63333 Asia/Kolkata
23.73333 92.71667 Asia/Kolkata
25.01667 83.01667 Asia/Kolkata
18.7 76.93333 Asia/Kolkata
19.08333 74.73333 Asia/Kolkata
23.03333 72.61667 Asia/Kolkata
27.18333 78.01667 Asia/Kolkata
23.83639 91.275 Asia/Kolkata
23.7 76.01667 Asia/Kolkata
17.2 76.35 Asia/Kolkata
29.4 78.68333 Asia/Kolkata
9.16667 76.73333 Asia/Kolkata
23.5 86.66667 Asia/Kolkata
15.63333 77.28333 Asia/Kolkata
19.66667 78.53333 Asia/Kolkata
15.81667 79.98333 Asia/Kolkata
27.18333 77.76667 Asia/Kolkata
21.25722 77.50861 Asia/Kolkata
24.48333 72.78333 Asia/Kolkata
24.6 72.7 Asia/Kolkata
30.15 74.18333 Asia/Kolkata
26.33333 90.66667 Asia/Kolkata
21.77861 87.75361 Asia/Kolkata
22.06046 88.10975 Asia/Kolkata
22.94889 88.01889 Asia/Kolkata
22.64083 88.22028 Asia/Kolkata
22.6275 88.29806 Asia/Kolkata
22.63222 88.34861 Asia/Kolkata
22.59222 88.23778 Asia/Kolkata
22.58694 88.17306 Asia/Kolkata
22.54389 88.13806 Asia/Kolkata
22.50833 88.21528 Asia/Kolkata
22.46528 88.15167 Asia/Kolkata
22.10833 88.07889 Asia/Kolkata
22.28806 88.50917 Asia/Kolkata
34.07917 74.71111 Asia/Kolkata
17.46972 78.1275 Asia/Kolkata
17.44944 78.68528 Asia/Kolkata
13.29361 77.80167 Asia/Kolkata
31.43278 75.7175 Asia/Kolkata
13.03194 80.1575 Asia/Kolkata
12.9725 80.20917 Asia/Kolkata
12.97139 80.24722 Asia/Kolkata
12.8525 80.04667 Asia/Kolkata
19.1164 72.90471 Asia/Kolkata
14.0943 74.4845 Asia/Kolkata
18.53017 73.85263 Asia/Kolkata
28.49615 77.53601 Asia/Kolkata
30.67995 76.72211 Asia/Kolkata
22.60197 75.69649 Asia/Kolkata
21.65685 85.64349 Asia/Kolkata
19.15096 72.99625 Asia/Kolkata
10.10764 76.35158 Asia/Kolkata
27.70148 76.19917 Asia/Kolkata
30.5819 74.83298 Asia/Kolkata
9.97985 76.57381 Asia/Kolkata
10.10695 76.47366 Asia/Kolkata
13.40665 80.1238 Asia/Kolkata
20.37175 72.90493 Asia/Kolkata
30.95783 76.79136 Asia/Kolkata
28.58 77.33 Asia/Kolkata
28.21024 76.86056 Asia/Kolkata
23.08166 77.53328 Asia/Kolkata
24.19973 82.67535 Asia/Kolkata
26.70421 89.14547 Asia/Kolkata
26.84766 89.37558 Asia/Kolkata
13.65 79.42 Asia/Kolkata
19.05577 79.493 Asia/Kolkata
15.89794 79.32129 Asia/Kolkata
17.36687 78.5242 Asia/Kolkata
19.65399 78.51213 Asia/Kolkata
16.28584 81.25464 Asia/Kolkata
17.34769 78.55757 Asia/Kolkata
17.44781 78.52633 Asia/Kolkata
18.96506 79.47475 Asia/Kolkata
14.47516 78.8354 Asia/Kolkata
19.66781 78.5289 Asia/Kolkata
17.7 83.21667 Asia/Kolkata
17.98102 80.7547 Asia/Kolkata
14.54519 77.10552 Asia/Kolkata
16.07114 80.54944 Asia/Kolkata
17.50107 78.45818 Asia/Kolkata
16.94516 82.2385 Asia/Kolkata
17.58152 80.67651 Asia/Kolkata
26.50284 90.96937 Asia/Kolkata
17.24968 80.86899 Asia/Kolkata
16.48531 80.66746 Asia/Kolkata
26.37957 92.34146 Asia/Kolkata
27.10467 93.69518 Asia/Kolkata
17.49313 78.30196 Asia/Kolkata
26.20597 93.80951 Asia/Kolkata
25.74903 93.16998 Asia/Kolkata
23.18 88.58 Asia/Kolkata
22.86416 88.63701 Asia/Kolkata
23.65176 87.16475 Asia/Kolkata
30.96853 74.99106 Asia/Kolkata
37.14817 42.70008 Asia/Baghdad
30.04054 47.91677 Asia/Baghdad
34.89046 44.62994 Asia/Baghdad
34.60712 43.67822 Asia/Baghdad
36.48944 43.11917 Asia/Baghdad
36.80902 43.03886 Asia/Baghdad
34.20096 43.87562 Asia/Baghdad
32.71472 44.28028 Asia/Baghdad
36.61631 44.53111 Asia/Baghdad
34.48164 41.9093 Asia/Baghdad
36.27146 43.37373 Asia/Baghdad
33.74986 45.5541 Asia/Baghdad
36.08214 44.62778 Asia/Baghdad
34.69587 44.9661 Asia/Baghdad
35.46806 44.39222 Asia/Baghdad
32.61603 44.02488 Asia/Baghdad
36.19257 44.01062 Asia/Baghdad
33.64159 42.82509 Asia/Baghdad
35.17778 45.98611 Asia/Baghdad
34.13972 42.37806 Asia/Baghdad
35.53653 44.83167 Asia/Baghdad
34.93114 43.487 Asia/Baghdad
33.74846 44.65545 Asia/Baghdad
35.6183 45.9541 Asia/Baghdad
34.01638 44.14531 Asia/Baghdad
33.34058 44.40088 Asia/Baghdad
30.39447 47.70114 Asia/Baghdad
32.92556 44.77583 Asia/Baghdad
35.56113 45.43749 Asia/Baghdad
31.33198 45.2944 Asia/Baghdad
31.58737 44.66016 Asia/Baghdad
31.41171 46.17418 Asia/Baghdad
31.96362 44.59904 Asia/Baghdad
33.03807 40.28482 Asia/Baghdad
31.52629 45.20246 Asia/Baghdad
33.42257 43.30584 Asia/Baghdad
31.05205 46.26102 Asia/Baghdad
31.99854 44.33976 Asia/Baghdad
34.46589 41.9331 Asia/Baghdad
32.29799 44.68281 Asia/Baghdad
32.78332 44.28251 Asia/Baghdad
31.80416 44.49107 Asia/Baghdad
33.97861 44.93694 Asia/Baghdad
36.33464 43.09777 Asia/Baghdad
36.335 43.11889 Asia/Baghdad
31.57232 47.17529 Asia/Baghdad
32.5147 45.81902 Asia/Baghdad
33.81996 44.52637 Asia/Baghdad
32.46326 46.68637 Asia/Baghdad
32.54333 44.2225 Asia/Baghdad
32.48014 44.43526 Asia/Baghdad
32.17349 46.0433 Asia/Baghdad
30.71861 47.72083 Asia/Baghdad
31.74311 44.6196 Asia/Baghdad
30.96972 46.72278 Asia/Baghdad
29.97763 48.47412 Asia/Baghdad
33.3538 43.77951 Asia/Baghdad
30.53488 47.78885 Asia/Baghdad
32.90583 45.06163 Asia/Baghdad
31.84064 47.1396 Asia/Baghdad
32.06898 45.25055 Asia/Baghdad
33.84667 44.23444 Asia/Baghdad
31.99051 44.93063 Asia/Baghdad
33.30833 44.185 Asia/Baghdad
30.49721 47.81491 Asia/Baghdad
36.32092 41.87542 Asia/Baghdad
34.79088 48.57073 Asia/Tehran
34.3838 47.0553 Asia/Tehran
34.0734 47.9725 Asia/Tehran
35.23455 52.30942 Asia/Tehran
35.43944 51.56889 Asia/Tehran
36.213 49.196 Asia/Tehran
37.08701 55.17397 Asia/Tehran
30.81271 56.56399 Asia/Tehran
36.6736 48.4787 Asia/Tehran
31.89722 54.3675 Asia/Tehran
35.3242 51.6457 Asia/Tehran
35.27401 59.21949 Asia/Tehran
36.81626 50.87376 Asia/Tehran
35.69439 51.42151 Asia/Tehran
37.8008 48.9037 Asia/Tehran
36.0696 49.6959 Asia/Tehran
36.4009 47.1133 Asia/Tehran
31.7471 54.20882 Asia/Tehran
38.08 46.2919 Asia/Tehran
33.59586 56.92437 Asia/Tehran
34.7837 47.6004 Asia/Tehran
29.452 55.6814 Asia/Tehran
32.0455 48.8567 Asia/Tehran
32.1942 48.2436 Asia/Tehran
29.6036 52.5388 Asia/Tehran
32.32556 50.86444 Asia/Tehran
30.1165 55.1186 Asia/Tehran
30.64972 48.66472 Asia/Tehran
35.57291 53.39714 Asia/Tehran
31.41667 51.56667 Asia/Tehran
35.0213 50.3566 Asia/Tehran
36.56332 53.06009 Asia/Tehran
36.24992 46.2735 Asia/Tehran
35.3144 46.9923 Asia/Tehran
38.1973 44.7653 Asia/Tehran
36.6793 46.5669 Asia/Tehran
36.2126 57.68191 Asia/Tehran
37.13761 50.288 Asia/Tehran
35.4846 51.0829 Asia/Tehran
31.26562 56.80545 Asia/Tehran
37.28077 49.58319 Asia/Tehran
30.8946 49.4092 Asia/Tehran
31.27997 49.60351 Asia/Tehran
30.4067 55.9939 Asia/Tehran
37.106 58.50955 Asia/Tehran
35.1664 47.8045 Asia/Tehran
34.6401 50.8764 Asia/Tehran
26.9581 56.2719 Asia/Tehran
36.2797 50.0049 Asia/Tehran
38.8915 45.0255 Asia/Tehran
32.27174 50.98008 Asia/Tehran
33.72654 59.18439 Asia/Tehran
34.46109 45.86264 Asia/Tehran
35.308 51.7267 Asia/Tehran
36.701 45.1413 Asia/Tehran
35.0434 46.3565 Asia/Tehran
39.6482 47.9174 Asia/Tehran
37.0397 45.0983 Asia/Tehran
37.55274 45.07605 Asia/Tehran
30.75972 49.705 Asia/Tehran
30.11405 51.52174 Asia/Tehran
36.21329 58.79576 Asia/Tehran
29.1988 54.3278 Asia/Tehran
36.65079 53.29905 Asia/Tehran
35.9521 50.6075 Asia/Tehran
36.9553 45.388 Asia/Tehran
34.1885 48.377 Asia/Tehran
27.1467 57.0801 Asia/Tehran
36.9694 46.1027 Asia/Tehran
32.25014 54.01658 Asia/Tehran
31.59168 54.43178 Asia/Tehran
31.9364 49.3039 Asia/Tehran
29.92266 56.57434 Asia/Tehran
36.297 59.6062 Asia/Tehran
38.4329 45.7749 Asia/Tehran
34.2969 48.8235 Asia/Tehran
35.6659 50.9767 Asia/Tehran
36.7631 45.7222 Asia/Tehran
37.19701 50.15351 Asia/Tehran
33.535 47.6061 Asia/Tehran
26.55778 54.01944 Asia/Tehran
38.5503 44.9521 Asia/Tehran
33.22052 50.31497 Asia/Tehran
30.4397 48.1664 Asia/Tehran
33.48778 48.35583 Asia/Tehran
33.64226 50.0789 Asia/Tehran
37.6189 48.5258 Asia/Tehran
34.31417 47.065 Asia/Tehran
30.28321 57.07879 Asia/Tehran
29.61949 51.65415 Asia/Tehran
35.23831 58.46558 Asia/Tehran
33.98308 51.43644 Asia/Tehran
35.8355 51.0103 Asia/Tehran
34.5043 47.9653 Asia/Tehran
34.7956 46.9355 Asia/Tehran
37.38071 55.49164 Asia/Tehran
36.64115 52.91244 Asia/Tehran
34.79611 46.51722 Asia/Tehran
33.6374 46.4227 Asia/Tehran
34.2721 47.5861 Asia/Tehran
34.79922 48.51456 Asia/Tehran
36.83866 54.43475 Asia/Tehran
37.25004 55.16721 Asia/Tehran
34.35286 58.68365 Asia/Tehran
33.4537 50.28836 Asia/Tehran
27.6652 54.1371 Asia/Tehran
29.5791 50.517 Asia/Tehran
28.8438 52.5707 Asia/Tehran
36.68642 52.52255 Asia/Tehran
28.9383 53.6482 Asia/Tehran
32.25694 50.56096 Asia/Tehran
37.07645 57.51009 Asia/Tehran
36.41819 54.97628 Asia/Tehran
30.3586 50.7981 Asia/Tehran
33.9905 50.6838 Asia/Tehran
32.6941 47.2679 Asia/Tehran
30.7949 50.56458 Asia/Tehran
31.5608 48.1831 Asia/Tehran
28.75194 54.54444 Asia/Tehran
36.1683 54.348 Asia/Tehran
35.71778 52.065 Asia/Tehran
36.64546 59.12123 Asia/Tehran
36.655 51.4204 Asia/Tehran
28.9684 50.8385 Asia/Tehran
36.521 46.2089 Asia/Tehran
33.8973 48.7516 Asia/Tehran
31.96523 51.2873 Asia/Tehran
29.2666 51.2159 Asia/Tehran
37.47473 57.32903 Asia/Tehran
32.86628 59.22114 Asia/Tehran
35.8668 47.60506 Asia/Tehran
37.3404 46.0561 Asia/Tehran
36.69235 53.55262 Asia/Tehran
30.5959 50.2417 Asia/Tehran
35.9975 45.8853 Asia/Tehran
26.55792 54.88067 Asia/Tehran
30.42983 49.07628 Asia/Tehran
37.4727 49.4622 Asia/Tehran
27.1865 56.2808 Asia/Tehran
29.106 58.357 Asia/Tehran
34.9072 48.4414 Asia/Tehran
31.61283 55.41067 Asia/Tehran
36.70251 52.6576 Asia/Tehran
36.55132 52.67895 Asia/Tehran
33.6095 48.9307 Asia/Tehran
37.4779 47.0508 Asia/Tehran
38.4291 48.872 Asia/Tehran
37.25962 49.94444 Asia/Tehran
34.78241 48.11853 Asia/Tehran
33.3761 52.3694 Asia/Tehran
32.31001 54.01747 Asia/Tehran
38.2498 48.2933 Asia/Tehran
34.09174 49.68916 Asia/Tehran
36.46961 52.35072 Asia/Tehran
36.3187 49.1678 Asia/Tehran
35.6596 51.0593 Asia/Tehran
33.40063 49.6949 Asia/Tehran
33.8633 48.2619 Asia/Tehran
29.2464 52.7793 Asia/Tehran
37.4776 45.8943 Asia/Tehran
31.3203 48.6693 Asia/Tehran
38.4774 47.0699 Asia/Tehran
30.7006 49.8315 Asia/Tehran
36.06667 50.55 Asia/Tehran
36.1468 49.218 Asia/Tehran
32.9926 47.4198 Asia/Tehran
31.1608 52.6506 Asia/Tehran
30.3392 48.3043 Asia/Tehran
35.3022 51.5358 Asia/Tehran
32.03389 51.87944 Asia/Tehran
32.68462 51.60129 Asia/Tehran
32.5767 51.455 Asia/Tehran
32.6344 51.3668 Asia/Tehran
32.70018 51.52113 Asia/Tehran
32.55603 51.5263 Asia/Tehran
32.5553 51.50974 Asia/Tehran
32.65722 51.67761 Asia/Tehran
32.79978 51.69553 Asia/Tehran
32.61528 51.55556 Asia/Tehran
31.0287 61.5012 Asia/Tehran
29.4963 60.8629 Asia/Tehran
35.244 60.6225 Asia/Tehran
34.74 60.7756 Asia/Tehran
36.5449 61.1577 Asia/Tehran
28.22107 61.21582 Asia/Tehran
27.20245 60.68476 Asia/Tehran
25.2919 60.643 Asia/Tehran
65.68353 -18.0878 Atlantic/Reykjavik
64.13548 -21.89541 Atlantic/Reykjavik
64.11234 -21.91298 Atlantic/Reykjavik
64.0671 -21.93774 Atlantic/Reykjavik
36.95374 14.53318 Europe/Rome
38.07625 13.44485 Europe/Rome
38.67428 16.0951 Europe/Rome
39.93042 18.35533 Europe/Rome
38.01584 12.51077 Europe/Rome
37.98365 13.69555 Europe/Rome
38.35627 16.0109 Europe/Rome
37.08515 15.273 Europe/Rome
39.30222 9.20306 Europe/Rome
38.28333 16.26667 Europe/Rome
38.27788 16.302 Europe/Rome
39.30056 9.09083 Europe/Rome
39.25667 9.1675 Europe/Rome
37.29805 14.84078 Europe/Rome
36.79024 14.70139 Europe/Rome
37.50693 13.08399 Europe/Rome
37.57605 15.09809 Europe/Rome
39.2633 16.6996 Europe/Rome
37.48412 13.98542 Europe/Rome
39.5751 16.6349 Europe/Rome
36.82424 14.94779 Europe/Rome
38.48408 15.979 Europe/Rome
37.49844 13.26415 Europe/Rome
39.33199 16.18439 Europe/Rome
38.11047 15.66129 Europe/Rome
36.92824 14.71719 Europe/Rome
39.24194 9.18389 Europe/Rome
36.73054 14.84989 Europe/Rome
37.29344 13.52636 Europe/Rome
37.38595 14.36717 Europe/Rome
37.57095 14.90268 Europe/Rome
38.04395 13.12004 Europe/Rome
39.35989 16.03959 Europe/Rome
38.35862 15.84915 Europe/Rome
37.19264 13.76496 Europe/Rome
38.11582 13.35976 Europe/Rome
37.32825 14.74478 Europe/Rome
36.71864 15.0907 Europe/Rome
39.90306 8.59194 Europe/Rome
36.89014 15.06929 Europe/Rome
37.14864 14.39128 Europe/Rome
38.98333 16.31667 Europe/Rome
39.4048 16.15799 Europe/Rome
39.25611 9.13972 Europe/Rome
38.08005 13.28854 Europe/Rome
36.84594 14.77399 Europe/Rome
37.51815 15.03089 Europe/Rome
38.03385 13.45175 Europe/Rome
38.22117 15.23828 Europe/Rome
38.19327 15.54969 Europe/Rome
37.66414 12.58804 Europe/Rome
37.59215 15.02989 Europe/Rome
37.79664 12.43518 Europe/Rome
37.10714 13.94607 Europe/Rome
37.28545 14.99959 Europe/Rome
39.31139 8.535 Europe/Rome
38.42397 15.899 Europe/Rome
37.72686 15.18769 Europe/Rome
37.08034 14.23068 Europe/Rome
37.08695 15.15189 Europe/Rome
37.31754 13.66226 Europe/Rome
38.03704 12.58653 Europe/Rome
37.55885 14.28917 Europe/Rome
39.0851 17.11781 Europe/Rome
39.30999 16.25019 Europe/Rome
39.595 16.5177 Europe/Rome
36.95134 14.61018 Europe/Rome
38.89079 16.5987 Europe/Rome
37.50213 15.08719 Europe/Rome
39.8203 16.21229 Europe/Rome
37.68264 12.79164 Europe/Rome
39.783 16.31819 Europe/Rome
40.01502 18.16303 Europe/Rome
37.26955 15.01259 Europe/Rome
38.15665 13.17764 Europe/Rome
39.16722 8.52222 Europe/Rome
39.17444 8.97111 Europe/Rome
37.35894 13.85036 Europe/Rome
37.48774 14.04497 Europe/Rome
37.23682 14.5132 Europe/Rome
39.20738 9.13462 Europe/Rome
37.78946 14.83418 Europe/Rome
37.64515 14.86708 Europe/Rome
37.59465 14.98338 Europe/Rome
38.14756 15.21048 Europe/Rome
38.07935 13.50935 Europe/Rome
36.91134 15.1395 Europe/Rome
37.23655 15.21969 Europe/Rome
39.29056 8.99694 Europe/Rome
38.38333 16.13333 Europe/Rome
37.99835 12.95654 Europe/Rome
37.32084 13.58876 Europe/Rome
37.66175 14.83478 Europe/Rome
39.4929 16.38379 Europe/Rome
37.60616 15.12019 Europe/Rome
37.60666 15.16279 Europe/Rome
37.59596 15.14019 Europe/Rome
37.55825 15.13459 Europe/Rome
44.48966 11.21831 Europe/Rome
44.99467 9.00862 Europe/Rome
45.97667 12.30333 Europe/Rome
42.41783 12.10856 Europe/Rome
45.61188 9.36705 Europe/Rome
45.73833 12.23361 Europe/Rome
40.92161 14.18983 Europe/Rome
45.35238 10.84019 Europe/Rome
45.4276 11.95941 Europe/Rome
44.48296 11.0048 Europe/Rome
45.31166 8.86114 Europe/Rome
40.65981 14.43174 Europe/Rome
45.55729 11.5409 Europe/Rome
43.87354 10.2558 Europe/Rome
44.92947 10.51859 Europe/Rome
45.43419 10.99779 Europe/Rome
41.6824 13.41774 Europe/Rome
45.32306 8.41533 Europe/Rome
43.78956 7.60872 Europe/Rome
45.43861 12.32667 Europe/Rome
45.12385 7.63542 Europe/Rome
41.66922 12.77929 Europe/Rome
42.12434 14.70592 Europe/Rome
45.82908 8.82193 Europe/Rome
41.04533 16.88509 Europe/Rome
45.01406 8.64144 Europe/Rome
45.6483 11.30639 Europe/Rome
43.72621 12.63633 Europe/Rome
43.30555 12.33655 Europe/Rome
46.06194 13.24222 Europe/Rome
41.06673 16.92599 Europe/Rome
45.64861 13.78 Europe/Rome
45.42307 9.06644 Europe/Rome
45.66667 12.245 Europe/Rome
45.52378 9.58946 Europe/Rome
46.06787 11.12108 Europe/Rome
45.43477 8.73684 Europe/Rome
41.27273 16.41537 Europe/Rome
45.71097 8.90764 Europe/Rome
44.89784 8.86374 Europe/Rome
41.69074 15.29634 Europe/Rome
40.78392 14.3708 Europe/Rome
40.74971 14.46464 Europe/Rome
45.07049 7.68682 Europe/Rome
43.21255 13.29008 Europe/Rome
42.77994 12.40456 Europe/Rome
41.96092 12.79888 Europe/Rome
45.7088 11.47959 Europe/Rome
40.81221 14.49644 Europe/Rome
41.28967 13.24127 Europe/Rome
42.56713 12.64987 Europe/Rome
41.98884 14.98953 Europe/Rome
41.13113 16.54528 Europe/Rome
42.66123 13.69901 Europe/Rome
42.25419 11.75657 Europe/Rome
40.47611 17.22972 Europe/Rome
44.99187 10.74309 Europe/Rome
42.04793 13.92911 Europe/Rome
40.43653 18.04182 Europe/Rome
42.45344 14.14101 Europe/Rome
42.74484 12.73737 Europe/Rome
40.62601 14.37441 Europe/Rome
41.71572 13.6141 Europe/Rome
46.169 9.86915 Europe/Rome
40.87161 14.43613 Europe/Rome
45.68297 8.70873 Europe/Rome
43.79235 11.09792 Europe/Rome
43.32004 11.33283 Europe/Rome
41.50331 13.0641 Europe/Rome
45.66013 9.16028 Europe/Rome
45.14025 7.76802 Europe/Rome
44.27317 9.39683 Europe/Rome
45.53449 9.23401 Europe/Rome
43.83455 11.19522 Europe/Rome
41.24082 13.93282 Europe/Rome
45.68338 9.72286 Europe/Rome
45.65238 9.20034 Europe/Rome
43.70926 13.21667 Europe/Rome
45.57627 9.12454 Europe/Rome
45.39139 11.80051 Europe/Rome
45.49624 9.29323 Europe/Rome
45.57361 12.10611 Europe/Rome
45.7124 11.35739 Europe/Rome
43.75423 11.18794 Europe/Rome
44.59646 10.6907 Europe/Rome
40.75281 14.53384 Europe/Rome
44.30905 8.47715 Europe/Rome
44.64404 7.65593 Europe/Rome
40.40342 17.55841 Europe/Rome
44.55086 10.7847 Europe/Rome
40.72722 8.56028 Europe/Rome
44.11145 9.95859 Europe/Rome
45.62787 9.03464 Europe/Rome
40.81081 14.61984 Europe/Rome
40.65733 17.70721 Europe/Rome
40.79303 16.75529 Europe/Rome
43.22986 13.68608 Europe/Rome
44.06326 12.44657 Europe/Rome
40.72371 14.54654 Europe/Rome
40.94571 14.23143 Europe/Rome
40.86691 14.40483 Europe/Rome
41.08021 14.25653 Europe/Rome
41.68974 15.37604 Europe/Rome
43.57465 12.14304 Europe/Rome
42.04334 14.72722 Europe/Rome
43.81725 7.7772 Europe/Rome
40.48528 17.99972 Europe/Rome
41.83844 15.56535 Europe/Rome
43.68994 10.81741 Europe/Rome
45.10585 7.75452 Europe/Rome
41.71314 15.63985 Europe/Rome
44.46777 11.41401 Europe/Rome
40.82981 14.50444 Europe/Rome
43.76164 10.4404 Europe/Rome
45.39574 9.28757 Europe/Rome
43.56667 11.53333 Europe/Rome
41.70494 15.71905 Europe/Rome
45.38789 11.03799 Europe/Rome
44.63997 11.18411 Europe/Rome
40.45662 17.3787 Europe/Rome
40.83071 14.33763 Europe/Rome
45.4199 9.26628 Europe/Rome
45.63167 12.56944 Europe/Rome
43.65905 11.18952 Europe/Rome
45.39689 11.2844 Europe/Rome
42.94562 13.88157 Europe/Rome
45.62537 8.78344 Europe/Rome
44.64644 7.48623 Europe/Rome
44.81476 9.97958 Europe/Rome
40.67797 14.76599 Europe/Rome
45.95412 12.50274 Europe/Rome
41.30025 13.02815 Europe/Rome
41.11363 16.48778 Europe/Rome
41.01113 17.00549 Europe/Rome
45.38187 9.16055 Europe/Rome
45.07998 11.79301 Europe/Rome
45.8896 11.03868 Europe/Rome
43.40793 10.47291 Europe/Rome
42.66635 14.0227 Europe/Rome
45.52458 9.74836 Europe/Rome
41.89474 12.4839 Europe/Rome
45.07155 7.52622 Europe/Rome
45.03404 7.51842 Europe/Rome
44.06333 12.58083 Europe/Rome
42.40723 12.85918 Europe/Rome
43.99942 12.65689 Europe/Rome
45.53087 9.04014 Europe/Rome
44.69825 10.63125 Europe/Rome
43.40386 13.55978 Europe/Rome
44.4175 12.20111 Europe/Rome
44.3496 9.22796 Europe/Rome
43.84985 10.98072 Europe/Rome
40.92261 14.15013 Europe/Rome
40.85183 17.12129 Europe/Rome
43.88425 11.09092 Europe/Rome
40.83196 14.11001 Europe/Rome
40.64432 15.80857 Europe/Rome
40.835 8.39722 Europe/Rome
43.25826 13.75829 Europe/Rome
43.18266 13.79339 Europe/Rome
45.77528 12.83861 Europe/Rome
43.30696 13.72058 Europe/Rome
40.81563 14.33716 Europe/Rome
45.96389 12.6575 Europe/Rome
43.66164 10.63171 Europe/Rome
40.64251 14.87345 Europe/Rome
43.77365 11.42823 Europe/Rome
40.74611 14.49734 Europe/Rome
40.90791 14.38623 Europe/Rome
41.66701 12.50518 Europe/Rome
40.99593 17.21589 Europe/Rome
40.21151 16.67259 Europe/Rome
40.80311 14.53904 Europe/Rome
43.47244 11.14562 Europe/Rome
43.92125 10.92361 Europe/Rome
40.39062 16.55679 Europe/Rome
43.71553 10.39659 Europe/Rome
45.2975 12.04278 Europe/Rome
44.99054 7.46372 Europe/Rome
42.93482 10.52212 Europe/Rome
45.50502 9.33151 Europe/Rome
43.58333 13.5 Europe/Rome
44.88333 7.35 Europe/Rome
43.95224 10.2248 Europe/Rome
44.39296 11.34231 Europe/Rome
45.04676 9.69937 Europe/Rome
43.89705 10.69041 Europe/Rome
45.4308 9.3087 Europe/Rome
42.46024 14.21021 Europe/Rome
43.90357 12.89026 Europe/Rome
43.1122 12.38878 Europe/Rome
46.065 11.23798 Europe/Rome
44.33146 10.835 Europe/Rome
45.18446 9.16145 Europe/Rome
44.80266 10.32898 Europe/Rome
45.55497 8.94564 Europe/Rome
41.05753 16.70078 Europe/Rome
45.60368 9.89716 Europe/Rome
41.83552 12.88899 Europe/Rome
40.57889 17.03861 Europe/Rome
40.75151 14.61334 Europe/Rome
45.67417 12.16389 Europe/Rome
45.41519 11.88181 Europe/Rome
45.56899 9.16483 Europe/Rome
40.84981 14.49014 Europe/Rome
40.72763 17.57641 Europe/Rome
43.48276 13.48748 Europe/Rome
42.71953 12.11156 Europe/Rome
42.34964 14.40391 Europe/Rome
41.32993 15.71066 Europe/Rome
40.49733 17.64131 Europe/Rome
45.00384 7.53992 Europe/Rome
45.87977 8.40802 Europe/Rome
40.92137 9.48563 Europe/Rome
45.78 12.48806 Europe/Rome
40.32106 9.32973 Europe/Rome
44.76085 8.78955 Europe/Rome
45.53097 9.13474 Europe/Rome
45.44056 8.61684 Europe/Rome
45.59037 9.19954 Europe/Rome
40.92201 14.53294 Europe/Rome
41.03383 16.98959 Europe/Rome
40.79073 17.1317 Europe/Rome
40.74211 14.67294 Europe/Rome
40.75221 14.64014 Europe/Rome
44.99404 7.64182 Europe/Rome
41.45907 12.66037 Europe/Rome
45.55667 8.97774 Europe/Rome
45.52879 10.93798 Europe/Rome
42.51713 12.52307 Europe/Rome
40.17512 18.02972 Europe/Rome
40.83333 14.25 Europe/Rome
40.91131 14.20513 Europe/Rome
45.58837 9.22625 Europe/Rome
40.63528 17.03528 Europe/Rome
45.58248 9.27485 Europe/Rome
45.41518 10.39068 Europe/Rome
43.52765 11.56893 Europe/Rome
42.50394 14.138 Europe/Rome
42.05252 12.61868 Europe/Rome
43.92165 11.03112 Europe/Rome
45.50369 11.412 Europe/Rome
43.88355 10.77361 Europe/Rome
45.77528 12.03889 Europe/Rome
43.87035 10.81231 Europe/Rome
45.24239 11.74901 Europe/Rome
40.95593 17.2896 Europe/Rome
45.80722 13.53417 Europe/Rome
41.11411 13.89352 Europe/Rome
44.39603 7.81764 Europe/Rome
44.99625 7.69032 Europe/Rome
41.20023 16.59748 Europe/Rome
41.06094 17.08729 Europe/Rome
45.565 12.235 Europe/Rome
41.09743 16.77798 Europe/Rome
44.64783 10.92539 Europe/Rome
45.49444 12.10472 Europe/Rome
44.88677 11.0662 Europe/Rome
45.43 12.11972 Europe/Rome
41.25761 13.72371 Europe/Rome
45.46427 9.18951 Europe/Rome
45.49028 12.2425 Europe/Rome
40.55883 17.80831 Europe/Rome
40.78121 14.75694 Europe/Rome
46.66817 11.15953 Europe/Rome
42.03492 12.64238 Europe/Rome
45.50127 9.42525 Europe/Rome
40.92051 14.23003 Europe/Rome
40.99652 15.65136 Europe/Rome
45.35877 9.32395 Europe/Rome
45.65818 9.15974 Europe/Rome
40.66852 16.60158 Europe/Rome
43.86764 10.3397 Europe/Rome
40.59194 17.11528 Europe/Rome
44.02204 10.11409 Europe/Rome
40.70313 17.3339 Europe/Rome
45.5475 12.15 Europe/Rome
42.90594 12.33586 Europe/Rome
41.76242 12.64438 Europe/Rome
44.03837 10.04142 Europe/Rome
40.92631 14.45813 Europe/Rome
45.69208 9.17974 Europe/Rome
41.03221 14.30093 Europe/Rome
40.89761 14.19053 Europe/Rome
44.52476 10.8655 Europe/Rome
45.16031 10.79784 Europe/Rome
41.62594 15.90936 Europe/Rome
40.40182 17.63351 Europe/Rome
45.79078 8.88193 Europe/Rome
40.11842 18.29893 Europe/Rome
45.46457 8.88264 Europe/Rome
41.04001 14.37683 Europe/Rome
43.29816 13.44108 Europe/Rome
44.42707 11.90763 Europe/Rome
41.50823 15.33505 Europe/Rome
43.84357 10.50585 Europe/Rome
45.31357 9.50286 Europe/Rome
43.54264 10.316 Europe/Rome
45.61236 9.23985 Europe/Rome
45.59677 9.12904 Europe/Rome
41.73212 12.27654 Europe/Rome
45.59597 8.90784 Europe/Rome
45.18318 11.3091 Europe/Rome
45.85317 9.39005 Europe/Rome
40.35703 18.17202 Europe/Rome
41.46614 12.9043 Europe/Rome
40.55293 17.72051 Europe/Rome
43.77135 11.10282 Europe/Rome
44.11054 9.84339 Europe/Rome
42.35055 13.39954 Europe/Rome
42.21674 14.38822 Europe/Rome
46.42679 11.33841 Europe/Rome
45.57176 9.02681 Europe/Rome
41.95372 12.07347 Europe/Rome
45.46723 7.87617 Europe/Rome
41.60022 14.23822 Europe/Rome
40.73333 13.95 Europe/Rome
40.73523 13.94813 Europe/Rome
43.8875 8.03083 Europe/Rome
44.35227 11.71582 Europe/Rome
45.53311 12.64475 Europe/Rome
43.52142 13.24368 Europe/Rome
42.01667 12.75 Europe/Rome
43.35445 12.57246 Europe/Rome
43.23165 12.78096 Europe/Rome
40.93591 14.25983 Europe/Rome
45.06805 7.57762 Europe/Rome
40.53923 17.4337 Europe/Rome
41.78362 12.67568 Europe/Rome
42.77142 11.10794 Europe/Rome
40.81972 16.42278 Europe/Rome
40.69561 14.51234 Europe/Rome
45.94088 13.62167 Europe/Rome
45.53157 9.40465 Europe/Rome
45.69408 9.21124 Europe/Rome
42.76115 13.9537 Europe/Rome
40.92741 14.19103 Europe/Rome
41.18643 16.66738 Europe/Rome
40.79673 16.92359 Europe/Rome
40.58012 16.75609 Europe/Rome
45.40108 10.27927 Europe/Rome
41.70132 12.69178 Europe/Rome
44.40632 8.93386 Europe/Rome
45.57647 9.07174 Europe/Rome
40.05556 17.97639 Europe/Rome
45.66019 8.79164 Europe/Rome
40.14292 18.07042 Europe/Rome
40.17402 18.16822 Europe/Rome
41.21408 13.57082 Europe/Rome
43.72784 10.81011 Europe/Rome
41.64002 13.3401 Europe/Rome
40.95551 14.27273 Europe/Rome
40.93931 14.27403 Europe/Rome
41.81632 12.66868 Europe/Rome
40.53063 17.58521 Europe/Rome
42.42064 14.28701 Europe/Rome
44.54894 7.72013 Europe/Rome
44.57296 10.847 Europe/Rome
41.25626 13.60588 Europe/Rome
44.22361 12.05278 Europe/Rome
41.35411 13.43131 Europe/Rome
42.92633 10.76162 Europe/Rome
42.94404 12.70107 Europe/Rome
41.46093 15.54925 Europe/Rome
41.76747 12.22907 Europe/Rome
43.76667 11.25 Europe/Rome
44.53596 10.8218 Europe/Rome
44.83457 11.2939 Europe/Rome
43.61955 11.46933 Europe/Rome
44.86396 10.06668 Europe/Rome
44.82678 11.62071 Europe/Rome
43.16466 13.72329 Europe/Rome
41.69532 13.2564 Europe/Rome
46.02351 11.907 Europe/Rome
40.83443 17.3584 Europe/Rome
43.82036 13.01206 Europe/Rome
43.62326 13.40307 Europe/Rome
44.2857 11.88334 Europe/Rome
43.34065 12.90726 Europe/Rome
45.22289 11.65871 Europe/Rome
40.80631 14.36093 Europe/Rome
45.80958 9.23124 Europe/Rome
43.71754 10.94142 Europe/Rome
40.61421 15.05785 Europe/Rome
46.1165 8.29313 Europe/Rome
45.62018 9.20424 Europe/Rome
45.46798 10.53398 Europe/Rome
45.64928 9.60395 Europe/Rome
45.55307 9.18344 Europe/Rome
44.39733 7.54453 Europe/Rome
45.13617 10.02797 Europe/Rome
45.35877 9.66826 Europe/Rome
45.57386 8.18172 Europe/Rome
43.27467 11.98533 Europe/Rome
45.43099 9.11093 Europe/Rome
44.76977 10.7823 Europe/Rome
45.49697 9.02614 Europe/Rome
45.53957 9.17094 Europe/Rome
45.98972 12.70194 Europe/Rome
43.12904 12.28736 Europe/Rome
41.14553 16.41387 Europe/Rome
44.89298 11.82492 Europe/Rome
40.27152 18.05652 Europe/Rome
40.96623 17.11479 Europe/Rome
45.89056 12.2925 Europe/Rome
45.80998 9.08744 Europe/Rome
44.6975 12.18583 Europe/Rome
45.53004 9.27795 Europe/Rome
43.58884 10.47481 Europe/Rome
45.07755 7.57242 Europe/Rome
41.72622 13.00359 Europe/Rome
43.41514 11.13142 Europe/Rome
42.09325 11.79674 Europe/Rome
42.29523 12.40917 Europe/Rome
43.46905 12.23045 Europe/Rome
45.65 11.7842 Europe/Rome
41.5908 12.82808 Europe/Rome
45.23505 7.59882 Europe/Rome
45.55707 9.22185 Europe/Rome
41.80162 12.60658 Europe/Rome
45.18935 7.88722 Europe/Rome
45.22556 12.28889 Europe/Rome
42.36094 14.13801 Europe/Rome
45.01395 7.82233 Europe/Rome
44.31771 9.32241 Europe/Rome
45.53588 9.93036 Europe/Rome
44.19987 12.3991 Europe/Rome
44.1391 12.24315 Europe/Rome
45.63038 9.15224 Europe/Rome
44.26204 12.34812 Europe/Rome
41.99262 12.09237 Europe/Rome
43.54704 11.04022 Europe/Rome
45.52716 9.3334 Europe/Rome
41.26403 15.90046 Europe/Rome
45.19208 11.2103 Europe/Rome
40.85661 14.35773 Europe/Rome
44.72597 11.28821 Europe/Rome
40.64393 17.5155 Europe/Rome
43.31283 10.52401 Europe/Rome
41.56772 13.3333 Europe/Rome
45.13556 12.08167 Europe/Rome
40.70091 14.70564 Europe/Rome
43.95747 12.73755 Europe/Rome
45.39058 10.48528 Europe/Rome
41.03353 13.94217 Europe/Rome
44.39857 11.58482 Europe/Rome
44.57707 11.36071 Europe/Rome
40.62722 16.93689 Europe/Rome
40.88643 17.1655 Europe/Rome
40.71551 14.48914 Europe/Rome
45.6761 11.926 Europe/Rome
44.59577 11.052 Europe/Rome
43.61004 10.96922 Europe/Rome
43.46316 13.54518 Europe/Rome
41.48762 13.83151 Europe/Rome
45.52358 9.51395 Europe/Rome
40.90664 14.29587 Europe/Rome
41.08322 14.33493 Europe/Rome
45.17785 7.64262 Europe/Rome
43.67914 10.49941 Europe/Rome
40.95713 16.92039 Europe/Rome
40.90921 14.34573 Europe/Rome
45.13336 8.45714 Europe/Rome
44.47557 11.27591 Europe/Rome
41.01141 14.12482 Europe/Rome
44.06294 10.06069 Europe/Rome
44.78237 10.8777 Europe/Rome
44.84704 7.71803 Europe/Rome
40.94201 14.29943 Europe/Rome
45.67368 9.23594 Europe/Rome
41.09971 14.22013 Europe/Rome
43.87474 10.5728 Europe/Rome
40.42441 15.08085 Europe/Rome
45.72938 9.14054 Europe/Rome
41.21963 16.06747 Europe/Rome
41.56003 14.66753 Europe/Rome
43.82455 11.13242 Europe/Rome
40.66531 15.10585 Europe/Rome
43.90974 10.2419 Europe/Rome
43.85595 11.16272 Europe/Rome
40.95711 14.31023 Europe/Rome
45.61128 8.84914 Europe/Rome
45.46919 10.85058 Europe/Rome
44.53707 11.53362 Europe/Rome
40.92321 14.42393 Europe/Rome
45.55301 9.29907 Europe/Rome
40.62773 17.93682 Europe/Rome
45.53792 9.18921 Europe/Rome
46.71503 11.65598 Europe/Rome
45.52478 10.22727 Europe/Rome
42.10306 12.17543 Europe/Rome
44.70304 7.85563 Europe/Rome
40.77561 14.49944 Europe/Rome
43.95535 11.38352 Europe/Rome
45.70237 8.45813 Europe/Rome
44.88618 11.41081 Europe/Rome
46.49272 11.33358 Europe/Rome
44.49381 11.33875 Europe/Rome
45.54647 9.12054 Europe/Rome
41.11083 16.68938 Europe/Rome
41.24203 16.50438 Europe/Rome
45.55986 8.05002 Europe/Rome
45.69798 9.66895 Europe/Rome
41.12952 14.78614 Europe/Rome
46.145 12.22139 Europe/Rome
45.02314 7.58392 Europe/Rome
40.61091 14.98515 Europe/Rome
43.06794 12.54556 Europe/Rome
45.7676 11.7357 Europe/Rome
40.74771 14.77084 Europe/Rome
41.31183 16.29077 Europe/Rome
41.11773 16.85118 Europe/Rome
45.47506 8.9978 Europe/Rome
40.81322 14.16807 Europe/Rome
43.75115 11.32252 Europe/Rome
44.41247 11.97713 Europe/Rome
40.79926 14.07846 Europe/Rome
42.04023 13.4388 Europe/Rome
40.97321 14.20493 Europe/Rome
40.91442 14.78874 Europe/Rome
44.89795 8.20684 Europe/Rome
43.06914 12.61646 Europe/Rome
42.85483 13.5749 Europe/Rome
45.51929 11.33859 Europe/Rome
40.91441 14.26713 Europe/Rome
41.72012 12.67588 Europe/Rome
41.14892 15.08334 Europe/Rome
44.61237 11.83562 Europe/Rome
43.46139 11.87691 Europe/Rome
45.55227 9.07654 Europe/Rome
41.6123 12.51433 Europe/Rome
45.62668 9.32555 Europe/Rome
41.58951 12.65009 Europe/Rome
45.73736 7.3166 Europe/Rome
41.44749 12.62707 Europe/Rome
40.74271 14.57144 Europe/Rome
41.23063 16.29087 Europe/Rome
43.59816 13.51008 Europe/Rome
41.74472 13.15269 Europe/Rome
40.82923 16.55368 Europe/Rome
45.10195 7.52662 Europe/Rome
40.55889 8.31806 Europe/Rome
44.91245 8.61894 Europe/Rome
45.76378 9.79636 Europe/Rome
45.34749 11.86721 Europe/Rome
44.04997 8.21829 Europe/Rome
41.72873 12.6608 Europe/Rome
44.6999 8.0347 Europe/Rome
41.73062 13.3402 Europe/Rome
40.3469 14.99655 Europe/Rome
40.92601 14.31083 Europe/Rome
45.05639 12.05778 Europe/Rome
41.00423 16.87169 Europe/Rome
44.67514 8.46755 Europe/Rome
40.89783 16.84249 Europe/Rome
40.95071 14.37623 Europe/Rome
45.39821 8.91678 Europe/Rome
45.36099 11.79141 Europe/Rome
45.49306 12.16056 Europe/Rome
45.92136 8.55183 Europe/Rome
45.35052 9.20268 Europe/Rome
45.64788 10.26487 Europe/Rome
45.41617 9.10784 Europe/Rome
41.99362 12.72238 Europe/Rome
38.96589 16.3092 Europe/Rome
45.66972 8.81996 Europe/Rome
45.47771 9.05574 Europe/Rome
41.00581 14.45583 Europe/Rome
41.05181 14.33133 Europe/Rome
40.87681 14.13943 Europe/Rome
37.58125 15.07089 Europe/Rome
40.89921 14.27663 Europe/Rome
40.87841 14.34293 Europe/Rome
37.55555 15.06299 Europe/Rome
45.44207 9.09445 Europe/Rome
42.00167 12.62194 Europe/Rome
49.18804 -2.10491 Europe/Jersey
17.99107 -76.95742 America/Jamaica
17.97024 -76.86722 America/Jamaica
17.94144 -77.10898 America/Jamaica
18.00747 -76.78319 America/Jamaica
18.47116 -77.91883 America/Jamaica
17.96454 -77.24515 America/Jamaica
18.04168 -77.50714 America/Jamaica
18.13683 -77.03171 America/Jamaica
17.99702 -76.79358 America/Jamaica
18.01248 -76.79928 America/Jamaica
31.95126 35.81979 Asia/Amman
31.88542 35.8543 Asia/Amman
31.87207 36.00501 Asia/Amman
31.71667 35.8 Asia/Amman
32.27639 35.59938 Asia/Amman
32.28082 35.89929 Asia/Amman
31.15889 35.69278 Asia/Amman
32.55556 35.85 Asia/Amman
32.07275 36.08796 Asia/Amman
32.50722 35.8575 Asia/Amman
30.83752 35.60442 Asia/Amman
32.03917 35.72722 Asia/Amman
32.55917 36.00694 Asia/Amman
32.3063 35.75653 Asia/Amman
31.95522 35.94503 Asia/Amman
31.9167 35.95 Asia/Amman
32.34289 36.20804 Asia/Amman
31.18248 35.69999 Asia/Amman
32.02581 35.86458 Asia/Amman
29.52667 35.00778 Asia/Amman
32.33385 35.75238 Asia/Amman
31.16368 35.76204 Asia/Amman
32.01778 36.04639 Asia/Amman
33.73333 135.98333 Asia/Tokyo
35.43889 139.35972 Asia/Tokyo
34.63333 134.98333 Asia/Tokyo
35.29167 139.58556 Asia/Tokyo
35.48889 139.38861 Asia/Tokyo
39 139.91667 Asia/Tokyo
33.73333 130.98333 Asia/Tokyo
36.3 139.88333 Asia/Tokyo
35.15 139.06667 Asia/Tokyo
35.88528 139.83944 Asia/Tokyo
36.25 138.98333 Asia/Tokyo
37.68333 138.88333 Asia/Tokyo
36.11667 139.2 Asia/Tokyo
35.88333 139.63333 Asia/Tokyo
35.43333 133.33333 Asia/Tokyo
35.28361 139.66722 Asia/Tokyo
35.44778 139.6425 Asia/Tokyo
34.96667 136.61667 Asia/Tokyo
35.11626 136.19768 Asia/Tokyo
34.87009 135.7027 Asia/Tokyo
32.5 130.6 Asia/Tokyo
36.56667 137.13333 Asia/Tokyo
35.43333 133.25 Asia/Tokyo
35.06801 136.02327 Asia/Tokyo
34.91667 134.96667 Asia/Tokyo
35.82255 139.83905 Asia/Tokyo
34.61667 135.6 Asia/Tokyo
33.96667 132.11667 Asia/Tokyo
33.16667 130.4 Asia/Tokyo
35 134.55 Asia/Tokyo
34.18583 131.47139 Asia/Tokyo
33.01667 130.68333 Asia/Tokyo
34.86667 138.33333 Asia/Tokyo
36.8 139.93333 Asia/Tokyo
35.82639 139.70444 Asia/Tokyo
34.06667 134.15 Asia/Tokyo
36.56583 139.88361 Asia/Tokyo
32.68333 130.66667 Asia/Tokyo
33.10806 131.78778 Asia/Tokyo
32.19056 130.02278 Asia/Tokyo
33.1 129.98333 Asia/Tokyo
35.67056 139.88861 Asia/Tokyo
36.8 137.4 Asia/Tokyo
33.56667 130.5 Asia/Tokyo
36.68333 139.96667 Asia/Tokyo
34.88333 135.8 Asia/Tokyo
35.61667 139.11667 Asia/Tokyo
34.75 136.13333 Asia/Tokyo
32.9 130.68333 Asia/Tokyo
36.4 138.26667 Asia/Tokyo
33.94306 131.25111 Asia/Tokyo
35.05 134 Asia/Tokyo
35.16667 136.71667 Asia/Tokyo
38.72167 139.82167 Asia/Tokyo
36.45 136.63333 Asia/Tokyo
35.64547 136.0558 Asia/Tokyo
32.1 131.4 Asia/Tokyo
33.07056 131.85722 Asia/Tokyo
33.26667 131.15 Asia/Tokyo
36.66667 136.73333 Asia/Tokyo
37.65 138.93333 Asia/Tokyo
34.73028 136.50861 Asia/Tokyo
35.08333 137.15 Asia/Tokyo
36.29991 137.90108 Asia/Tokyo
35.53333 134.83333 Asia/Tokyo
34.78244 135.46932 Asia/Tokyo
34.81667 137.4 Asia/Tokyo
34.76667 137.38333 Asia/Tokyo
34.7 136.93333 Asia/Tokyo
36.69528 137.21139 Asia/Tokyo
35.5 134.23333 Asia/Tokyo
33.36667 130.51667 Asia/Tokyo
34.48333 134.18333 Asia/Tokyo
34.5 135.6 Asia/Tokyo
36.25 138.9 Asia/Tokyo
26.185 127.675 Asia/Tokyo
35.6895 139.69171 Asia/Tokyo
34.05 131.81667 Asia/Tokyo
34.06583 134.55944 Asia/Tokyo
35.79916 139.46903 Asia/Tokyo
34.88333 136.85 Asia/Tokyo
35.35 137.18333 Asia/Tokyo
37.13333 138.76667 Asia/Tokyo
32.83333 129.85 Asia/Tokyo
37.46667 139 Asia/Tokyo
36.38333 139.73333 Asia/Tokyo
34.48333 136.85 Asia/Tokyo
34.58333 135.83333 Asia/Tokyo
39.9 139.96667 Asia/Tokyo
34.55 135.8 Asia/Tokyo
35.98426 137.99721 Asia/Tokyo
34.86667 134.55 Asia/Tokyo
34.98333 139.86667 Asia/Tokyo
36.25 139.53333 Asia/Tokyo
31.48333 130.7 Asia/Tokyo
35.36667 136.53333 Asia/Tokyo
33.35 130.68333 Asia/Tokyo
36.36667 139.58333 Asia/Tokyo
35.72526 139.5383 Asia/Tokyo
35.72389 139.52222 Asia/Tokyo
34.81667 135.76667 Asia/Tokyo
33.73333 135.36667 Asia/Tokyo
34.48333 133.95 Asia/Tokyo
32.91667 130.56667 Asia/Tokyo
36.3 139.11667 Asia/Tokyo
34.85 136.91667 Asia/Tokyo
33.2 130.01667 Asia/Tokyo
34.33833 132.91667 Asia/Tokyo
35.90393 136.16687 Asia/Tokyo
32.96667 131.4 Asia/Tokyo
36.13333 137.25 Asia/Tokyo
34.84833 135.61678 Asia/Tokyo
36.33333 139.01667 Asia/Tokyo
34.79936 135.35697 Asia/Tokyo
36.75 137.01667 Asia/Tokyo
32.13333 131.5 Asia/Tokyo
34.34028 134.04333 Asia/Tokyo
34.51667 135.43333 Asia/Tokyo
34.78333 133.61667 Asia/Tokyo
34.91667 136.98333 Asia/Tokyo
33.5567 131.44506 Asia/Tokyo
35.31667 137.13333 Asia/Tokyo
34.66667 137.26667 Asia/Tokyo
33.63333 130.8 Asia/Tokyo
34.275 133.75 Asia/Tokyo
35.69278 139.41806 Asia/Tokyo
34.88333 136.58333 Asia/Tokyo
36.65 138.31667 Asia/Tokyo
36.03799 138.11308 Asia/Tokyo
33.36667 133.28333 Asia/Tokyo
34.35 134.9 Asia/Tokyo
32.93333 132.73333 Asia/Tokyo
34.76143 135.51567 Asia/Tokyo
37.83333 139.23333 Asia/Tokyo
36.03333 139.73333 Asia/Tokyo
31.65 131.01667 Asia/Tokyo
35.82028 139.80444 Asia/Tokyo
34.66667 133.75 Asia/Tokyo
35.25 136.71667 Asia/Tokyo
36.06667 139.6 Asia/Tokyo
34.85 133.01667 Asia/Tokyo
34.97694 138.38306 Asia/Tokyo
37.76667 139.01667 Asia/Tokyo
36.01667 139.66667 Asia/Tokyo
34.78333 134.71667 Asia/Tokyo
37.03333 138.85 Asia/Tokyo
36.1 137.96667 Asia/Tokyo
34.9 137.5 Asia/Tokyo
34.55 133.26667 Asia/Tokyo
33.71667 130.43333 Asia/Tokyo
35.60902 139.73017 Asia/Tokyo
33.95 130.95 Asia/Tokyo
36.3 139.98333 Asia/Tokyo
34.66667 138.95 Asia/Tokyo
34.18333 135.2 Asia/Tokyo
36.78333 137.06667 Asia/Tokyo
34.81667 138.18333 Asia/Tokyo
32.78333 130.36667 Asia/Tokyo
35.83333 139.58333 Asia/Tokyo
34.32333 134.17333 Asia/Tokyo
31.46667 131.105 Asia/Tokyo
36.48333 139 Asia/Tokyo
37.95 139.33333 Asia/Tokyo
35.23333 137.1 Asia/Tokyo
33.15 130.46667 Asia/Tokyo
31.81667 130.3 Asia/Tokyo
36.06667 139.71667 Asia/Tokyo
33.15917 129.72278 Asia/Tokyo
35.06667 135.21667 Asia/Tokyo
33.61667 130.53333 Asia/Tokyo
36.31667 139.58333 Asia/Tokyo
37.61667 138.95 Asia/Tokyo
34.88333 135.23333 Asia/Tokyo
34.5 135.85 Asia/Tokyo
36.21667 138.48333 Asia/Tokyo
38.91667 139.855 Asia/Tokyo
35.55 133.23333 Asia/Tokyo
34.31667 133.85 Asia/Tokyo
36.26667 139.25 Asia/Tokyo
36.1 139.8 Asia/Tokyo
34.58333 135.46667 Asia/Tokyo
35.95694 139.38889 Asia/Tokyo
32.95 131.9 Asia/Tokyo
34.68333 138.2 Asia/Tokyo
35.55306 139.35444 Asia/Tokyo
33.24932 130.2988 Asia/Tokyo
35.94647 136.18498 Asia/Tokyo
35.65 138.5 Asia/Tokyo
38.08333 138.43333 Asia/Tokyo
32.86667 130.86667 Asia/Tokyo
36.3 139.8 Asia/Tokyo
34.6 138.21667 Asia/Tokyo
34.06667 136.2 Asia/Tokyo
35.6 138.93333 Asia/Tokyo
35.00444 135.86833 Asia/Tokyo
34.2 132.21667 Asia/Tokyo
36.3 139.36667 Asia/Tokyo
34.69374 135.50218 Asia/Tokyo
34.41667 133.2 Asia/Tokyo
34.00139 131.18361 Asia/Tokyo
35.98106 136.48727 Asia/Tokyo
34.85 134.93333 Asia/Tokyo
34.28333 132.26667 Asia/Tokyo
33.03333 130.45 Asia/Tokyo
32.92139 129.95389 Asia/Tokyo
35.78389 139.24306 Asia/Tokyo
36.43333 139.28333 Asia/Tokyo
36.5 137.86667 Asia/Tokyo
32.06667 130.61667 Asia/Tokyo
36 139.55722 Asia/Tokyo
34.95 137.16667 Asia/Tokyo
34.66167 133.935 Asia/Tokyo
36.05659 138.0451 Asia/Tokyo
33.2 130.35 Asia/Tokyo
37.3 138.8 Asia/Tokyo
33.23806 131.6125 Asia/Tokyo
35.30306 139.30278 Asia/Tokyo
35.85111 139.51667 Asia/Tokyo
34.1 131.4 Asia/Tokyo
36.05 139.26667 Asia/Tokyo
35.35 136.61667 Asia/Tokyo
35.25556 139.15972 Asia/Tokyo
35.18333 132.5 Asia/Tokyo
35 136.96667 Asia/Tokyo
32.81667 129.88333 Asia/Tokyo
35.49576 135.74604 Asia/Tokyo
36.93333 137.5 Asia/Tokyo
35.1 138.86667 Asia/Tokyo
36.63333 139.05 Asia/Tokyo
36.53333 136.61667 Asia/Tokyo
33.73333 130.73333 Asia/Tokyo
35.94111 139.85806 Asia/Tokyo
32.58333 131.66667 Asia/Tokyo
34.98333 134.96667 Asia/Tokyo
34.86667 137.05 Asia/Tokyo
30.73333 131 Asia/Tokyo
34.71667 135.33333 Asia/Tokyo
35.7 138.45 Asia/Tokyo
35.30333 139.25333 Asia/Tokyo
36.75 139.61667 Asia/Tokyo
37.8 139.11667 Asia/Tokyo
34.98333 133.46667 Asia/Tokyo
33.95933 133.31672 Asia/Tokyo
37.90222 139.02361 Asia/Tokyo
31.6 131.36667 Asia/Tokyo
35.73333 139.65 Asia/Tokyo
28.36667 129.48333 Asia/Tokyo
34.18333 134.61667 Asia/Tokyo
34.68505 135.80485 Asia/Tokyo
37.05 136.96667 Asia/Tokyo
36.76667 137.33333 Asia/Tokyo
35.48333 137.5 Asia/Tokyo
33.60139 131.18417 Asia/Tokyo
36.58333 138.85 Asia/Tokyo
36.75 138.36667 Asia/Tokyo
32.98333 132.93333 Asia/Tokyo
33.81667 130.7 Asia/Tokyo
26.2125 127.68111 Asia/Tokyo
35.18147 136.90641 Asia/Tokyo
26.58806 127.97611 Asia/Tokyo
32.74472 129.87361 Asia/Tokyo
35.8563 139.90266 Asia/Tokyo
37.45 138.85 Asia/Tokyo
36.65139 138.18111 Asia/Tokyo
35.38333 136.26667 Asia/Tokyo
34.61667 136.08333 Asia/Tokyo
35.70611 139.55944 Asia/Tokyo
33.28333 134.15 Asia/Tokyo
37.68333 139.18333 Asia/Tokyo
38.23333 139.48333 Asia/Tokyo
34.96545 135.70415 Asia/Tokyo
37.06667 138.88333 Asia/Tokyo
35.93556 139.30444 Asia/Tokyo
35.06667 135.98333 Asia/Tokyo
34.73333 135.56667 Asia/Tokyo
34.83333 137.93333 Asia/Tokyo
35.36667 137.25 Asia/Tokyo
35.08333 137.06667 Asia/Tokyo
34.8 132.85 Asia/Tokyo
35.53333 135.18333 Asia/Tokyo
31.91111 131.42389 Asia/Tokyo
31.73333 131.06667 Asia/Tokyo
33.71667 130.66667 Asia/Tokyo
37.53333 138.93333 Asia/Tokyo
36.01667 139.98333 Asia/Tokyo
35.41667 137.13333 Asia/Tokyo
35.68351 139.55963 Asia/Tokyo
35.11667 138.91667 Asia/Tokyo
34.82691 135.47057 Asia/Tokyo
35.53333 136.91667 Asia/Tokyo
35.48333 139.45 Asia/Tokyo
32.21667 130.4 Asia/Tokyo
34.96667 136.16667 Asia/Tokyo
36.21706 136.15185 Asia/Tokyo
34.8 134.98333 Asia/Tokyo
34.4 133.08333 Asia/Tokyo
36.41667 139.8 Asia/Tokyo
36.21667 139.38333 Asia/Tokyo
36.51667 136.56667 Asia/Tokyo
34.56667 136.53333 Asia/Tokyo
36.23333 137.96667 Asia/Tokyo
35.47222 133.05056 Asia/Tokyo
35.78333 139.9 Asia/Tokyo
32.65 130.66667 Asia/Tokyo
34.56667 135.55 Asia/Tokyo
34.66667 131.85 Asia/Tokyo
36.1534 136.27029 Asia/Tokyo
36.31667 138.26667 Asia/Tokyo
34.28333 133.78333 Asia/Tokyo
31.26667 130.31667 Asia/Tokyo
37.75 138.88333 Asia/Tokyo
35.45 135.33333 Asia/Tokyo
36.39111 139.06083 Asia/Tokyo
33.55 130.2 Asia/Tokyo
35.54028 139.45083 Asia/Tokyo
35.02107 135.75385 Asia/Tokyo
35.06667 136.7 Asia/Tokyo
31.71667 130.26667 Asia/Tokyo
35.01667 135.96667 Asia/Tokyo
33.31667 130.51667 Asia/Tokyo
35.35 136.78333 Asia/Tokyo
36.13333 139.7 Asia/Tokyo
34.23333 132.56667 Asia/Tokyo
35.43333 133.81667 Asia/Tokyo
34.58333 133.76667 Asia/Tokyo
32.78972 130.74167 Asia/Tokyo
36.13333 139.38333 Asia/Tokyo
36.06667 139.66667 Asia/Tokyo
34 131.86667 Asia/Tokyo
34.8 137.35889 Asia/Tokyo
36.71667 137.1 Asia/Tokyo
35.88333 139.78333 Asia/Tokyo
34.7184 137.53175 Asia/Tokyo
36.05 139.51667 Asia/Tokyo
35.33165 136.87042 Asia/Tokyo
36.31667 138.43333 Asia/Tokyo
35 136.51667 Asia/Tokyo
34 134.58333 Asia/Tokyo
36.40263 136.45088 Asia/Tokyo
35.28333 136.91667 Asia/Tokyo
35.70222 139.47556 Asia/Tokyo
31.73333 130.76667 Asia/Tokyo
35.69139 139.50111 Asia/Tokyo
36.18333 139.71667 Asia/Tokyo
33.73333 130.46667 Asia/Tokyo
35.66389 138.56833 Asia/Tokyo
36.18333 139.13333 Asia/Tokyo
35.72639 139.48389 Asia/Tokyo
33.55972 133.53111 Asia/Tokyo
34.6913 135.183 Asia/Tokyo
31.98333 130.98333 Asia/Tokyo
33.41667 131.61667 Asia/Tokyo
33.83333 130.83333 Asia/Tokyo
37.65 139.86667 Asia/Tokyo
35.16667 135.91667 Asia/Tokyo
34.46667 135.36667 Asia/Tokyo
35.37472 139.9225 Asia/Tokyo
36.1 139.58333 Asia/Tokyo
36.4 139.33333 Asia/Tokyo
32.98333 130.81667 Asia/Tokyo
36.11667 139.6 Asia/Tokyo
35.52056 139.71722 Asia/Tokyo
33.58333 130.83333 Asia/Tokyo
34.81667 135.41667 Asia/Tokyo
35.805 139.72056 Asia/Tokyo
35.90861 139.48528 Asia/Tokyo
36.06173 136.50101 Asia/Tokyo
35.97639 139.75361 Asia/Tokyo
35.24762 136.97229 Asia/Tokyo
37.36667 138.55 Asia/Tokyo
35.85444 139.96889 Asia/Tokyo
33.10611 130.09056 Asia/Tokyo
34.50896 135.7929 Asia/Tokyo
34.58333 135.61667 Asia/Tokyo
31.41667 130.31667 Asia/Tokyo
34.5 133.5 Asia/Tokyo
35.36667 136.76667 Asia/Tokyo
34.98333 136.98333 Asia/Tokyo
33.4425 129.96972 Asia/Tokyo
33.3 130.36667 Asia/Tokyo
36.55 139.73333 Asia/Tokyo
31.38333 130.85 Asia/Tokyo
34.11667 133.65 Asia/Tokyo
34.53333 133.38333 Asia/Tokyo
35.13333 136.8 Asia/Tokyo
36.4 139 Asia/Tokyo
33.78333 130.98333 Asia/Tokyo
36.59444 136.62556 Asia/Tokyo
34.81667 138.13333 Asia/Tokyo
34.06667 134.35 Asia/Tokyo
34.53333 133.58333 Asia/Tokyo
37.65 139.05 Asia/Tokyo
35.68333 139.55 Asia/Tokyo
36.43333 139.91667 Asia/Tokyo
36.7 137.36667 Asia/Tokyo
34.85 136.45 Asia/Tokyo
35 135.58333 Asia/Tokyo
37.86667 139.11667 Asia/Tokyo
35.30889 139.55028 Asia/Tokyo
34.76667 134.85 Asia/Tokyo
34.76667 138.01667 Asia/Tokyo
35.41667 136.86667 Asia/Tokyo
35.96536 140.64474 Asia/Tokyo
31.73333 130.66667 Asia/Tokyo
34.45 135.35 Asia/Tokyo
34.15 135.2 Asia/Tokyo
31.56018 130.55814 Asia/Tokyo
34.73333 135.58333 Asia/Tokyo
35.36667 132.76667 Asia/Tokyo
34.5 135.4 Asia/Tokyo
34.48333 135.43333 Asia/Tokyo
32.08333 130.36667 Asia/Tokyo
35.94278 139.69194 Asia/Tokyo
34.7 137.85 Asia/Tokyo
35.28333 136.86667 Asia/Tokyo
34.15 132.18333 Asia/Tokyo
36.05 139.9 Asia/Tokyo
34.25 135.31667 Asia/Tokyo
35.72528 139.21778 Asia/Tokyo
26.12472 127.66944 Asia/Tokyo
37.03333 137.85 Asia/Tokyo
34.96667 139.08333 Asia/Tokyo
34.78427 135.40126 Asia/Tokyo
34.8 137.01667 Asia/Tokyo
26.42333 127.82139 Asia/Tokyo
34.06667 134.43333 Asia/Tokyo
36.11667 139.96667 Asia/Tokyo
24.34478 124.15717 Asia/Tokyo
36.31667 139.2 Asia/Tokyo
35.39056 139.30778 Asia/Tokyo
34.48333 136.7 Asia/Tokyo
35.65 138.63333 Asia/Tokyo
32.84111 130.04306 Asia/Tokyo
35.37995 136.94295 Asia/Tokyo
33.55 133.43333 Asia/Tokyo
34.28333 133.18333 Asia/Tokyo
35.25 136.78333 Asia/Tokyo
35.82756 137.95378 Asia/Tokyo
33.26667 129.88333 Asia/Tokyo
36.71667 139.68333 Asia/Tokyo
34.68333 135.7 Asia/Tokyo
34.82208 135.4298 Asia/Tokyo
34.01667 133.8 Asia/Tokyo
31.61667 130.4 Asia/Tokyo
33.63333 130.68333 Asia/Tokyo
36.85 138.36667 Asia/Tokyo
35.51965 137.82074 Asia/Tokyo
34.76855 136.13013 Asia/Tokyo
35.3 136.8 Asia/Tokyo
35.71972 139.92472 Asia/Tokyo
31.23333 130.65 Asia/Tokyo
34.81641 135.56828 Asia/Tokyo
34.6 133.46667 Asia/Tokyo
36.3396 137.88254 Asia/Tokyo
36.23333 139.18333 Asia/Tokyo
32.45 130.2 Asia/Tokyo
34.05 131.56667 Asia/Tokyo
32.21667 130.75 Asia/Tokyo
33.31667 130.93333 Asia/Tokyo
34.66667 136.46667 Asia/Tokyo
34.39627 132.45937 Asia/Tokyo
35.32306 139.34222 Asia/Tokyo
35.43333 132.81667 Asia/Tokyo
24.8 125.28333 Asia/Tokyo
34.81352 135.64914 Asia/Tokyo
33.35972 129.55306 Asia/Tokyo
35.67306 139.40028 Asia/Tokyo
35 136.25 Asia/Tokyo
36.85 136.98333 Asia/Tokyo
34.81667 134.7 Asia/Tokyo
35.25 136.25 Asia/Tokyo
33.36667 131.53333 Asia/Tokyo
34.88333 136.98333 Asia/Tokyo
35.26667 139.58333 Asia/Tokyo
34.35 132.33333 Asia/Tokyo
35.83417 139.73639 Asia/Tokyo
34.31667 135.61667 Asia/Tokyo
36.16667 139.53333 Asia/Tokyo
35.85194 139.31806 Asia/Tokyo
34.88333 136.93333 Asia/Tokyo
31.71667 130.73333 Asia/Tokyo
34.7 137.73333 Asia/Tokyo
34.8 137.78333 Asia/Tokyo
34.88333 132.08333 Asia/Tokyo
36.88333 136.78333 Asia/Tokyo
34.53333 135.95 Asia/Tokyo
34.4 131.41667 Asia/Tokyo
26.33694 127.87194 Asia/Tokyo
35.37111 139.22361 Asia/Tokyo
35.65583 139.32389 Asia/Tokyo
35.12861 136.0976 Asia/Tokyo
36.13333 139.45 Asia/Tokyo
26.35444 127.86861 Asia/Tokyo
35 132.21667 Asia/Tokyo
35.3 138.93333 Asia/Tokyo
37.73333 139.16667 Asia/Tokyo
34.45 135.73333 Asia/Tokyo
34.35 135.7 Asia/Tokyo
35.41667 136.6 Asia/Tokyo
33.88333 135.15 Asia/Tokyo
26.26265 127.76147 Asia/Tokyo
35.42291 136.76039 Asia/Tokyo
34.83333 137.23333 Asia/Tokyo
35.30528 139.81861 Asia/Tokyo
35.69306 139.98333 Asia/Tokyo
34.48333 133.36667 Asia/Tokyo
34.75 137.91667 Asia/Tokyo
34.25 134.71667 Asia/Tokyo
33.60639 130.41806 Asia/Tokyo
36.55 136.86667 Asia/Tokyo
33.76667 130.46667 Asia/Tokyo
36.06443 136.22257 Asia/Tokyo
32.68806 128.84194 Asia/Tokyo
35.3 135.11667 Asia/Tokyo
36.1 139.45 Asia/Tokyo
36.2 139.28333 Asia/Tokyo
35.34194 139.47 Asia/Tokyo
36.25 139.65 Asia/Tokyo
36.25 139.08333 Asia/Tokyo
35.21667 138.61667 Asia/Tokyo
34.86667 138.26667 Asia/Tokyo
35.16667 138.68333 Asia/Tokyo
35.66667 139.48333 Asia/Tokyo
34.56667 133.23333 Asia/Tokyo
35.7 138.73333 Asia/Tokyo
34.7 135.61667 Asia/Tokyo
35.65556 139.55222 Asia/Tokyo
35 137.03333 Asia/Tokyo
35.9944 138.15428 Asia/Tokyo
33.49631 130.5156 Asia/Tokyo
35.32611 139.40389 Asia/Tokyo
35.99028 139.07639 Asia/Tokyo
26.30944 127.77028 Asia/Tokyo
33.27361 131.49194 Asia/Tokyo
35.3 135.25 Asia/Tokyo
35.08944 139.06861 Asia/Tokyo
34.72807 135.30264 Asia/Tokyo
36.33333 139.45 Asia/Tokyo
37.01667 138.25 Asia/Tokyo
36.31667 138.9 Asia/Tokyo
34.95828 137.08054 Asia/Tokyo
33.91667 134.65 Asia/Tokyo
33.41667 130.65 Asia/Tokyo
34.71667 135.41667 Asia/Tokyo
32.01667 130.2 Asia/Tokyo
34.75 134.4 Asia/Tokyo
33.5 133.9 Asia/Tokyo
34.80361 134.46806 Asia/Tokyo
35.96972 139.58861 Asia/Tokyo
26.33583 127.80139 Asia/Tokyo
31.45833 131.23333 Asia/Tokyo
33.955 131.95 Asia/Tokyo
34.36667 132.55 Asia/Tokyo
34.38333 131.2 Asia/Tokyo
35.97556 139.65083 Asia/Tokyo
35.8725 139.50972 Asia/Tokyo
35.85295 139.41212 Asia/Tokyo
35.73667 139.32361 Asia/Tokyo
35.67444 139.435 Asia/Tokyo
35.80472 139.60194 Asia/Tokyo
35.78944 139.62333 Asia/Tokyo
35.80944 139.6925 Asia/Tokyo
35.63639 139.57444 Asia/Tokyo
35.32528 139.89111 Asia/Tokyo
35.14 139.61917 Asia/Tokyo
34.22611 135.1675 Asia/Tokyo
33.75139 132.70139 Asia/Tokyo
33.22375 132.56001 Asia/Tokyo
33.91667 133.18333 Asia/Tokyo
33.5 132.55 Asia/Tokyo
33.83916 132.76574 Asia/Tokyo
33.78757 132.71124 Asia/Tokyo
34.01667 133.56667 Asia/Tokyo
33.97661 132.77767 Asia/Tokyo
39.16667 140.5 Asia/Tokyo
35.65 140.16667 Asia/Tokyo
37.91667 140.11667 Asia/Tokyo
39.3 140.56667 Asia/Tokyo
35.7 140.55 Asia/Tokyo
37.85 140.6 Asia/Tokyo
38.42738 141.21487 Asia/Tokyo
38.24056 140.36333 Asia/Tokyo
39.46667 141.95 Asia/Tokyo
35.65 140.31667 Asia/Tokyo
38.035 140.85111 Asia/Tokyo
38.53333 141.13333 Asia/Tokyo
35.96667 140.13333 Asia/Tokyo
36.2 140.1 Asia/Tokyo
35.9 140.08333 Asia/Tokyo
39.31667 141.53333 Asia/Tokyo
36.35 140.3 Asia/Tokyo
38.39306 140.88611 Asia/Tokyo
35.55 140.36667 Asia/Tokyo
38.35361 140.36972 Asia/Tokyo
38.0025 140.19111 Asia/Tokyo
36.71667 140.71667 Asia/Tokyo
37.28333 140.38333 Asia/Tokyo
39.68333 140.98333 Asia/Tokyo
35.71667 140.26667 Asia/Tokyo
38.00333 140.61833 Asia/Tokyo
35.8 140.06667 Asia/Tokyo
38.31667 141.03333 Asia/Tokyo
38.75861 140.30083 Asia/Tokyo
38.26889 140.87194 Asia/Tokyo
35.88333 140.5 Asia/Tokyo
35.71667 140.23333 Asia/Tokyo
38.3725 140.2725 Asia/Tokyo
35.9 140.18333 Asia/Tokyo
38.32361 140.97444 Asia/Tokyo
39.36667 141.9 Asia/Tokyo
36.86667 140.03333 Asia/Tokyo
36.55 140.41667 Asia/Tokyo
35.85 140.61667 Asia/Tokyo
39.45 140.48333 Asia/Tokyo
36.28333 140.41667 Asia/Tokyo
38.05 140.73361 Asia/Tokyo
35.25 140.38333 Asia/Tokyo
39.07167 141.71667 Asia/Tokyo
38.60333 140.40194 Asia/Tokyo
36.31667 140.6 Asia/Tokyo
35.51667 140.31667 Asia/Tokyo
37.58333 140.43333 Asia/Tokyo
35.6 140.41667 Asia/Tokyo
35.78333 140.31667 Asia/Tokyo
35.68333 140.03333 Asia/Tokyo
37.48333 141 Asia/Tokyo
36.05 140.16667 Asia/Tokyo
38.10361 140.035 Asia/Tokyo
37.51667 140.4 Asia/Tokyo
36.51667 140.18333 Asia/Tokyo
35.93333 140 Asia/Tokyo
39.70361 141.1525 Asia/Tokyo
36.43333 140.01667 Asia/Tokyo
35.41667 140.3 Asia/Tokyo
39.13333 141.13333 Asia/Tokyo
39.63667 141.9525 Asia/Tokyo
36.34139 140.44667 Asia/Tokyo
37.43333 140.48333 Asia/Tokyo
38.37357 141.06105 Asia/Tokyo
36.46667 140.1 Asia/Tokyo
37.91667 140.76667 Asia/Tokyo
36.26667 140.1 Asia/Tokyo
36.96667 140.05 Asia/Tokyo
37.4 140.38333 Asia/Tokyo
38.55 141.05 Asia/Tokyo
39.28333 141.11667 Asia/Tokyo
36.78333 140.75 Asia/Tokyo
35.1 140.1 Asia/Tokyo
35.13333 140.3 Asia/Tokyo
36.38333 140.53333 Asia/Tokyo
35.89767 140.49943 Asia/Tokyo
36.38333 140.26667 Asia/Tokyo
36.65 140.15 Asia/Tokyo
38.15389 140.27361 Asia/Tokyo
39.26667 141.88333 Asia/Tokyo
37.96667 140.78333 Asia/Tokyo
36.35 140.1 Asia/Tokyo
38.10472 140.85944 Asia/Tokyo
37.05 140.88333 Asia/Tokyo
35.93333 140.55 Asia/Tokyo
36.18333 140.26667 Asia/Tokyo
38.41667 141.3 Asia/Tokyo
37.15 140.45 Asia/Tokyo
37.56667 140.11667 Asia/Tokyo
38.91667 141.13333 Asia/Tokyo
35.51667 140.08333 Asia/Tokyo
37.81667 140.55 Asia/Tokyo
36.6 140.65 Asia/Tokyo
38.43889 140.40056 Asia/Tokyo
35.73333 140.83333 Asia/Tokyo
39.38333 141.11667 Asia/Tokyo
38.57167 140.95556 Asia/Tokyo
37.43333 140.58333 Asia/Tokyo
36.46667 140.56667 Asia/Tokyo
37.75 140.46778 Asia/Tokyo
35.91667 140.11667 Asia/Tokyo
35.95 140.31667 Asia/Tokyo
36.76667 140.35 Asia/Tokyo
35.60472 140.12333 Asia/Tokyo
35.71667 140.65 Asia/Tokyo
36.03333 140.2 Asia/Tokyo
39.71667 140.11667 Asia/Tokyo
35.86667 140.01667 Asia/Tokyo
39.71806 140.10333 Asia/Tokyo
43.82278 144.10444 Asia/Tokyo
45.40944 141.67389 Asia/Tokyo
42.63694 141.60333 Asia/Tokyo
43.21694 141.51694 Asia/Tokyo
41.28944 141.21694 Asia/Tokyo
43.55278 141.90639 Asia/Tokyo
40.22194 140.36944 Asia/Tokyo
43.48639 141.90556 Asia/Tokyo
42.33389 142.36694 Asia/Tokyo
42.55 141.35 Asia/Tokyo
43.35 142.38333 Asia/Tokyo
43.65899 145.13197 Asia/Tokyo
43.06417 141.34694 Asia/Tokyo
43.93444 141.64278 Asia/Tokyo
42.99167 143.20028 Asia/Tokyo
43.18944 141.00222 Asia/Tokyo
40.26861 140.56833 Asia/Tokyo
42.91722 143.20444 Asia/Tokyo
40.20389 140.02417 Asia/Tokyo
43.32361 145.575 Asia/Tokyo
44.35056 142.45778 Asia/Tokyo
41.88333 140.68333 Asia/Tokyo
40.70722 140.58944 Asia/Tokyo
42.31722 140.98806 Asia/Tokyo
44.3525 143.3525 Asia/Tokyo
40.68361 141.35972 Asia/Tokyo
43.20389 140.77028 Asia/Tokyo
45.37139 141.82111 Asia/Tokyo
42.975 144.37472 Asia/Tokyo
40.63889 140.59222 Asia/Tokyo
40.80611 140.38611 Asia/Tokyo
43.80306 143.89083 Asia/Tokyo
41.81667 140.65 Asia/Tokyo
42.97444 140.50889 Asia/Tokyo
43.20028 141.75972 Asia/Tokyo
43.23972 141.35389 Asia/Tokyo
40.20694 141.30167 Asia/Tokyo
42.97583 141.56722 Asia/Tokyo
40.59306 140.4725 Asia/Tokyo
40.18361 140.78722 Asia/Tokyo
41.77583 140.73667 Asia/Tokyo
40.5 141.5 Asia/Tokyo
43.70806 142.03917 Asia/Tokyo
43.10806 141.55056 Asia/Tokyo
42.46806 140.86806 Asia/Tokyo
42.81944 141.65222 Asia/Tokyo
43.32472 141.85861 Asia/Tokyo
43.50972 142.18556 Asia/Tokyo
43.76778 142.37028 Asia/Tokyo
40.82444 140.74 Asia/Tokyo
44.02127 144.26971 Asia/Tokyo
40.80444 140.44139 Asia/Tokyo
36.56922 136.91162 Asia/Tokyo
34.80447 136.54645 Asia/Tokyo
34.76615 135.62759 Asia/Tokyo
36.39659 140.53479 Asia/Tokyo
35.95633 140.32356 Asia/Tokyo
35.72878 139.29415 Asia/Tokyo
33.53567 130.47861 Asia/Tokyo
35.77674 140.00075 Asia/Tokyo
35.48199 137.02166 Asia/Tokyo
35.73691 136.95852 Asia/Tokyo
37.14828 138.23642 Asia/Tokyo
35.90807 139.65657 Asia/Tokyo
35.75459 139.46852 Asia/Tokyo
33.56543 131.73157 Asia/Tokyo
35.48933 138.68832 Asia/Tokyo
35.02954 135.75666 Asia/Tokyo
0.60751 34.76966 Africa/Nairobi
1.7471 40.05732 Africa/Nairobi
-3.39452 38.56304 Africa/Nairobi
-1.03326 37.06933 Africa/Nairobi
-0.17333 35.86382 Africa/Nairobi
-0.42013 36.94759 Africa/Nairobi
0.0421 36.36734 Africa/Nairobi
-1.07829 35.86012 Africa/Nairobi
0.01667 37.07283 Africa/Nairobi
-0.28333 36.06667 Africa/Nairobi
-0.71667 36.43591 Africa/Nairobi
-1.28333 36.81667 Africa/Nairobi
0.33511 34.4864 Africa/Nairobi
-0.15406 35.19878 Africa/Nairobi
3.5167 39.05842 Africa/Nairobi
-4.05466 39.66359 Africa/Nairobi
-0.2479 35.73743 Africa/Nairobi
-1.06344 34.47313 Africa/Nairobi
0.04626 37.65587 Africa/Nairobi
2.32839 37.98986 Africa/Nairobi
1.09679 36.69799 Africa/Nairobi
3.93663 41.86701 Africa/Nairobi
-3.21748 40.1191 Africa/Nairobi
-1.80409 37.62034 Africa/Nairobi
-1.51667 37.26667 Africa/Nairobi
0.39361 34.30528 Africa/Nairobi
3.11911 35.59727 Africa/Nairobi
-2.27169 40.90201 Africa/Nairobi
-1.36696 38.01056 Africa/Nairobi
1.01572 35.00622 Africa/Nairobi
-0.10221 34.76171 Africa/Nairobi
-0.68174 34.76666 Africa/Nairobi
-3.63045 39.84992 Africa/Nairobi
-1.16667 36.83333 Africa/Nairobi
-0.49886 37.28031 Africa/Nairobi
-0.36774 35.28314 Africa/Nairobi
-0.7 37.18333 Africa/Nairobi
0.28422 34.75228 Africa/Nairobi
0.35462 37.58218 Africa/Nairobi
-0.52731 34.45714 Africa/Nairobi
-0.45355 39.64011 Africa/Nairobi
-0.53112 37.4506 Africa/Nairobi
0.52036 35.26992 Africa/Nairobi
0.5635 34.56055 Africa/Nairobi
-1.45071 36.98245 Africa/Nairobi
0.06116 34.28823 Africa/Nairobi
39.93373 69.57 Asia/Bishkek
39.83856 69.53058 Asia/Bishkek
42.45568 76.18613 Asia/Bishkek
42.84172 75.30158 Asia/Bishkek
41.34612 72.21708 Asia/Bishkek
42.52222 72.24194 Asia/Bishkek
42.3414 78.00558 Asia/Bishkek
40.76994 73.30068 Asia/Bishkek
40.51506 72.80826 Asia/Bishkek
41.42866 75.99111 Asia/Bishkek
40.25684 72.12793 Asia/Bishkek
40.7046 72.86666 Asia/Tashkent
42.49068 78.39362 Asia/Bishkek
42.81423 73.84813 Asia/Bishkek
42.89106 74.85077 Asia/Bishkek
40.93333 73 Asia/Bishkek
42.64944 77.08225 Asia/Bishkek
42.87 74.59 Asia/Bishkek
41.0376 72.74586 Asia/Bishkek
41.17017 75.81058 Asia/Bishkek
41.87079 72.9402 Asia/Bishkek
11.56245 104.91601 Asia/Phnom_Penh
11.48333 104.95 Asia/Phnom_Penh
13.52586 105.9683 Asia/Phnom_Penh
13.58588 102.97369 Asia/Phnom_Penh
13.36667 103.85 Asia/Phnom_Penh
12.53878 103.9192 Asia/Phnom_Penh
10.98081 104.77828 Asia/Phnom_Penh
13.81667 104.96667 Asia/Phnom_Penh
13.65611 102.5625 Asia/Phnom_Penh
12.84895 102.60928 Asia/Phnom_Penh
13.48784 106.98158 Asia/Phnom_Penh
11.6175 102.98056 Asia/Phnom_Penh
12.48811 106.01879 Asia/Phnom_Penh
10.61667 104.18333 Asia/Phnom_Penh
12.71112 104.88873 Asia/Phnom_Penh
11.45 104.53333 Asia/Phnom_Penh
10.60932 103.52958 Asia/Phnom_Penh
12.25 104.66667 Asia/Phnom_Penh
11.99339 105.4635 Asia/Phnom_Penh
13.10271 103.19822 Asia/Phnom_Penh
1.3278 172.97696 Pacific/Tarawa
-12.16672 44.39944 Indian/Comoro
-11.70216 43.25506 Indian/Comoro
17.29484 -62.7261 America/St_Kitts
37.90889 126.16111 Asia/Pyongyang
39.15278 127.44361 Asia/Pyongyang
38.95389 127.89167 Asia/Pyongyang
38.99139 125.97778 Asia/Pyongyang
39.19833 125.69 Asia/Pyongyang
38.75417 125.645 Asia/Pyongyang
39.49472 125.47361 Asia/Pyongyang
38.41667 126.23333 Asia/Pyongyang
39.59806 125.60972 Asia/Pyongyang
38.50722 125.75583 Asia/Pyongyang
39.9475 127.87111 Asia/Pyongyang
39.03385 125.75432 Asia/Pyongyang
37.93472 125.36194 Asia/Pyongyang
38.7375 125.40778 Asia/Pyongyang
39.98111 125.24472 Asia/Pyongyang
39.86722 126.03028 Asia/Pyongyang
39.43806 127.24306 Asia/Pyongyang
38.85583 127.41806 Asia/Pyongyang
39.1425 126.09611 Asia/Pyongyang
37.97083 126.55444 Asia/Pyongyang
38.67028 125.77611 Asia/Pyongyang
39.83167 127.61861 Asia/Pyongyang
38.79861 125.79194 Asia/Pyongyang
38.71028 127.59833 Asia/Pyongyang
39.91833 127.53639 Asia/Pyongyang
38.04056 125.71472 Asia/Pyongyang
38.86389 125.8 Asia/Pyongyang
39.69333 125.21028 Asia/Pyongyang
38.25083 125.09611 Asia/Pyongyang
38.39917 125.61556 Asia/Pyongyang
38.24306 125.78 Asia/Pyongyang
39.61778 125.66472 Asia/Pyongyang
39.0425 127.52389 Asia/Pyongyang
38.51083 125.49417 Asia/Pyongyang
40.02472 128.15972 Asia/Pyongyang
40.19944 124.53167 Asia/Pyongyang
42.34222 130.39667 Asia/Pyongyang
40.10056 124.39806 Asia/Pyongyang
40.38944 125.04667 Asia/Pyongyang
40.01833 127.45472 Asia/Pyongyang
42.95722 129.99333 Asia/Pyongyang
41.71361 129.68444 Asia/Pyongyang
42.95 129.86667 Asia/Pyongyang
42.24889 130.30028 Asia/Pyongyang
42.22609 129.20776 Asia/Pyongyang
41.58778 129.60611 Asia/Pyongyang
40.96417 129.32778 Asia/Pyongyang
41.09028 128.29333 Asia/Pyongyang
40.96946 126.58523 Asia/Pyongyang
40.32306 128.65528 Asia/Pyongyang
41.39756 128.17873 Asia/Pyongyang
40.02528 127.95583 Asia/Pyongyang
42.44278 129.75083 Asia/Pyongyang
41.79556 129.77583 Asia/Pyongyang
42.52056 130.39528 Asia/Pyongyang
36.1125 129.35222 Asia/Seoul
34.74417 127.73778 Asia/Seoul
35.99417 129.345 Asia/Seoul
37.18447 128.46821 Asia/Seoul
36.82167 128.63083 Asia/Seoul
35.275 126.50944 Asia/Seoul
36.175 127.77639 Asia/Seoul
35.9675 128.93083 Asia/Seoul
38.09404 127.07577 Asia/Seoul
37.29583 127.63389 Asia/Seoul
36.67756 126.84272 Asia/Seoul
35.33861 129.03861 Asia/Seoul
37.48972 127.49056 Asia/Seoul
37.83311 127.06169 Asia/Seoul
38.10583 127.98944 Asia/Seoul
37.35139 127.94528 Asia/Seoul
35.84509 127.14752 Asia/Seoul
35.99175 128.39732 Asia/Seoul
35.53722 129.31667 Asia/Seoul
37.7415 127.0474 Asia/Seoul
36.89444 126.62972 Asia/Seoul
36.9714 126.4542 Asia/Seoul
36.32139 127.41972 Asia/Seoul
35.87028 128.59111 Asia/Seoul
36.34931 126.59772 Asia/Seoul
37.1759 128.9889 Asia/Seoul
37.29111 127.00889 Asia/Seoul
34.94808 127.48947 Asia/Seoul
37.56826 126.97783 Asia/Seoul
36.78167 126.45222 Asia/Seoul
36.24083 128.2975 Asia/Seoul
36.91556 127.13139 Asia/Seoul
38.20833 128.59111 Asia/Seoul
36.41528 128.16056 Asia/Seoul
37.44056 129.17083 Asia/Seoul
36.28194 126.9125 Asia/Seoul
35.10278 129.04028 Asia/Seoul
37.49889 126.78306 Asia/Seoul
35.72806 126.73194 Asia/Seoul
34.84494 127.34293 Asia/Seoul
36.03222 129.365 Asia/Seoul
37.15222 127.07056 Asia/Seoul
36.78361 127.00417 Asia/Seoul
36.3012 127.568 Asia/Seoul
38.37881 128.4676 Asia/Seoul
36.20389 127.08472 Asia/Seoul
35.41 127.38583 Asia/Seoul
35.02833 126.7175 Asia/Seoul
37.85944 126.785 Asia/Seoul
36.59458 128.19946 Asia/Seoul
34.79361 126.38861 Asia/Seoul
35.49333 128.74889 Asia/Seoul
35.20806 128.5725 Asia/Seoul
35.82333 128.73778 Asia/Seoul
35.84278 129.21167 Asia/Seoul
34.97528 127.58917 Asia/Seoul
37.41 127.25722 Asia/Seoul
35.15472 126.91556 Asia/Seoul
35.20944 127.46444 Asia/Seoul
37.5986 127.1394 Asia/Seoul
36.23472 128.57278 Asia/Seoul
35.97861 126.71139 Asia/Seoul
36.10306 127.48889 Asia/Seoul
36.1136 128.336 Asia/Seoul
37.65639 126.835 Asia/Seoul
34.97631 128.32361 Asia/Seoul
36.45556 127.12472 Asia/Seoul
34.85028 128.58861 Asia/Seoul
36.81083 127.79472 Asia/Seoul
35.43333 126.7 Asia/Seoul
35.80167 126.88889 Asia/Seoul
35.23417 128.88111 Asia/Seoul
36.12176 128.11981 Asia/Seoul
35.24417 129.21389 Asia/Seoul
37.83101 127.51059 Asia/Seoul
37.75556 128.89611 Asia/Seoul
37.74722 126.48556 Asia/Seoul
35.94389 126.95444 Asia/Seoul
37.45361 126.73167 Asia/Seoul
35.61306 127.27944 Asia/Seoul
37.27917 127.4425 Asia/Seoul
35.05944 126.985 Asia/Seoul
37.20682 126.8169 Asia/Seoul
38.10611 127.70667 Asia/Seoul
36.6009 126.665 Asia/Seoul
37.6918 127.8857 Asia/Seoul
35.91333 128.82 Asia/Seoul
34.57111 126.59889 Asia/Seoul
37.87472 127.73417 Asia/Seoul
35.82194 127.14889 Asia/Seoul
36.43351 129.057 Asia/Seoul
36.63722 127.48972 Asia/Seoul
36.8065 127.1522 Asia/Seoul
35.19278 128.08472 Asia/Seoul
35.14944 128.65972 Asia/Seoul
36.85667 127.44333 Asia/Seoul
35.79167 127.42528 Asia/Seoul
33.50972 126.52194 Asia/Seoul
35.22806 128.68111 Asia/Seoul
35.64842 127.51523 Asia/Seoul
37.3925 126.92694 Asia/Seoul
37.01083 127.27028 Asia/Seoul
37.32361 126.82194 Asia/Seoul
36.56556 128.725 Asia/Seoul
33.46444 126.31833 Asia/Seoul
34.8825 128.62667 Asia/Seoul
36.12944 127.1 Asia/Seoul
37.54389 129.10694 Asia/Seoul
37.29167 127.50778 Asia/Seoul
37.43861 127.13778 Asia/Seoul
37.54 127.20556 Asia/Seoul
37.6525 127.3075 Asia/Seoul
37.63667 127.21417 Asia/Seoul
35.40611 129.16861 Asia/Seoul
37.58972 127.22028 Asia/Seoul
35.24972 128.52 Asia/Seoul
35.80167 128.50083 Asia/Seoul
37.47722 126.86639 Asia/Seoul
34.8262 126.10863 Asia/Seoul
35.54145 128.49506 Asia/Seoul
29.26917 47.97806 Asia/Kuwait
29.3032 48.04881 Asia/Kuwait
29.33389 48.07611 Asia/Kuwait
29.14583 48.09472 Asia/Kuwait
29.09611 48.13278 Asia/Kuwait
29.145 48.13028 Asia/Kuwait
29.36972 47.97833 Asia/Kuwait
29.3375 47.65806 Asia/Kuwait
29.0825 48.13028 Asia/Kuwait
29.17389 48.12111 Asia/Kuwait
29.2775 47.95861 Asia/Kuwait
29.365 48.00139 Asia/Kuwait
29.29583 48.07861 Asia/Kuwait
29.295 47.93306 Asia/Kuwait
29.25722 48.05722 Asia/Kuwait
19.28692 -81.36706 America/Cayman
51.23333 51.36667 Asia/Oral
49.47444 57.42333 Asia/Aqtobe
46.95307 54.01978 Asia/Oral
50.25028 58.43472 Asia/Aqtobe
50.06667 54.86667 Asia/Aqtobe
48.82639 58.14417 Asia/Aqtobe
47.06667 51.86667 Asia/Oral
47.11667 51.88333 Asia/Oral
50.27969 57.20718 Asia/Aqtobe
43.65 51.2 Asia/Aqtau
51.16778 52.995 Asia/Oral
49.73861 84.27194 Asia/Almaty
45.48778 64.07806 Asia/Qyzylorda
47.78333 67.76667 Asia/Almaty
52.19083 61.20056 Asia/Qyzylorda
43.56667 69.75 Asia/Almaty
45.85361 62.15611 Asia/Qyzylorda
42.9 71.36667 Asia/Almaty
47.46667 84.86667 Asia/Almaty
43.915 67.24806 Asia/Qyzylorda
42.53333 70.33333 Asia/Almaty
45.24222 77.98222 Asia/Almaty
43.33333 68.25 Asia/Almaty
50.05444 72.94833 Asia/Almaty
44.83 78.82389 Asia/Almaty
44.77278 65.5525 Asia/Qyzylorda
43.30444 77.23278 Asia/Almaty
45.01556 78.37389 Asia/Almaty
45 77.91667 Asia/Almaty
42.3 69.6 Asia/Almaty
43.59833 73.76139 Asia/Almaty
50.63278 81.90556 Asia/Almaty
52.93333 70.2 Asia/Almaty
50.41111 80.2275 Asia/Almaty
41.45 69.16667 Asia/Almaty
45.41 79.91861 Asia/Almaty
49.78 72.85056 Asia/Almaty
52.95944 63.12056 Asia/Qyzylorda
44.85278 65.50917 Asia/Qyzylorda
53.16667 63.58333 Asia/Qyzylorda
43.16667 70.46667 Asia/Almaty
49.79889 73.09944 Asia/Almaty
43.88333 77.08333 Asia/Almaty
54.87528 69.16278 Asia/Almaty
52.3 76.95 Asia/Almaty
44.16278 80 Asia/Almaty
49.97889 82.61028 Asia/Almaty
43 71.5 Asia/Almaty
42.87278 73.17861 Asia/Almaty
52.62917 70.41694 Asia/Almaty
52.53694 62.49361 Asia/Qyzylorda
45.61667 63.31667 Asia/Almaty
50.355 83.51333 Asia/Almaty
42.18301 69.88393 Asia/Almaty
53.2775 69.38833 Asia/Almaty
43.52061 68.5094 Asia/Almaty
43.3552 77.45245 Asia/Almaty
49.33139 81.57611 Asia/Almaty
52.03333 76.91667 Asia/Almaty
43.42778 77.02222 Asia/Almaty
51.66667 75.36667 Asia/Almaty
44.16667 66.75 Asia/Qyzylorda
41.25472 67.96917 Asia/Almaty
43.36028 76.85778 Asia/Almaty
42.41667 69.83333 Asia/Almaty
46.84806 74.995 Asia/Almaty
47.97139 80.43917 Asia/Almaty
51.8 68.33333 Asia/Almaty
42.42523 68.79499 Asia/Almaty
50.24861 66.91139 Asia/Almaty
46.8 61.66667 Asia/Qyzylorda
51.1801 71.44598 Asia/Almaty
43.25 76.95 Asia/Almaty
53.2925 69.60111 Asia/Almaty
49.63111 72.85389 Asia/Almaty
52.35444 71.88694 Asia/Almaty
17.96667 102.6 Asia/Vientiane
20.41972 104.04389 Asia/Vientiane
16.56598 104.751 Asia/Vientiane
15.11667 105.78333 Asia/Vientiane
20.7 101.98333 Asia/Vientiane
18.93333 102.45 Asia/Vientiane
18.37778 103.66007 Asia/Vientiane
17.41027 104.83068 Asia/Vientiane
19.88556 102.13472 Asia/Vientiane
20.24528 100.45472 Asia/Vientiane
19.45907 103.17999 Asia/Vientiane
33.84972 35.90417 Asia/Beirut
34.43667 35.84972 Asia/Beirut
33.27333 35.19389 Asia/Beirut
33.56306 35.36889 Asia/Beirut
33.9 35.48333 Asia/Beirut
33.98083 35.61778 Asia/Beirut
34.12111 35.64806 Asia/Beirut
33.4075 35.48278 Asia/Beirut
34.25083 36.01056 Asia/Beirut
33.88894 35.49442 Asia/Beirut
34.00583 36.21806 Asia/Beirut
33.37889 35.48389 Asia/Beirut
13.9957 -61.00614 America/St_Lucia
47.14151 9.52154 Europe/Vaduz
7.0281 79.9014 Asia/Colombo
5.96667 80.41667 Asia/Colombo
6.9779 79.8898 Asia/Colombo
8.7514 80.4971 Asia/Colombo
9.81667 80.16667 Asia/Colombo
8.5711 81.2335 Asia/Colombo
6.68278 80.39917 Asia/Colombo
8.0362 79.8283 Asia/Colombo
9.81667 80.23333 Asia/Colombo
6.8905 79.9015 Asia/Colombo
6.9685 79.8836 Asia/Colombo
6.7132 79.9026 Asia/Colombo
6.97078 80.78286 Asia/Colombo
7.2083 79.8358 Asia/Colombo
6.933 79.9297 Asia/Colombo
6.773 79.8816 Asia/Colombo
5.94851 80.53528 Asia/Colombo
6.848 79.9265 Asia/Colombo
7.4863 80.3623 Asia/Colombo
6.90278 79.90833 Asia/Colombo
6.9269 79.9095 Asia/Colombo
6.9329 79.8848 Asia/Colombo
6.9553 79.922 Asia/Colombo
7.1643 79.8757 Asia/Colombo
7.2955 80.6356 Asia/Colombo
7.048 79.8937 Asia/Colombo
6.5831 79.9593 Asia/Colombo
7.41667 81.81667 Asia/Colombo
9.66845 80.00742 Asia/Colombo
7.0744 79.8919 Asia/Colombo
6.844 80.0024 Asia/Colombo
6.9909 79.883 Asia/Colombo
6.8916 80.5955 Asia/Colombo
6.9012 80.0852 Asia/Colombo
7.1643 80.5696 Asia/Colombo
6.0367 80.217 Asia/Colombo
6.8293 79.863 Asia/Colombo
7.7782 81.6038 Asia/Colombo
7.86 80.65167 Asia/Colombo
6.93194 79.84778 Asia/Colombo
7.57583 79.79528 Asia/Colombo
6.4788 79.9828 Asia/Colombo
6.4254 79.9948 Asia/Colombo
7.7102 81.6924 Asia/Colombo
6.8964 79.9181 Asia/Colombo
6.9895 81.0557 Asia/Colombo
8.35647 80.41726 Asia/Colombo
7.28333 81.66667 Asia/Colombo
6.2355 80.0538 Asia/Colombo
6.06667 -8.12806 Africa/Monrovia
7.57944 -8.53778 Africa/Monrovia
8.42194 -9.74778 Africa/Monrovia
6.30054 -10.7969 Africa/Monrovia
6.53 -10.35167 Africa/Monrovia
4.375 -7.71694 Africa/Monrovia
5.01111 -9.03889 Africa/Monrovia
6.99556 -9.47222 Africa/Monrovia
5.88083 -10.04667 Africa/Monrovia
6.44611 -10.6125 Africa/Monrovia
-30.40001 27.70027 Africa/Maseru
-30.11537 28.68936 Africa/Maseru
-30.15137 27.47691 Africa/Maseru
-29.31667 27.48333 Africa/Maseru
-28.88333 27.9 Africa/Maseru
-29.823 27.23744 Africa/Maseru
-28.87185 28.04501 Africa/Maseru
-28.76659 28.24936 Africa/Maseru
55.6 26.41667 Europe/Vilnius
54.68916 25.2798 Europe/Vilnius
55.5 25.6 Europe/Vilnius
55.25 24.75 Europe/Vilnius
55.98139 22.24722 Europe/Vilnius
55.25222 22.28972 Europe/Vilnius
55.34889 21.48306 Europe/Vilnius
55.93333 23.31667 Europe/Vilnius
55.96667 25.58333 Europe/Vilnius
55.81667 23.53333 Europe/Vilnius
55.91139 21.84417 Europe/Vilnius
55.73333 24.35 Europe/Vilnius
55.9175 21.06861 Europe/Vilnius
56.31667 22.33333 Europe/Vilnius
54.56667 23.35 Europe/Vilnius
55.87583 21.25083 Europe/Vilnius
55.71722 21.1175 Europe/Vilnius
55.28833 23.97472 Europe/Vilnius
54.9 23.9 Europe/Vilnius
55.08333 24.28333 Europe/Vilnius
55.71278 21.40333 Europe/Vilnius
54.01667 23.96667 Europe/Vilnius
54.4 24.05 Europe/Vilnius
54.91525 23.96831 Europe/Vilnius
49.61167 6.13 Europe/Luxembourg
49.49583 5.98056 Europe/Luxembourg
49.48056 6.0875 Europe/Luxembourg
57.54108 25.42751 Europe/Riga
57.38944 21.56056 Europe/Riga
56.53333 21.01667 Europe/Riga
56.96694 23.15528 Europe/Riga
56.86014 24.36544 Europe/Riga
56.946 24.10589 Europe/Riga
56.5 27.31667 Europe/Riga
56.8162 24.61401 Europe/Riga
56.51667 21.01667 Europe/Riga
56.968 23.77038 Europe/Riga
56.65 23.71278 Europe/Riga
56.49903 25.85735 Europe/Riga
55.88333 26.53333 Europe/Riga
57.31188 25.27456 Europe/Riga
32.08361 23.97639 Africa/Tripoli
31.66818 20.25205 Africa/Tripoli
32.76704 22.63669 Africa/Tripoli
32.11667 20.06667 Africa/Tripoli
30.9522 20.12022 Africa/Tripoli
24.20487 23.2857 Africa/Tripoli
32.53414 20.57911 Africa/Tripoli
32.76162 22.24237 Africa/Tripoli
32.48798 20.83174 Africa/Tripoli
24.1989 23.29093 Africa/Tripoli
32.76272 21.75506 Africa/Tripoli
32.19 20.59653 Africa/Tripoli
30.75545 20.22626 Africa/Tripoli
32.9312 12.08199 Africa/Tripoli
32.46738 14.56874 Africa/Tripoli
32.94701 11.86647 Africa/Tripoli
32.06329 12.52859 Africa/Tripoli
29.1614 16.13904 Africa/Tripoli
32.43502 13.6332 Africa/Tripoli
32.87519 13.18746 Africa/Tripoli
32.88167 13.35056 Africa/Tripoli
31.20892 16.58866 Africa/Tripoli
32.75668 12.57159 Africa/Tripoli
32.79335 12.48845 Africa/Tripoli
27.03766 14.42832 Africa/Tripoli
31.86848 10.9812 Africa/Tripoli
25.91552 13.91839 Africa/Tripoli
31.44514 12.98013 Africa/Tripoli
32.37535 15.09254 Africa/Tripoli
32.61667 14 Africa/Tripoli
29.12684 15.94772 Africa/Tripoli
32.75222 12.72778 Africa/Tripoli
24.96472 10.17278 Africa/Tripoli
32.17222 13.02028 Africa/Tripoli
27.54956 14.27138 Africa/Tripoli
31.75662 13.99422 Africa/Tripoli
31.92231 12.25195 Africa/Tripoli
26.59206 12.78049 Africa/Tripoli
32.64861 14.26191 Africa/Tripoli
27.05 14.4 Africa/Tripoli
34.94282 -2.7329 Africa/Casablanca
30.33241 -5.8384 Africa/Casablanca
32.24634 -8.52941 Africa/Casablanca
29.69742 -9.73162 Africa/Casablanca
35.23619 -3.95453 Africa/Casablanca
31.51571 -5.53428 Africa/Casablanca
33.89469 -6.3065 Africa/Casablanca
35.57109 -5.37242 Africa/Casablanca
34.21 -4.01 Africa/Casablanca
30.47028 -8.87695 Africa/Casablanca
34.4073 -2.89732 Africa/Casablanca
34.53822 -4.64143 Africa/Casablanca
28.43799 -11.10321 Africa/Casablanca
35.78058 -5.81365 Africa/Casablanca
34.04845 -4.42795 Africa/Casablanca
34.68664 -6.00272 Africa/Casablanca
34.30652 -6.30618 Africa/Casablanca
34.26478 -5.92598 Africa/Casablanca
34.22149 -5.70775 Africa/Casablanca
29.37975 -10.17299 Africa/Casablanca
32.65236 -8.42769 Africa/Casablanca
33.0007 -7.62035 Africa/Casablanca
33.83186 -4.828 Africa/Casablanca
34.03892 -6.8166 Africa/Casablanca
32.29939 -9.23718 Africa/Casablanca
34.01325 -6.83255 Africa/Casablanca
30.39467 -9.20897 Africa/Casablanca
34.68052 -1.90764 Africa/Casablanca
32.8627 -6.57359 Africa/Casablanca
34.79711 -5.58224 Africa/Casablanca
30.91894 -6.89341 Africa/Casablanca
35.17404 -2.92866 Africa/Casablanca
33.70659 -7.39442 Africa/Casablanca
32.6852 -4.74512 Africa/Casablanca
33.89352 -5.54727 Africa/Casablanca
34.57396 -5.95372 Africa/Casablanca
35.61662 -5.27522 Africa/Casablanca
31.63148 -8.00828 Africa/Casablanca
35.19321 -6.15572 Africa/Casablanca
35.00174 -5.90534 Africa/Casablanca
32.88108 -6.9063 Africa/Casablanca
32.9391 -5.6686 Africa/Casablanca
34.26101 -6.5802 Africa/Casablanca
32.5977 -6.26844 Africa/Casablanca
34.31 -2.16 Africa/Casablanca
35.15 -3.86 Africa/Casablanca
34.22568 -3.35361 Africa/Casablanca
28.98696 -10.05738 Africa/Casablanca
32.50093 -6.69062 Africa/Casablanca
34.03715 -4.9998 Africa/Casablanca
31.5125 -9.77 Africa/Casablanca
33.25492 -8.50602 Africa/Casablanca
33.68786 -5.371 Africa/Casablanca
34.58319 -2.50612 Africa/Casablanca
35.16878 -5.2636 Africa/Casablanca
33.59278 -7.61916 Africa/Casablanca
33.78942 -7.15968 Africa/Casablanca
34.92 -2.32 Africa/Casablanca
32.33725 -6.34984 Africa/Casablanca
33.26553 -7.58754 Africa/Casablanca
33.43443 -5.22126 Africa/Casablanca
33.28952 -8.3425 Africa/Casablanca
35.46502 -6.0348 Africa/Casablanca
33.82404 -6.06627 Africa/Casablanca
34.95368 -2.10027 Africa/Casablanca
30.42018 -9.59815 Africa/Casablanca
33.84984 -7.04037 Africa/Casablanca
32.90046 -6.77464 Africa/Casablanca
43.74965 7.43701 Europe/Monaco
43.73333 7.41667 Europe/Monaco
48.16806 27.305 Europe/Chisinau
47.20417 27.79583 Europe/Chisinau
46.84028 29.64333 Europe/Chisinau
47.14139 28.61028 Europe/Chisinau
47.65639 28.04722 Europe/Chisinau
48.15583 28.2975 Europe/Chisinau
46.72806 29.70778 Europe/Chisinau
47.76639 29.00111 Europe/Chisinau
47.38306 28.82306 Europe/Chisinau
46.64417 29.41389 Europe/Chisinau
47.89333 28.30139 Europe/Chisinau
47.26306 29.16083 Europe/Chisinau
48.035 27.81611 Europe/Chisinau
46.30028 28.65722 Europe/Chisinau
47.00556 28.8575 Europe/Chisinau
46.055 28.83028 Europe/Chisinau
45.9075 28.19444 Europe/Chisinau
46.83056 29.47111 Europe/Chisinau
47.76167 27.92889 Europe/Chisinau
42.44111 19.26361 Europe/Podgorica
42.7731 18.94446 Europe/Podgorica
42.39063 18.91417 Europe/Podgorica
18.06667 -63.08333 America/Guadeloupe
-18.16667 49.38333 Indian/Antananarivo
-22.81667 47.28333 Indian/Antananarivo
-17.35 49.03333 Indian/Antananarivo
-17.46667 49.2 Indian/Antananarivo
-23.35 47.6 Indian/Antananarivo
-18.76667 46.03333 Indian/Antananarivo
-25.3 45.48333 Indian/Antananarivo
-16.78333 47.65 Indian/Antananarivo
-23.35 43.66667 Indian/Antananarivo
-25.03185 46.99874 Indian/Antananarivo
-19.16667 46.73333 Indian/Antananarivo
-16.91667 49.58333 Indian/Antananarivo
-14.26667 50.16667 Indian/Antananarivo
-22.9 44.53333 Indian/Antananarivo
-20.58333 48.53333 Indian/Antananarivo
-20.28333 44.28333 Indian/Antananarivo
-18.93333 48.2 Indian/Antananarivo
-19.51667 45.46667 Indian/Antananarivo
-16.1 46.63333 Indian/Antananarivo
-20.05 48.11667 Indian/Antananarivo
-15.43333 49.73333 Indian/Antananarivo
-18.91667 47.8 Indian/Antananarivo
-21.21667 48.33333 Indian/Antananarivo
-16.16667 49.76667 Indian/Antananarivo
-22.13333 48.01667 Indian/Antananarivo
-19.9 48.8 Indian/Antananarivo
-15.71667 46.31667 Indian/Antananarivo
-21.88333 47.43333 Indian/Antananarivo
-21.15 46.58333 Indian/Antananarivo
-22.4 46.11667 Indian/Antananarivo
-21.3 47.63333 Indian/Antananarivo
-21.43333 47.08333 Indian/Antananarivo
-18.43333 46.56667 Indian/Antananarivo
-17.36667 49.41667 Indian/Antananarivo
-19.4 46.95 Indian/Antananarivo
-22.81667 47.83333 Indian/Antananarivo
-20.23333 47.38333 Indian/Antananarivo
-23.71667 44.38333 Indian/Antananarivo
-19.83333 46.85 Indian/Antananarivo
-21.66667 45.16667 Indian/Antananarivo
-19.7 44.55 Indian/Antananarivo
-25.16667 45.05 Indian/Antananarivo
-14.55 48.73333 Indian/Antananarivo
-19.01667 47.18333 Indian/Antananarivo
-14.87959 47.98751 Indian/Antananarivo
-12.2787 49.29171 Indian/Antananarivo
-19.65 47.31667 Indian/Antananarivo
-18.91368 47.53613 Indian/Antananarivo
-14.90033 50.27876 Indian/Antananarivo
-18.31667 47.11667 Indian/Antananarivo
-22.28333 44.51667 Indian/Antananarivo
-18.4 47.86667 Indian/Antananarivo
-13.4 48.26667 Indian/Antananarivo
-17.01667 48.58333 Indian/Antananarivo
-14.65 49.65 Indian/Antananarivo
-17.58333 48.21667 Indian/Antananarivo
-24.7 44.75 Indian/Antananarivo
-25.16667 46.08333 Indian/Antananarivo
-20.51667 47.25 Indian/Antananarivo
-25.03333 46.38333 Indian/Antananarivo
-17.83333 48.41667 Indian/Antananarivo
-19.38333 47.41667 Indian/Antananarivo
-20.55 46.8 Indian/Antananarivo
-16.46667 46.71667 Indian/Antananarivo
-13.68333 48.45 Indian/Antananarivo
-21.83333 46.93333 Indian/Antananarivo
7.08971 171.38027 Pacific/Majuro
7.08931 171.3805 Pacific/Majuro
41.98028 21.06417 Europe/Skopje
41.88278 22.50917 Europe/Skopje
41.71556 21.77556 Europe/Skopje
42.01056 20.97139 Europe/Skopje
42.07444 21.0525 Europe/Skopje
41.91583 21.53056 Europe/Skopje
41.4375 22.64333 Europe/Skopje
41.17806 20.67611 Europe/Skopje
41.74583 22.19583 Europe/Skopje
42 21.43333 Europe/Skopje
42 21.32778 Europe/Skopje
41.08889 21.01222 Europe/Skopje
41.63833 22.46472 Europe/Skopje
41.34639 21.55444 Europe/Skopje
41.11722 20.80194 Europe/Skopje
41.87861 20.88389 Europe/Skopje
41.48389 22.08917 Europe/Skopje
42.15639 21.58528 Europe/Skopje
42.13222 21.71444 Europe/Skopje
42.20194 22.33194 Europe/Skopje
41.91639 22.41278 Europe/Skopje
41.51444 20.96333 Europe/Skopje
41.43306 22.01194 Europe/Skopje
41.94444 20.92667 Europe/Skopje
41.79722 20.90833 Europe/Skopje
41.13917 22.5025 Europe/Skopje
41.96722 22.76944 Europe/Skopje
41.525 20.5275 Europe/Skopje
41.96722 20.98083 Europe/Skopje
41.92361 20.91361 Europe/Skopje
41.03111 21.34028 Europe/Skopje
41.99611 21.58222 Europe/Skopje
12.3536 -4.7761 Africa/Bamako
16.77348 -3.00742 Africa/Bamako
11.3135 -5.6697 Africa/Bamako
13.4317 -6.2157 Africa/Bamako
13.30335 -4.89562 Africa/Bamako
14.49302 -4.1942 Africa/Bamako
13.7021 -6.0659 Africa/Bamako
12.39173 -5.46421 Africa/Bamako
12.86273 -7.55985 Africa/Bamako
13.5728 -8.0339 Africa/Bamako
14.4586 -11.4352 Africa/Bamako
14.4417 -11.4341 Africa/Bamako
12.74409 -8.07257 Africa/Bamako
11.93333 -8.41667 Africa/Bamako
16.27167 -0.04472 Africa/Bamako
13.90608 -4.55332 Africa/Bamako
11.41699 -7.47899 Africa/Bamako
13.55 -7.45 Africa/Bamako
12.65 -8 Africa/Bamako
13.8065 -10.8321 Africa/Bamako
20.46667 94.88333 Asia/Rangoon
17.03333 95.65 Asia/Rangoon
20.43333 96.15 Asia/Rangoon
16.6 95.18333 Asia/Rangoon
16.71667 95.93333 Asia/Rangoon
18.93333 96.43333 Asia/Rangoon
16.76194 96.52778 Asia/Rangoon
19.31667 95.18333 Asia/Rangoon
16.92056 97.37139 Asia/Rangoon
17.65 95.8 Asia/Rangoon
17.3 96.58333 Asia/Rangoon
17.21194 96.30333 Asia/Rangoon
14.08333 98.2 Asia/Rangoon
20.78333 97.03333 Asia/Rangoon
20.01667 95.55 Asia/Rangoon
16.77222 96.23778 Asia/Rangoon
20.15 92.9 Asia/Rangoon
22.56667 95.7 Asia/Rangoon
21.8787 95.97965 Asia/Rangoon
16.80528 96.15611 Asia/Rangoon
18.48333 96.43333 Asia/Rangoon
19.73333 96.21667 Asia/Rangoon
18.81667 95.21667 Asia/Rangoon
16.28333 95.68333 Asia/Rangoon
17.33667 96.47972 Asia/Rangoon
18.48333 95.5 Asia/Rangoon
21.33333 95.1 Asia/Rangoon
17.95 96.73333 Asia/Rangoon
25.38333 97.4 Asia/Rangoon
21.46667 95.38333 Asia/Rangoon
18.28333 95.31667 Asia/Rangoon
16.25778 97.71639 Asia/Rangoon
16.38333 95.26667 Asia/Rangoon
16.49139 97.62556 Asia/Rangoon
22.11667 95.13333 Asia/Rangoon
22.91667 96.5 Asia/Rangoon
20.18333 94.88333 Asia/Rangoon
12.43333 98.6 Asia/Rangoon
20.86667 95.86667 Asia/Rangoon
22.03333 96.46667 Asia/Rangoon
23.63333 94.41667 Asia/Rangoon
16.73333 95.65 Asia/Rangoon
16.53139 97.61111 Asia/Rangoon
21.97473 96.08359 Asia/Rangoon
20.15 94.91667 Asia/Rangoon
19.67417 97.20944 Asia/Rangoon
17.78333 95.75 Asia/Rangoon
22.93333 97.75 Asia/Rangoon
21.6 96.13333 Asia/Rangoon
17.3 97.01667 Asia/Rangoon
16.43333 95.73333 Asia/Rangoon
16.08 97.5675 Asia/Rangoon
16.90556 96.56306 Asia/Rangoon
16.70528 96.00167 Asia/Rangoon
17.63333 95.46667 Asia/Rangoon
20.88333 94.81667 Asia/Rangoon
16.28333 95.4 Asia/Rangoon
24.26667 97.23333 Asia/Rangoon
16.78333 94.73333 Asia/Rangoon
19.36667 95.21667 Asia/Rangoon
19.745 96.12972 Asia/Rangoon
49.98111 92.06667 Asia/Ulaanbaatar
48.96833 89.9625 Asia/Ulaanbaatar
48.00556 91.64194 Asia/Ulaanbaatar
47.90771 106.88324 Asia/Ulaanbaatar
50.23139 106.20778 Asia/Ulaanbaatar
49.63417 100.1625 Asia/Ulaanbaatar
45.7625 106.27083 Asia/Ulaanbaatar
44.67024 102.17491 Asia/Ulaanbaatar
49.03333 104.08333 Asia/Ulaanbaatar
48.85229 106.45786 Asia/Ulaanbaatar
49.48667 105.92278 Asia/Ulaanbaatar
43.57083 104.425 Asia/Ulaanbaatar
44.88239 110.11631 Asia/Ulaanbaatar
48.8125 103.53472 Asia/Ulaanbaatar
46.71667 100.11667 Asia/Ulaanbaatar
46.68056 113.27917 Asia/Ulaanbaatar
47.70693 106.95276 Asia/Ulaanbaatar
22.20056 113.54611 Asia/Macau
15.21217 145.75372 Pacific/Saipan
15.21233 145.7545 Pacific/Saipan
14.66792 -61.04558 America/Martinique
14.78392 -60.99274 America/Martinique
14.75 -60.96667 America/Martinique
14.67751 -60.94228 America/Martinique
14.608 -61.00933 America/Martinique
14.61557 -60.90308 America/Martinique
14.73758 -60.96286 America/Martinique
14.60892 -61.07334 America/Martinique
14.57529 -60.97453 America/Martinique
18.08581 -15.9785 Africa/Nouakchott
20.93098 -17.03465 Africa/Nouakchott
16.70555 -62.21292 America/Montserrat
16.79183 -62.21058 America/Montserrat
35.87611 14.535 Europe/Malta
35.89972 14.51472 Europe/Malta
35.87601 14.472 Europe/Malta
35.90917 14.42556 Europe/Malta
35.89722 14.46111 Europe/Malta
-20.29806 57.47833 Indian/Mauritius
-20.05472 57.54528 Indian/Mauritius
-20.2175 57.52083 Indian/Mauritius
-20.26444 57.47194 Indian/Mauritius
-20.16194 57.49889 Indian/Mauritius
-20.40806 57.7 Indian/Mauritius
-20.035 57.64306 Indian/Mauritius
-20.31472 57.52028 Indian/Mauritius
-20.18972 57.71444 Indian/Mauritius
-20.25417 57.74528 Indian/Mauritius
-20.135 57.52111 Indian/Mauritius
4.1748 73.50888 Indian/Maldives
-9.93333 33.93333 Africa/Blantyre
-15.38596 35.3188 Africa/Blantyre
-13.7804 34.4587 Africa/Blantyre
-11.01863 33.85746 Africa/Blantyre
-16.91995 35.26199 Africa/Blantyre
-12.92744 34.29614 Africa/Blantyre
-11.9 33.6 Africa/Blantyre
-16.03163 35.5 Africa/Blantyre
-13.79841 32.88019 Africa/Blantyre
-14.47815 35.26448 Africa/Blantyre
-15.05064 35.20353 Africa/Blantyre
-13.96692 33.78725 Africa/Blantyre
-13.03333 33.48333 Africa/Blantyre
-14.3779 34.33322 Africa/Blantyre
-15.78499 35.00854 Africa/Blantyre
-14.97928 34.95575 Africa/Blantyre
24.85 -98.15 America/Monterrey
19.79778 -99.10167 America/Mexico_City
17.65 -99.5 America/Mexico_City
20.65 -98.6 America/Mexico_City
19.93333 -97.96667 America/Mexico_City
18.65 -99.2 America/Mexico_City
18.88188 -99.06715 America/Mexico_City
18.68333 -99.25 America/Mexico_City
19.40222 -99.5325 America/Mexico_City
18.78333 -99.23333 America/Mexico_City
19.26222 -99.1075 America/Mexico_City
20.3 -97.96667 America/Mexico_City
19.41667 -97 America/Mexico_City
19.26667 -98.93333 America/Mexico_City
19.37333 -99.225 America/Mexico_City
17.98333 -92.91667 America/Mexico_City
19.47861 -99.09583 America/Mexico_City
19.41528 -99.56167 America/Mexico_City
19.2 -96.13333 America/Mexico_City
16.35 -92.55 America/Mexico_City
25.66667 -97.83333 America/Monterrey
20.68812 -88.19936 America/Merida
20.88333 -89.75 America/Merida
16.75 -93.11667 America/Mexico_City
20.95 -97.4 America/Mexico_City
19.64722 -99.17083 America/Mexico_City
19.68472 -99.12917 America/Mexico_City
20.08333 -98.36667 America/Mexico_City
20.05 -99.35 America/Mexico_City
19.28833 -99.66722 America/Mexico_City
19.03333 -98.26667 America/Mexico_City
18.63333 -99.16667 America/Mexico_City
17.55 -98.55 America/Mexico_City
19.96667 -97.21667 America/Mexico_City
19.28333 -99.16667 America/Mexico_City
19.52694 -99.22167 America/Mexico_City
19.28167 -99.00333 America/Mexico_City
21.15 -88.15 America/Merida
19.83333 -98.98333 America/Mexico_City
17.56732 -99.39799 America/Mexico_City
20.4 -89.53333 America/Merida
19.17972 -99.47056 America/Mexico_City
20.2 -99.26667 America/Mexico_City
19.81667 -97.35 America/Mexico_City
19.51667 -98.88333 America/Mexico_City
19.90806 -99.1475 America/Mexico_City
20.51667 -99.86667 America/Mexico_City
19.64694 -99.24667 America/Mexico_City
18.98333 -99.1 America/Mexico_City
19.71389 -99.22444 America/Mexico_City
19.90528 -99.34389 America/Mexico_City
18.96667 -97.9 America/Mexico_City
19.06747 -97.96595 America/Mexico_City
19.74556 -99.18333 America/Mexico_City
17.48333 -91.43333 America/Mexico_City
19.10167 -99.59306 America/Mexico_City
18.96667 -99.6 America/Mexico_City
18.85 -99.23333 America/Mexico_City
21.06667 -97.63333 America/Mexico_City
18.35 -99.85 America/Mexico_City
20.2 -89.28333 America/Merida
18.45 -97.38333 America/Mexico_City
18.88333 -97.73333 America/Mexico_City
17.55 -92.95 America/Mexico_City
18.55 -99.6 America/Mexico_City
14.9 -92.28333 America/Mexico_City
21.35 -98.23333 America/Mexico_City
22.21667 -97.85 America/Monterrey
21.26667 -98.78333 America/Mexico_City
16.33333 -95.23333 America/Mexico_City
18.46667 -95.3 America/Mexico_City
16.31667 -98.01667 America/Mexico_City
20.03333 -98.35 America/Mexico_City
19.25 -98.4 America/Mexico_City
19.97861 -99.04722 America/Mexico_City
17.01667 -96.73333 America/Mexico_City
19.73333 -98.96667 America/Mexico_City
19.31667 -98.18333 America/Mexico_City
19.13333 -97.65 America/Mexico_City
19.51667 -98.91667 America/Mexico_City
19.66583 -99.09639 America/Mexico_City
19.35611 -99.6625 America/Mexico_City
19.28194 -99.73861 America/Mexico_City
19.45 -98.86667 America/Mexico_City
19.26167 -99.53528 America/Mexico_City
19.28333 -98.43333 America/Mexico_City
19.25 -98.18333 America/Mexico_City
19.68333 -98.86667 America/Mexico_City
20.38333 -100 America/Mexico_City
18.1 -96.11667 America/Mexico_City
19.33333 -98.85 America/Mexico_City
19.1 -98.25 America/Mexico_City
16.75 -92.63333 America/Mexico_City
18.45 -95.21667 America/Mexico_City
16.16667 -95.2 America/Mexico_City
21.93333 -99.98333 America/Mexico_City
25.98333 -98.1 America/Monterrey
18.83333 -97.15 America/Mexico_City
26.08333 -98.28333 America/Matamoros
15.85 -97.06667 America/Mexico_City
18.61472 -99.31806 America/Mexico_City
19.05 -98.2 America/Mexico_City
20.25 -99.2 America/Mexico_City
21.28333 -89.66667 America/Merida
20.53315 -97.45946 America/Mexico_City
19.41667 -99.2 America/Mexico_City
20.6274 -87.07987 America/Cancun
15.70105 -93.22998 America/Mexico_City
20.13333 -88.91667 America/Merida
19.56667 -97.23333 America/Mexico_City
18.4 -93.23333 America/Mexico_City
20.45 -97.31667 America/Mexico_City
22.05 -98.16667 America/Mexico_City
18.9 -97.61667 America/Mexico_City
17.51667 -91.96667 America/Mexico_City
20.11697 -98.73329 America/Mexico_City
19.05 -98.8 America/Mexico_City
20.3 -89.41667 America/Merida
18.85 -97.1 America/Mexico_City
16.68333 -98.41667 America/Mexico_City
16.76667 -93.36667 America/Mexico_City
19.27194 -99.45889 America/Mexico_City
17.06667 -92.25 America/Mexico_City
17.05 -96.71667 America/Mexico_City
27.47629 -99.51639 America/Matamoros
18.81667 -97.16667 America/Mexico_City
19.62194 -99.31306 America/Mexico_City
19.47851 -99.23963 America/Mexico_City
21.35 -97.68333 America/Mexico_City
18.06667 -94.4 America/Mexico_City
21.1 -89.28333 America/Merida
15.36667 -92.23333 America/Mexico_City
25.2 -99.81667 America/Monterrey
19.06667 -98.23333 America/Mexico_City
20.23333 -99.21667 America/Mexico_City
19.93333 -96.83333 America/Mexico_City
22.33333 -97.86667 America/Monterrey
17.98333 -94.51667 America/Mexico_City
19.19194 -99.02361 America/Mexico_City
16.33333 -96.6 America/Mexico_City
19.25361 -99.60778 America/Mexico_City
20.96667 -89.61667 America/Merida
16.88333 -95.03333 America/Mexico_City
25.88333 -97.5 America/Matamoros
20.06667 -97.05 America/Mexico_City
15.43333 -92.9 America/Mexico_City
19.28333 -99.23333 America/Mexico_City
17.76667 -92.6 America/Mexico_City
19.76667 -98.98333 America/Mexico_City
19.35 -98.96667 America/Mexico_City
24.85778 -99.56778 America/Monterrey
19.285 -99.51556 America/Mexico_City
18.61667 -95.51667 America/Mexico_City
16.31667 -91.98333 America/Mexico_City
17.93333 -94.08333 America/Mexico_City
20.11667 -96.96667 America/Mexico_City
18.6 -96.15 America/Mexico_City
20.93333 -89.56667 America/Merida
16.43333 -95.01667 America/Mexico_City
18.61667 -99.18333 America/Mexico_City
18.86667 -99.18333 America/Mexico_City
18.13333 -93.08333 America/Mexico_City
19.53333 -96.91667 America/Mexico_City
18.6 -98.46667 America/Mexico_City
16.57855 -95.10223 America/Mexico_City
18.83333 -99.68333 America/Mexico_City
19.31667 -98.88333 America/Mexico_City
19.35111 -99.05194 America/Mexico_City
18.85 -97.06667 America/Mexico_City
19.39667 -99.08472 America/Mexico_City
20.48333 -99.23333 America/Mexico_City
18.35 -99.53333 America/Mexico_City
21.01667 -89.86667 America/Merida
15.15 -92.46667 America/Mexico_City
18.3 -99.35 America/Mexico_City
17.85 -93.38333 America/Mexico_City
21.13333 -98.41667 America/Mexico_City
19.15 -98.4 America/Mexico_City
20.18333 -98.05 America/Mexico_City
19.15 -96.95 America/Mexico_City
17.8 -97.76667 America/Mexico_City
18.58333 -92.65 America/Merida
18.9 -97 America/Mexico_City
19.58333 -88.05 America/Cancun
18.60891 -90.74544 America/Merida
18.83333 -99.18333 America/Mexico_City
17.75 -91.76667 America/Mexico_City
22.73333 -98.96667 America/Monterrey
19.60111 -99.0525 America/Mexico_City
18.06667 -93.16667 America/Mexico_City
18.91667 -99.25 America/Mexico_City
19.08333 -98.26667 America/Mexico_City
18.8 -98.95 America/Mexico_City
19.67111 -99.18278 America/Mexico_City
20.03333 -98.3 America/Mexico_City
19.35583 -99.30111 America/Mexico_City
20.50833 -86.94583 America/Cancun
19.775 -99.20722 America/Mexico_City
19.32889 -99.16028 America/Mexico_City
18 -94.61667 America/Mexico_City
18.36667 -95.8 America/Mexico_City
18.88333 -96.93333 America/Mexico_City
19.33333 -98.16667 America/Mexico_City
16.25 -92.13333 America/Mexico_City
18.26667 -93.21667 America/Mexico_City
20.48333 -97.45 America/Mexico_City
18.15 -94.41667 America/Mexico_City
19.45 -96.96667 America/Mexico_City
19.63167 -99.11028 America/Mexico_City
23.73333 -99.13333 America/Monterrey
21.98333 -99.01667 America/Mexico_City
18.98333 -97.45 America/Mexico_City
19.76667 -98.58333 America/Mexico_City
19.41361 -99.03306 America/Mexico_City
26.35 -99.1 America/Monterrey
18.8 -97.18333 America/Mexico_City
22.26667 -97.83333 America/Monterrey
21.95 -100 America/Mexico_City
19.42847 -99.12766 America/Mexico_City
18.63333 -91.83333 America/Merida
16.73333 -93.71667 America/Mexico_City
19.06406 -98.30352 America/Mexico_City
17.55 -99.5 America/Mexico_City
17.6 -99.16667 America/Mexico_City
19.83333 -98.03333 America/Mexico_City
19.55 -98.9 America/Mexico_City
19.41667 -98.9 America/Mexico_City
20.66667 -88.56667 America/Merida
16.7 -93 America/Mexico_City
18.5 -88.3 America/Cancun
19.35 -90.71667 America/Merida
19.26667 -98.9 America/Mexico_City
21.2 -97.73333 America/Mexico_City
18.41667 -95.11667 America/Mexico_City
18.36667 -95.75 America/Mexico_City
22 -99.66667 America/Mexico_City
17.98333 -93.36667 America/Mexico_City
19.19139 -99.46694 America/Mexico_City
21.17429 -86.84656 America/Cancun
19.85 -90.53333 America/Merida
19.58333 -98.58333 America/Mexico_City
25.58333 -99.98333 America/Monterrey
16.8 -93.26667 America/Mexico_City
19.58333 -96.93333 America/Mexico_City
19.48889 -99.18361 America/Mexico_City
18.5 -98.76667 America/Mexico_City
18.90815 -98.43613 America/Mexico_City
19.79889 -99.87444 America/Mexico_City
19.55833 -99.26139 America/Mexico_City
19.41667 -98.15 America/Mexico_City
19.71667 -98.41667 America/Mexico_City
19.03333 -98.05 America/Mexico_City
19.11667 -98.76667 America/Mexico_City
18.76667 -95.76667 America/Mexico_City
19.76667 -97.23333 America/Mexico_City
18.35 -97.31667 America/Mexico_City
22.4 -97.91667 America/Monterrey
18.15 -94.4 America/Mexico_City
20.91667 -97.68333 America/Mexico_City
18.36667 -97.25 America/Mexico_City
18.13333 -94.13333 America/Mexico_City
20.26667 -98.93333 America/Mexico_City
17.95 -94.91667 America/Mexico_City
18.98333 -97.78333 America/Mexico_City
18.2 -98.05 America/Mexico_City
16.86336 -99.8901 America/Mexico_City
19.1 -97.95 America/Mexico_City
17.03083 -96.6975 America/Mexico_City
25.8425 -99.61611 America/Monterrey
17.7875 -92.89583 America/Mexico_City
19.31528 -97.925 America/Mexico_City
19.21528 -98.23889 America/Mexico_City
19.31389 -98.24167 America/Mexico_City
19.1625 -98.20417 America/Mexico_City
19.11944 -98.17083 America/Mexico_City
16.08333 -93.75 America/Mexico_City
16.23333 -93.9 America/Mexico_City
19.32722 -99.60667 America/Mexico_City
19.305 -99.5925 America/Mexico_City
19.385 -99.165 America/Mexico_City
19.43 -99.09917 America/Mexico_City
19.4225 -99.20278 America/Mexico_City
19.41722 -99.15694 America/Mexico_City
19.36028 -99.35139 America/Mexico_City
19.69972 -99.1475 America/Mexico_City
19.68694 -99.25 America/Mexico_City
19.65889 -99.12694 America/Mexico_City
25.66667 -108.64167 America/Mazatlan
19.2366 -104.56512 America/Mexico_City
32.28694 -115.105 America/Tijuana
17.63333 -101.55 America/Mexico_City
20.63333 -103.06667 America/Mexico_City
19.61667 -103.43333 America/Mexico_City
20.71667 -103.4 America/Mexico_City
19.98333 -102.26667 America/Mexico_City
20.23333 -103.58333 America/Mexico_City
22.77559 -102.57218 America/Mexico_City
19.83333 -101.71667 America/Mexico_City
20.2 -101.15 America/Mexico_City
20.33333 -102.3 America/Mexico_City
20.51667 -100.98333 America/Mexico_City
26.93333 -101.45 America/Monterrey
25.81667 -100.58333 America/Monterrey
19.25 -103.73333 America/Mexico_City
20.38333 -101.2 America/Mexico_City
19.18333 -100.13333 America/Mexico_City
19.41667 -102.06667 America/Mexico_City
20.15 -101.18333 America/Mexico_City
19.55 -103.4 America/Mexico_City
25.55 -103.43333 America/Monterrey
20.61667 -103.23333 America/Mexico_City
20.65 -103.31667 America/Mexico_City
20.46667 -103.45 America/Mexico_City
32.53333 -117.01667 America/Tijuana
20.78333 -103.48333 America/Mexico_City
20.88333 -103.83333 America/Mexico_City
21.5 -104.9 America/Mazatlan
20.81667 -102.73333 America/Mexico_City
19.18333 -102.85 America/Mexico_City
21.43333 -102.58333 America/Mexico_City
18.9 -100.15 America/Mexico_City
18.91667 -103.88333 America/Mexico_City
32.56667 -116.63333 America/Tijuana
19.9 -102.13333 America/Mexico_City
19.63333 -103.25 America/Mexico_City
20.66667 -103.7 America/Mexico_City
19.23333 -101.46667 America/Mexico_City
23.63333 -103.65 America/Mexico_City
22.2 -100.95 America/Mexico_City
20.93333 -101.43333 America/Mexico_City
19.86667 -103.61667 America/Mexico_City
21.81667 -105.21667 America/Mazatlan
25.41667 -100.15 America/Monterrey
20.73333 -100.45 America/Mexico_City
28.38333 -113.35 America/Santa_Isabel
25.67325 -100.45813 America/Monterrey
20.51667 -103.41667 America/Mexico_City
25.43333 -103.21667 America/Monterrey
25.75 -100.3 America/Monterrey
21.01667 -102.35 America/Mexico_City
24.8325 -105.33639 America/Monterrey
20.91667 -100.75 America/Mexico_City
32.46306 -114.77667 America/Hermosillo
22.15 -100.98333 America/Mexico_City
21.3 -100.51667 America/Mexico_City
21.3 -100.51667 America/Mexico_City
22.89088 -109.91238 America/Mazatlan
21.25 -102.3 America/Mexico_City
21 -100.38333 America/Mexico_City
23.05888 -109.69771 America/Mazatlan
21.01667 -101.85 America/Mexico_City
31 -114.86667 America/Tijuana
21.48333 -101.21667 America/Mexico_City
27.08333 -101.53333 America/Monterrey
20.53333 -103.46667 America/Mexico_City
20.21667 -100.88333 America/Mexico_City
25.41667 -101 America/Monterrey
20.56667 -101.2 America/Mexico_City
20.06667 -102.71667 America/Mexico_City
26.5 -100.16667 America/Monterrey
27.85 -101.11667 America/Monterrey
32.33333 -117.03333 America/Tijuana
20.86667 -101.51667 America/Mexico_City
23.83333 -103.03333 America/Mexico_City
22.23333 -102.3 America/Mexico_City
25.55 -100.96667 America/Monterrey
20.6 -100.38333 America/Mexico_City
20.08333 -101.5 America/Mexico_City
20.62041 -105.23066 America/Mexico_City
31.31667 -113.53333 America/Hermosillo
28.7 -100.51667 America/Matamoros
17.51667 -101.26667 America/Mexico_City
20.43333 -101.73333 America/Mexico_City
19.51667 -101.6 America/Mexico_City
25.41667 -102.18333 America/Monterrey
19.65 -102.06667 America/Mexico_City
27.91667 -101.41667 America/Monterrey
29.56667 -104.41667 America/Ojinaga
20.35 -102.76667 America/Mexico_City
20.75 -103.43333 America/Mexico_City
30.41667 -107.91667 America/Chihuahua
27.95 -101.21667 America/Monterrey
19.01667 -102.1 America/Mexico_City
21.36667 -102.85 America/Mexico_City
24.76722 -107.69444 America/Mazatlan
28.41667 -100.76667 America/Monterrey
20.13333 -101.2 America/Mexico_City
19.70078 -101.18443 America/Mexico_City
25.66667 -100.31667 America/Monterrey
26.9 -101.41667 America/Monterrey
32.65194 -115.46833 America/Tijuana
28.28333 -105.48333 America/Chihuahua
27.88333 -101.51667 America/Monterrey
23.21667 -106.41667 America/Mazatlan
23.65 -100.65 America/Mexico_City
25.53333 -103.25 America/Monterrey
21 -101.28333 America/Mexico_City
19.9 -100.45 America/Mexico_City
19.05011 -104.31879 America/Mexico_City
31.71667 -116.56667 America/Tijuana
30.63333 -110.95 America/Hermosillo
29.2 -108.11667 America/Chihuahua
19.73333 -100.26667 America/Mexico_City
25.76667 -108.96667 America/Mazatlan
22.26667 -101.96667 America/Mexico_City
21.11667 -101.66667 America/Mexico_City
20.56667 -103.31667 America/Mexico_City
20.35 -102 America/Mexico_City
24.16667 -110.3 America/Mazatlan
17.98333 -102.23333 America/Mexico_City
21.35 -101.91667 America/Mexico_City
20.28333 -102.56667 America/Mexico_City
20.65 -101 America/Mexico_City
20.3 -103.43333 America/Mexico_City
19.98333 -102.71667 America/Mexico_City
28.33333 -105.4 America/Chihuahua
22.65 -103 America/Mexico_City
20.36667 -101.06667 America/Mexico_City
20.3 -102.71667 America/Mexico_City
21.2 -102.46667 America/Mexico_City
21.03333 -104.36667 America/Mazatlan
20.7 -105.2 America/Mexico_City
20.68333 -101.35 America/Mexico_City
18.58333 -100.88333 America/Mexico_City
26.83333 -109.63333 America/Hermosillo
26.93333 -105.66667 America/Chihuahua
19.4 -100.36667 America/Mexico_City
31.30862 -110.94217 America/Hermosillo
30.71667 -112.15 America/Hermosillo
29.06667 -110.96667 America/Hermosillo
27.93333 -110.9 America/Hermosillo
25.56667 -108.45 America/Mazatlan
21.01667 -101.25 America/Mexico_City
25.46667 -108.1 America/Mazatlan
25.68333 -100.25 America/Monterrey
22.75 -102.51667 America/Mexico_City
20.66667 -103.33333 America/Mexico_City
25.56667 -103.5 America/Monterrey
25.75 -108.83333 America/Mazatlan
25.81667 -100.33333 America/Monterrey
25.66667 -100.4 America/Monterrey
23.16667 -102.88333 America/Mexico_City
22.85 -105.8 America/Mazatlan
31.86667 -116.61667 America/Tijuana
21.51667 -102.23333 America/Mexico_City
27.96667 -110.81667 America/Hermosillo
23.38333 -105.38333 America/Monterrey
20.33333 -104.5 America/Mexico_City
20.53333 -100.45 America/Mexico_City
19.8 -104.21667 America/Mexico_City
24.03333 -104.66667 America/Monterrey
24.79944 -107.38972 America/Mazatlan
28.41667 -106.86667 America/Chihuahua
24.5875 -107.3875 America/Mazatlan
20.48333 -100.93333 America/Mexico_City
21.23333 -104.91667 America/Mazatlan
20.71667 -100.76667 America/Mexico_City
19.23333 -103.71667 America/Mexico_City
27.48333 -109.93333 America/Hermosillo
25.53333 -103.53333 America/Monterrey
31.73333 -106.48333 America/Ojinaga
19.68333 -100.56667 America/Mexico_City
19.68333 -103.48333 America/Mexico_City
28.19013 -105.47012 America/Chihuahua
25.03694 -111.66444 America/Mazatlan
27.66667 -105.16667 America/Chihuahua
27.23333 -100.15 America/Matamoros
18.31667 -100.65 America/Mexico_City
29.32322 -100.95217 America/Matamoros
28.63333 -106.08333 America/Chihuahua
20.3 -103.2 America/Mexico_City
20.51667 -100.81667 America/Mexico_City
26.78333 -101.41667 America/Monterrey
30.95 -110.3 America/Hermosillo
24.65361 -107.54111 America/Mazatlan
21.85 -102.71667 America/Mexico_City
22.95 -102.7 America/Mexico_City
19.76667 -104.36667 America/Mexico_City
17.2 -100.43333 America/Mexico_City
20.55 -102.51667 America/Mexico_City
18.93333 -103.96667 America/Mexico_City
18.28333 -100.26667 America/Mexico_City
20.7 -102.35 America/Mexico_City
25.76667 -100.2 America/Monterrey
19.08333 -102.35 America/Mexico_City
20.56667 -100.7 America/Mexico_City
20.45 -100.61667 America/Mexico_City
20.55 -104.03333 America/Mexico_City
28.33333 -100.85 America/Monterrey
25.28333 -100.01667 America/Monterrey
21.88333 -102.3 America/Mexico_City
31.3 -109.56667 America/Hermosillo
22.49639 -105.35944 America/Mazatlan
20.03333 -100.73333 America/Mexico_City
20.45 -101.51667 America/Mexico_City
21.15611 -100.93083 America/Mexico_City
18.01667 -102.21667 America/Mexico_City
17.95833 -102.2 America/Mexico_City
19.38611 -99.16204 America/Mexico_City
19.49156 -99.12475 America/Mexico_City
19.38119 -99.13685 America/Mexico_City
6.32649 99.8432 Asia/Kuala_Lumpur
3.936 102.3626 Asia/Kuala_Lumpur
3.7899 101.857 Asia/Kuala_Lumpur
1.8548 102.9325 Asia/Kuala_Lumpur
1.8681 103.1124 Asia/Kuala_Lumpur
1.51 103.5141 Asia/Kuala_Lumpur
1.4866 103.3896 Asia/Kuala_Lumpur
1.4726 103.878 Asia/Kuala_Lumpur
1.7381 103.8999 Asia/Kuala_Lumpur
1.6006 103.6419 Asia/Kuala_Lumpur
1.6561 103.6032 Asia/Kuala_Lumpur
1.53333 103.66667 Asia/Kuala_Lumpur
1.4655 103.7578 Asia/Kuala_Lumpur
2.0251 103.3328 Asia/Kuala_Lumpur
2.0136 103.0659 Asia/Kuala_Lumpur
2.4312 103.8405 Asia/Kuala_Lumpur
2.5148 102.8158 Asia/Kuala_Lumpur
2.2673 102.5453 Asia/Kuala_Lumpur
2.0442 102.5689 Asia/Kuala_Lumpur
2.0441 102.6527 Asia/Kuala_Lumpur
2.385 103.021 Asia/Kuala_Lumpur
3.35 101.25 Asia/Kuala_Lumpur
3.38333 101.41667 Asia/Kuala_Lumpur
3.31667 101.46667 Asia/Kuala_Lumpur
3.08507 101.53281 Asia/Kuala_Lumpur
3.03333 101.45 Asia/Kuala_Lumpur
4.25 103.41667 Asia/Kuala_Lumpur
4.1842 102.0468 Asia/Kuala_Lumpur
5.73333 115.93333 Asia/Kuching
5.9749 116.0724 Asia/Kuching
5.90702 116.10146 Asia/Kuching
5.9258 116.06094 Asia/Kuching
5.8231 116.0466 Asia/Kuching
5.9538 116.6641 Asia/Kuching
4.48178 118.61118 Asia/Kuching
5.27667 115.24167 Asia/Kuching
5.3473 115.7455 Asia/Kuching
5.0268 118.327 Asia/Kuching
5.8402 118.1179 Asia/Kuching
5.3378 116.1602 Asia/Kuching
4.2498 117.8871 Asia/Kuching
4.6374 103.4368 Asia/Kuala_Lumpur
4.5141 103.4483 Asia/Kuala_Lumpur
5.36499 100.56177 Asia/Kuala_Lumpur
5.0108 100.54101 Asia/Kuala_Lumpur
5.72744 100.50876 Asia/Kuala_Lumpur
4.95 100.63333 Asia/Kuala_Lumpur
4.85 100.73333 Asia/Kuala_Lumpur
4.76667 100.93333 Asia/Kuala_Lumpur
4.5841 101.0829 Asia/Kuala_Lumpur
4.8823 101.9644 Asia/Kuala_Lumpur
5.3302 103.1408 Asia/Kuala_Lumpur
5.2056 103.2059 Asia/Kuala_Lumpur
2.4701 102.2302 Asia/Kuala_Lumpur
2.3804 102.2089 Asia/Kuala_Lumpur
2.2654 102.2801 Asia/Kuala_Lumpur
2.269 102.1427 Asia/Kuala_Lumpur
2.196 102.2405 Asia/Kuala_Lumpur
2.81667 101.5 Asia/Kuala_Lumpur
2.88333 101.5 Asia/Kuala_Lumpur
2.9516 101.843 Asia/Kuala_Lumpur
2.7297 101.9381 Asia/Kuala_Lumpur
2.5228 101.7959 Asia/Kuala_Lumpur
2.8079 102.4049 Asia/Kuala_Lumpur
2.7389 102.2487 Asia/Kuala_Lumpur
3.4836 103.3996 Asia/Kuala_Lumpur
3.4854 102.3484 Asia/Kuala_Lumpur
3.4506 102.4176 Asia/Kuala_Lumpur
5.3991 100.36382 Asia/Kuala_Lumpur
5.38333 100.38333 Asia/Kuala_Lumpur
5.36301 100.4667 Asia/Kuala_Lumpur
5.16586 100.47793 Asia/Kuala_Lumpur
5.12671 100.49316 Asia/Kuala_Lumpur
5.48032 100.49849 Asia/Kuala_Lumpur
5.41123 100.33543 Asia/Kuala_Lumpur
3.3646 101.6041 Asia/Kuala_Lumpur
3.3213 101.5767 Asia/Kuala_Lumpur
3.10726 101.60671 Asia/Kuala_Lumpur
3.1412 101.68653 Asia/Kuala_Lumpur
3.7698 100.9879 Asia/Kuala_Lumpur
3.6746 100.9867 Asia/Kuala_Lumpur
3.8077 103.326 Asia/Kuala_Lumpur
4.46916 101.04107 Asia/Kuala_Lumpur
4.3 101.15 Asia/Kuala_Lumpur
4.16667 101.2 Asia/Kuala_Lumpur
4.11667 101.28333 Asia/Kuala_Lumpur
4.2323 100.6298 Asia/Kuala_Lumpur
4.0259 101.0213 Asia/Kuala_Lumpur
5.81717 100.47381 Asia/Kuala_Lumpur
5.647 100.48772 Asia/Kuala_Lumpur
5.51707 100.4265 Asia/Kuala_Lumpur
5.8 102.15 Asia/Kuala_Lumpur
1.55 110.33333 Asia/Kuching
1.24722 111.45278 Asia/Kuching
2.11667 111.51667 Asia/Kuching
2.3 111.81667 Asia/Kuching
6.4414 100.19862 Asia/Kuala_Lumpur
6.26812 100.42167 Asia/Kuala_Lumpur
6.1 100.3 Asia/Kuala_Lumpur
6.12104 100.36014 Asia/Kuala_Lumpur
6.04934 102.13987 Asia/Kuala_Lumpur
6.13328 102.2386 Asia/Kuala_Lumpur
6.8837 116.8477 Asia/Kuching
2.01667 112.93333 Asia/Kuching
3.16667 113.03333 Asia/Kuching
4.75 115 Asia/Kuching
4.4148 114.0089 Asia/Kuching
1.6 103.81667 Asia/Kuala_Lumpur
5.46061 100.30742 Asia/Kuala_Lumpur
2.6579 101.5629 Asia/Kuala_Lumpur
5.46339 100.38144 Asia/Kuala_Lumpur
6.03333 102.28333 Asia/Kuala_Lumpur
3.2856 101.519 Asia/Kuala_Lumpur
4.4557 100.6288 Asia/Kuala_Lumpur
3.2594 101.5541 Asia/Kuala_Lumpur
2.2186 102.1995 Asia/Kuala_Lumpur
3.4242 101.1849 Asia/Kuala_Lumpur
5.32699 100.27348 Asia/Kuala_Lumpur
1.8278 103.3 Asia/Kuala_Lumpur
5.91667 102.21667 Asia/Kuala_Lumpur
2.35 102.11667 Asia/Kuala_Lumpur
6 102.25 Asia/Kuala_Lumpur
3.21667 101.31667 Asia/Kuala_Lumpur
2.2152 102.2851 Asia/Kuala_Lumpur
3.15 101.53333 Asia/Kuala_Lumpur
2.8033 101.7972 Asia/Kuala_Lumpur
3.03333 101.75 Asia/Kuala_Lumpur
2.2139 102.3278 Asia/Kuala_Lumpur
2.2594 102.1838 Asia/Kuala_Lumpur
3.51667 101.9 Asia/Kuala_Lumpur
2.2487 102.246 Asia/Kuala_Lumpur
2.93527 101.69112 Asia/Kuala_Lumpur
5.28883 115.26924 Asia/Kuching
-25.05194 33.64417 Africa/Maputo
-19.60944 34.74306 Africa/Maputo
-25.02694 33.09889 Africa/Maputo
-16.15639 33.58667 Africa/Maputo
-17.87861 36.88833 Africa/Maputo
-12.97395 40.51775 Africa/Maputo
-15.11646 39.2666 Africa/Maputo
-14.54278 40.67278 Africa/Maputo
-13.12556 38.99972 Africa/Maputo
-11.31667 40.35 Africa/Maputo
-15.03417 40.73583 Africa/Maputo
-23.85972 35.34722 Africa/Maputo
-25.96222 32.45889 Africa/Maputo
-25.96528 32.58917 Africa/Maputo
-24.71167 33.88278 Africa/Maputo
-13.31278 35.24056 Africa/Maputo
-23.865 35.38333 Africa/Maputo
-14.80306 36.53722 Africa/Maputo
-24.53333 32.98333 Africa/Maputo
-19.11639 33.48333 Africa/Maputo
-24.68667 33.53056 Africa/Maputo
-19.84361 34.83889 Africa/Maputo
-16.2325 39.90861 Africa/Maputo
-14.87056 37.00444 Africa/Maputo
-17.5 24.26667 Africa/Windhoek
-22.55941 17.08323 Africa/Windhoek
-22.68333 14.53333 Africa/Windhoek
-17.93333 19.76667 Africa/Windhoek
-23.31667 17.08333 Africa/Windhoek
-20.46369 16.64772 Africa/Windhoek
-17.78333 15.68333 Africa/Windhoek
-21.98333 16.91667 Africa/Windhoek
-26.64806 15.15944 Africa/Windhoek
-26.58333 18.13333 Africa/Windhoek
-19.56667 18.11667 Africa/Windhoek
-22.45 18.96667 Africa/Windhoek
-22.9575 14.50528 Africa/Windhoek
-22.27631 166.4572 Pacific/Noumea
-22.28333 166.58333 Pacific/Noumea
-22.15 166.45 Pacific/Noumea
13.79562 5.2553 Africa/Niamey
13.80716 8.9881 Africa/Niamey
14.21167 1.45306 Africa/Niamey
13.56271 7.04848 Africa/Niamey
13.75737 7.9874 Africa/Niamey
14.00776 0.75306 Africa/Niamey
14.97089 8.88786 Africa/Niamey
14.8888 5.2692 Africa/Niamey
13.51366 2.1098 Africa/Niamey
14.24953 13.10921 Africa/Niamey
13.70727 9.15013 Africa/Niamey
13.95532 7.67122 Africa/Niamey
13.42309 8.47485 Africa/Niamey
13.5 7.10174 Africa/Niamey
12.99826 8.90991 Africa/Niamey
14.073 5.96 Africa/Niamey
14.4605 5.2437 Africa/Niamey
11.88435 3.44919 Africa/Niamey
13.049 3.1937 Africa/Niamey
13.63933 4.02875 Africa/Niamey
13.31536 12.61134 Africa/Niamey
14.51056 6.765 Africa/Niamey
14.73075 0.91738 Africa/Niamey
17.0187 8.0168 Africa/Niamey
-29.05459 167.96628 Pacific/Norfolk
11.42406 5.22879 Africa/Lagos
9.81278 6.15583 Africa/Lagos
11.11128 7.7227 Africa/Lagos
9.2 12.48333 Africa/Lagos
4.92472 6.26417 Africa/Lagos
7.85 9.78333 Africa/Lagos
11.81067 8.84505 Africa/Lagos
5.51667 5.75 Africa/Lagos
8.93333 8.6 Africa/Lagos
6.7 6.33333 Africa/Lagos
5.52627 7.48959 Africa/Lagos
5.48986 6.00743 Africa/Lagos
5.8086 8.0812 Africa/Lagos
5.93576 7.0793 Africa/Lagos
6.65504 6.38687 Africa/Lagos
10.0706 6.1906 Africa/Lagos
12.40386 4.64692 Africa/Lagos
12.57317 6.05971 Africa/Lagos
7.26667 9.98333 Africa/Lagos
9.17583 7.18083 Africa/Lagos
13.06092 5.23902 Africa/Lagos
10.98134 8.05749 Africa/Lagos
8.66667 3.38333 Africa/Lagos
6.84323 3.64776 Africa/Lagos
5.89405 5.67666 Africa/Lagos
11.10298 5.25068 Africa/Lagos
11.54912 8.57464 Africa/Lagos
11.7091 11.0694 Africa/Lagos
4.77742 7.0134 Africa/Lagos
9.98333 10.93333 Africa/Lagos
8.73333 5.75 Africa/Lagos
9.33333 9.45 Africa/Lagos
5.95666 6.84904 Africa/Lagos
7.85 3.93333 Africa/Lagos
8.05 4.76667 Africa/Lagos
7.1962 5.58681 Africa/Lagos
5.48333 7.03041 Africa/Lagos
7.08921 7.64989 Africa/Lagos
7.55 3.43333 Africa/Lagos
6.14543 6.78845 Africa/Lagos
7.1 4.83333 Africa/Lagos
7.6 4.18333 Africa/Lagos
9.21667 3.18333 Africa/Lagos
4.74159 7.08488 Africa/Lagos
5.83523 7.35989 Africa/Lagos
7.81667 4.91667 Africa/Lagos
5.61455 7.81191 Africa/Lagos
6.17818 6.52429 Africa/Lagos
5.7089 6.81026 Africa/Lagos
6.65471 8.79764 Africa/Lagos
7.59464 6.22476 Africa/Lagos
8.15 4.71667 Africa/Lagos
7.7899 5.7117 Africa/Lagos
6.66737 9.17157 Africa/Lagos
4.70844 6.79307 Africa/Lagos
9.46667 12.03333 Africa/Lagos
6.8561 7.3927 Africa/Lagos
6.01986 6.91478 Africa/Lagos
5.75555 7.10627 Africa/Lagos
6.15164 6.84458 Africa/Lagos
12.8791 10.4526 Africa/Lagos
8.52944 7.72417 Africa/Lagos
11.0956 11.3327 Africa/Lagos
10.26761 13.26436 Africa/Lagos
12.87405 6.48754 Africa/Lagos
12.67863 13.60792 Africa/Lagos
9.28333 5.05 Africa/Lagos
7.38333 4.26667 Africa/Lagos
9.61389 6.55694 Africa/Lagos
12.36113 13.8246 Africa/Lagos
11.78529 7.62175 Africa/Lagos
7.7411 8.5121 Africa/Lagos
11.84644 13.16027 Africa/Lagos
12.1131 12.8274 Africa/Lagos
7.80236 6.743 Africa/Lagos
10.38697 8.57262 Africa/Lagos
9.05 6.56667 Africa/Lagos
7.46667 4.06667 Africa/Lagos
6.45306 3.39583 Africa/Lagos
8.86667 5.41667 Africa/Lagos
8.48333 8.51667 Africa/Lagos
5.70632 6.43741 Africa/Lagos
10.04469 11.21217 Africa/Lagos
13.15514 10.63423 Africa/Lagos
12.92342 13.56058 Africa/Lagos
10.39989 5.46949 Africa/Lagos
9.08333 3.85 Africa/Lagos
8.84861 7.87361 Africa/Lagos
12.58979 6.57791 Africa/Lagos
7.16667 9.28333 Africa/Lagos
12.98943 7.60063 Africa/Lagos
11.2471 10.561 Africa/Lagos
12.00012 8.51672 Africa/Lagos
11.84864 3.65761 Africa/Lagos
9.6225 3.94806 Africa/Lagos
9.6 8.38333 Africa/Lagos
9.58333 8.3 Africa/Lagos
10.52224 7.43828 Africa/Lagos
9.86667 7.95 Africa/Lagos
7.82873 6.0731 Africa/Lagos
9.91667 8.9 Africa/Lagos
9.28333 12.46667 Africa/Lagos
12.2175 4.37917 Africa/Lagos
9.13333 4.83333 Africa/Lagos
8.88333 11.36667 Africa/Lagos
7.63333 4.18333 Africa/Lagos
5.2004 7.98475 Africa/Lagos
6.37831 8.03524 Africa/Lagos
7.4632 5.4281 Africa/Lagos
8.26667 5.83333 Africa/Lagos
7.87377 5.07692 Africa/Lagos
6.91002 3.66557 Africa/Lagos
7.85 4.33333 Africa/Lagos
8.5 4.55 Africa/Lagos
7.83333 4.48333 Africa/Lagos
13.72918 5.29752 Africa/Lagos
7.61667 4.73333 Africa/Lagos
6.88653 3.0205 Africa/Lagos
8.01667 4.9 Africa/Lagos
5.17938 7.71082 Africa/Lagos
5.96669 8.70632 Africa/Lagos
7.91667 4.66667 Africa/Lagos
7.35 4.18333 Africa/Lagos
7.49133 5.23225 Africa/Lagos
7.8139 5.0742 Africa/Lagos
6.81609 3.91588 Africa/Lagos
7.68333 4.81667 Africa/Lagos
6.9798 4.00329 Africa/Lagos
5.85377 6.86026 Africa/Lagos
7.66676 5.13205 Africa/Lagos
6.01669 7.02654 Africa/Lagos
7.4527 8.6098 Africa/Lagos
7.43333 3.28333 Africa/Lagos
8.83333 3.75 Africa/Lagos
8.75 4.13333 Africa/Lagos
7.50251 5.06258 Africa/Lagos
6.81437 3.19575 Africa/Lagos
7.1127 5.1159 Africa/Lagos
7.10651 6.73415 Africa/Lagos
8.18333 9.75 Africa/Lagos
10.48536 5.14502 Africa/Lagos
7.38778 3.89639 Africa/Lagos
12.4498 10.0444 Africa/Lagos
11.10359 13.71298 Africa/Lagos
11.91521 7.9346 Africa/Lagos
11.27639 9.88583 Africa/Lagos
13.35612 5.23664 Africa/Lagos
12.16278 6.66135 Africa/Lagos
12.14358 5.10809 Africa/Lagos
12.62791 9.39119 Africa/Lagos
10.16138 12.7399 Africa/Lagos
10.28969 11.16729 Africa/Lagos
6.7 11.26667 Africa/Lagos
12.897 11.9304 Africa/Lagos
7.46667 4.35 Africa/Lagos
11.86136 8.99714 Africa/Lagos
12.871 11.0482 Africa/Lagos
10.17506 11.16458 Africa/Lagos
8.43333 12.06667 Africa/Lagos
12.37066 14.21731 Africa/Lagos
11.52325 7.30813 Africa/Lagos
7.71361 3.91722 Africa/Lagos
6.4373 8.0862 Africa/Lagos
4.80293 8.25341 Africa/Lagos
6.58412 3.98336 Africa/Lagos
6.17851 7.01156 Africa/Lagos
6.9825 7.457 Africa/Lagos
6.4402 7.4943 Africa/Lagos
7.43614 5.45932 Africa/Lagos
5.1009 6.81411 Africa/Lagos
6.74206 6.139 Africa/Lagos
4.64122 7.92092 Africa/Lagos
7.9 4.31667 Africa/Lagos
6.65915 7.75961 Africa/Lagos
8.21667 5.51667 Africa/Lagos
7.65 4.91667 Africa/Lagos
6.62917 8.05925 Africa/Lagos
6.60086 3.48818 Africa/Lagos
10.85009 8.199 Africa/Lagos
11.7594 9.33921 Africa/Lagos
10.82306 10.77167 Africa/Lagos
8.38333 8.35 Africa/Lagos
12.02396 13.91646 Africa/Lagos
10.20864 11.38586 Africa/Lagos
13.03594 8.31631 Africa/Lagos
11.5538 11.4085 Africa/Lagos
10.99375 10.41179 Africa/Lagos
11.15604 12.75758 Africa/Lagos
11.747 11.9608 Africa/Lagos
4.9517 8.322 Africa/Lagos
5.35091 5.50758 Africa/Lagos
9.8 8.86667 Africa/Lagos
4.73407 6.86345 Africa/Lagos
4.4522 7.16808 Africa/Lagos
8.93333 4.78333 Africa/Lagos
10.61285 12.19458 Africa/Lagos
11.44588 9.4984 Africa/Lagos
12.45389 4.1975 Africa/Lagos
11.01537 6.78036 Africa/Lagos
9.86472 11.22528 Africa/Lagos
9.08333 6.01667 Africa/Lagos
6.33504 5.62749 Africa/Lagos
5.55718 7.63676 Africa/Lagos
7.85868 10.97187 Africa/Lagos
10.31344 9.84327 Africa/Lagos
8.6 6.41667 Africa/Lagos
11.5221 13.68558 Africa/Lagos
6.41667 2.88333 Africa/Lagos
10.42949 3.81495 Africa/Lagos
11.6765 10.1948 Africa/Lagos
6.21009 7.07411 Africa/Lagos
6.0742 7.4786 Africa/Lagos
7.06667 6.26667 Africa/Lagos
6.20064 6.73384 Africa/Lagos
12.74334 4.52687 Africa/Lagos
7.70927 5.04474 Africa/Lagos
7.33333 4.18333 Africa/Lagos
10.95967 8.39155 Africa/Lagos
5.78917 7.83829 Africa/Lagos
8.91667 8.38333 Africa/Lagos
7.25256 5.19312 Africa/Lagos
6.7081 7.3167 Africa/Lagos
7.56229 6.65497 Africa/Lagos
6.11 7.0724 Africa/Lagos
6.25183 6.19337 Africa/Lagos
5.89312 7.93735 Africa/Lagos
6.6 2.93333 Africa/Lagos
9.05735 7.48976 Africa/Lagos
7.15 3.35 Africa/Lagos
6.31625 8.11691 Africa/Lagos
5.10658 7.36667 Africa/Lagos
4.76254 6.752 Africa/Lagos
12.19732 -86.09706 America/Managua
13.48082 -86.58208 America/Managua
13.04328 -86.90645 America/Managua
13.73321 -84.77725 America/Managua
11.84854 -86.43839 America/Managua
11.90949 -86.20351 America/Managua
11.43716 -85.82632 America/Managua
12.93435 -85.22354 America/Managua
12.15965 -84.21952 America/Managua
14.03507 -83.38882 America/Managua
13.63208 -86.47516 America/Managua
11.68758 -84.45616 America/Managua
11.75696 -86.05286 America/Managua
12.26593 -86.56474 America/Managua
12.92559 -85.91747 America/Managua
11.97444 -86.09417 America/Managua
11.9184 -86.14467 America/Managua
12.13282 -86.2504 America/Managua
12.43787 -86.87804 America/Managua
12.34 -86.67528 America/Managua
12.10629 -85.36452 America/Managua
11.84962 -86.19903 America/Managua
13.09171 -86.00177 America/Managua
13.92222 -86.12346 America/Managua
11.92988 -85.95602 America/Managua
13.09185 -86.35384 America/Managua
12.6633 -87.16722 America/Managua
11.99008 -86.30954 America/Managua
11.85842 -86.23878 America/Managua
12.48199 -87.17336 America/Managua
12.62937 -87.13105 America/Managua
12.57758 -87.02705 America/Managua
12.38386 -85.51157 America/Managua
12.47224 -85.6586 America/Managua
12.01366 -83.76353 America/Managua
12.15889 -86.34417 America/Managua
52.5125 6.09444 Europe/Amsterdam
51.8175 4.63333 Europe/Amsterdam
52.13833 6.20139 Europe/Amsterdam
51.47167 4.65556 Europe/Amsterdam
52.0575 4.49306 Europe/Amsterdam
51.93 6.07083 Europe/Amsterdam
52.09 5.23333 Europe/Amsterdam
52.33 5.54167 Europe/Amsterdam
52.37487 4.53409 Europe/Amsterdam
51.81 5.24444 Europe/Amsterdam
52.45313 4.81356 Europe/Amsterdam
52.43854 4.82643 Europe/Amsterdam
52.87545 5.99691 Europe/Amsterdam
52.085 4.88333 Europe/Amsterdam
51.42897 4.30355 Europe/Amsterdam
51.92648 6.41705 Europe/Amsterdam
51.9725 6.71944 Europe/Amsterdam
53.14417 7.03472 Europe/Amsterdam
51.97417 5.34167 Europe/Amsterdam
51.80917 5.725 Europe/Amsterdam
52.35917 6.59306 Europe/Amsterdam
51.95583 5.97222 Europe/Amsterdam
51.81 4.89444 Europe/Amsterdam
52.3075 5.04167 Europe/Amsterdam
51.25167 5.70694 Europe/Amsterdam
52.14583 4.40278 Europe/Amsterdam
51.97 5.66667 Europe/Amsterdam
52.045 4.65139 Europe/Amsterdam
51.6825 5.07083 Europe/Amsterdam
51.38667 5.44444 Europe/Amsterdam
51.65333 5.2875 Europe/Amsterdam
52.17 6.14167 Europe/Amsterdam
52.1275 4.44861 Europe/Amsterdam
52.22167 4.48472 Europe/Amsterdam
52.07417 4.35972 Europe/Amsterdam
52.495 5.07083 Europe/Amsterdam
51.4425 3.57361 Europe/Amsterdam
53.0275 7.10833 Europe/Amsterdam
51.9125 4.34167 Europe/Amsterdam
51.9925 5.09167 Europe/Amsterdam
51.525 5.975 Europe/Amsterdam
51.37 6.16806 Europe/Amsterdam
52.46 4.65 Europe/Amsterdam
51.995 5.97361 Europe/Amsterdam
51.41833 5.40278 Europe/Amsterdam
51.61667 5.54861 Europe/Amsterdam
51.54833 3.66667 Europe/Amsterdam
52.02863 5.55891 Europe/Amsterdam
53.10667 6.87917 Europe/Amsterdam
51.35083 5.45972 Europe/Amsterdam
52.09083 5.12222 Europe/Amsterdam
52.6625 5.60139 Europe/Amsterdam
52.2375 4.82639 Europe/Amsterdam
51.66083 5.61944 Europe/Amsterdam
52.4075 6.78472 Europe/Amsterdam
51.44889 5.51978 Europe/Amsterdam
51.55551 5.0913 Europe/Amsterdam
51.88667 5.42917 Europe/Amsterdam
51.33583 3.82778 Europe/Amsterdam
51.34417 6.13611 Europe/Amsterdam
52.7875 6.12083 Europe/Amsterdam
51.58417 4.31944 Europe/Amsterdam
52.645 6.21111 Europe/Amsterdam
52.98947 6.9504 Europe/Amsterdam
51.845 4.32917 Europe/Amsterdam
51.385 5.71111 Europe/Amsterdam
52.17333 5.29167 Europe/Amsterdam
53.03297 5.6589 Europe/Amsterdam
51.82083 4.77639 Europe/Amsterdam
50.99833 5.86944 Europe/Amsterdam
51.5675 5.45972 Europe/Amsterdam
51.69917 5.30417 Europe/Amsterdam
52.00167 4.16528 Europe/Amsterdam
52.07667 4.29861 Europe/Amsterdam
51.6225 5.43194 Europe/Amsterdam
51.91917 4.38889 Europe/Amsterdam
52.10461 4.27557 Europe/Amsterdam
52.7875 4.79861 Europe/Amsterdam
51.53167 4.55833 Europe/Amsterdam
51.9225 4.47917 Europe/Amsterdam
51.53083 4.46528 Europe/Amsterdam
51.19417 5.9875 Europe/Amsterdam
52.03634 4.32501 Europe/Amsterdam
51.8725 4.60278 Europe/Amsterdam
51.8575 4.42222 Europe/Amsterdam
51.95917 5.56806 Europe/Amsterdam
52.38583 6.275 Europe/Amsterdam
52.25917 5.60694 Europe/Amsterdam
52.505 4.95972 Europe/Amsterdam
52.01954 4.42946 Europe/Amsterdam
51.83167 4.6875 Europe/Amsterdam
51.82417 4.4125 Europe/Amsterdam
51.765 5.51806 Europe/Amsterdam
51.645 4.85972 Europe/Amsterdam
52.31333 6.92917 Europe/Amsterdam
52.445 5.90139 Europe/Amsterdam
51.57917 5.18889 Europe/Amsterdam
52.18 4.46944 Europe/Amsterdam
50.9175 5.88611 Europe/Amsterdam
52.37917 5.78611 Europe/Amsterdam
51.47 5.55278 Europe/Amsterdam
52.26167 4.49306 Europe/Amsterdam
52.23583 4.44722 Europe/Amsterdam
51.8425 5.85278 Europe/Amsterdam
52.22 5.48611 Europe/Amsterdam
52.02917 5.08056 Europe/Amsterdam
51.28583 5.74861 Europe/Amsterdam
52.29583 5.1625 Europe/Amsterdam
51.99417 4.20972 Europe/Amsterdam
52.20667 4.8625 Europe/Amsterdam
51.7575 4.16528 Europe/Amsterdam
51.5 3.61389 Europe/Amsterdam
52.69583 6.19444 Europe/Amsterdam
50.8875 5.75 Europe/Amsterdam
50.84833 5.68889 Europe/Amsterdam
51.92333 4.25 Europe/Amsterdam
52.13917 5.04167 Europe/Amsterdam
52.26083 7.00417 Europe/Amsterdam
51.6275 5.075 Europe/Amsterdam
52.26 4.55694 Europe/Amsterdam
51.8327 5.7934 Europe/Amsterdam
51.98667 6.56667 Europe/Amsterdam
52.1325 5.43194 Europe/Amsterdam
52.50833 5.475 Europe/Amsterdam
52.15833 4.52917 Europe/Amsterdam
52.15833 4.49306 Europe/Amsterdam
53.20139 5.80859 Europe/Amsterdam
51.89333 5.09167 Europe/Amsterdam
53.1625 6.37639 Europe/Amsterdam
51.91667 4.60278 Europe/Amsterdam
53.23235 6.56804 Europe/Amsterdam
50.86583 6.0625 Europe/Amsterdam
52.20333 4.39861 Europe/Amsterdam
52.555 5.91111 Europe/Amsterdam
52.02 5.04306 Europe/Amsterdam
52.29917 5.24167 Europe/Amsterdam
52.02833 5.16806 Europe/Amsterdam
52.30833 5.58194 Europe/Amsterdam
52.6425 5.05972 Europe/Amsterdam
53.16167 6.76111 Europe/Amsterdam
52.7225 6.47639 Europe/Amsterdam
52.3025 4.68889 Europe/Amsterdam
50.92387 5.92528 Europe/Amsterdam
52.22333 5.17639 Europe/Amsterdam
51.48583 5.1375 Europe/Amsterdam
52.29083 4.58333 Europe/Amsterdam
52.26583 6.79306 Europe/Amsterdam
51.84417 4.63889 Europe/Amsterdam
51.48167 5.66111 Europe/Amsterdam
51.83333 4.13333 Europe/Amsterdam
52.60252 4.68815 Europe/Amsterdam
51.73362 5.52672 Europe/Amsterdam
50.88365 5.98154 Europe/Amsterdam
52.67144 4.84862 Europe/Amsterdam
52.95929 5.91854 Europe/Amsterdam
52.38723 6.04016 Europe/Amsterdam
52.34992 4.62301 Europe/Amsterdam
52.51108 4.67165 Europe/Amsterdam
53.17477 5.42244 Europe/Amsterdam
52.73416 4.77682 Europe/Amsterdam
53.17667 6.63037 Europe/Amsterdam
52.34167 5.62083 Europe/Amsterdam
52.57583 6.61944 Europe/Amsterdam
52.38084 4.63683 Europe/Amsterdam
52.15667 6.73889 Europe/Amsterdam
53.21917 6.56667 Europe/Amsterdam
51.77667 5.93611 Europe/Amsterdam
52.01667 4.70833 Europe/Amsterdam
51.83652 4.97243 Europe/Amsterdam
51.52083 5.06667 Europe/Amsterdam
51.50417 3.88889 Europe/Amsterdam
51.69833 5.97361 Europe/Amsterdam
51.87333 6.37639 Europe/Amsterdam
51.42167 5.55972 Europe/Amsterdam
51.88083 5.28889 Europe/Amsterdam
51.70167 4.85694 Europe/Amsterdam
52.29833 5.62222 Europe/Amsterdam
52.3475 5.98333 Europe/Amsterdam
52.21833 6.89583 Europe/Amsterdam
52.70333 5.29167 Europe/Amsterdam
52.77917 6.90694 Europe/Amsterdam
52.71083 5.74861 Europe/Amsterdam
51.91917 5.84167 Europe/Amsterdam
52.4475 5.84306 Europe/Amsterdam
51.44083 5.47778 Europe/Amsterdam
52.1 6.64861 Europe/Amsterdam
51.3575 5.31806 Europe/Amsterdam
52.03333 5.65833 Europe/Amsterdam
51.94667 6.01389 Europe/Amsterdam
51.88833 5.60556 Europe/Amsterdam
52.525 5.71806 Europe/Amsterdam
51.70667 4.80417 Europe/Amsterdam
52.05333 5.28056 Europe/Amsterdam
53.11254 6.0989 Europe/Amsterdam
51.81 4.67361 Europe/Amsterdam
51.62667 4.93889 Europe/Amsterdam
51.965 6.28889 Europe/Amsterdam
52.33964 4.96256 Europe/Amsterdam
52.255 6.16389 Europe/Amsterdam
52.95988 4.75933 Europe/Amsterdam
53.33 6.91806 Europe/Amsterdam
52.00667 4.35556 Europe/Amsterdam
51.90488 4.45315 Europe/Amsterdam
52.11 5.18056 Europe/Amsterdam
52.51167 6.25694 Europe/Amsterdam
51.955 5.22778 Europe/Amsterdam
51.73083 5.87917 Europe/Amsterdam
51.30417 5.58889 Europe/Amsterdam
52.54833 4.66944 Europe/Amsterdam
51.91667 4.56667 Europe/Amsterdam
51.92917 4.57778 Europe/Amsterdam
52.27333 5.16111 Europe/Amsterdam
52.24304 5.37884 Europe/Amsterdam
50.94667 5.97083 Europe/Amsterdam
52.09 6.15556 Europe/Amsterdam
52.67417 4.80556 Europe/Amsterdam
52.43417 4.99583 Europe/Amsterdam
51.58656 4.77596 Europe/Amsterdam
51.59083 5.32917 Europe/Amsterdam
52.075 4.65556 Europe/Amsterdam
51.42333 3.73472 Europe/Amsterdam
52.30136 6.7482 Europe/Amsterdam
51.03167 5.80972 Europe/Amsterdam
52.92333 6.79306 Europe/Amsterdam
52.0825 4.75 Europe/Amsterdam
52.4025 4.62222 Europe/Amsterdam
51.36833 5.22083 Europe/Amsterdam
52.48333 4.65694 Europe/Amsterdam
51.86083 5.76667 Europe/Amsterdam
51.5075 5.39028 Europe/Amsterdam
51.99 4.49861 Europe/Amsterdam
51.31917 5.35833 Europe/Amsterdam
51.495 4.29167 Europe/Amsterdam
52.0775 4.54444 Europe/Amsterdam
50.94083 5.79722 Europe/Amsterdam
52.14 5.58472 Europe/Amsterdam
51.85667 4.53472 Europe/Amsterdam
52.21167 5.2875 Europe/Amsterdam
51.40417 5.74861 Europe/Amsterdam
52.99667 6.5625 Europe/Amsterdam
51.98 5.91111 Europe/Amsterdam
52.21 5.96944 Europe/Amsterdam
53.04417 6.70139 Europe/Amsterdam
52.37403 4.88969 Europe/Amsterdam
52.30083 4.86389 Europe/Amsterdam
52.155 5.3875 Europe/Amsterdam
52.12917 4.65546 Europe/Amsterdam
52.37025 5.21413 Europe/Amsterdam
52.35667 6.6625 Europe/Amsterdam
52.63167 4.74861 Europe/Amsterdam
51.86583 4.66111 Europe/Amsterdam
51.925 6.58056 Europe/Amsterdam
52.25917 4.75972 Europe/Amsterdam
52.3075 4.97222 Europe/Amsterdam
51.99313 4.47865 Europe/Amsterdam
52.04098 4.36981 Europe/Amsterdam
63.43049 10.39506 Europe/Oslo
69.6489 18.95508 Europe/Oslo
59.26754 10.40762 Europe/Oslo
64.01487 11.49537 Europe/Oslo
58.97005 5.73332 Europe/Oslo
59.20962 9.60897 Europe/Oslo
59.28391 11.10962 Europe/Oslo
58.85244 5.73521 Europe/Oslo
59.13118 10.21665 Europe/Oslo
59.14054 9.6561 Europe/Oslo
59.91273 10.74609 Europe/Oslo
59.43403 10.65771 Europe/Oslo
62.73752 7.15912 Europe/Oslo
66.31278 14.14278 Europe/Oslo
61.11514 10.46628 Europe/Oslo
59.05328 10.03517 Europe/Oslo
63.11045 7.72795 Europe/Oslo
58.14671 7.9956 Europe/Oslo
59.66858 9.65017 Europe/Oslo
59.41721 10.48343 Europe/Oslo
59.41378 5.268 Europe/Oslo
68.79833 16.54165 Europe/Oslo
60.7945 11.06798 Europe/Oslo
59.12478 11.38754 Europe/Oslo
60.79574 10.69155 Europe/Oslo
59.2181 10.9298 Europe/Oslo
59.74389 10.20449 Europe/Oslo
67.28 14.40501 Europe/Oslo
60.39299 5.32415 Europe/Oslo
58.46151 8.77253 Europe/Oslo
62.47225 6.15492 Europe/Oslo
60.4 5.18333 Europe/Oslo
60.30504 5.28236 Europe/Oslo
27.98333 83.76667 Asia/Kathmandu
28.13099 82.29726 Asia/Kathmandu
28.5 81.13333 Asia/Kathmandu
27.86731 83.5467 Asia/Kathmandu
26.65411 86.2087 Asia/Kathmandu
26.53333 86.75 Asia/Kathmandu
28.26689 83.96851 Asia/Kathmandu
27.67658 85.31417 Asia/Kathmandu
27.58138 85.5124 Asia/Kathmandu
26.85679 85.55811 Asia/Kathmandu
28.91667 80.33333 Asia/Kathmandu
26.72958 86.49515 Asia/Kathmandu
27.67988 85.27544 Asia/Kathmandu
27.37469 87.2039 Asia/Kathmandu
27.70169 85.3206 Asia/Kathmandu
26.71828 85.90646 Asia/Kathmandu
26.64711 85.80081 Asia/Kathmandu
26.66667 87.28333 Asia/Kathmandu
26.90943 87.92824 Asia/Kathmandu
27.42839 85.03219 Asia/Kathmandu
28.23333 81.33333 Asia/Kathmandu
26.76667 85.26667 Asia/Kathmandu
26.81248 87.28355 Asia/Kathmandu
26.98333 87.33333 Asia/Kathmandu
28.70792 80.59611 Asia/Kathmandu
29.84121 80.52874 Asia/Kathmandu
28.84434 81.71011 Asia/Kathmandu
29.29842 80.58059 Asia/Kathmandu
27.70055 83.44836 Asia/Kathmandu
27.01043 84.87735 Asia/Kathmandu
26.4831 87.28337 Asia/Kathmandu
27.68333 84.43333 Asia/Kathmandu
27.5 83.45 Asia/Kathmandu
26.54404 88.09436 Asia/Kathmandu
27.62979 85.52138 Asia/Kathmandu
28.27189 83.58976 Asia/Kathmandu
28.60194 81.63389 Asia/Kathmandu
28.05 81.61667 Asia/Kathmandu
-0.54756 166.91729 Pacific/Nauru
-19.05952 -169.91867 Pacific/Niue
-41.28664 174.77557 Pacific/Auckland
-39.93333 175.05 Pacific/Auckland
-44.4 171.25 Pacific/Auckland
-38.68333 176.08333 Pacific/Auckland
-37.2 174.95 Pacific/Auckland
-41.13333 174.85 Pacific/Auckland
-40.91667 175.01667 Pacific/Auckland
-40.35 175.61667 Pacific/Auckland
-36.8 174.75 Pacific/Auckland
-39.06667 174.08333 Pacific/Auckland
-41.28333 173.28333 Pacific/Auckland
-39.48333 176.91667 Pacific/Auckland
-36.99282 174.87986 Pacific/Auckland
-36.96807 174.79875 Pacific/Auckland
-41.21667 174.91667 Pacific/Auckland
-46.4 168.35 Pacific/Auckland
-39.6381 176.84918 Pacific/Auckland
-37.78333 175.28333 Pacific/Auckland
-45.87416 170.50361 Pacific/Auckland
-43.53333 172.63333 Pacific/Auckland
-36.86667 174.76667 Pacific/Auckland
-40.63333 175.275 Pacific/Auckland
-38.65333 178.00417 Pacific/Auckland
-40.95972 175.6575 Pacific/Auckland
-37.68611 176.16667 Pacific/Auckland
-37.06572 174.94393 Pacific/Auckland
-37.95855 176.98545 Pacific/Auckland
-35.73167 174.32391 Pacific/Auckland
-37.87822 175.4402 Pacific/Auckland
-38.13874 176.24516 Pacific/Auckland
-41.51603 173.9528 Pacific/Auckland
-41.13827 175.0502 Pacific/Auckland
-39.53333 176.85 Pacific/Auckland
-36.91754 174.65773 Pacific/Auckland
22.56667 59.52889 Asia/Muscat
24.3643 56.74681 Asia/Muscat
23.31667 58.01667 Asia/Muscat
24.74333 56.46583 Asia/Muscat
17.01505 54.09237 Asia/Muscat
24.17222 56.88861 Asia/Muscat
22.93333 57.53333 Asia/Muscat
23.61387 58.5922 Asia/Muscat
26.17993 56.24774 Asia/Muscat
22.93333 57.76667 Asia/Muscat
23.22573 56.51572 Asia/Muscat
22.69057 58.53337 Asia/Muscat
23.40786 58.1283 Asia/Muscat
23.55563 58.3989 Asia/Muscat
23.70739 57.88908 Asia/Muscat
22.97339 57.29958 Asia/Muscat
22.45 58.8 Asia/Muscat
23.84944 57.43861 Asia/Muscat
23.67027 58.18911 Asia/Muscat
23.39083 57.42444 Asia/Muscat
24.53611 56.56556 Asia/Muscat
23.98864 57.09838 Asia/Muscat
24.25088 55.79312 Asia/Muscat
22.3756 57.52334 Asia/Muscat
23.59051 56.55008 Asia/Muscat
8.93333 -79.7 America/Panama
8.88333 -79.63333 America/Panama
9.08333 -79.38333 America/Panama
8.1 -80.98333 America/Panama
9.03333 -79.5 America/Panama
8.28333 -82.86667 America/Panama
8.36667 -82.43333 America/Panama
8.9936 -79.51973 America/Panama
9.08333 -79.28333 America/Panama
8.91667 -79.71667 America/Panama
9.08333 -79.53333 America/Panama
8.51667 -82.61667 America/Panama
8.88028 -79.78333 America/Panama
9.13333 -79.53333 America/Panama
8.42729 -82.43085 America/Panama
9.35917 -79.90139 America/Panama
7.96667 -80.43333 America/Panama
9.15 -79.61667 America/Panama
9.16667 -79.1 America/Panama
9.43333 -82.51667 America/Panama
9.36056 -79.84361 America/Panama
8.95 -79.65 America/Panama
9.11667 -79.55 America/Panama
8.25 -80.55 America/Panama
-5.9 -76.08333 America/Lima
-8.41667 -78.75 America/Lima
-8.45917 -76.46333 America/Lima
-3.56667 -80.44139 America/Lima
-8.11599 -79.02998 America/Lima
-8.18417 -76.5125 America/Lima
-9.28951 -76.00876 America/Lima
-4.92694 -80.34472 America/Lima
-4.57722 -81.27194 America/Lima
-4.90389 -80.68528 America/Lima
-5.55694 -80.82222 America/Lima
-7.95889 -79.23917 America/Lima
-7.43278 -79.50583 America/Lima
-6.92222 -79.58417 America/Lima
-6.05861 -77.165 America/Lima
-4.83778 -80.64556 America/Lima
-8.37915 -74.55387 America/Lima
-5.2 -80.63333 America/Lima
-6.83667 -79.93417 America/Lima
-6.71778 -79.76972 America/Lima
-5.08917 -81.11444 America/Lima
-7.73333 -79.30222 America/Lima
-7.40056 -79.57139 America/Lima
-6.05 -76.96667 America/Lima
-6.87806 -79.87222 America/Lima
-8.17111 -79.00917 America/Lima
-4.87778 -80.70528 America/Lima
-5.4 -80.75 America/Lima
-8.08965 -78.9602 America/Lima
-5.61111 -78.435 America/Lima
-6.70111 -79.90611 America/Lima
-7.17785 -76.73135 America/Lima
-5.70806 -78.80472 America/Lima
-3.74806 -73.24722 America/Lima
-9.53333 -77.53333 America/Lima
-9.9329 -76.24153 America/Lima
-7.8 -78.06667 America/Lima
-7.25 -79.48333 America/Lima
-6.63889 -79.78889 America/Lima
-9.02306 -78.61556 America/Lima
-5.0925 -80.1625 America/Lima
-6.64056 -79.38917 America/Lima
-7.79139 -79.22167 America/Lima
-9.08528 -78.57833 America/Lima
-6.77361 -79.84167 America/Lima
-7.21667 -79.45 America/Lima
-6.23169 -77.86903 America/Lima
-5.26667 -80.68333 America/Lima
-7.16378 -78.50027 America/Lima
-7.05614 -76.5911 America/Lima
-5.75611 -78.44111 America/Lima
-4.26083 -80.8875 America/Lima
-3.50306 -80.27306 America/Lima
-16.25 -69.08333 America/Lima
-10.24111 -76.64556 America/Lima
-11.41972 -75.69083 America/Lima
-12.73333 -69.18333 America/Lima
-18.00556 -70.24833 America/Lima
-14.26944 -71.22611 America/Lima
-11.25222 -74.63861 America/Lima
-13.07512 -76.38352 America/Lima
-12.86667 -72.71667 America/Lima
-12.11667 -77.05 America/Lima
-13.66667 -76.15 America/Lima
-15.83333 -70.03333 America/Lima
-12.6 -69.18333 America/Lima
-13.7 -76.21667 America/Lima
-10.66667 -77.83333 America/Lima
-13.07541 -76.31719 America/Lima
-14.83333 -74.95 America/Lima
-17.19556 -70.93528 America/Lima
-17.02306 -72.01472 America/Lima
-15.21194 -75.11028 America/Lima
-12.65806 -76.63083 America/Lima
-12.04318 -77.02824 America/Lima
-11.51893 -75.89935 America/Lima
-11.15889 -75.99306 America/Lima
-15.5 -70.13333 America/Lima
-11.77584 -75.49656 America/Lima
-13.05927 -76.35269 America/Lima
-17.63944 -71.3375 America/Lima
-16.08333 -69.66667 America/Lima
-14.06528 -75.73083 America/Lima
-11.07 -77.59944 America/Lima
-10.06806 -78.15222 America/Lima
-11.495 -77.20778 America/Lima
-12.93333 -74.25 America/Lima
-12.06667 -75.23333 America/Lima
-12.76667 -74.98333 America/Lima
-11.09639 -77.61389 America/Lima
-11.10667 -77.605 America/Lima
-13.51833 -71.97806 America/Lima
-11.94306 -76.70944 America/Lima
-13.40985 -76.13235 America/Lima
-10.43333 -76.53333 America/Lima
-11.57139 -77.26722 America/Lima
-10.68333 -76.26667 America/Lima
-16.62278 -72.71111 America/Lima
-12.06667 -77.15 America/Lima
-10.75 -77.76667 America/Lima
-14.88639 -70.58889 America/Lima
-13.15833 -74.22389 America/Lima
-16.39889 -71.535 America/Lima
-13.65556 -73.38722 America/Lima
-13.63389 -72.88139 America/Lima
-14.63126 -69.44638 America/Lima
-17.63333 -149.6 Pacific/Tahiti
-17.53333 -149.56667 Pacific/Tahiti
-17.55 -149.6 Pacific/Tahiti
-3.55342 143.62678 Pacific/Port_Moresby
-9.44314 147.17972 Pacific/Port_Moresby
-8.75367 148.25342 Pacific/Port_Moresby
-5.8581 144.22744 Pacific/Port_Moresby
-6.17608 143.65134 Pacific/Port_Moresby
-5.22465 145.79656 Pacific/Port_Moresby
-6.72208 146.98469 Pacific/Port_Moresby
-4.35202 152.26327 Pacific/Port_Moresby
-5.55021 150.14285 Pacific/Port_Moresby
-6.08336 145.38735 Pacific/Port_Moresby
-9.07628 143.20919 Pacific/Port_Moresby
-7.2 146.65 Pacific/Port_Moresby
-6.21319 155.53674 Pacific/Port_Moresby
6.91028 122.07389 Asia/Manila
7.68333 124.66667 Asia/Manila
13.5848 124.2374 Asia/Manila
17.57472 120.38694 Asia/Manila
10.9 123.07778 Asia/Manila
14.2277 121.3292 Asia/Manila
8.0731 125.9558 Asia/Manila
7.90639 125.09417 Asia/Manila
15.97611 120.57111 Asia/Manila
10.57396 123.3933 Asia/Manila
6.33444 124.95278 Asia/Manila
17.61306 121.72694 Asia/Manila
8.04583 126.06361 Asia/Manila
10.37917 123.64194 Asia/Manila
13.4585 123.6805 Asia/Manila
14.2897 120.7168 Asia/Manila
14.5612 121.2195 Asia/Manila
15.11667 120.61667 Asia/Manila
14.5678 121.1394 Asia/Manila
14.02889 121.59111 Asia/Manila
15.48889 120.59861 Asia/Manila
14.6753 120.9389 Asia/Manila
9.51528 123.15833 Asia/Manila
8.06 123.75 Asia/Manila
9.07833 126.19861 Asia/Manila
14.4968 121.2846 Asia/Manila
14.08627 121.14975 Asia/Manila
11.10944 125.01556 Asia/Manila
10.57222 122.89667 Asia/Manila
14.0925 121.02194 Asia/Manila
10.73722 122.96639 Asia/Manila
10.24472 123.84944 Asia/Manila
15.5883 120.9192 Asia/Manila
8.45611 125.78417 Asia/Manila
16.9355 120.4442 Asia/Manila
8.53889 124.75694 Asia/Manila
9.6475 123.85556 Asia/Manila
13.16667 123.71667 Asia/Manila
6.6925 124.67639 Asia/Manila
17.4189 121.4443 Asia/Manila
13.35861 123.73361 Asia/Manila
13.87992 120.92311 Asia/Manila
9.78389 125.48889 Asia/Manila
6.36667 124.73333 Asia/Manila
14.87999 120.23433 Asia/Manila
10.06306 122.85528 Asia/Manila
12.97389 123.99333 Asia/Manila
16.51918 121.18124 Asia/Manila
17.6528 121.691 Asia/Manila
4.66278 119.39278 Asia/Manila
9.75194 122.40417 Asia/Manila
14.21567 120.97137 Asia/Manila
9.3584 123.285 Asia/Manila
14.9438 120.6229 Asia/Manila
6.22 124.69361 Asia/Manila
13.96444 121.53 Asia/Manila
15.29546 120.64653 Asia/Manila
14.9923 120.7057 Asia/Manila
14.1079 121.14136 Asia/Manila
15.16222 120.5675 Asia/Manila
16.68808 121.5487 Asia/Manila
14.31222 121.11139 Asia/Manila
14.9974 120.6183 Asia/Manila
16.9518 120.4445 Asia/Manila
6.55361 125.47083 Asia/Manila
15.7654 119.9092 Asia/Manila
14.9132 120.5626 Asia/Manila
14.2814 121.4161 Asia/Manila
9.3337 122.8637 Asia/Manila
16.0009 120.4023 Asia/Manila
15.0955 120.767 Asia/Manila
14.9996 120.7808 Asia/Manila
14.3595 121.0473 Asia/Manila
13.7975 121.03417 Asia/Manila
14.06694 121.32583 Asia/Manila
18.1725 120.5953 Asia/Manila
15.0143 120.0803 Asia/Manila
15.42194 120.60056 Asia/Manila
15.1454 120.9789 Asia/Manila
16.88206 121.58694 Asia/Manila
14.6982 121.1236 Asia/Manila
7.5 126 Asia/Manila
14.97418 120.15733 Asia/Manila
15.0401 120.7883 Asia/Manila
15.363 120.9639 Asia/Manila
14.81389 121.04528 Asia/Manila
13.35 123.55 Asia/Manila
12.35275 121.06761 Asia/Manila
15.0809 120.941 Asia/Manila
15.3525 120.8327 Asia/Manila
13.34806 122.52056 Asia/Manila
8.53556 125.95 Asia/Manila
16.61591 120.31663 Asia/Manila
15.0286 120.6898 Asia/Manila
10.1624 123.7076 Asia/Manila
15.3062 120.856 Asia/Manila
15.2462 120.8496 Asia/Manila
14.94 120.14722 Asia/Manila
14.5402 121.3602 Asia/Manila
14.76778 120.54306 Asia/Manila
7.07444 125.70833 Asia/Manila
10.94472 123.42417 Asia/Manila
12.8346 120.769 Asia/Manila
17.11894 121.62014 Asia/Manila
11.58528 122.75111 Asia/Manila
10.3215 119.3451 Asia/Manila
12.57513 122.27081 Asia/Manila
15.7118 121.1061 Asia/Manila
6.95194 121.96361 Asia/Manila
15.6653 120.6406 Asia/Manila
16.78416 121.53504 Asia/Manila
14.6 120.9833 Asia/Manila
15.5512 120.8147 Asia/Manila
7.73028 125.09889 Asia/Manila
10.5203 122.8017 Asia/Manila
14.27917 121.08528 Asia/Manila
14.90167 120.84917 Asia/Manila
9.73917 118.73528 Asia/Manila
14.5833 120.9667 Asia/Manila
15.0711 120.5423 Asia/Manila
6.22167 125.06398 Asia/Manila
13.2923 123.4855 Asia/Manila
14.88722 120.85722 Asia/Manila
15.0466 120.5181 Asia/Manila
10.2708 123.5839 Asia/Manila
13.0357 121.48842 Asia/Manila
14.4854 121.3062 Asia/Manila
13.5572 123.2735 Asia/Manila
14.66 120.56528 Asia/Manila
14.2325 121.3648 Asia/Manila
15.35116 121.00393 Asia/Manila
14.11667 120.96667 Asia/Manila
11.10778 122.64194 Asia/Manila
7.37035 124.26973 Asia/Manila
10.88333 123.36667 Asia/Manila
15.3207 120.8368 Asia/Manila
14.83111 120.78917 Asia/Manila
7.13528 125.88972 Asia/Manila
14.96806 120.95472 Asia/Manila
15.6689 120.5806 Asia/Manila
14.4008 121.4685 Asia/Manila
14.865 120.95722 Asia/Manila
15.1835 120.6455 Asia/Manila
11.25111 125.00639 Asia/Manila
7.30806 125.68417 Asia/Manila
11.1575 124.99083 Asia/Manila
15.54139 121.08611 Asia/Manila
13.975 121.69194 Asia/Manila
7.14167 124.38056 Asia/Manila
14.3647 121.4829 Asia/Manila
10.45 122.81667 Asia/Manila
8.14583 123.84444 Asia/Manila
8.48583 123.80444 Asia/Manila
11.00639 124.6075 Asia/Manila
14.62056 120.58167 Asia/Manila
14.8006 120.5371 Asia/Manila
14.82917 120.28278 Asia/Manila
14.7098 120.9362 Asia/Manila
14.42922 120.87989 Asia/Manila
14.9109 121.0493 Asia/Manila
7.5866 125.8237 Asia/Manila
14.0722 120.6332 Asia/Manila
9.26877 118.4043 Asia/Manila
14.31812 120.76609 Asia/Manila
14.1364 121.4165 Asia/Manila
13.61917 123.18139 Asia/Manila
10.20898 123.758 Asia/Manila
7.60778 125.96639 Asia/Manila
13.4075 123.3724 Asia/Manila
7.8275 123.4782 Asia/Manila
10.60516 123.0417 Asia/Manila
15.71611 120.90306 Asia/Manila
14.67889 120.26611 Asia/Manila
14.5115 121.2393 Asia/Manila
14.7314 121.1417 Asia/Manila
7.81528 126.05444 Asia/Manila
8.0844 123.491 Asia/Manila
10.24498 123.7964 Asia/Manila
7.19083 124.53028 Asia/Manila
14.73694 120.96083 Asia/Manila
15.0646 120.7198 Asia/Manila
14.1093 123.0109 Asia/Manila
14.12861 120.90583 Asia/Manila
14.19111 121.73111 Asia/Manila
6.95508 126.21655 Asia/Manila
15.5363 119.9502 Asia/Manila
12.36667 123.61667 Asia/Manila
14.896 120.7092 Asia/Manila
14.4361 120.4857 Asia/Manila
14.75778 120.94833 Asia/Manila
8.83333 125.11667 Asia/Manila
8 124.28333 Asia/Manila
7.99861 124.29278 Asia/Manila
7.76333 125.00528 Asia/Manila
14.2733 120.7377 Asia/Manila
8.16667 124.21667 Asia/Manila
10.63111 122.97889 Asia/Manila
12.52044 121.43851 Asia/Manila
8.36972 124.86444 Asia/Manila
14.6042 120.9822 Asia/Manila
15.1012 120.5604 Asia/Manila
16.07 120.4025 Asia/Manila
10.32361 123.92222 Asia/Manila
14.5832 121.0409 Asia/Manila
16.8632 120.7829 Asia/Manila
7.215 126.53972 Asia/Manila
10.95803 123.123 Asia/Manila
16.0438 120.4861 Asia/Manila
13.2233 120.596 Asia/Manila
14.23417 121.15806 Asia/Manila
14.04472 121.15861 Asia/Manila
6.543 121.8753 Asia/Manila
6.27917 125.28167 Asia/Manila
14.84194 120.81167 Asia/Manila
6.415 125.61167 Asia/Manila
13.3188 123.7385 Asia/Manila
8.15 125.08333 Asia/Manila
5.96917 125.28944 Asia/Manila
14.7069 121.1306 Asia/Manila
13.93333 121.46667 Asia/Manila
8.1183 123.4455 Asia/Manila
6.76667 125.18333 Asia/Manila
13.6604 123.1869 Asia/Manila
6.86472 124.44167 Asia/Manila
15.2151 120.6596 Asia/Manila
15.2216 120.5736 Asia/Manila
10.13361 124.84472 Asia/Manila
10.48333 122.98333 Asia/Manila
6.89814 126.00961 Asia/Manila
14.2973 121.4598 Asia/Manila
13.93139 121.61722 Asia/Manila
14.11333 121.55694 Asia/Manila
14.9405 120.6011 Asia/Manila
14.1798 121.2234 Asia/Manila
13.88444 122.26417 Asia/Manila
14.78333 121 Asia/Manila
13.93944 121.17389 Asia/Manila
16.02182 120.23194 Asia/Manila
14.56194 120.59833 Asia/Manila
10.3991 123.9992 Asia/Manila
14.1313 121.4362 Asia/Manila
13.2997 123.4386 Asia/Manila
8.94417 125.50194 Asia/Manila
13.13722 123.73444 Asia/Manila
15.5865 121.1834 Asia/Manila
16.455 120.5875 Asia/Manila
10.31028 123.94944 Asia/Manila
15.44125 120.72863 Asia/Manila
12.5698 125.0141 Asia/Manila
18.19889 120.59361 Asia/Manila
7.9745 123.7458 Asia/Manila
6.6525 124.52111 Asia/Manila
10.3239 123.0215 Asia/Manila
10.42333 122.92083 Asia/Manila
14.1532 122.8303 Asia/Manila
6.50306 124.84694 Asia/Manila
7.00833 125.08944 Asia/Manila
14.44426 120.90164 Asia/Manila
11.70611 122.36444 Asia/Manila
9.98889 122.81222 Asia/Manila
7.10667 124.82917 Asia/Manila
14.2906 122.6917 Asia/Manila
6.05222 121.00222 Asia/Manila
8.65417 124.75556 Asia/Manila
14.353 121.3225 Asia/Manila
15.3275 120.9192 Asia/Manila
16.36389 120.67694 Asia/Manila
6.62944 124.605 Asia/Manila
10.2048 122.9888 Asia/Manila
6.7085 121.9711 Asia/Manila
12.7026 124.0362 Asia/Manila
13.425 123.41778 Asia/Manila
7.78444 122.58611 Asia/Manila
14.19528 120.87694 Asia/Manila
14.42972 120.93667 Asia/Manila
10.69694 122.56444 Asia/Manila
8.25 124.4 Asia/Manila
17.14854 121.88924 Asia/Manila
15.3276 119.978 Asia/Manila
10.2706 122.8507 Asia/Manila
10.09889 122.87056 Asia/Manila
14.8314 120.5081 Asia/Manila
14.8321 120.7336 Asia/Manila
14.83861 120.97972 Asia/Manila
13.91972 122.10028 Asia/Manila
16.06667 120.68333 Asia/Manila
15.6605 120.7683 Asia/Manila
10.1214 123.2742 Asia/Manila
14.83333 120.88333 Asia/Manila
13.6978 123.4892 Asia/Manila
5.82417 125.20333 Asia/Manila
15.6065 120.5978 Asia/Manila
14.38694 120.88167 Asia/Manila
15.35076 121.04773 Asia/Manila
15.6025 121.0515 Asia/Manila
15.3072 120.9464 Asia/Manila
10.84028 123.49917 Asia/Manila
10.87754 122.9677 Asia/Manila
9.31028 123.30806 Asia/Manila
7.68 125.005 Asia/Manila
16.00292 120.21154 Asia/Manila
7.84083 125.04444 Asia/Manila
8.58944 123.34139 Asia/Manila
14.8732 120.4651 Asia/Manila
6.74972 125.35722 Asia/Manila
15.03333 120.7 Asia/Manila
7.07306 125.61278 Asia/Manila
14.32944 120.93667 Asia/Manila
8.64639 123.42083 Asia/Manila
10.52083 124.02722 Asia/Manila
9.56972 123.77111 Asia/Manila
14.1122 122.9553 Asia/Manila
13.902 121.0521 Asia/Manila
7.22361 124.24639 Asia/Manila
10.25194 123.94944 Asia/Manila
10.3766 123.9573 Asia/Manila
15.32546 120.65723 Asia/Manila
13.92667 121.49333 Asia/Manila
10.455 124.0106 Asia/Manila
7.67306 126.08889 Asia/Manila
10.31672 123.89071 Asia/Manila
14.4825 120.91694 Asia/Manila
11.77528 124.88611 Asia/Manila
12.4989 124.6377 Asia/Manila
13.59194 122.32306 Asia/Manila
14.93039 120.20143 Asia/Manila
14.3132 121.0576 Asia/Manila
10.59306 124.01778 Asia/Manila
11.29917 124.67556 Asia/Manila
14.486 121.2289 Asia/Manila
10.1061 123.6402 Asia/Manila
15.32936 120.59123 Asia/Manila
10.38639 123.19639 Asia/Manila
13.93111 121.42333 Asia/Manila
15.0956 120.8267 Asia/Manila
15.6866 120.4128 Asia/Manila
14.91639 120.76583 Asia/Manila
14.6228 121.0897 Asia/Manila
12.06722 124.60417 Asia/Manila
14.1447 121.3152 Asia/Manila
13.9575 122.2875 Asia/Manila
13.83224 120.6322 Asia/Manila
16.0111 120.36 Asia/Manila
13.4117 121.1803 Asia/Manila
14.21167 121.16528 Asia/Manila
13.93244 120.81327 Asia/Manila
13.7068 123.2087 Asia/Manila
14.5786 121.1222 Asia/Manila
8.48222 124.64722 Asia/Manila
10.95056 123.28556 Asia/Manila
15.2488 120.8548 Asia/Manila
7.40722 125.73306 Asia/Manila
15.48694 120.9675 Asia/Manila
17.42782 121.76955 Asia/Manila
9.12361 125.53444 Asia/Manila
14.95806 120.91778 Asia/Manila
8.18778 124.17694 Asia/Manila
15.73333 120.58333 Asia/Manila
8.16722 125.99083 Asia/Manila
6.72028 124.80194 Asia/Manila
15.08833 120.66722 Asia/Manila
12.67139 123.875 Asia/Manila
14.79278 120.87889 Asia/Manila
13.4322 123.517 Asia/Manila
8.50833 124.75944 Asia/Manila
8.97694 125.40889 Asia/Manila
7.20417 124.43972 Asia/Manila
15.2896 120.0245 Asia/Manila
11.60806 125.43194 Asia/Manila
5.02917 119.77306 Asia/Manila
15.6321 121.1448 Asia/Manila
11.0517 124.0055 Asia/Manila
14.79833 120.92611 Asia/Manila
8.21528 126.31639 Asia/Manila
10.77306 122.98278 Asia/Manila
16.0307 120.2712 Asia/Manila
14.4646 121.1929 Asia/Manila
10.1948 122.8581 Asia/Manila
13.86278 121.48944 Asia/Manila
8.75611 125.7675 Asia/Manila
16.4812 121.1497 Asia/Manila
10.6785 124.8006 Asia/Manila
9.36361 122.80111 Asia/Manila
15.8127 120.4557 Asia/Manila
14.1831 121.2848 Asia/Manila
16.53083 120.33306 Asia/Manila
13.7917 121.0085 Asia/Manila
13.3528 123.3677 Asia/Manila
13.75944 121.06 Asia/Manila
18.0554 120.56489 Asia/Manila
14.5234 121.2651 Asia/Manila
11.1683 123.7223 Asia/Manila
6.78611 125.21333 Asia/Manila
6.42389 124.77833 Asia/Manila
13.85 121.2 Asia/Manila
16.3865 121.1066 Asia/Manila
14.95472 120.89694 Asia/Manila
13.93714 120.73224 Asia/Manila
14.67611 120.53611 Asia/Manila
10.5039 123.7156 Asia/Manila
14.81667 120.86667 Asia/Manila
9.59111 123.12278 Asia/Manila
8.60722 125.91444 Asia/Manila
16.41639 120.59306 Asia/Manila
10.53333 122.83333 Asia/Manila
16.50588 121.19014 Asia/Manila
14.45903 120.92903 Asia/Manila
10.66667 122.95 Asia/Manila
15.08333 120.51667 Asia/Manila
13.4549 123.3653 Asia/Manila
13.99583 121.905 Asia/Manila
9.5506 122.5164 Asia/Manila
16.394 120.3545 Asia/Manila
15.1505 120.7697 Asia/Manila
18.3566 121.6406 Asia/Manila
14.95333 120.77 Asia/Manila
14.58639 121.17528 Asia/Manila
14.5266 121.1536 Asia/Manila
15.15 120.58333 Asia/Manila
14.9285 121.03 Asia/Manila
14.17056 120.92361 Asia/Manila
16.77936 121.69734 Asia/Manila
15.5 120.843 Asia/Manila
14.06391 121.24649 Asia/Manila
6.10179 125.29048 Asia/Manila
16.08333 120.1 Asia/Manila
10.747 125.0107 Asia/Manila
14.72222 120.53833 Asia/Manila
14.6627 121.0328 Asia/Manila
7.16083 124.475 Asia/Manila
24.38953 67.99234 Asia/Karachi
33.85782 73.76043 Asia/Karachi
27.59102 68.61786 Asia/Karachi
27.52948 68.7617 Asia/Karachi
31.34111 69.44806 Asia/Karachi
34.06028 72.46778 Asia/Karachi
28.81239 70.52405 Asia/Karachi
32.34631 74.89987 Asia/Karachi
29.12122 71.74459 Asia/Karachi
32.44583 74.11594 Asia/Karachi
31.55 73.83333 Asia/Karachi
27.44808 67.79776 Asia/Karachi
30.03333 72.35 Asia/Karachi
34.18799 71.76286 Asia/Karachi
25.80722 66.62194 Asia/Karachi
28.17943 68.04454 Asia/Karachi
25.36157 69.73624 Asia/Karachi
28.16524 69.7289 Asia/Karachi
26.00228 63.04399 Asia/Karachi
34.07041 72.62358 Asia/Karachi
30.97433 72.48269 Asia/Karachi
28.24034 68.77588 Asia/Karachi
24.74745 67.92353 Asia/Karachi
26.94423 68.11602 Asia/Karachi
30.70484 70.65051 Asia/Karachi
32.22168 70.37928 Asia/Karachi
34.30056 71.65361 Asia/Karachi
25.12301 68.53577 Asia/Karachi
25.42718 68.53619 Asia/Karachi
25.46263 68.71923 Asia/Karachi
25.76374 68.66098 Asia/Karachi
31.03333 73.13333 Asia/Karachi
24.88333 68.81667 Asia/Karachi
30.52708 72.24011 Asia/Karachi
32.92973 72.41583 Asia/Karachi
35.47908 72.24385 Asia/Karachi
34.11988 72.46987 Asia/Karachi
27.70516 68.85738 Asia/Karachi
31.86388 73.51122 Asia/Karachi
32.46162 74.18234 Asia/Karachi
27.03333 67.85 Asia/Karachi
26.03031 68.80898 Asia/Karachi
31.82611 72.53984 Asia/Karachi
29.5448 67.8764 Asia/Karachi
32.5101 74.54313 Asia/Karachi
29.87967 71.29313 Asia/Karachi
31.90991 70.87741 Asia/Karachi
27.95706 68.63789 Asia/Karachi
31.71306 73.97833 Asia/Karachi
31.46333 74.1 Asia/Karachi
32.26232 75.16682 Asia/Karachi
29.57646 71.02116 Asia/Karachi
26.15497 68.65093 Asia/Karachi
25.92406 68.61999 Asia/Karachi
27.84771 67.9055 Asia/Karachi
34.2139 71.55523 Asia/Karachi
26.42794 67.863 Asia/Karachi
32.08361 72.67111 Asia/Karachi
30.59489 71.96977 Asia/Karachi
32.82591 70.78136 Asia/Karachi
32.90122 73.75591 Asia/Karachi
31.71667 73.38333 Asia/Karachi
26.04656 68.9481 Asia/Karachi
32.47455 74.35209 Asia/Karachi
26.13839 68.27361 Asia/Karachi
31.97446 72.33087 Asia/Karachi
30.66667 73.1 Asia/Karachi
28.30623 70.13065 Asia/Karachi
27.69223 68.89506 Asia/Karachi
30.88333 73.6 Asia/Karachi
33.6007 73.0679 Asia/Karachi
27.80296 68.28708 Asia/Karachi
27.28792 68.50505 Asia/Karachi
29.10351 70.32504 Asia/Karachi
31.22083 74.25194 Asia/Karachi
31.25444 74.21778 Asia/Karachi
27.18333 67.95 Asia/Karachi
30.199 67.00971 Asia/Karachi
27.58676 68.00103 Asia/Karachi
30.29238 71.67117 Asia/Karachi
30.58028 66.99611 Asia/Karachi
30.76667 72.43333 Asia/Karachi
33.24024 72.26609 Asia/Karachi
31.89781 73.27302 Asia/Karachi
32.58789 73.04564 Asia/Karachi
32.42973 73.57745 Asia/Karachi
34.008 71.57849 Asia/Karachi
31.01667 73.85 Asia/Karachi
32.26413 74.66342 Asia/Karachi
25.26311 63.47103 Asia/Karachi
27.85685 69.11156 Asia/Karachi
30.34104 73.38664 Asia/Karachi
32.10384 70.97235 Asia/Karachi
26.77475 68.30197 Asia/Karachi
34.01056 71.79889 Asia/Karachi
30.80806 73.44583 Asia/Karachi
29.55423 66.02148 Asia/Karachi
26.24833 68.40955 Asia/Karachi
26.84238 68.12298 Asia/Karachi
31.96284 73.97225 Asia/Karachi
27.66477 68.36217 Asia/Karachi
27.38091 67.91678 Asia/Karachi
32.1 74.88333 Asia/Karachi
31.9132 74.51756 Asia/Karachi
24.85869 69.40238 Asia/Karachi
31.4475 73.69722 Asia/Karachi
30.07537 71.19213 Asia/Karachi
34.37 73.47111 Asia/Karachi
30.89222 73.49889 Asia/Karachi
33.90727 73.39295 Asia/Karachi
31.8025 74.26167 Asia/Karachi
30.19556 71.47528 Asia/Karachi
26.66459 68.00156 Asia/Karachi
24.73183 69.79811 Asia/Karachi
32.24861 72.11083 Asia/Karachi
28.02271 69.54886 Asia/Karachi
25.5251 69.0159 Asia/Karachi
34.77584 72.36249 Asia/Karachi
30.163 73.56854 Asia/Karachi
32.5741 71.52639 Asia/Karachi
30.44004 72.35431 Asia/Karachi
28.1085 68.0254 Asia/Karachi
27.18067 67.82288 Asia/Karachi
25.04312 68.65568 Asia/Karachi
25.59609 68.44666 Asia/Karachi
29.79936 66.84505 Asia/Karachi
34.20195 72.05254 Asia/Karachi
34.33333 73.2 Asia/Karachi
31.89306 72.38167 Asia/Karachi
32.58339 73.48432 Asia/Karachi
31.58417 73.68778 Asia/Karachi
30.83333 72.8 Asia/Karachi
32.55362 73.21234 Asia/Karachi
29.80028 72.17583 Asia/Karachi
29.86371 67.32978 Asia/Karachi
30.37053 68.59798 Asia/Karachi
29.54051 71.63357 Asia/Karachi
30.96128 70.93904 Asia/Karachi
27.55508 68.21414 Asia/Karachi
31.82414 72.79971 Asia/Karachi
32.70235 73.96 Asia/Karachi
32.60795 70.91142 Asia/Karachi
31.54972 74.34361 Asia/Karachi
32.15656 74.11701 Asia/Karachi
33.38325 71.33858 Asia/Karachi
25.17847 69.56913 Asia/Karachi
32.52848 73.97368 Asia/Karachi
32.45872 71.47754 Asia/Karachi
31.92861 70.45917 Asia/Karachi
28.5522 70.46837 Asia/Karachi
25.366 68.31216 Asia/Karachi
31.1725 74.09972 Asia/Karachi
32.18954 73.02798 Asia/Karachi
30.19438 66.99299 Asia/Karachi
32.5883 74.49321 Asia/Karachi
33.51667 73.91667 Asia/Karachi
32.33298 74.54798 Asia/Karachi
27.34272 68.70834 Asia/Karachi
30.4692 70.96714 Asia/Karachi
33.58196 71.44929 Asia/Karachi
32.29667 72.3525 Asia/Karachi
31.50056 73.26667 Asia/Karachi
25.82847 69.37719 Asia/Karachi
32.64793 73.01058 Asia/Karachi
32.81493 73.86359 Asia/Karachi
28.58333 65.41667 Asia/Karachi
28.64534 70.6567 Asia/Karachi
27.84358 69.41373 Asia/Karachi
29.9173 71.16184 Asia/Karachi
31.83167 73.62306 Asia/Karachi
34.06 72.88944 Asia/Karachi
27.09359 67.73366 Asia/Karachi
29.58111 72.23639 Asia/Karachi
28.06526 69.70076 Asia/Karachi
31.11556 74.44667 Asia/Karachi
28.43333 69.58333 Asia/Karachi
31.22741 70.94923 Asia/Karachi
24.9056 67.0822 Asia/Karachi
30.76629 74.12311 Asia/Karachi
27.06011 68.20909 Asia/Karachi
28.24396 69.18235 Asia/Karachi
33.74734 73.51107 Asia/Karachi
31.97444 74.22444 Asia/Karachi
30.43333 73.05 Asia/Karachi
32.84389 71.36694 Asia/Karachi
30.73333 72.65 Asia/Karachi
32.15639 71.25944 Asia/Karachi
32.78333 72.7 Asia/Karachi
31.97718 73.60259 Asia/Karachi
29.02252 66.5916 Asia/Karachi
32.965 71.55697 Asia/Karachi
33.5913 73.38676 Asia/Karachi
29.62382 71.91673 Asia/Karachi
31.36917 74.36528 Asia/Karachi
30.40677 71.8667 Asia/Karachi
26.69206 67.6133 Asia/Karachi
25.04992 61.7468 Asia/Karachi
31.56781 73.18347 Asia/Karachi
25.95507 68.88821 Asia/Karachi
32.93313 73.72637 Asia/Karachi
32.36139 72.62278 Asia/Karachi
31.27154 72.32842 Asia/Karachi
32.2898 72.2813 Asia/Karachi
29.51654 70.85032 Asia/Karachi
31.33333 73.43333 Asia/Karachi
33.42985 72.01928 Asia/Karachi
29.64414 70.59544 Asia/Karachi
29.50687 71.21917 Asia/Karachi
32.63744 74.21829 Asia/Karachi
31.80552 72.27711 Asia/Karachi
28.281 68.43876 Asia/Karachi
33.72148 73.04329 Asia/Karachi
25.39242 68.37366 Asia/Karachi
30.74179 73.82358 Asia/Karachi
27.21088 68.41598 Asia/Karachi
33.90914 72.49212 Asia/Karachi
34.0536 73.16571 Asia/Karachi
30.45016 73.69888 Asia/Karachi
29.71222 72.55528 Asia/Karachi
30.43333 72.7 Asia/Karachi
29.61333 73.13889 Asia/Karachi
32.27917 71.55389 Asia/Karachi
33.99997 72.93409 Asia/Karachi
33.53108 71.05934 Asia/Karachi
25.81308 68.42273 Asia/Karachi
32.06786 73.68545 Asia/Karachi
32.64132 74.56904 Asia/Karachi
25.12163 62.32541 Asia/Karachi
32.57284 74.07897 Asia/Karachi
32.16167 74.18831 Asia/Karachi
33.25579 73.30255 Asia/Karachi
31.14874 72.68656 Asia/Karachi
28.00604 69.31608 Asia/Karachi
28.13906 69.08305 Asia/Karachi
24.74241 67.58565 Asia/Karachi
30.83388 71.90529 Asia/Karachi
27.35145 68.52051 Asia/Karachi
29.1925 72.85361 Asia/Karachi
32.18333 75.06667 Asia/Karachi
31.17959 71.20946 Asia/Karachi
31.88722 72.4144 Asia/Karachi
29.46631 73.03403 Asia/Karachi
31.41667 73.08333 Asia/Karachi
32.04219 74.25944 Asia/Karachi
29.7995 71.71958 Asia/Karachi
29.74931 73.24324 Asia/Karachi
31.83587 71.43625 Asia/Karachi
35.20583 71.87556 Asia/Karachi
30.67091 73.65292 Asia/Karachi
32.63976 73.72202 Asia/Karachi
31.21667 72.98333 Asia/Karachi
25.15579 69.11129 Asia/Karachi
25.5 69.56667 Asia/Karachi
29.57991 71.75213 Asia/Karachi
31.83269 70.9024 Asia/Karachi
30.05614 70.63477 Asia/Karachi
29.03069 69.15099 Asia/Karachi
26.45834 68.31915 Asia/Karachi
32.87498 71.57012 Asia/Karachi
32.32426 74.34974 Asia/Karachi
31.79123 71.10394 Asia/Karachi
29.55767 70.37615 Asia/Karachi
26.73286 67.77631 Asia/Karachi
29.47527 67.65135 Asia/Karachi
30.96397 73.98029 Asia/Karachi
31.75 73.8 Asia/Karachi
25.51667 69.76667 Asia/Karachi
32.72006 72.98486 Asia/Karachi
29.8 72.86667 Asia/Karachi
31.72 72.97889 Asia/Karachi
30.53333 72.7 Asia/Karachi
32.34692 74.70612 Asia/Karachi
34.14345 71.73173 Asia/Karachi
30.9236 66.45116 Asia/Karachi
32.93338 72.85853 Asia/Karachi
30.75 73.03333 Asia/Karachi
30.16667 72.65 Asia/Karachi
32.42888 74.3633 Asia/Karachi
25.80437 68.49265 Asia/Karachi
32.97568 74.07926 Asia/Karachi
32.48318 72.9097 Asia/Karachi
31.56918 72.64899 Asia/Karachi
26.55831 67.72139 Asia/Karachi
32.27204 72.90425 Asia/Karachi
31.62525 71.06574 Asia/Karachi
31.2 73.95 Asia/Karachi
26.2271 66.3111 Asia/Karachi
34.61667 71.97139 Asia/Karachi
30.58318 73.829 Asia/Karachi
32.98541 70.6027 Asia/Karachi
29.4 71.68333 Asia/Karachi
29.99866 73.2536 Asia/Karachi
24.65599 68.83841 Asia/Karachi
31.99283 74.66834 Asia/Karachi
33.77311 72.3741 Asia/Karachi
30.29399 73.07354 Asia/Karachi
34.00583 71.93 Asia/Karachi
29.38464 70.91157 Asia/Karachi
34.00361 72.12611 Asia/Karachi
29.14309 71.25976 Asia/Karachi
34.14685 73.21449 Asia/Karachi
30.55083 73.39083 Asia/Karachi
33.99667 72.01306 Asia/Karachi
30.67539 71.74828 Asia/Karachi
27.34167 68.03194 Asia/Karachi
25.13111 68.86759 Asia/Karachi
27.2127 68.46883 Asia/Karachi
34.06186 71.99166 Asia/Karachi
32.55492 73.2122 Asia/Karachi
52.0488 20.44599 Europe/Warsaw
52.30376 21.16018 Europe/Warsaw
50.72314 23.25196 Europe/Warsaw
52.9855 22.24319 Europe/Warsaw
52.29271 21.10539 Europe/Warsaw
52.59278 21.4584 Europe/Warsaw
52.34006 21.24207 Europe/Warsaw
49.98738 20.06473 Europe/Warsaw
52.22977 21.01178 Europe/Warsaw
51.53131 20.00855 Europe/Warsaw
50.44767 23.41616 Europe/Warsaw
50.01381 20.98698 Europe/Warsaw
50.57304 21.67937 Europe/Warsaw
53.56259 20.98747 Europe/Warsaw
51.21898 22.69621 Europe/Warsaw
54.11175 22.93087 Europe/Warsaw
52.25221 21.26902 Europe/Warsaw
50.56307 21.16593 Europe/Warsaw
51.0374 21.07126 Europe/Warsaw
50.58286 22.05334 Europe/Warsaw
52.40677 22.25307 Europe/Warsaw
53.40715 23.50228 Europe/Warsaw
52.22944 20.23838 Europe/Warsaw
51.95485 20.15837 Europe/Warsaw
51.11311 20.87162 Europe/Warsaw
52.42719 22.86231 Europe/Warsaw
52.16772 22.29006 Europe/Warsaw
49.55573 22.2056 Europe/Warsaw
50.68265 21.74898 Europe/Warsaw
50.04132 21.99901 Europe/Warsaw
50.05229 21.60891 Europe/Warsaw
51.76437 20.25493 Europe/Warsaw
51.78333 22.61667 Europe/Warsaw
51.40253 21.14714 Europe/Warsaw
52.7025 21.08276 Europe/Warsaw
51.41655 21.96939 Europe/Warsaw
50.05912 22.49408 Europe/Warsaw
49.78498 22.76728 Europe/Warsaw
53.01907 20.88029 Europe/Warsaw
52.17072 20.81214 Europe/Warsaw
52.62348 20.37552 Europe/Warsaw
53.62744 21.81253 Europe/Warsaw
51.47604 21.44995 Europe/Warsaw
52.18435 20.83952 Europe/Warsaw
52.0814 21.02397 Europe/Warsaw
52.10577 21.26129 Europe/Warsaw
52.80245 21.89507 Europe/Warsaw
50.92936 21.38525 Europe/Warsaw
53.08621 21.57566 Europe/Warsaw
51.37569 20.27827 Europe/Warsaw
53.77995 20.49416 Europe/Warsaw
54.03374 22.50704 Europe/Warsaw
49.47783 20.03228 Europe/Warsaw
49.62177 20.69705 Europe/Warsaw
52.43022 20.71652 Europe/Warsaw
50.51987 22.13968 Europe/Warsaw
53.86437 21.30507 Europe/Warsaw
53.11278 20.3841 Europe/Warsaw
52.17935 21.57251 Europe/Warsaw
52.11879 20.67155 Europe/Warsaw
50.28709 21.4239 Europe/Warsaw
51.9864 22.78248 Europe/Warsaw
52.32065 21.10474 Europe/Warsaw
51.929 22.37956 Europe/Warsaw
51.25 22.56667 Europe/Warsaw
51.46026 22.60952 Europe/Warsaw
53.17806 22.05935 Europe/Warsaw
52.33413 20.88602 Europe/Warsaw
54.12588 20.57954 Europe/Warsaw
52.40149 20.92664 Europe/Warsaw
51.30121 22.88135 Europe/Warsaw
52.9911 22.88422 Europe/Warsaw
50.06871 22.22912 Europe/Warsaw
49.68866 21.77058 Europe/Warsaw
50.98464 23.1742 Europe/Warsaw
50.9236 22.22706 Europe/Warsaw
51.58294 21.54779 Europe/Warsaw
52.0938 21.11761 Europe/Warsaw
51.19166 20.40607 Europe/Warsaw
52.33953 21.19589 Europe/Warsaw
50.87033 20.62752 Europe/Warsaw
54.07676 21.37527 Europe/Warsaw
52.13012 21.08148 Europe/Warsaw
52.13707 21.23589 Europe/Warsaw
50.63945 20.30454 Europe/Warsaw
49.74506 21.47252 Europe/Warsaw
50.01623 22.67776 Europe/Warsaw
50.80502 23.89251 Europe/Warsaw
52.74328 23.58122 Europe/Warsaw
52.10387 20.6337 Europe/Warsaw
53.64728 22.45537 Europe/Warsaw
49.65563 21.16035 Europe/Warsaw
54.03811 21.76441 Europe/Warsaw
54.08134 21.49551 Europe/Warsaw
51.89747 21.61466 Europe/Warsaw
53.82824 22.36469 Europe/Warsaw
53.23958 20.17004 Europe/Warsaw
51.55912 21.84829 Europe/Warsaw
50.05146 21.41141 Europe/Warsaw
52.88141 20.61996 Europe/Warsaw
51.14312 23.4716 Europe/Warsaw
50.47078 20.71884 Europe/Warsaw
49.96911 20.60606 Europe/Warsaw
49.96905 20.43028 Europe/Warsaw
50.54114 22.72204 Europe/Warsaw
52.76512 23.18647 Europe/Warsaw
53.13333 23.15 Europe/Warsaw
52.03238 23.11652 Europe/Warsaw
54.25354 20.80819 Europe/Warsaw
53.84321 22.97979 Europe/Warsaw
49.68529 19.19243 Europe/Warsaw
50.04523 18.70062 Europe/Warsaw
53.36346 17.04082 Europe/Warsaw
51.12637 15.91979 Europe/Warsaw
51.93548 15.50643 Europe/Warsaw
51.14942 15.00835 Europe/Warsaw
51.85561 19.40623 Europe/Warsaw
51.59915 18.93974 Europe/Warsaw
50.48766 19.41679 Europe/Warsaw
51.64205 15.13727 Europe/Warsaw
49.29899 19.94885 Europe/Warsaw
51.61759 15.31486 Europe/Warsaw
50.32492 18.78576 Europe/Warsaw
50.58969 16.81239 Europe/Warsaw
52.32512 17.56519 Europe/Warsaw
51.1 17.03333 Europe/Warsaw
50.00377 18.47205 Europe/Warsaw
52.64817 19.0678 Europe/Warsaw
51.22097 18.56964 Europe/Warsaw
54.60568 18.23559 Europe/Warsaw
53.27787 16.47122 Europe/Warsaw
50.77141 16.28432 Europe/Warsaw
52.80842 17.19961 Europe/Warsaw
49.88335 19.49292 Europe/Warsaw
49.72153 18.80198 Europe/Warsaw
54.58048 16.86194 Europe/Warsaw
50.13717 18.96641 Europe/Warsaw
52.01548 18.50055 Europe/Warsaw
50.15931 19.46966 Europe/Warsaw
53.04063 16.45628 Europe/Warsaw
53.01375 18.59814 Europe/Warsaw
54.09242 18.77787 Europe/Warsaw
50.44548 18.86147 Europe/Warsaw
53.70791 16.69937 Europe/Warsaw
53.42894 14.55302 Europe/Warsaw
52.61201 16.57794 Europe/Warsaw
53.91053 14.24712 Europe/Warsaw
50.29636 18.91726 Europe/Warsaw
52.24751 15.53355 Europe/Warsaw
53.40953 18.44742 Europe/Warsaw
50.85976 16.3282 Europe/Warsaw
53.77464 15.77671 Europe/Warsaw
50.84378 16.48859 Europe/Warsaw
52.41289 17.08503 Europe/Warsaw
52.08362 15.62513 Europe/Warsaw
50.5107 18.30056 Europe/Warsaw
50.96264 16.35006 Europe/Warsaw
53.96396 18.52638 Europe/Warsaw
53.33672 15.0499 Europe/Warsaw
52.22843 17.27617 Europe/Warsaw
52.08868 17.01508 Europe/Warsaw
50.28682 19.10385 Europe/Warsaw
54.4418 18.56003 Europe/Warsaw
53.08371 18.22572 Europe/Warsaw
54.46405 17.02872 Europe/Warsaw
52.35088 14.56065 Europe/Warsaw
49.97524 19.82869 Europe/Warsaw
52.8568 19.66913 Europe/Warsaw
51.59584 18.73023 Europe/Warsaw
50.32738 19.02901 Europe/Warsaw
53.06603 19.40941 Europe/Warsaw
50.05857 18.41703 Europe/Warsaw
50.09713 18.54179 Europe/Warsaw
54.57092 18.38802 Europe/Warsaw
50.2584 18.85632 Europe/Warsaw
54.60528 18.34717 Europe/Warsaw
51.60946 16.85852 Europe/Warsaw
50.40026 18.90232 Europe/Warsaw
51.06713 19.44477 Europe/Warsaw
50.0502 18.47626 Europe/Warsaw
50.09195 18.21928 Europe/Warsaw
50.4 18.63333 Europe/Warsaw
49.98037 18.95382 Europe/Warsaw
54.26217 18.63625 Europe/Warsaw
50.32124 17.57461 Europe/Warsaw
52.40692 16.92993 Europe/Warsaw
51.50391 16.07261 Europe/Warsaw
53.55214 14.57182 Europe/Warsaw
52.54682 19.70638 Europe/Warsaw
51.89636 17.78549 Europe/Warsaw
51.40547 19.70321 Europe/Warsaw
53.15145 16.73782 Europe/Warsaw
50.38017 18.92653 Europe/Warsaw
51.66446 19.35473 Europe/Warsaw
51.96336 19.29139 Europe/Warsaw
50.03437 19.21037 Europe/Warsaw
51.65501 17.80686 Europe/Warsaw
53.69671 19.96486 Europe/Warsaw
50.15595 18.77924 Europe/Warsaw
50.66667 17.95 Europe/Warsaw
50.2813 19.56503 Europe/Warsaw
51.21338 17.38986 Europe/Warsaw
50.9466 17.2926 Europe/Warsaw
52.64739 16.81406 Europe/Warsaw
50.4575 17.31308 Europe/Warsaw
50.47379 17.33437 Europe/Warsaw
52.3195 16.12844 Europe/Warsaw
53.67437 15.1163 Europe/Warsaw
51.80333 15.71702 Europe/Warsaw
50.58008 16.50164 Europe/Warsaw
51.07592 17.72284 Europe/Warsaw
53.14214 17.60181 Europe/Warsaw
50.5752 19.32461 Europe/Warsaw
50.20745 19.16668 Europe/Warsaw
49.83383 19.9383 Europe/Warsaw
50.17103 18.9041 Europe/Warsaw
52.44461 15.57801 Europe/Warsaw
54.03591 19.0266 Europe/Warsaw
52.34705 16.89267 Europe/Warsaw
50.66897 18.6844 Europe/Warsaw
51.40089 16.20149 Europe/Warsaw
51.12014 15.28768 Europe/Warsaw
52.10714 19.94525 Europe/Warsaw
51.75 19.46667 Europe/Warsaw
50.10396 19.31568 Europe/Warsaw
51.84034 16.57494 Europe/Warsaw
51.21006 16.1619 Europe/Warsaw
50.14264 19.13149 Europe/Warsaw
52.05959 19.19972 Europe/Warsaw
54.53921 17.75012 Europe/Warsaw
50.14952 18.84215 Europe/Warsaw
51.59056 19.13278 Europe/Warsaw
53.72495 18.93114 Europe/Warsaw
52.23064 19.36409 Europe/Warsaw
51.69868 17.43738 Europe/Warsaw
50.47515 17.96539 Europe/Warsaw
50.08333 19.91667 Europe/Warsaw
54.19438 16.17222 Europe/Warsaw
52.58713 14.64953 Europe/Warsaw
54.12226 17.98119 Europe/Warsaw
52.08829 16.64866 Europe/Warsaw
51.74776 19.32564 Europe/Warsaw
52.22338 18.25121 Europe/Warsaw
54.17565 15.58342 Europe/Warsaw
52.20024 18.63865 Europe/Warsaw
50.21971 18.65067 Europe/Warsaw
50.97281 18.21816 Europe/Warsaw
50.43488 16.66145 Europe/Warsaw
49.88214 19.22333 Europe/Warsaw
50.34984 18.22606 Europe/Warsaw
50.25842 19.02754 Europe/Warsaw
54.33424 18.19735 Europe/Warsaw
50.78314 16.03037 Europe/Warsaw
51.76109 18.09102 Europe/Warsaw
50.89973 15.72899 Europe/Warsaw
51.02102 17.32095 Europe/Warsaw
50.20528 19.27498 Europe/Warsaw
51.05132 16.19347 Europe/Warsaw
49.95542 18.57479 Europe/Warsaw
51.97266 17.50256 Europe/Warsaw
52.79886 18.26388 Europe/Warsaw
53.59601 19.56849 Europe/Warsaw
51.94956 14.72837 Europe/Warsaw
53.25243 14.48831 Europe/Warsaw
53.9165 15.20027 Europe/Warsaw
53.48411 18.75366 Europe/Warsaw
52.42938 19.46194 Europe/Warsaw
51.88247 17.01226 Europe/Warsaw
52.73679 15.22878 Europe/Warsaw
53.56392 14.82854 Europe/Warsaw
52.53481 17.58259 Europe/Warsaw
50.31505 17.38355 Europe/Warsaw
51.96463 19.71565 Europe/Warsaw
51.66361 16.0845 Europe/Warsaw
50.29761 18.67658 Europe/Warsaw
54.51889 18.53188 Europe/Warsaw
54.35205 18.64637 Europe/Warsaw
53.14821 18.17036 Europe/Warsaw
54.1522 19.40884 Europe/Warsaw
50.7282 16.65141 Europe/Warsaw
50.31818 19.2374 Europe/Warsaw
50.79646 19.12409 Europe/Warsaw
50.15007 18.67762 Europe/Warsaw
50.31542 19.07824 Europe/Warsaw
49.91342 19.00479 Europe/Warsaw
49.75133 18.63213 Europe/Warsaw
50.13546 19.40203 Europe/Warsaw
53.16905 15.42054 Europe/Warsaw
50.30582 18.9742 Europe/Warsaw
53.69554 17.55701 Europe/Warsaw
52.99505 16.9198 Europe/Warsaw
53.18463 18.60466 Europe/Warsaw
53.34855 18.4251 Europe/Warsaw
54.17057 17.49187 Europe/Warsaw
50.34802 18.93282 Europe/Warsaw
53.1235 18.00762 Europe/Warsaw
50.86079 17.4674 Europe/Warsaw
53.25967 19.39653 Europe/Warsaw
54.37971 19.81959 Europe/Warsaw
51.26418 15.5697 Europe/Warsaw
50.75514 16.20494 Europe/Warsaw
50.90747 14.95634 Europe/Warsaw
50.09 19.09291 Europe/Warsaw
49.82245 19.04686 Europe/Warsaw
50.69075 16.623 Europe/Warsaw
54.00696 15.98751 Europe/Warsaw
51.36883 19.35671 Europe/Warsaw
50.32607 19.12565 Europe/Warsaw
49.85497 19.33834 Europe/Warsaw
51.81965 19.30384 Europe/Warsaw
51.02134 17.31649 Europe/Warsaw
46.76544 -56.16949 America/Miquelon
-25.06597 -130.10147 Pacific/Pitcairn
18.42745 -67.15407 America/Puerto_Rico
18.47244 -66.71573 America/Puerto_Rico
18.4505 -66.53851 America/Puerto_Rico
18.39856 -66.15572 America/Puerto_Rico
18.23412 -66.0485 America/Puerto_Rico
18.40411 -66.20878 America/Puerto_Rico
18.38078 -65.95739 America/Puerto_Rico
18.44134 -66.11822 America/Puerto_Rico
18.11191 -66.166 America/Puerto_Rico
18.32579 -65.65238 America/Puerto_Rico
17.98413 -66.11378 America/Puerto_Rico
18.35745 -66.111 America/Puerto_Rico
18.14968 -65.82738 America/Puerto_Rico
18.44995 -66.18156 America/Puerto_Rico
18.42745 -66.49212 America/Puerto_Rico
18.20107 -67.13962 America/Puerto_Rico
18.01108 -66.61406 America/Puerto_Rico
18.46633 -66.10572 America/Puerto_Rico
18.35467 -66.00739 America/Puerto_Rico
18.44439 -66.38767 America/Puerto_Rico
18.03496 -66.8499 America/Puerto_Rico
31.287 34.25952 Asia/Gaza
31.44861 34.3925 Asia/Gaza
31.34018 34.30627 Asia/Gaza
31.52722 34.48634 Asia/Gaza
31.5 34.46667 Asia/Gaza
31.41655 34.34525 Asia/Gaza
31.5464 34.49514 Asia/Gaza
31.5353 34.53579 Asia/Gaza
31.34337 34.32337 Asia/Gaza
31.43944 34.40306 Asia/Gaza
31.31913 34.34005 Asia/Gaza
31.45 35.08333 Asia/Hebron
32.31037 35.02863 Asia/Hebron
32.32091 35.36989 Asia/Hebron
31.57855 35.14017 Asia/Hebron
31.89502 35.18459 Asia/Hebron
32.18924 34.97034 Asia/Hebron
32.41255 35.28823 Asia/Hebron
32.22111 35.25444 Asia/Hebron
32.45884 35.2951 Asia/Hebron
31.55874 34.97436 Asia/Hebron
31.57858 35.10544 Asia/Hebron
31.507 35.02652 Asia/Hebron
31.70619 35.20032 Asia/Hebron
31.71545 35.18788 Asia/Hebron
31.51591 35.16416 Asia/Hebron
32.21207 35.28561 Asia/Hebron
31.40967 34.97329 Asia/Hebron
31.39671 35.06611 Asia/Hebron
31.85539 35.23402 Asia/Hebron
31.86667 35.45 Asia/Hebron
32.4846 35.2267 Asia/Hebron
31.53333 35.1 Asia/Hebron
31.90721 35.22061 Asia/Hebron
31.77078 35.26916 Asia/Hebron
31.37389 34.34085 Asia/Gaza
31.43954 34.38053 Asia/Gaza
31.78336 35.23388 Asia/Jerusalem
38.95525 -8.98966 Europe/Lisbon
38.87094 -9.06823 Europe/Lisbon
39.09109 -9.2586 Europe/Lisbon
39.60199 -8.40924 Europe/Lisbon
38.80097 -9.37826 Europe/Lisbon
38.44451 -9.10149 Europe/Lisbon
38.5244 -8.8882 Europe/Lisbon
38.82183 -9.09079 Europe/Lisbon
38.70123 -9.3276 Europe/Lisbon
39.23333 -8.68333 Europe/Lisbon
38.84629 -9.08748 Europe/Lisbon
38.79202 -9.10801 Europe/Lisbon
38.77457 -9.3276 Europe/Lisbon
38.80235 -9.19216 Europe/Lisbon
38.75657 -9.25451 Europe/Lisbon
37.06946 -8.10064 Europe/Lisbon
38.86101 -9.06453 Europe/Lisbon
37.13856 -8.53775 Europe/Lisbon
39.29379 -7.43122 Europe/Lisbon
38.76771 -9.19935 Europe/Lisbon
39.91674 -8.62847 Europe/Lisbon
38.63106 -8.91376 Europe/Lisbon
38.66667 -9.15 Europe/Lisbon
39.3558 -9.38112 Europe/Lisbon
38.69282 -9.35412 Europe/Lisbon
38.56902 -8.90126 Europe/Lisbon
38.69569 -9.29143 Europe/Lisbon
37.0286 -7.8411 Europe/Lisbon
38.79269 -9.1838 Europe/Lisbon
38.70675 -8.97388 Europe/Lisbon
38.70279 -9.41606 Europe/Lisbon
39.46203 -8.7118 Europe/Lisbon
38.65078 -8.99038 Europe/Lisbon
39.74769 -8.93228 Europe/Lisbon
38.83091 -9.16845 Europe/Lisbon
37.13772 -8.01968 Europe/Lisbon
38.71667 -9.13333 Europe/Lisbon
38.71446 -9.2422 Europe/Lisbon
39.74362 -8.80705 Europe/Lisbon
37.06831 -7.7937 Europe/Lisbon
37.10202 -8.67422 Europe/Lisbon
32.63333 -16.9 Atlantic/Madeira
37.01937 -7.93223 Europe/Lisbon
38.56667 -7.9 Europe/Lisbon
38.70571 -9.39773 Europe/Lisbon
39.46667 -8.46667 Europe/Lisbon
38.64502 -9.14843 Europe/Lisbon
38.62059 -9.19149 Europe/Lisbon
39.82219 -7.49087 Europe/Lisbon
38.6979 -9.42146 Europe/Lisbon
38.72706 -9.24671 Europe/Lisbon
38.69105 -9.32215 Europe/Lisbon
38.66667 -9.2 Europe/Lisbon
38.80358 -9.12808 Europe/Lisbon
32.63333 -16.96667 Atlantic/Madeira
39.40326 -9.13839 Europe/Lisbon
38.76698 -9.29793 Europe/Lisbon
38.77721 -9.26078 Europe/Lisbon
38.66314 -9.0724 Europe/Lisbon
38.61667 -9.1 Europe/Lisbon
38.62961 -9.11557 Europe/Lisbon
38.75382 -9.23083 Europe/Lisbon
38.67902 -9.1569 Europe/Lisbon
38.70245 -9.22936 Europe/Lisbon
38.73366 -9.40928 Europe/Lisbon
37.08819 -8.2503 Europe/Lisbon
40.66101 -7.90971 Europe/Lisbon
41.30062 -7.74413 Europe/Lisbon
41.10751 -8.59632 Europe/Lisbon
41.13363 -8.61742 Europe/Lisbon
41.35481 -8.7434 Europe/Lisbon
41.69323 -8.83287 Europe/Lisbon
41.19501 -8.51027 Europe/Lisbon
40.55364 -7.22594 Europe/Lisbon
41.1864 -8.65172 Europe/Lisbon
41.15508 -8.50412 Europe/Lisbon
41.20042 -8.60251 Europe/Lisbon
40.90229 -8.48969 Europe/Lisbon
41.18259 -8.55811 Europe/Lisbon
41.38344 -8.76364 Europe/Lisbon
41.14961 -8.61099 Europe/Lisbon
41.76719 -8.58393 Europe/Lisbon
41.41103 -8.74897 Europe/Lisbon
40.85862 -8.62513 Europe/Lisbon
41.12466 -8.58463 Europe/Lisbon
42.07892 -8.48076 Europe/Lisbon
41.18207 -8.68908 Europe/Lisbon
41.23574 -8.6199 Europe/Lisbon
41.21457 -8.61348 Europe/Lisbon
41.191 -8.70027 Europe/Lisbon
40.5999 -8.66732 Europe/Lisbon
41.44444 -8.29619 Europe/Lisbon
41.14454 -8.53223 Europe/Lisbon
40.92535 -8.54277 Europe/Lisbon
41.17087 -8.5257 Europe/Lisbon
41.45423 -8.168 Europe/Lisbon
41.5361 -8.78201 Europe/Lisbon
41.53333 -8.78333 Europe/Lisbon
41.22176 -8.55462 Europe/Lisbon
41.1053 -7.32097 Europe/Lisbon
40.28601 -7.50396 Europe/Lisbon
40.20564 -8.41955 Europe/Lisbon
41.12402 -8.65104 Europe/Lisbon
41.80582 -6.75719 Europe/Lisbon
41.55032 -8.42005 Europe/Lisbon
41.33979 -8.5518 Europe/Lisbon
41.53876 -8.61505 Europe/Lisbon
41.19392 -8.54133 Europe/Lisbon
40.64427 -8.64554 Europe/Lisbon
41.21061 -8.5866 Europe/Lisbon
37.73333 -25.66667 Atlantic/Azores
7.50043 134.62355 Pacific/Palau
-25.75 -56.43333 America/Asuncion
-25.36667 -57.61667 America/Asuncion
-25.33333 -57.53333 America/Asuncion
-25.38333 -57.63333 America/Asuncion
-25.53333 -54.61667 America/Asuncion
-26.86879 -58.29346 America/Asuncion
-25.36667 -57.6 America/Asuncion
-25.18333 -57.53333 America/Asuncion
-25.34682 -57.60647 America/Asuncion
-25.38333 -57.33333 America/Asuncion
-25.48333 -57.35 America/Asuncion
-25.31667 -57.6 America/Asuncion
-27.33333 -55.9 America/Asuncion
-25.41667 -56.45 America/Asuncion
-23.40639 -57.43444 America/Asuncion
-25.16667 -57.55 America/Asuncion
-25.35 -57.41667 America/Asuncion
-25.45 -56.01667 America/Asuncion
-25.38333 -57.15 America/Asuncion
-25.30066 -57.63591 America/Asuncion
25.41694 51.40389 Asia/Qatar
25.29194 51.42444 Asia/Qatar
25.17151 51.60337 Asia/Qatar
25.68389 51.50583 Asia/Qatar
25.27932 51.52245 Asia/Qatar
-21.3393 55.47811 Indian/Reunion
-21.00963 55.27071 Indian/Reunion
-21.28585 55.41124 Indian/Reunion
-21.17059 55.28824 Indian/Reunion
-21.37838 55.61918 Indian/Reunion
-20.90614 55.60886 Indian/Reunion
-20.89684 55.5496 Indian/Reunion
-20.88231 55.4504 Indian/Reunion
-21.03801 55.71937 Indian/Reunion
-20.96333 55.65031 Indian/Reunion
-21.28309 55.51802 Indian/Reunion
-20.9373 55.29189 Indian/Reunion
-20.92892 55.33513 Indian/Reunion
-21.21962 55.31513 Indian/Reunion
43.66667 25.36667 Europe/Bucharest
47.2 23.05 Europe/Bucharest
45.38333 23.26667 Europe/Bucharest
44.46667 26.13333 Europe/Bucharest
47.71667 24.43333 Europe/Bucharest
47.35 25.36667 Europe/Bucharest
46.63333 27.73333 Europe/Bucharest
44.71667 26.63333 Europe/Bucharest
43.75 24.86667 Europe/Bucharest
46.56667 23.78333 Europe/Bucharest
45.16667 28.8 Europe/Bucharest
46 26.13333 Europe/Bucharest
47.2 26.36667 Europe/Bucharest
46.54245 24.55747 Europe/Bucharest
45.05 23.28333 Europe/Bucharest
44.93333 25.45 Europe/Bucharest
45.74944 21.22722 Europe/Bucharest
45.84973 27.43441 Europe/Bucharest
47.63333 26.25 Europe/Bucharest
44.43333 24.36667 Europe/Bucharest
47.23333 22.8 Europe/Bucharest
47.93333 23.88333 Europe/Bucharest
45.8 24.15 Europe/Bucharest
45.86667 25.78333 Europe/Bucharest
45.96667 23.56667 Europe/Bucharest
47.8 22.88333 Europe/Bucharest
46.8 21.65 Europe/Bucharest
45.6174 25.69427 Europe/Bucharest
44.11667 24.98333 Europe/Bucharest
46.91667 26.91667 Europe/Bucharest
45.58333 25.45 Europe/Bucharest
45.1 24.36667 Europe/Bucharest
45.38333 27.05 Europe/Bucharest
45.30083 21.88917 Europe/Bucharest
46.78333 24.71667 Europe/Bucharest
47.85 25.91667 Europe/Bucharest
44.38333 26.16667 Europe/Bucharest
44.95 26.01667 Europe/Bucharest
44.85 24.86667 Europe/Bucharest
46.91667 26.33333 Europe/Bucharest
45.41667 23.36667 Europe/Bucharest
45.45 23.41667 Europe/Bucharest
44.45 26.2 Europe/Bucharest
45.83333 23.2 Europe/Bucharest
47.06667 21.93333 Europe/Bucharest
44.08333 26.63333 Europe/Bucharest
46.3 25.3 Europe/Bucharest
46.38333 23.85 Europe/Bucharest
44.31667 28.6 Europe/Bucharest
44.80333 22.97194 Europe/Bucharest
44.98333 25.65 Europe/Bucharest
46.46667 26.48333 Europe/Bucharest
45.01667 26.45 Europe/Bucharest
46.35 25.8 Europe/Bucharest
46.16667 24.35 Europe/Bucharest
44.25 28.28333 Europe/Bucharest
47.35 22.33333 Europe/Bucharest
43.8 28.58333 Europe/Bucharest
45.36029 23.23832 Europe/Bucharest
45.68861 21.90306 Europe/Bucharest
46.48333 24.1 Europe/Bucharest
47.16667 27.6 Europe/Bucharest
46.67361 28.05944 Europe/Bucharest
45.75 22.9 Europe/Bucharest
47.55 25.9 Europe/Bucharest
43.88333 25.96667 Europe/Bucharest
47.03333 23.91667 Europe/Bucharest
46.71667 25.61667 Europe/Bucharest
45.45 28.05 Europe/Bucharest
44.71667 25.31667 Europe/Bucharest
45.7 27.18333 Europe/Bucharest
44.55 23.51667 Europe/Bucharest
44.38333 27.83333 Europe/Bucharest
47.45 26.3 Europe/Bucharest
45.85 24.96667 Europe/Bucharest
44.63194 22.65611 Europe/Bucharest
44.65 24.26667 Europe/Bucharest
47.95 26.4 Europe/Bucharest
45.88333 22.9 Europe/Bucharest
47.15 23.86667 Europe/Bucharest
45.13333 24.68333 Europe/Bucharest
45.83333 23.36667 Europe/Bucharest
44.31667 23.8 Europe/Bucharest
43.78333 24.5 Europe/Bucharest
44.18333 28.65 Europe/Bucharest
46.41667 26.43333 Europe/Bucharest
45.7 25.45 Europe/Bucharest
46.76667 23.6 Europe/Bucharest
45.71667 24.15 Europe/Bucharest
47.53333 25.56667 Europe/Bucharest
46.55 23.88333 Europe/Bucharest
44.36667 28.01667 Europe/Bucharest
47.68333 22.46667 Europe/Bucharest
45.41667 22.21667 Europe/Bucharest
44.11667 24.35 Europe/Bucharest
43.99111 22.93278 Europe/Bucharest
45.15 26.83333 Europe/Bucharest
46.71667 26.7 Europe/Bucharest
44.56139 25.94889 Europe/Bucharest
44.43225 26.10626 Europe/Bucharest
45.18333 25.66667 Europe/Bucharest
45.64861 25.60613 Europe/Bucharest
45.26667 27.98333 Europe/Bucharest
46.13333 22.78333 Europe/Bucharest
47.75 26.66667 Europe/Bucharest
47.65527 24.66328 Europe/Bucharest
45.37333 21.70917 Europe/Bucharest
46.18333 23.91667 Europe/Bucharest
47.13316 24.50011 Europe/Bucharest
46.23333 27.66667 Europe/Bucharest
44.35 24.1 Europe/Bucharest
44.01667 23.35 Europe/Bucharest
45.03333 25.85 Europe/Bucharest
47.66189 23.69215 Europe/Bucharest
47.65729 23.56808 Europe/Bucharest
46.56667 26.9 Europe/Bucharest
46.18333 21.31667 Europe/Bucharest
43.98333 25.33333 Europe/Bucharest
46.06667 23.58333 Europe/Bucharest
46.3 23.71667 Europe/Bucharest
46.1 27.16667 Europe/Bucharest
47.65331 23.57949 Europe/Bucharest
44.92142 25.45575 Europe/Bucharest
46.35795 25.80405 Europe/Bucharest
47.2469 26.72291 Europe/Bucharest
45.38361 20.38194 Europe/Belgrade
44.84306 20.40111 Europe/Belgrade
43.90358 22.26405 Europe/Belgrade
45.11667 21.30361 Europe/Belgrade
42.55139 21.90028 Europe/Belgrade
43.61694 21.0025 Europe/Belgrade
44.985 20.16083 Europe/Belgrade
44.67653 20.39232 Europe/Belgrade
44.36548 20.95885 Europe/Belgrade
44.66278 20.93 Europe/Belgrade
45.9275 20.07722 Europe/Belgrade
43.23417 21.58806 Europe/Belgrade
44.61528 21.1825 Europe/Belgrade
43.15306 22.58611 Europe/Belgrade
44.87083 20.64028 Europe/Belgrade
44.65486 20.20017 Europe/Belgrade
43.13667 20.51222 Europe/Belgrade
44.94366 20.21931 Europe/Belgrade
43.32472 21.90333 Europe/Belgrade
44.22639 22.53083 Europe/Belgrade
42.99806 21.94611 Europe/Belgrade
44.38534 20.2557 Europe/Belgrade
43.58 21.33389 Europe/Belgrade
43.72583 20.68944 Europe/Belgrade
44.01667 20.91667 Europe/Belgrade
45.82972 20.46528 Europe/Belgrade
43.98139 21.26222 Europe/Belgrade
45.04917 20.07917 Europe/Belgrade
44.03333 20.45 Europe/Belgrade
44.03972 21.31639 Europe/Belgrade
43.9275 21.37 Europe/Belgrade
43.89139 20.34972 Europe/Belgrade
44.07833 22.09528 Europe/Belgrade
44.80401 20.46513 Europe/Belgrade
45.61632 20.03331 Europe/Belgrade
44.30694 20.56 Europe/Belgrade
45.57139 19.64083 Europe/Belgrade
44.27513 19.89821 Europe/Belgrade
43.85861 19.84878 Europe/Belgrade
46.1 19.66667 Europe/Belgrade
44.97639 19.61222 Europe/Belgrade
45.77417 19.11222 Europe/Belgrade
44.74667 19.69 Europe/Belgrade
45.00806 19.82222 Europe/Belgrade
45.25167 19.83694 Europe/Belgrade
45.81516 19.6318 Europe/Belgrade
45.25083 19.39194 Europe/Belgrade
45.67111 18.98472 Europe/Belgrade
43.56484 22.25023 Europe/Belgrade
57.87944 34.9925 Europe/Moscow
54.35369 41.91986 Europe/Moscow
48.02108 40.12282 Europe/Moscow
55.73402 36.85918 Europe/Moscow
55.17111 59.65083 Asia/Yekaterinburg
47.1474 42.4721 Europe/Moscow
55.59528 38.12028 Europe/Moscow
53.53395 33.72798 Europe/Moscow
50.98448 44.77347 Europe/Volgograd
53.39972 49.49528 Europe/Samara
51.84861 41.46056 Europe/Moscow
44.13944 43.01972 Europe/Moscow
52.331 35.3711 Europe/Moscow
55.744 38.01684 Europe/Moscow
46.84518 40.30834 Europe/Moscow
44.40694 43.88056 Europe/Moscow
55.9825 37.18139 Europe/Moscow
55.84376 48.51784 Europe/Moscow
43.85804 41.5894 Europe/Moscow
56.64051 43.39446 Europe/Moscow
53.20356 45.19227 Europe/Moscow
54.7605 38.87841 Europe/Moscow
69.41541 30.81355 Europe/Moscow
55.3195 52.06942 Europe/Moscow
56.50339 39.67911 Europe/Moscow
56.63877 47.89078 Europe/Moscow
46.7055 38.2739 Europe/Moscow
44.0328 42.8813 Europe/Moscow
44.04444 42.86056 Europe/Moscow
51.3513 48.2766 Europe/Volgograd
62.58999 50.85939 Europe/Moscow
45.04446 38.7958 Europe/Moscow
52.62366 38.50169 Europe/Moscow
50.949 43.73781 Europe/Volgograd
55.76127 52.06493 Europe/Moscow
55.38283 39.03233 Europe/Moscow
46.56564 40.65621 Europe/Moscow
53.14806 38.09924 Europe/Moscow
51.0333 59.8742 Asia/Yekaterinburg
54.48028 37.6975 Europe/Moscow
55.06667 32.69639 Europe/Moscow
57.62987 39.87368 Europe/Moscow
57.30331 47.88611 Europe/Volgograd
56.2719 54.93086 Asia/Yekaterinburg
44.98901 38.94324 Europe/Moscow
57.59125 34.56453 Europe/Moscow
45.58128 39.66409 Europe/Moscow
55.3175 42.17444 Europe/Moscow
60.70763 28.75283 Europe/Moscow
56.25 42.16667 Europe/Moscow
55.21039 34.29508 Europe/Moscow
56.22602 51.06557 Europe/Volgograd
60.02043 30.63716 Europe/Moscow
57.04865 53.98717 Europe/Samara
55.39934 37.80829 Europe/Moscow
55.3125 38.66361 Europe/Moscow
51.67204 39.1843 Europe/Moscow
48.78583 44.77973 Europe/Volgograd
55.86638 48.3594 Europe/Moscow
52.03333 47.38333 Europe/Volgograd
56.03361 35.96944 Europe/Moscow
59.2187 39.8886 Europe/Moscow
59.9258 32.33819 Europe/Moscow
57.43931 41.15553 Europe/Moscow
48.71939 44.50184 Europe/Volgograd
47.51361 42.15139 Europe/Moscow
55.6 37.21667 Europe/Moscow
56.13655 40.39658 Europe/Moscow
43.03667 44.66778 Europe/Moscow
55.55239 37.70966 Europe/Moscow
57.21276 41.93012 Europe/Moscow
58.07894 54.6557 Asia/Yekaterinburg
54.35533 38.26843 Europe/Moscow
61.07006 42.0983 Europe/Moscow
60.76186 46.31352 Europe/Moscow
56.34 30.54517 Europe/Moscow
50.2035 38.1067 Europe/Moscow
57.97472 33.2525 Europe/Moscow
53.98179 38.17118 Europe/Moscow
51.98486 42.26147 Europe/Moscow
56.99081 52.18517 Europe/Samara
45.21436 39.69033 Europe/Moscow
54.9366 58.1757 Asia/Yekaterinburg
44.0834 41.9763 Europe/Moscow
52.04662 39.7288 Europe/Moscow
50.8047 42.0138 Europe/Volgograd
43.12861 45.54306 Europe/Moscow
52.84591 32.67394 Europe/Moscow
54.33333 48.4 Europe/Volgograd
63.56705 53.68348 Europe/Moscow
57.5275 38.33167 Europe/Moscow
54.78517 56.04562 Asia/Yekaterinburg
43.93778 42.51167 Europe/Moscow
54.2996 59.4521 Asia/Yekaterinburg
43.38278 42.91833 Europe/Moscow
56.86194 35.89306 Europe/Moscow
54.60666 53.7097 Asia/Yekaterinburg
57.88529 39.5406 Europe/Moscow
54.20444 37.61111 Europe/Moscow
55.60111 36.46806 Europe/Moscow
44.1053 39.0802 Europe/Moscow
47.64611 42.10194 Europe/Moscow
52.5798 33.7644 Europe/Moscow
43.30556 44.97889 Europe/Moscow
55.48498 37.30736 Europe/Moscow
59.54 30.8775 Europe/Moscow
57.0436 34.96221 Europe/Moscow
55.6562 37.94713 Europe/Moscow
53.5303 49.3461 Europe/Samara
45.61694 38.94528 Europe/Moscow
59.64511 33.52937 Europe/Moscow
45.85472 40.12528 Europe/Moscow
56.85796 40.53692 Europe/Moscow
43.48333 44.13611 Europe/Moscow
45.27055 37.38716 Europe/Moscow
45.36333 40.19 Europe/Moscow
52.73167 41.43389 Europe/Moscow
55.9077 53.9355 Asia/Yekaterinburg
47.23617 38.89688 Europe/Moscow
53.1585 48.4681 Europe/Samara
61.67642 50.80994 Europe/Moscow
44.02556 43.05028 Europe/Moscow
61.11213 28.86321 Europe/Moscow
54.67501 20.13473 Europe/Kaliningrad
45.33528 42.85472 Europe/Moscow
44.1901 42.6595 Europe/Moscow
54.1223 36.49657 Europe/Moscow
48.60734 42.84621 Europe/Volgograd
43.1875 44.90194 Europe/Moscow
54.09989 35.34254 Europe/Moscow
54.90083 38.07083 Europe/Moscow
56.38333 38.56667 Europe/Moscow
50.78543 36.4831 Europe/Moscow
53.62462 55.95015 Asia/Yekaterinburg
45.0428 41.9734 Europe/Moscow
51.29667 37.84167 Europe/Moscow
46.6311 38.6742 Europe/Moscow
46.53 39.04972 Europe/Moscow
52.58444 32.76333 Europe/Moscow
57.99439 31.36081 Europe/Moscow
55.80799 38.1805 Europe/Moscow
57.58475 48.95844 Europe/Volgograd
55.08392 21.87851 Europe/Kaliningrad
59.9 29.11667 Europe/Moscow
63.60229 53.88175 Europe/Moscow
61.70532 30.69178 Europe/Moscow
52.4338 53.1583 Asia/Yekaterinburg
56.18333 36.98333 Europe/Moscow
51.1631 54.99176 Asia/Yekaterinburg
59.61956 56.77289 Asia/Yekaterinburg
59.46167 40.12056 Europe/Moscow
56.15 37.96667 Europe/Moscow
43.59917 39.72569 Europe/Moscow
55.98553 40.01111 Europe/Moscow
54.7818 32.0401 Europe/Moscow
58.73222 50.17722 Europe/Volgograd
45.2558 38.1256 Europe/Moscow
59.11817 28.09137 Europe/Moscow
53.82486 39.55053 Europe/Moscow
55.94806 37.29778 Europe/Moscow
54.993 57.6982 Asia/Yekaterinburg
52.71806 58.66583 Asia/Yekaterinburg
56.84865 41.38833 Europe/Moscow
59.80917 30.38167 Europe/Moscow
55.5005 46.41288 Europe/Moscow
45.12833 42.02556 Europe/Moscow
54.3204 40.874 Europe/Moscow
59.20998 38.51066 Europe/Moscow
50.40967 36.9136 Europe/Moscow
51.87555 36.90432 Europe/Moscow
55.49972 37.55972 Europe/Moscow
55.89303 38.0586 Europe/Moscow
54.00917 37.51194 Europe/Moscow
55.57253 39.53682 Europe/Moscow
58.36909 45.51558 Europe/Moscow
57.67579 46.61136 Europe/Moscow
47.70911 40.21443 Europe/Moscow
44.85407 38.67686 Europe/Moscow
54.03481 38.40172 Europe/Moscow
60.15328 59.95205 Asia/Yekaterinburg
69.06889 33.41622 Europe/Moscow
64.5635 39.8302 Europe/Moscow
60.09802 29.96378 Europe/Moscow
60.1444 30.20165 Europe/Moscow
54.92139 37.41083 Europe/Moscow
56.3 38.13333 Europe/Moscow
55.53079 45.46611 Europe/Moscow
52.45861 44.21689 Europe/Moscow
51.68526 39.02787 Europe/Moscow
47.51943 40.8112 Europe/Moscow
56.78749 44.49297 Europe/Moscow
53.36831 34.10328 Europe/Moscow
63.74147 34.32218 Europe/Moscow
55.0425 59.04 Asia/Yekaterinburg
54.93583 43.32352 Europe/Moscow
51.56667 46.03333 Europe/Volgograd
56.47633 53.79782 Europe/Samara
54.1838 45.1749 Europe/Moscow
51.78662 51.78662 Asia/Yekaterinburg
59.89444 30.26417 Europe/Moscow
53.2 50.15 Europe/Samara
46.4747 41.54114 Europe/Moscow
53.38365 55.90773 Asia/Yekaterinburg
55.15 33.21667 Europe/Moscow
56.26241 34.32817 Europe/Moscow
54.72774 39.51731 Europe/Moscow
58.0446 38.84259 Europe/Moscow
53.7038 40.1109 Europe/Moscow
54.6269 39.6916 Europe/Moscow
54.06387 44.9509 Europe/Moscow
55.78514 37.35481 Europe/Moscow
52.26041 43.78745 Europe/Volgograd
47.23563 39.71219 Europe/Moscow
57.1914 39.41394 Europe/Moscow
51.1209 38.5116 Europe/Moscow
50.19828 39.56726 Europe/Moscow
53.95278 32.86389 Europe/Moscow
55.66853 39.87488 Europe/Moscow
57.10513 41.73048 Europe/Moscow
56.80097 59.93028 Asia/Yekaterinburg
55.76111 37.8575 Europe/Moscow
50.53439 36.68462 Europe/Moscow
54.0658 54.9468 Asia/Yekaterinburg
52.67139 41.90333 Europe/Moscow
44.04861 43.05944 Europe/Moscow
56.01722 37.86667 Europe/Moscow
59.68733 30.34046 Europe/Moscow
54.83389 37.62639 Europe/Moscow
57.8136 28.3496 Europe/Moscow
54.86639 37.21778 Europe/Moscow
46.70289 41.72717 Europe/Moscow
43.75741 44.0297 Europe/Moscow
53.9 53.93333 Asia/Yekaterinburg
51.4 46.03333 Europe/Volgograd
57.38698 41.28666 Europe/Moscow
61.03928 30.12907 Europe/Moscow
46.0497 38.1747 Europe/Moscow
51.68406 39.0727 Europe/Moscow
51.1945 42.2457 Europe/Moscow
67.36611 32.49806 Europe/Moscow
69.19889 33.44778 Europe/Moscow
55.91797 39.17242 Europe/Moscow
53.65237 52.12738 Europe/Samara
60.91124 34.17064 Europe/Moscow
55.42418 37.55472 Europe/Moscow
52.9336 33.4464 Europe/Moscow
53.70944 37.29194 Europe/Moscow
59.51833 34.16639 Europe/Moscow
55.92639 39.46104 Europe/Moscow
61.78491 34.34691 Europe/Moscow
52.30639 45.39167 Europe/Volgograd
59.885 29.89583 Europe/Moscow
58.59382 35.80244 Europe/Moscow
56.90528 59.94361 Asia/Yekaterinburg
58.01741 56.28552 Asia/Yekaterinburg
56.73934 38.85626 Europe/Moscow
53.20066 45.00464 Europe/Moscow
65.14716 57.22439 Europe/Moscow
55.78071 38.65448 Europe/Moscow
46.1377 39.7832 Europe/Moscow
50.45778 40.10806 Europe/Moscow
45.0244 39.10679 Europe/Moscow
50.05 46.88333 Europe/Volgograd
54.85998 38.55062 Europe/Moscow
53.37596 51.3452 Europe/Samara
59.7775 30.81806 Europe/Moscow
44.39333 41.52056 Europe/Moscow
57.34395 28.35368 Europe/Moscow
50.8664 39.07562 Europe/Moscow
57.14667 33.10753 Europe/Moscow
57.28288 55.45669 Asia/Yekaterinburg
51.20487 58.56685 Asia/Yekaterinburg
46.87139 42.05917 Europe/Moscow
51.7727 55.0988 Asia/Yekaterinburg
52.96508 36.0785 Europe/Moscow
55.80672 38.96178 Europe/Moscow
43.32111 45.05222 Europe/Moscow
63.90607 38.14035 Europe/Moscow
58.67001 52.1932 Europe/Volgograd
68.1432 33.25287 Europe/Moscow
43.23806 46.00167 Europe/Moscow
54.48147 53.47103 Asia/Yekaterinburg
55.67798 37.27773 Europe/Moscow
57.89035 54.7202 Asia/Yekaterinburg
55.09681 36.61006 Europe/Moscow
57.94042 55.33397 Asia/Yekaterinburg
61.66903 40.19907 Europe/Moscow
54.42766 50.80511 Europe/Moscow
50.7631 37.8691 Europe/Moscow
52.5396 31.9275 Europe/Moscow
51.30719 39.21732 Europe/Moscow
50.45917 48.14306 Europe/Volgograd
54.14961 48.38907 Europe/Volgograd
51.20301 58.32665 Asia/Yekaterinburg
45.23756 38.98059 Europe/Moscow
47.76037 39.93335 Europe/Moscow
44.72439 37.76752 Europe/Moscow
45.95139 40.70056 Europe/Moscow
43.96222 43.63417 Europe/Moscow
54.0105 38.2846 Europe/Moscow
54.0384 39.7479 Europe/Moscow
53.0959 49.9462 Europe/Samara
45.117 41.0267 Europe/Moscow
64.4165 40.8122 Europe/Moscow
47.42018 40.09132 Europe/Moscow
56.11095 47.47755 Europe/Moscow
50.5287 42.6747 Europe/Volgograd
45.4948 41.22075 Europe/Moscow
58.51667 31.28333 Europe/Moscow
51.64177 39.40863 Europe/Moscow
55.86647 38.4438 Europe/Moscow
58.6293 59.8118 Asia/Yekaterinburg
57.91944 59.965 Asia/Yekaterinburg
56.32867 44.00205 Europe/Moscow
53.53041 43.67664 Europe/Moscow
55.63657 51.82447 Europe/Moscow
53.71886 46.07118 Europe/Moscow
50.02824 45.45953 Europe/Volgograd
69.41285 30.22198 Europe/Moscow
44.11806 43.40278 Europe/Moscow
44.6333 41.9444 Europe/Moscow
56.02094 29.92844 Europe/Moscow
43.23861 45.05028 Europe/Moscow
57.45881 40.57471 Europe/Moscow
56.22107 32.77738 Europe/Moscow
44.66667 44.91667 Europe/Moscow
56.09199 54.27032 Asia/Yekaterinburg
52.802 51.166 Europe/Samara
43.22611 44.765 Europe/Moscow
55.52917 42.20028 Europe/Moscow
67.67126 53.08697 Europe/Moscow
43.5575 43.85083 Europe/Moscow
55.38752 36.73307 Europe/Moscow
43.49806 43.61889 Europe/Moscow
55.84854 37.17788 Europe/Moscow
55.72545 52.41122 Europe/Moscow
55.91163 37.73076 Europe/Moscow
55.56556 42.04167 Europe/Moscow
68.97917 33.09251 Europe/Moscow
53.27657 36.57334 Europe/Moscow
56.44469 52.22763 Europe/Samara
55.50194 36.02722 Europe/Moscow
43.74917 44.68806 Europe/Moscow
44.41222 40.79361 Europe/Moscow
55.75222 37.61556 Europe/Moscow
53.44354 41.81065 Europe/Moscow
48.35502 41.82627 Europe/Moscow
55.84244 38.19362 Europe/Moscow
67.93972 32.87389 Europe/Moscow
61.33289 44.53598 Europe/Moscow
44.21028 43.13528 Europe/Moscow
48.92265 40.39673 Europe/Moscow
50.06 43.2379 Europe/Volgograd
52.8978 40.4907 Europe/Moscow
59.80083 30.54778 Europe/Moscow
55.72706 53.10258 Europe/Moscow
55.90818 52.29047 Europe/Moscow
52.96667 55.91667 Asia/Yekaterinburg
55.3343 41.6295 Europe/Moscow
62.91713 34.45689 Europe/Moscow
45.45151 39.02485 Europe/Moscow
56.63388 47.80405 Europe/Moscow
51.4128 57.595 Asia/Yekaterinburg
44.60778 40.10583 Europe/Moscow
51.71111 46.74861 Europe/Volgograd
58.32889 44.76406 Europe/Moscow
55.01457 36.47185 Europe/Moscow
43.50917 44.59306 Europe/Moscow
55.64776 38.02486 Europe/Moscow
42.97638 47.50236 Europe/Moscow
53.41861 59.04722 Asia/Yekaterinburg
53.86639 34.44778 Europe/Moscow
55.67719 37.89322 Europe/Moscow
55.58271 37.90516 Europe/Moscow
58.10861 57.80528 Asia/Yekaterinburg
56.03282 45.0422 Europe/Moscow
54.97661 39.0444 Europe/Moscow
58.73722 29.84528 Europe/Moscow
55.86959 38.20065 Europe/Moscow
59.90612 29.77253 Europe/Moscow
60.726 33.55306 Europe/Moscow
56.00972 37.48194 Europe/Moscow
52.42534 37.60689 Europe/Moscow
50.98405 39.51545 Europe/Moscow
52.60311 39.57076 Europe/Moscow
55.7083 38.9542 Europe/Moscow
51.63069 35.2775 Europe/Moscow
44.10667 42.97333 Europe/Moscow
48.70233 45.19798 Europe/Volgograd
54.60256 52.46087 Europe/Moscow
46.3214 39.3877 Europe/Moscow
55.93333 37.83333 Europe/Moscow
53.01942 39.16639 Europe/Moscow
43.90886 39.33137 Europe/Moscow
56.01931 39.94848 Europe/Moscow
44.63417 40.73556 Europe/Moscow
43.6625 43.53694 Europe/Moscow
53.11675 46.60037 Europe/Moscow
51.4781 57.3552 Asia/Yekaterinburg
58.29056 59.75917 Asia/Yekaterinburg
46.5599 39.6321 Europe/Moscow
55.33833 59.44056 Asia/Yekaterinburg
51.73733 36.18735 Europe/Moscow
55.58183 38.91994 Europe/Moscow
44.88 40.59861 Europe/Moscow
51.6601 35.6521 Europe/Moscow
43.20184 46.0881 Europe/Moscow
57.4368 56.9593 Asia/Yekaterinburg
52.76667 55.78333 Asia/Yekaterinburg
55.41333 42.5325 Europe/Moscow
56.1855 50.8944 Europe/Moscow
59.01729 54.656 Asia/Yekaterinburg
55.57957 36.70392 Europe/Moscow
56.14733 44.19787 Europe/Moscow
44.92934 37.99117 Europe/Moscow
45.4375 40.57556 Europe/Moscow
59.99361 29.77361 Europe/Moscow
47.89221 40.07037 Europe/Moscow
59.73833 30.08944 Europe/Moscow
60.40783 57.08199 Asia/Yekaterinburg
56.60585 57.76686 Asia/Yekaterinburg
58.08016 55.75411 Asia/Yekaterinburg
45.84604 41.51885 Europe/Moscow
55.82036 37.33017 Europe/Moscow
45.0488 38.9725 Europe/Moscow
45.36614 38.21171 Europe/Moscow
56.1 38.13333 Europe/Moscow
51.02389 45.69694 Europe/Volgograd
56.3419 46.56353 Europe/Moscow
54.03746 35.77159 Europe/Moscow
54.03876 43.91386 Europe/Moscow
56.35722 41.31917 Europe/Moscow
67.56616 30.47576 Europe/Moscow
52.58861 41.50944 Europe/Moscow
50.32113 44.80119 Europe/Volgograd
61.25745 46.64963 Europe/Moscow
47.63009 43.14068 Europe/Volgograd
55.63546 37.8445 Europe/Moscow
58.3089 48.348 Europe/Volgograd
57.76647 40.92686 Europe/Moscow
64.571 30.57667 Europe/Moscow
54.11896 37.54459 Europe/Moscow
61.31433 47.16914 Europe/Moscow
45.46899 39.45136 Europe/Moscow
47.58278 41.09222 Europe/Moscow
54.80639 35.92778 Europe/Moscow
62.20565 34.26138 Europe/Moscow
56.7 36.76667 Europe/Moscow
59.62056 30.39 Europe/Moscow
59.75069 30.58856 Europe/Moscow
55.07944 38.77833 Europe/Moscow
56.29929 39.38304 Europe/Moscow
56.93486 41.0915 Europe/Moscow
52.76019 32.23934 Europe/Moscow
56.33333 36.73333 Europe/Moscow
55.36352 37.52984 Europe/Moscow
43.84694 46.715 Europe/Moscow
43.20697 46.8679 Europe/Moscow
59.04709 57.64767 Asia/Yekaterinburg
43.91333 42.72083 Europe/Moscow
56.15273 38.85509 Europe/Moscow
52.65494 42.72236 Europe/Moscow
67.61475 33.67274 Europe/Moscow
59.88101 30.99507 Europe/Moscow
58.55386 50.03986 Europe/Volgograd
58.59665 49.66007 Europe/Volgograd
54.06686 34.29955 Europe/Moscow
59.44712 32.02049 Europe/Moscow
53.93361 37.92792 Europe/Moscow
59.37331 28.61343 Europe/Moscow
57.43914 42.12894 Europe/Moscow
53.4706 51.4743 Europe/Samara
53.23333 50.65 Europe/Samara
56.86667 37.35 Europe/Moscow
53.97164 38.53186 Europe/Moscow
56.25699 37.99544 Europe/Moscow
43.51484 39.86825 Europe/Moscow
44.84694 38.385 Europe/Moscow
55.89704 37.42969 Europe/Moscow
43.25 46.58806 Europe/Moscow
47.41958 47.25678 Europe/Volgograd
44.4258 39.5362 Europe/Moscow
55.78874 49.12214 Europe/Moscow
54.75306 58.19556 Asia/Yekaterinburg
42.88165 47.63919 Europe/Moscow
54.94111 41.39528 Europe/Moscow
54.84444 38.16694 Europe/Moscow
57.35917 37.60806 Europe/Moscow
53.12292 34.98517 Europe/Moscow
43.77399 41.91419 Europe/Moscow
43.30361 44.89639 Europe/Moscow
56.31667 38.7 Europe/Moscow
43.22639 44.63806 Europe/Moscow
46.0849 38.9596 Europe/Moscow
67.162 32.41229 Europe/Moscow
55.50861 47.49556 Europe/Moscow
46.10995 48.07364 Europe/Volgograd
50.09833 45.41601 Europe/Volgograd
48.31779 40.25948 Europe/Moscow
54.5293 36.27542 Europe/Moscow
51.5 44.4758 Europe/Volgograd
55.91417 37.82556 Europe/Moscow
54.70649 20.51095 Europe/Kaliningrad
48.69099 43.52638 Europe/Volgograd
50.4258 41.0261 Europe/Moscow
58.69999 59.49482 Asia/Yekaterinburg
56.85 53.23333 Europe/Samara
42.56955 47.86447 Europe/Moscow
55.97111 37.92083 Europe/Moscow
56.99719 40.97139 Europe/Moscow
55.91997 36.86867 Europe/Moscow
53.45446 56.04149 Asia/Yekaterinburg
45.71806 42.90361 Europe/Moscow
53.85672 46.35622 Europe/Volgograd
44.09264 43.09113 Europe/Moscow
44.84222 38.56686 Europe/Moscow
57.55583 53.05472 Europe/Samara
55.61113 40.65186 Europe/Moscow
54.59222 22.19972 Europe/Kaliningrad
45.35383 40.69465 Europe/Moscow
48.05306 39.93111 Europe/Moscow
43.35694 46.10528 Europe/Moscow
51.28167 37.54583 Europe/Moscow
58.83862 57.55325 Asia/Yekaterinburg
58.88 40.2525 Europe/Moscow
52.49657 39.93593 Europe/Moscow
43.30833 45.70028 Europe/Moscow
51.45792 41.97636 Europe/Moscow
44.63083 39.13 Europe/Moscow
44.02361 43.09222 Europe/Moscow
48.80454 44.47439 Europe/Volgograd
56.65139 43.475 Europe/Moscow
55.60928 36.98212 Europe/Moscow
58.13941 52.65842 Europe/Samara
44.86208 40.07195 Europe/Moscow
44.15194 43.46972 Europe/Moscow
44.5622 38.0848 Europe/Moscow
51.4666 58.4552 Asia/Yekaterinburg
57.30185 39.85331 Europe/Moscow
59.57639 30.12833 Europe/Moscow
58.37884 42.34633 Europe/Moscow
55.55291 34.99537 Europe/Moscow
57.25363 41.10849 Europe/Moscow
55.96056 38.04556 Europe/Moscow
55.73321 38.46458 Europe/Moscow
49.77133 43.66222 Europe/Volgograd
61.81281 50.72834 Europe/Moscow
51.5 46.11667 Europe/Volgograd
44.9271 38.90852 Europe/Moscow
46.30778 44.25583 Europe/Moscow
55.72445 38.20908 Europe/Moscow
55.78959 38.44671 Europe/Moscow
55.88431 38.7864 Europe/Moscow
43.21222 44.82306 Europe/Moscow
55.62945 37.85654 Europe/Moscow
56.23889 43.46306 Europe/Moscow
55.49106 54.86883 Asia/Yekaterinburg
53.59782 34.33825 Europe/Moscow
49.05825 44.829 Europe/Volgograd
56.73333 37.16667 Europe/Moscow
45.45518 41.97411 Europe/Moscow
53.97106 38.33627 Europe/Moscow
48.33372 39.94776 Europe/Moscow
55.4413 37.75367 Europe/Moscow
55.90411 37.56064 Europe/Moscow
58.46195 56.41646 Asia/Yekaterinburg
56.35 37.51667 Europe/Moscow
45.90889 43.35472 Europe/Moscow
45.21782 39.22833 Europe/Moscow
54.21386 49.61838 Europe/Volgograd
54.15077 33.28151 Europe/Moscow
42.0678 48.28987 Europe/Moscow
55.86861 37.12222 Europe/Moscow
54.22145 55.03434 Asia/Yekaterinburg
53.25155 39.15553 Europe/Moscow
58.1908 40.17171 Europe/Moscow
42.11589 48.19204 Europe/Moscow
58.3013 57.8131 Asia/Yekaterinburg
59.11667 31.68333 Europe/Moscow
55.36311 50.64244 Europe/Moscow
54.57648 55.37642 Asia/Yekaterinburg
50.94095 37.80693 Europe/Moscow
54.63345 21.81557 Europe/Kaliningrad
56.50722 56.07661 Asia/Yekaterinburg
56 38.36667 Europe/Moscow
44.22333 42.05778 Europe/Moscow
59.13333 37.9 Europe/Moscow
55.1477 37.47728 Europe/Moscow
43.28333 43.13333 Europe/Moscow
56.13222 47.25194 Europe/Moscow
56.76864 54.11484 Asia/Yekaterinburg
52.9771 49.7086 Europe/Samara
52.7807 52.2635 Asia/Yekaterinburg
42.819 47.1192 Europe/Moscow
58.47861 41.53 Europe/Moscow
50.82389 40.60917 Europe/Moscow
54.95 48.28333 Europe/Moscow
53.6554 52.442 Asia/Yekaterinburg
54.5378 52.7985 Europe/Moscow
44.78389 44.16583 Europe/Moscow
45.80603 38.99959 Europe/Moscow
53.25209 34.37167 Europe/Moscow
55.42112 38.26188 Europe/Moscow
58.38778 33.91546 Europe/Moscow
51.36713 42.08494 Europe/Moscow
56.35808 44.07477 Europe/Moscow
57.8799 34.1068 Europe/Moscow
59.47405 33.84853 Europe/Moscow
49.93577 40.545 Europe/Moscow
56.10333 43.51583 Europe/Moscow
53.77166 38.12408 Europe/Moscow
51.09611 40.03333 Europe/Moscow
55.035 55.97806 Asia/Yekaterinburg
45.09778 43.43639 Europe/Moscow
55.42111 55.54278 Asia/Yekaterinburg
57.78506 36.69651 Europe/Moscow
52.982 49.4333 Europe/Samara
43.19217 44.54313 Europe/Moscow
59.41519 56.81237 Asia/Yekaterinburg
53.96306 58.39806 Asia/Yekaterinburg
44.76733 39.87424 Europe/Moscow
55.45978 38.44358 Europe/Moscow
50.61074 36.58015 Europe/Moscow
53.81194 36.13194 Europe/Moscow
54.11667 54.11667 Asia/Yekaterinburg
48.18585 40.77424 Europe/Moscow
46.07802 40.86651 Europe/Moscow
52.59333 58.3225 Asia/Yekaterinburg
54.39304 53.26023 Europe/Moscow
47.13975 39.75181 Europe/Moscow
53.65533 47.11229 Europe/Volgograd
57.97963 53.01057 Europe/Samara
51.5502 43.1667 Europe/Volgograd
55.80945 37.95806 Europe/Moscow
52.02782 47.8007 Europe/Volgograd
56.49056 43.59194 Europe/Moscow
55.18161 36.6606 Europe/Moscow
43.68806 43.53694 Europe/Moscow
54.9417 58.8083 Asia/Yekaterinburg
47.32411 40.38791 Europe/Moscow
47.10779 39.41648 Europe/Moscow
43.16 46 Europe/Moscow
51.87185 45.00775 Europe/Volgograd
46.34968 48.04076 Europe/Volgograd
54.9973 57.2722 Asia/Yekaterinburg
45.37278 44.22056 Europe/Moscow
55.39485 43.83992 Europe/Moscow
56.09151 49.87783 Europe/Moscow
44.9892 41.1234 Europe/Moscow
64.5401 40.5433 Europe/Moscow
43.29 45.87 Europe/Moscow
43.17 44.29 Europe/Moscow
44.46472 39.73417 Europe/Moscow
55.55194 37.0801 Europe/Moscow
67.56414 33.4031 Europe/Moscow
51.4842 40.4299 Europe/Moscow
44.89084 37.3239 Europe/Moscow
54.90442 52.3154 Europe/Moscow
54.51 37.08 Europe/Moscow
50.6309 38.6903 Europe/Moscow
56.4 38.71667 Europe/Moscow
44.71417 43.00083 Europe/Moscow
59.15804 57.56241 Asia/Yekaterinburg
56.39516 38.71216 Europe/Moscow
54.8421 46.5813 Europe/Moscow
47.25838 39.86675 Europe/Moscow
44.8546 38.3031 Europe/Moscow
48.27955 46.17217 Europe/Volgograd
56.52034 52.99422 Europe/Moscow
44.90212 38.84393 Europe/Moscow
43.42896 39.92391 Europe/Moscow
43.19 45.28 Europe/Moscow
44.87258 38.16111 Europe/Moscow
53.7 53.66667 Asia/Yekaterinburg
55.83596 48.6582 Europe/Moscow
51.5655 34.6822 Europe/Moscow
43.04222 44.22222 Europe/Moscow
47.52972 39.41833 Europe/Moscow
43.66974 39.66863 Europe/Moscow
55.9686 43.0912 Europe/Moscow
41.89184 48.41247 Europe/Moscow
58.63667 59.80222 Asia/Yekaterinburg
54.815 58.45917 Asia/Yekaterinburg
62.76409 40.33746 Europe/Moscow
48.5842 45.7338 Europe/Volgograd
53.13333 46.58333 Europe/Moscow
44.64417 41.84528 Europe/Moscow
55.61194 37.29611 Europe/Moscow
55.59911 37.35495 Europe/Moscow
65.99389 57.52806 Europe/Moscow
43.54417 44.58944 Europe/Moscow
56.5042 66.55153 Asia/Yekaterinburg
53.7074 84.9493 Asia/Omsk
56.811 61.3254 Asia/Yekaterinburg
53.25417 83.69361 Asia/Omsk
54.4418 61.2536 Asia/Yekaterinburg
55.72306 84.88611 Asia/Novokuznetsk
58.44972 92.17028 Asia/Krasnoyarsk
54.75472 61.32083 Asia/Yekaterinburg
56.8575 60.6125 Asia/Yekaterinburg
55.8736 85.4265 Asia/Novokuznetsk
52.9273 78.58 Asia/Omsk
56.65358 66.30058 Asia/Yekaterinburg
67.5 64 Europe/Moscow
67.58333 63.95 Europe/Moscow
58.04556 60.55333 Asia/Yekaterinburg
56.96667 60.58083 Asia/Yekaterinburg
56.05667 60.23056 Asia/Yekaterinburg
55.30361 89.83389 Asia/Krasnoyarsk
56.11472 94.58611 Asia/Krasnoyarsk
60.12472 64.77583 Asia/Yekaterinburg
57.15222 65.52722 Asia/Yekaterinburg
58.04575 63.69605 Asia/Yekaterinburg
54.0979 61.5773 Asia/Yekaterinburg
55.2769 85.6163 Asia/Novokuznetsk
56.5 84.96667 Asia/Krasnoyarsk
55.23528 84.38583 Asia/Novosibirsk
58.2 68.26667 Asia/Yekaterinburg
55.9342 98.0044 Asia/Irkutsk
56.06 85.61972 Asia/Novokuznetsk
58.04254 65.27258 Asia/Yekaterinburg
55.21904 75.98284 Asia/Novosibirsk
52.7657 87.8894 Asia/Novokuznetsk
64.9225 77.785 Asia/Yekaterinburg
56.89436 74.37096 Asia/Omsk
69.4865 88.3972 Asia/Krasnoyarsk
56.8804 60.0213 Asia/Yekaterinburg
57.01095 63.73254 Asia/Yekaterinburg
56.49528 60.81917 Asia/Yekaterinburg
53.7841 82.3116 Asia/Novosibirsk
61.25 73.41667 Asia/Yekaterinburg
56.9094 62.0296 Asia/Yekaterinburg
60.7333 77.5889 Asia/Krasnoyarsk
56.98921 60.46662 Asia/Yekaterinburg
61.36139 63.58417 Asia/Yekaterinburg
56.1325 93.35889 Asia/Krasnoyarsk
52.9978 78.6449 Asia/Omsk
53.325 91.93556 Asia/Krasnoyarsk
55.22611 63.29611 Asia/Yekaterinburg
55.54028 89.20083 Asia/Krasnoyarsk
56.0852 63.6335 Asia/Yekaterinburg
59.60334 60.5787 Asia/Yekaterinburg
53.0875 91.39972 Asia/Krasnoyarsk
66.53 66.60194 Asia/Yekaterinburg
51.5 81.25 Asia/Omsk
57.37005 61.40428 Asia/Yekaterinburg
57.09013 61.67692 Asia/Yekaterinburg
54.9159 85.6385 Asia/Novokuznetsk
53.9059 86.719 Asia/Novokuznetsk
61.23333 73.33333 Asia/Yekaterinburg
54.6012 86.2459 Asia/Novokuznetsk
56.44222 60.18778 Asia/Yekaterinburg
54.36914 60.81361 Asia/Yekaterinburg
53.6239 87.3598 Asia/Novokuznetsk
55 73.4 Asia/Omsk
54.9872 82.7166 Asia/Novosibirsk
62.14056 65.39361 Asia/Yekaterinburg
63.19936 75.45067 Asia/Yekaterinburg
66.08333 76.63333 Asia/Yekaterinburg
53.30972 83.62389 Asia/Omsk
55.0415 82.9346 Asia/Novosibirsk
53.7557 87.1099 Asia/Novokuznetsk
53.3917 83.9363 Asia/Omsk
69.3535 88.2027 Asia/Krasnoyarsk
58.07756 60.7202 Asia/Yekaterinburg
60.9344 76.5531 Asia/Yekaterinburg
54.9076 99.0276 Asia/Irkutsk
57.4953 60.2112 Asia/Yekaterinburg
61.09979 72.60349 Asia/Yekaterinburg
56.00639 90.39139 Asia/Krasnoyarsk
65.53333 72.51667 Asia/Yekaterinburg
53.709 87.8014 Asia/Novokuznetsk
53.71028 91.6875 Asia/Krasnoyarsk
55.045 60.10833 Asia/Yekaterinburg
53.69417 88.06028 Asia/Novokuznetsk
61.0296 76.1136 Asia/Yekaterinburg
52.01972 85.91139 Asia/Omsk
56.21389 87.74722 Asia/Novokuznetsk
54.4589 83.3764 Asia/Novosibirsk
58.23583 92.48278 Asia/Krasnoyarsk
54.6567 86.1737 Asia/Novokuznetsk
66.65722 66.41833 Asia/Yekaterinburg
51.7 94.45 Asia/Krasnoyarsk
55.714 60.5528 Asia/Yekaterinburg
55.44753 78.32181 Asia/Novosibirsk
54.91028 64.43194 Asia/Yekaterinburg
55.45 65.33333 Asia/Yekaterinburg
54.36635 77.29805 Asia/Novosibirsk
52.566 78.9385 Asia/Omsk
56.00972 92.79167 Asia/Krasnoyarsk
58.3638 60.0407 Asia/Yekaterinburg
59.7666 60.2086 Asia/Yekaterinburg
54.9198 82.9909 Asia/Novosibirsk
54.8913 61.3969 Asia/Yekaterinburg
55.131 61.6792 Asia/Yekaterinburg
61.31333 63.33194 Asia/Yekaterinburg
58.31306 82.90889 Asia/Krasnoyarsk
58.68806 99.18444 Asia/Krasnoyarsk
55.0218 82.202 Asia/Novosibirsk
53.99 86.6621 Asia/Novokuznetsk
57.42972 60.05972 Asia/Yekaterinburg
61.00417 69.00194 Asia/Yekaterinburg
55.33333 86.08333 Asia/Novokuznetsk
55.53333 86.05 Asia/Novokuznetsk
69.37861 87.74389 Asia/Krasnoyarsk
56.289 62.5841 Asia/Yekaterinburg
55.8909 60.7616 Asia/Yekaterinburg
53.05222 60.64667 Asia/Yekaterinburg
59.7654 60.0085 Asia/Yekaterinburg
53.73772 78.04026 Asia/Novosibirsk
55.4808 60.2157 Asia/Yekaterinburg
56.20167 95.7175 Asia/Krasnoyarsk
56.84278 62.71111 Asia/Yekaterinburg
56.4185 61.9329 Asia/Yekaterinburg
53.7884 81.3423 Asia/Omsk
53.5347 87.2457 Asia/Novokuznetsk
55.05286 74.57511 Asia/Omsk
60.69111 60.42056 Asia/Yekaterinburg
54.6366 83.3045 Asia/Novosibirsk
56.11281 69.49015 Asia/Yekaterinburg
57.67052 63.071 Asia/Yekaterinburg
56.23583 96.065 Asia/Krasnoyarsk
54.28333 85.93333 Asia/Novokuznetsk
50.99417 81.46611 Asia/Omsk
51.95611 85.955 Asia/Omsk
69.40583 86.17778 Asia/Krasnoyarsk
55.95944 92.36194 Asia/Krasnoyarsk
56.704 60.0879 Asia/Yekaterinburg
56.0815 99.6342 Asia/Irkutsk
53.82361 91.28417 Asia/Krasnoyarsk
54.2222 83.381 Asia/Novosibirsk
55.15444 61.42972 Asia/Yekaterinburg
54.9749 60.3633 Asia/Yekaterinburg
57.04096 65.72018 Asia/Yekaterinburg
55.91056 94.89833 Asia/Krasnoyarsk
55.67167 84.39806 Asia/Novosibirsk
56.20778 89.53417 Asia/Krasnoyarsk
56.78028 62.04944 Asia/Yekaterinburg
52.53639 85.20722 Asia/Omsk
56.9083 60.8019 Asia/Yekaterinburg
55.6 86.2 Asia/Novokuznetsk
54.7551 83.0967 Asia/Novosibirsk
63.71194 66.67222 Asia/Yekaterinburg
54.4165 86.2976 Asia/Novokuznetsk
53.36056 83.76361 Asia/Omsk
55.35709 78.35697 Asia/Novosibirsk
56.9979 86.1452 Asia/Krasnoyarsk
57.00993 61.45776 Asia/Yekaterinburg
57.3555 61.86865 Asia/Yekaterinburg
56.6977 60.8369 Asia/Yekaterinburg
56.081 86.0285 Asia/Novokuznetsk
52.4926 82.7822 Asia/Omsk
57.85158 61.69627 Asia/Yekaterinburg
54.8523 83.106 Asia/Novosibirsk
56.2694 90.4993 Asia/Krasnoyarsk
52.655 90.09278 Asia/Krasnoyarsk
53.71556 91.42917 Asia/Krasnoyarsk
56.085 60.73139 Asia/Yekaterinburg
55.75556 60.70278 Asia/Yekaterinburg
56.25111 93.53194 Asia/Krasnoyarsk
57.24389 60.08389 Asia/Yekaterinburg
56.60056 84.88639 Asia/Krasnoyarsk
64.434 76.50261 Asia/Yekaterinburg
62.09611 77.475 Asia/Yekaterinburg
63.78977 74.52301 Asia/Yekaterinburg
61.61945 72.15546 Asia/Yekaterinburg
53.9202 102.0442 Asia/Irkutsk
56.5768 104.1217 Asia/Irkutsk
53.75 127.26667 Asia/Yakutsk
62.03389 129.73306 Asia/Yakutsk
47.53778 134.7525 Asia/Vladivostok
43.10562 131.87353 Asia/Vladivostok
56.1201 101.1636 Asia/Irkutsk
56.7938 105.7672 Asia/Irkutsk
58.00056 102.66194 Asia/Irkutsk
43.8025 132.00944 Asia/Vladivostok
52.75194 103.64528 Asia/Irkutsk
51.82605 107.60979 Asia/Irkutsk
66.41667 112.4 Asia/Yakutsk
55.16667 124.71667 Asia/Yakutsk
54.5559 100.5905 Asia/Irkutsk
43.30583 132.08167 Asia/Vladivostok
42.98361 132.40472 Asia/Vladivostok
51.4 128.13333 Asia/Yakutsk
44.59778 132.82111 Asia/Vladivostok
51.65944 103.70611 Asia/Irkutsk
52 127.7 Asia/Yakutsk
52.21389 104.1 Asia/Irkutsk
55.65 109.31667 Asia/Irkutsk
49.7838 129.4121 Asia/Yakutsk
51.273 108.843 Asia/Yakutsk
43.12583 133.13389 Asia/Vladivostok
56.6709 124.6491 Asia/Yakutsk
51.98333 116.58333 Asia/Yakutsk
42.83333 132.89472 Asia/Vladivostok
62.53528 113.96111 Asia/Yakutsk
46.47639 134.25694 Asia/Vladivostok
45.47944 133.42444 Asia/Vladivostok
60.72528 114.92778 Asia/Yakutsk
50.35083 106.44944 Asia/Irkutsk
50.0979 118.0369 Asia/Yakutsk
50.55199 137.01524 Asia/Vladivostok
48.48083 135.09278 Asia/Vladivostok
44.2661 135.0572 Asia/Vladivostok
52.29778 104.29639 Asia/Irkutsk
51.28333 106.5 Asia/Irkutsk
45.93389 133.7225 Asia/Vladivostok
52.03333 113.55 Asia/Yakutsk
53.15611 103.0675 Asia/Irkutsk
50.38333 116.51667 Asia/Yakutsk
43.1225 132.35833 Asia/Vladivostok
57.85056 114.19333 Asia/Irkutsk
50.26667 127.53333 Asia/Yakutsk
48.8 132.95 Asia/Vladivostok
46.81611 134.26 Asia/Vladivostok
50.91583 128.46111 Asia/Yakutsk
51.5151 104.1402 Asia/Irkutsk
66 111.5 Asia/Yakutsk
43.3675 132.19861 Asia/Vladivostok
44.16306 133.27333 Asia/Vladivostok
52.53667 103.88639 Asia/Irkutsk
50.22611 136.89944 Asia/Vladivostok
58.60306 125.38944 Asia/Yakutsk
44.56039 135.57009 Asia/Vladivostok
56.1325 101.61417 Asia/Irkutsk
54.1088 102.1648 Asia/Irkutsk
48.44306 135.13556 Asia/Vladivostok
52.93056 158.40278 Asia/Kamchatka
46.95812 142.73366 Asia/Sakhalin
53.1873 158.3805 Asia/Kamchatka
49.08686 140.25434 Asia/Vladivostok
48.9723 140.28784 Asia/Vladivostok
49.21667 143.11667 Asia/Magadan
53.0452 158.6483 Asia/Kamchatka
53.57389 142.94778 Asia/Sakhalin
53.14056 140.73 Asia/Vladivostok
46.67333 141.86333 Asia/Sakhalin
59.56667 150.8 Asia/Magadan
46.63277 142.7813 Asia/Sakhalin
47.05111 142.05694 Asia/Sakhalin
54.65382 19.91104 Europe/Kaliningrad
55.59944 37.03861 Europe/Moscow
62.26537 74.47906 Asia/Yekaterinburg
60.74985 72.85824 Asia/Yekaterinburg
61.25439 75.2124 Asia/Yekaterinburg
57.6198 63.0784 Asia/Yekaterinburg
55.75533 38.09578 Europe/Moscow
56.0038 40.33905 Europe/Moscow
62.43965 71.76029 Asia/Yekaterinburg
61.71982 75.36827 Asia/Yekaterinburg
60.97944 76.92421 Asia/Yekaterinburg
-1.9487 30.4347 Africa/Kigali
-1.49984 29.63497 Africa/Kigali
-2.479 29.5564 Africa/Kigali
-1.94995 30.05885 Africa/Kigali
-2.0597 29.3482 Africa/Kigali
-2.1597 30.5427 Africa/Kigali
-2.07444 29.75667 Africa/Kigali
-2.4846 28.9075 Africa/Kigali
-1.70278 29.25639 Africa/Kigali
-1.5763 30.0675 Africa/Kigali
-2.59667 29.73944 Africa/Kigali
24.08912 38.06374 Asia/Riyadh
25.02126 37.2685 Asia/Riyadh
31.67252 38.66374 Asia/Riyadh
21.21406 41.6331 Asia/Riyadh
26.57332 50.03694 Asia/Riyadh
28.38333 36.58333 Asia/Riyadh
30.49987 38.21603 Asia/Riyadh
24.49258 39.58572 Asia/Riyadh
26.48522 50.04046 Asia/Riyadh
16.59601 42.94435 Asia/Riyadh
29.96974 40.20641 Asia/Riyadh
26.64986 49.95223 Asia/Riyadh
17.1495 42.62537 Asia/Riyadh
26.70791 50.06194 Asia/Riyadh
22.79856 39.03493 Asia/Riyadh
20.00054 42.6052 Asia/Riyadh
17.4924 44.12766 Asia/Riyadh
21.42667 39.82611 Asia/Riyadh
18.30639 42.72917 Asia/Riyadh
16.88917 42.55111 Asia/Riyadh
21.51694 39.21917 Asia/Riyadh
27.52188 41.69073 Asia/Riyadh
27.35134 35.69014 Asia/Riyadh
26.32599 43.97497 Asia/Riyadh
25.9371 49.67761 Asia/Riyadh
23.78292 38.79047 Asia/Riyadh
26.29945 44.81542 Asia/Riyadh
26.30324 50.13528 Asia/Riyadh
25.36276 49.72504 Asia/Riyadh
21.27028 40.41583 Asia/Riyadh
20.46067 45.57792 Asia/Riyadh
24.68773 46.72185 Asia/Riyadh
25.86944 43.4973 Asia/Riyadh
30.97531 41.03808 Asia/Riyadh
19.14547 42.1201 Asia/Riyadh
31.33176 37.34282 Asia/Riyadh
26.24551 36.45248 Asia/Riyadh
28.31117 46.12729 Asia/Riyadh
26.5208 50.02452 Asia/Riyadh
25.38333 49.66667 Asia/Riyadh
25.41 49.58083 Asia/Riyadh
24.46861 39.61417 Asia/Riyadh
26.27944 50.20833 Asia/Riyadh
28.43905 48.49132 Asia/Riyadh
21.61694 39.69806 Asia/Riyadh
27.01122 49.65826 Asia/Riyadh
25.36457 49.56532 Asia/Riyadh
26.14422 43.65933 Asia/Riyadh
25.43333 49.63333 Asia/Riyadh
20.01288 41.46767 Asia/Riyadh
23.9065 42.91724 Asia/Riyadh
23.99146 47.16266 Asia/Riyadh
24.50772 44.39237 Asia/Riyadh
26.43442 50.10326 Asia/Riyadh
16.96887 42.83251 Asia/Riyadh
18.21639 42.50528 Asia/Riyadh
-9.43333 159.95 Pacific/Guadalcanal
-4.61667 55.45 Indian/Mahe
12.9096 23.47406 Africa/Khartoum
14.40118 33.51989 Africa/Khartoum
12.9061 31.2158 Africa/Khartoum
15.64453 32.47773 Africa/Khartoum
18.4254 37.729 Africa/Khartoum
13.01667 31.86667 Africa/Khartoum
13.15 33.93333 Africa/Khartoum
16.6915 33.4341 Africa/Khartoum
19.1059 37.3321 Africa/Khartoum
13.56907 33.56718 Africa/Khartoum
13.18087 32.73999 Africa/Khartoum
13.41667 33.66667 Africa/Khartoum
13.1629 32.66347 Africa/Khartoum
18.55 31.85 Africa/Khartoum
14.0361 33.1712 Africa/Khartoum
15.45099 36.39998 Africa/Khartoum
11.01667 29.71667 Africa/Khartoum
13.51667 35.76667 Africa/Khartoum
12.05 29.65 Africa/Khartoum
19.61745 37.21644 Africa/Khartoum
18.02158 33.98299 Africa/Khartoum
13.7 30.36667 Africa/Khartoum
17.70217 33.98638 Africa/Khartoum
13.31667 33.88333 Africa/Khartoum
11.8659 34.3869 Africa/Khartoum
12.71667 30.65 Africa/Khartoum
12.7 28.43333 Africa/Khartoum
13.18421 30.21669 Africa/Khartoum
14.8648 32.3668 Africa/Khartoum
11.03333 27.73333 Africa/Khartoum
14.2459 32.9891 Africa/Khartoum
15.55177 32.53241 Africa/Khartoum
13.45 22.45 Africa/Khartoum
14.9398 33.234 Africa/Khartoum
13.41667 34.63333 Africa/Khartoum
14.75264 33.29836 Africa/Khartoum
13.62793 25.34936 Africa/Khartoum
18.26196 33.90812 Africa/Khartoum
14.0012 32.3116 Africa/Khartoum
13.2 34.16667 Africa/Khartoum
17.59278 33.95917 Africa/Khartoum
11.7891 34.3592 Africa/Khartoum
12.35 29.25 Africa/Khartoum
11.4562 31.2285 Africa/Khartoum
63.82842 20.25972 Europe/Stockholm
64.75067 20.95279 Europe/Stockholm
65.31717 21.47944 Europe/Stockholm
65.58415 22.15465 Europe/Stockholm
67.85572 20.22513 Europe/Stockholm
65.82518 21.68864 Europe/Stockholm
55.42966 13.82041 Europe/Stockholm
57.64089 18.29602 Europe/Stockholm
56.87767 14.80906 Europe/Stockholm
57.7584 16.63733 Europe/Stockholm
59.61617 16.55276 Europe/Stockholm
57.18604 14.04001 Europe/Stockholm
57.10557 12.25078 Europe/Stockholm
58.38075 12.3234 Europe/Stockholm
59.53436 18.07758 Europe/Stockholm
59.8585 17.64543 Europe/Stockholm
59.51839 17.91128 Europe/Stockholm
58.34784 11.9424 Europe/Stockholm
59.19858 17.83317 Europe/Stockholm
59.2 17.9 Europe/Stockholm
58.28365 12.28864 Europe/Stockholm
55.37514 13.15691 Europe/Stockholm
59.4439 18.06872 Europe/Stockholm
62.39129 17.3063 Europe/Stockholm
59.36128 17.97114 Europe/Stockholm
59.33258 18.0649 Europe/Stockholm
59.36004 18.00086 Europe/Stockholm
59.42804 17.95093 Europe/Stockholm
59.19554 17.62525 Europe/Stockholm
58.39118 13.84506 Europe/Stockholm
60.61667 16.76667 Europe/Stockholm
57.7395 12.10642 Europe/Stockholm
63.1792 14.63566 Europe/Stockholm
59.33879 18.08487 Europe/Stockholm
57.26455 16.44837 Europe/Stockholm
63.29091 18.71525 Europe/Stockholm
59.27412 15.2066 Europe/Stockholm
58.753 17.00788 Europe/Stockholm
59.75799 18.70496 Europe/Stockholm
58.59419 16.1826 Europe/Stockholm
57.65307 14.69676 Europe/Stockholm
59.31053 18.16372 Europe/Stockholm
58.53706 15.03649 Europe/Stockholm
57.6554 12.01378 Europe/Stockholm
59.62157 17.85476 Europe/Stockholm
55.60587 13.00073 Europe/Stockholm
55.70584 13.19321 Europe/Stockholm
58.41086 15.62157 Europe/Stockholm
58.50517 13.15765 Europe/Stockholm
59.36667 18.13333 Europe/Stockholm
57.77051 12.26904 Europe/Stockholm
55.8708 12.83016 Europe/Stockholm
57.48719 12.07612 Europe/Stockholm
57.87096 11.98054 Europe/Stockholm
59.30978 14.10808 Europe/Stockholm
56.03129 14.15242 Europe/Stockholm
59.51404 15.99255 Europe/Stockholm
59.40316 17.94479 Europe/Stockholm
58.99587 16.20721 Europe/Stockholm
59.3793 13.50357 Europe/Stockholm
56.16156 15.58661 Europe/Stockholm
59.32667 14.52386 Europe/Stockholm
56.1706 14.86188 Europe/Stockholm
56.66157 16.36163 Europe/Stockholm
57.78145 14.15618 Europe/Stockholm
59.42268 17.83508 Europe/Stockholm
57.78596 14.30214 Europe/Stockholm
59.23705 17.98192 Europe/Stockholm
56.04673 12.69437 Europe/Stockholm
56.15905 13.76638 Europe/Stockholm
62.63228 17.93794 Europe/Stockholm
59.16775 18.14478 Europe/Stockholm
56.67446 12.85676 Europe/Stockholm
57.70716 11.96679 Europe/Stockholm
60.67452 17.14174 Europe/Stockholm
60.60357 15.62597 Europe/Stockholm
58.17347 13.55068 Europe/Stockholm
56.90552 12.49118 Europe/Stockholm
55.83928 13.30393 Europe/Stockholm
59.36661 16.5077 Europe/Stockholm
59.63607 17.07768 Europe/Stockholm
59.34 17.94 Europe/Stockholm
60.4858 15.43714 Europe/Stockholm
57.72101 12.9401 Europe/Stockholm
59.33333 18.28333 Europe/Stockholm
59.2978 18.0514 Europe/Stockholm
56.2428 12.86219 Europe/Stockholm
57.93033 12.53345 Europe/Stockholm
59.47944 18.29967 Europe/Stockholm
57.69195 11.91605 Europe/Stockholm
1.28967 103.85007 Asia/Singapore
-15.93872 -5.71675 Atlantic/St_Helena
46.155 15.05333 Europe/Ljubljana
46.35917 15.11028 Europe/Ljubljana
46.42005 15.87018 Europe/Ljubljana
45.80397 15.16886 Europe/Ljubljana
46.55472 15.64667 Europe/Ljubljana
46.05108 14.50513 Europe/Ljubljana
46.23887 14.35561 Europe/Ljubljana
45.54694 13.72944 Europe/Ljubljana
46.23092 15.26044 Europe/Ljubljana
78.2186 15.64007 Arctic/Longyearbyen
48.88836 21.68479 Europe/Bratislava
48.62858 21.71954 Europe/Bratislava
49.29859 20.6862 Europe/Bratislava
48.94464 20.56154 Europe/Bratislava
48.98857 22.15099 Europe/Bratislava
48.66008 20.53758 Europe/Bratislava
48.38284 20.0224 Europe/Bratislava
48.99839 21.23393 Europe/Bratislava
49.06144 20.29798 Europe/Bratislava
48.75434 21.9195 Europe/Bratislava
48.71395 21.25808 Europe/Bratislava
49.1357 20.43352 Europe/Bratislava
48.93707 21.91625 Europe/Bratislava
49.29175 21.27271 Europe/Bratislava
48.57442 19.15324 Europe/Bratislava
48.38553 18.40063 Europe/Bratislava
49.22315 18.73941 Europe/Bratislava
48.59184 18.84958 Europe/Bratislava
48.3774 17.58723 Europe/Bratislava
48.89452 18.04436 Europe/Bratislava
48.8449 17.22635 Europe/Bratislava
48.67922 17.36697 Europe/Bratislava
48.15127 17.88062 Europe/Bratislava
49.0748 19.30751 Europe/Bratislava
49.12494 18.32596 Europe/Bratislava
48.77446 18.6275 Europe/Bratislava
49.12153 18.42169 Europe/Bratislava
48.59479 17.82591 Europe/Bratislava
48.28986 17.26664 Europe/Bratislava
48.62861 18.38456 Europe/Bratislava
47.98544 18.16195 Europe/Bratislava
48.75763 17.8309 Europe/Bratislava
48.30763 18.08453 Europe/Bratislava
49.06651 18.92399 Europe/Bratislava
48.43604 17.02188 Europe/Bratislava
48.33249 19.66708 Europe/Bratislava
49.08061 19.62218 Europe/Bratislava
48.21563 18.60705 Europe/Bratislava
49.3 18.78333 Europe/Bratislava
47.76356 18.12263 Europe/Bratislava
48.43174 17.8031 Europe/Bratislava
48.7276 18.76012 Europe/Bratislava
48.19001 17.72747 Europe/Bratislava
47.99268 17.61211 Europe/Bratislava
48.95981 18.16634 Europe/Bratislava
49.20983 19.30341 Europe/Bratislava
48.56082 19.41954 Europe/Bratislava
49.43503 18.78895 Europe/Bratislava
48.80431 19.63631 Europe/Bratislava
48.14816 17.10674 Europe/Bratislava
48.73946 19.15349 Europe/Bratislava
48.7213 18.25754 Europe/Bratislava
8.33833 -13.07194 Africa/Freetown
8.76667 -12.78333 Africa/Freetown
8.88333 -12.05 Africa/Freetown
8.68333 -12.53333 Africa/Freetown
8.41667 -10.83333 Africa/Freetown
7.87667 -11.1875 Africa/Freetown
9.58333 -11.55 Africa/Freetown
8.484 -13.22994 Africa/Freetown
7.96472 -11.73833 Africa/Freetown
43.93333 12.45 Europe/San_Marino
12.58333 -16.27194 Africa/Dakar
13.15 -14.11667 Africa/Dakar
14.63333 -16.23333 Africa/Dakar
14.78333 -16.96667 Africa/Dakar
12.70806 -15.55694 Africa/Dakar
16.01793 -16.48962 Africa/Dakar
16.4625 -15.70083 Africa/Dakar
14.77389 -17.06028 Africa/Dakar
14.35 -16.41667 Africa/Dakar
14.75 -17.4 Africa/Dakar
13.75 -15.8 Africa/Dakar
14.51278 -17.005 Africa/Dakar
15.3338 -15.4766 Africa/Dakar
15.1097 -16.6218 Africa/Dakar
14.79032 -15.90803 Africa/Dakar
12.88333 -14.95 Africa/Dakar
12.5579 -12.1743 Africa/Dakar
14.91917 -17.12111 Africa/Dakar
14.16667 -16.83333 Africa/Dakar
14.26667 -15.95 Africa/Dakar
14.70889 -17.45528 Africa/Dakar
15.34844 -15.47993 Africa/Dakar
14.6937 -17.44406 Africa/Dakar
12.81028 -16.22639 Africa/Dakar
2.6185 44.8938 Africa/Mogadishu
1.78784 44.52999 Africa/Mogadishu
11.47197 49.87282 Africa/Mogadishu
2.03711 45.34375 Africa/Mogadishu
3.79376 42.5445 Africa/Mogadishu
-0.35817 42.54536 Africa/Mogadishu
0.48829 42.78535 Africa/Mogadishu
2.78087 45.50048 Africa/Mogadishu
0.06968 42.74497 Africa/Mogadishu
9.56 44.065 Africa/Mogadishu
6.76972 47.43083 Africa/Mogadishu
7.9803 49.8164 Africa/Mogadishu
10.61616 47.36795 Africa/Mogadishu
3.84878 47.18064 Africa/Mogadishu
2.80257 44.07806 Africa/Mogadishu
3.85375 45.56744 Africa/Mogadishu
9.52212 45.53363 Africa/Mogadishu
11.28421 49.18158 Africa/Mogadishu
10.43959 45.01432 Africa/Mogadishu
4.73583 45.20361 Africa/Mogadishu
3.11383 43.6498 Africa/Mogadishu
2.34464 42.27644 Africa/Mogadishu
2.1381 45.1212 Africa/Mogadishu
8.47738 47.35971 Africa/Mogadishu
5.86638 -55.16682 America/Paramaribo
5.7 -55.23333 America/Paramaribo
4.09504 30.67792 Africa/Juba
4.57213 28.39549 Africa/Juba
7.70286 27.9953 Africa/Juba
8.76816 27.40019 Africa/Juba
4.41333 32.56778 Africa/Juba
7.27369 28.68223 Africa/Juba
6.802 29.69123 Africa/Juba
9.53342 31.66048 Africa/Juba
4.85165 31.58247 Africa/Juba
8.53324 28.10037 Africa/Juba
6.20593 31.55633 Africa/Juba
0.33654 6.72732 Africa/Sao_Tome
13.5 -88.86667 America/El_Salvador
13.35 -88.45 America/El_Salvador
13.73472 -89.15139 America/El_Salvador
13.73417 -89.71472 America/El_Salvador
13.71889 -89.72417 America/El_Salvador
13.86667 -88.63333 America/El_Salvador
13.63333 -88.8 America/El_Salvador
13.48333 -88.46667 America/El_Salvador
13.99417 -89.55972 America/El_Salvador
13.68935 -89.18718 America/El_Salvador
13.38333 -88.35 America/El_Salvador
13.48333 -88.18333 America/El_Salvador
13.78333 -88.91667 America/El_Salvador
13.65889 -89.18306 America/El_Salvador
13.7 -88.1 America/El_Salvador
13.835 -89.27444 America/El_Salvador
13.28333 -88.55 America/El_Salvador
13.67694 -89.27972 America/El_Salvador
14.33333 -89.45 America/El_Salvador
13.74028 -89.21306 America/El_Salvador
13.33694 -87.84389 America/El_Salvador
13.48833 -89.32222 America/El_Salvador
13.74472 -89.67306 America/El_Salvador
13.70167 -89.10944 America/El_Salvador
13.72417 -89.17028 America/El_Salvador
13.73611 -89.18139 America/El_Salvador
13.71667 -88.93333 America/El_Salvador
13.98667 -89.68111 America/El_Salvador
14.03333 -88.93333 America/El_Salvador
13.74556 -89.20639 America/El_Salvador
13.80722 -89.17917 America/El_Salvador
13.67306 -89.24083 America/El_Salvador
13.92139 -89.845 America/El_Salvador
13.95722 -89.18972 America/El_Salvador
13.59278 -89.8275 America/El_Salvador
18.026 -63.04582 America/Lower_Princes
33.96736 36.65741 Asia/Damascus
35.26548 36.71062 Asia/Damascus
34.88902 35.88659 Asia/Damascus
36.47362 37.09672 Asia/Damascus
34.66842 36.25995 Asia/Damascus
34.84063 36.73093 Asia/Damascus
32.73564 36.06694 Asia/Damascus
34.56077 38.28055 Asia/Damascus
36.34814 37.5309 Asia/Damascus
35.29159 36.74876 Asia/Damascus
34.85444 40.60833 Asia/Damascus
32.82944 36.15933 Asia/Damascus
35.86447 36.80591 Asia/Damascus
36.13809 36.45322 Asia/Damascus
35.01127 37.05324 Asia/Damascus
34.81836 36.11376 Asia/Damascus
33.43544 36.08148 Asia/Damascus
32.99233 36.06018 Asia/Damascus
34.1527 36.74152 Asia/Damascus
36.3771 36.99238 Asia/Damascus
35.0653 36.3406 Asia/Damascus
36.52815 37.95495 Asia/Damascus
36.01337 36.67254 Asia/Damascus
35.44063 36.64995 Asia/Damascus
35.37449 36.60298 Asia/Damascus
36.11684 36.51528 Asia/Damascus
35.61376 36.56449 Asia/Damascus
34.89469 36.49582 Asia/Damascus
35.81426 36.32062 Asia/Damascus
33.8074 36.74036 Asia/Damascus
36.8175 38.01111 Asia/Damascus
35.36137 35.92562 Asia/Damascus
33.53994 36.36716 Asia/Damascus
33.01797 36.12806 Asia/Damascus
35.93062 36.63393 Asia/Damascus
34.72682 36.72339 Asia/Damascus
33.55899 36.36432 Asia/Damascus
35.13179 36.75783 Asia/Damascus
35.25995 36.60572 Asia/Damascus
36.20124 37.16117 Asia/Damascus
33.57175 36.4027 Asia/Damascus
33.5102 36.29128 Asia/Damascus
36.15694 37.70778 Asia/Damascus
35.33588 40.14084 Asia/Damascus
37.07279 40.65199 Asia/Damascus
33.45897 36.23742 Asia/Damascus
35.95554 36.71393 Asia/Damascus
35.18188 35.94871 Asia/Damascus
33.72488 36.1005 Asia/Damascus
36.58662 37.04628 Asia/Damascus
33.61033 36.3107 Asia/Damascus
35.84688 38.54427 Asia/Damascus
32.70896 36.56951 Asia/Damascus
33.07227 36.18382 Asia/Damascus
36.07696 37.37251 Asia/Damascus
34.92653 36.73238 Asia/Damascus
35.95 39.01667 Asia/Damascus
34.02547 36.72837 Asia/Damascus
33.73848 36.60071 Asia/Damascus
34.50914 36.57975 Asia/Damascus
33.12594 35.82461 Asia/Damascus
34.22956 37.24066 Asia/Damascus
35.02089 40.45346 Asia/Damascus
35.51484 35.77684 Asia/Damascus
33.3572 36.24278 Asia/Damascus
36.50237 40.74772 Asia/Damascus
32.74932 36.30994 Asia/Damascus
36.36994 37.51788 Asia/Damascus
36.51598 36.86305 Asia/Damascus
36.21252 36.76978 Asia/Damascus
34.45 40.91861 Asia/Damascus
34.6927 40.827 Asia/Damascus
-26.31667 31.13333 Africa/Mbabane
-26.48333 31.36667 Africa/Mbabane
-26.46667 31.2 Africa/Mbabane
21.46122 -71.14188 America/Grand_Turk
11.0297 20.2827 Africa/Ndjamena
13.82916 20.8324 Africa/Ndjamena
9.1429 18.3923 Africa/Ndjamena
9.3642 14.9046 Africa/Ndjamena
13.2954 19.6966 Africa/Ndjamena
12.10672 15.0444 Africa/Ndjamena
13.64084 16.49007 Africa/Ndjamena
8.56667 16.08333 Africa/Ndjamena
12.18441 18.69303 Africa/Ndjamena
9.7639 14.1539 Africa/Ndjamena
12.996 15.72927 Africa/Ndjamena
12.47554 15.43647 Africa/Ndjamena
14.12116 15.3103 Africa/Ndjamena
9.3952 16.2989 Africa/Ndjamena
9.45149 18.94493 Africa/Ndjamena
8.91256 17.55392 Africa/Ndjamena
9.30859 15.80658 Africa/Ndjamena
11.80909 15.86323 Africa/Ndjamena
8.65 16.85 Africa/Ndjamena
10.28056 15.37222 Africa/Ndjamena
11.9801 18.2138 Africa/Ndjamena
8.98327 16.31991 Africa/Ndjamena
13.2154 18.3353 Africa/Ndjamena
-49.35 70.21667 Indian/Kerguelen
6.33333 1.53333 Africa/Lome
6.42611 1.21333 Africa/Lome
9.03333 1.41667 Africa/Lome
8.56667 0.98333 Africa/Lome
8.98333 1.13333 Africa/Lome
6.95 1.16667 Africa/Lome
9.76806 1.10528 Africa/Lome
10.35917 0.47083 Africa/Lome
6.13748 1.21227 Africa/Lome
6.9 0.63333 Africa/Lome
9.55111 1.18611 Africa/Lome
10.86389 0.205 Africa/Lome
9.25 0.78333 Africa/Lome
9.35 1.26667 Africa/Lome
7.58333 0.6 Africa/Lome
7.53333 1.13333 Africa/Lome
6.23333 1.6 Africa/Lome
7.88481 98.40008 Asia/Bangkok
7.88489 98.38557 Asia/Bangkok
12.27081 99.87203 Asia/Bangkok
13.11189 99.94467 Asia/Bangkok
7.55633 99.61141 Asia/Bangkok
8.16453 99.68039 Asia/Bangkok
17.61289 99.21612 Asia/Bangkok
15.46063 99.89166 Asia/Bangkok
13.96118 99.64122 Asia/Bangkok
13.9 99.76667 Asia/Bangkok
16.86968 99.12898 Asia/Bangkok
9.14011 99.33311 Asia/Bangkok
17.00778 99.823 Asia/Bangkok
17.51692 99.75978 Asia/Bangkok
17.31597 99.83186 Asia/Bangkok
18.62828 98.89572 Asia/Bangkok
18.74486 99.11953 Asia/Bangkok
8.17911 99.85425 Asia/Bangkok
13.53671 99.81712 Asia/Bangkok
9.96583 98.63476 Asia/Bangkok
12.38466 99.90983 Asia/Bangkok
11.82098 99.7841 Asia/Bangkok
9.11072 99.23208 Asia/Bangkok
7.89059 98.3981 Asia/Bangkok
13.69234 99.84969 Asia/Bangkok
19.19203 99.87883 Asia/Bangkok
18.52617 98.93936 Asia/Bangkok
8.43333 99.96667 Asia/Bangkok
16.71667 98.56667 Asia/Bangkok
20.43353 99.87617 Asia/Bangkok
16.98403 98.51665 Asia/Bangkok
20.14675 99.85256 Asia/Bangkok
15.751 99.78925 Asia/Bangkok
9.94561 99.07847 Asia/Bangkok
18.58054 99.00745 Asia/Bangkok
18.29232 99.49277 Asia/Bangkok
12.08283 99.85431 Asia/Bangkok
8.07257 98.91052 Asia/Bangkok
13.24025 99.82428 Asia/Bangkok
16.0617 99.86058 Asia/Bangkok
7.91779 98.33322 Asia/Bangkok
14.00412 99.54832 Asia/Bangkok
16.48344 99.52153 Asia/Bangkok
7.78933 99.63478 Asia/Bangkok
12.57065 99.95876 Asia/Bangkok
18.68703 98.91939 Asia/Bangkok
19.16242 99.99342 Asia/Bangkok
13.51824 99.95469 Asia/Bangkok
10.5 99.16667 Asia/Bangkok
13.62 99.59169 Asia/Bangkok
19.90858 99.8325 Asia/Bangkok
18.79038 98.98468 Asia/Bangkok
12.8 99.96667 Asia/Bangkok
14.32361 99.51611 Asia/Bangkok
17.04325 99.08128 Asia/Bangkok
13.81629 99.87739 Asia/Bangkok
8.80036 99.36372 Asia/Bangkok
17.03056 99.7525 Asia/Bangkok
11.21259 99.51167 Asia/Bangkok
13.69157 99.92982 Asia/Bangkok
16.65 103.3 Asia/Bangkok
17.2675 103.22472 Asia/Bangkok
7.20411 100.54508 Asia/Bangkok
14.98333 102.65 Asia/Bangkok
13.64172 100.83272 Asia/Bangkok
17.3072 102.18886 Asia/Bangkok
15.79408 104.1451 Asia/Bangkok
6.86617 101.36894 Asia/Bangkok
6.53995 101.28128 Asia/Bangkok
14.5945 100.33825 Asia/Bangkok
15.65778 101.10603 Asia/Bangkok
15.19319 104.8628 Asia/Bangkok
17.30097 101.7685 Asia/Bangkok
14.22689 100.7155 Asia/Bangkok
13.50325 102.18115 Asia/Bangkok
17.62557 100.09421 Asia/Bangkok
15.37939 100.0245 Asia/Bangkok
17.41567 102.78589 Asia/Bangkok
15.22483 104.85289 Asia/Bangkok
12.24364 102.51514 Asia/Bangkok
12.95772 99.90555 Asia/Bangkok
14.56739 100.72598 Asia/Bangkok
16.16003 100.59656 Asia/Bangkok
12.62137 102.0048 Asia/Bangkok
17.85003 102.58139 Asia/Bangkok
16.22094 100.41978 Asia/Bangkok
6.25947 102.05461 Asia/Bangkok
14.88181 103.49364 Asia/Bangkok
14.47418 100.12218 Asia/Bangkok
6.02977 101.96586 Asia/Bangkok
7.19882 100.5951 Asia/Bangkok
15.11481 104.32938 Asia/Bangkok
13.17372 100.93111 Asia/Bangkok
14.88786 100.40464 Asia/Bangkok
17.92851 103.95519 Asia/Bangkok
17.47531 103.45753 Asia/Bangkok
6.62314 100.06676 Asia/Bangkok
12.66644 100.90073 Asia/Bangkok
14.53333 100.91667 Asia/Bangkok
13.41456 100.00264 Asia/Bangkok
13.54753 100.27362 Asia/Bangkok
13.59934 100.59675 Asia/Bangkok
13.72698 100.21526 Asia/Bangkok
17.16116 104.14725 Asia/Bangkok
13.81411 102.07222 Asia/Bangkok
6.63883 100.42342 Asia/Bangkok
16.0567 103.65309 Asia/Bangkok
12.68095 101.25798 Asia/Bangkok
7.77768 100.32134 Asia/Bangkok
6.29678 101.72844 Asia/Bangkok
14.60592 103.12081 Asia/Bangkok
14.04992 101.36864 Asia/Bangkok
16.88425 101.88467 Asia/Bangkok
16.3765 102.12853 Asia/Bangkok
13.65854 100.53362 Asia/Bangkok
14.72526 100.79536 Asia/Bangkok
14.35167 100.57739 Asia/Bangkok
18.14589 100.14103 Asia/Bangkok
18.03333 103.16667 Asia/Bangkok
16.82481 100.25858 Asia/Bangkok
16.44184 100.34879 Asia/Bangkok
15.24467 105.22908 Asia/Bangkok
16.41904 101.16056 Asia/Bangkok
7.61786 100.07792 Asia/Bangkok
13.46804 101.09532 Asia/Bangkok
13.74787 101.3553 Asia/Bangkok
13.44581 101.18445 Asia/Bangkok
14.45736 100.36989 Asia/Bangkok
6.86814 101.25009 Asia/Bangkok
14.01346 100.53049 Asia/Bangkok
8.35109 100.20195 Asia/Bangkok
13.91301 100.49883 Asia/Bangkok
14.70802 101.41614 Asia/Bangkok
13.86075 100.51477 Asia/Bangkok
15.18014 102.25694 Asia/Bangkok
15.99025 101.06183 Asia/Bangkok
17.87847 102.742 Asia/Bangkok
14.34062 100.86733 Asia/Bangkok
17.20406 102.44068 Asia/Bangkok
6.42639 101.82308 Asia/Bangkok
14.6377 102.79138 Asia/Bangkok
18.79292 100.72972 Asia/Bangkok
17.77036 102.18947 Asia/Bangkok
15.70472 100.13717 Asia/Bangkok
14.97066 102.10196 Asia/Bangkok
17.41081 104.77856 Asia/Bangkok
13.8196 100.04427 Asia/Bangkok
14.20463 101.21295 Asia/Bangkok
14.46281 100.60832 Asia/Bangkok
16.54531 104.72351 Asia/Bangkok
16.18483 103.30067 Asia/Bangkok
14.79808 100.65397 Asia/Bangkok
16.77983 101.24225 Asia/Bangkok
17.49052 101.72749 Asia/Bangkok
12.48164 102.07375 Asia/Bangkok
17.4257 102.56692 Asia/Bangkok
16.541 104.05004 Asia/Bangkok
13.75 100.51667 Asia/Bangkok
13.6533 100.25972 Asia/Bangkok
12.77972 101.64831 Asia/Bangkok
16.44671 102.833 Asia/Bangkok
14.52541 102.24591 Asia/Bangkok
14.06467 100.64578 Asia/Bangkok
16.70008 104.0902 Asia/Bangkok
15.65558 103.58361 Asia/Bangkok
16.28053 101.95422 Asia/Bangkok
14.64056 104.64992 Asia/Bangkok
16.33839 103.57564 Asia/Bangkok
16.43281 103.50658 Asia/Bangkok
14.58617 100.99758 Asia/Bangkok
16.10864 102.25808 Asia/Bangkok
13.95114 101.71769 Asia/Bangkok
7.00836 100.47668 Asia/Bangkok
14.90598 105.07836 Asia/Bangkok
17.98372 100.05217 Asia/Bangkok
15.2085 101.77138 Asia/Bangkok
16.5443 102.09924 Asia/Bangkok
16.18953 100.85958 Asia/Bangkok
13.3622 100.98345 Asia/Bangkok
14.72844 102.16524 Asia/Bangkok
12.60961 102.10447 Asia/Bangkok
15.81047 102.02881 Asia/Bangkok
15.18636 100.12353 Asia/Bangkok
15.2 101.13333 Asia/Bangkok
13.6882 101.07156 Asia/Bangkok
15.58552 102.42587 Asia/Bangkok
5.77434 101.07231 Asia/Bangkok
16.01667 103.95 Asia/Bangkok
14.02775 100.75603 Asia/Bangkok
12.93333 100.88333 Asia/Bangkok
17.14272 102.97261 Asia/Bangkok
16.05997 102.73097 Asia/Bangkok
13.59072 100.10748 Asia/Bangkok
14.61544 100.72731 Asia/Bangkok
13.97738 100.77776 Asia/Bangkok
16.75847 100.11742 Asia/Bangkok
14.892 100.31728 Asia/Bangkok
13.54297 100.99333 Asia/Bangkok
14.22783 100.57589 Asia/Bangkok
16.02781 100.37917 Asia/Bangkok
14.02188 100.17183 Asia/Bangkok
13.04704 100.92891 Asia/Bangkok
13.805 100.47283 Asia/Bangkok
16.57831 100.30034 Asia/Bangkok
13.91783 100.42403 Asia/Bangkok
14.37394 100.48528 Asia/Bangkok
17.699 103.25957 Asia/Bangkok
12.72543 101.0553 Asia/Bangkok
13.07147 101.00314 Asia/Bangkok
13.99904 100.54962 Asia/Bangkok
13.58333 100.81667 Asia/Bangkok
13.69276 102.50128 Asia/Bangkok
15.85851 104.62883 Asia/Bangkok
14.89944 101.70833 Asia/Bangkok
7.8894 98.38523 Asia/Bangkok
7.84468 98.33897 Asia/Bangkok
7.90963 98.40248 Asia/Bangkok
38.31408 69.03784 Asia/Dushanbe
37.80396 69.64416 Asia/Dushanbe
37.71485 68.83456 Asia/Dushanbe
39.9142 69.00328 Asia/Dushanbe
38.51437 68.23327 Asia/Dushanbe
37.83399 68.78186 Asia/Dushanbe
39.49524 67.60931 Asia/Dushanbe
37.49219 69.40356 Asia/Dushanbe
38.55632 69.01354 Asia/Dushanbe
38.38917 69.32272 Asia/Dushanbe
37.91453 69.78303 Asia/Dushanbe
37.58823 68.65886 Asia/Dushanbe
37.48974 71.55304 Asia/Dushanbe
36.72484 71.61331 Asia/Dushanbe
38.52504 68.55124 Asia/Dushanbe
38.53575 68.77905 Asia/Dushanbe
38.09578 69.33998 Asia/Dushanbe
37.61366 69.70611 Asia/Dushanbe
38.46613 68.80533 Asia/Dushanbe
40.16704 69.50398 Asia/Dushanbe
40.28527 69.62792 Asia/Dushanbe
40.28434 70.44257 Asia/Dushanbe
40.12648 70.62526 Asia/Dushanbe
40.23417 69.69481 Asia/Dushanbe
-8.55861 125.57361 Asia/Dili
39.51075 54.36713 Asia/Ashgabat
37.34821 59.61431 Asia/Ashgabat
39.20611 54.59056 Asia/Ashgabat
39.24463 55.51536 Asia/Ashgabat
38.05067 58.20559 Asia/Ashgabat
38.43624 57.43158 Asia/Ashgabat
37.95 58.38333 Asia/Ashgabat
37.88757 58.516 Asia/Ashgabat
41.65041 59.9164 Asia/Ashgabat
40.02216 52.95518 Asia/Ashgabat
42.32773 59.15442 Asia/Ashgabat
42.12824 59.67101 Asia/Ashgabat
41.83625 59.96662 Asia/Ashgabat
37.29886 62.35975 Asia/Ashgabat
37.38338 60.50544 Asia/Ashgabat
39.4816 62.91374 Asia/Ashgabat
38.78392 63.88035 Asia/Ashgabat
37.59378 61.83031 Asia/Ashgabat
37.83574 65.21058 Asia/Ashgabat
37.81244 66.04656 Asia/Ashgabat
39.07328 63.57862 Asia/Ashgabat
37.61852 62.16715 Asia/Ashgabat
41.18746 61.4036 Asia/Ashgabat
36.40291 10.14292 Africa/Tunis
36.83408 10.04057 Africa/Tunis
36.81897 10.16579 Africa/Tunis
33.91968 8.13352 Africa/Tunis
32.92967 10.45177 Africa/Tunis
35.57244 8.67031 Africa/Tunis
36.78333 10.63333 Africa/Tunis
35.89174 8.55276 Africa/Tunis
35.82539 10.63699 Africa/Tunis
36.08497 9.37082 Africa/Tunis
35.03823 9.48494 Africa/Tunis
35.78333 10.8 Africa/Tunis
34.74056 10.76028 Africa/Tunis
36.67931 10.29195 Africa/Tunis
36.76806 10.27528 Africa/Tunis
35.41808 10.99475 Africa/Tunis
36.57861 10.85861 Africa/Tunis
36.84757 11.09386 Africa/Tunis
33.70439 8.96903 Africa/Tunis
35.64773 10.89046 Africa/Tunis
36.86108 10.33161 Africa/Tunis
36.37419 9.90651 Africa/Tunis
37.0629 10.11829 Africa/Tunis
34.425 8.78417 Africa/Tunis
33.88146 10.0982 Africa/Tunis
33.87309 7.87765 Africa/Tunis
36.45606 10.73763 Africa/Tunis
33.80813 10.99228 Africa/Tunis
37.04045 9.66557 Africa/Tunis
35.72917 10.58082 Africa/Tunis
37.23618 9.91448 Africa/Tunis
36.68312 10.58431 Africa/Tunis
37.15368 9.78594 Africa/Tunis
37.23737 9.86313 Africa/Tunis
36.80803 10.0972 Africa/Tunis
36.64964 9.61231 Africa/Tunis
36.50114 8.78024 Africa/Tunis
33.50398 11.11216 Africa/Tunis
35.62231 10.75696 Africa/Tunis
33.87576 10.85745 Africa/Tunis
35.8609 10.60313 Africa/Tunis
36.72866 10.34163 Africa/Tunis
36.81806 10.305 Africa/Tunis
33.46632 9.0203 Africa/Tunis
37.16667 9.75 Africa/Tunis
36.46798 10.75167 Africa/Tunis
37.27442 9.87391 Africa/Tunis
36.46917 10.78222 Africa/Tunis
36.72564 9.18169 Africa/Tunis
35.50056 11.06056 Africa/Tunis
35.23722 11.115 Africa/Tunis
34.3827 8.15549 Africa/Tunis
35.70915 10.67397 Africa/Tunis
35.6781 10.09633 Africa/Tunis
35.16758 8.83651 Africa/Tunis
34.75 10.78333 Africa/Tunis
35.78333 10.83333 Africa/Tunis
36.67446 10.15633 Africa/Tunis
34.32081 8.40158 Africa/Tunis
36.87818 10.32466 Africa/Tunis
35.50472 11.06222 Africa/Tunis
36.17424 8.70486 Africa/Tunis
35.3 10.71667 Africa/Tunis
36.4 10.61667 Africa/Tunis
33.89152 9.79629 Africa/Tunis
35.61739 10.99466 Africa/Tunis
37.1691 10.03478 Africa/Tunis
35.8691 10.5653 Africa/Tunis
36.44766 10.7557 Africa/Tunis
-21.13333 -175.2 Pacific/Tongatapu
37.57362 44.28716 Europe/Istanbul
39.82 34.80444 Europe/Istanbul
37.3405 40.82558 Europe/Istanbul
36.95589 36.2 Europe/Istanbul
39.63806 34.46722 Europe/Istanbul
37.37556 27.26778 Europe/Istanbul
37.34026 28.14279 Europe/Istanbul
38.29556 31.17778 Europe/Istanbul
38.10228 35.35704 Europe/Istanbul
37.23528 39.76306 Europe/Istanbul
39.17321 41.45593 Europe/Istanbul
38.49239 43.38311 Europe/Istanbul
39.71011 39.70213 Europe/Istanbul
38.67351 29.4058 Europe/Istanbul
38.32292 26.76403 Europe/Istanbul
38.6296 34.91198 Europe/Istanbul
37.16708 38.79392 Europe/Istanbul
38.49533 27.6997 Europe/Istanbul
39.10829 39.54711 Europe/Istanbul
38.1519 27.36223 Europe/Istanbul
38.08877 27.73508 Europe/Istanbul
36.5017 30.52723 Europe/Istanbul
39.54237 29.49866 Europe/Istanbul
38.50667 42.28167 Europe/Istanbul
36.91876 34.8784 Europe/Istanbul
38.6908 35.5538 Europe/Istanbul
39.91361 28.15778 Europe/Istanbul
36.97614 38.42495 Europe/Istanbul
39.81012 35.18596 Europe/Istanbul
39.18554 27.60945 Europe/Istanbul
38.96338 41.02931 Europe/Istanbul
37.7482 27.40614 Europe/Istanbul
37.75502 39.31667 Europe/Istanbul
39.74833 37.01611 Europe/Istanbul
37.52278 42.45944 Europe/Istanbul
39.0882 28.97767 Europe/Istanbul
38.14194 41.00806 Europe/Istanbul
37.24972 42.46944 Europe/Istanbul
36.37778 33.93444 Europe/Istanbul
37.93262 41.94025 Europe/Istanbul
37.41926 31.84527 Europe/Istanbul
36.91694 31.09889 Europe/Istanbul
38.93925 33.5386 Europe/Istanbul
38.10444 30.54861 Europe/Istanbul
37.30514 44.5742 Europe/Istanbul
37.95137 27.36849 Europe/Istanbul
38.19686 26.83902 Europe/Istanbul
39.35186 36.40976 Europe/Istanbul
38.07944 31.36639 Europe/Istanbul
37.92448 28.92516 Europe/Istanbul
38.46472 30.26946 Europe/Istanbul
38.48258 28.14774 Europe/Istanbul
36.26917 36.56722 Europe/Istanbul
39.57716 32.14132 Europe/Istanbul
37.48685 37.29961 Europe/Istanbul
39.23361 42.86344 Europe/Istanbul
39.98031 41.67635 Europe/Istanbul
37.07417 36.24778 Europe/Istanbul
38.73728 34.03866 Europe/Istanbul
36.82944 28.77083 Europe/Istanbul
38.2278 27.96955 Europe/Istanbul
37.0778 41.2178 Europe/Istanbul
37.00972 37.79417 Europe/Istanbul
37.96583 34.67935 Europe/Istanbul
38.625 34.71222 Europe/Istanbul
37.91631 28.32225 Europe/Istanbul
36.64389 33.43885 Europe/Istanbul
38.74525 41.50693 Europe/Istanbul
37.21807 28.3665 Europe/Istanbul
39.06147 34.38286 Europe/Istanbul
37.31639 27.78389 Europe/Istanbul
37.4247 41.33934 Europe/Istanbul
36.79526 34.61792 Europe/Istanbul
38.60754 27.06938 Europe/Istanbul
36.855 28.27417 Europe/Istanbul
37.31309 40.74357 Europe/Istanbul
38.61202 27.42646 Europe/Istanbul
36.78667 31.44306 Europe/Istanbul
39.14265 42.54008 Europe/Istanbul
38.35018 38.31667 Europe/Istanbul
38.45917 40.6475 Europe/Istanbul
39.42417 29.98333 Europe/Istanbul
37.85562 27.2566 Europe/Istanbul
37.92717 41.70282 Europe/Istanbul
36.32573 30.30302 Europe/Istanbul
39.09514 33.07989 Europe/Istanbul
38.49722 41.01222 Europe/Istanbul
38.54726 28.64976 Europe/Istanbul
38.19383 41.48853 Europe/Istanbul
37.45517 35.81573 Europe/Istanbul
38.72139 39.86806 Europe/Istanbul
37.06498 30.19565 Europe/Istanbul
37.87135 32.48464 Europe/Istanbul
37.19333 40.585 Europe/Istanbul
37.58104 29.26639 Europe/Istanbul
39.14583 34.16389 Europe/Istanbul
39.10638 27.66925 Europe/Istanbul
39.84528 33.50639 Europe/Istanbul
36.49939 36.35755 Europe/Istanbul
36.71611 37.115 Europe/Istanbul
39.67306 33.61361 Europe/Istanbul
36.59778 30.56056 Europe/Istanbul
38.42621 27.41731 Europe/Istanbul
38.73222 35.48528 Europe/Istanbul
37.71596 33.55064 Europe/Istanbul
37.18111 33.215 Europe/Istanbul
39.71944 43.05139 Europe/Istanbul
38.95583 40.03861 Europe/Istanbul
39.34919 42.11239 Europe/Istanbul
39.3575 33.72389 Europe/Istanbul
37.78552 38.6237 Europe/Istanbul
37.5847 36.9264 Europe/Istanbul
37.37389 36.09611 Europe/Istanbul
38.23972 32.21139 Europe/Istanbul
38.41273 27.13838 Europe/Istanbul
37.76444 30.55222 Europe/Istanbul
36.58718 36.17347 Europe/Istanbul
37.85222 27.72361 Europe/Istanbul
37.26506 35.65717 Europe/Istanbul
38.27917 31.91389 Europe/Istanbul
39.92371 44.045 Europe/Istanbul
37.34114 41.8895 Europe/Istanbul
38.22572 42.42776 Europe/Istanbul
39.3606 41.70222 Europe/Istanbul
37.58686 38.95505 Europe/Istanbul
36.98776 32.45674 Europe/Istanbul
38.64631 35.44937 Europe/Istanbul
38.57739 42.02811 Europe/Istanbul
37.78361 37.63667 Europe/Istanbul
38.02096 36.4973 Europe/Istanbul
38.75071 40.56035 Europe/Istanbul
39.18342 36.07189 Europe/Istanbul
39.04167 29.41 Europe/Istanbul
36.26942 32.31792 Europe/Istanbul
37.05944 37.3825 Europe/Istanbul
38.6703 26.75656 Europe/Istanbul
36.62167 29.11639 Europe/Istanbul
39.78561 26.34083 Europe/Istanbul
39.77667 30.52056 Europe/Istanbul
39.90861 41.27694 Europe/Istanbul
39.75222 39.49278 Europe/Istanbul
36.64043 32.89179 Europe/Istanbul
38.26533 39.76212 Europe/Istanbul
37.51333 34.04672 Europe/Istanbul
36.60498 34.30836 Europe/Istanbul
39.02869 43.35864 Europe/Istanbul
39.01972 31.15 Europe/Istanbul
39.343 29.25847 Europe/Istanbul
36.73583 29.91775 Europe/Istanbul
39.92083 33.23083 Europe/Istanbul
39.80333 42.67361 Europe/Istanbul
38.20591 37.1983 Europe/Istanbul
38.67431 39.22321 Europe/Istanbul
37.87462 30.85042 Europe/Istanbul
39.59611 27.02444 Europe/Istanbul
39.58596 28.62568 Europe/Istanbul
36.86158 36.22885 Europe/Istanbul
39.54694 44.08417 Europe/Istanbul
37.91583 40.21889 Europe/Istanbul
39.54056 43.67135 Europe/Istanbul
38.065 30.16557 Europe/Istanbul
38.39056 35.49222 Europe/Istanbul
37.77417 29.0875 Europe/Istanbul
39.04607 28.65889 Europe/Istanbul
37.54545 41.71966 Europe/Istanbul
38.54583 37.50583 Europe/Istanbul
37.5732 32.77446 Europe/Istanbul
38.24963 27.13429 Europe/Istanbul
37.57444 43.74083 Europe/Istanbul
37.32722 42.19028 Europe/Istanbul
37.61167 28.06139 Europe/Istanbul
36.84722 40.05 Europe/Istanbul
37.02472 35.8175 Europe/Istanbul
38.32278 26.30639 Europe/Istanbul
38.13613 39.44929 Europe/Istanbul
38.59167 31.02861 Europe/Istanbul
37.74959 37.29622 Europe/Istanbul
39.50041 26.97269 Europe/Istanbul
37.72028 30.29083 Europe/Istanbul
39.09292 42.27031 Europe/Istanbul
37.45917 30.595 Europe/Istanbul
36.1082 32.96113 Europe/Istanbul
39.90778 30.03667 Europe/Istanbul
37.3625 38.52667 Europe/Istanbul
37.89056 34.55889 Europe/Istanbul
38.71111 31.04861 Europe/Istanbul
37.03833 27.42917 Europe/Istanbul
37.84861 40.66583 Europe/Istanbul
37.02944 37.99028 Europe/Istanbul
38.88472 40.49389 Europe/Istanbul
39.3925 28.13111 Europe/Istanbul
37.67735 31.72458 Europe/Istanbul
37.69278 37.86111 Europe/Istanbul
39.12074 27.18052 Europe/Istanbul
36.48911 36.22331 Europe/Istanbul
36.86278 31.05556 Europe/Istanbul
38.21741 27.64744 Europe/Istanbul
37.88738 41.13221 Europe/Istanbul
38.56791 38.82382 Europe/Istanbul
38.73707 29.75194 Europe/Istanbul
39.64917 27.88611 Europe/Istanbul
37.19724 36.57658 Europe/Istanbul
39.31905 26.6954 Europe/Istanbul
37.84501 27.83963 Europe/Istanbul
39.92083 40.695 Europe/Istanbul
36.90812 30.69556 Europe/Istanbul
36.20655 36.15722 Europe/Istanbul
39.91987 32.85427 Europe/Istanbul
36.07508 32.83691 Europe/Istanbul
38.79975 26.97203 Europe/Istanbul
38.35083 28.51718 Europe/Istanbul
36.54375 31.99982 Europe/Istanbul
38.3575 31.41639 Europe/Istanbul
38.37255 34.02537 Europe/Istanbul
38.91852 27.84006 Europe/Istanbul
39.66028 35.88361 Europe/Istanbul
36.71111 38.9475 Europe/Istanbul
38.75178 42.48135 Europe/Istanbul
38.75667 30.54333 Europe/Istanbul
38.24769 36.91399 Europe/Istanbul
37.76441 38.27629 Europe/Istanbul
38.80389 42.73422 Europe/Istanbul
37.00167 35.32889 Europe/Istanbul
36.64951 36.22956 Europe/Istanbul
39.96833 32.73083 Europe/Istanbul
36.76591 28.8028 Europe/Istanbul
41.45139 31.79305 Europe/Istanbul
40.30306 35.88639 Europe/Istanbul
40.99441 28.90417 Europe/Istanbul
40.95411 39.86405 Europe/Istanbul
40.26444 29.65306 Europe/Istanbul
40.77667 29.72972 Europe/Istanbul
40.65502 29.27693 Europe/Istanbul
40.98894 28.67582 Europe/Istanbul
41.14361 35.45472 Europe/Istanbul
41.04583 39.27639 Europe/Istanbul
41.26598 26.6885 Europe/Istanbul
41.13139 37.2825 Europe/Istanbul
41.01643 29.12476 Europe/Istanbul
40.3875 36.08111 Europe/Istanbul
41.005 39.72694 Europe/Istanbul
41.01545 34.04013 Europe/Istanbul
40.31389 36.55444 Europe/Istanbul
41.00694 38.81389 Europe/Istanbul
41.20917 36.97389 Europe/Istanbul
41.02931 28.54978 Europe/Istanbul
41.21167 36.46 Europe/Istanbul
40.97801 27.50852 Europe/Istanbul
40.75972 36.3225 Europe/Istanbul
41.5098 34.21414 Europe/Istanbul
40.16444 38.08667 Europe/Istanbul
40.91258 40.11676 Europe/Istanbul
40.1675 34.37389 Europe/Istanbul
40.83128 35.64788 Europe/Istanbul
41.06046 28.98717 Europe/Istanbul
42.02683 35.16252 Europe/Istanbul
41.08022 28.22605 Europe/Istanbul
40.28833 38.42361 Europe/Istanbul
40.33806 42.57306 Europe/Istanbul
40.69141 30.26738 Europe/Istanbul
41.28667 36.33 Europe/Istanbul
41.25083 32.69417 Europe/Istanbul
41.02083 40.52194 Europe/Istanbul
40.35722 30.01417 Europe/Istanbul
40.97818 34.8047 Europe/Istanbul
40.48917 29.30889 Europe/Istanbul
40.98472 37.87889 Europe/Istanbul
40.54983 42.00126 Europe/Istanbul
40.94458 40.26746 Europe/Istanbul
40.59167 36.95167 Europe/Istanbul
40.18593 31.35179 Europe/Istanbul
40.37528 28.88222 Europe/Istanbul
41.00315 28.53768 Europe/Istanbul
40.87333 35.46306 Europe/Istanbul
40.93567 29.15507 Europe/Istanbul
40.89 26.90111 Europe/Istanbul
41.40385 27.35918 Europe/Istanbul
40.87444 37.26389 Europe/Istanbul
40.82472 37.34667 Europe/Istanbul
41.05336 30.85278 Europe/Istanbul
40.46972 32.65056 Europe/Istanbul
41.73508 27.22521 Europe/Istanbul
40.19828 29.21237 Europe/Istanbul
40.85583 26.63028 Europe/Istanbul
40.12978 39.43607 Europe/Istanbul
40.23167 32.68389 Europe/Istanbul
41.09258 28.33172 Europe/Istanbul
41.37805 33.77528 Europe/Istanbul
40.60199 43.09495 Europe/Istanbul
41.07096 30.78543 Europe/Istanbul
40.69144 29.61568 Europe/Istanbul
40.21525 28.40378 Europe/Istanbul
41.20488 32.62768 Europe/Istanbul
40.15669 43.13424 Europe/Istanbul
40.42861 29.72111 Europe/Istanbul
40.76694 29.91694 Europe/Istanbul
41.01384 28.94966 Europe/Istanbul
40.73528 34.47389 Europe/Istanbul
40.07806 29.51333 Europe/Istanbul
40.04583 42.17278 Europe/Istanbul
41.40597 41.43787 Europe/Istanbul
40.79944 30.74806 Europe/Istanbul
41.21311 27.10688 Europe/Istanbul
40.97056 35.66222 Europe/Istanbul
40.21876 29.19487 Europe/Istanbul
40.99028 28.61528 Europe/Istanbul
40.78833 37.60167 Europe/Istanbul
40.46028 39.48139 Europe/Istanbul
41.03083 39.00306 Europe/Istanbul
40.1049 27.65399 Europe/Istanbul
40.71667 29.82861 Europe/Istanbul
40.91698 38.38741 Europe/Istanbul
40.5075 30.2925 Europe/Istanbul
40.80083 32.19694 Europe/Istanbul
40.43094 29.15969 Europe/Istanbul
40.41028 26.67083 Europe/Istanbul
40.80276 29.43068 Europe/Istanbul
40.94082 30.48583 Europe/Istanbul
41.02778 37.50139 Europe/Istanbul
41.03333 28.67528 Europe/Istanbul
41.0435 28.87619 Europe/Istanbul
41.28262 31.41806 Europe/Istanbul
40.3996 27.79348 Europe/Istanbul
40.66889 36.5675 Europe/Istanbul
41.67719 26.55597 Europe/Istanbul
40.83889 31.16389 Europe/Istanbul
41.21917 31.95583 Europe/Istanbul
40.23861 33.03222 Europe/Istanbul
40.54889 34.95333 Europe/Istanbul
41.15917 27.8 Europe/Istanbul
41.285 28.00028 Europe/Istanbul
40.81164 32.89358 Europe/Istanbul
41.09228 40.72924 Europe/Istanbul
41.42639 32.07556 Europe/Istanbul
41.14324 28.46154 Europe/Istanbul
41.19889 36.72194 Europe/Istanbul
40.59995 33.6153 Europe/Istanbul
40.14556 26.40639 Europe/Istanbul
40.03328 27.05236 Europe/Istanbul
40.19167 29.06111 Europe/Istanbul
40.93717 38.22907 Europe/Istanbul
41.46889 34.76667 Europe/Istanbul
40.73583 31.60611 Europe/Istanbul
40.14192 29.97932 Europe/Istanbul
40.22806 27.24222 Europe/Istanbul
40.1675 31.92111 Europe/Istanbul
41.05199 39.22844 Europe/Istanbul
40.25889 40.22778 Europe/Istanbul
41.63583 32.3375 Europe/Istanbul
40.35222 27.97667 Europe/Istanbul
41.03903 28.85671 Europe/Istanbul
41.56778 35.90694 Europe/Istanbul
41.4325 27.09306 Europe/Istanbul
41.18222 41.81944 Europe/Istanbul
40.94958 39.92756 Europe/Istanbul
41.34917 41.30694 Europe/Istanbul
41.19111 40.9875 Europe/Istanbul
41.10871 42.70222 Europe/Istanbul
40.93854 40.05842 Europe/Istanbul
40.65333 35.83306 Europe/Istanbul
41.1694 31.37917 Europe/Istanbul
40.16833 34.8425 Europe/Istanbul
40.685 30.62222 Europe/Istanbul
41.08663 31.11623 Europe/Istanbul
41.0212 39.57146 Europe/Istanbul
40.78056 30.40333 Europe/Istanbul
40.94794 38.71148 Europe/Istanbul
41.01295 28.88593 Europe/Istanbul
41.00711 28.88795 Europe/Istanbul
36.71549 28.70436 Europe/Istanbul
40.9833 29.1167 Europe/Istanbul
41.09307 28.80203 Europe/Istanbul
40.982 28.6399 Europe/Istanbul
41.02072 28.58502 Europe/Istanbul
39.9179 32.86268 Europe/Istanbul
41.00231 28.8598 Europe/Istanbul
41.10652 28.86847 Europe/Istanbul
40.96072 29.27067 Europe/Istanbul
41.00244 29.23187 Europe/Istanbul
38.37396 27.1352 Europe/Istanbul
10.63333 -61.38333 America/Port_of_Spain
10.63471 -61.36671 America/Port_of_Spain
11.18333 -60.73333 America/Port_of_Spain
10.58333 -61.11667 America/Port_of_Spain
10.28333 -61.46667 America/Port_of_Spain
10.66617 -61.51657 America/Port_of_Spain
10.18333 -61.68333 America/Port_of_Spain
10.283 -61.44605 America/Port_of_Spain
10.30618 -61.44671 America/Port_of_Spain
10.51667 -61.41667 America/Port_of_Spain
10.63333 -61.28333 America/Port_of_Spain
-8.52425 179.19417 Pacific/Funafuti
23.70944 120.54333 Asia/Taipei
23.12493 120.46138 Asia/Taipei
25.04776 121.53185 Asia/Taipei
22.99083 120.21333 Asia/Taipei
24.1469 120.6839 Asia/Taipei
24.88373 121.29043 Asia/Taipei
25.01427 121.46719 Asia/Taipei
23.96639 120.96952 Asia/Taipei
23.91566 120.66387 Asia/Taipei
23.5654 119.58627 Asia/Taipei
23.74639 120.7525 Asia/Taipei
22.61626 120.31333 Asia/Taipei
24.757 121.753 Asia/Taipei
23.97694 121.60444 Asia/Taipei
24.80361 120.96861 Asia/Taipei
22.00417 120.74389 Asia/Taipei
25.12825 121.7419 Asia/Taipei
24.99368 121.29696 Asia/Taipei
22.7583 121.1444 Asia/Taipei
23.95908 120.68516 Asia/Taipei
-6.16394 39.19793 Africa/Dar_es_Salaam
-5.05589 39.72938 Africa/Dar_es_Salaam
-9.11667 32.93333 Africa/Dar_es_Salaam
-7.00667 39.29849 Africa/Dar_es_Salaam
-3.28333 31.52583 Africa/Dar_es_Salaam
-5.10361 30.39111 Africa/Dar_es_Salaam
-3.49194 31.96389 Africa/Dar_es_Salaam
-7.1 31.23333 Africa/Dar_es_Salaam
-2.68333 33 Africa/Dar_es_Salaam
-3.36667 36.85 Africa/Dar_es_Salaam
-5.06667 32.05 Africa/Dar_es_Salaam
-9.3 32.76667 Africa/Dar_es_Salaam
-5.06667 32.73333 Africa/Dar_es_Salaam
-9.25 33.65 Africa/Dar_es_Salaam
-3.88333 33.2 Africa/Dar_es_Salaam
-1.35 34.36667 Africa/Dar_es_Salaam
-5.06667 39.1 Africa/Dar_es_Salaam
-5.01667 32.8 Africa/Dar_es_Salaam
-7.96667 31.61667 Africa/Dar_es_Salaam
-3.51667 33.51667 Africa/Dar_es_Salaam
-3.36667 33.95 Africa/Dar_es_Salaam
-6.43333 39.55 Africa/Dar_es_Salaam
-1.23333 34.43333 Africa/Dar_es_Salaam
-4.81667 34.75 Africa/Dar_es_Salaam
-5.63333 32.76667 Africa/Dar_es_Salaam
-3.66667 33.43333 Africa/Dar_es_Salaam
-4.36667 34.2 Africa/Dar_es_Salaam
-4.75 34.53333 Africa/Dar_es_Salaam
-4.06667 37.73333 Africa/Dar_es_Salaam
-2.7175 30.6325 Africa/Dar_es_Salaam
-8.7 34.38333 Africa/Dar_es_Salaam
-5 34.73333 Africa/Dar_es_Salaam
-3.55 33.4 Africa/Dar_es_Salaam
-4.21667 33.18333 Africa/Dar_es_Salaam
-1.9 34.01667 Africa/Dar_es_Salaam
-3.18333 33.78333 Africa/Dar_es_Salaam
-2.63333 33.98333 Africa/Dar_es_Salaam
-1.14389 31.39472 Africa/Dar_es_Salaam
-1.79833 31.55111 Africa/Dar_es_Salaam
-9.33333 34.76667 Africa/Dar_es_Salaam
-5.10917 31.04194 Africa/Dar_es_Salaam
-2.96667 33.33333 Africa/Dar_es_Salaam
-6.75 38.11667 Africa/Dar_es_Salaam
-2.51222 30.65583 Africa/Dar_es_Salaam
-4.46667 35.45 Africa/Dar_es_Salaam
-7.51667 31.05 Africa/Dar_es_Salaam
-2.51667 32.9 Africa/Dar_es_Salaam
-3.55 33.6 Africa/Dar_es_Salaam
-6.25 38.66667 Africa/Dar_es_Salaam
-1.5 33.8 Africa/Dar_es_Salaam
-1.98333 32.91667 Africa/Dar_es_Salaam
-4.96667 34.88333 Africa/Dar_es_Salaam
-5.16667 38.78333 Africa/Dar_es_Salaam
-1.85 34.7 Africa/Dar_es_Salaam
-9.01667 34.8 Africa/Dar_es_Salaam
-3.35 35.85 Africa/Dar_es_Salaam
-4.55 34.85 Africa/Dar_es_Salaam
-6.53333 37.2 Africa/Dar_es_Salaam
-6.35 36.48333 Africa/Dar_es_Salaam
-6.36546 31.04091 Africa/Dar_es_Salaam
-3.35 37.33333 Africa/Dar_es_Salaam
-6.81667 37.66667 Africa/Dar_es_Salaam
-9 32.98333 Africa/Dar_es_Salaam
-8.8 35.81667 Africa/Dar_es_Salaam
-9.78333 34.51667 Africa/Dar_es_Salaam
-6.7 38.73333 Africa/Dar_es_Salaam
-4.93333 38.93333 Africa/Dar_es_Salaam
-7.11799 39.20782 Africa/Dar_es_Salaam
-4.83 29.65806 Africa/Dar_es_Salaam
-2.85 33.08333 Africa/Dar_es_Salaam
-7.40409 36.98309 Africa/Dar_es_Salaam
-3.28333 32.85 Africa/Dar_es_Salaam
-5.95 34.13333 Africa/Dar_es_Salaam
-3.56667 36.95 Africa/Dar_es_Salaam
-8.9 33.45 Africa/Dar_es_Salaam
-4.8 38.21667 Africa/Dar_es_Salaam
-4.8 38.25 Africa/Dar_es_Salaam
-8.3 31.51667 Africa/Dar_es_Salaam
-2.68333 33.98333 Africa/Dar_es_Salaam
-3.63333 32.18333 Africa/Dar_es_Salaam
-5.05 38.61667 Africa/Dar_es_Salaam
-8.93333 36.13333 Africa/Dar_es_Salaam
-3.13333 33.53333 Africa/Dar_es_Salaam
-4.73333 38.1 Africa/Dar_es_Salaam
-8.85 34.83333 Africa/Dar_es_Salaam
-9.93333 35.33333 Africa/Dar_es_Salaam
-2.58333 33.35 Africa/Dar_es_Salaam
-4.01667 35.76667 Africa/Dar_es_Salaam
-6.8 39.25 Africa/Dar_es_Salaam
-6.36667 37.36667 Africa/Dar_es_Salaam
-7.25 35.06667 Africa/Dar_es_Salaam
-5.13333 32.53333 Africa/Dar_es_Salaam
-4.78333 38.28333 Africa/Dar_es_Salaam
-6.45 38.33333 Africa/Dar_es_Salaam
-9.76667 37.93333 Africa/Dar_es_Salaam
-3.78333 37.61667 Africa/Dar_es_Salaam
-3.45 33.95 Africa/Dar_es_Salaam
-8.56667 32.05 Africa/Dar_es_Salaam
-9.58333 33.85 Africa/Dar_es_Salaam
-4.9 35.78333 Africa/Dar_es_Salaam
-9.16667 33.53333 Africa/Dar_es_Salaam
-3.61667 33.86667 Africa/Dar_es_Salaam
-3.08333 34.15 Africa/Dar_es_Salaam
-7.41667 30.6 Africa/Dar_es_Salaam
-4.26667 34.36667 Africa/Dar_es_Salaam
-3.28333 36.98333 Africa/Dar_es_Salaam
-6.83333 36.98333 Africa/Dar_es_Salaam
-4.87694 29.62667 Africa/Dar_es_Salaam
-7.60361 37.00438 Africa/Dar_es_Salaam
-7.69916 36.95722 Africa/Dar_es_Salaam
-3.58639 30.72028 Africa/Dar_es_Salaam
-7.72178 38.93749 Africa/Dar_es_Salaam
-2.15 33.45 Africa/Dar_es_Salaam
-6.71667 36.36667 Africa/Dar_es_Salaam
-6.76667 38.91667 Africa/Dar_es_Salaam
-9.23333 33.61667 Africa/Dar_es_Salaam
-3.02028 31.89583 Africa/Dar_es_Salaam
-1.4 31.73333 Africa/Dar_es_Salaam
-4.57667 30.1025 Africa/Dar_es_Salaam
-2.85 32.43333 Africa/Dar_es_Salaam
-3.33333 35.66667 Africa/Dar_es_Salaam
-1.61861 31.62 Africa/Dar_es_Salaam
-3.28278 30.96417 Africa/Dar_es_Salaam
-3.83333 32.6 Africa/Dar_es_Salaam
-2.63861 30.46778 Africa/Dar_es_Salaam
-7.2 35.73333 Africa/Dar_es_Salaam
-5.7 34.48333 Africa/Dar_es_Salaam
-3.9 32.93333 Africa/Dar_es_Salaam
-7.76667 35.7 Africa/Dar_es_Salaam
-9.48333 33.9 Africa/Dar_es_Salaam
-7.6766 36.03658 Africa/Dar_es_Salaam
-4.66667 34.86667 Africa/Dar_es_Salaam
-8.9 34.58333 Africa/Dar_es_Salaam
-5.13333 34.76667 Africa/Dar_es_Salaam
-8.85 33.85 Africa/Dar_es_Salaam
-4.28333 33.88333 Africa/Dar_es_Salaam
-4.56667 34.63333 Africa/Dar_es_Salaam
-8.13333 36.68333 Africa/Dar_es_Salaam
-4.5 37.9 Africa/Dar_es_Salaam
-6.15 36.86667 Africa/Dar_es_Salaam
-4.26667 35.85 Africa/Dar_es_Salaam
-4.06667 35.38333 Africa/Dar_es_Salaam
-6.17221 35.73947 Africa/Dar_es_Salaam
-6.82349 39.26951 Africa/Dar_es_Salaam
-4.21667 35.55 Africa/Dar_es_Salaam
-8.85 34.01667 Africa/Dar_es_Salaam
-2.63778 31.76694 Africa/Dar_es_Salaam
-5.41667 38.01667 Africa/Dar_es_Salaam
-6.63784 38.35396 Africa/Dar_es_Salaam
-7.58333 31.26667 Africa/Dar_es_Salaam
-5.24586 39.76659 Africa/Dar_es_Salaam
-1.76667 33.96667 Africa/Dar_es_Salaam
-3.02361 31.87472 Africa/Dar_es_Salaam
-7.63369 39.05818 Africa/Dar_es_Salaam
-2.05 33.86667 Africa/Dar_es_Salaam
-1.33167 31.81222 Africa/Dar_es_Salaam
-2.87056 30.52806 Africa/Dar_es_Salaam
-2.63194 31.30889 Africa/Dar_es_Salaam
-4.36667 35.08333 Africa/Dar_es_Salaam
-4.23333 35.41667 Africa/Dar_es_Salaam
-2.8 33.98333 Africa/Dar_es_Salaam
-6.43333 38.9 Africa/Dar_es_Salaam
-4.21667 35.75 Africa/Dar_es_Salaam
-3.36667 36.68333 Africa/Dar_es_Salaam
-11.3 35.03333 Africa/Dar_es_Salaam
-10.75 39.63333 Africa/Dar_es_Salaam
-10.68333 35.65 Africa/Dar_es_Salaam
-10.33333 39.28333 Africa/Dar_es_Salaam
-10.95 39.28333 Africa/Dar_es_Salaam
-10.68333 39.83333 Africa/Dar_es_Salaam
-10.9 38.5 Africa/Dar_es_Salaam
-10.38333 39.15 Africa/Dar_es_Salaam
-10.38333 38.76667 Africa/Dar_es_Salaam
-10.26667 40.18333 Africa/Dar_es_Salaam
-10.93333 35.01667 Africa/Dar_es_Salaam
-10.75 34.9 Africa/Dar_es_Salaam
-10.71667 38.8 Africa/Dar_es_Salaam
-10.58333 35.4 Africa/Dar_es_Salaam
-10.56667 38.8 Africa/Dar_es_Salaam
-10.9 39.33333 Africa/Dar_es_Salaam
-10 39.71667 Africa/Dar_es_Salaam
-10.71667 39.73333 Africa/Dar_es_Salaam
-10.8 35.05 Africa/Dar_es_Salaam
-3.56182 36.97895 Africa/Dar_es_Salaam
50.58518 34.4849 Europe/Kiev
48.63013 37.55259 Europe/Zaporozhye
47.43694 35.27417 Europe/Zaporozhye
49.07866 30.96754 Europe/Kiev
48.0114 38.26444 Europe/Zaporozhye
49.66886 32.03773 Europe/Kiev
49.80597 24.89436 Europe/Uzhgorod
48.71278 32.66472 Europe/Zaporozhye
49.69664 36.36109 Europe/Zaporozhye
50.26487 28.67669 Europe/Kiev
49.03705 28.11201 Europe/Kiev
48.34731 33.50034 Europe/Zaporozhye
49.24545 30.1102 Europe/Kiev
50.5206 26.24251 Europe/Kiev
47.82289 35.19031 Europe/Zaporozhye
46.62211 31.10131 Europe/Simferopol
45.20091 33.36655 Europe/Simferopol
48.23331 38.21137 Europe/Zaporozhye
48.1298 37.8594 Europe/Zaporozhye
44.50013 34.16148 Europe/Simferopol
50.27975 31.76246 Europe/Kiev
50.58476 30.4898 Europe/Kiev
48.15 22.76667 Europe/Uzhgorod
47.56494 31.33078 Europe/Zaporozhye
47.60103 37.49674 Europe/Zaporozhye
50.29078 36.94108 Europe/Zaporozhye
50.85253 24.32364 Europe/Uzhgorod
50.38913 30.3705 Europe/Kiev
49.23278 28.48097 Europe/Kiev
49.01502 31.06211 Europe/Kiev
50.18692 30.31346 Europe/Kiev
48.61667 22.3 Europe/Uzhgorod
48.74838 30.22184 Europe/Kiev
48.67448 28.84641 Europe/Kiev
46.61842 32.7189 Europe/Simferopol
49.27837 23.50618 Europe/Uzhgorod
48.03876 38.59685 Europe/Zaporozhye
47.25522 35.71238 Europe/Zaporozhye
49.55589 25.60556 Europe/Uzhgorod
48.94832 38.49166 Europe/Zaporozhye
49.04894 33.24106 Europe/Zaporozhye
48.08964 39.65243 Europe/Zaporozhye
49.41029 38.15035 Europe/Zaporozhye
48.54853 22.99578 Europe/Uzhgorod
50.9216 34.80029 Europe/Kiev
49.26223 23.85609 Europe/Uzhgorod
49.29416 23.56357 Europe/Uzhgorod
49.75764 27.20342 Europe/Kiev
49.27832 38.91069 Europe/Zaporozhye
48.56818 38.64352 Europe/Zaporozhye
50.47438 24.28288 Europe/Uzhgorod
48.02612 38.77225 Europe/Zaporozhye
49.22242 31.88714 Europe/Kiev
48.86667 37.61667 Europe/Zaporozhye
50.30155 26.86506 Europe/Kiev
49.73177 29.66569 Europe/Kiev
46.1161 32.91124 Europe/Simferopol
48.32044 35.51792 Europe/Zaporozhye
44.95719 34.11079 Europe/Simferopol
49.00687 31.39471 Europe/Kiev
51.86296 33.4698 Europe/Kiev
50.18545 27.06365 Europe/Kiev
48.06252 38.51665 Europe/Zaporozhye
44.58883 33.5224 Europe/Simferopol
51.33795 26.60191 Europe/Kiev
49.5183 23.19752 Europe/Uzhgorod
45.13424 33.59996 Europe/Simferopol
49.01229 38.37967 Europe/Zaporozhye
46.84334 30.07919 Europe/Simferopol
48.08332 39.37764 Europe/Zaporozhye
50.75104 33.47471 Europe/Kiev
50.62308 26.22743 Europe/Kiev
45.45727 28.29317 Europe/Simferopol
50.49613 29.22911 Europe/Kiev
48.41333 33.71056 Europe/Zaporozhye
51.33745 33.87066 Europe/Kiev
50.59324 32.38761 Europe/Kiev
48.63328 38.37804 Europe/Zaporozhye
49.59373 34.54073 Europe/Zaporozhye
50.12424 27.51087 Europe/Kiev
47.48444 36.25361 Europe/Zaporozhye
48.57528 35.1068 Europe/Zaporozhye
50.2445 32.51073 Europe/Kiev
48.62988 38.54806 Europe/Zaporozhye
48.04433 30.85073 Europe/Kiev
50.08768 31.4795 Europe/Kiev
48.43779 38.84384 Europe/Zaporozhye
48.53341 35.8705 Europe/Zaporozhye
51.3246 28.80351 Europe/Kiev
47.56731 35.78575 Europe/Zaporozhye
48.66961 33.11593 Europe/Zaporozhye
46.47747 30.73262 Europe/Simferopol
46.61283 31.54982 Europe/Simferopol
50.10997 30.62272 Europe/Kiev
47.69308 32.5216 Europe/Zaporozhye
50.72576 24.16265 Europe/Uzhgorod
48.32635 31.52852 Europe/Kiev
48.63798 35.24548 Europe/Zaporozhye
50.59412 27.6165 Europe/Kiev
46.75451 33.34864 Europe/Zaporozhye
50.93152 31.58282 Europe/Kiev
51.04801 31.88688 Europe/Kiev
45.44789 34.73839 Europe/Simferopol
47.57119 34.39637 Europe/Zaporozhye
50.34004 26.64171 Europe/Kiev
48.6348 24.56936 Europe/Uzhgorod
46.96591 31.9974 Europe/Zaporozhye
48.43919 22.71779 Europe/Uzhgorod
48.34578 39.65824 Europe/Zaporozhye
48.44598 27.7982 Europe/Kiev
44.43333 34.08333 Europe/Simferopol
49.96854 33.60886 Europe/Kiev
49.82302 36.05069 Europe/Zaporozhye
46.84891 35.36533 Europe/Zaporozhye
47.06667 37.5 Europe/Zaporozhye
47.63543 34.62769 Europe/Zaporozhye
50.77233 29.23833 Europe/Kiev
48.04782 37.92576 Europe/Zaporozhye
49.94691 35.92907 Europe/Zaporozhye
49.83826 24.02324 Europe/Uzhgorod
48.40507 39.22675 Europe/Zaporozhye
50.75932 25.34244 Europe/Uzhgorod
48.56705 39.31706 Europe/Zaporozhye
50.01625 32.99694 Europe/Kiev
48.88937 36.31755 Europe/Zaporozhye
48.90485 38.44207 Europe/Zaporozhye
48.68496 29.23678 Europe/Kiev
50.45466 30.5238 Europe/Kiev
51.68333 25.86667 Europe/Kiev
47.98522 37.2821 Europe/Zaporozhye
47.90966 33.38044 Europe/Zaporozhye
51.54775 33.38475 Europe/Kiev
49.0495 38.21792 Europe/Zaporozhye
50.09693 25.72459 Europe/Uzhgorod
49.09725 33.41972 Europe/Zaporozhye
48.13954 38.93715 Europe/Zaporozhye
48.98836 37.80225 Europe/Zaporozhye
45.95716 33.7965 Europe/Simferopol
49.3814 35.44021 Europe/Zaporozhye
48.29229 39.73729 Europe/Zaporozhye
48.28198 37.17585 Europe/Zaporozhye
49.65186 26.97252 Europe/Kiev
48.72305 37.55629 Europe/Zaporozhye
49.62125 37.70357 Europe/Zaporozhye
51.21526 24.70867 Europe/Uzhgorod
47.75305 29.53205 Europe/Kiev
49.41894 31.25865 Europe/Kiev
50.31723 29.0563 Europe/Kiev
50.95937 28.63855 Europe/Kiev
48.5277 37.7069 Europe/Zaporozhye
51.24032 33.20263 Europe/Kiev
49.59701 36.52618 Europe/Zaporozhye
48.53115 25.03649 Europe/Uzhgorod
50.83425 25.45821 Europe/Uzhgorod
48.63751 38.6428 Europe/Zaporozhye
48.5132 32.2597 Europe/Zaporozhye
45.45518 29.26367 Europe/Simferopol
48.1793 23.29909 Europe/Uzhgorod
49.42161 26.99653 Europe/Kiev
49.55979 27.95754 Europe/Kiev
46.65581 32.6178 Europe/Simferopol
48.04243 38.14728 Europe/Zaporozhye
49.98081 36.25272 Europe/Zaporozhye
45.3607 36.4706 Europe/Simferopol
49.71431 28.83385 Europe/Kiev
49.45668 35.12947 Europe/Zaporozhye
49.75182 31.46004 Europe/Kiev
48.6845 26.58559 Europe/Uzhgorod
49.01187 24.37308 Europe/Uzhgorod
49.45389 28.52608 Europe/Kiev
46.81371 33.48698 Europe/Zaporozhye
49.20875 37.24848 Europe/Zaporozhye
50.11947 26.82125 Europe/Kiev
45.34929 28.84079 Europe/Simferopol
48.9215 24.70972 Europe/Uzhgorod
50.52175 30.25055 Europe/Kiev
47.92498 38.20236 Europe/Zaporozhye
46.30495 30.65478 Europe/Simferopol
49.16374 26.58394 Europe/Kiev
48.34698 38.05419 Europe/Zaporozhye
51.67822 33.9162 Europe/Kiev
47.66389 36.25633 Europe/Zaporozhye
49.78465 23.64806 Europe/Uzhgorod
49.28489 31.44513 Europe/Kiev
46.5272 32.52417 Europe/Simferopol
46.17592 34.8034 Europe/Simferopol
48.33958 29.86791 Europe/Kiev
48.81143 29.38977 Europe/Kiev
50.37112 33.99099 Europe/Kiev
45.03677 35.37789 Europe/Simferopol
50.0767 29.9177 Europe/Kiev
47.49865 34.6574 Europe/Zaporozhye
45.71168 34.39274 Europe/Simferopol
48.39869 37.84787 Europe/Zaporozhye
48.88909 26.85636 Europe/Kiev
50.41694 25.73432 Europe/Uzhgorod
49.34991 23.50561 Europe/Uzhgorod
48.023 37.80224 Europe/Zaporozhye
48.9733 24.00944 Europe/Uzhgorod
48.11041 32.76449 Europe/Zaporozhye
47.74773 37.67384 Europe/Zaporozhye
48.46147 37.08524 Europe/Zaporozhye
48.45 34.98333 Europe/Zaporozhye
48.51134 34.6021 Europe/Zaporozhye
47.38169 34.97652 Europe/Zaporozhye
50.10659 36.12112 Europe/Zaporozhye
48.34072 38.4049 Europe/Zaporozhye
49.83588 36.68803 Europe/Zaporozhye
49.01709 25.79804 Europe/Uzhgorod
48.07613 39.79618 Europe/Zaporozhye
50.39105 24.23514 Europe/Uzhgorod
48.29149 25.94034 Europe/Uzhgorod
51.50551 31.28488 Europe/Kiev
49.42854 32.06207 Europe/Kiev
50.54345 30.21201 Europe/Kiev
48.511 38.67222 Europe/Zaporozhye
50.51809 30.80671 Europe/Kiev
50.08791 25.15027 Europe/Uzhgorod
50.3191 30.29728 Europe/Kiev
50.35269 30.95501 Europe/Kiev
49.28672 23.43238 Europe/Uzhgorod
45.68262 28.61487 Europe/Simferopol
49.54939 30.8744 Europe/Kiev
50.16466 35.52765 Europe/Zaporozhye
46.1952 30.34938 Europe/Simferopol
49.80939 30.11209 Europe/Kiev
49.45 24.93333 Europe/Uzhgorod
48.20555 22.64418 Europe/Uzhgorod
49.89928 28.60235 Europe/Kiev
46.7663 36.79882 Europe/Zaporozhye
51.15016 34.31287 Europe/Kiev
45.05462 34.60185 Europe/Simferopol
49.07717 27.68256 Europe/Kiev
47.93548 29.61982 Europe/Kiev
49.4627 36.85951 Europe/Zaporozhye
44.51428 33.60013 Europe/Simferopol
51.18076 32.75694 Europe/Kiev
44.75525 33.85782 Europe/Simferopol
48.13989 37.74255 Europe/Zaporozhye
45.99194 29.41824 Europe/Simferopol
48.59559 37.99987 Europe/Zaporozhye
46.10919 33.69206 Europe/Simferopol
47.66003 33.71369 Europe/Zaporozhye
48.11503 39.09128 Europe/Zaporozhye
47.79348 38.47768 Europe/Zaporozhye
44.67631 34.40778 Europe/Simferopol
48.46893 38.81669 Europe/Zaporozhye
50.31036 34.89879 Europe/Zaporozhye
49.00973 33.64546 Europe/Zaporozhye
51.51964 30.73434 Europe/Kiev
47.81777 31.18263 Europe/Kiev
3.46506 31.24689 Africa/Kampala
0.72833 32.51222 Africa/Kampala
0.40444 32.45944 Africa/Kampala
0.69299 34.18085 Africa/Kampala
1.71464 33.61113 Africa/Kampala
1.145 33.70944 Africa/Kampala
2.41669 30.98551 Africa/Kampala
-0.88333 29.65 Africa/Kampala
-0.87944 30.26417 Africa/Kampala
0.41861 33.17306 Africa/Kampala
2.47826 31.08894 Africa/Kampala
0.68944 32.42139 Africa/Kampala
0.35333 32.75528 Africa/Kampala
0.56055 31.39 Africa/Kampala
3.66088 31.72474 Africa/Kampala
0.4175 32.02278 Africa/Kampala
-0.608 30.65 Africa/Kampala
1.08209 34.17503 Africa/Kampala
1.67444 31.715 Africa/Kampala
-0.33379 31.73409 Africa/Kampala
0.84917 32.47306 Africa/Kampala
0.37722 32.91972 Africa/Kampala
2.235 32.90972 Africa/Kampala
0.63278 30.62139 Africa/Kampala
2.98056 34.13306 Africa/Kampala
3.27833 32.88667 Africa/Kampala
0.3475 32.64917 Africa/Kampala
0.7025 32.88861 Africa/Kampala
0.18333 30.08333 Africa/Kampala
0.1866 30.45393 Africa/Kampala
0.31628 32.58219 Africa/Kampala
-1.24857 29.98993 Africa/Kampala
0.43902 33.20317 Africa/Kampala
0.60917 33.46861 Africa/Kampala
1.43556 31.34361 Africa/Kampala
2.77457 32.29899 Africa/Kampala
0.66174 30.2748 Africa/Kampala
0.06444 32.44694 Africa/Kampala
-0.59167 30.62861 Africa/Kampala
0.64222 33.17444 Africa/Kampala
0.45444 34.07583 Africa/Kampala
0.76972 33.61306 Africa/Kampala
0.74139 30.04167 Africa/Kampala
0.57139 33.74167 Africa/Kampala
3.02013 30.91105 Africa/Kampala
3.37786 31.7909 Africa/Kampala
38.73289 -77.05803 America/New_York
33.40178 -86.95444 America/Chicago
37.08339 -88.60005 America/Chicago
33.52066 -86.80249 America/Chicago
33.64566 -86.6836 America/Chicago
30.60353 -87.9036 America/Chicago
34.60593 -86.98334 America/Chicago
31.22323 -85.39049 America/Chicago
34.80953 -87.64947 America/Chicago
31.31517 -85.85522 America/Chicago
30.52297 -87.90333 America/Chicago
34.79981 -87.67725 America/Chicago
34.01426 -86.00664 America/Chicago
33.29622 -86.8436 America/Chicago
33.47177 -86.80082 America/Chicago
33.40539 -86.81138 America/Chicago
33.45122 -86.99666 America/Chicago
34.73037 -86.5861 America/Chicago
34.69926 -86.74833 America/Chicago
30.69436 -88.04305 America/Chicago
32.36681 -86.29997 America/Chicago
33.50094 -86.75221 America/Chicago
33.22901 -87.57723 America/Chicago
32.64541 -85.37828 America/Chicago
33.61427 -85.83496 America/Chicago
33.28567 -86.80999 America/Chicago
32.47098 -85.00077 America/New_York
32.46402 -86.4597 America/Chicago
30.7388 -88.07889 America/Chicago
32.40736 -87.0211 America/Chicago
33.43594 -86.1058 America/Chicago
30.59019 -88.17084 America/Chicago
31.80877 -85.96995 America/Chicago
33.61983 -86.60888 America/Chicago
33.20984 -87.56917 America/Chicago
33.44872 -86.78777 America/Chicago
36.42952 -94.2316 America/Chicago
36.48146 -94.27326 America/Chicago
34.56454 -92.58683 America/Chicago
36.37285 -94.20882 America/Chicago
35.9273 -89.91898 America/Chicago
34.59593 -92.48905 America/Chicago
34.97453 -92.01653 America/Chicago
35.0887 -92.4421 America/Chicago
33.20763 -92.66627 America/Chicago
36.06258 -94.15743 America/Chicago
35.00815 -90.78983 America/Chicago
35.38592 -94.39855 America/Chicago
34.5037 -93.05518 America/Chicago
34.8662 -92.11015 America/Chicago
35.8423 -90.70428 America/Chicago
34.74648 -92.28959 America/Chicago
34.86676 -92.40432 America/Chicago
34.76954 -92.26709 America/Chicago
36.0584 -90.49733 America/Chicago
34.22843 -92.0032 America/Chicago
36.33202 -94.11854 America/Chicago
35.27842 -93.13379 America/Chicago
35.25064 -91.73625 America/Chicago
36.18814 -94.5405 America/Chicago
36.18674 -94.12881 America/Chicago
33.44179 -94.03769 America/Chicago
35.43676 -94.34827 America/Chicago
35.14648 -90.18454 America/Chicago
38.89511 -77.03637 America/New_York
39.62928 -75.65826 America/New_York
39.15817 -75.52437 America/New_York
39.44956 -75.71632 America/New_York
39.68372 -75.74966 America/New_York
39.74595 -75.54659 America/New_York
28.66111 -81.36562 America/New_York
28.68055 -81.50952 America/New_York
25.95648 -80.13921 America/New_York
27.89641 -81.84314 America/New_York
28.32667 -82.68343 America/New_York
27.42532 -82.59038 America/New_York
26.68451 -80.66756 America/New_York
30.46159 -87.31497 America/Chicago
27.89364 -82.24037 America/New_York
26.34508 -80.14671 America/New_York
26.35869 -80.0831 America/New_York
26.33981 -81.7787 America/New_York
26.52535 -80.06643 America/New_York
27.49893 -82.57482 America/New_York
27.9378 -82.28592 America/New_York
30.46881 -87.23608 America/Chicago
25.82176 -80.24116 America/New_York
28.33584 -81.35313 America/New_York
26.56285 -81.94953 America/New_York
25.94065 -80.2456 America/New_York
28.05002 -82.49287 America/New_York
28.06752 -82.52093 America/New_York
28.67778 -81.32785 America/New_York
28.07835 -82.56982 America/New_York
27.96585 -82.8001 America/New_York
28.54944 -81.77285 America/New_York
28.38612 -80.742 America/New_York
26.25175 -80.17894 America/New_York
26.05731 -80.27172 America/New_York
25.72149 -80.26838 America/New_York
26.27119 -80.2706 America/New_York
25.74593 -80.3045 America/New_York
25.63399 -80.43228 America/New_York
25.94815 -80.317 America/New_York
30.76213 -86.57051 America/Chicago
25.6151 -80.31061 America/New_York
25.58066 -80.34672 America/New_York
26.05231 -80.14393 America/New_York
26.06287 -80.2331 America/New_York
29.21081 -81.02283 America/New_York
29.02832 -81.30312 America/New_York
28.88305 -81.30868 America/New_York
26.31841 -80.09977 America/New_York
26.46146 -80.07282 America/New_York
28.90054 -81.26367 America/New_York
25.81954 -80.35533 America/New_York
28.01974 -82.77177 America/New_York
28.11085 -82.69482 America/New_York
30.42881 -87.17997 America/Chicago
28.98888 -80.90228 America/Kentucky/Monticello
28.01769 -82.50619 America/New_York
30.51881 -87.27275 America/Chicago
26.43814 -81.80675 America/New_York
28.85277 -81.68535 America/New_York
30.5102 -87.21247 America/Chicago
27.58031 -80.38672 America/New_York
26.12231 -80.14338 America/New_York
26.62168 -81.84059 America/New_York
27.44671 -80.32561 America/New_York
30.40576 -86.61884 America/Indiana/Tell_City
25.77288 -80.34783 America/New_York
30.11107 -81.64176 America/Kentucky/Monticello
29.65163 -82.32483 America/New_York
25.7076 -80.32561 America/New_York
26.18787 -81.69509 America/New_York
25.91176 -80.20033 America/New_York
26.62368 -80.12532 America/New_York
28.11418 -81.61785 America/New_York
25.9812 -80.14838 America/New_York
25.8576 -80.27811 America/New_York
25.8651 -80.3245 America/New_York
28.18779 -82.73955 America/New_York
26.0112 -80.14949 America/New_York
25.46872 -80.47756 America/New_York
26.41869 -81.4173 America/New_York
26.52036 -81.96398 America/New_York
25.96231 -80.17671 America/New_York
30.33218 -81.65565 America/New_York
30.29469 -81.39314 America/New_York
28.29306 -82.6901 America/New_York
26.93422 -80.09421 America/New_York
25.70816 -80.407 America/New_York
25.67927 -80.31727 America/New_York
24.5557 -81.78259 America/New_York
28.15585 -82.62121 America/New_York
28.30468 -81.41667 America/New_York
28.50167 -81.54091 America/New_York
28.07418 -82.47176 America/New_York
26.6159 -80.05699 America/New_York
26.61649 -80.10102 America/New_York
28.03947 -81.9498 America/New_York
30.12996 -81.76815 America/New_York
28.2189 -82.45759 America/New_York
27.90947 -82.78732 America/New_York
26.16647 -80.20838 America/New_York
26.14036 -80.21338 America/New_York
27.82114 -82.67927 America/New_York
28.81082 -81.87786 America/New_York
26.62535 -81.6248 America/New_York
25.49539 -80.42922 America/New_York
28.15112 -82.46148 America/New_York
30.24548 -85.64826 America/Chicago
28.62778 -81.36312 America/New_York
26.24453 -80.20644 America/New_York
28.38556 -81.36646 America/New_York
28.08363 -80.60811 America/New_York
28.53917 -80.672 America/New_York
25.77427 -80.19366 America/New_York
25.79065 -80.13005 America/New_York
25.94204 -80.2456 America/New_York
25.90871 -80.30866 America/New_York
25.98731 -80.23227 America/New_York
30.42103 -87.30747 America/Chicago
26.14204 -81.79481 America/New_York
30.40159 -86.86357 America/Chicago
29.02582 -80.927 America/New_York
25.94898 -80.21227 America/New_York
26.66729 -81.88009 America/New_York
26.2173 -80.22588 America/New_York
25.89009 -80.18671 America/New_York
25.93315 -80.16255 America/New_York
27.04422 -82.23593 America/New_York
28.47112 -81.42452 America/New_York
26.17231 -80.13199 America/New_York
29.1872 -82.14009 America/New_York
28.56917 -81.54396 America/New_York
25.94843 -80.1506 America/New_York
25.90232 -80.25033 America/New_York
28.53834 -81.37924 America/New_York
29.28581 -81.05589 America/New_York
28.67 -81.20812 America/New_York
30.59936 -87.16108 America/Chicago
28.03446 -80.58866 America/New_York
26.82339 -80.13865 America/New_York
27.16783 -80.26616 America/New_York
29.58497 -81.20784 America/New_York
28.07807 -82.76371 America/New_York
26.6359 -80.09615 America/New_York
30.17746 -81.38758 America/New_York
25.62177 -80.32477 America/New_York
30.15946 -85.65983 America/Chicago
26.31008 -80.23727 America/New_York
26.00315 -80.22394 America/New_York
30.42131 -87.21691 America/Chicago
28.55778 -81.4534 America/New_York
25.66705 -80.30811 America/New_York
27.8428 -82.69954 America/New_York
25.86898 -80.21699 America/New_York
28.01863 -82.11286 America/New_York
26.13421 -80.23184 America/New_York
28.14029 -81.45841 America/New_York
26.23786 -80.12477 America/New_York
30.23969 -81.38564 America/New_York
26.97617 -82.09064 America/New_York
29.13832 -80.99561 America/New_York
27.29393 -80.35033 America/New_York
25.53844 -80.40894 America/New_York
26.92978 -82.04537 America/New_York
26.91756 -82.07842 America/New_York
25.6105 -80.42971 America/New_York
27.86614 -82.32648 America/New_York
26.77534 -80.0581 America/New_York
28.35084 -80.72533 America/New_York
26.7084 -80.2306 America/New_York
27.72086 -82.43315 America/New_York
27.99085 -82.69316 America/New_York
28.2489 -81.28118 America/New_York
27.77086 -82.67927 America/New_York
26.4673 -81.80147 America/New_York
26.33924 -80.18755 America/New_York
28.80055 -81.27312 America/New_York
27.33643 -82.53065 America/New_York
27.81642 -80.47061 America/New_York
27.83975 -82.79121 America/New_York
27.4631 -82.58176 America/New_York
25.59761 -80.38061 America/New_York
28.39306 -81.3834 America/New_York
28.47688 -82.52546 America/New_York
27.19755 -80.25283 America/Kentucky/Monticello
27.71809 -82.35176 America/New_York
25.95065 -80.12282 America/New_York
26.15767 -80.28611 America/New_York
25.70594 -80.35228 America/New_York
30.43826 -84.28073 America/New_York
26.21286 -80.24977 America/New_York
25.75871 -80.39839 America/New_York
27.94752 -82.45843 America/New_York
28.14612 -82.75677 America/New_York
28.0353 -82.38926 America/New_York
25.67066 -80.40117 America/New_York
25.67149 -80.4445 America/New_York
28.93408 -81.95994 America/New_York
25.64205 -80.39839 America/New_York
28.61222 -80.80755 America/New_York
28.01057 -82.57732 America/New_York
25.74649 -80.36755 America/New_York
27.93789 -82.23644 America/New_York
27.09978 -82.45426 America/New_York
27.63864 -80.39727 America/New_York
28.69861 -81.42563 America/New_York
28.69861 -81.42563 America/New_York
26.65868 -80.24144 America/New_York
28.23973 -82.32787 America/New_York
28.05502 -82.60982 America/New_York
25.85704 -80.23699 America/New_York
28.07168 -80.65339 America/New_York
26.71534 -80.05337 America/New_York
30.42659 -87.27969 America/Chicago
27.81993 -82.68944 America/New_York
25.75482 -80.32727 America/New_York
26.10037 -80.39977 America/New_York
28.56528 -81.58618 America/New_York
28.02224 -81.73286 America/New_York
28.6 -81.33924 America/New_York
28.69889 -81.30812 America/New_York
30.45575 -86.63829 America/Chicago
34.06593 -84.67688 America/New_York
31.57851 -84.15574 America/New_York
34.07538 -84.29409 America/New_York
32.07239 -84.23269 America/New_York
33.96095 -83.37794 America/New_York
33.749 -84.38798 America/New_York
33.47097 -81.97484 America/New_York
33.75483 -84.26742 America/New_York
31.14995 -81.49149 America/New_York
34.50259 -84.95105 America/New_York
34.23676 -84.49076 America/New_York
33.58011 -85.07661 America/New_York
34.1651 -84.79994 America/New_York
32.46098 -84.98771 America/New_York
33.66761 -84.01769 America/New_York
34.7698 -84.97022 America/New_York
33.77483 -84.29631 America/New_York
33.7515 -84.74771 America/New_York
32.54044 -82.90375 America/New_York
34.00288 -84.14464 America/New_York
33.94621 -84.33465 America/New_York
33.67955 -84.43937 America/New_York
33.53375 -82.13067 America/New_York
33.44873 -84.45493 America/New_York
33.62205 -84.36909 America/New_York
34.29788 -83.82407 America/New_York
33.24678 -84.26409 America/New_York
31.84688 -81.59595 America/New_York
34.02343 -84.61549 America/New_York
30.79996 -81.68983 America/New_York
33.03929 -85.03133 America/New_York
33.95621 -83.98796 America/New_York
33.794 -84.66049 America/New_York
33.81574 -84.56194 America/New_York
32.84069 -83.6324 America/New_York
33.9526 -84.54993 America/New_York
33.51736 -82.07567 America/New_York
33.44734 -84.14686 America/New_York
33.08014 -83.2321 America/New_York
33.38067 -84.79966 America/New_York
33.8651 -84.33659 America/New_York
33.79038 -84.30603 America/New_York
33.81677 -84.31326 America/New_York
33.39678 -84.59576 America/New_York
32.11548 -81.24706 America/New_York
33.74538 -84.13158 America/New_York
33.57261 -84.41326 America/New_York
34.25704 -85.16467 America/New_York
34.02316 -84.36159 America/New_York
30.73051 -81.54649 America/New_York
33.92427 -84.37854 America/New_York
32.08354 -81.09983 America/New_York
33.88399 -84.51438 America/New_York
33.85733 -84.01991 America/New_York
32.44879 -81.78317 America/New_York
33.54428 -84.23381 America/New_York
34.10649 -84.03352 America/New_York
34.05149 -84.0713 America/New_York
30.83658 -83.97878 America/New_York
31.45046 -83.5085 America/New_York
33.85455 -84.21714 America/New_York
33.58706 -84.54243 America/New_York
30.8327 -83.27849 America/New_York
32.62098 -83.5999 America/New_York
32.00355 -80.97372 America/New_York
34.10149 -84.51938 America/New_York
38.8906 -90.18428 America/Chicago
38.52005 -89.98399 America/Chicago
38.57088 -90.19011 America/Chicago
37.72727 -89.21675 America/Chicago
39.49615 -88.17615 America/Chicago
38.67033 -89.98455 America/Chicago
39.84031 -88.9548 America/Chicago
38.6245 -90.15094 America/Chicago
38.81144 -89.95316 America/Chicago
38.58894 -89.99038 America/Chicago
38.9556 -90.18678 America/Chicago
38.70144 -90.14872 America/Chicago
39.73394 -90.22901 America/Chicago
37.73061 -88.93313 America/Chicago
39.48309 -88.37283 America/Chicago
38.31727 -88.90312 America/Chicago
38.59227 -89.91121 America/Chicago
39.9356 -91.40987 America/Chicago
39.80172 -89.64371 America/Chicago
38.91144 -90.15066 America/Chicago
39.16532 -86.52639 America/Indiana/Indianapolis
39.84338 -86.39777 America/Indiana/Indianapolis
39.97837 -86.11804 America/Indiana/Indianapolis
38.29674 -85.75996 America/Kentucky/Louisville
39.20144 -85.92138 America/Indiana/Indianapolis
37.97476 -87.55585 America/Chicago
39.95559 -86.01387 America/Indiana/Indianapolis
39.78504 -85.76942 America/Indiana/Indianapolis
39.61366 -86.10665 America/Indiana/Indianapolis
39.76838 -86.15804 America/Indiana/Indianapolis
38.39144 -86.93111 America/Indiana/Vincennes
38.27757 -85.73718 America/Kentucky/Louisville
39.83865 -86.02526 America/Indiana/Indianapolis
38.28562 -85.82413 America/Kentucky/Louisville
39.92894 -85.37025 America/Indiana/Indianapolis
39.70421 -86.39944 America/Indiana/Indianapolis
39.82894 -84.89024 America/Indiana/Indianapolis
38.95922 -85.89025 America/Indiana/Indianapolis
39.52144 -85.77692 America/Indiana/Indianapolis
39.4667 -87.41391 America/Indiana/Indianapolis
38.67727 -87.52863 America/Indiana/Vincennes
37.54557 -97.26893 America/Chicago
38.4039 -96.18166 America/Chicago
38.81084 -94.92719 America/Chicago
38.36446 -98.76481 America/Chicago
38.87918 -99.32677 America/Chicago
38.06084 -97.92977 America/Chicago
39.02861 -96.8314 America/Chicago
39.11417 -94.62746 America/Chicago
38.97167 -95.23525 America/Chicago
39.31111 -94.92246 America/Chicago
38.96667 -94.6169 America/Chicago
38.95362 -94.73357 America/Chicago
39.18361 -96.57167 America/Chicago
38.04668 -97.34504 America/Chicago
38.8814 -94.81913 America/Chicago
38.98223 -94.67079 America/Chicago
37.41088 -94.70496 America/Chicago
38.99167 -94.63357 America/Chicago
38.84028 -97.61142 America/Chicago
39.04167 -94.72024 America/Chicago
39.04833 -95.67804 America/Chicago
37.69224 -97.33754 America/Chicago
38.47841 -82.63794 America/New_York
36.99032 -86.4436 America/Chicago
39.02756 -84.72411 America/New_York
39.08367 -84.50855 America/New_York
37.64563 -84.77217 America/New_York
37.69395 -85.85913 America/New_York
39.01673 -84.60078 America/New_York
38.15979 -85.58774 America/Kentucky/Louisville
38.99895 -84.62661 America/New_York
39.07506 -84.44716 America/New_York
38.20091 -84.87328 America/New_York
38.2098 -84.55883 America/New_York
37.83615 -87.59001 America/Chicago
38.14285 -85.62413 America/Kentucky/Louisville
36.8656 -87.48862 America/Chicago
38.94312 -84.54411 America/New_York
38.19424 -85.5644 America/Kentucky/Louisville
37.98869 -84.47772 America/New_York
38.0498 -84.45855 America/New_York
38.25424 -85.75941 America/Kentucky/Louisville
37.3281 -87.49889 America/Chicago
36.61033 -88.31476 America/Chicago
38.16007 -85.65968 America/Kentucky/Louisville
39.09145 -84.49578 America/New_York
37.88063 -84.573 America/New_York
38.14118 -85.68774 America/Kentucky/Louisville
37.77422 -87.11333 America/Chicago
38.14535 -85.8583 America/Kentucky/Louisville
37.84035 -85.94913 America/New_York
37.74786 -84.29465 America/New_York
38.25285 -85.65579 America/Kentucky/Louisville
38.20007 -85.82274 America/Kentucky/Louisville
38.11118 -85.87024 America/Kentucky/Louisville
37.99008 -84.17965 America/New_York
31.31129 -92.44514 America/Chicago
30.45075 -91.15455 America/Chicago
29.6241 -90.7512 America/Chicago
32.51599 -93.73212 America/Chicago
30.55435 -91.03677 America/Chicago
29.9427 -89.9634 America/Chicago
29.84576 -90.10674 America/Chicago
29.91465 -90.05396 America/Chicago
30.50436 -90.4612 America/Chicago
29.90354 -90.07729 America/Chicago
29.59577 -90.71953 America/Chicago
29.99409 -90.24174 America/Chicago
30.22409 -92.01984 America/Chicago
30.21309 -93.2044 America/Chicago
30.06659 -90.48008 America/Chicago
29.89937 -90.10035 America/Chicago
29.98409 -90.15285 America/Chicago
29.97854 -90.16396 America/Chicago
32.50931 -92.1193 America/Chicago
31.76072 -93.08627 America/Chicago
30.00354 -91.81873 America/Chicago
29.95465 -90.07507 America/Chicago
30.53353 -92.08151 America/Chicago
30.30297 -90.97205 America/Chicago
32.52321 -92.63793 America/Chicago
30.4013 -91.00094 America/Chicago
32.52515 -93.75018 America/Chicago
30.27519 -89.78117 America/Chicago
30.23659 -93.37738 America/Chicago
29.9102 -90.03257 America/Chicago
39.00317 -76.97192 America/New_York
38.97845 -76.49218 America/New_York
39.25455 -76.69997 America/New_York
39.03206 -76.50274 America/New_York
39.07955 -77.07303 America/New_York
39.29038 -76.61219 America/New_York
39.3726 -77.43526 America/New_York
39.03483 -76.90747 America/New_York
38.98067 -77.10026 America/New_York
38.94278 -76.73028 America/New_York
39.05761 -76.93581 America/New_York
38.804 -76.90664 America/New_York
39.39427 -76.52358 America/New_York
39.27205 -76.73192 America/New_York
38.96372 -76.99081 America/New_York
39.10816 -76.99775 America/New_York
38.76511 -76.89831 America/New_York
39.48122 -76.64386 America/New_York
38.98067 -76.93692 America/New_York
39.24038 -76.83942 America/New_York
39.00178 -76.68747 America/New_York
39.65287 -78.76252 America/New_York
39.28844 -77.20387 America/New_York
39.25066 -76.52052 America/New_York
38.96206 -76.92192 America/New_York
38.77428 -76.07633 America/New_York
39.41872 -76.2944 America/New_York
39.40371 -76.95026 America/New_York
39.60678 -75.83327 America/New_York
39.21261 -76.71358 America/New_York
39.26733 -76.79831 America/New_York
39.30927 -76.47496 America/New_York
39.07622 -76.95775 America/New_York
39.18316 -76.64024 America/New_York
38.70734 -77.02303 America/New_York
39.41427 -77.41054 America/New_York
39.14344 -77.20137 America/New_York
39.17316 -77.27165 America/New_York
38.819 -76.99859 America/New_York
39.16261 -76.62469 America/New_York
39.13955 -76.54774 America/New_York
39.00455 -76.87553 America/New_York
39.64176 -77.71999 America/New_York
38.83289 -76.95942 America/New_York
38.95594 -76.94553 America/New_York
39.25094 -76.76469 America/New_York
39.10705 -76.48496 America/New_York
38.934 -76.89664 America/New_York
38.98872 -76.98136 America/New_York
39.09928 -76.84831 America/New_York
39.34066 -76.72219 America/New_York
39.09205 -76.81775 America/New_York
39.33427 -76.43941 America/New_York
39.34788 -76.76997 America/New_York
39.17677 -77.19526 America/New_York
39.53983 -76.35496 America/New_York
39.04455 -77.11887 America/New_York
39.08289 -77.26498 America/New_York
39.084 -76.70025 America/New_York
39.15316 -77.06692 America/New_York
39.41955 -76.78025 America/New_York
38.80345 -76.9897 America/New_York
39.37733 -76.53969 America/New_York
38.97956 -76.53052 America/New_York
39.10733 -76.57108 America/New_York
39.41261 -76.46357 America/New_York
39.37427 -76.72247 America/New_York
39.01817 -77.20859 America/New_York
39.36733 -76.79525 America/New_York
39.14539 -77.14415 America/New_York
39.46955 -76.82942 America/New_York
39.084 -77.15276 America/New_York
39.32011 -76.51552 America/New_York
39.33844 -76.47968 America/New_York
38.60317 -76.93858 America/New_York
38.36067 -75.59937 America/New_York
38.96789 -76.84497 America/New_York
39.13705 -76.6983 America/New_York
39.07039 -76.54524 America/New_York
38.99067 -77.02609 America/New_York
39.53316 -76.33746 America/New_York
39.129 -76.6258 America/New_York
39.06983 -76.85025 America/New_York
38.84872 -76.92386 America/New_York
38.97789 -77.00748 America/New_York
39.4015 -76.60191 America/New_York
38.62456 -76.93914 America/New_York
39.20705 -76.72692 America/New_York
39.57538 -76.99581 America/New_York
39.03983 -77.05526 America/New_York
39.03983 -76.99303 America/New_York
39.32288 -76.72803 America/New_York
38.55061 -90.33317 America/Chicago
38.43283 -90.37762 America/Chicago
38.59505 -90.54623 America/Chicago
38.81195 -94.5319 America/Chicago
39.01695 -94.28161 America/Chicago
37.30588 -89.51815 America/Chicago
38.66311 -90.57707 America/Chicago
38.64255 -90.32373 America/Chicago
38.95171 -92.33407 America/Chicago
38.5245 -90.35734 America/Chicago
38.66089 -90.42262 America/Chicago
39.09556 -94.35523 America/Chicago
37.78088 -90.42179 America/Chicago
38.74422 -90.30539 America/Chicago
38.78922 -90.32261 America/Chicago
39.20389 -94.55468 America/Chicago
38.88584 -94.53301 America/Chicago
39.70838 -91.35848 America/Chicago
38.77144 -90.37095 America/Chicago
39.09112 -94.41551 America/Chicago
38.5767 -92.17352 America/Chicago
37.08423 -94.51328 America/Chicago
39.09973 -94.57857 America/Chicago
38.58339 -90.40678 America/Chicago
38.91084 -94.38217 America/Chicago
38.53339 -90.27928 America/Chicago
39.24611 -94.41912 America/Chicago
38.597 -90.50929 America/Chicago
38.71311 -90.42984 America/Chicago
38.50839 -90.32289 America/Chicago
37.04339 -93.29435 America/Chicago
38.81061 -90.69985 America/Chicago
38.47005 -90.30456 America/Chicago
38.70116 -90.36234 America/Chicago
37.02089 -93.20602 America/Chicago
36.757 -90.39289 America/Chicago
38.80195 -94.45273 America/Chicago
39.00862 -94.46356 America/Chicago
37.95143 -91.77127 America/Chicago
38.78394 -90.48123 America/Chicago
39.76861 -94.84663 America/Chicago
38.62727 -90.19789 America/Chicago
38.80033 -90.62651 America/Chicago
38.70446 -93.22826 America/Chicago
36.87672 -89.58786 America/Chicago
38.78783 -90.21594 America/Chicago
37.21533 -93.29824 America/Chicago
38.65588 -90.30928 America/Chicago
38.76279 -93.73605 America/Chicago
38.59255 -90.35734 America/Chicago
38.81144 -90.85291 America/Chicago
38.58283 -90.6629 America/Chicago
30.39603 -88.88531 America/Chicago
32.2732 -89.98592 America/Chicago
34.20011 -90.57093 America/Chicago
32.34153 -90.32176 America/Chicago
33.49567 -88.42726 America/Chicago
30.38576 -88.61169 America/Chicago
33.41012 -91.06177 America/Chicago
33.51623 -90.17953 America/Chicago
30.36742 -89.09282 America/Chicago
31.32712 -89.29034 America/Chicago
34.95537 -90.03481 America/Chicago
32.29876 -90.18481 America/Chicago
31.69405 -89.13061 America/Chicago
32.46181 -90.11536 America/Chicago
32.36431 -88.70366 America/Chicago
31.56044 -91.40317 America/Chicago
30.41131 -88.82781 America/Chicago
34.96176 -89.82953 America/Chicago
34.3665 -89.51925 America/Chicago
30.36576 -88.55613 America/Chicago
32.27459 -90.13203 America/Chicago
32.42848 -90.13231 America/Chicago
34.98898 -90.01259 America/Chicago
33.4504 -88.81839 America/Chicago
34.25761 -88.70339 America/Chicago
32.35265 -90.87788 America/Chicago
30.40409 -89.0942 America/Chicago
35.35014 -80.20006 America/New_York
35.73265 -78.85029 America/New_York
35.70791 -79.81364 America/New_York
35.60095 -82.55402 America/New_York
36.2168 -81.67455 America/New_York
36.09569 -79.4378 America/New_York
35.91014 -79.07529 America/New_York
35.79154 -78.78112 America/New_York
35.9132 -79.05584 America/New_York
35.22709 -80.84313 America/New_York
35.65071 -78.45639 America/New_York
36.02153 -80.382 America/New_York
35.40875 -80.57951 America/New_York
35.4868 -80.86007 America/New_York
35.99403 -78.89862 America/New_York
36.48847 -79.7667 America/New_York
36.2946 -76.25105 America/New_York
35.05266 -78.87836 America/New_York
35.58432 -78.80001 America/New_York
35.71126 -78.61417 America/New_York
35.26208 -81.1873 America/New_York
35.38488 -77.99277 America/New_York
36.07264 -79.79198 America/New_York
35.61266 -77.36635 America/New_York
34.87905 -76.90133 America/New_York
36.32959 -78.39916 America/New_York
35.73319 -81.3412 America/New_York
35.95569 -80.00532 America/New_York
35.65127 -78.83362 America/New_York
34.97044 -78.94531 America/New_York
35.41069 -80.84285 America/New_York
35.07681 -80.66924 America/New_York
34.75405 -77.43024 America/New_York
35.48736 -80.62173 America/New_York
36.11986 -80.07365 America/New_York
35.26266 -77.58164 America/New_York
34.77405 -79.46282 America/New_York
35.91402 -81.53898 America/New_York
35.82403 -80.25338 America/New_York
34.61822 -79.00864 America/New_York
35.11681 -80.72368 America/New_York
35.17959 -80.64729 America/New_York
34.98543 -80.54951 America/New_York
35.58486 -80.81007 America/New_York
35.74541 -81.68482 America/New_York
35.82348 -78.82556 America/New_York
35.10849 -77.04411 America/New_York
35.7721 -78.63861 America/New_York
36.46154 -77.65415 America/New_York
35.93821 -77.79053 America/New_York
35.67097 -80.47423 America/New_York
35.47988 -79.1803 America/New_York
35.29235 -81.53565 America/New_York
35.78264 -80.8873 America/New_York
35.88264 -80.08199 America/New_York
35.97987 -78.50972 America/New_York
35.78682 -78.66389 America/New_York
34.22573 -77.94471 America/New_York
35.72127 -77.91554 America/New_York
36.09986 -80.24422 America/New_York
39.36428 -74.42293 America/New_York
39.42734 -75.23408 America/New_York
39.92595 -75.11962 America/New_York
39.93484 -75.03073 America/New_York
39.70289 -75.11184 America/New_York
39.7765 -74.86238 America/New_York
39.82428 -74.99767 America/New_York
39.95261 -74.99239 America/New_York
39.40206 -75.03934 America/New_York
39.74345 -74.28098 America/New_York
39.95622 -75.05795 America/New_York
39.38984 -74.52404 America/New_York
39.44595 -75.02879 America/New_York
39.95373 -74.19792 America/New_York
39.48623 -75.02573 America/New_York
39.68623 -74.99517 America/New_York
39.32924 -82.10126 America/New_York
39.70923 -84.06327 America/New_York
39.62839 -84.15938 America/New_York
39.162 -84.45689 America/New_York
39.96118 -82.99879 America/New_York
39.75895 -84.19161 America/New_York
39.82089 -84.01938 America/New_York
39.34589 -84.5605 America/New_York
39.29034 -84.50411 America/New_York
39.88145 -83.09296 America/New_York
39.3995 -84.56134 America/New_York
39.84395 -84.12466 America/New_York
39.6895 -84.16883 America/New_York
39.71368 -82.59933 America/New_York
39.43534 -84.20299 America/New_York
39.36006 -84.30994 America/New_York
39.64284 -84.28661 America/New_York
39.51506 -84.39828 America/New_York
39.15561 -84.45966 America/New_York
39.507 -84.74523 America/New_York
39.88423 -82.7535 America/New_York
38.73174 -82.99767 America/New_York
39.95479 -82.81212 America/New_York
39.77978 -84.1241 America/New_York
39.55228 -84.23327 America/New_York
39.92423 -83.80882 America/New_York
39.79728 -84.31133 America/New_York
39.99451 -83.06241 America/New_York
39.89061 -84.19883 America/New_York
39.21311 -84.59939 America/New_York
39.96673 -82.88546 America/New_York
39.68478 -83.92965 America/New_York
39.94035 -82.01319 America/New_York
34.77453 -96.67834 America/Chicago
34.63813 -99.33398 America/Chicago
34.17426 -97.14363 America/Chicago
36.74731 -95.98082 America/Chicago
35.51867 -97.63226 America/Chicago
35.94204 -95.88332 America/Chicago
36.0526 -95.79082 America/Chicago
35.05256 -97.93643 America/Chicago
36.3126 -95.61609 America/Chicago
35.44201 -97.44087 America/Chicago
34.5023 -97.95781 America/Chicago
33.99399 -96.37082 America/Chicago
35.65283 -97.4781 America/Chicago
35.53227 -97.95505 America/Chicago
36.39559 -97.87839 America/Chicago
36.02287 -95.96833 America/Chicago
34.60869 -98.39033 America/Chicago
34.93343 -95.76971 America/Chicago
35.44951 -97.3967 America/Chicago
35.33951 -97.4867 America/Chicago
35.74788 -95.36969 America/Chicago
35.38423 -97.72449 America/Chicago
35.22257 -97.43948 America/Chicago
35.46756 -97.51643 America/Chicago
36.26954 -95.85471 America/Chicago
36.70698 -97.08559 America/Chicago
36.13981 -96.10889 America/Chicago
35.9987 -96.11417 America/Chicago
35.32729 -96.9253 America/Chicago
36.11561 -97.05837 America/Chicago
35.91537 -94.96996 America/Chicago
36.15398 -95.99278 America/Chicago
35.50672 -97.76254 America/Chicago
39.93759 -77.6611 America/New_York
39.84956 -75.35575 America/New_York
39.94706 -75.29213 America/New_York
39.80066 -76.98304 America/New_York
39.95234 -75.16379 America/New_York
39.93067 -75.32019 America/New_York
39.96066 -75.60549 America/New_York
39.9626 -76.72774 America/New_York
33.56042 -81.71955 America/New_York
34.50344 -82.65013 America/New_York
32.77657 -79.93092 America/New_York
34.00071 -81.03481 America/New_York
33.836 -79.04781 America/New_York
34.82984 -82.60152 America/New_York
34.19543 -79.76256 America/New_York
32.98101 -80.03259 America/New_York
34.85262 -82.39401 America/New_York
34.1954 -82.16179 America/New_York
34.93873 -82.22706 America/New_York
32.91851 -80.02203 America/New_York
32.21632 -80.75261 America/New_York
33.98154 -81.23621 America/New_York
34.77873 -82.31012 America/New_York
32.79407 -79.86259 America/New_York
33.68906 -78.88669 America/New_York
33.5018 -81.96512 America/New_York
32.88856 -80.00751 America/New_York
34.92487 -81.02508 America/New_York
34.04543 -81.12926 America/New_York
34.04876 -81.14648 America/New_York
34.73706 -82.25428 America/New_York
33.6835 -78.99837 America/New_York
34.94957 -81.93205 America/New_York
34.04876 -81.10315 America/New_York
33.0185 -80.17565 America/New_York
33.92044 -80.34147 America/New_York
34.92039 -82.29623 America/New_York
34.90373 -82.33317 America/New_York
35.20453 -89.87398 America/Chicago
36.03312 -86.78278 America/Chicago
36.02506 -86.77917 America/Chicago
36.59511 -82.18874 America/New_York
35.04563 -85.30968 America/New_York
36.52977 -87.35945 America/Chicago
35.15952 -84.87661 America/New_York
35.04204 -89.66453 America/Chicago
35.61507 -87.03528 America/Chicago
36.16284 -85.50164 America/Chicago
36.03452 -89.38563 America/Chicago
34.99591 -85.15023 America/New_York
35.06535 -85.24912 America/New_York
35.01424 -85.2519 America/New_York
35.88452 -84.15353 America/New_York
35.92506 -86.86889 America/Chicago
36.38838 -86.44666 America/Chicago
35.08676 -89.81009 America/Chicago
36.32311 -86.71333 America/Chicago
36.16316 -82.83099 America/New_York
36.30477 -86.62 America/Chicago
35.61452 -88.81395 America/Chicago
36.31344 -82.35347 America/New_York
36.54843 -82.56182 America/New_York
35.96064 -83.92074 America/New_York
36.01562 -86.58194 America/Chicago
36.20811 -86.2911 America/Chicago
35.75647 -83.97046 America/New_York
35.14953 -90.04898 America/Chicago
36.21398 -83.29489 America/New_York
36.20006 -86.51861 America/Chicago
35.84562 -86.39027 America/Chicago
36.16589 -86.78444 America/Chicago
35.08676 -90.05676 America/Chicago
36.01036 -84.26964 America/New_York
35.48341 -86.46027 America/Chicago
35.98284 -86.5186 America/Chicago
35.75118 -86.93 America/Chicago
36.50921 -86.885 America/Chicago
35.36202 -86.20943 America/Chicago
32.44874 -99.73314 America/Chicago
26.18369 -98.12306 America/Chicago
29.93244 -95.38021 America/Chicago
27.75225 -98.06972 America/Chicago
33.10317 -96.67055 America/Chicago
29.42385 -95.2441 America/Chicago
29.16941 -95.43188 America/Chicago
32.73569 -97.10807 America/Chicago
29.99883 -95.1766 America/Chicago
30.26715 -97.74306 America/Chicago
32.72874 -96.62277 America/Chicago
29.7355 -94.97743 America/Chicago
30.08605 -94.10185 America/Chicago
32.84402 -97.14307 America/Chicago
29.70579 -95.45883 America/Chicago
31.05601 -97.46445 America/Chicago
32.67319 -97.46058 America/Chicago
30.16688 -96.39774 America/Chicago
25.90175 -97.49748 America/Chicago
31.70932 -98.99116 America/Chicago
30.51353 -97.73973 America/Chicago
30.67436 -96.36996 America/Chicago
32.54208 -97.32085 America/Chicago
29.87522 -98.26251 America/Chicago
32.95373 -96.89028 America/Chicago
32.58847 -96.95612 America/Chicago
30.5052 -97.82029 America/Chicago
29.77606 -95.11465 America/Chicago
29.56162 -98.22696 America/Chicago
29.73884 -95.758 America/Chicago
32.34764 -97.38668 America/Chicago
29.77828 -95.17188 America/Chicago
30.62798 -96.33441 America/Chicago
32.88096 -97.15501 America/Chicago
30.31188 -95.45605 America/Chicago
29.51801 -98.31612 America/Chicago
32.95457 -97.01501 America/Chicago
31.12406 -97.90308 America/Chicago
33.15401 -97.06473 America/Chicago
27.80058 -97.39638 America/Chicago
32.09543 -96.46887 America/Chicago
29.96911 -95.69717 America/Chicago
32.78306 -96.80667 America/Chicago
32.58986 -96.85695 America/Chicago
29.70523 -95.12382 America/Chicago
33.75566 -96.53666 America/Chicago
33.21484 -97.13307 America/Chicago
29.46079 -95.05132 America/Chicago
26.17035 -98.05195 America/Chicago
32.6518 -96.90834 America/Chicago
26.30174 -98.16334 America/Chicago
32.32931 -96.62527 America/Chicago
32.83707 -97.08195 America/Chicago
32.92651 -96.89612 America/Chicago
33.01457 -97.09696 America/Chicago
32.72541 -97.32085 America/Chicago
29.53885 -95.44744 America/Chicago
29.5294 -95.20104 America/Chicago
33.15067 -96.82361 America/Chicago
33.62594 -97.13335 America/Chicago
29.30135 -94.7977 America/Chicago
32.91262 -96.63888 America/Chicago
31.43516 -97.74391 America/Chicago
30.63269 -97.67723 America/Chicago
32.74596 -96.99778 America/Chicago
32.93429 -97.07807 America/Chicago
33.13845 -96.11081 America/Chicago
29.94827 -93.91712 America/Chicago
32.79957 -97.26918 America/Chicago
31.08351 -97.65974 America/Chicago
26.19063 -97.6961 America/Chicago
33.09179 -97.04668 America/Chicago
29.76328 -95.36327 America/Chicago
29.99883 -95.26216 America/Chicago
30.72353 -95.55078 America/Chicago
32.82346 -97.17057 America/Chicago
32.81402 -96.94889 America/Chicago
30.4427 -97.77501 America/Chicago
32.93457 -97.25168 America/Chicago
30.04743 -99.14032 America/Chicago
31.11712 -97.7278 America/Chicago
27.51587 -97.85611 America/Chicago
29.99835 -97.87153 America/Chicago
29.66578 -95.01937 America/Chicago
29.03386 -95.43439 America/Chicago
32.59208 -96.75611 America/Chicago
27.50641 -99.50754 America/Chicago
29.50745 -95.09493 America/Chicago
30.57881 -97.85307 America/Chicago
33.04623 -96.99417 America/Chicago
33.16262 -96.93751 America/Chicago
32.5007 -94.74049 America/Chicago
31.33824 -94.7291 America/Chicago
32.56319 -97.14168 America/Chicago
32.54487 -94.36742 America/Chicago
26.20341 -98.23001 America/Chicago
33.19762 -96.61527 America/Chicago
26.1498 -97.91361 America/Chicago
32.7668 -96.59916 America/Chicago
32.48236 -96.99445 America/Chicago
32.80846 -98.11282 America/Chicago
26.21591 -98.32529 America/Chicago
29.69384 -95.66495 America/Chicago
29.61857 -95.53772 America/Chicago
33.15679 -94.96827 America/Chicago
33.01512 -96.61305 America/Chicago
31.60351 -94.65549 America/Chicago
29.97438 -93.9924 America/Chicago
29.703 -98.12445 America/Chicago
29.59412 -95.68078 America/Chicago
32.8343 -97.2289 America/Chicago
30.09299 -93.73655 America/Chicago
31.76212 -95.63079 America/Chicago
33.66094 -95.55551 America/Chicago
29.69106 -95.2091 America/Chicago
29.56357 -95.28605 America/Chicago
29.62607 -95.73162 America/Chicago
30.43937 -97.62 America/Chicago
26.1948 -98.18362 America/Chicago
33.01984 -96.69889 America/Chicago
29.89883 -93.92878 America/Chicago
27.87725 -97.32388 America/Chicago
32.94818 -96.72972 America/Chicago
32.93123 -96.45971 America/Chicago
29.55718 -95.80856 America/Chicago
30.50826 -97.6789 America/Chicago
32.9029 -96.56388 America/Chicago
32.97623 -96.59527 America/Chicago
32.86013 -97.36391 America/Chicago
29.42412 -98.49363 America/Chicago
26.13258 -97.6311 America/Chicago
26.18924 -98.15529 America/Chicago
29.88327 -97.94139 America/Chicago
29.55217 -98.26973 America/Chicago
29.56884 -97.96473 America/Chicago
33.63566 -96.60888 America/Chicago
29.66301 -95.23549 America/Chicago
32.94124 -97.13418 America/Chicago
30.07994 -95.41716 America/Chicago
29.61607 -95.55772 America/Chicago
32.2207 -98.20226 America/Chicago
29.61968 -95.63495 America/Chicago
33.13845 -95.60107 America/Chicago
30.57076 -97.40944 America/Chicago
31.09823 -97.34278 America/Chicago
32.73596 -96.27526 America/Chicago
33.42512 -94.04769 America/Chicago
29.38384 -94.9027 America/Chicago
33.08901 -96.88639 America/Chicago
30.15799 -95.48938 America/Chicago
32.35126 -95.30106 America/Chicago
29.54801 -98.29112 America/Chicago
32.85013 -96.80028 America/Chicago
29.20968 -99.78617 America/Chicago
28.80527 -97.0036 America/Chicago
31.54933 -97.14667 America/Chicago
32.85791 -97.25474 America/Chicago
32.38653 -96.84833 America/Chicago
32.7593 -97.79725 America/Chicago
26.15952 -97.99084 America/Chicago
32.75957 -97.45835 America/Chicago
33.91371 -98.49339 America/Chicago
33.01512 -96.53888 America/Chicago
38.80484 -77.04692 America/New_York
38.83039 -77.19637 America/New_York
38.88101 -77.10428 America/New_York
39.04372 -77.48749 America/New_York
38.85039 -77.1297 America/New_York
37.22957 -80.41394 America/New_York
37.52487 -77.55777 America/New_York
36.59649 -82.18847 America/New_York
38.73178 -77.4311 America/New_York
38.79345 -77.27165 America/New_York
37.22764 -80.01282 America/New_York
38.84039 -77.42888 America/New_York
38.89428 -77.4311 America/New_York
38.02931 -78.47668 America/New_York
38.56984 -77.26693 America/New_York
36.81904 -76.27494 America/New_York
37.35682 -77.44165 America/New_York
37.12985 -80.40894 America/New_York
37.26804 -77.40726 America/New_York
38.47318 -77.99666 America/New_York
38.63706 -77.31109 America/New_York
36.58597 -79.39502 America/New_York
37.03737 -76.33161 America/New_York
38.84622 -77.30637 America/New_York
38.78206 -77.14637 America/New_York
38.30318 -77.46054 America/New_York
38.99817 -77.28832 America/New_York
37.02987 -76.34522 America/New_York
38.44957 -78.86892 America/New_York
38.96955 -77.3861 America/New_York
37.54598 -77.32776 America/New_York
37.30432 -77.2872 America/New_York
38.74761 -77.08303 America/New_York
38.89511 -77.21165 America/New_York
38.86456 -77.18776 America/New_York
38.68789 -77.29776 America/New_York
37.64292 -77.50887 America/New_York
39.11566 -77.5636 America/New_York
38.81845 -77.14331 America/New_York
38.75984 -77.57499 America/New_York
38.70428 -77.22776 America/New_York
37.41375 -79.14225 America/New_York
38.75095 -77.47527 America/New_York
38.93428 -77.17748 America/New_York
37.60876 -77.37331 America/New_York
38.87428 -77.22693 America/New_York
38.61095 -77.33971 America/New_York
36.97876 -76.428 America/New_York
36.84681 -76.28522 America/New_York
38.88095 -77.30082 America/New_York
37.22793 -77.40193 America/New_York
36.83543 -76.29827 America/New_York
36.82098 -76.36883 America/New_York
37.13179 -80.57645 America/New_York
38.96872 -77.3411 America/New_York
37.55376 -77.46026 America/New_York
37.27097 -79.94143 America/New_York
38.78872 -77.11276 America/New_York
37.29347 -80.05476 America/New_York
37.65042 -77.61249 America/New_York
38.92094 -77.50388 America/New_York
36.71709 -76.59023 America/New_York
38.78928 -77.1872 America/New_York
38.14958 -79.0717 America/New_York
39.00622 -77.4286 America/New_York
38.79289 -77.49749 America/New_York
36.72821 -76.58356 America/New_York
37.59015 -77.55638 America/New_York
38.91872 -77.23109 America/New_York
38.90122 -77.26526 America/New_York
36.85293 -75.97798 America/New_York
38.06847 -78.88947 America/New_York
37.4032 -79.17808 America/New_York
38.77261 -77.22109 America/New_York
39.18566 -78.16333 America/New_York
38.93983 -77.28609 America/New_York
38.71678 -77.13276 America/New_York
37.77817 -81.18816 America/New_York
38.34982 -81.63262 America/New_York
39.28065 -80.34453 America/New_York
39.48508 -80.14258 America/New_York
38.41925 -82.44515 America/New_York
39.45621 -77.96389 America/New_York
39.62953 -79.9559 America/New_York
39.26674 -81.56151 America/New_York
34.81509 -92.22432 America/Chicago
39.48061 -86.05499 America/Indiana/Indianapolis
39.33312 -82.9824 America/New_York
28.98276 -95.9694 America/Chicago
33.24428 -86.81638 America/Chicago
34.26759 -86.20887 America/Chicago
33.65983 -85.83163 America/Chicago
34.80287 -86.97167 America/Chicago
32.60986 -85.48078 America/Chicago
41.91031 -88.07174 America/Chicago
42.6987 -71.13506 America/New_York
42.40843 -71.01199 America/New_York
41.39482 -73.45401 America/New_York
41.07871 -73.46929 America/New_York
42.2942 -83.30993 America/Detroit
41.52005 -81.55624 America/New_York
41.78232 -72.61203 America/New_York
41.27621 -72.86843 America/New_York
41.10565 -73.39845 America/New_York
41.14121 -73.26373 America/New_York
41.71982 -72.83204 America/New_York
41.71232 -72.60815 America/New_York
41.28899 -72.68176 America/New_York
41.39593 -72.89677 America/New_York
41.76371 -72.68509 America/New_York
41.83871 -71.86924 America/New_York
41.43982 -72.01424 America/New_York
41.27954 -72.59843 America/New_York
41.77593 -72.52148 America/New_York
41.76593 -72.23369 America/New_York
41.53815 -72.80704 America/New_York
41.56232 -72.65065 America/New_York
41.22232 -73.0565 America/New_York
41.47899 -72.15119 America/New_York
41.48593 -73.05066 America/New_York
41.66121 -72.77954 America/New_York
41.30815 -72.92816 America/New_York
41.35565 -72.09952 America/New_York
41.69788 -72.72371 America/New_York
41.39093 -72.85954 America/New_York
41.13815 -73.54346 America/New_York
41.1176 -73.4079 America/New_York
41.52426 -72.07591 America/New_York
41.67649 -71.91507 America/New_York
41.67454 -72.85816 America/New_York
41.39676 -73.07594 America/New_York
41.31649 -73.09316 America/New_York
41.82371 -72.6212 America/New_York
41.48148 -73.21317 America/New_York
41.05343 -73.53873 America/New_York
41.80843 -72.24952 America/New_York
41.18454 -73.13317 America/New_York
41.80065 -73.12122 America/New_York
41.24287 -73.20067 America/New_York
41.45704 -72.82316 America/New_York
41.55815 -73.0515 America/New_York
41.76204 -72.74204 America/New_York
41.27065 -72.94705 America/New_York
41.81843 -73.14372 America/New_York
41.14149 -73.3579 America/New_York
41.71427 -72.65259 America/New_York
41.71065 -72.20813 America/New_York
41.69982 -72.15702 America/New_York
41.60232 -72.98677 America/New_York
42.03471 -93.61994 America/Chicago
41.72971 -93.60577 America/Chicago
41.52448 -90.51569 America/Chicago
40.80754 -91.11292 America/Chicago
42.52776 -92.44547 America/Chicago
42.00833 -91.64407 America/Chicago
41.84447 -90.18874 America/Chicago
41.8167 -90.22263 America/Chicago
41.60304 -93.72411 America/Chicago
41.6764 -91.58045 America/Chicago
41.26194 -95.86083 America/Chicago
41.52364 -90.57764 America/Chicago
41.60054 -93.60911 America/Chicago
42.50056 -90.66457 America/Chicago
42.49747 -94.16802 America/Chicago
41.66113 -91.53017 America/Chicago
41.67304 -93.69772 America/Chicago
42.03417 -91.59768 America/Chicago
42.04943 -92.90798 America/Chicago
43.15357 -93.20104 America/Chicago
41.42447 -91.04321 America/Chicago
41.69971 -93.04798 America/Chicago
41.02001 -92.4113 America/Chicago
42.49999 -96.40031 America/Chicago
41.62666 -93.71217 America/Chicago
42.49276 -92.34296 America/Chicago
41.57199 -93.74531 America/Chicago
41.9317 -87.98896 America/Chicago
42.16558 -88.29425 America/Chicago
41.66892 -87.73866 America/Chicago
42.08836 -87.98063 America/Chicago
41.76058 -88.32007 America/Chicago
41.99503 -88.18563 America/Chicago
41.85003 -88.31257 America/Chicago
41.88142 -87.88312 America/Chicago
42.26391 -88.84427 America/Chicago
41.95503 -87.94007 America/Chicago
41.85059 -87.79367 America/Chicago
41.95753 -88.0809 America/Chicago
40.4842 -88.99369 America/Chicago
41.65726 -87.68005 America/Chicago
41.69864 -88.0684 America/Chicago
41.1417 -87.87504 America/Chicago
41.14198 -87.86115 America/Chicago
41.75003 -87.80422 America/Chicago
41.82392 -87.85173 America/Chicago
42.15141 -87.95979 America/Chicago
41.73392 -87.7795 America/Chicago
41.61559 -87.52949 America/Chicago
41.91253 -88.13479 America/Chicago
42.12114 -88.25786 America/Chicago
42.21197 -88.23814 America/Chicago
40.11642 -88.24338 America/Chicago
41.85003 -87.65005 America/Chicago
41.50615 -87.6356 America/Chicago
41.84559 -87.75394 America/Chicago
41.56809 -87.72033 America/Chicago
41.55475 -88.09867 America/Chicago
42.24113 -88.3162 America/Chicago
40.12448 -87.63002 America/Chicago
41.75198 -87.97395 America/Chicago
41.92947 -88.75036 America/Chicago
42.17114 -87.84451 America/Chicago
42.03336 -87.8834 America/Chicago
41.83892 -89.47955 America/Chicago
41.63892 -87.60727 America/Chicago
41.80892 -88.01117 America/Chicago
41.50087 -90.4443 America/Chicago
40.66615 -89.5801 America/Chicago
42.03725 -88.28119 America/Chicago
42.00392 -87.97035 America/Chicago
41.89947 -87.94034 America/Chicago
41.92114 -87.80923 America/Chicago
42.04114 -87.69006 America/Chicago
41.72059 -87.70172 America/Chicago
41.49587 -87.84866 America/Chicago
41.93531 -87.86562 America/Chicago
42.29669 -89.62123 America/Chicago
40.94782 -90.37124 America/Chicago
41.88753 -88.30535 America/Chicago
41.87753 -88.06701 America/Chicago
42.06975 -87.78784 America/Chicago
41.6292 -87.93089 America/Chicago
42.34447 -88.04175 America/Chicago
42.3703 -87.90202 America/Chicago
41.99947 -88.14507 America/Chicago
41.61003 -87.64671 America/Chicago
42.18169 -87.80034 America/Chicago
41.80086 -87.93701 America/Chicago
42.04281 -88.0798 America/Chicago
41.60003 -87.93811 America/Chicago
41.55726 -87.6656 America/Chicago
42.16808 -88.42814 America/Chicago
41.52503 -88.08173 America/Chicago
41.12003 -87.86115 America/Chicago
41.80503 -87.86923 America/Chicago
42.25863 -87.84062 America/Chicago
42.19697 -88.09341 America/Chicago
42.18169 -88.33036 America/Chicago
41.56476 -87.53893 America/Chicago
41.67364 -88.00173 America/Chicago
42.28308 -87.95313 America/Chicago
41.80114 -88.07479 America/Chicago
41.58948 -88.05784 America/Chicago
41.88003 -88.00784 America/Chicago
42.32002 -89.05816 America/Chicago
42.34724 -89.039 America/Chicago
40.45921 -90.6718 America/Chicago
41.50392 -87.7131 America/Chicago
41.8792 -87.84312 America/Chicago
42.33336 -88.26675 America/Chicago
41.90059 -87.85673 America/Chicago
41.52614 -87.88922 America/Chicago
41.5067 -90.51513 America/Chicago
41.73058 -88.3459 America/Chicago
40.61282 -89.45926 America/Chicago
42.04059 -87.78256 America/Chicago
42.06642 -87.93729 America/Chicago
42.26308 -88.00397 America/Chicago
41.78586 -88.14729 America/Chicago
41.51198 -87.96561 America/Chicago
42.01892 -87.80284 America/Chicago
40.5142 -88.99063 America/Chicago
41.80614 -88.3273 America/Chicago
42.32558 -87.84118 America/Chicago
40.71754 -89.58426 America/Chicago
42.12753 -87.82895 America/Chicago
41.60281 -87.74394 America/Chicago
41.71087 -87.75811 America/Chicago
41.88503 -87.7845 America/Chicago
41.63031 -87.85394 America/Chicago
41.68281 -88.35146 America/Chicago
41.34559 -88.84258 America/Chicago
42.1103 -88.03424 America/Chicago
41.6967 -87.817 America/Chicago
41.49142 -87.67449 America/Chicago
42.01114 -87.84062 America/Chicago
40.56754 -89.64066 America/Chicago
40.69365 -89.58899 America/Chicago
41.62697 -88.20395 America/Chicago
42.0953 -87.93757 America/Chicago
41.50948 -90.57875 America/Chicago
42.27113 -89.094 America/Chicago
42.08419 -88.01313 America/Chicago
41.64753 -88.08951 America/Chicago
41.98475 -88.07979 America/Chicago
42.35336 -88.09341 America/Chicago
42.37169 -88.09008 America/Chicago
41.91419 -88.30869 America/Chicago
42.03336 -88.08341 America/Chicago
41.52003 -88.20173 America/Chicago
42.03336 -87.73339 America/Chicago
41.99419 -88.2923 America/Chicago
41.60087 -87.60699 America/Chicago
41.78864 -89.69622 America/Chicago
42.02558 -88.17841 America/Chicago
41.98892 -88.68675 America/Chicago
41.57337 -87.78449 America/Chicago
40.11059 -88.20727 America/Chicago
42.21809 -87.96513 America/Chicago
41.88975 -87.98895 America/Chicago
40.70365 -89.40731 America/Chicago
42.36363 -87.84479 America/Chicago
41.88475 -88.20396 America/Chicago
41.85059 -87.882 America/Chicago
41.79586 -87.97562 America/Chicago
41.86614 -88.10701 America/Chicago
42.13919 -87.92896 America/Chicago
42.07225 -87.72284 America/Chicago
41.74697 -88.05034 America/Chicago
42.31474 -88.4487 America/Chicago
41.64114 -88.44729 America/Chicago
42.44613 -87.83285 America/Chicago
40.10532 -85.68025 America/Indiana/Indianapolis
40.04115 -86.87445 America/Indiana/Indianapolis
41.41698 -87.36531 America/Chicago
41.4942 -87.52171 America/Chicago
41.6392 -87.45476 America/Chicago
41.68199 -85.97667 America/Indiana/Indianapolis
41.1306 -85.12886 America/Indiana/Indianapolis
40.27948 -86.51084 America/Indiana/Indianapolis
41.59337 -87.34643 America/Chicago
41.58227 -85.83444 America/Indiana/Indianapolis
41.74813 -86.12569 America/Indiana/Indianapolis
41.52837 -87.42365 America/Chicago
41.58337 -87.50004 America/Chicago
41.55365 -87.45198 America/Chicago
41.53226 -87.25504 America/Chicago
40.8831 -85.49748 America/Indiana/Indianapolis
40.48643 -86.1336 America/Indiana/Indianapolis
41.6106 -86.72252 America/Chicago
40.4167 -86.87529 America/Indiana/Indianapolis
40.04837 -86.46917 America/Indiana/Indianapolis
40.75448 -86.35667 America/Indiana/Indianapolis
40.55837 -85.65914 America/Indiana/Indianapolis
41.48281 -87.33281 America/Chicago
41.70754 -86.89503 America/Chicago
41.66199 -86.15862 America/Indiana/Indianapolis
40.19338 -85.38636 America/Indiana/Indianapolis
41.56448 -87.51254 America/Chicago
40.04559 -86.0086 America/Indiana/Indianapolis
41.57587 -87.17615 America/Chicago
41.47892 -87.45476 America/Chicago
41.68338 -86.25001 America/Indiana/Indianapolis
41.47309 -87.06114 America/Chicago
40.42587 -86.90807 America/Indiana/Indianapolis
40.04282 -86.12749 America/Indiana/Indianapolis
42.10482 -70.94532 America/New_York
42.48509 -71.43284 America/New_York
42.06954 -72.61481 America/New_York
42.85842 -70.93005 America/New_York
42.37537 -72.51925 America/New_York
42.41537 -71.15644 America/New_York
42.26121 -71.4634 America/New_York
41.94454 -71.28561 America/New_York
42.19454 -71.83563 America/New_York
41.70011 -70.29947 America/New_York
42.39593 -71.17867 America/New_York
42.55843 -70.88005 America/New_York
42.55343 -70.85366 America/New_York
42.55843 -71.26895 America/New_York
42.35843 -71.05977 America/New_York
42.22232 -70.99949 America/New_York
42.08343 -71.01838 America/New_York
42.33176 -71.12116 America/New_York
42.50482 -71.19561 America/New_York
42.3751 -71.10561 America/New_York
42.15843 -71.14477 America/New_York
42.59981 -71.36728 America/New_York
42.39176 -71.03283 America/New_York
42.17256 -72.59491 America/New_York
42.46037 -71.34895 America/New_York
42.57509 -70.93005 America/New_York
42.24177 -71.16616 America/New_York
42.67037 -71.30201 America/New_York
42.06454 -72.51259 America/New_York
42.26676 -72.66898 America/New_York
42.02454 -71.12866 America/New_York
42.40843 -71.05366 America/New_York
41.6376 -70.90365 America/New_York
41.70149 -71.15505 America/New_York
42.58342 -71.8023 America/New_York
42.27926 -71.41617 America/New_York
42.29732 -71.43701 America/New_York
42.08343 -71.39672 America/New_York
42.57509 -71.99813 America/New_York
42.61593 -70.66199 America/New_York
42.20704 -71.68562 America/New_York
42.58759 -72.59953 America/New_York
42.11316 -70.81199 America/New_York
42.7762 -71.07728 America/New_York
42.35176 -71.86341 America/New_York
42.20426 -72.6162 America/New_York
42.30982 -71.12033 America/New_York
42.70704 -71.16311 America/New_York
42.52509 -71.75979 America/New_York
42.44732 -71.2245 America/New_York
42.0501 -72.58287 America/New_York
42.63342 -71.31617 America/New_York
42.16009 -72.47592 America/New_York
42.46676 -70.94949 America/New_York
42.4251 -71.06616 America/New_York
42.03343 -71.21894 America/New_York
42.5001 -70.85783 America/New_York
42.34909 -71.54546 America/New_York
42.41843 -71.10616 America/New_York
42.45843 -71.06616 America/New_York
42.7262 -71.19089 America/New_York
42.13982 -71.51617 America/New_York
42.24954 -71.06616 America/New_York
42.28343 -71.3495 America/New_York
42.28343 -71.23283 America/New_York
41.63622 -70.9342 America/New_York
42.81259 -70.87728 America/New_York
42.33704 -71.20922 America/New_York
42.18343 -72.59953 America/New_York
42.32509 -72.6412 America/New_York
41.96677 -71.18699 America/New_York
42.19454 -71.1995 America/New_York
42.15843 -72.32869 America/New_York
42.52787 -70.92866 America/New_York
42.45008 -73.24538 America/New_York
42.25288 -71.00227 America/New_York
42.1626 -71.04116 America/New_York
42.52565 -71.09533 America/New_York
42.13066 -70.91616 America/New_York
42.51954 -70.89672 America/New_York
42.46482 -71.01005 America/New_York
42.29593 -71.71285 America/New_York
41.76955 -71.12866 America/New_York
42.3876 -71.0995 America/New_York
42.33343 -71.04949 America/New_York
42.25842 -72.57453 America/New_York
42.50982 -70.94949 America/New_York
42.0751 -72.03341 America/New_York
42.10148 -72.58981 America/New_York
42.4801 -71.0995 America/New_York
42.1251 -71.10227 America/New_York
42.38343 -71.41617 America/New_York
41.74816 -71.18977 America/New_York
41.9001 -71.08977 America/New_York
42.61065 -71.23422 America/New_York
42.50648 -71.07283 America/New_York
42.37649 -71.23561 America/New_York
42.37093 -71.18283 America/New_York
42.29649 -71.29256 America/New_York
42.10704 -72.62037 America/New_York
42.12509 -72.74954 America/New_York
42.57926 -71.43784 America/New_York
42.22093 -70.93977 America/New_York
42.54648 -71.17367 America/New_York
42.45232 -71.137 America/New_York
42.3751 -70.98283 America/New_York
42.47926 -71.15228 America/New_York
42.26259 -71.80229 America/New_York
41.70567 -70.22863 America/New_York
44.09785 -70.23117 America/New_York
44.31062 -69.77949 America/New_York
44.80118 -68.77781 America/New_York
43.49258 -70.45338 America/New_York
43.91452 -69.96533 America/New_York
44.10035 -70.21478 America/New_York
43.66147 -70.25533 America/New_York
43.50092 -70.44283 America/New_York
43.64147 -70.24088 America/New_York
43.63897 -70.31533 America/New_York
44.55201 -69.63171 America/New_York
43.57036 -70.38783 America/New_York
43.67703 -70.37116 America/New_York
41.89755 -84.03717 America/Detroit
42.25754 -83.21104 America/Detroit
42.97225 -85.95365 America/Detroit
42.27756 -83.74088 America/Detroit
42.68753 -83.2341 America/Detroit
42.32115 -85.17971 America/Detroit
43.59447 -83.88886 America/Detroit
42.5467 -83.21132 America/Detroit
42.99947 -83.61634 America/Detroit
42.30865 -83.48216 America/Detroit
42.58698 -82.91992 America/Detroit
42.32226 -83.17631 America/Detroit
42.33698 -83.27326 America/Detroit
42.33143 -83.04575 America/Detroit
42.73698 -84.48387 America/Detroit
42.46837 -82.95547 America/Detroit
42.48531 -83.37716 America/Detroit
42.46059 -83.13465 America/Detroit
43.01253 -83.68746 America/Detroit
42.95947 -85.48975 America/Detroit
42.32559 -83.33104 America/Detroit
42.96336 -85.66809 America/Detroit
42.90975 -85.76309 America/Detroit
42.44365 -82.90686 America/Detroit
42.39282 -83.04964 America/Detroit
42.74698 -84.40108 America/Detroit
42.46254 -83.10409 America/Detroit
42.78752 -86.10893 America/Detroit
42.64059 -84.51525 America/Detroit
42.24587 -84.40135 America/Detroit
42.90725 -85.79198 America/Detroit
42.29171 -85.58723 America/Detroit
42.86947 -85.64475 America/Detroit
42.73254 -84.55553 America/Detroit
42.25059 -83.17854 America/Detroit
42.36837 -83.35271 America/Detroit
42.48587 -83.1052 America/Detroit
46.54354 -87.39542 America/Detroit
43.61558 -84.24721 America/Detroit
41.91643 -83.39771 America/Detroit
42.59726 -82.87798 America/Detroit
43.59781 -84.76751 America/Detroit
43.23418 -86.24839 America/Detroit
43.1689 -86.26395 America/Detroit
42.48059 -83.47549 America/Detroit
42.45948 -83.18271 America/Detroit
42.72226 -84.42747 America/Detroit
42.9978 -84.17664 America/Detroit
42.63892 -83.29105 America/Detroit
42.97086 -82.42491 America/Detroit
42.20115 -85.58 America/Detroit
42.38337 -83.2966 America/Detroit
42.65837 -83.14993 America/Detroit
42.22226 -83.3966 America/Detroit
42.49726 -82.93714 America/Detroit
42.48948 -83.14465 America/Detroit
43.41947 -83.95081 America/Detroit
42.67087 -83.03298 America/Detroit
42.47337 -83.22187 America/Detroit
42.21393 -83.19381 America/Detroit
42.49698 -82.88881 America/Detroit
42.58031 -83.0302 America/Detroit
42.24087 -83.26965 America/Detroit
42.13949 -83.17826 America/Detroit
42.60559 -83.14993 America/Detroit
43.00141 -85.76809 America/Detroit
42.47754 -83.0277 America/Detroit
42.70225 -83.40272 America/Detroit
42.7392 -84.62082 America/Detroit
42.28143 -83.38632 America/Detroit
42.3242 -83.40021 America/Detroit
42.21421 -83.14992 America/Detroit
42.91336 -85.70531 America/Detroit
42.24115 -83.61299 America/Detroit
43.64801 -93.36827 America/Chicago
45.2333 -93.29134 America/Chicago
45.19774 -93.38718 America/Chicago
44.73191 -93.21772 America/Chicago
43.66663 -92.97464 America/Chicago
45.1608 -93.23495 America/Chicago
44.8408 -93.29828 America/Chicago
45.07608 -93.33273 America/Chicago
45.09413 -93.35634 America/Chicago
45.17191 -93.87469 America/Chicago
44.76774 -93.27772 America/Chicago
45.18885 -93.39745 America/Chicago
44.86219 -93.53079 America/Chicago
44.78941 -93.60218 America/Chicago
45.0408 -93.263 America/Chicago
45.11997 -93.28773 America/Chicago
44.82774 -92.94382 America/Chicago
45.03274 -93.36023 America/Chicago
46.78327 -92.10658 America/Chicago
44.80413 -93.16689 America/Chicago
44.85469 -93.47079 America/Chicago
44.88969 -93.34995 America/Chicago
45.30385 -93.56718 America/Chicago
44.29496 -93.26883 America/Chicago
44.64024 -93.14355 America/Chicago
45.27886 -92.98522 America/Chicago
45.08608 -93.26328 America/Chicago
45.00969 -93.34912 America/Chicago
45.25024 -93.24995 America/Chicago
44.7433 -92.85243 America/Chicago
47.42715 -92.93769 America/Chicago
44.92496 -93.46273 America/Chicago
44.84802 -93.04272 America/Chicago
44.64969 -93.24272 America/Chicago
45.16024 -93.08883 America/Chicago
44.16358 -93.9994 America/Chicago
45.07246 -93.45579 America/Chicago
44.95302 -92.99522 America/Chicago
44.97997 -93.26384 America/Chicago
44.9133 -93.50329 America/Chicago
44.94107 -93.4419 America/Chicago
46.87385 -96.76758 America/Chicago
45.06552 -93.20189 America/Chicago
45.03802 -93.38662 America/Chicago
44.4583 -93.1616 America/Chicago
44.96302 -92.96494 America/Chicago
44.08385 -93.22604 America/Chicago
45.01052 -93.45551 America/Chicago
44.7133 -93.42273 America/Chicago
45.2611 -93.45 America/Chicago
44.56247 -92.5338 America/Chicago
44.8833 -93.283 America/Chicago
44.02163 -92.4699 America/Chicago
44.73941 -93.12577 America/Chicago
45.00608 -93.15661 America/Chicago
45.5608 -94.16249 America/Chicago
44.9483 -93.34801 America/Chicago
45.20996 -93.66496 America/Chicago
44.94441 -93.09327 America/Chicago
45.62163 -94.20694 America/Chicago
44.77913 -93.33634 America/Chicago
44.79802 -93.5269 America/Chicago
45.07913 -93.14717 America/Chicago
44.89274 -93.03494 America/Chicago
45.05636 -92.80604 America/Chicago
45.15969 -93.34967 America/Chicago
44.91608 -93.10161 America/Chicago
45.08469 -93.00994 America/Chicago
45.12191 -95.04334 America/Chicago
44.04996 -91.63932 America/Chicago
44.92386 -92.95938 America/Chicago
40.19475 -92.58325 America/Chicago
46.87719 -96.7898 America/Chicago
47.92526 -97.03285 America/Chicago
46.91054 -98.70844 America/Chicago
46.87497 -96.90036 America/Chicago
41.13667 -95.89084 America/Chicago
41.42973 -97.36838 America/Chicago
41.43333 -96.49808 America/Chicago
40.92501 -98.34201 America/Chicago
40.58612 -98.38839 America/Chicago
40.69946 -99.08148 America/Chicago
41.18389 -96.03113 America/Chicago
40.8 -96.66696 America/Chicago
42.02834 -97.417 America/Chicago
41.25861 -95.93779 America/Chicago
41.15444 -96.04224 America/Chicago
42.94647 -71.5159 America/New_York
43.20814 -71.53757 America/New_York
42.88064 -71.32729 America/New_York
42.89175 -71.31201 America/New_York
43.19786 -70.87367 America/New_York
43.24202 -71.53813 America/New_York
42.93369 -72.27814 America/New_York
43.52785 -71.47035 America/New_York
42.99564 -71.45479 America/New_York
42.86509 -71.4934 America/New_York
42.76537 -71.46757 America/New_York
43.07176 -70.76255 America/New_York
43.30453 -70.97562 America/New_York
42.78842 -71.20089 America/New_York
40.22039 -74.01208 America/New_York
40.58038 -74.28515 America/New_York
40.66871 -74.11431 America/New_York
40.79371 -74.15014 America/New_York
40.9276 -73.99736 America/New_York
40.80677 -74.18542 America/New_York
40.57733 -74.2282 America/New_York
40.82149 -73.98764 America/New_York
40.85843 -74.16376 America/New_York
40.57455 -74.30209 America/New_York
40.65844 -74.29959 America/New_York
40.88399 -74.5621 America/New_York
40.94065 -73.99681 America/New_York
40.42788 -74.41598 America/New_York
40.76732 -74.20487 America/New_York
40.51872 -74.4121 America/New_York
40.66399 -74.2107 America/New_York
40.90399 -74.11848 America/New_York
40.89288 -73.97264 America/New_York
40.26983 -74.79988 America/New_York
40.94038 -74.13181 America/New_York
40.52927 -74.31598 America/New_York
40.85093 -73.97014 America/New_York
40.88149 -74.1132 America/New_York
40.88593 -74.04347 America/New_York
40.94926 -74.15375 America/New_York
40.70121 -74.23015 America/New_York
40.74399 -74.03236 America/New_York
40.93288 -74.65933 America/New_York
40.94399 -74.67072 America/New_York
40.73232 -74.23487 America/New_York
40.57538 -74.32237 America/New_York
40.72816 -74.07764 America/New_York
40.76843 -74.14542 America/New_York
40.09789 -74.21764 America/New_York
40.62205 -74.24459 America/New_York
40.79593 -74.31487 America/New_York
40.88232 -74.0832 America/New_York
40.30428 -73.99236 America/New_York
40.81204 -74.12431 America/New_York
40.75982 -74.4171 America/New_York
40.73121 -74.27348 America/New_York
40.82593 -74.20903 America/New_York
40.79677 -74.48154 America/New_York
40.48622 -74.45182 America/New_York
40.93268 -74.01785 America/New_York
40.73566 -74.17237 America/New_York
40.78843 -74.1332 America/New_York
40.80427 -74.01208 America/New_York
40.6301 -74.42737 America/New_York
40.82232 -74.15987 America/New_York
40.41483 -74.36543 America/New_York
40.77066 -74.23265 America/New_York
40.84816 -73.99764 America/New_York
40.94454 -74.07542 America/New_York
40.85788 -74.42599 America/New_York
40.85677 -74.12848 America/New_York
40.91677 -74.17181 America/New_York
40.50677 -74.26542 America/New_York
40.61682 -74.41727 America/New_York
40.08317 -74.06819 America/New_York
40.60816 -74.27765 America/New_York
40.97926 -74.11653 America/New_York
40.66427 -74.2632 America/New_York
40.82649 -74.10681 America/New_York
40.45927 -74.36098 America/New_York
40.46538 -74.33043 America/New_York
40.65538 -74.38987 America/New_York
40.79672 -74.05532 America/New_York
40.4976 -74.48849 America/New_York
40.40816 -74.35432 America/New_York
40.74899 -74.26126 America/New_York
40.57927 -74.41154 America/New_York
40.4465 -74.38598 America/New_York
40.74149 -74.35959 America/New_York
40.8976 -74.01597 America/New_York
40.30428 -74.10042 America/New_York
40.21705 -74.74294 America/New_York
40.6976 -74.2632 America/New_York
40.77955 -74.02375 America/New_York
40.92538 -74.27654 America/New_York
41.13121 -74.36737 America/New_York
40.78788 -74.01431 America/New_York
40.79871 -74.23904 America/New_York
40.65899 -74.34737 America/New_York
40.02789 -74.86905 America/New_York
40.5576 -74.28459 America/New_York
41.00954 -74.17292 America/New_York
42.65258 -73.75623 America/New_York
42.93869 -74.18819 America/New_York
42.93173 -76.56605 America/New_York
40.65649 -73.6093 America/New_York
42.99812 -78.18752 America/New_York
40.7251 -73.24539 America/New_York
41.50482 -73.96958 America/New_York
40.66871 -73.52707 America/New_York
40.60177 -73.99403 America/New_York
40.74427 -73.48207 America/New_York
42.09869 -75.91797 America/New_York
40.78121 -73.24623 America/New_York
43.14756 -77.55055 America/New_York
40.6501 -73.94958 America/New_York
42.88645 -78.87837 America/New_York
40.85843 -73.09955 America/New_York
40.79065 -73.20178 America/New_York
42.90339 -78.75475 America/New_York
42.77424 -73.70012 America/New_York
40.84288 -73.29289 America/New_York
40.57788 -73.99403 America/New_York
40.68149 -73.39984 America/New_York
40.86871 -73.00149 America/New_York
42.60118 -76.18048 America/New_York
40.76177 -73.32929 America/New_York
42.90395 -78.69225 America/New_York
40.80482 -73.33623 America/New_York
40.71399 -73.55902 America/New_York
40.67343 -73.43651 America/New_York
40.66677 -73.88236 America/New_York
40.87676 -73.32456 America/New_York
40.76704 -72.99622 America/New_York
40.94149 -73.10594 America/New_York
40.8876 -73.83291 America/New_York
42.96339 -78.80392 America/New_York
42.0898 -76.80773 America/New_York
40.70094 -73.71291 America/New_York
40.83121 -73.02955 America/New_York
40.72371 -73.70485 America/New_York
40.70732 -73.67596 America/New_York
40.6576 -73.58318 America/New_York
40.72677 -73.6343 America/New_York
40.86232 -73.63374 America/New_York
43.05285 -74.34375 America/New_York
41.03288 -73.84291 America/New_York
40.96899 -73.71263 America/New_York
40.82565 -73.20261 America/New_York
40.70621 -73.61874 America/New_York
40.76843 -73.52513 America/New_York
40.81232 -73.07844 America/New_York
40.81538 -73.04511 America/New_York
40.86815 -73.42568 America/New_York
40.85343 -73.41151 America/New_York
43.2134 -77.57972 America/New_York
40.72982 -73.21039 America/New_York
42.44063 -76.49661 America/New_York
42.097 -79.23533 America/New_York
42.11563 -75.95881 America/New_York
42.96589 -78.87004 America/New_York
40.88621 -73.25734 America/New_York
41.92704 -73.99736 America/New_York
41.34204 -74.16792 America/New_York
42.82561 -78.82337 America/New_York
40.8351 -73.13122 America/New_York
40.72593 -73.51429 America/New_York
40.68677 -73.37345 America/New_York
43.17061 -78.69031 America/New_York
40.58844 -73.65791 America/New_York
40.65483 -73.6718 America/New_York
40.94871 -73.73263 America/New_York
40.78343 -73.96625 America/New_York
40.68066 -73.47429 America/New_York
40.68038 -73.45512 America/New_York
40.80204 -72.84094 America/New_York
40.8176 -73.00011 America/New_York
40.79343 -73.41512 America/New_York
40.66288 -73.55152 America/New_York
41.44593 -74.42293 America/New_York
40.74927 -73.64068 America/New_York
41.11121 -74.06848 America/New_York
40.9126 -73.83708 America/New_York
41.08871 -74.01347 America/New_York
41.1476 -73.98931 America/New_York
40.91149 -73.78235 America/New_York
40.71427 -74.00597 America/New_York
41.50343 -74.01042 America/New_York
43.0945 -79.05671 America/New_York
40.6976 -73.42512 America/New_York
40.71649 -73.32179 America/New_York
40.73621 -73.26262 America/New_York
40.69149 -73.53346 America/New_York
40.70093 -73.46207 America/New_York
43.03867 -78.8642 America/New_York
40.6851 -73.7018 America/New_York
40.63871 -73.64013 America/New_York
41.16287 -73.86152 America/New_York
43.45535 -76.5105 America/New_York
41.05899 -74.02181 America/New_York
41.29009 -73.92042 America/New_York
40.77649 -73.46735 America/New_York
44.69949 -73.45291 America/New_York
41.00176 -73.66568 America/New_York
40.82566 -73.69819 America/New_York
41.70037 -73.92097 America/New_York
43.15478 -77.61556 America/New_York
40.65871 -73.64124 America/New_York
43.21285 -75.45573 America/New_York
40.81538 -73.11233 America/New_York
40.67871 -73.58902 America/New_York
42.78702 -73.97096 America/New_York
40.98065 -73.68374 America/New_York
43.08313 -73.78457 America/New_York
40.73593 -73.08206 America/New_York
41.0051 -73.78458 America/New_York
42.81424 -73.93957 America/New_York
40.66593 -73.48818 America/New_York
40.86649 -73.03566 America/New_York
40.80149 -72.8676 America/New_York
40.85593 -73.20067 America/New_York
41.11315 -74.04375 America/New_York
40.82621 -73.50207 America/New_York
43.04812 -76.14742 America/New_York
43.02033 -78.88032 America/New_York
42.72841 -73.69179 America/New_York
40.70038 -73.59291 America/New_York
43.1009 -75.23266 America/New_York
40.66427 -73.70846 America/New_York
40.68371 -73.51013 America/New_York
43.97478 -75.91076 America/New_York
42.68313 -73.77845 America/New_York
40.71816 -73.35429 America/New_York
40.70482 -73.65013 America/New_York
40.70621 -73.30623 America/New_York
42.85006 -78.79975 America/New_York
40.75566 -73.58763 America/New_York
41.03399 -73.76291 America/New_York
40.63205 -73.71263 America/New_York
40.93121 -73.89875 America/New_York
41.08144 -81.51901 America/New_York
40.91534 -81.10593 America/New_York
40.86867 -82.31822 America/New_York
41.86505 -80.78981 America/New_York
41.31755 -81.34539 America/New_York
41.10172 -80.76452 America/New_York
41.45171 -82.03542 America/New_York
41.45976 -82.01959 America/New_York
41.50532 -82.0282 America/New_York
41.01283 -81.60512 America/New_York
41.48477 -81.92208 America/New_York
41.36616 -81.8543 America/New_York
41.02423 -80.66285 America/New_York
41.37477 -83.65132 America/New_York
41.31394 -81.68513 America/New_York
41.39838 -81.80458 America/New_York
41.23811 -81.8418 America/New_York
40.79895 -81.37845 America/New_York
41.4995 -81.69541 America/New_York
41.13394 -81.48456 America/New_York
41.28449 -84.35578 America/New_York
40.29867 -83.06796 America/New_York
40.09923 -83.11408 America/New_York
41.53311 -81.57901 America/New_York
41.65394 -81.45039 America/New_York
41.36838 -82.10765 America/New_York
41.5931 -81.52679 America/New_York
41.44144 -81.8643 America/New_York
41.04422 -83.64993 America/New_York
41.35033 -83.12186 America/New_York
40.01923 -82.87934 America/New_York
41.417 -81.60596 America/New_York
40.94589 -81.48317 America/New_York
40.0334 -83.15825 America/New_York
41.24006 -81.44067 America/New_York
41.15367 -81.35789 America/New_York
41.48199 -81.79819 America/New_York
40.74255 -84.10523 America/New_York
41.45282 -82.18237 America/New_York
40.75839 -82.51545 America/New_York
41.41533 -81.56596 America/New_York
40.58867 -83.12852 America/New_York
40.23645 -83.36714 America/New_York
40.79672 -81.52151 America/New_York
41.51922 -81.4579 America/New_York
41.13839 -81.86375 America/New_York
41.66616 -81.33955 America/New_York
41.36144 -81.81291 America/New_York
40.3934 -82.48572 America/New_York
40.48979 -81.44567 America/New_York
40.05812 -82.40126 America/New_York
41.18284 -80.76536 America/New_York
40.87589 -81.40234 America/New_York
41.4156 -81.92347 America/New_York
41.38949 -82.01903 America/New_York
41.31366 -81.72457 America/New_York
41.24255 -82.61573 America/New_York
41.64366 -83.48688 America/New_York
41.72449 -81.24566 America/New_York
41.40477 -81.72291 America/New_York
41.39005 -81.75958 America/New_York
41.557 -83.62716 America/New_York
40.14477 -84.24244 America/New_York
41.4756 -81.8393 America/New_York
41.44894 -82.70796 America/New_York
41.47394 -81.53707 America/New_York
40.28422 -84.1555 America/New_York
41.38978 -81.44123 America/New_York
41.52311 -81.51846 America/New_York
40.36979 -80.63396 America/New_York
41.1595 -81.44039 America/New_York
41.23922 -81.34594 America/New_York
41.3145 -81.83569 America/New_York
41.71894 -83.71299 America/New_York
41.10145 -81.44178 America/New_York
41.1145 -83.17797 America/New_York
41.66394 -83.55521 America/New_York
40.0395 -84.20328 America/New_York
41.31256 -81.44011 America/New_York
41.02561 -81.72985 America/New_York
41.23756 -80.81842 America/New_York
40.12617 -82.92907 America/New_York
41.45532 -81.91792 America/New_York
41.63977 -81.4065 America/New_York
40.80506 -81.93514 America/New_York
41.09978 -80.64952 America/New_York
40.60843 -75.49018 America/New_York
40.5584 -79.95811 America/New_York
40.55951 -79.95867 America/New_York
40.51868 -78.39474 America/New_York
41.33591 -75.99631 America/New_York
40.33813 -79.97894 America/New_York
40.32757 -80.0395 America/New_York
40.62593 -75.37046 America/New_York
40.20148 -77.18887 America/New_York
40.68843 -75.22073 America/New_York
42.12922 -80.08506 America/New_York
40.2737 -76.88442 America/New_York
40.95842 -75.97465 America/New_York
41.23339 -80.44868 America/New_York
40.32674 -78.92197 America/New_York
40.08927 -75.39602 America/New_York
40.03788 -76.30551 America/New_York
40.2415 -75.28379 America/New_York
40.34093 -76.41135 America/New_York
40.15511 -74.82877 America/New_York
40.23093 -75.52212 America/New_York
40.34785 -79.86422 America/New_York
40.42118 -79.7881 America/New_York
40.35535 -80.0495 America/New_York
40.10677 -75.95272 America/New_York
40.4284 -79.69754 America/New_York
41.00367 -80.34701 America/New_York
40.1215 -75.3399 America/New_York
40.50118 -79.83922 America/New_York
40.13038 -75.51491 America/New_York
40.44062 -79.99589 America/New_York
40.50035 -79.74949 America/New_York
40.24537 -75.64963 America/New_York
40.04622 -75.35991 America/New_York
40.33565 -75.92687 America/New_York
41.40897 -75.66241 America/New_York
40.7934 -77.86 America/New_York
40.3359 -80.08339 America/New_York
40.3634 -79.86644 America/New_York
40.66676 -75.49991 America/New_York
41.24591 -75.88131 America/New_York
40.44174 -79.88199 America/New_York
41.24119 -77.00108 America/New_York
40.144 -75.11573 America/New_York
41.74066 -71.30866 America/New_York
41.67705 -71.26616 America/New_York
41.89066 -71.39228 America/New_York
41.7001 -71.68284 America/New_York
41.77982 -71.43728 America/New_York
41.96677 -71.43284 America/New_York
41.81371 -71.37005 America/New_York
41.54566 -71.29144 America/New_York
41.4901 -71.31283 America/New_York
41.5501 -71.46617 America/New_York
41.8501 -71.46617 America/New_York
41.87871 -71.38256 America/New_York
41.60232 -71.25033 America/New_York
41.82399 -71.41283 America/New_York
41.92204 -71.54951 America/New_York
41.7001 -71.41617 America/New_York
41.69689 -71.52194 America/New_York
41.3776 -71.82729 America/New_York
42.00288 -71.51478 America/New_York
45.4647 -98.48648 America/Chicago
44.31136 -96.79839 America/Chicago
43.70943 -98.0298 America/Chicago
43.54997 -96.70033 America/Chicago
44.89941 -97.11507 America/Chicago
44.47588 -73.21207 America/New_York
44.54394 -73.14791 America/New_York
43.61062 -72.97261 America/New_York
44.46699 -73.17096 America/New_York
44.26193 -88.41538 America/Chicago
44.48221 -88.0701 America/Chicago
43.45777 -88.83733 America/Chicago
42.50835 -89.03178 America/Chicago
43.06057 -88.10648 America/Chicago
42.8078 -87.92425 America/Chicago
42.95974 -87.86147 America/Chicago
44.44888 -88.06038 America/Chicago
44.81135 -91.49849 America/Chicago
42.96083 -89.46984 America/Chicago
43.775 -88.43883 America/Chicago
42.88863 -88.03842 America/Chicago
43.22862 -88.11037 America/Chicago
44.51916 -88.01983 America/Chicago
42.9614 -88.01259 America/Chicago
44.5436 -88.08816 America/Chicago
42.68279 -89.01872 America/Chicago
44.27804 -88.27205 America/Chicago
42.58474 -87.82119 America/Chicago
43.80136 -91.23958 America/Chicago
43.07305 -89.40123 America/Chicago
44.08861 -87.65758 America/Chicago
44.66885 -90.1718 America/Chicago
44.20221 -88.4465 America/Chicago
43.1789 -88.11731 America/Chicago
44.87552 -91.91934 America/Chicago
43.2364 -87.98453 America/Chicago
43.09722 -89.50429 America/Chicago
43.0389 -87.90647 America/Chicago
42.90585 -88.13898 America/Chicago
44.18582 -88.46261 America/Chicago
42.9764 -88.10842 America/Chicago
43.84635 -91.24819 America/Chicago
42.88585 -87.86314 America/Chicago
43.11167 -88.49927 America/Chicago
43.88441 -91.23514 America/Chicago
44.02471 -88.54261 America/Chicago
42.55308 -87.93341 America/Chicago
42.72613 -87.78285 America/Chicago
43.75083 -87.71453 America/Chicago
42.91057 -87.86064 America/Chicago
44.52358 -89.57456 America/Chicago
43.1836 -89.21373 America/Chicago
46.72077 -92.10408 America/Chicago
43.19472 -88.72899 America/Chicago
43.01168 -88.23148 America/Chicago
44.95914 -89.63012 America/Chicago
43.04946 -88.00759 America/Chicago
43.01668 -88.00703 America/Chicago
43.42528 -88.18343 America/Chicago
44.38358 -89.81735 America/Chicago
40.41896 -80.58952 America/New_York
40.4084 -80.53924 America/New_York
40.06396 -80.72091 America/New_York
41.34621 -73.079 America/New_York
41.16704 -73.20483 America/New_York
41.67176 -72.94927 America/New_York
41.27954 -72.8151 America/New_York
41.49899 -72.90066 America/New_York
34.39916 -118.91815 America/Los_Angeles
33.37032 -112.58378 America/Phoenix
35.14778 -114.5683 America/Phoenix
32.8795 -111.75735 America/Phoenix
32.32341 -110.9951 America/Phoenix
32.29785 -110.9187 America/Phoenix
33.30616 -111.84125 America/Phoenix
31.34455 -109.54534 America/Phoenix
32.14119 -111.02843 America/Phoenix
33.61309 -112.3246 America/Phoenix
32.7559 -111.55484 America/Phoenix
35.19807 -111.65127 America/Phoenix
33.03145 -111.38734 America/Phoenix
32.29396 -111.00982 America/Phoenix
32.65783 -114.41189 America/Phoenix
33.61171 -111.71736 America/Phoenix
33.35283 -111.78903 America/Phoenix
33.53865 -112.18599 America/Phoenix
33.43532 -112.35821 America/Phoenix
31.85425 -110.9937 America/Phoenix
35.18944 -114.05301 America/Phoenix
34.4839 -114.32245 America/Phoenix
32.43674 -111.22538 America/Phoenix
33.05811 -112.04764 America/Phoenix
33.42227 -111.82264 America/Phoenix
31.34038 -110.93425 America/Phoenix
32.39091 -110.96649 America/Phoenix
34.23087 -111.32514 America/Phoenix
33.5806 -112.23738 America/Phoenix
33.44838 -112.07404 America/Phoenix
34.54002 -112.4685 America/Phoenix
34.58941 -112.32525 America/Phoenix
33.24866 -111.6343 America/Phoenix
31.47148 -110.97648 America/Phoenix
31.95758 -110.95565 America/Phoenix
32.487 -114.78218 America/Phoenix
33.50921 -111.89903 America/Phoenix
31.55454 -110.30369 America/Phoenix
33.59754 -112.27182 America/Phoenix
33.66198 -112.34127 America/Phoenix
33.63059 -112.33322 America/Phoenix
32.25174 -110.73731 America/Phoenix
33.41477 -111.90931 America/Phoenix
33.41421 -111.94348 America/Phoenix
32.22174 -110.92648 America/Phoenix
32.72532 -114.6244 America/Phoenix
34.58277 -117.40922 America/Los_Angeles
34.14306 -118.73787 America/Los_Angeles
34.13639 -118.77453 America/Los_Angeles
37.76521 -122.24164 America/Los_Angeles
37.88687 -122.29775 America/Los_Angeles
34.09529 -118.12701 America/Los_Angeles
33.56504 -117.72712 America/Los_Angeles
34.18973 -118.13118 America/Los_Angeles
37.36605 -121.82718 America/Los_Angeles
38.17492 -122.2608 America/Los_Angeles
33.83529 -117.9145 America/Los_Angeles
38.70824 -121.32995 America/Los_Angeles
38.00492 -121.80579 America/Los_Angeles
34.50083 -117.18588 America/Los_Angeles
34.13973 -118.03534 America/Los_Angeles
35.11859 -120.59073 America/Los_Angeles
33.86585 -118.08312 America/Los_Angeles
35.20913 -118.82843 America/Los_Angeles
37.69465 -122.11385 America/Los_Angeles
35.48942 -120.67073 America/Los_Angeles
37.34772 -120.60908 America/Los_Angeles
36.00412 -120.12903 America/Los_Angeles
34.03612 -117.99118 America/Los_Angeles
34.13362 -117.90756 America/Los_Angeles
35.37329 -119.01871 America/Los_Angeles
34.08529 -117.9609 America/Los_Angeles
33.92557 -116.87641 America/Los_Angeles
34.89859 -117.02282 America/Los_Angeles
34.86971 -117.05615 America/Los_Angeles
38.02909 -121.96163 America/Los_Angeles
38.02676 -121.94194 America/Los_Angeles
33.92946 -116.97725 America/Los_Angeles
33.97751 -118.18702 America/Los_Angeles
33.96529 -118.15146 America/Los_Angeles
33.88168 -118.11701 America/Los_Angeles
37.52021 -122.2758 America/Los_Angeles
38.04936 -122.15858 America/Los_Angeles
37.87159 -122.27275 America/Los_Angeles
34.07362 -118.40036 America/Los_Angeles
34.07029 -117.39588 America/Los_Angeles
33.6103 -114.59635 America/Los_Angeles
32.80755 -116.93642 America/Los_Angeles
32.97866 -115.53027 America/Los_Angeles
33.91668 -117.90006 America/Los_Angeles
37.92687 -121.72577 America/Los_Angeles
33.86751 -117.99812 America/Los_Angeles
34.18084 -118.30897 America/Los_Angeles
37.5841 -122.36608 America/Los_Angeles
34.15778 -118.63842 America/Los_Angeles
32.67895 -115.49888 America/Los_Angeles
34.21639 -119.0376 America/Los_Angeles
38.66879 -120.98716 America/Los_Angeles
37.28717 -121.94996 America/Los_Angeles
33.15809 -117.35059 America/Los_Angeles
38.61713 -121.32828 America/Los_Angeles
33.83141 -118.28202 America/Los_Angeles
34.48888 -118.62287 America/Los_Angeles
37.6941 -122.08635 America/Los_Angeles
33.77974 -116.46529 America/Los_Angeles
37.59493 -120.95771 America/Los_Angeles
33.85835 -118.06479 America/Los_Angeles
39.72849 -121.83748 America/Los_Angeles
34.01223 -117.68894 America/Los_Angeles
33.9938 -117.75888 America/Los_Angeles
37.123 -120.26018 America/Los_Angeles
32.64005 -117.0842 America/Los_Angeles
38.70712 -121.28106 America/Los_Angeles
34.09668 -117.71978 America/Los_Angeles
38.95823 -122.62637 America/Los_Angeles
36.82523 -119.70292 America/Los_Angeles
33.6803 -116.17389 America/Los_Angeles
36.86634 -119.62069 America/Los_Angeles
34.0739 -117.31365 America/Los_Angeles
33.89585 -118.22007 America/Los_Angeles
37.97798 -122.03107 America/Los_Angeles
36.09801 -119.5604 America/Los_Angeles
33.87529 -117.56644 America/Los_Angeles
32.68589 -117.18309 America/Los_Angeles
33.64113 -117.91867 America/Los_Angeles
34.09001 -117.89034 America/Los_Angeles
33.96057 -118.18535 America/Los_Angeles
34.02112 -118.39647 America/Los_Angeles
37.323 -122.03218 America/Los_Angeles
33.81696 -118.03729 America/Los_Angeles
37.70577 -122.46192 America/Los_Angeles
33.46697 -117.69811 America/Los_Angeles
37.82159 -121.99996 America/Los_Angeles
38.54491 -121.74052 America/Los_Angeles
35.76884 -119.24705 America/Los_Angeles
33.96112 -116.50168 America/Los_Angeles
34.02862 -117.81034 America/Los_Angeles
36.54328 -119.38707 America/Los_Angeles
38.44546 -121.8233 America/Los_Angeles
33.94001 -118.13257 America/Los_Angeles
34.13945 -117.97729 America/Los_Angeles
37.70215 -121.93579 America/Los_Angeles
33.89807 -118.19535 America/Los_Angeles
33.74002 -116.93891 America/Los_Angeles
34.0239 -118.17202 America/Los_Angeles
37.46883 -122.14108 America/Los_Angeles
32.79477 -116.96253 America/Los_Angeles
32.792 -115.56305 America/Los_Angeles
37.91576 -122.31164 America/Los_Angeles
38.68574 -121.08217 America/Los_Angeles
34.06862 -118.02757 America/Los_Angeles
33.91918 -118.41647 America/Los_Angeles
38.4088 -121.37162 America/Los_Angeles
33.03699 -117.29198 America/Los_Angeles
33.11921 -117.08642 America/Los_Angeles
38.64463 -121.27217 America/Los_Angeles
38.24936 -122.03997 America/Los_Angeles
33.37642 -117.25115 America/Los_Angeles
38.49602 -121.40884 America/Los_Angeles
38.67796 -121.17606 America/Los_Angeles
34.09223 -117.43505 America/Los_Angeles
38.67877 -121.35114 America/Los_Angeles
37.55855 -122.27108 America/Los_Angeles
33.70918 -117.95367 America/Los_Angeles
37.54827 -121.98857 America/Los_Angeles
36.74773 -119.77237 America/Los_Angeles
33.87029 -117.92534 America/Los_Angeles
38.25464 -121.29995 America/Los_Angeles
33.77391 -117.94145 America/Los_Angeles
33.88835 -118.30896 America/Los_Angeles
37.00578 -121.56828 America/Los_Angeles
34.01168 -117.48477 America/Los_Angeles
34.14251 -118.25508 America/Los_Angeles
34.13612 -117.86534 America/Los_Angeles
34.43583 -119.82764 America/Los_Angeles
38.76323 -121.16384 America/Los_Angeles
36.3208 -121.24381 America/Los_Angeles
33.99307 -117.96868 America/Los_Angeles
36.32745 -119.64568 America/Los_Angeles
33.9164 -118.35257 America/Los_Angeles
37.66882 -122.0808 America/Los_Angeles
33.74752 -116.97197 America/Los_Angeles
38.01714 -122.28858 America/Los_Angeles
33.86224 -118.39952 America/Los_Angeles
34.42639 -117.30088 America/Los_Angeles
34.12834 -117.20865 America/Los_Angeles
36.85245 -121.4016 America/Los_Angeles
34.09834 -118.32674 America/Los_Angeles
33.6603 -117.99923 America/Los_Angeles
33.98168 -118.22507 America/Los_Angeles
32.58394 -117.11308 America/Los_Angeles
33.72058 -116.21556 America/Los_Angeles
33.96168 -118.35313 America/Los_Angeles
33.66946 -117.82311 America/Los_Angeles
34.41333 -119.86097 America/Los_Angeles
34.19917 -118.18785 America/Los_Angeles
33.93196 -117.94617 America/Los_Angeles
32.84727 -117.2742 America/Los_Angeles
32.76783 -117.02308 America/Los_Angeles
33.91724 -118.01201 America/Los_Angeles
33.8464 -118.04673 America/Los_Angeles
32.70811 -116.99725 America/Los_Angeles
34.02001 -117.94951 America/Los_Angeles
33.66336 -116.31001 America/Los_Angeles
34.10084 -117.76784 America/Los_Angeles
33.57086 -117.63561 America/Los_Angeles
37.88576 -122.11802 America/Los_Angeles
38.42102 -121.42384 America/Los_Angeles
33.54225 -117.78311 America/Los_Angeles
33.5956 -117.70031 America/Los_Angeles
33.52253 -117.70755 America/Los_Angeles
33.6103 -117.72533 America/Los_Angeles
33.66808 -117.32726 America/Los_Angeles
33.64697 -117.68922 America/Los_Angeles
32.85727 -116.92225 America/Los_Angeles
33.85363 -118.13396 America/Los_Angeles
35.25968 -118.91427 America/Los_Angeles
34.69804 -118.13674 America/Los_Angeles
37.8227 -121.27661 America/Los_Angeles
33.88724 -118.35257 America/Los_Angeles
32.74255 -117.03142 America/Los_Angeles
36.30078 -119.78291 America/Los_Angeles
33.93807 -118.35258 America/Los_Angeles
38.89156 -121.29301 America/Los_Angeles
39.12767 -121.5508 America/Los_Angeles
36.98356 -121.98052 America/Los_Angeles
37.68187 -121.76801 America/Los_Angeles
38.1302 -121.27245 America/Los_Angeles
34.04835 -117.26115 America/Los_Angeles
33.79224 -118.31507 America/Los_Angeles
34.63915 -120.45794 America/Los_Angeles
33.76696 -118.18923 America/Los_Angeles
37.38522 -122.11413 America/Los_Angeles
34.05223 -118.24368 America/Los_Angeles
37.05828 -120.84992 America/Los_Angeles
37.22661 -121.97468 America/Los_Angeles
33.93029 -118.21146 America/Los_Angeles
36.96134 -120.06072 America/Los_Angeles
33.88474 -118.41091 America/Los_Angeles
37.79743 -121.21605 America/Los_Angeles
36.6844 -121.80217 America/Los_Angeles
38.01937 -122.13413 America/Los_Angeles
33.98668 -118.18535 America/Los_Angeles
33.83335 -117.29615 America/Los_Angeles
33.72835 -117.14642 America/Los_Angeles
37.45383 -122.18219 America/Los_Angeles
37.30216 -120.48297 America/Los_Angeles
37.59855 -122.38719 America/Los_Angeles
37.42827 -121.90662 America/Los_Angeles
33.99251 -117.51644 America/Los_Angeles
33.60002 -117.672 America/Los_Angeles
37.6391 -120.99688 America/Los_Angeles
34.14806 -117.99895 America/Los_Angeles
34.07751 -117.68978 America/Los_Angeles
34.00946 -118.10535 America/Los_Angeles
36.60024 -121.89468 America/Los_Angeles
34.06251 -118.12285 America/Los_Angeles
34.28556 -118.88204 America/Los_Angeles
37.83493 -122.12969 America/Los_Angeles
33.93752 -117.23059 America/Los_Angeles
37.1305 -121.65439 America/Los_Angeles
37.38605 -122.08385 America/Los_Angeles
33.55391 -117.21392 America/Los_Angeles
38.29714 -122.28553 America/Los_Angeles
32.67811 -117.0992 America/Los_Angeles
37.52966 -122.04024 America/Los_Angeles
33.61891 -117.92895 America/Los_Angeles
35.04275 -120.476 America/Los_Angeles
33.93113 -117.54866 America/Los_Angeles
34.16056 -118.26452 America/Los_Angeles
38.68574 -121.37217 America/Los_Angeles
33.90224 -118.08173 America/Los_Angeles
38.10742 -122.5697 America/Los_Angeles
37.76659 -120.84715 America/Los_Angeles
37.80437 -122.2708 America/Los_Angeles
37.99742 -121.71245 America/Los_Angeles
33.19587 -117.37948 America/Los_Angeles
35.41968 -119.01955 America/Los_Angeles
34.06334 -117.65089 America/Los_Angeles
33.78779 -117.85311 America/Los_Angeles
38.67851 -121.22578 America/Los_Angeles
34.86526 -120.436 America/Los_Angeles
37.87715 -122.17969 America/Los_Angeles
39.51378 -121.55636 America/Los_Angeles
34.1975 -119.17705 America/Los_Angeles
34.19084 -119.2415 America/Los_Angeles
37.61383 -122.48692 America/Los_Angeles
36.61774 -121.91662 America/Los_Angeles
33.72224 -116.37446 America/Los_Angeles
33.8303 -116.54529 America/Los_Angeles
34.57943 -118.11646 America/Los_Angeles
37.44188 -122.14302 America/Los_Angeles
39.75961 -121.62192 America/Los_Angeles
33.88946 -118.15979 America/Los_Angeles
34.14778 -118.14452 America/Los_Angeles
35.62664 -120.691 America/Los_Angeles
37.4716 -121.12966 America/Los_Angeles
33.78252 -117.22865 America/Los_Angeles
38.23242 -122.63665 America/Los_Angeles
33.98307 -118.09674 America/Los_Angeles
38.00437 -122.29886 America/Los_Angeles
38.02798 -121.88468 America/Los_Angeles
33.87224 -117.87034 America/Los_Angeles
37.94798 -122.0608 America/Los_Angeles
37.66243 -121.87468 America/Los_Angeles
34.05529 -117.75228 America/Los_Angeles
34.14778 -119.19511 America/Los_Angeles
36.06523 -119.01677 America/Los_Angeles
32.96282 -117.03586 America/Los_Angeles
36.77579 -121.66967 America/Los_Angeles
33.04171 -116.86808 America/Los_Angeles
38.58907 -121.30273 America/Los_Angeles
34.1064 -117.59311 America/Los_Angeles
33.73974 -116.41279 America/Los_Angeles
33.74446 -118.38702 America/Los_Angeles
32.74727 -116.9353 America/Los_Angeles
33.64086 -117.6031 America/Los_Angeles
34.05557 -117.18254 America/Los_Angeles
33.84918 -118.38841 America/Los_Angeles
37.48522 -122.23635 America/Los_Angeles
36.59634 -119.4504 America/Los_Angeles
34.1064 -117.37032 America/Los_Angeles
37.93576 -122.34775 America/Los_Angeles
35.62246 -117.6709 America/Los_Angeles
38.69101 -121.44857 America/Los_Angeles
37.73604 -120.93549 America/Los_Angeles
33.95335 -117.39616 America/Los_Angeles
38.79073 -121.23578 America/Los_Angeles
38.33964 -122.7011 America/Los_Angeles
34.86414 -118.16341 America/Los_Angeles
34.08057 -118.07285 America/Los_Angeles
38.55185 -121.36467 America/Los_Angeles
38.75212 -121.28801 America/Los_Angeles
33.97612 -117.90534 America/Los_Angeles
33.99613 -117.4056 America/Los_Angeles
38.58157 -121.4944 America/Los_Angeles
36.67774 -121.6555 America/Los_Angeles
34.10834 -117.28977 America/Los_Angeles
37.63049 -122.41108 America/Los_Angeles
37.50716 -122.26052 America/Los_Angeles
33.42697 -117.61199 America/Los_Angeles
32.71533 -117.15726 America/Los_Angeles
34.10668 -117.80673 America/Los_Angeles
34.28195 -118.43897 America/Los_Angeles
37.77493 -122.41942 America/Los_Angeles
34.09611 -118.10583 America/Los_Angeles
33.78391 -116.95864 America/Los_Angeles
37.33939 -121.89496 America/Los_Angeles
33.50169 -117.66255 America/Los_Angeles
37.72493 -122.15608 America/Los_Angeles
37.68104 -122.12441 America/Los_Angeles
35.28275 -120.65962 America/Los_Angeles
33.14337 -117.16614 America/Los_Angeles
37.56299 -122.32553 America/Los_Angeles
37.96215 -122.34553 America/Los_Angeles
33.73585 -118.29229 America/Los_Angeles
37.97353 -122.53109 America/Los_Angeles
37.77993 -121.97802 America/Los_Angeles
36.70801 -119.55597 America/Los_Angeles
33.74557 -117.86783 America/Los_Angeles
34.42083 -119.69819 America/Los_Angeles
37.35411 -121.95524 America/Los_Angeles
34.39166 -118.54259 America/Los_Angeles
36.97412 -122.0308 America/Los_Angeles
33.94724 -118.08535 America/Los_Angeles
34.95303 -120.43572 America/Los_Angeles
34.01945 -118.49119 America/Los_Angeles
34.35417 -119.05927 America/Los_Angeles
38.44047 -122.71443 America/Los_Angeles
32.83838 -116.97392 America/Los_Angeles
37.26383 -122.02301 America/Los_Angeles
33.74141 -118.10479 America/Los_Angeles
36.61107 -121.85162 America/Los_Angeles
36.57078 -119.61208 America/Los_Angeles
35.50051 -119.27178 America/Los_Angeles
34.26945 -118.78148 America/Los_Angeles
36.42469 -121.32632 America/Los_Angeles
34.05195 -118.04673 America/Los_Angeles
33.95474 -118.21202 America/Los_Angeles
38.93324 -119.98435 America/Los_Angeles
34.11612 -118.15035 America/Los_Angeles
37.65466 -122.40775 America/Los_Angeles
34.01279 -117.90478 America/Los_Angeles
33.96085 -118.04173 America/Los_Angeles
39.11656 -121.63913 America/Los_Angeles
32.74477 -116.99892 America/Los_Angeles
33.80252 -117.99312 America/Los_Angeles
37.9577 -121.29078 America/Los_Angeles
38.23825 -122.04024 America/Los_Angeles
33.70919 -117.19726 America/Los_Angeles
37.36883 -122.03635 America/Los_Angeles
33.49364 -117.14836 America/Los_Angeles
34.10723 -118.05785 America/Los_Angeles
34.17056 -118.83759 America/Los_Angeles
33.83585 -118.34063 America/Los_Angeles
37.73965 -121.42522 America/Los_Angeles
39.32796 -120.18325 America/Los_Angeles
36.20773 -119.34734 America/Los_Angeles
37.49466 -120.84659 America/Los_Angeles
33.74585 -117.82617 America/Los_Angeles
33.76446 -117.79394 America/Los_Angeles
34.13556 -116.05417 America/Los_Angeles
39.15017 -123.20778 America/Los_Angeles
37.59583 -122.01917 America/Los_Angeles
37.59577 -122.01913 America/Los_Angeles
34.1389 -118.35341 America/Los_Angeles
34.09751 -117.64839 America/Los_Angeles
38.35658 -121.98774 America/Los_Angeles
34.04529 -117.94367 America/Los_Angeles
38.10409 -122.25664 America/Los_Angeles
34.27834 -119.29317 America/Los_Angeles
34.53611 -117.29116 America/Los_Angeles
34.50055 -118.11646 America/Los_Angeles
36.33023 -119.29206 America/Los_Angeles
33.20004 -117.24254 America/Los_Angeles
34.02029 -117.86534 America/Los_Angeles
37.90631 -122.06496 America/Los_Angeles
33.96807 -118.22507 America/Los_Angeles
36.91023 -121.75689 America/Los_Angeles
33.82168 -118.29257 America/Los_Angeles
34.06862 -117.93895 America/Los_Angeles
34.09001 -118.36174 America/Los_Angeles
34.05168 -117.9684 America/Los_Angeles
38.58046 -121.53023 America/Los_Angeles
33.75918 -118.00673 America/Los_Angeles
33.9414 -118.3023 America/Los_Angeles
33.97918 -118.03284 America/Los_Angeles
33.59891 -117.28004 America/Los_Angeles
33.91696 -118.25507 America/Los_Angeles
38.54713 -122.81638 America/Los_Angeles
32.83116 -116.93336 America/Los_Angeles
38.67852 -121.7733 America/Los_Angeles
33.88863 -117.81311 America/Los_Angeles
39.14045 -121.61691 America/Los_Angeles
34.03362 -117.04309 America/Los_Angeles
34.11417 -116.43224 America/Los_Angeles
39.80276 -105.08748 America/Denver
39.72943 -104.83192 America/Denver
39.98526 -104.82053 America/Denver
39.92054 -105.08665 America/Denver
38.44098 -105.24245 America/Denver
39.37221 -104.85609 America/Denver
39.58471 -104.90109 America/Denver
39.57916 -104.87692 America/Denver
38.85861 -104.69886 America/Denver
39.09193 -108.44898 America/Denver
38.83388 -104.82136 America/Denver
39.58777 -105.06943 America/Denver
39.80832 -104.93387 America/Denver
39.73915 -104.9847 America/Denver
37.27528 -107.88007 America/Denver
39.64777 -104.98776 America/Denver
38.68222 -104.70081 America/Denver
39.75554 -105.2211 America/Denver
39.06387 -108.55065 America/Denver
39.55388 -104.96943 America/Denver
39.57726 -105.11687 America/Denver
39.9936 -105.08971 America/Denver
39.70471 -105.08137 America/Denver
39.61332 -105.01665 America/Denver
39.97776 -105.13193 America/Denver
38.47832 -107.87617 America/Denver
39.88554 -104.9872 America/Denver
39.5186 -104.76136 America/Denver
38.25445 -104.60914 America/Denver
38.35 -104.72275 America/Denver
39.83776 -105.00137 America/Denver
39.58721 -104.95276 America/Denver
39.86804 -104.97192 America/Denver
39.83665 -105.0372 America/Denver
39.7661 -105.07721 America/Denver
37.7528 -100.01708 America/Chicago
37.97169 -100.87266 America/Chicago
37.04308 -100.921 America/Chicago
32.89953 -105.96026 America/Denver
35.08449 -106.65114 America/Denver
32.42067 -104.22884 America/Denver
34.4048 -103.20523 America/Denver
36.72806 -108.21869 America/Denver
35.52808 -108.74258 America/Denver
32.70261 -103.13604 America/Denver
32.31232 -106.77834 America/Denver
35.23338 -106.66447 America/Denver
33.39427 -104.52302 America/Denver
35.68698 -105.9378 America/Denver
35.01005 -106.67808 America/Denver
35.97859 -114.83249 America/Los_Angeles
39.1638 -119.7674 America/Los_Angeles
36.02525 -115.24194 America/Los_Angeles
39.60797 -119.25183 America/Los_Angeles
36.0397 -114.98194 America/Los_Angeles
36.17497 -115.13722 America/Los_Angeles
36.80553 -114.06719 America/Los_Angeles
36.19886 -115.1175 America/Los_Angeles
36.20829 -115.98391 America/Los_Angeles
36.09719 -115.14666 America/Los_Angeles
39.52963 -119.8138 America/Los_Angeles
39.64908 -119.70741 America/Los_Angeles
39.53491 -119.75269 America/Los_Angeles
36.10803 -115.245 America/Los_Angeles
39.5963 -119.77602 America/Los_Angeles
36.21108 -115.07306 America/Los_Angeles
36.07081 -115.08416 America/Los_Angeles
36.12997 -115.11889 America/Los_Angeles
35.222 -101.8313 America/Chicago
32.2504 -101.47874 America/Chicago
29.36273 -100.89676 America/Chicago
28.70914 -100.49952 America/Chicago
31.75872 -106.48693 America/Denver
34.81506 -102.3977 America/Chicago
31.69261 -106.20748 America/Denver
33.57786 -101.85517 America/Chicago
31.99735 -102.07791 America/Chicago
31.84568 -102.36764 America/Chicago
35.53616 -100.95987 America/Chicago
34.18479 -101.70684 America/Chicago
31.46377 -100.43704 America/Chicago
31.65456 -106.30331 America/Denver
31.63622 -106.29054 America/Denver
31.84235 -102.49876 America/Chicago
37.67748 -113.06189 America/Denver
37.10415 -113.58412 America/Denver
37.13054 -113.50829 America/Denver
35.59412 -119.34095 America/Los_Angeles
33.86726 -112.14682 America/Phoenix
33.41505 -111.54958 America/Phoenix
33.4356 -112.3496 America/Phoenix
58.30194 -134.41972 America/Juneau
40.86652 -124.08284 America/Los_Angeles
40.84235 -124.06367 America/Los_Angeles
40.80207 -124.16367 America/Los_Angeles
40.94652 -124.10062 America/Los_Angeles
40.58654 -122.39168 America/Los_Angeles
40.41628 -120.65301 America/Los_Angeles
40.01499 -105.27055 America/Denver
40.05026 -105.04998 America/Denver
40.37637 -104.69219 America/Denver
40.58526 -105.08442 America/Denver
40.42331 -104.70913 America/Denver
40.16721 -105.10193 America/Denver
40.39776 -105.07498 America/Denver
40.47748 -104.90136 America/Denver
43.6135 -116.20345 America/Boise
43.66294 -116.68736 America/Boise
47.67768 -116.78047 America/Los_Angeles
43.69544 -116.35401 America/Boise
43.46658 -112.03414 America/Boise
43.49183 -116.42012 America/Boise
46.41655 -117.01766 America/Los_Angeles
46.38044 -116.97543 America/Los_Angeles
43.61211 -116.39151 America/Boise
46.73239 -117.00017 America/Los_Angeles
43.54072 -116.56346 America/Boise
42.8713 -112.44553 America/Boise
47.71796 -116.95159 America/Los_Angeles
43.82602 -111.78969 America/Boise
42.56297 -114.46087 America/Boise
45.78329 -108.50069 America/Denver
45.67965 -111.03856 America/Denver
46.00382 -112.53474 America/Denver
47.50024 -111.30081 America/Denver
46.59271 -112.03611 America/Denver
48.19579 -114.31291 America/Denver
46.87215 -113.994 America/Denver
46.80833 -100.78374 America/Chicago
46.87918 -102.78962 America/Denver
46.82666 -100.88958 America/North_Dakota/New_Salem
48.23251 -101.29627 America/Chicago
41.12389 -100.76542 America/Chicago
41.86663 -103.66717 America/Denver
40.83242 -115.76312 America/Los_Angeles
44.63651 -123.10593 America/Los_Angeles
45.49428 -122.86705 America/Los_Angeles
42.20681 -121.73722 America/Los_Angeles
42.19458 -122.70948 America/Los_Angeles
45.48706 -122.80371 America/Los_Angeles
44.05817 -121.31531 America/Los_Angeles
45.55789 -122.8676 America/Los_Angeles
45.2629 -122.69259 America/Los_Angeles
42.37596 -122.91643 America/Los_Angeles
43.3665 -124.21789 America/Los_Angeles
44.56457 -123.26204 America/Los_Angeles
44.05207 -123.08675 America/Los_Angeles
45.51984 -123.11066 America/Los_Angeles
44.9279 -122.98371 America/Los_Angeles
42.43901 -123.32839 America/Los_Angeles
45.49818 -122.43148 America/Los_Angeles
44.98595 -122.98287 America/Los_Angeles
45.84041 -119.28946 America/Los_Angeles
45.52289 -122.98983 America/Los_Angeles
44.99012 -123.02621 America/Los_Angeles
42.22487 -121.78167 America/Los_Angeles
45.42067 -122.67065 America/Los_Angeles
44.53651 -122.90703 America/Los_Angeles
45.47984 -122.56731 America/Los_Angeles
45.21012 -123.19872 America/Los_Angeles
42.32652 -122.87559 America/Los_Angeles
45.44623 -122.63926 America/Los_Angeles
45.30012 -122.97316 America/Los_Angeles
45.41679 -122.64009 America/Los_Angeles
45.35734 -122.60676 America/Los_Angeles
45.67208 -118.7886 America/Los_Angeles
45.52345 -122.67621 America/Los_Angeles
44.27262 -121.17392 America/Los_Angeles
43.2165 -123.34174 America/Los_Angeles
44.9429 -123.0351 America/Los_Angeles
45.35651 -122.8401 America/Los_Angeles
44.04624 -123.02203 America/Los_Angeles
45.43123 -122.77149 America/Los_Angeles
45.53929 -122.38731 America/Los_Angeles
45.38401 -122.76399 America/Los_Angeles
45.36568 -122.61231 America/Los_Angeles
45.29984 -122.77371 America/Los_Angeles
45.14373 -122.85537 America/Los_Angeles
44.08054 -103.23101 America/Denver
40.88939 -111.88077 America/Denver
41.51021 -112.0155 America/Denver
40.918 -111.87216 America/Denver
41.11078 -112.02605 America/Denver
41.13967 -112.0505 America/Denver
40.61967 -111.81021 America/Denver
40.52467 -111.86382 America/Denver
40.31412 -112.00688 America/Denver
40.69995 -111.81049 America/Denver
40.9805 -111.88744 America/Denver
40.51411 -112.03299 America/Denver
40.44995 -111.77021 America/Denver
40.66884 -111.82466 America/Denver
41.03522 -111.93855 America/Denver
40.65995 -111.99633 America/Denver
41.06022 -111.97105 America/Denver
40.39162 -111.85077 America/Denver
41.73549 -111.83439 America/Denver
40.70911 -112.10161 America/Denver
40.61106 -111.89994 America/Denver
40.68689 -111.87549 America/Denver
40.66689 -111.88799 America/Denver
41.30716 -111.96022 America/Denver
40.84856 -111.90688 America/Denver
41.223 -111.97383 America/Denver
40.2969 -111.69465 America/Denver
40.0444 -111.73215 America/Denver
40.36412 -111.73854 America/Denver
40.23384 -111.65853 America/Denver
40.52189 -111.9391 America/Denver
41.16161 -112.02633 America/Denver
40.76078 -111.89105 America/Denver
40.57204 -111.86056 America/Denver
40.58106 -111.85077 America/Denver
40.34912 -111.90466 America/Denver
40.56384 -111.94938 America/Denver
40.56217 -111.92966 America/Denver
41.19189 -111.97133 America/Denver
40.71884 -111.88827 America/Denver
40.11496 -111.65492 America/Denver
40.16523 -111.61075 America/Denver
41.08939 -112.06467 America/Denver
40.66772 -111.93883 America/Denver
40.53078 -112.29828 America/Denver
40.60967 -111.9391 America/Denver
40.69161 -112.00105 America/Denver
46.97537 -123.81572 America/Los_Angeles
48.5126 -122.61267 America/Los_Angeles
48.19871 -122.12514 America/Los_Angeles
47.30732 -122.22845 America/Los_Angeles
45.78095 -122.53343 America/Los_Angeles
47.61038 -122.20068 America/Los_Angeles
48.75955 -122.48822 America/Los_Angeles
47.17705 -122.18651 America/Los_Angeles
47.76232 -122.2054 America/Los_Angeles
47.56732 -122.63264 America/Los_Angeles
47.47038 -122.34679 America/Los_Angeles
45.58706 -122.39954 America/Los_Angeles
46.71621 -122.9543 America/Los_Angeles
47.74427 -122.07735 America/Los_Angeles
47.35818 -122.12216 America/Los_Angeles
47.40177 -122.32429 America/Los_Angeles
47.81065 -122.37736 America/Los_Angeles
46.99651 -120.54785 America/Los_Angeles
47.97898 -122.20208 America/Los_Angeles
47.44843 -122.15734 America/Los_Angeles
47.32232 -122.31262 America/Los_Angeles
45.68456 -122.5751 America/Los_Angeles
47.09621 -122.35873 America/Los_Angeles
47.05288 -122.29428 America/Los_Angeles
45.67151 -122.66288 America/Los_Angeles
47.5301 -122.03262 America/Los_Angeles
47.75732 -122.24401 America/Los_Angeles
46.21125 -119.13723 America/Los_Angeles
47.38093 -122.23484 America/Los_Angeles
47.68149 -122.20874 America/Los_Angeles
47.03426 -122.82319 America/Los_Angeles
48.0151 -122.06374 America/Los_Angeles
47.17176 -122.51846 America/Los_Angeles
46.13817 -122.93817 America/Los_Angeles
47.82093 -122.31513 America/Los_Angeles
47.39272 -122.04641 America/Los_Angeles
47.85093 -122.2393 America/Los_Angeles
48.05176 -122.17708 America/Los_Angeles
47.57065 -122.22207 America/Los_Angeles
47.8601 -122.2043 America/Los_Angeles
47.85538 -121.97096 America/Los_Angeles
47.13014 -119.27808 America/Los_Angeles
48.42122 -122.33405 America/Los_Angeles
47.78815 -122.30874 America/Los_Angeles
47.94454 -122.30458 America/Los_Angeles
47.81954 -122.17624 America/Los_Angeles
48.29316 -122.64322 America/Los_Angeles
47.03787 -122.9007 America/Los_Angeles
47.64995 -117.23991 America/Los_Angeles
45.66651 -122.56093 America/Los_Angeles
47.15538 -122.43401 America/Los_Angeles
46.23958 -119.10057 America/Los_Angeles
48.11815 -123.43074 America/Los_Angeles
46.73127 -117.17962 America/Los_Angeles
47.18538 -122.2929 America/Los_Angeles
47.67399 -122.12151 America/Los_Angeles
47.48288 -122.21707 America/Los_Angeles
46.28569 -119.28446 America/Los_Angeles
45.71067 -122.64899 America/Los_Angeles
47.64177 -122.0804 America/Los_Angeles
47.44846 -122.29217 America/Los_Angeles
47.60621 -122.33207 America/Los_Angeles
47.75565 -122.34152 America/Los_Angeles
47.64454 -122.69487 America/Los_Angeles
47.14121 -122.27012 America/Los_Angeles
47.10399 -122.43457 America/Los_Angeles
47.65966 -117.42908 America/Los_Angeles
47.67323 -117.23937 America/Los_Angeles
46.32374 -120.00865 America/Los_Angeles
47.25288 -122.44429 America/Los_Angeles
47.47399 -122.26096 America/Los_Angeles
47.00732 -122.90931 America/Los_Angeles
47.23565 -122.5504 America/Los_Angeles
45.63873 -122.66149 America/Los_Angeles
46.06458 -118.34302 America/Los_Angeles
47.42346 -120.31035 America/Los_Angeles
47.5776 -122.10123 America/Los_Angeles
47.99343 -122.1018 America/Los_Angeles
47.62621 -122.52124 America/Los_Angeles
46.60207 -120.5059 America/Los_Angeles
42.86663 -106.31308 America/Denver
41.13998 -104.82025 America/Denver
44.29109 -105.50222 America/Denver
41.31137 -105.5911 America/Denver
41.58746 -109.2029 America/Denver
44.79719 -106.95618 America/Denver
40.3769 -111.79576 America/Denver
20.89472 -156.47 Pacific/Honolulu
21.40222 -157.73944 Pacific/Honolulu
21.41806 -157.80361 Pacific/Honolulu
20.785 -156.46556 Pacific/Honolulu
21.34694 -158.08583 Pacific/Honolulu
21.45 -158.00111 Pacific/Honolulu
21.39722 -157.97333 Pacific/Honolulu
21.50278 -158.02361 Pacific/Honolulu
20.89111 -156.50472 Pacific/Honolulu
21.38667 -158.00917 Pacific/Honolulu
21.34417 -158.03083 Pacific/Honolulu
19.72972 -155.09 Pacific/Honolulu
21.30694 -157.85833 Pacific/Honolulu
64.83778 -147.71639 America/Anchorage
61.21806 -149.90028 America/Anchorage
64.8 -147.53333 America/Anchorage
34.13216 -84.30067 America/New_York
34.02893 -84.19858 America/New_York
25.5783 -80.3377 America/New_York
28.5641 -81.2114 America/New_York
35.139 -79.00603 America/New_York
47.86602 -122.1551 America/Los_Angeles
38.46449 -121.34692 America/Los_Angeles
27.9915 -82.4529 America/New_York
41.44987 -72.81892 America/New_York
39.55429 -76.37309 America/New_York
39.50506 -76.31976 America/New_York
40.93064 -73.10178 America/New_York
42.98499 -78.87798 America/New_York
37.70573 -92.15717 America/Chicago
27.98269 -82.37878 America/New_York
28.33287 -81.64738 America/New_York
28.10545 -82.52594 America/New_York
33.72672 -84.27246 America/New_York
28.0692 -82.43609 America/New_York
27.61638 -80.41308 America/New_York
25.7065 -80.4388 America/New_York
27.92386 -82.37939 America/New_York
38.6025 -121.37854 America/Los_Angeles
47.4943 -122.24092 America/Los_Angeles
32.76397 -116.96877 America/Los_Angeles
33.96772 -118.24438 America/Los_Angeles
31.13489 -97.77561 America/Chicago
47.72049 -122.23167 America/Los_Angeles
34.23216 -118.23529 America/Los_Angeles
47.41052 -122.17369 America/Los_Angeles
38.74728 -104.71439 America/Denver
47.67887 -122.02833 America/Los_Angeles
33.976 -118.06909 America/Los_Angeles
36.10269 -115.38112 America/Los_Angeles
21.35237 -158.08655 Pacific/Honolulu
21.4936 -158.06151 Pacific/Honolulu
33.1911 -111.528 America/Phoenix
44.84096 -93.29843 America/Chicago
35.33676 -106.59296 America/Denver
34.19731 -118.64398 America/Los_Angeles
-32.68333 -57.63333 America/Montevideo
-33.53889 -56.88861 America/Montevideo
-33.23333 -54.38333 America/Montevideo
-31.73333 -55.98333 America/Montevideo
-34.45333 -56.39056 America/Montevideo
-34.3375 -56.71361 America/Montevideo
-34.8 -54.91667 America/Montevideo
-31.38333 -57.96667 America/Montevideo
-34.48333 -54.33333 America/Montevideo
-30.90534 -55.55076 America/Montevideo
-34.665 -56.21944 America/Montevideo
-32.32139 -58.07556 America/Montevideo
-34.86028 -56.05222 America/Montevideo
-34.71667 -55.95 America/Montevideo
-34.83346 -56.16735 America/Montevideo
-34.37 -55.225 America/Montevideo
-33.25583 -58.01917 America/Montevideo
-32.36667 -54.18333 America/Montevideo
-34.9 -54.95 America/Montevideo
-34.72639 -56.22 America/Montevideo
-34.76167 -56.22361 America/Montevideo
-33.1325 -58.29556 America/Montevideo
-34.09556 -56.21417 America/Montevideo
-33.41306 -56.50056 America/Montevideo
-33.54417 -58.19722 America/Montevideo
-34.76333 -56.38528 America/Montevideo
-34.46667 -57.85 America/Montevideo
-33.98917 -58.28556 America/Montevideo
-34.52278 -56.27778 America/Montevideo
-30.4 -56.46667 America/Montevideo
42.45306 59.61028 Asia/Samarkand
42.40043 59.46005 Asia/Samarkand
43.06874 58.90372 Asia/Samarkand
39.96056 68.39583 Asia/Tashkent
39.40222 67.24306 Asia/Samarkand
37.22417 67.27833 Asia/Samarkand
37.99944 67.7875 Asia/Samarkand
39.05778 66.83417 Asia/Samarkand
39.65417 66.95972 Asia/Samarkand
38.86667 65.8 Asia/Samarkand
39.25528 65.15278 Asia/Samarkand
39.08425 66.83318 Asia/Samarkand
39.89889 66.25611 Asia/Samarkand
39.0375 65.585 Asia/Samarkand
39.53333 63.83333 Asia/Samarkand
39.72278 64.55167 Asia/Samarkand
38.62083 66.24806 Asia/Samarkand
39.85778 64.44833 Asia/Samarkand
39.71611 66.66417 Asia/Samarkand
38.26667 67.9 Asia/Samarkand
39.03361 66.57222 Asia/Samarkand
39.92028 66.86111 Asia/Samarkand
39.76472 67.27139 Asia/Samarkand
39.77472 64.42861 Asia/Samarkand
38.82139 65.65306 Asia/Samarkand
38.20611 67.19861 Asia/Samarkand
39.92139 65.92528 Asia/Samarkand
40.98333 68.9 Asia/Tashkent
40.37583 70.81556 Asia/Tashkent
41.11202 69.0471 Asia/Tashkent
40.275 68.8225 Asia/Tashkent
40.02539 65.96095 Asia/Samarkand
41.18722 71.73333 Asia/Tashkent
41.11919 70.09406 Asia/Tashkent
40.03028 64.515 Asia/Samarkand
41.08073 71.92331 Asia/Tashkent
41.55 60.63333 Asia/Samarkand
40.12694 68.49444 Asia/Tashkent
41.11371 72.07915 Asia/Tashkent
41.0321 69.36253 Asia/Tashkent
40.99984 71.51162 Asia/Tashkent
40.47722 71.76778 Asia/Tashkent
41.26465 69.21627 Asia/Tashkent
40.92339 71.57214 Asia/Tashkent
40.84361 68.66167 Asia/Tashkent
41.65583 60.3025 Asia/Samarkand
40.12 64.50139 Asia/Samarkand
41.37222 69.38167 Asia/Tashkent
41.535 60.34556 Asia/Samarkand
40.52861 70.9425 Asia/Tashkent
40.89722 69.35056 Asia/Tashkent
40.00778 66.23694 Asia/Samarkand
41.29444 69.67639 Asia/Tashkent
40.87361 71.10889 Asia/Tashkent
40.31528 67.95444 Asia/Tashkent
40.84472 69.59833 Asia/Tashkent
40.90639 69.63833 Asia/Tashkent
40.56139 65.68861 Asia/Samarkand
41.55 61.01667 Asia/Samarkand
40.08444 65.37917 Asia/Samarkand
40.9983 71.67257 Asia/Tashkent
40.47111 71.72472 Asia/Tashkent
42.11556 60.05972 Asia/Samarkand
40.64153 72.23868 Asia/Tashkent
40.30222 71.97444 Asia/Tashkent
40.73192 72.76177 Asia/Tashkent
40.43583 71.76722 Asia/Tashkent
41.38972 69.465 Asia/Tashkent
40.66886 72.56002 Asia/Tashkent
41.37833 60.36389 Asia/Samarkand
40.91667 72.11667 Asia/Tashkent
41.25611 71.55083 Asia/Tashkent
40.11583 67.84222 Asia/Tashkent
41.55389 69.70083 Asia/Tashkent
41.31944 61.07417 Asia/Samarkand
41.84472 60.39194 Asia/Samarkand
40.48972 68.78417 Asia/Tashkent
40.1 64.68333 Asia/Samarkand
41.55806 69.77083 Asia/Tashkent
40.66194 68.17222 Asia/Tashkent
40.38421 71.78432 Asia/Tashkent
40.52472 68.03583 Asia/Tashkent
41.00936 71.2287 Asia/Tashkent
41.46889 69.58222 Asia/Tashkent
40.93639 68.76972 Asia/Tashkent
41.06656 71.85527 Asia/Tashkent
40.81083 69.19861 Asia/Tashkent
40.43583 70.61028 Asia/Tashkent
41.69111 60.7525 Asia/Samarkand
41.20972 69.33417 Asia/Tashkent
40.22083 69.26972 Asia/Tashkent
41.01667 70.14361 Asia/Tashkent
40.78206 72.34424 Asia/Tashkent
40.39194 71.47417 Asia/Tashkent
40.52204 72.07292 Asia/Tashkent
41.90236 12.45332 Europe/Vatican
13.15872 -61.22475 America/St_Vincent
13.15833 -61.235 America/St_Vincent
11.03333 -63.86278 America/Caracas
9.43889 -64.47278 America/Caracas
8.5931 -70.2261 America/Caracas
9.35029 -65.32452 America/Caracas
10.08 -69.12611 America/Caracas
10.03863 -67.48938 America/Caracas
9.33278 -69.12083 America/Caracas
9.21554 -66.00734 America/Caracas
9.31778 -70.60361 America/Caracas
10.16202 -68.00765 America/Caracas
8.00861 -62.39889 America/Caracas
10.22388 -67.47288 America/Caracas
9.06222 -62.05102 America/Caracas
9.37084 -70.43472 America/Caracas
9.91861 -68.30472 America/Caracas
7.8188 -72.22427 America/Caracas
10.23409 -66.66308 America/Caracas
10.53642 -71.51104 America/Caracas
10.21302 -67.42365 America/Caracas
9.91152 -67.35381 America/Caracas
8.03125 -72.26053 America/Caracas
8.88724 -64.16512 America/Caracas
10.263 -67.7858 America/Caracas
10.33991 -68.74247 America/Caracas
7.76694 -72.225 America/Caracas
9.00003 -71.91352 America/Caracas
9.66124 -68.58268 America/Caracas
7.81454 -72.4431 America/Caracas
10.38854 -66.95179 America/Caracas
7.70131 -72.35569 America/Caracas
10.3258 -72.31343 America/Caracas
9.92866 -69.6201 America/Caracas
11.6956 -70.19957 America/Caracas
11.65806 -70.215 America/Caracas
10.21667 -64.61667 America/Caracas
10.47306 -68.0125 America/Caracas
5.66389 -67.62361 America/Caracas
10.95796 -63.84906 America/Caracas
10.47226 -66.80155 America/Caracas
10.17389 -67.54194 America/Caracas
10.11649 -66.77609 America/Caracas
10.15526 -68.5663 America/Caracas
8.41667 -71.13333 America/Caracas
10.48889 -68.20028 America/Caracas
8.59524 -71.1434 America/Caracas
10.29644 -67.71813 America/Caracas
10.24694 -67.59583 America/Caracas
10.63167 -71.64056 America/Caracas
10.59621 -66.95485 America/Caracas
10.06444 -72.545 America/Caracas
10.02588 -69.24166 America/Caracas
10.49389 -66.82863 America/Caracas
10.21844 -67.33017 America/Caracas
10.25237 -67.18472 America/Caracas
10.13008 -71.25946 America/Caracas
10.59901 -66.9346 America/Caracas
11.08172 -63.96549 America/Caracas
10.57806 -62.29772 America/Caracas
10.08528 -67.77917 America/Caracas
10.4762 -66.54266 America/Caracas
10.46736 -66.60663 America/Caracas
9.04183 -69.74206 America/Caracas
10.22609 -67.877 America/Caracas
8.61366 -71.65284 America/Caracas
9.78734 -69.7937 America/Caracas
8.88752 -64.24544 America/Caracas
10.30589 -67.63212 America/Caracas
10.42994 -66.8161 America/Caracas
8.54665 -71.24087 America/Caracas
10.46354 -64.1775 America/Caracas
10.15892 -66.88575 America/Caracas
11.4045 -69.67344 America/Caracas
8.35122 -62.64102 America/Caracas
8.12923 -63.54086 America/Caracas
10.16028 -68.895 America/Caracas
10.24247 -66.85723 America/Caracas
10.49598 -66.85347 America/Caracas
10.08621 -67.91982 America/Caracas
10.48666 -66.73799 America/Caracas
10.60383 -67.03034 America/Caracas
10.66781 -63.25849 America/Caracas
10.34985 -66.98632 America/Caracas
10.17554 -70.08274 America/Caracas
10.48801 -66.87919 America/Caracas
10.61119 -66.85215 America/Caracas
9.30571 -64.35841 America/Caracas
8.92416 -67.42929 America/Caracas
10.18634 -67.45935 America/Caracas
10.38828 -71.43992 America/Caracas
10.43398 -66.87662 America/Caracas
10.07389 -69.32278 America/Caracas
8.76222 -70.41111 America/Caracas
10.13333 -64.7 America/Caracas
9.56009 -69.21475 America/Caracas
9.86005 -66.38139 America/Caracas
9.55451 -69.19564 America/Caracas
9.75 -63.17667 America/Caracas
8.22027 -72.2439 America/Caracas
10.46941 -66.83063 America/Caracas
10.35782 -66.80252 America/Caracas
10.4883 -66.78608 America/Caracas
7.24241 -70.73236 America/Caracas
7.88782 -67.47236 America/Caracas
18.41667 -64.61667 America/Tortola
18.3419 -64.9307 America/St_Thomas
18.66667 105.66667 Asia/Ho_Chi_Minh
21.7 104.86667 Asia/Ho_Chi_Minh
10.34599 107.08426 Asia/Ho_Chi_Minh
21.31 105.59667 Asia/Ho_Chi_Minh
10.25 105.96667 Asia/Ho_Chi_Minh
18.66667 105.66667 Asia/Ho_Chi_Minh
21.30194 105.43083 Asia/Ho_Chi_Minh
21.03333 106.78333 Asia/Ho_Chi_Minh
13.08333 109.3 Asia/Ho_Chi_Minh
21.82333 105.21806 Asia/Ho_Chi_Minh
9.93472 106.34528 Asia/Ho_Chi_Minh
10.96667 106.65 Asia/Ho_Chi_Minh
10.75 106.66667 Asia/Ho_Chi_Minh
19.8 105.76667 Asia/Ho_Chi_Minh
21.59278 105.84417 Asia/Ho_Chi_Minh
20.45 106.33333 Asia/Ho_Chi_Minh
11.3 106.1 Asia/Ho_Chi_Minh
10.53333 106.41667 Asia/Ho_Chi_Minh
15.56667 108.48333 Asia/Ho_Chi_Minh
21.13778 105.505 Asia/Ho_Chi_Minh
21.31667 103.9 Asia/Ho_Chi_Minh
13.45 109.21667 Asia/Ho_Chi_Minh
9.60333 105.98 Asia/Ho_Chi_Minh
22.34023 103.84415 Asia/Ho_Chi_Minh
10.3 105.76667 Asia/Ho_Chi_Minh
10.01667 105.08333 Asia/Ho_Chi_Minh
13.76667 109.23333 Asia/Ho_Chi_Minh
15.11667 108.8 Asia/Ho_Chi_Minh
13.98333 108 Asia/Ho_Chi_Minh
20.54111 105.91389 Asia/Ho_Chi_Minh
11.28333 106.13333 Asia/Ho_Chi_Minh
10.93333 108.1 Asia/Ho_Chi_Minh
11.56667 108.98333 Asia/Ho_Chi_Minh
20.25389 105.975 Asia/Ho_Chi_Minh
12.25 109.18333 Asia/Ho_Chi_Minh
20.41667 106.16667 Asia/Ho_Chi_Minh
10.35 106.35 Asia/Ho_Chi_Minh
10.38333 105.41667 Asia/Ho_Chi_Minh
22.48333 103.95 Asia/Ho_Chi_Minh
21.83333 106.73333 Asia/Ho_Chi_Minh
14.35 108 Asia/Ho_Chi_Minh
20.65 106.06667 Asia/Ho_Chi_Minh
16.46667 107.6 Asia/Ho_Chi_Minh
20.95111 107.08 Asia/Ho_Chi_Minh
15.87944 108.335 Asia/Ho_Chi_Minh
20.81333 105.33833 Asia/Ho_Chi_Minh
18.33333 105.9 Asia/Ho_Chi_Minh
10.38333 104.48333 Asia/Ho_Chi_Minh
21.0245 105.84117 Asia/Ho_Chi_Minh
20.85611 106.68222 Asia/Ho_Chi_Minh
20.93333 106.31667 Asia/Ho_Chi_Minh
22.83333 104.98333 Asia/Ho_Chi_Minh
20.9725 105.77722 Asia/Ho_Chi_Minh
11.53333 106.91667 Asia/Ho_Chi_Minh
17.48333 106.6 Asia/Ho_Chi_Minh
16.81625 107.10031 Asia/Ho_Chi_Minh
21.38333 103.01667 Asia/Ho_Chi_Minh
16.06778 108.22083 Asia/Ho_Chi_Minh
11.94646 108.44193 Asia/Ho_Chi_Minh
10.96667 106.46667 Asia/Ho_Chi_Minh
10.7 105.11667 Asia/Ho_Chi_Minh
10.45 105.63333 Asia/Ho_Chi_Minh
22.66667 106.25 Asia/Ho_Chi_Minh
10.03333 105.78333 Asia/Ho_Chi_Minh
10.41667 106.96667 Asia/Ho_Chi_Minh
10.61667 106.66667 Asia/Ho_Chi_Minh
11.92144 109.15913 Asia/Ho_Chi_Minh
21.01667 107.3 Asia/Ho_Chi_Minh
9.17694 105.15 Asia/Ho_Chi_Minh
12.66667 108.05 Asia/Ho_Chi_Minh
20.07806 105.86028 Asia/Ho_Chi_Minh
10.95 106.81667 Asia/Ho_Chi_Minh
10.23333 106.38333 Asia/Ho_Chi_Minh
21.18333 106.05 Asia/Ho_Chi_Minh
9.285 105.72444 Asia/Ho_Chi_Minh
21.26667 106.2 Asia/Ho_Chi_Minh
22.13333 105.83333 Asia/Ho_Chi_Minh
-17.73381 168.32188 Pacific/Efate
-13.28163 -176.17453 Pacific/Wallis
-13.83333 -171.76666 Pacific/Apia
42.82306 20.9675 Europe/Belgrade
42.37056 21.15528 Europe/Belgrade
42.35861 20.825 Europe/Belgrade
42.43306 21.03972 Europe/Belgrade
42.21389 20.73972 Europe/Belgrade
42.67272 21.16688 Europe/Belgrade
42.91056 21.19306 Europe/Belgrade
42.65913 20.28828 Europe/Belgrade
42.39944 20.65472 Europe/Belgrade
42.55528 20.76333 Europe/Belgrade
43.10389 20.80278 Europe/Belgrade
42.88333 20.86667 Europe/Belgrade
42.63944 21.09611 Europe/Belgrade
42.78083 20.4875 Europe/Belgrade
42.46045 21.46986 Europe/Belgrade
42.0625 20.65333 Europe/Belgrade
42.54018 20.28793 Europe/Belgrade
14.1951 43.31518 Asia/Aden
14.29804 44.37795 Asia/Aden
13.57952 44.02091 Asia/Aden
15.17177 44.32442 Asia/Aden
15.35472 44.20667 Asia/Aden
15.31586 44.30858 Asia/Aden
13.05667 44.88194 Asia/Aden
13.96667 44.18333 Asia/Aden
15.69425 43.60582 Asia/Aden
13.83446 44.11469 Asia/Aden
14.51635 43.32446 Asia/Aden
15.05835 43.28506 Asia/Aden
14.54248 49.12424 Asia/Aden
14.79781 42.95452 Asia/Aden
13.98523 45.57272 Asia/Aden
12.77944 45.03667 Asia/Aden
-12.77944 45.22722 Indian/Mayotte
-12.73361 45.20417 Indian/Mayotte
-26.1625 27.8725 Africa/Johannesburg
-25.53695 26.07512 Africa/Johannesburg
-27.1974 25.98311 Africa/Johannesburg
-25.33177 31.01166 Africa/Johannesburg
-25.87133 29.23323 Africa/Johannesburg
-26.31906 27.6486 Africa/Johannesburg
-27.8549 26.36583 Africa/Johannesburg
-27.98644 26.70661 Africa/Johannesburg
-28.11396 24.84753 Africa/Johannesburg
-24.88333 28.28333 Africa/Johannesburg
-27.76952 30.79165 Africa/Johannesburg
-26.95659 24.7284 Africa/Johannesburg
-27.36542 29.88175 Africa/Johannesburg
-28.10391 26.86593 Africa/Johannesburg
-27.20841 26.94855 Africa/Johannesburg
-26.67313 27.92615 Africa/Johannesburg
-26.71171 27.83795 Africa/Johannesburg
-28.44776 21.25612 Africa/Johannesburg
-31.58893 28.78443 Africa/Johannesburg
-30.20674 30.79776 Africa/Johannesburg
-28.33523 31.41617 Africa/Johannesburg
-33.75757 25.3971 Africa/Johannesburg
-23.83322 30.16351 Africa/Johannesburg
-22.94564 30.48497 Africa/Johannesburg
-28.41098 26.70107 Africa/Johannesburg
-29.20932 26.83898 Africa/Johannesburg
-25.99636 28.2268 Africa/Johannesburg
-32.57076 27.42396 Africa/Johannesburg
-26.84493 26.76829 Africa/Johannesburg
-29.32816 31.28954 Africa/Johannesburg
-26.93366 29.24152 Africa/Johannesburg
-26.25 28.4 Africa/Johannesburg
-26.26781 27.85849 Africa/Johannesburg
-32.72173 25.58804 Africa/Johannesburg
-25.11319 29.04454 Africa/Johannesburg
-28.31971 27.62082 Africa/Johannesburg
-26.55 29.16667 Africa/Johannesburg
-30.28666 30.75316 Africa/Johannesburg
-27.18871 25.32931 Africa/Johannesburg
-26.81358 27.81695 Africa/Johannesburg
-25.66756 27.24208 Africa/Johannesburg
-29.87196 30.27235 Africa/Johannesburg
-28.78301 32.03768 Africa/Johannesburg
-27.80138 28.42726 Africa/Johannesburg
-26.1844 27.70203 Africa/Johannesburg
-31.89756 26.87533 Africa/Johannesburg
-31.87101 26.97862 Africa/Johannesburg
-25.74486 28.18783 Africa/Johannesburg
-24.19436 29.00974 Africa/Johannesburg
-26.71667 27.1 Africa/Johannesburg
-30.74137 30.45499 Africa/Johannesburg
-33.91799 25.57007 Africa/Johannesburg
-33.59057 26.89104 Africa/Johannesburg
-34.05274 23.3716 Africa/Johannesburg
-27.00706 30.81323 Africa/Johannesburg
-23.90448 29.46885 Africa/Johannesburg
-29.61678 30.39278 Africa/Johannesburg
-28.52423 28.81582 Africa/Johannesburg
-23.94299 31.14107 Africa/Johannesburg
-26.9033 27.45727 Africa/Johannesburg
-27.78324 24.68768 Africa/Johannesburg
-33.59067 22.2014 Africa/Johannesburg
-26.98023 26.67272 Africa/Johannesburg
-24.7 28.4 Africa/Johannesburg
-23.88782 30.28708 Africa/Johannesburg
-26.43138 28.47713 Africa/Johannesburg
-27.75796 29.9318 Africa/Johannesburg
-25.47448 30.97033 Africa/Johannesburg
-29.81292 30.63646 Africa/Johannesburg
-29.56822 30.18618 Africa/Johannesburg
-27.98299 30.71769 Africa/Johannesburg
-25.85 25.63333 Africa/Johannesburg
-31.49285 25.00633 Africa/Johannesburg
-25.77507 29.46482 Africa/Johannesburg
-22.35131 30.0396 Africa/Johannesburg
4 -30.8636 Africa/Johannesburg
-25.49768 28.10065 Africa/Johannesburg
-25.0978 30.45967 Africa/Johannesburg
-23.04385 29.90319 Africa/Johannesburg
-26.152 26.15968 Africa/Johannesburg
-24.2 29.5 Africa/Johannesburg
-31.70312 27.2329 Africa/Johannesburg
-29.19448 27.45739 Africa/Johannesburg
-27.83333 26.75 Africa/Johannesburg
-34.00333 24.73142 Africa/Johannesburg
-26.08577 27.77515 Africa/Johannesburg
-27.65036 27.23488 Africa/Johannesburg
-26.27391 29.2253 Africa/Johannesburg
-25.4332 31.95478 Africa/Johannesburg
-30.54723 29.42412 Africa/Johannesburg
-34.03627 23.04713 Africa/Johannesburg
-26.85213 26.66672 Africa/Johannesburg
-28.73226 24.76232 Africa/Johannesburg
-26.20227 28.04363 Africa/Johannesburg
-29.47795 30.23057 Africa/Johannesburg
-27.97654 27.02423 Africa/Johannesburg
-26.15881 29.71528 Africa/Johannesburg
-27.28115 27.9709 Africa/Johannesburg
-26.50476 28.35921 Africa/Johannesburg
-28.27276 29.12946 Africa/Johannesburg
-33.30422 26.53276 Africa/Johannesburg
-32.25215 24.53075 Africa/Johannesburg
-23.30246 30.71868 Africa/Johannesburg
-33.963 22.46173 Africa/Johannesburg
-25.61692 27.99471 Africa/Johannesburg
-27.26667 28.48333 Africa/Johannesburg
-32.77477 26.63376 Africa/Johannesburg
-26.48862 27.49387 Africa/Johannesburg
-28.87097 31.89961 Africa/Johannesburg
-26.53333 29.98333 Africa/Johannesburg
-28.76197 31.89329 Africa/Johannesburg
-26.53333 29.06667 Africa/Johannesburg
-23.6649 27.74426 Africa/Johannesburg
-33.01529 27.91162 Africa/Johannesburg
-29.8579 31.0292 Africa/Johannesburg
-28.16678 30.23371 Africa/Johannesburg
-23.69339 30.14002 Africa/Johannesburg
-27.0177 30.44979 Africa/Johannesburg
-26.1466 28.68322 Africa/Johannesburg
-30.64966 24.0123 Africa/Johannesburg
-25.67088 28.52364 Africa/Johannesburg
-32.16422 25.61918 Africa/Johannesburg
-27.91402 25.16111 Africa/Johannesburg
-26.36094 27.39767 Africa/Johannesburg
-32.33083 28.14981 Africa/Johannesburg
-25.81015 28.74248 Africa/Johannesburg
-25.63473 27.78022 Africa/Johannesburg
-26.23656 28.36938 Africa/Johannesburg
-29.27016 26.7052 Africa/Johannesburg
-27.3887 26.61701 Africa/Johannesburg
-26.21197 28.25958 Africa/Johannesburg
-27.64686 25.60697 Africa/Johannesburg
-29.12106 26.214 Africa/Johannesburg
-32.84721 27.44218 Africa/Johannesburg
-28.23078 28.30707 Africa/Johannesburg
-26.45794 29.46553 Africa/Johannesburg
-26.18848 28.32078 Africa/Johannesburg
-32.35671 22.58295 Africa/Johannesburg
-25.78842 31.05319 Africa/Johannesburg
-29.53897 31.21439 Africa/Johannesburg
-26.66331 28.59016 Africa/Johannesburg
-27.75431 26.64382 Africa/Johannesburg
-30.69366 26.71141 Africa/Johannesburg
-25.69619 28.74918 Africa/Johannesburg
-25.96361 28.13778 Africa/Johannesburg
-25.85891 28.18577 Africa/Johannesburg
-33.64651 19.44852 Africa/Johannesburg
-33.93462 18.86676 Africa/Johannesburg
-33.01167 17.9442 Africa/Johannesburg
-33.70468 18.96552 Africa/Johannesburg
-33.4608 18.72714 Africa/Johannesburg
-34.4187 19.23446 Africa/Johannesburg
-34.15152 19.01509 Africa/Johannesburg
-33.36889 19.31096 Africa/Johannesburg
-33.92584 18.42322 Africa/Johannesburg
-33.56668 18.48335 Africa/Johannesburg
-9.34506 28.73396 Africa/Lusaka
-8.84024 31.36587 Africa/Lusaka
-9.7915 29.07913 Africa/Lusaka
-16.53818 28.70876 Africa/Lusaka
-17.47593 24.29684 Africa/Lusaka
-11.36491 29.55652 Africa/Lusaka
-14.24264 31.3253 Africa/Lusaka
-12.95867 28.63659 Africa/Lusaka
-14.97832 27.06188 Africa/Lusaka
-12.54982 28.24071 Africa/Lusaka
-11.83431 31.45287 Africa/Lusaka
-16.28333 27.48333 Africa/Lusaka
-15.24835 23.12741 Africa/Lusaka
-15.85601 27.748 Africa/Lusaka
-11.19976 28.89431 Africa/Lusaka
-15.40669 28.28713 Africa/Lusaka
-13.13667 28.41661 Africa/Lusaka
-17.84194 25.85425 Africa/Lusaka
-12.79614 28.24767 Africa/Lusaka
-10.21289 31.18084 Africa/Lusaka
-13.96518 28.68086 Africa/Lusaka
-12.09514 26.42727 Africa/Lusaka
-12.83778 28.09192 Africa/Lusaka
-15.76911 28.18136 Africa/Lusaka
-14.4469 28.44644 Africa/Lusaka
-16.80648 26.95306 Africa/Lusaka
-13.63333 32.65 Africa/Lusaka
-12.52897 27.88382 Africa/Lusaka
-12.36475 27.82286 Africa/Lusaka
-20.32674 30.06648 Africa/Harare
-17.92353 25.84299 Africa/Harare
-19.67016 30.00589 Africa/Harare
-18.52785 32.12843 Africa/Harare
-19.03333 29.78333 Africa/Harare
-17.88333 30.7 Africa/Harare
-18.9707 32.67086 Africa/Harare
-20.06373 30.82766 Africa/Harare
-18.18527 31.55193 Africa/Harare
-18.9281 29.81486 Africa/Harare
-16.80993 29.69247 Africa/Harare
-16.51667 28.8 Africa/Harare
-18.33328 29.91534 Africa/Harare
-18.36927 26.50194 Africa/Harare
-17.82935 31.05389 Africa/Harare
-19.45 29.81667 Africa/Harare
-18.20476 28.9349 Africa/Harare
-21.05 31.66667 Africa/Harare
-20.18833 32.62364 Africa/Harare
-17.36667 30.2 Africa/Harare
-18.13021 30.14074 Africa/Harare
-20.15 28.58333 Africa/Harare
-17.30192 31.33056 Africa/Harare
-22.21667 30 Africa/Harare
-17.89 31.1475 Africa/Harare
-18.01274 31.07555 Africa/Harare');
  }
}

